#include "config-all-clusters-bucket-spaces.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalAllClustersBucketSpacesType::CONFIG_DEF_MD5("fe076e3aab3f4ffbfc49ed1695357507");
const vespalib::string InternalAllClustersBucketSpacesType::CONFIG_DEF_NAME("all-clusters-bucket-spaces");
const vespalib::string InternalAllClustersBucketSpacesType::CONFIG_DEF_NAMESPACE("vespa.config.content");
const int64_t InternalAllClustersBucketSpacesType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content",
"cluster{}.documentType{}.bucketSpace string",
};
const ::config::StringVector InternalAllClustersBucketSpacesType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalAllClustersBucketSpacesType::Cluster::DocumentType::DocumentType() noexcept
    : bucketSpace()
{
}

InternalAllClustersBucketSpacesType::Cluster::DocumentType::DocumentType(const DocumentType & __rhs) noexcept = default;
InternalAllClustersBucketSpacesType::Cluster::DocumentType & InternalAllClustersBucketSpacesType::Cluster::DocumentType::operator = (const DocumentType & __rhs) noexcept = default;
InternalAllClustersBucketSpacesType::Cluster::DocumentType::DocumentType(DocumentType && __rhs) noexcept = default;
InternalAllClustersBucketSpacesType::Cluster::DocumentType & InternalAllClustersBucketSpacesType::Cluster::DocumentType::operator = (DocumentType && __rhs) noexcept = default;
InternalAllClustersBucketSpacesType::Cluster::DocumentType::~DocumentType() = default; 
InternalAllClustersBucketSpacesType::Cluster::DocumentType::DocumentType(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    bucketSpace = ConfigParser::parse<vespalib::string>("bucketSpace", __lines);
    ConfigParser::stripLinesForKey("bucketSpace", __remainingValuesToParse);
}

bool
InternalAllClustersBucketSpacesType::Cluster::DocumentType::
operator==(const DocumentType& __rhs) const noexcept
{
    return (bucketSpace == __rhs.bucketSpace);
}

bool
InternalAllClustersBucketSpacesType::Cluster::DocumentType::
operator!=(const DocumentType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalAllClustersBucketSpacesType::Cluster::DocumentType::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("bucketSpace");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(bucketSpace));
    }
}

InternalAllClustersBucketSpacesType::Cluster::DocumentType::DocumentType(const vespalib::slime::Inspector & __inspector)
{
    bucketSpace = __inspector["bucketSpace"]["value"].asString().make_string();
}

InternalAllClustersBucketSpacesType::Cluster::DocumentType::DocumentType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    bucketSpace = ::config::internal::ValueConverter<vespalib::string>()("bucketSpace", __inspector["bucketSpace"]);
}

InternalAllClustersBucketSpacesType::Cluster::Cluster()
    : documentType()
{
}

InternalAllClustersBucketSpacesType::Cluster::Cluster(const Cluster & __rhs) = default;
InternalAllClustersBucketSpacesType::Cluster & InternalAllClustersBucketSpacesType::Cluster::operator = (const Cluster & __rhs) = default;
InternalAllClustersBucketSpacesType::Cluster::Cluster(Cluster && __rhs) noexcept = default;
InternalAllClustersBucketSpacesType::Cluster & InternalAllClustersBucketSpacesType::Cluster::operator = (Cluster && __rhs) noexcept = default;
InternalAllClustersBucketSpacesType::Cluster::~Cluster() = default; 
InternalAllClustersBucketSpacesType::Cluster::Cluster(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    documentType = ConfigParser::parseMap<DocumentType>("documentType", __lines);
    ConfigParser::stripLinesForKey("documentType", __remainingValuesToParse);
}

bool
InternalAllClustersBucketSpacesType::Cluster::
operator==(const Cluster& __rhs) const noexcept
{
    return (documentType == __rhs.documentType);
}

bool
InternalAllClustersBucketSpacesType::Cluster::
operator!=(const Cluster& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalAllClustersBucketSpacesType::Cluster::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("documentType");
        __c.setString("type", "map");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & entry : documentType) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("key", vespalib::Memory(entry.first));
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            entry.second.serialize(__c4);
        }
    }
}

InternalAllClustersBucketSpacesType::Cluster::Cluster(const vespalib::slime::Inspector & __inspector)
{
    for (size_t __i = 0; __i < __inspector["documentType"]["value"].children(); __i++) {
        documentType[__inspector["documentType"]["value"][__i]["key"].asString().make_string()] = DocumentType(__inspector["documentType"]["value"][__i]["value"]);
    }
}

InternalAllClustersBucketSpacesType::Cluster::Cluster(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::MapInserter<DocumentType> __documentTypeInserter(documentType);
    __inspector["documentType"].traverse(__documentTypeInserter);
}

InternalAllClustersBucketSpacesType::InternalAllClustersBucketSpacesType()
    : cluster()
{
}

InternalAllClustersBucketSpacesType::InternalAllClustersBucketSpacesType(const InternalAllClustersBucketSpacesType & __rhs) = default;
InternalAllClustersBucketSpacesType & InternalAllClustersBucketSpacesType::operator =(const InternalAllClustersBucketSpacesType & __rhs) = default;
InternalAllClustersBucketSpacesType::InternalAllClustersBucketSpacesType(InternalAllClustersBucketSpacesType && __rhs) noexcept = default;
InternalAllClustersBucketSpacesType & InternalAllClustersBucketSpacesType::operator =(InternalAllClustersBucketSpacesType && __rhs) noexcept = default;
InternalAllClustersBucketSpacesType::~InternalAllClustersBucketSpacesType() = default; 
InternalAllClustersBucketSpacesType::InternalAllClustersBucketSpacesType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        cluster = ConfigParser::parseMap<Cluster>("cluster", __lines);
        ConfigParser::stripLinesForKey("cluster", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalAllClustersBucketSpacesType::
operator==(const InternalAllClustersBucketSpacesType& __rhs) const noexcept
{
    return (cluster == __rhs.cluster);
}

bool
InternalAllClustersBucketSpacesType::
operator!=(const InternalAllClustersBucketSpacesType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalAllClustersBucketSpacesType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("cluster");
        __c.setString("type", "map");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & entry : cluster) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("key", vespalib::Memory(entry.first));
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            entry.second.serialize(__c4);
        }
    }
}

InternalAllClustersBucketSpacesType::InternalAllClustersBucketSpacesType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["cluster"]["value"].children(); __i++) {
        cluster[__inspector["cluster"]["value"][__i]["key"].asString().make_string()] = Cluster(__inspector["cluster"]["value"][__i]["value"]);
    }
}

InternalAllClustersBucketSpacesType::InternalAllClustersBucketSpacesType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::MapInserter<Cluster> __clusterInserter(cluster);
    __inspector["cluster"].traverse(__clusterInserter);
}

} // namespace internal

} // namespace vespa::config::content
