#include "config-application-id.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace cloud::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalApplicationIdType::CONFIG_DEF_MD5("fde6b077639d9f215db08d33b94fec26");
const vespalib::string InternalApplicationIdType::CONFIG_DEF_NAME("application-id");
const vespalib::string InternalApplicationIdType::CONFIG_DEF_NAMESPACE("cloud.config");
const int64_t InternalApplicationIdType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=cloud.config",
"tenant string default=\"(unknowntenant)\"",
"application string default=\"(unknownapplication)\"",
"instance string default=\"(unknowninstance)\"",
};
const ::config::StringVector InternalApplicationIdType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalApplicationIdType::InternalApplicationIdType() noexcept
    : tenant("(unknowntenant)"),
      application("(unknownapplication)"),
      instance("(unknowninstance)")
{
}

InternalApplicationIdType::InternalApplicationIdType(const InternalApplicationIdType & __rhs) noexcept = default;
InternalApplicationIdType & InternalApplicationIdType::operator =(const InternalApplicationIdType & __rhs) noexcept = default;
InternalApplicationIdType::InternalApplicationIdType(InternalApplicationIdType && __rhs) noexcept = default;
InternalApplicationIdType & InternalApplicationIdType::operator =(InternalApplicationIdType && __rhs) noexcept = default;
InternalApplicationIdType::~InternalApplicationIdType() = default; 
InternalApplicationIdType::InternalApplicationIdType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        tenant = ConfigParser::parse<vespalib::string>("tenant", __lines, "(unknowntenant)");
        ConfigParser::stripLinesForKey("tenant", __remainingValuesToParse);
        application = ConfigParser::parse<vespalib::string>("application", __lines, "(unknownapplication)");
        ConfigParser::stripLinesForKey("application", __remainingValuesToParse);
        instance = ConfigParser::parse<vespalib::string>("instance", __lines, "(unknowninstance)");
        ConfigParser::stripLinesForKey("instance", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalApplicationIdType::
operator==(const InternalApplicationIdType& __rhs) const noexcept
{
    return (tenant == __rhs.tenant &&
            application == __rhs.application &&
            instance == __rhs.instance);
}

bool
InternalApplicationIdType::
operator!=(const InternalApplicationIdType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalApplicationIdType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tenant");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(tenant));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("application");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(application));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("instance");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(instance));
    }
}

InternalApplicationIdType::InternalApplicationIdType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    tenant = __inspector["tenant"]["value"].asString().make_string();
    application = __inspector["application"]["value"].asString().make_string();
    instance = __inspector["instance"]["value"].asString().make_string();
}

InternalApplicationIdType::InternalApplicationIdType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    tenant = ::config::internal::ValueConverter<vespalib::string>()(__inspector["tenant"], "(unknowntenant)");
    application = ::config::internal::ValueConverter<vespalib::string>()(__inspector["application"], "(unknownapplication)");
    instance = ::config::internal::ValueConverter<vespalib::string>()(__inspector["instance"], "(unknowninstance)");
}

} // namespace internal

} // namespace cloud::config
