#include "config-attributes.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalAttributesType::CONFIG_DEF_MD5("2f1fe4e3cf48edc1514b4e1fa9f70e34");
const vespalib::string InternalAttributesType::CONFIG_DEF_NAME("attributes");
const vespalib::string InternalAttributesType::CONFIG_DEF_NAMESPACE("vespa.config.search");
const int64_t InternalAttributesType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search",
"attribute[].name string",
"attribute[].datatype enum { STRING, BOOL, UINT2, UINT4, INT8, INT16, INT32, INT64, FLOAT16, FLOAT, DOUBLE, PREDICATE, TENSOR, REFERENCE, RAW, NONE } default=NONE",
"attribute[].collectiontype enum { SINGLE, ARRAY, WEIGHTEDSET } default=SINGLE",
"attribute[].dictionary.type enum { BTREE, HASH, BTREE_AND_HASH } default = BTREE",
"attribute[].dictionary.match enum { CASE_SENSITIVE, CASE_INSENSITIVE, CASED, UNCASED } default=UNCASED",
"attribute[].match enum { CASED, UNCASED } default=UNCASED",
"attribute[].removeifzero bool default=false",
"attribute[].createifnonexistent bool default=false",
"attribute[].fastsearch bool default=false",
"attribute[].paged bool default=false",
"attribute[].ismutable bool default=false",
"attribute[].sortascending bool default=true",
"attribute[].sortfunction enum { RAW, LOWERCASE, UCA } default=UCA",
"attribute[].sortstrength enum { PRIMARY, SECONDARY, TERTIARY, QUATERNARY, IDENTICAL } default=PRIMARY",
"attribute[].sortlocale string default=\"\"",
"attribute[].enableonlybitvector bool default=false",
"attribute[].fastaccess bool default=false",
"attribute[].arity int default=8",
"attribute[].lowerbound long default=-9223372036854775808",
"attribute[].upperbound long default=9223372036854775807",
"attribute[].densepostinglistthreshold double default=0.40",
"attribute[].tensortype string default=\"\"",
"attribute[].imported bool default=false",
"attribute[].maxuncommittedmemory long default=130000",
"attribute[].distancemetric enum { EUCLIDEAN, ANGULAR, GEODEGREES, INNERPRODUCT, HAMMING, PRENORMALIZED_ANGULAR, DOTPRODUCT } default=EUCLIDEAN",
"attribute[].index.hnsw.enabled bool default=false",
"attribute[].index.hnsw.maxlinkspernode int default=16",
"attribute[].index.hnsw.neighborstoexploreatinsert int default=200",
"attribute[].index.hnsw.multithreadedindexing bool default=true",
};
const ::config::StringVector InternalAttributesType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalAttributesType::Attribute::Datatype
InternalAttributesType::Attribute::getDatatype(const vespalib::string& name)
{
    if (name == "STRING") {
        return Datatype::STRING;
    } else if (name == "BOOL") {
        return Datatype::BOOL;
    } else if (name == "UINT2") {
        return Datatype::UINT2;
    } else if (name == "UINT4") {
        return Datatype::UINT4;
    } else if (name == "INT8") {
        return Datatype::INT8;
    } else if (name == "INT16") {
        return Datatype::INT16;
    } else if (name == "INT32") {
        return Datatype::INT32;
    } else if (name == "INT64") {
        return Datatype::INT64;
    } else if (name == "FLOAT16") {
        return Datatype::FLOAT16;
    } else if (name == "FLOAT") {
        return Datatype::FLOAT;
    } else if (name == "DOUBLE") {
        return Datatype::DOUBLE;
    } else if (name == "PREDICATE") {
        return Datatype::PREDICATE;
    } else if (name == "TENSOR") {
        return Datatype::TENSOR;
    } else if (name == "REFERENCE") {
        return Datatype::REFERENCE;
    } else if (name == "RAW") {
        return Datatype::RAW;
    } else if (name == "NONE") {
        return Datatype::NONE;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalAttributesType::Attribute::getDatatypeName(Datatype t)
{
    switch (t) {
        case Datatype::STRING: return "STRING";
        case Datatype::BOOL: return "BOOL";
        case Datatype::UINT2: return "UINT2";
        case Datatype::UINT4: return "UINT4";
        case Datatype::INT8: return "INT8";
        case Datatype::INT16: return "INT16";
        case Datatype::INT32: return "INT32";
        case Datatype::INT64: return "INT64";
        case Datatype::FLOAT16: return "FLOAT16";
        case Datatype::FLOAT: return "FLOAT";
        case Datatype::DOUBLE: return "DOUBLE";
        case Datatype::PREDICATE: return "PREDICATE";
        case Datatype::TENSOR: return "TENSOR";
        case Datatype::REFERENCE: return "REFERENCE";
        case Datatype::RAW: return "RAW";
        case Datatype::NONE: return "NONE";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalAttributesType::Attribute::Datatype InternalAttributesType::Attribute::InternalDatatypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::getDatatype(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalAttributesType::Attribute::Datatype InternalAttributesType::Attribute::InternalDatatypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalAttributesType::Attribute::getDatatype(__inspector.asString().make_string());
}
InternalAttributesType::Attribute::Datatype InternalAttributesType::Attribute::InternalDatatypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Datatype __eDefault) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::getDatatype(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalAttributesType::Attribute::Collectiontype
InternalAttributesType::Attribute::getCollectiontype(const vespalib::string& name)
{
    if (name == "SINGLE") {
        return Collectiontype::SINGLE;
    } else if (name == "ARRAY") {
        return Collectiontype::ARRAY;
    } else if (name == "WEIGHTEDSET") {
        return Collectiontype::WEIGHTEDSET;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalAttributesType::Attribute::getCollectiontypeName(Collectiontype t)
{
    switch (t) {
        case Collectiontype::SINGLE: return "SINGLE";
        case Collectiontype::ARRAY: return "ARRAY";
        case Collectiontype::WEIGHTEDSET: return "WEIGHTEDSET";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalAttributesType::Attribute::Collectiontype InternalAttributesType::Attribute::InternalCollectiontypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::getCollectiontype(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalAttributesType::Attribute::Collectiontype InternalAttributesType::Attribute::InternalCollectiontypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalAttributesType::Attribute::getCollectiontype(__inspector.asString().make_string());
}
InternalAttributesType::Attribute::Collectiontype InternalAttributesType::Attribute::InternalCollectiontypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Collectiontype __eDefault) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::getCollectiontype(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalAttributesType::Attribute::Dictionary::Type
InternalAttributesType::Attribute::Dictionary::getType(const vespalib::string& name)
{
    if (name == "BTREE") {
        return Type::BTREE;
    } else if (name == "HASH") {
        return Type::HASH;
    } else if (name == "BTREE_AND_HASH") {
        return Type::BTREE_AND_HASH;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalAttributesType::Attribute::Dictionary::getTypeName(Type t)
{
    switch (t) {
        case Type::BTREE: return "BTREE";
        case Type::HASH: return "HASH";
        case Type::BTREE_AND_HASH: return "BTREE_AND_HASH";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalAttributesType::Attribute::Dictionary::Type InternalAttributesType::Attribute::Dictionary::InternalTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::Dictionary::getType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalAttributesType::Attribute::Dictionary::Type InternalAttributesType::Attribute::Dictionary::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalAttributesType::Attribute::Dictionary::getType(__inspector.asString().make_string());
}
InternalAttributesType::Attribute::Dictionary::Type InternalAttributesType::Attribute::Dictionary::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::Dictionary::getType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalAttributesType::Attribute::Dictionary::Match
InternalAttributesType::Attribute::Dictionary::getMatch(const vespalib::string& name)
{
    if (name == "CASE_SENSITIVE") {
        return Match::CASE_SENSITIVE;
    } else if (name == "CASE_INSENSITIVE") {
        return Match::CASE_INSENSITIVE;
    } else if (name == "CASED") {
        return Match::CASED;
    } else if (name == "UNCASED") {
        return Match::UNCASED;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalAttributesType::Attribute::Dictionary::getMatchName(Match t)
{
    switch (t) {
        case Match::CASE_SENSITIVE: return "CASE_SENSITIVE";
        case Match::CASE_INSENSITIVE: return "CASE_INSENSITIVE";
        case Match::CASED: return "CASED";
        case Match::UNCASED: return "UNCASED";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalAttributesType::Attribute::Dictionary::Match InternalAttributesType::Attribute::Dictionary::InternalMatchConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::Dictionary::getMatch(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalAttributesType::Attribute::Dictionary::Match InternalAttributesType::Attribute::Dictionary::InternalMatchConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalAttributesType::Attribute::Dictionary::getMatch(__inspector.asString().make_string());
}
InternalAttributesType::Attribute::Dictionary::Match InternalAttributesType::Attribute::Dictionary::InternalMatchConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Match __eDefault) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::Dictionary::getMatch(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalAttributesType::Attribute::Dictionary::Dictionary() noexcept
    : type(Type::BTREE),
      match(Match::UNCASED)
{
}

InternalAttributesType::Attribute::Dictionary::Dictionary(const Dictionary & __rhs) noexcept = default;
InternalAttributesType::Attribute::Dictionary & InternalAttributesType::Attribute::Dictionary::operator = (const Dictionary & __rhs) noexcept = default;
InternalAttributesType::Attribute::Dictionary::Dictionary(Dictionary && __rhs) noexcept = default;
InternalAttributesType::Attribute::Dictionary & InternalAttributesType::Attribute::Dictionary::operator = (Dictionary && __rhs) noexcept = default;
InternalAttributesType::Attribute::Dictionary::~Dictionary() = default; 
InternalAttributesType::Attribute::Dictionary::Dictionary(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    type = getType(ConfigParser::parse<vespalib::string>("type", __lines, "BTREE"));
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
    match = getMatch(ConfigParser::parse<vespalib::string>("match", __lines, "UNCASED"));
    ConfigParser::stripLinesForKey("match", __remainingValuesToParse);
}

bool
InternalAttributesType::Attribute::Dictionary::
operator==(const Dictionary& __rhs) const noexcept
{
    return (type == __rhs.type &&
            match == __rhs.match);
}

bool
InternalAttributesType::Attribute::Dictionary::
operator!=(const Dictionary& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalAttributesType::Attribute::Dictionary::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getTypeName(type)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("match");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getMatchName(match)));
    }
}

InternalAttributesType::Attribute::Dictionary::Dictionary(const vespalib::slime::Inspector & __inspector)
{
    type = getType(__inspector["type"]["value"].asString().make_string());
    match = getMatch(__inspector["match"]["value"].asString().make_string());
}

InternalAttributesType::Attribute::Dictionary::Dictionary(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    type = InternalTypeConverter()(__inspector["type"], Type::BTREE);
    match = InternalMatchConverter()(__inspector["match"], Match::UNCASED);
}

InternalAttributesType::Attribute::Match
InternalAttributesType::Attribute::getMatch(const vespalib::string& name)
{
    if (name == "CASED") {
        return Match::CASED;
    } else if (name == "UNCASED") {
        return Match::UNCASED;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalAttributesType::Attribute::getMatchName(Match t)
{
    switch (t) {
        case Match::CASED: return "CASED";
        case Match::UNCASED: return "UNCASED";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalAttributesType::Attribute::Match InternalAttributesType::Attribute::InternalMatchConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::getMatch(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalAttributesType::Attribute::Match InternalAttributesType::Attribute::InternalMatchConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalAttributesType::Attribute::getMatch(__inspector.asString().make_string());
}
InternalAttributesType::Attribute::Match InternalAttributesType::Attribute::InternalMatchConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Match __eDefault) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::getMatch(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalAttributesType::Attribute::Sortfunction
InternalAttributesType::Attribute::getSortfunction(const vespalib::string& name)
{
    if (name == "RAW") {
        return Sortfunction::RAW;
    } else if (name == "LOWERCASE") {
        return Sortfunction::LOWERCASE;
    } else if (name == "UCA") {
        return Sortfunction::UCA;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalAttributesType::Attribute::getSortfunctionName(Sortfunction t)
{
    switch (t) {
        case Sortfunction::RAW: return "RAW";
        case Sortfunction::LOWERCASE: return "LOWERCASE";
        case Sortfunction::UCA: return "UCA";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalAttributesType::Attribute::Sortfunction InternalAttributesType::Attribute::InternalSortfunctionConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::getSortfunction(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalAttributesType::Attribute::Sortfunction InternalAttributesType::Attribute::InternalSortfunctionConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalAttributesType::Attribute::getSortfunction(__inspector.asString().make_string());
}
InternalAttributesType::Attribute::Sortfunction InternalAttributesType::Attribute::InternalSortfunctionConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Sortfunction __eDefault) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::getSortfunction(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalAttributesType::Attribute::Sortstrength
InternalAttributesType::Attribute::getSortstrength(const vespalib::string& name)
{
    if (name == "PRIMARY") {
        return Sortstrength::PRIMARY;
    } else if (name == "SECONDARY") {
        return Sortstrength::SECONDARY;
    } else if (name == "TERTIARY") {
        return Sortstrength::TERTIARY;
    } else if (name == "QUATERNARY") {
        return Sortstrength::QUATERNARY;
    } else if (name == "IDENTICAL") {
        return Sortstrength::IDENTICAL;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalAttributesType::Attribute::getSortstrengthName(Sortstrength t)
{
    switch (t) {
        case Sortstrength::PRIMARY: return "PRIMARY";
        case Sortstrength::SECONDARY: return "SECONDARY";
        case Sortstrength::TERTIARY: return "TERTIARY";
        case Sortstrength::QUATERNARY: return "QUATERNARY";
        case Sortstrength::IDENTICAL: return "IDENTICAL";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalAttributesType::Attribute::Sortstrength InternalAttributesType::Attribute::InternalSortstrengthConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::getSortstrength(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalAttributesType::Attribute::Sortstrength InternalAttributesType::Attribute::InternalSortstrengthConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalAttributesType::Attribute::getSortstrength(__inspector.asString().make_string());
}
InternalAttributesType::Attribute::Sortstrength InternalAttributesType::Attribute::InternalSortstrengthConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Sortstrength __eDefault) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::getSortstrength(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalAttributesType::Attribute::Distancemetric
InternalAttributesType::Attribute::getDistancemetric(const vespalib::string& name)
{
    if (name == "EUCLIDEAN") {
        return Distancemetric::EUCLIDEAN;
    } else if (name == "ANGULAR") {
        return Distancemetric::ANGULAR;
    } else if (name == "GEODEGREES") {
        return Distancemetric::GEODEGREES;
    } else if (name == "INNERPRODUCT") {
        return Distancemetric::INNERPRODUCT;
    } else if (name == "HAMMING") {
        return Distancemetric::HAMMING;
    } else if (name == "PRENORMALIZED_ANGULAR") {
        return Distancemetric::PRENORMALIZED_ANGULAR;
    } else if (name == "DOTPRODUCT") {
        return Distancemetric::DOTPRODUCT;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalAttributesType::Attribute::getDistancemetricName(Distancemetric t)
{
    switch (t) {
        case Distancemetric::EUCLIDEAN: return "EUCLIDEAN";
        case Distancemetric::ANGULAR: return "ANGULAR";
        case Distancemetric::GEODEGREES: return "GEODEGREES";
        case Distancemetric::INNERPRODUCT: return "INNERPRODUCT";
        case Distancemetric::HAMMING: return "HAMMING";
        case Distancemetric::PRENORMALIZED_ANGULAR: return "PRENORMALIZED_ANGULAR";
        case Distancemetric::DOTPRODUCT: return "DOTPRODUCT";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalAttributesType::Attribute::Distancemetric InternalAttributesType::Attribute::InternalDistancemetricConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::getDistancemetric(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalAttributesType::Attribute::Distancemetric InternalAttributesType::Attribute::InternalDistancemetricConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalAttributesType::Attribute::getDistancemetric(__inspector.asString().make_string());
}
InternalAttributesType::Attribute::Distancemetric InternalAttributesType::Attribute::InternalDistancemetricConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Distancemetric __eDefault) {
    if (__inspector.valid()) {
        return InternalAttributesType::Attribute::getDistancemetric(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalAttributesType::Attribute::Index::Hnsw::Hnsw() noexcept
    : enabled(false),
      maxlinkspernode(16),
      neighborstoexploreatinsert(200),
      multithreadedindexing(true)
{
}

InternalAttributesType::Attribute::Index::Hnsw::Hnsw(const Hnsw & __rhs) noexcept = default;
InternalAttributesType::Attribute::Index::Hnsw & InternalAttributesType::Attribute::Index::Hnsw::operator = (const Hnsw & __rhs) noexcept = default;
InternalAttributesType::Attribute::Index::Hnsw::Hnsw(Hnsw && __rhs) noexcept = default;
InternalAttributesType::Attribute::Index::Hnsw & InternalAttributesType::Attribute::Index::Hnsw::operator = (Hnsw && __rhs) noexcept = default;
InternalAttributesType::Attribute::Index::Hnsw::~Hnsw() = default; 
InternalAttributesType::Attribute::Index::Hnsw::Hnsw(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    enabled = ConfigParser::parse<bool>("enabled", __lines, false);
    ConfigParser::stripLinesForKey("enabled", __remainingValuesToParse);
    maxlinkspernode = ConfigParser::parse<int32_t>("maxlinkspernode", __lines, 16);
    ConfigParser::stripLinesForKey("maxlinkspernode", __remainingValuesToParse);
    neighborstoexploreatinsert = ConfigParser::parse<int32_t>("neighborstoexploreatinsert", __lines, 200);
    ConfigParser::stripLinesForKey("neighborstoexploreatinsert", __remainingValuesToParse);
    multithreadedindexing = ConfigParser::parse<bool>("multithreadedindexing", __lines, true);
    ConfigParser::stripLinesForKey("multithreadedindexing", __remainingValuesToParse);
}

bool
InternalAttributesType::Attribute::Index::Hnsw::
operator==(const Hnsw& __rhs) const noexcept
{
    return (enabled == __rhs.enabled &&
            maxlinkspernode == __rhs.maxlinkspernode &&
            neighborstoexploreatinsert == __rhs.neighborstoexploreatinsert &&
            multithreadedindexing == __rhs.multithreadedindexing);
}

bool
InternalAttributesType::Attribute::Index::Hnsw::
operator!=(const Hnsw& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalAttributesType::Attribute::Index::Hnsw::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enabled");
        __c.setString("type", "bool");
        __c.setBool("value", enabled);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxlinkspernode");
        __c.setString("type", "int");
        __c.setLong("value", maxlinkspernode);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("neighborstoexploreatinsert");
        __c.setString("type", "int");
        __c.setLong("value", neighborstoexploreatinsert);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("multithreadedindexing");
        __c.setString("type", "bool");
        __c.setBool("value", multithreadedindexing);
    }
}

InternalAttributesType::Attribute::Index::Hnsw::Hnsw(const vespalib::slime::Inspector & __inspector)
{
    enabled = __inspector["enabled"]["value"].asBool();
    maxlinkspernode = __inspector["maxlinkspernode"]["value"].asLong();
    neighborstoexploreatinsert = __inspector["neighborstoexploreatinsert"]["value"].asLong();
    multithreadedindexing = __inspector["multithreadedindexing"]["value"].asBool();
}

InternalAttributesType::Attribute::Index::Hnsw::Hnsw(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    enabled = ::config::internal::ValueConverter<bool>()(__inspector["enabled"], false);
    maxlinkspernode = ::config::internal::ValueConverter<int32_t>()(__inspector["maxlinkspernode"], 16);
    neighborstoexploreatinsert = ::config::internal::ValueConverter<int32_t>()(__inspector["neighborstoexploreatinsert"], 200);
    multithreadedindexing = ::config::internal::ValueConverter<bool>()(__inspector["multithreadedindexing"], true);
}

InternalAttributesType::Attribute::Index::Index() noexcept
    : hnsw()
{
}

InternalAttributesType::Attribute::Index::Index(const Index & __rhs) noexcept = default;
InternalAttributesType::Attribute::Index & InternalAttributesType::Attribute::Index::operator = (const Index & __rhs) noexcept = default;
InternalAttributesType::Attribute::Index::Index(Index && __rhs) noexcept = default;
InternalAttributesType::Attribute::Index & InternalAttributesType::Attribute::Index::operator = (Index && __rhs) noexcept = default;
InternalAttributesType::Attribute::Index::~Index() = default; 
InternalAttributesType::Attribute::Index::Index(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    hnsw = ConfigParser::parseStruct<Hnsw>("hnsw", __lines);
    ConfigParser::stripLinesForKey("hnsw", __remainingValuesToParse);
}

bool
InternalAttributesType::Attribute::Index::operator==(const Index& __rhs) const noexcept
{
    return (hnsw == __rhs.hnsw);
}

bool
InternalAttributesType::Attribute::Index::operator!=(const Index& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalAttributesType::Attribute::Index::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("hnsw");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        hnsw.serialize(__c2);
    }
}

InternalAttributesType::Attribute::Index::Index(const vespalib::slime::Inspector & __inspector)
{
    hnsw = Hnsw(__inspector["hnsw"]["value"]);
}

InternalAttributesType::Attribute::Index::Index(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    hnsw = ::config::internal::ValueConverter<Hnsw>()(__inspector["hnsw"]);
}

InternalAttributesType::Attribute::Attribute() noexcept
    : name(),
      datatype(Datatype::NONE),
      collectiontype(Collectiontype::SINGLE),
      dictionary(),
      match(Match::UNCASED),
      removeifzero(false),
      createifnonexistent(false),
      fastsearch(false),
      paged(false),
      ismutable(false),
      sortascending(true),
      sortfunction(Sortfunction::UCA),
      sortstrength(Sortstrength::PRIMARY),
      sortlocale(""),
      enableonlybitvector(false),
      fastaccess(false),
      arity(8),
      lowerbound(LONG_MIN),
      upperbound(9223372036854775807),
      densepostinglistthreshold(0.40),
      tensortype(""),
      imported(false),
      maxuncommittedmemory(130000),
      distancemetric(Distancemetric::EUCLIDEAN),
      index()
{
}

InternalAttributesType::Attribute::Attribute(const Attribute & __rhs) noexcept = default;
InternalAttributesType::Attribute & InternalAttributesType::Attribute::operator = (const Attribute & __rhs) noexcept = default;
InternalAttributesType::Attribute::Attribute(Attribute && __rhs) noexcept = default;
InternalAttributesType::Attribute & InternalAttributesType::Attribute::operator = (Attribute && __rhs) noexcept = default;
InternalAttributesType::Attribute::~Attribute() = default; 
InternalAttributesType::Attribute::Attribute(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    datatype = getDatatype(ConfigParser::parse<vespalib::string>("datatype", __lines, "NONE"));
    ConfigParser::stripLinesForKey("datatype", __remainingValuesToParse);
    collectiontype = getCollectiontype(ConfigParser::parse<vespalib::string>("collectiontype", __lines, "SINGLE"));
    ConfigParser::stripLinesForKey("collectiontype", __remainingValuesToParse);
    dictionary = ConfigParser::parseStruct<Dictionary>("dictionary", __lines);
    ConfigParser::stripLinesForKey("dictionary", __remainingValuesToParse);
    match = getMatch(ConfigParser::parse<vespalib::string>("match", __lines, "UNCASED"));
    ConfigParser::stripLinesForKey("match", __remainingValuesToParse);
    removeifzero = ConfigParser::parse<bool>("removeifzero", __lines, false);
    ConfigParser::stripLinesForKey("removeifzero", __remainingValuesToParse);
    createifnonexistent = ConfigParser::parse<bool>("createifnonexistent", __lines, false);
    ConfigParser::stripLinesForKey("createifnonexistent", __remainingValuesToParse);
    fastsearch = ConfigParser::parse<bool>("fastsearch", __lines, false);
    ConfigParser::stripLinesForKey("fastsearch", __remainingValuesToParse);
    paged = ConfigParser::parse<bool>("paged", __lines, false);
    ConfigParser::stripLinesForKey("paged", __remainingValuesToParse);
    ismutable = ConfigParser::parse<bool>("ismutable", __lines, false);
    ConfigParser::stripLinesForKey("ismutable", __remainingValuesToParse);
    sortascending = ConfigParser::parse<bool>("sortascending", __lines, true);
    ConfigParser::stripLinesForKey("sortascending", __remainingValuesToParse);
    sortfunction = getSortfunction(ConfigParser::parse<vespalib::string>("sortfunction", __lines, "UCA"));
    ConfigParser::stripLinesForKey("sortfunction", __remainingValuesToParse);
    sortstrength = getSortstrength(ConfigParser::parse<vespalib::string>("sortstrength", __lines, "PRIMARY"));
    ConfigParser::stripLinesForKey("sortstrength", __remainingValuesToParse);
    sortlocale = ConfigParser::parse<vespalib::string>("sortlocale", __lines, "");
    ConfigParser::stripLinesForKey("sortlocale", __remainingValuesToParse);
    enableonlybitvector = ConfigParser::parse<bool>("enableonlybitvector", __lines, false);
    ConfigParser::stripLinesForKey("enableonlybitvector", __remainingValuesToParse);
    fastaccess = ConfigParser::parse<bool>("fastaccess", __lines, false);
    ConfigParser::stripLinesForKey("fastaccess", __remainingValuesToParse);
    arity = ConfigParser::parse<int32_t>("arity", __lines, 8);
    ConfigParser::stripLinesForKey("arity", __remainingValuesToParse);
    lowerbound = ConfigParser::parse<int64_t>("lowerbound", __lines, LONG_MIN);
    ConfigParser::stripLinesForKey("lowerbound", __remainingValuesToParse);
    upperbound = ConfigParser::parse<int64_t>("upperbound", __lines, 9223372036854775807);
    ConfigParser::stripLinesForKey("upperbound", __remainingValuesToParse);
    densepostinglistthreshold = ConfigParser::parse<double>("densepostinglistthreshold", __lines, 0.40);
    ConfigParser::stripLinesForKey("densepostinglistthreshold", __remainingValuesToParse);
    tensortype = ConfigParser::parse<vespalib::string>("tensortype", __lines, "");
    ConfigParser::stripLinesForKey("tensortype", __remainingValuesToParse);
    imported = ConfigParser::parse<bool>("imported", __lines, false);
    ConfigParser::stripLinesForKey("imported", __remainingValuesToParse);
    maxuncommittedmemory = ConfigParser::parse<int64_t>("maxuncommittedmemory", __lines, 130000);
    ConfigParser::stripLinesForKey("maxuncommittedmemory", __remainingValuesToParse);
    distancemetric = getDistancemetric(ConfigParser::parse<vespalib::string>("distancemetric", __lines, "EUCLIDEAN"));
    ConfigParser::stripLinesForKey("distancemetric", __remainingValuesToParse);
    index = ConfigParser::parseStruct<Index>("index", __lines);
    ConfigParser::stripLinesForKey("index", __remainingValuesToParse);
}

bool
InternalAttributesType::Attribute::operator==(const Attribute& __rhs) const noexcept
{
    return (name == __rhs.name &&
            datatype == __rhs.datatype &&
            collectiontype == __rhs.collectiontype &&
            dictionary == __rhs.dictionary &&
            match == __rhs.match &&
            removeifzero == __rhs.removeifzero &&
            createifnonexistent == __rhs.createifnonexistent &&
            fastsearch == __rhs.fastsearch &&
            paged == __rhs.paged &&
            ismutable == __rhs.ismutable &&
            sortascending == __rhs.sortascending &&
            sortfunction == __rhs.sortfunction &&
            sortstrength == __rhs.sortstrength &&
            sortlocale == __rhs.sortlocale &&
            enableonlybitvector == __rhs.enableonlybitvector &&
            fastaccess == __rhs.fastaccess &&
            arity == __rhs.arity &&
            lowerbound == __rhs.lowerbound &&
            upperbound == __rhs.upperbound &&
            densepostinglistthreshold == __rhs.densepostinglistthreshold &&
            tensortype == __rhs.tensortype &&
            imported == __rhs.imported &&
            maxuncommittedmemory == __rhs.maxuncommittedmemory &&
            distancemetric == __rhs.distancemetric &&
            index == __rhs.index);
}

bool
InternalAttributesType::Attribute::operator!=(const Attribute& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalAttributesType::Attribute::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("datatype");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getDatatypeName(datatype)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("collectiontype");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getCollectiontypeName(collectiontype)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("dictionary");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        dictionary.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("match");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getMatchName(match)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("removeifzero");
        __c.setString("type", "bool");
        __c.setBool("value", removeifzero);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("createifnonexistent");
        __c.setString("type", "bool");
        __c.setBool("value", createifnonexistent);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fastsearch");
        __c.setString("type", "bool");
        __c.setBool("value", fastsearch);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("paged");
        __c.setString("type", "bool");
        __c.setBool("value", paged);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ismutable");
        __c.setString("type", "bool");
        __c.setBool("value", ismutable);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("sortascending");
        __c.setString("type", "bool");
        __c.setBool("value", sortascending);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("sortfunction");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getSortfunctionName(sortfunction)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("sortstrength");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getSortstrengthName(sortstrength)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("sortlocale");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(sortlocale));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enableonlybitvector");
        __c.setString("type", "bool");
        __c.setBool("value", enableonlybitvector);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fastaccess");
        __c.setString("type", "bool");
        __c.setBool("value", fastaccess);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("arity");
        __c.setString("type", "int");
        __c.setLong("value", arity);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("lowerbound");
        __c.setString("type", "long");
        __c.setLong("value", lowerbound);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("upperbound");
        __c.setString("type", "long");
        __c.setLong("value", upperbound);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("densepostinglistthreshold");
        __c.setString("type", "double");
        __c.setDouble("value", densepostinglistthreshold);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tensortype");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(tensortype));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("imported");
        __c.setString("type", "bool");
        __c.setBool("value", imported);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxuncommittedmemory");
        __c.setString("type", "long");
        __c.setLong("value", maxuncommittedmemory);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("distancemetric");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getDistancemetricName(distancemetric)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("index");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        index.serialize(__c2);
    }
}

InternalAttributesType::Attribute::Attribute(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    datatype = getDatatype(__inspector["datatype"]["value"].asString().make_string());
    collectiontype = getCollectiontype(__inspector["collectiontype"]["value"].asString().make_string());
    dictionary = Dictionary(__inspector["dictionary"]["value"]);
    match = getMatch(__inspector["match"]["value"].asString().make_string());
    removeifzero = __inspector["removeifzero"]["value"].asBool();
    createifnonexistent = __inspector["createifnonexistent"]["value"].asBool();
    fastsearch = __inspector["fastsearch"]["value"].asBool();
    paged = __inspector["paged"]["value"].asBool();
    ismutable = __inspector["ismutable"]["value"].asBool();
    sortascending = __inspector["sortascending"]["value"].asBool();
    sortfunction = getSortfunction(__inspector["sortfunction"]["value"].asString().make_string());
    sortstrength = getSortstrength(__inspector["sortstrength"]["value"].asString().make_string());
    sortlocale = __inspector["sortlocale"]["value"].asString().make_string();
    enableonlybitvector = __inspector["enableonlybitvector"]["value"].asBool();
    fastaccess = __inspector["fastaccess"]["value"].asBool();
    arity = __inspector["arity"]["value"].asLong();
    lowerbound = __inspector["lowerbound"]["value"].asLong();
    upperbound = __inspector["upperbound"]["value"].asLong();
    densepostinglistthreshold = __inspector["densepostinglistthreshold"]["value"].asDouble();
    tensortype = __inspector["tensortype"]["value"].asString().make_string();
    imported = __inspector["imported"]["value"].asBool();
    maxuncommittedmemory = __inspector["maxuncommittedmemory"]["value"].asLong();
    distancemetric = getDistancemetric(__inspector["distancemetric"]["value"].asString().make_string());
    index = Index(__inspector["index"]["value"]);
}

InternalAttributesType::Attribute::Attribute(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    datatype = InternalDatatypeConverter()(__inspector["datatype"], Datatype::NONE);
    collectiontype = InternalCollectiontypeConverter()(__inspector["collectiontype"], Collectiontype::SINGLE);
    dictionary = ::config::internal::ValueConverter<Dictionary>()(__inspector["dictionary"]);
    match = InternalMatchConverter()(__inspector["match"], Match::UNCASED);
    removeifzero = ::config::internal::ValueConverter<bool>()(__inspector["removeifzero"], false);
    createifnonexistent = ::config::internal::ValueConverter<bool>()(__inspector["createifnonexistent"], false);
    fastsearch = ::config::internal::ValueConverter<bool>()(__inspector["fastsearch"], false);
    paged = ::config::internal::ValueConverter<bool>()(__inspector["paged"], false);
    ismutable = ::config::internal::ValueConverter<bool>()(__inspector["ismutable"], false);
    sortascending = ::config::internal::ValueConverter<bool>()(__inspector["sortascending"], true);
    sortfunction = InternalSortfunctionConverter()(__inspector["sortfunction"], Sortfunction::UCA);
    sortstrength = InternalSortstrengthConverter()(__inspector["sortstrength"], Sortstrength::PRIMARY);
    sortlocale = ::config::internal::ValueConverter<vespalib::string>()(__inspector["sortlocale"], "");
    enableonlybitvector = ::config::internal::ValueConverter<bool>()(__inspector["enableonlybitvector"], false);
    fastaccess = ::config::internal::ValueConverter<bool>()(__inspector["fastaccess"], false);
    arity = ::config::internal::ValueConverter<int32_t>()(__inspector["arity"], 8);
    lowerbound = ::config::internal::ValueConverter<int64_t>()(__inspector["lowerbound"], LONG_MIN);
    upperbound = ::config::internal::ValueConverter<int64_t>()(__inspector["upperbound"], 9223372036854775807);
    densepostinglistthreshold = ::config::internal::ValueConverter<double>()(__inspector["densepostinglistthreshold"], 0.40);
    tensortype = ::config::internal::ValueConverter<vespalib::string>()(__inspector["tensortype"], "");
    imported = ::config::internal::ValueConverter<bool>()(__inspector["imported"], false);
    maxuncommittedmemory = ::config::internal::ValueConverter<int64_t>()(__inspector["maxuncommittedmemory"], 130000);
    distancemetric = InternalDistancemetricConverter()(__inspector["distancemetric"], Distancemetric::EUCLIDEAN);
    index = ::config::internal::ValueConverter<Index>()(__inspector["index"]);
}

InternalAttributesType::InternalAttributesType()
    : attribute()
{
}

InternalAttributesType::InternalAttributesType(const InternalAttributesType & __rhs) = default;
InternalAttributesType & InternalAttributesType::operator =(const InternalAttributesType & __rhs) = default;
InternalAttributesType::InternalAttributesType(InternalAttributesType && __rhs) noexcept = default;
InternalAttributesType & InternalAttributesType::operator =(InternalAttributesType && __rhs) noexcept = default;
InternalAttributesType::~InternalAttributesType() = default; 
InternalAttributesType::InternalAttributesType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        attribute = ConfigParser::parseArray<AttributeVector>("attribute", __lines);
        ConfigParser::stripLinesForKey("attribute", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalAttributesType::operator==(const InternalAttributesType& __rhs) const noexcept
{
    return (attribute == __rhs.attribute);
}

bool
InternalAttributesType::operator!=(const InternalAttributesType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalAttributesType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("attribute");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : attribute) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalAttributesType::InternalAttributesType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["attribute"]["value"].children(); __i++) {
        attribute.push_back(Attribute(__inspector["attribute"]["value"][__i]["value"]));
    }
}

InternalAttributesType::InternalAttributesType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<AttributeVector> __attributeInserter(attribute);
    __inspector["attribute"].traverse(__attributeInserter);
}

} // namespace internal

} // namespace vespa::config::search
