/**
 * @class vespa::config::search::Attributes
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_SEARCH_ATTRIBUTES_H
#define CLOUD_CONFIG_VESPA_CONFIG_SEARCH_ATTRIBUTES_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::search {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalAttributesType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalAttributesType> UP;
    class Attribute {
    public:
        enum class Datatype { STRING, BOOL, UINT2, UINT4, INT8, INT16, INT32, INT64, FLOAT16, FLOAT, DOUBLE, PREDICATE, TENSOR, REFERENCE, RAW, NONE };
        typedef std::vector<Datatype> DatatypeVector;
        typedef std::map<vespalib::string, Datatype> DatatypeMap;
        static Datatype getDatatype(const vespalib::string&);
        static vespalib::string getDatatypeName(Datatype e);

        struct InternalDatatypeConverter {
            Datatype operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Datatype operator()(const ::vespalib::slime::Inspector & __inspector);
            Datatype operator()(const ::vespalib::slime::Inspector & __inspector, Datatype __eDefault);
        };
        enum class Collectiontype { SINGLE, ARRAY, WEIGHTEDSET };
        typedef std::vector<Collectiontype> CollectiontypeVector;
        typedef std::map<vespalib::string, Collectiontype> CollectiontypeMap;
        static Collectiontype getCollectiontype(const vespalib::string&);
        static vespalib::string getCollectiontypeName(Collectiontype e);

        struct InternalCollectiontypeConverter {
            Collectiontype operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Collectiontype operator()(const ::vespalib::slime::Inspector & __inspector);
            Collectiontype operator()(const ::vespalib::slime::Inspector & __inspector, Collectiontype __eDefault);
        };
        class Dictionary {
        public:
            enum class Type { BTREE, HASH, BTREE_AND_HASH };
            typedef std::vector<Type> TypeVector;
            typedef std::map<vespalib::string, Type> TypeMap;
            static Type getType(const vespalib::string&);
            static vespalib::string getTypeName(Type e);

            struct InternalTypeConverter {
                Type operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                Type operator()(const ::vespalib::slime::Inspector & __inspector);
                Type operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault);
            };
            enum class Match { CASE_SENSITIVE, CASE_INSENSITIVE, CASED, UNCASED };
            typedef std::vector<Match> MatchVector;
            typedef std::map<vespalib::string, Match> MatchMap;
            static Match getMatch(const vespalib::string&);
            static vespalib::string getMatchName(Match e);

            struct InternalMatchConverter {
                Match operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                Match operator()(const ::vespalib::slime::Inspector & __inspector);
                Match operator()(const ::vespalib::slime::Inspector & __inspector, Match __eDefault);
            };
            Dictionary(const ::config::StringVector & __lines);
            Dictionary(const vespalib::slime::Inspector & __inspector);
            Dictionary(const ::config::ConfigPayload & __payload);
            Dictionary()  noexcept;
            Dictionary(const Dictionary & __rhs) noexcept;
            Dictionary & operator = (const Dictionary & __rhs) noexcept;
            Dictionary(Dictionary && __rhs) noexcept;
            Dictionary & operator = (Dictionary && __rhs) noexcept;
            ~Dictionary();

            bool operator==(const Dictionary& __rhs) const noexcept;
            bool operator!=(const Dictionary& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            Type type; // Default: BTREE
            Match match; // Default: UNCASED
        };
        typedef std::vector<Dictionary> DictionaryVector;

        typedef std::map<vespalib::string, Dictionary> DictionaryMap;

        enum class Match { CASED, UNCASED };
        typedef std::vector<Match> MatchVector;
        typedef std::map<vespalib::string, Match> MatchMap;
        static Match getMatch(const vespalib::string&);
        static vespalib::string getMatchName(Match e);

        struct InternalMatchConverter {
            Match operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Match operator()(const ::vespalib::slime::Inspector & __inspector);
            Match operator()(const ::vespalib::slime::Inspector & __inspector, Match __eDefault);
        };
        enum class Sortfunction { RAW, LOWERCASE, UCA };
        typedef std::vector<Sortfunction> SortfunctionVector;
        typedef std::map<vespalib::string, Sortfunction> SortfunctionMap;
        static Sortfunction getSortfunction(const vespalib::string&);
        static vespalib::string getSortfunctionName(Sortfunction e);

        struct InternalSortfunctionConverter {
            Sortfunction operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Sortfunction operator()(const ::vespalib::slime::Inspector & __inspector);
            Sortfunction operator()(const ::vespalib::slime::Inspector & __inspector, Sortfunction __eDefault);
        };
        enum class Sortstrength { PRIMARY, SECONDARY, TERTIARY, QUATERNARY, IDENTICAL };
        typedef std::vector<Sortstrength> SortstrengthVector;
        typedef std::map<vespalib::string, Sortstrength> SortstrengthMap;
        static Sortstrength getSortstrength(const vespalib::string&);
        static vespalib::string getSortstrengthName(Sortstrength e);

        struct InternalSortstrengthConverter {
            Sortstrength operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Sortstrength operator()(const ::vespalib::slime::Inspector & __inspector);
            Sortstrength operator()(const ::vespalib::slime::Inspector & __inspector, Sortstrength __eDefault);
        };
        enum class Distancemetric { EUCLIDEAN, ANGULAR, GEODEGREES, INNERPRODUCT, HAMMING, PRENORMALIZED_ANGULAR, DOTPRODUCT };
        typedef std::vector<Distancemetric> DistancemetricVector;
        typedef std::map<vespalib::string, Distancemetric> DistancemetricMap;
        static Distancemetric getDistancemetric(const vespalib::string&);
        static vespalib::string getDistancemetricName(Distancemetric e);

        struct InternalDistancemetricConverter {
            Distancemetric operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Distancemetric operator()(const ::vespalib::slime::Inspector & __inspector);
            Distancemetric operator()(const ::vespalib::slime::Inspector & __inspector, Distancemetric __eDefault);
        };
        class Index {
        public:
            class Hnsw {
            public:
                Hnsw(const ::config::StringVector & __lines);
                Hnsw(const vespalib::slime::Inspector & __inspector);
                Hnsw(const ::config::ConfigPayload & __payload);
                Hnsw()  noexcept;
                Hnsw(const Hnsw & __rhs) noexcept;
                Hnsw & operator = (const Hnsw & __rhs) noexcept;
                Hnsw(Hnsw && __rhs) noexcept;
                Hnsw & operator = (Hnsw && __rhs) noexcept;
                ~Hnsw();

                bool operator==(const Hnsw& __rhs) const noexcept;
                bool operator!=(const Hnsw& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /**
                 * Configuration parameters for a hnsw index used together with
                 * a 1-dimensional indexed tensor for approximate nearest
                 * neighbor search.
                 */
                bool enabled; // Default: false
                int32_t maxlinkspernode; // Default: 16
                int32_t neighborstoexploreatinsert; // Default: 200

                /**
                 * Whether multi-threaded indexing is enabled for this hnsw
                 * index.
                 */
                bool multithreadedindexing; // Default: true
            };
            typedef std::vector<Hnsw> HnswVector;

            typedef std::map<vespalib::string, Hnsw> HnswMap;

            Index(const ::config::StringVector & __lines);
            Index(const vespalib::slime::Inspector & __inspector);
            Index(const ::config::ConfigPayload & __payload);
            Index()  noexcept;
            Index(const Index & __rhs) noexcept;
            Index & operator = (const Index & __rhs) noexcept;
            Index(Index && __rhs) noexcept;
            Index & operator = (Index && __rhs) noexcept;
            ~Index();

            bool operator==(const Index& __rhs) const noexcept;
            bool operator!=(const Index& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            Hnsw hnsw;
        };
        typedef std::vector<Index> IndexVector;

        typedef std::map<vespalib::string, Index> IndexMap;

        Attribute(const ::config::StringVector & __lines);
        Attribute(const vespalib::slime::Inspector & __inspector);
        Attribute(const ::config::ConfigPayload & __payload);
        Attribute()  noexcept;
        Attribute(const Attribute & __rhs) noexcept;
        Attribute & operator = (const Attribute & __rhs) noexcept;
        Attribute(Attribute && __rhs) noexcept;
        Attribute & operator = (Attribute && __rhs) noexcept;
        ~Attribute();

        bool operator==(const Attribute& __rhs) const noexcept;
        bool operator!=(const Attribute& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        vespalib::string name;
        Datatype datatype; // Default: NONE
        Collectiontype collectiontype; // Default: SINGLE
        Dictionary dictionary;
        Match match; // Default: UNCASED
        bool removeifzero; // Default: false
        bool createifnonexistent; // Default: false
        bool fastsearch; // Default: false
        bool paged; // Default: false

        /** An attribute marked mutable can be updated by a query. */
        bool ismutable; // Default: false
        bool sortascending; // Default: true
        Sortfunction sortfunction; // Default: UCA
        Sortstrength sortstrength; // Default: PRIMARY
        vespalib::string sortlocale; // Default: ""

        /**
         * Allow only bitvector postings, i.e. drop btree postings to save
         * memory.?
         */
        bool enableonlybitvector; // Default: false

        /**
         * Allow fast access to this attribute at all times.
         * If so, attribute is kept in memory also for non-searchable documents.
         */
        bool fastaccess; // Default: false
        int32_t arity; // Default: 8
        int64_t lowerbound; // Default: -9223372036854775808
        int64_t upperbound; // Default: 9223372036854775807

        /** The threshold is given as a ratio of the corpus size */
        double densepostinglistthreshold; // Default: 0.40

        /** Specification of tensor type if this attribute is of type TENSOR. */
        vespalib::string tensortype; // Default: ""

        /**
         * Whether this is an imported attribute (from parent document db) or
         * not.
         */
        bool imported; // Default: false

        /**
         * Max amount of uncommitted memory during feed. (Default just shy of
         * 128k)
         */
        int64_t maxuncommittedmemory; // Default: 130000

        /**
         * The distance metric to use for nearest neighbor search.
         * Is only used when the attribute is a 1-dimensional indexed tensor.
         */
        Distancemetric distancemetric; // Default: EUCLIDEAN
        Index index;
    };
    typedef std::vector<Attribute> AttributeVector;

    typedef std::map<vespalib::string, Attribute> AttributeMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalAttributesType(const ::config::ConfigValue & __value);
    InternalAttributesType(const ::config::ConfigDataBuffer & __value);
    InternalAttributesType(const ::config::ConfigPayload & __payload);
    InternalAttributesType() ;
    InternalAttributesType(const InternalAttributesType & __rhs);
    InternalAttributesType & operator = (const InternalAttributesType & __rhs);
    InternalAttributesType(InternalAttributesType && __rhs) noexcept;
    InternalAttributesType & operator = (InternalAttributesType && __rhs) noexcept;
    ~InternalAttributesType();

    bool operator==(const InternalAttributesType& __rhs) const noexcept;
    bool operator!=(const InternalAttributesType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    AttributeVector attribute;
};

} // namespace internal

typedef internal::InternalAttributesType AttributesConfigBuilder;
typedef const internal::InternalAttributesType AttributesConfig;

} // namespace vespa::config::search
#endif // VESPA_config_VESPA_CONFIG_SEARCH_ATTRIBUTES_H
