#include "config-bucketspaces.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalBucketspacesType::CONFIG_DEF_MD5("52ea664c409e9f9a0cb0032f0bfe1df8");
const vespalib::string InternalBucketspacesType::CONFIG_DEF_NAME("bucketspaces");
const vespalib::string InternalBucketspacesType::CONFIG_DEF_NAMESPACE("vespa.config.content.core");
const int64_t InternalBucketspacesType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content.core",
"documenttype[].name string",
"documenttype[].bucketspace string",
};
const ::config::StringVector InternalBucketspacesType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalBucketspacesType::Documenttype::Documenttype() noexcept
    : name(),
      bucketspace()
{
}

InternalBucketspacesType::Documenttype::Documenttype(const Documenttype & __rhs) noexcept = default;
InternalBucketspacesType::Documenttype & InternalBucketspacesType::Documenttype::operator = (const Documenttype & __rhs) noexcept = default;
InternalBucketspacesType::Documenttype::Documenttype(Documenttype && __rhs) noexcept = default;
InternalBucketspacesType::Documenttype & InternalBucketspacesType::Documenttype::operator = (Documenttype && __rhs) noexcept = default;
InternalBucketspacesType::Documenttype::~Documenttype() = default; 
InternalBucketspacesType::Documenttype::Documenttype(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    bucketspace = ConfigParser::parse<vespalib::string>("bucketspace", __lines);
    ConfigParser::stripLinesForKey("bucketspace", __remainingValuesToParse);
}

bool
InternalBucketspacesType::Documenttype::
operator==(const Documenttype& __rhs) const noexcept
{
    return (name == __rhs.name &&
            bucketspace == __rhs.bucketspace);
}

bool
InternalBucketspacesType::Documenttype::
operator!=(const Documenttype& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalBucketspacesType::Documenttype::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("bucketspace");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(bucketspace));
    }
}

InternalBucketspacesType::Documenttype::Documenttype(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    bucketspace = __inspector["bucketspace"]["value"].asString().make_string();
}

InternalBucketspacesType::Documenttype::Documenttype(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    bucketspace = ::config::internal::ValueConverter<vespalib::string>()("bucketspace", __inspector["bucketspace"]);
}

InternalBucketspacesType::InternalBucketspacesType()
    : documenttype()
{
}

InternalBucketspacesType::InternalBucketspacesType(const InternalBucketspacesType & __rhs) = default;
InternalBucketspacesType & InternalBucketspacesType::operator =(const InternalBucketspacesType & __rhs) = default;
InternalBucketspacesType::InternalBucketspacesType(InternalBucketspacesType && __rhs) noexcept = default;
InternalBucketspacesType & InternalBucketspacesType::operator =(InternalBucketspacesType && __rhs) noexcept = default;
InternalBucketspacesType::~InternalBucketspacesType() = default; 
InternalBucketspacesType::InternalBucketspacesType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        documenttype = ConfigParser::parseArray<DocumenttypeVector>("documenttype", __lines);
        ConfigParser::stripLinesForKey("documenttype", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalBucketspacesType::
operator==(const InternalBucketspacesType& __rhs) const noexcept
{
    return (documenttype == __rhs.documenttype);
}

bool
InternalBucketspacesType::
operator!=(const InternalBucketspacesType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalBucketspacesType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("documenttype");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : documenttype) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalBucketspacesType::InternalBucketspacesType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["documenttype"]["value"].children(); __i++) {
        documenttype.push_back(Documenttype(__inspector["documenttype"]["value"][__i]["value"]));
    }
}

InternalBucketspacesType::InternalBucketspacesType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<DocumenttypeVector> __documenttypeInserter(documenttype);
    __inspector["documenttype"].traverse(__documenttypeInserter);
}

} // namespace internal

} // namespace vespa::config::content::core
