#include "config-cluster-info.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace cloud::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalClusterInfoType::CONFIG_DEF_MD5("1ac343c1a3847be9291c3e07b7916f24");
const vespalib::string InternalClusterInfoType::CONFIG_DEF_NAME("cluster-info");
const vespalib::string InternalClusterInfoType::CONFIG_DEF_NAMESPACE("cloud.config");
const int64_t InternalClusterInfoType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=cloud.config",
"clusterId string",
"nodeCount int",
"nodeIndices[] int",
"services[].index int default=-1",
"services[].hostname string default=\"(nohostname)\"",
"services[].ports[].number int default=-1",
"services[].ports[].tags string default=\"(notags)\"",
};
const ::config::StringVector InternalClusterInfoType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalClusterInfoType::Services::Ports::Ports() noexcept
    : number(-1),
      tags("(notags)")
{
}

InternalClusterInfoType::Services::Ports::Ports(const Ports & __rhs) noexcept = default;
InternalClusterInfoType::Services::Ports & InternalClusterInfoType::Services::Ports::operator = (const Ports & __rhs) noexcept = default;
InternalClusterInfoType::Services::Ports::Ports(Ports && __rhs) noexcept = default;
InternalClusterInfoType::Services::Ports & InternalClusterInfoType::Services::Ports::operator = (Ports && __rhs) noexcept = default;
InternalClusterInfoType::Services::Ports::~Ports() = default; 
InternalClusterInfoType::Services::Ports::Ports(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    number = ConfigParser::parse<int32_t>("number", __lines, -1);
    ConfigParser::stripLinesForKey("number", __remainingValuesToParse);
    tags = ConfigParser::parse<vespalib::string>("tags", __lines, "(notags)");
    ConfigParser::stripLinesForKey("tags", __remainingValuesToParse);
}

bool
InternalClusterInfoType::Services::Ports::operator==(const Ports& __rhs) const noexcept
{
    return (number == __rhs.number &&
            tags == __rhs.tags);
}

bool
InternalClusterInfoType::Services::Ports::operator!=(const Ports& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalClusterInfoType::Services::Ports::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("number");
        __c.setString("type", "int");
        __c.setLong("value", number);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tags");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(tags));
    }
}

InternalClusterInfoType::Services::Ports::Ports(const vespalib::slime::Inspector & __inspector)
{
    number = __inspector["number"]["value"].asLong();
    tags = __inspector["tags"]["value"].asString().make_string();
}

InternalClusterInfoType::Services::Ports::Ports(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    number = ::config::internal::ValueConverter<int32_t>()(__inspector["number"], -1);
    tags = ::config::internal::ValueConverter<vespalib::string>()(__inspector["tags"], "(notags)");
}

InternalClusterInfoType::Services::Services()
    : index(-1),
      hostname("(nohostname)"),
      ports()
{
}

InternalClusterInfoType::Services::Services(const Services & __rhs) = default;
InternalClusterInfoType::Services & InternalClusterInfoType::Services::operator = (const Services & __rhs) = default;
InternalClusterInfoType::Services::Services(Services && __rhs) noexcept = default;
InternalClusterInfoType::Services & InternalClusterInfoType::Services::operator = (Services && __rhs) noexcept = default;
InternalClusterInfoType::Services::~Services() = default; 
InternalClusterInfoType::Services::Services(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    index = ConfigParser::parse<int32_t>("index", __lines, -1);
    ConfigParser::stripLinesForKey("index", __remainingValuesToParse);
    hostname = ConfigParser::parse<vespalib::string>("hostname", __lines, "(nohostname)");
    ConfigParser::stripLinesForKey("hostname", __remainingValuesToParse);
    ports = ConfigParser::parseArray<PortsVector>("ports", __lines);
    ConfigParser::stripLinesForKey("ports", __remainingValuesToParse);
}

bool
InternalClusterInfoType::Services::operator==(const Services& __rhs) const noexcept
{
    return (index == __rhs.index &&
            hostname == __rhs.hostname &&
            ports == __rhs.ports);
}

bool
InternalClusterInfoType::Services::operator!=(const Services& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalClusterInfoType::Services::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("index");
        __c.setString("type", "int");
        __c.setLong("value", index);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("hostname");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(hostname));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ports");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : ports) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalClusterInfoType::Services::Services(const vespalib::slime::Inspector & __inspector)
{
    index = __inspector["index"]["value"].asLong();
    hostname = __inspector["hostname"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["ports"]["value"].children(); __i++) {
        ports.push_back(Ports(__inspector["ports"]["value"][__i]["value"]));
    }
}

InternalClusterInfoType::Services::Services(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    index = ::config::internal::ValueConverter<int32_t>()(__inspector["index"], -1);
    hostname = ::config::internal::ValueConverter<vespalib::string>()(__inspector["hostname"], "(nohostname)");
    ::config::internal::VectorInserter<PortsVector> __portsInserter(ports);
    __inspector["ports"].traverse(__portsInserter);
}

InternalClusterInfoType::InternalClusterInfoType()
    : clusterId(),
      nodeCount(0),
      nodeIndices(),
      services()
{
}

InternalClusterInfoType::InternalClusterInfoType(const InternalClusterInfoType & __rhs) = default;
InternalClusterInfoType & InternalClusterInfoType::operator =(const InternalClusterInfoType & __rhs) = default;
InternalClusterInfoType::InternalClusterInfoType(InternalClusterInfoType && __rhs) noexcept = default;
InternalClusterInfoType & InternalClusterInfoType::operator =(InternalClusterInfoType && __rhs) noexcept = default;
InternalClusterInfoType::~InternalClusterInfoType() = default; 
InternalClusterInfoType::InternalClusterInfoType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        clusterId = ConfigParser::parse<vespalib::string>("clusterId", __lines);
        ConfigParser::stripLinesForKey("clusterId", __remainingValuesToParse);
        nodeCount = ConfigParser::parse<int32_t>("nodeCount", __lines);
        ConfigParser::stripLinesForKey("nodeCount", __remainingValuesToParse);
        nodeIndices = ConfigParser::parseArray<::config::IntVector>("nodeIndices", __lines);
        ConfigParser::stripLinesForKey("nodeIndices", __remainingValuesToParse);
        services = ConfigParser::parseArray<ServicesVector>("services", __lines);
        ConfigParser::stripLinesForKey("services", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalClusterInfoType::operator==(const InternalClusterInfoType& __rhs) const noexcept
{
    return (clusterId == __rhs.clusterId &&
            nodeCount == __rhs.nodeCount &&
            nodeIndices == __rhs.nodeIndices &&
            services == __rhs.services);
}

bool
InternalClusterInfoType::operator!=(const InternalClusterInfoType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalClusterInfoType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("clusterId");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(clusterId));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("nodeCount");
        __c.setString("type", "int");
        __c.setLong("value", nodeCount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("nodeIndices");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : nodeIndices) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "int");
            __c3.setLong("value", child);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("services");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : services) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalClusterInfoType::InternalClusterInfoType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    clusterId = __inspector["clusterId"]["value"].asString().make_string();
    nodeCount = __inspector["nodeCount"]["value"].asLong();
    for (size_t __i = 0; __i < __inspector["nodeIndices"]["value"].children(); __i++) {
        nodeIndices.push_back(__inspector["nodeIndices"]["value"][__i]["value"].asLong());
    }
    for (size_t __i = 0; __i < __inspector["services"]["value"].children(); __i++) {
        services.push_back(Services(__inspector["services"]["value"][__i]["value"]));
    }
}

InternalClusterInfoType::InternalClusterInfoType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    clusterId = ::config::internal::ValueConverter<vespalib::string>()("clusterId", __inspector["clusterId"]);
    nodeCount = ::config::internal::ValueConverter<int32_t>()("nodeCount", __inspector["nodeCount"]);
    ::config::internal::VectorInserter<::config::IntVector> __nodeIndicesInserter(nodeIndices);
    __inspector["nodeIndices"].traverse(__nodeIndicesInserter);
    ::config::internal::VectorInserter<ServicesVector> __servicesInserter(services);
    __inspector["services"].traverse(__servicesInserter);
}

} // namespace internal

} // namespace cloud::config
