#include "config-cluster-list.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace cloud::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalClusterListType::CONFIG_DEF_MD5("60753c6ae77aed1d217248dc8118fba7");
const vespalib::string InternalClusterListType::CONFIG_DEF_NAME("cluster-list");
const vespalib::string InternalClusterListType::CONFIG_DEF_NAMESPACE("cloud.config");
const int64_t InternalClusterListType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=cloud.config",
"storage[].name string",
"storage[].configid string default=\"\"",
};
const ::config::StringVector InternalClusterListType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalClusterListType::Storage::Storage() noexcept
    : name(),
      configid("")
{
}

InternalClusterListType::Storage::Storage(const Storage & __rhs) noexcept = default;
InternalClusterListType::Storage & InternalClusterListType::Storage::operator = (const Storage & __rhs) noexcept = default;
InternalClusterListType::Storage::Storage(Storage && __rhs) noexcept = default;
InternalClusterListType::Storage & InternalClusterListType::Storage::operator = (Storage && __rhs) noexcept = default;
InternalClusterListType::Storage::~Storage() = default; 
InternalClusterListType::Storage::Storage(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    configid = ConfigParser::parse<vespalib::string>("configid", __lines, "");
    ConfigParser::stripLinesForKey("configid", __remainingValuesToParse);
}

bool
InternalClusterListType::Storage::operator==(const Storage& __rhs) const noexcept
{
    return (name == __rhs.name &&
            configid == __rhs.configid);
}

bool
InternalClusterListType::Storage::operator!=(const Storage& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalClusterListType::Storage::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("configid");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(configid));
    }
}

InternalClusterListType::Storage::Storage(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    configid = __inspector["configid"]["value"].asString().make_string();
}

InternalClusterListType::Storage::Storage(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    configid = ::config::internal::ValueConverter<vespalib::string>()(__inspector["configid"], "");
}

InternalClusterListType::InternalClusterListType()
    : storage()
{
}

InternalClusterListType::InternalClusterListType(const InternalClusterListType & __rhs) = default;
InternalClusterListType & InternalClusterListType::operator =(const InternalClusterListType & __rhs) = default;
InternalClusterListType::InternalClusterListType(InternalClusterListType && __rhs) noexcept = default;
InternalClusterListType & InternalClusterListType::operator =(InternalClusterListType && __rhs) noexcept = default;
InternalClusterListType::~InternalClusterListType() = default; 
InternalClusterListType::InternalClusterListType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        storage = ConfigParser::parseArray<StorageVector>("storage", __lines);
        ConfigParser::stripLinesForKey("storage", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalClusterListType::operator==(const InternalClusterListType& __rhs) const noexcept
{
    return (storage == __rhs.storage);
}

bool
InternalClusterListType::operator!=(const InternalClusterListType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalClusterListType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("storage");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : storage) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalClusterListType::InternalClusterListType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["storage"]["value"].children(); __i++) {
        storage.push_back(Storage(__inspector["storage"]["value"][__i]["value"]));
    }
}

InternalClusterListType::InternalClusterListType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<StorageVector> __storageInserter(storage);
    __inspector["storage"].traverse(__storageInserter);
}

} // namespace internal

} // namespace cloud::config
