#include "config-configserver.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace cloud::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalConfigserverType::CONFIG_DEF_MD5("ae4934a53979edc9e34bb683f720295a");
const vespalib::string InternalConfigserverType::CONFIG_DEF_NAME("configserver");
const vespalib::string InternalConfigserverType::CONFIG_DEF_NAMESPACE("cloud.config");
const int64_t InternalConfigserverType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=cloud.config",
"rpcport int default=19070",
"httpport int default=19071",
"numRpcThreads int default=0",
"zookeeperserver[].hostname string",
"zookeeperserver[].port int default=2181",
"zookeeper.barrierTimeout long default=360",
"zookeeperLocalhostAffinity bool default=true",
"sessionLifetime long default=3600",
"barrierWaitForAllTimeout long default=5",
"configModelPluginDir[] string",
"configServerDBDir string default=\"var/db/vespa/config_server/serverdb/\"",
"configDefinitionsDir string default=\"share/vespa/configdefinitions/\"",
"fileReferencesDir string default=\"var/db/vespa/filedistribution/\"",
"maxApplicationPackageSize long default=8589934592",
"masterGeneration long default=0",
"multitenant bool default=false",
"numDelayedResponseThreads int default=1",
"serverId string default=\"localhost\"",
"hostedVespa bool default=false",
"numParallelTenantLoaders int default=4",
"numRedeploymentThreads int default=4",
"applicationDirectory string default=\"conf/configserver-app\"",
"cloud string default=\"default\"",
"environment string default=\"prod\"",
"region string default=\"default\"",
"system string default=\"main\"",
"zoneDnsSuffixes[] string",
"maxgetconfigclients int default=1000000",
"maxoutputbuffersize int default=65536",
"useVespaVersionInRequest bool default=true",
"payloadCompressionType enum { UNCOMPRESSED, LZ4 } default=LZ4",
"loadBalancerAddress string default=\"\"",
"athenzDnsSuffix string default=\"\"",
"ztsUrl string default=\"\"",
"maintainerIntervalMinutes int default=30",
"keepUnusedFileReferencesMinutes int default=300",
"keepSessionsWithUnknownStatusHours int default=24",
"maxDurationOfBootstrap long default=7200",
"sleepTimeWhenRedeployingFails long default=15",
"canReturnEmptySentinelConfig bool default=false",
"serverNodeType enum {config, controller} default=config",
};
const ::config::StringVector InternalConfigserverType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalConfigserverType::Zookeeperserver::Zookeeperserver() noexcept
    : hostname(),
      port(2181)
{
}

InternalConfigserverType::Zookeeperserver::Zookeeperserver(const Zookeeperserver & __rhs) noexcept = default;
InternalConfigserverType::Zookeeperserver & InternalConfigserverType::Zookeeperserver::operator = (const Zookeeperserver & __rhs) noexcept = default;
InternalConfigserverType::Zookeeperserver::Zookeeperserver(Zookeeperserver && __rhs) noexcept = default;
InternalConfigserverType::Zookeeperserver & InternalConfigserverType::Zookeeperserver::operator = (Zookeeperserver && __rhs) noexcept = default;
InternalConfigserverType::Zookeeperserver::~Zookeeperserver() = default; 
InternalConfigserverType::Zookeeperserver::Zookeeperserver(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    hostname = ConfigParser::parse<vespalib::string>("hostname", __lines);
    ConfigParser::stripLinesForKey("hostname", __remainingValuesToParse);
    port = ConfigParser::parse<int32_t>("port", __lines, 2181);
    ConfigParser::stripLinesForKey("port", __remainingValuesToParse);
}

bool
InternalConfigserverType::Zookeeperserver::
operator==(const Zookeeperserver& __rhs) const noexcept
{
    return (hostname == __rhs.hostname &&
            port == __rhs.port);
}

bool
InternalConfigserverType::Zookeeperserver::
operator!=(const Zookeeperserver& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalConfigserverType::Zookeeperserver::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("hostname");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(hostname));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("port");
        __c.setString("type", "int");
        __c.setLong("value", port);
    }
}

InternalConfigserverType::Zookeeperserver::Zookeeperserver(const vespalib::slime::Inspector & __inspector)
{
    hostname = __inspector["hostname"]["value"].asString().make_string();
    port = __inspector["port"]["value"].asLong();
}

InternalConfigserverType::Zookeeperserver::Zookeeperserver(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    hostname = ::config::internal::ValueConverter<vespalib::string>()("hostname", __inspector["hostname"]);
    port = ::config::internal::ValueConverter<int32_t>()(__inspector["port"], 2181);
}

InternalConfigserverType::Zookeeper::Zookeeper() noexcept
    : barrierTimeout(360)
{
}

InternalConfigserverType::Zookeeper::Zookeeper(const Zookeeper & __rhs) noexcept = default;
InternalConfigserverType::Zookeeper & InternalConfigserverType::Zookeeper::operator = (const Zookeeper & __rhs) noexcept = default;
InternalConfigserverType::Zookeeper::Zookeeper(Zookeeper && __rhs) noexcept = default;
InternalConfigserverType::Zookeeper & InternalConfigserverType::Zookeeper::operator = (Zookeeper && __rhs) noexcept = default;
InternalConfigserverType::Zookeeper::~Zookeeper() = default; 
InternalConfigserverType::Zookeeper::Zookeeper(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    barrierTimeout = ConfigParser::parse<int64_t>("barrierTimeout", __lines, 360);
    ConfigParser::stripLinesForKey("barrierTimeout", __remainingValuesToParse);
}

bool
InternalConfigserverType::Zookeeper::operator==(const Zookeeper& __rhs) const noexcept
{
    return (barrierTimeout == __rhs.barrierTimeout);
}

bool
InternalConfigserverType::Zookeeper::operator!=(const Zookeeper& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalConfigserverType::Zookeeper::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("barrierTimeout");
        __c.setString("type", "long");
        __c.setLong("value", barrierTimeout);
    }
}

InternalConfigserverType::Zookeeper::Zookeeper(const vespalib::slime::Inspector & __inspector)
{
    barrierTimeout = __inspector["barrierTimeout"]["value"].asLong();
}

InternalConfigserverType::Zookeeper::Zookeeper(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    barrierTimeout = ::config::internal::ValueConverter<int64_t>()(__inspector["barrierTimeout"], 360);
}

InternalConfigserverType::PayloadCompressionType
InternalConfigserverType::getPayloadCompressionType(const vespalib::string& name)
{
    if (name == "UNCOMPRESSED") {
        return PayloadCompressionType::UNCOMPRESSED;
    } else if (name == "LZ4") {
        return PayloadCompressionType::LZ4;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalConfigserverType::getPayloadCompressionTypeName(PayloadCompressionType t)
{
    switch (t) {
        case PayloadCompressionType::UNCOMPRESSED: return "UNCOMPRESSED";
        case PayloadCompressionType::LZ4: return "LZ4";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalConfigserverType::PayloadCompressionType InternalConfigserverType::InternalPayloadCompressionTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalConfigserverType::getPayloadCompressionType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalConfigserverType::PayloadCompressionType InternalConfigserverType::InternalPayloadCompressionTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalConfigserverType::getPayloadCompressionType(__inspector.asString().make_string());
}
InternalConfigserverType::PayloadCompressionType InternalConfigserverType::InternalPayloadCompressionTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, PayloadCompressionType __eDefault) {
    if (__inspector.valid()) {
        return InternalConfigserverType::getPayloadCompressionType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalConfigserverType::ServerNodeType
InternalConfigserverType::getServerNodeType(const vespalib::string& name)
{
    if (name == "config") {
        return ServerNodeType::config;
    } else if (name == "controller") {
        return ServerNodeType::controller;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalConfigserverType::getServerNodeTypeName(ServerNodeType t)
{
    switch (t) {
        case ServerNodeType::config: return "config";
        case ServerNodeType::controller: return "controller";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalConfigserverType::ServerNodeType InternalConfigserverType::InternalServerNodeTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalConfigserverType::getServerNodeType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalConfigserverType::ServerNodeType InternalConfigserverType::InternalServerNodeTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalConfigserverType::getServerNodeType(__inspector.asString().make_string());
}
InternalConfigserverType::ServerNodeType InternalConfigserverType::InternalServerNodeTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, ServerNodeType __eDefault) {
    if (__inspector.valid()) {
        return InternalConfigserverType::getServerNodeType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalConfigserverType::InternalConfigserverType()
    : rpcport(19070),
      httpport(19071),
      numRpcThreads(0),
      zookeeperserver(),
      zookeeper(),
      zookeeperLocalhostAffinity(true),
      sessionLifetime(3600),
      barrierWaitForAllTimeout(5),
      configModelPluginDir(),
      configServerDBDir("var/db/vespa/config_server/serverdb/"),
      configDefinitionsDir("share/vespa/configdefinitions/"),
      fileReferencesDir("var/db/vespa/filedistribution/"),
      maxApplicationPackageSize(8589934592),
      masterGeneration(0),
      multitenant(false),
      numDelayedResponseThreads(1),
      serverId("localhost"),
      hostedVespa(false),
      numParallelTenantLoaders(4),
      numRedeploymentThreads(4),
      applicationDirectory("conf/configserver-app"),
      cloud("default"),
      environment("prod"),
      region("default"),
      system("main"),
      zoneDnsSuffixes(),
      maxgetconfigclients(1000000),
      maxoutputbuffersize(65536),
      useVespaVersionInRequest(true),
      payloadCompressionType(PayloadCompressionType::LZ4),
      loadBalancerAddress(""),
      athenzDnsSuffix(""),
      ztsUrl(""),
      maintainerIntervalMinutes(30),
      keepUnusedFileReferencesMinutes(300),
      keepSessionsWithUnknownStatusHours(24),
      maxDurationOfBootstrap(7200),
      sleepTimeWhenRedeployingFails(15),
      canReturnEmptySentinelConfig(false),
      serverNodeType(ServerNodeType::config)
{
}

InternalConfigserverType::InternalConfigserverType(const InternalConfigserverType & __rhs) = default;
InternalConfigserverType & InternalConfigserverType::operator =(const InternalConfigserverType & __rhs) = default;
InternalConfigserverType::InternalConfigserverType(InternalConfigserverType && __rhs) noexcept = default;
InternalConfigserverType & InternalConfigserverType::operator =(InternalConfigserverType && __rhs) noexcept = default;
InternalConfigserverType::~InternalConfigserverType() = default; 
InternalConfigserverType::InternalConfigserverType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        rpcport = ConfigParser::parse<int32_t>("rpcport", __lines, 19070);
        ConfigParser::stripLinesForKey("rpcport", __remainingValuesToParse);
        httpport = ConfigParser::parse<int32_t>("httpport", __lines, 19071);
        ConfigParser::stripLinesForKey("httpport", __remainingValuesToParse);
        numRpcThreads = ConfigParser::parse<int32_t>("numRpcThreads", __lines, 0);
        ConfigParser::stripLinesForKey("numRpcThreads", __remainingValuesToParse);
        zookeeperserver = ConfigParser::parseArray<ZookeeperserverVector>("zookeeperserver", __lines);
        ConfigParser::stripLinesForKey("zookeeperserver", __remainingValuesToParse);
        zookeeper = ConfigParser::parseStruct<Zookeeper>("zookeeper", __lines);
        ConfigParser::stripLinesForKey("zookeeper", __remainingValuesToParse);
        zookeeperLocalhostAffinity = ConfigParser::parse<bool>("zookeeperLocalhostAffinity", __lines, true);
        ConfigParser::stripLinesForKey("zookeeperLocalhostAffinity", __remainingValuesToParse);
        sessionLifetime = ConfigParser::parse<int64_t>("sessionLifetime", __lines, 3600);
        ConfigParser::stripLinesForKey("sessionLifetime", __remainingValuesToParse);
        barrierWaitForAllTimeout = ConfigParser::parse<int64_t>("barrierWaitForAllTimeout", __lines, 5);
        ConfigParser::stripLinesForKey("barrierWaitForAllTimeout", __remainingValuesToParse);
        configModelPluginDir = ConfigParser::parseArray<::config::StringVector>("configModelPluginDir", __lines);
        ConfigParser::stripLinesForKey("configModelPluginDir", __remainingValuesToParse);
        configServerDBDir = ConfigParser::parse<vespalib::string>("configServerDBDir", __lines, "var/db/vespa/config_server/serverdb/");
        ConfigParser::stripLinesForKey("configServerDBDir", __remainingValuesToParse);
        configDefinitionsDir = ConfigParser::parse<vespalib::string>("configDefinitionsDir", __lines, "share/vespa/configdefinitions/");
        ConfigParser::stripLinesForKey("configDefinitionsDir", __remainingValuesToParse);
        fileReferencesDir = ConfigParser::parse<vespalib::string>("fileReferencesDir", __lines, "var/db/vespa/filedistribution/");
        ConfigParser::stripLinesForKey("fileReferencesDir", __remainingValuesToParse);
        maxApplicationPackageSize = ConfigParser::parse<int64_t>("maxApplicationPackageSize", __lines, 8589934592);
        ConfigParser::stripLinesForKey("maxApplicationPackageSize", __remainingValuesToParse);
        masterGeneration = ConfigParser::parse<int64_t>("masterGeneration", __lines, 0);
        ConfigParser::stripLinesForKey("masterGeneration", __remainingValuesToParse);
        multitenant = ConfigParser::parse<bool>("multitenant", __lines, false);
        ConfigParser::stripLinesForKey("multitenant", __remainingValuesToParse);
        numDelayedResponseThreads = ConfigParser::parse<int32_t>("numDelayedResponseThreads", __lines, 1);
        ConfigParser::stripLinesForKey("numDelayedResponseThreads", __remainingValuesToParse);
        serverId = ConfigParser::parse<vespalib::string>("serverId", __lines, "localhost");
        ConfigParser::stripLinesForKey("serverId", __remainingValuesToParse);
        hostedVespa = ConfigParser::parse<bool>("hostedVespa", __lines, false);
        ConfigParser::stripLinesForKey("hostedVespa", __remainingValuesToParse);
        numParallelTenantLoaders = ConfigParser::parse<int32_t>("numParallelTenantLoaders", __lines, 4);
        ConfigParser::stripLinesForKey("numParallelTenantLoaders", __remainingValuesToParse);
        numRedeploymentThreads = ConfigParser::parse<int32_t>("numRedeploymentThreads", __lines, 4);
        ConfigParser::stripLinesForKey("numRedeploymentThreads", __remainingValuesToParse);
        applicationDirectory = ConfigParser::parse<vespalib::string>("applicationDirectory", __lines, "conf/configserver-app");
        ConfigParser::stripLinesForKey("applicationDirectory", __remainingValuesToParse);
        cloud = ConfigParser::parse<vespalib::string>("cloud", __lines, "default");
        ConfigParser::stripLinesForKey("cloud", __remainingValuesToParse);
        environment = ConfigParser::parse<vespalib::string>("environment", __lines, "prod");
        ConfigParser::stripLinesForKey("environment", __remainingValuesToParse);
        region = ConfigParser::parse<vespalib::string>("region", __lines, "default");
        ConfigParser::stripLinesForKey("region", __remainingValuesToParse);
        system = ConfigParser::parse<vespalib::string>("system", __lines, "main");
        ConfigParser::stripLinesForKey("system", __remainingValuesToParse);
        zoneDnsSuffixes = ConfigParser::parseArray<::config::StringVector>("zoneDnsSuffixes", __lines);
        ConfigParser::stripLinesForKey("zoneDnsSuffixes", __remainingValuesToParse);
        maxgetconfigclients = ConfigParser::parse<int32_t>("maxgetconfigclients", __lines, 1000000);
        ConfigParser::stripLinesForKey("maxgetconfigclients", __remainingValuesToParse);
        maxoutputbuffersize = ConfigParser::parse<int32_t>("maxoutputbuffersize", __lines, 65536);
        ConfigParser::stripLinesForKey("maxoutputbuffersize", __remainingValuesToParse);
        useVespaVersionInRequest = ConfigParser::parse<bool>("useVespaVersionInRequest", __lines, true);
        ConfigParser::stripLinesForKey("useVespaVersionInRequest", __remainingValuesToParse);
        payloadCompressionType = getPayloadCompressionType(ConfigParser::parse<vespalib::string>("payloadCompressionType", __lines, "LZ4"));
        ConfigParser::stripLinesForKey("payloadCompressionType", __remainingValuesToParse);
        loadBalancerAddress = ConfigParser::parse<vespalib::string>("loadBalancerAddress", __lines, "");
        ConfigParser::stripLinesForKey("loadBalancerAddress", __remainingValuesToParse);
        athenzDnsSuffix = ConfigParser::parse<vespalib::string>("athenzDnsSuffix", __lines, "");
        ConfigParser::stripLinesForKey("athenzDnsSuffix", __remainingValuesToParse);
        ztsUrl = ConfigParser::parse<vespalib::string>("ztsUrl", __lines, "");
        ConfigParser::stripLinesForKey("ztsUrl", __remainingValuesToParse);
        maintainerIntervalMinutes = ConfigParser::parse<int32_t>("maintainerIntervalMinutes", __lines, 30);
        ConfigParser::stripLinesForKey("maintainerIntervalMinutes", __remainingValuesToParse);
        keepUnusedFileReferencesMinutes = ConfigParser::parse<int32_t>("keepUnusedFileReferencesMinutes", __lines, 300);
        ConfigParser::stripLinesForKey("keepUnusedFileReferencesMinutes", __remainingValuesToParse);
        keepSessionsWithUnknownStatusHours = ConfigParser::parse<int32_t>("keepSessionsWithUnknownStatusHours", __lines, 24);
        ConfigParser::stripLinesForKey("keepSessionsWithUnknownStatusHours", __remainingValuesToParse);
        maxDurationOfBootstrap = ConfigParser::parse<int64_t>("maxDurationOfBootstrap", __lines, 7200);
        ConfigParser::stripLinesForKey("maxDurationOfBootstrap", __remainingValuesToParse);
        sleepTimeWhenRedeployingFails = ConfigParser::parse<int64_t>("sleepTimeWhenRedeployingFails", __lines, 15);
        ConfigParser::stripLinesForKey("sleepTimeWhenRedeployingFails", __remainingValuesToParse);
        canReturnEmptySentinelConfig = ConfigParser::parse<bool>("canReturnEmptySentinelConfig", __lines, false);
        ConfigParser::stripLinesForKey("canReturnEmptySentinelConfig", __remainingValuesToParse);
        serverNodeType = getServerNodeType(ConfigParser::parse<vespalib::string>("serverNodeType", __lines, "config"));
        ConfigParser::stripLinesForKey("serverNodeType", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalConfigserverType::
operator==(const InternalConfigserverType& __rhs) const noexcept
{
    return (rpcport == __rhs.rpcport &&
            httpport == __rhs.httpport &&
            numRpcThreads == __rhs.numRpcThreads &&
            zookeeperserver == __rhs.zookeeperserver &&
            zookeeper == __rhs.zookeeper &&
            zookeeperLocalhostAffinity == __rhs.zookeeperLocalhostAffinity &&
            sessionLifetime == __rhs.sessionLifetime &&
            barrierWaitForAllTimeout == __rhs.barrierWaitForAllTimeout &&
            configModelPluginDir == __rhs.configModelPluginDir &&
            configServerDBDir == __rhs.configServerDBDir &&
            configDefinitionsDir == __rhs.configDefinitionsDir &&
            fileReferencesDir == __rhs.fileReferencesDir &&
            maxApplicationPackageSize == __rhs.maxApplicationPackageSize &&
            masterGeneration == __rhs.masterGeneration &&
            multitenant == __rhs.multitenant &&
            numDelayedResponseThreads == __rhs.numDelayedResponseThreads &&
            serverId == __rhs.serverId &&
            hostedVespa == __rhs.hostedVespa &&
            numParallelTenantLoaders == __rhs.numParallelTenantLoaders &&
            numRedeploymentThreads == __rhs.numRedeploymentThreads &&
            applicationDirectory == __rhs.applicationDirectory &&
            cloud == __rhs.cloud &&
            environment == __rhs.environment &&
            region == __rhs.region &&
            system == __rhs.system &&
            zoneDnsSuffixes == __rhs.zoneDnsSuffixes &&
            maxgetconfigclients == __rhs.maxgetconfigclients &&
            maxoutputbuffersize == __rhs.maxoutputbuffersize &&
            useVespaVersionInRequest == __rhs.useVespaVersionInRequest &&
            payloadCompressionType == __rhs.payloadCompressionType &&
            loadBalancerAddress == __rhs.loadBalancerAddress &&
            athenzDnsSuffix == __rhs.athenzDnsSuffix &&
            ztsUrl == __rhs.ztsUrl &&
            maintainerIntervalMinutes == __rhs.maintainerIntervalMinutes &&
            keepUnusedFileReferencesMinutes == __rhs.keepUnusedFileReferencesMinutes &&
            keepSessionsWithUnknownStatusHours == __rhs.keepSessionsWithUnknownStatusHours &&
            maxDurationOfBootstrap == __rhs.maxDurationOfBootstrap &&
            sleepTimeWhenRedeployingFails == __rhs.sleepTimeWhenRedeployingFails &&
            canReturnEmptySentinelConfig == __rhs.canReturnEmptySentinelConfig &&
            serverNodeType == __rhs.serverNodeType);
}

bool
InternalConfigserverType::
operator!=(const InternalConfigserverType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalConfigserverType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("rpcport");
        __c.setString("type", "int");
        __c.setLong("value", rpcport);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("httpport");
        __c.setString("type", "int");
        __c.setLong("value", httpport);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("numRpcThreads");
        __c.setString("type", "int");
        __c.setLong("value", numRpcThreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("zookeeperserver");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : zookeeperserver) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("zookeeper");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        zookeeper.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("zookeeperLocalhostAffinity");
        __c.setString("type", "bool");
        __c.setBool("value", zookeeperLocalhostAffinity);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("sessionLifetime");
        __c.setString("type", "long");
        __c.setLong("value", sessionLifetime);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("barrierWaitForAllTimeout");
        __c.setString("type", "long");
        __c.setLong("value", barrierWaitForAllTimeout);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("configModelPluginDir");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : configModelPluginDir) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("configServerDBDir");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(configServerDBDir));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("configDefinitionsDir");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(configDefinitionsDir));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fileReferencesDir");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(fileReferencesDir));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxApplicationPackageSize");
        __c.setString("type", "long");
        __c.setLong("value", maxApplicationPackageSize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("masterGeneration");
        __c.setString("type", "long");
        __c.setLong("value", masterGeneration);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("multitenant");
        __c.setString("type", "bool");
        __c.setBool("value", multitenant);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("numDelayedResponseThreads");
        __c.setString("type", "int");
        __c.setLong("value", numDelayedResponseThreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("serverId");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(serverId));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("hostedVespa");
        __c.setString("type", "bool");
        __c.setBool("value", hostedVespa);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("numParallelTenantLoaders");
        __c.setString("type", "int");
        __c.setLong("value", numParallelTenantLoaders);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("numRedeploymentThreads");
        __c.setString("type", "int");
        __c.setLong("value", numRedeploymentThreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("applicationDirectory");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(applicationDirectory));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("cloud");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(cloud));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("environment");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(environment));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("region");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(region));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("system");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(system));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("zoneDnsSuffixes");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : zoneDnsSuffixes) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxgetconfigclients");
        __c.setString("type", "int");
        __c.setLong("value", maxgetconfigclients);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxoutputbuffersize");
        __c.setString("type", "int");
        __c.setLong("value", maxoutputbuffersize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("useVespaVersionInRequest");
        __c.setString("type", "bool");
        __c.setBool("value", useVespaVersionInRequest);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("payloadCompressionType");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getPayloadCompressionTypeName(payloadCompressionType)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("loadBalancerAddress");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(loadBalancerAddress));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("athenzDnsSuffix");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(athenzDnsSuffix));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ztsUrl");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(ztsUrl));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maintainerIntervalMinutes");
        __c.setString("type", "int");
        __c.setLong("value", maintainerIntervalMinutes);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("keepUnusedFileReferencesMinutes");
        __c.setString("type", "int");
        __c.setLong("value", keepUnusedFileReferencesMinutes);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("keepSessionsWithUnknownStatusHours");
        __c.setString("type", "int");
        __c.setLong("value", keepSessionsWithUnknownStatusHours);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxDurationOfBootstrap");
        __c.setString("type", "long");
        __c.setLong("value", maxDurationOfBootstrap);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("sleepTimeWhenRedeployingFails");
        __c.setString("type", "long");
        __c.setLong("value", sleepTimeWhenRedeployingFails);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("canReturnEmptySentinelConfig");
        __c.setString("type", "bool");
        __c.setBool("value", canReturnEmptySentinelConfig);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("serverNodeType");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getServerNodeTypeName(serverNodeType)));
    }
}

InternalConfigserverType::InternalConfigserverType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    rpcport = __inspector["rpcport"]["value"].asLong();
    httpport = __inspector["httpport"]["value"].asLong();
    numRpcThreads = __inspector["numRpcThreads"]["value"].asLong();
    for (size_t __i = 0; __i < __inspector["zookeeperserver"]["value"].children(); __i++) {
        zookeeperserver.push_back(Zookeeperserver(__inspector["zookeeperserver"]["value"][__i]["value"]));
    }
    zookeeper = Zookeeper(__inspector["zookeeper"]["value"]);
    zookeeperLocalhostAffinity = __inspector["zookeeperLocalhostAffinity"]["value"].asBool();
    sessionLifetime = __inspector["sessionLifetime"]["value"].asLong();
    barrierWaitForAllTimeout = __inspector["barrierWaitForAllTimeout"]["value"].asLong();
    for (size_t __i = 0; __i < __inspector["configModelPluginDir"]["value"].children(); __i++) {
        configModelPluginDir.push_back(__inspector["configModelPluginDir"]["value"][__i]["value"].asString().make_string());
    }
    configServerDBDir = __inspector["configServerDBDir"]["value"].asString().make_string();
    configDefinitionsDir = __inspector["configDefinitionsDir"]["value"].asString().make_string();
    fileReferencesDir = __inspector["fileReferencesDir"]["value"].asString().make_string();
    maxApplicationPackageSize = __inspector["maxApplicationPackageSize"]["value"].asLong();
    masterGeneration = __inspector["masterGeneration"]["value"].asLong();
    multitenant = __inspector["multitenant"]["value"].asBool();
    numDelayedResponseThreads = __inspector["numDelayedResponseThreads"]["value"].asLong();
    serverId = __inspector["serverId"]["value"].asString().make_string();
    hostedVespa = __inspector["hostedVespa"]["value"].asBool();
    numParallelTenantLoaders = __inspector["numParallelTenantLoaders"]["value"].asLong();
    numRedeploymentThreads = __inspector["numRedeploymentThreads"]["value"].asLong();
    applicationDirectory = __inspector["applicationDirectory"]["value"].asString().make_string();
    cloud = __inspector["cloud"]["value"].asString().make_string();
    environment = __inspector["environment"]["value"].asString().make_string();
    region = __inspector["region"]["value"].asString().make_string();
    system = __inspector["system"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["zoneDnsSuffixes"]["value"].children(); __i++) {
        zoneDnsSuffixes.push_back(__inspector["zoneDnsSuffixes"]["value"][__i]["value"].asString().make_string());
    }
    maxgetconfigclients = __inspector["maxgetconfigclients"]["value"].asLong();
    maxoutputbuffersize = __inspector["maxoutputbuffersize"]["value"].asLong();
    useVespaVersionInRequest = __inspector["useVespaVersionInRequest"]["value"].asBool();
    payloadCompressionType = getPayloadCompressionType(__inspector["payloadCompressionType"]["value"].asString().make_string());
    loadBalancerAddress = __inspector["loadBalancerAddress"]["value"].asString().make_string();
    athenzDnsSuffix = __inspector["athenzDnsSuffix"]["value"].asString().make_string();
    ztsUrl = __inspector["ztsUrl"]["value"].asString().make_string();
    maintainerIntervalMinutes = __inspector["maintainerIntervalMinutes"]["value"].asLong();
    keepUnusedFileReferencesMinutes = __inspector["keepUnusedFileReferencesMinutes"]["value"].asLong();
    keepSessionsWithUnknownStatusHours = __inspector["keepSessionsWithUnknownStatusHours"]["value"].asLong();
    maxDurationOfBootstrap = __inspector["maxDurationOfBootstrap"]["value"].asLong();
    sleepTimeWhenRedeployingFails = __inspector["sleepTimeWhenRedeployingFails"]["value"].asLong();
    canReturnEmptySentinelConfig = __inspector["canReturnEmptySentinelConfig"]["value"].asBool();
    serverNodeType = getServerNodeType(__inspector["serverNodeType"]["value"].asString().make_string());
}

InternalConfigserverType::InternalConfigserverType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    rpcport = ::config::internal::ValueConverter<int32_t>()(__inspector["rpcport"], 19070);
    httpport = ::config::internal::ValueConverter<int32_t>()(__inspector["httpport"], 19071);
    numRpcThreads = ::config::internal::ValueConverter<int32_t>()(__inspector["numRpcThreads"], 0);
    ::config::internal::VectorInserter<ZookeeperserverVector> __zookeeperserverInserter(zookeeperserver);
    __inspector["zookeeperserver"].traverse(__zookeeperserverInserter);
    zookeeper = ::config::internal::ValueConverter<Zookeeper>()(__inspector["zookeeper"]);
    zookeeperLocalhostAffinity = ::config::internal::ValueConverter<bool>()(__inspector["zookeeperLocalhostAffinity"], true);
    sessionLifetime = ::config::internal::ValueConverter<int64_t>()(__inspector["sessionLifetime"], 3600);
    barrierWaitForAllTimeout = ::config::internal::ValueConverter<int64_t>()(__inspector["barrierWaitForAllTimeout"], 5);
    ::config::internal::VectorInserter<::config::StringVector> __configModelPluginDirInserter(configModelPluginDir);
    __inspector["configModelPluginDir"].traverse(__configModelPluginDirInserter);
    configServerDBDir = ::config::internal::ValueConverter<vespalib::string>()(__inspector["configServerDBDir"], "var/db/vespa/config_server/serverdb/");
    configDefinitionsDir = ::config::internal::ValueConverter<vespalib::string>()(__inspector["configDefinitionsDir"], "share/vespa/configdefinitions/");
    fileReferencesDir = ::config::internal::ValueConverter<vespalib::string>()(__inspector["fileReferencesDir"], "var/db/vespa/filedistribution/");
    maxApplicationPackageSize = ::config::internal::ValueConverter<int64_t>()(__inspector["maxApplicationPackageSize"], 8589934592);
    masterGeneration = ::config::internal::ValueConverter<int64_t>()(__inspector["masterGeneration"], 0);
    multitenant = ::config::internal::ValueConverter<bool>()(__inspector["multitenant"], false);
    numDelayedResponseThreads = ::config::internal::ValueConverter<int32_t>()(__inspector["numDelayedResponseThreads"], 1);
    serverId = ::config::internal::ValueConverter<vespalib::string>()(__inspector["serverId"], "localhost");
    hostedVespa = ::config::internal::ValueConverter<bool>()(__inspector["hostedVespa"], false);
    numParallelTenantLoaders = ::config::internal::ValueConverter<int32_t>()(__inspector["numParallelTenantLoaders"], 4);
    numRedeploymentThreads = ::config::internal::ValueConverter<int32_t>()(__inspector["numRedeploymentThreads"], 4);
    applicationDirectory = ::config::internal::ValueConverter<vespalib::string>()(__inspector["applicationDirectory"], "conf/configserver-app");
    cloud = ::config::internal::ValueConverter<vespalib::string>()(__inspector["cloud"], "default");
    environment = ::config::internal::ValueConverter<vespalib::string>()(__inspector["environment"], "prod");
    region = ::config::internal::ValueConverter<vespalib::string>()(__inspector["region"], "default");
    system = ::config::internal::ValueConverter<vespalib::string>()(__inspector["system"], "main");
    ::config::internal::VectorInserter<::config::StringVector> __zoneDnsSuffixesInserter(zoneDnsSuffixes);
    __inspector["zoneDnsSuffixes"].traverse(__zoneDnsSuffixesInserter);
    maxgetconfigclients = ::config::internal::ValueConverter<int32_t>()(__inspector["maxgetconfigclients"], 1000000);
    maxoutputbuffersize = ::config::internal::ValueConverter<int32_t>()(__inspector["maxoutputbuffersize"], 65536);
    useVespaVersionInRequest = ::config::internal::ValueConverter<bool>()(__inspector["useVespaVersionInRequest"], true);
    payloadCompressionType = InternalPayloadCompressionTypeConverter()(__inspector["payloadCompressionType"], PayloadCompressionType::LZ4);
    loadBalancerAddress = ::config::internal::ValueConverter<vespalib::string>()(__inspector["loadBalancerAddress"], "");
    athenzDnsSuffix = ::config::internal::ValueConverter<vespalib::string>()(__inspector["athenzDnsSuffix"], "");
    ztsUrl = ::config::internal::ValueConverter<vespalib::string>()(__inspector["ztsUrl"], "");
    maintainerIntervalMinutes = ::config::internal::ValueConverter<int32_t>()(__inspector["maintainerIntervalMinutes"], 30);
    keepUnusedFileReferencesMinutes = ::config::internal::ValueConverter<int32_t>()(__inspector["keepUnusedFileReferencesMinutes"], 300);
    keepSessionsWithUnknownStatusHours = ::config::internal::ValueConverter<int32_t>()(__inspector["keepSessionsWithUnknownStatusHours"], 24);
    maxDurationOfBootstrap = ::config::internal::ValueConverter<int64_t>()(__inspector["maxDurationOfBootstrap"], 7200);
    sleepTimeWhenRedeployingFails = ::config::internal::ValueConverter<int64_t>()(__inspector["sleepTimeWhenRedeployingFails"], 15);
    canReturnEmptySentinelConfig = ::config::internal::ValueConverter<bool>()(__inspector["canReturnEmptySentinelConfig"], false);
    serverNodeType = InternalServerNodeTypeConverter()(__inspector["serverNodeType"], ServerNodeType::config);
}

} // namespace internal

} // namespace cloud::config
