/**
 * @class cloud::config::Configserver
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_CLOUD_CONFIG_CONFIGSERVER_H
#define CLOUD_CONFIG_CLOUD_CONFIG_CONFIGSERVER_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace cloud::config {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalConfigserverType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalConfigserverType> UP;
    class Zookeeperserver {
    public:
        Zookeeperserver(const ::config::StringVector & __lines);
        Zookeeperserver(const vespalib::slime::Inspector & __inspector);
        Zookeeperserver(const ::config::ConfigPayload & __payload);
        Zookeeperserver()  noexcept;
        Zookeeperserver(const Zookeeperserver & __rhs) noexcept;
        Zookeeperserver & operator = (const Zookeeperserver & __rhs) noexcept;
        Zookeeperserver(Zookeeperserver && __rhs) noexcept;
        Zookeeperserver & operator = (Zookeeperserver && __rhs) noexcept;
        ~Zookeeperserver();

        bool operator==(const Zookeeperserver& __rhs) const noexcept;
        bool operator!=(const Zookeeperserver& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** ZooKeeper */
        vespalib::string hostname;
        int32_t port; // Default: 2181
    };
    typedef std::vector<Zookeeperserver> ZookeeperserverVector;

    typedef std::map<vespalib::string, Zookeeperserver> ZookeeperserverMap;

    class Zookeeper {
    public:
        Zookeeper(const ::config::StringVector & __lines);
        Zookeeper(const vespalib::slime::Inspector & __inspector);
        Zookeeper(const ::config::ConfigPayload & __payload);
        Zookeeper()  noexcept;
        Zookeeper(const Zookeeper & __rhs) noexcept;
        Zookeeper & operator = (const Zookeeper & __rhs) noexcept;
        Zookeeper(Zookeeper && __rhs) noexcept;
        Zookeeper & operator = (Zookeeper && __rhs) noexcept;
        ~Zookeeper();

        bool operator==(const Zookeeper& __rhs) const noexcept;
        bool operator!=(const Zookeeper& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * ZK write timeout in seconds, must be long enough to write application
         * packages to other nodes
         */
        int64_t barrierTimeout; // Default: 360
    };
    typedef std::vector<Zookeeper> ZookeeperVector;

    typedef std::map<vespalib::string, Zookeeper> ZookeeperMap;

    enum class PayloadCompressionType { UNCOMPRESSED, LZ4 };
    typedef std::vector<PayloadCompressionType> PayloadCompressionTypeVector;
    typedef std::map<vespalib::string, PayloadCompressionType> PayloadCompressionTypeMap;
    static PayloadCompressionType getPayloadCompressionType(const vespalib::string&);
    static vespalib::string getPayloadCompressionTypeName(PayloadCompressionType e);

    struct InternalPayloadCompressionTypeConverter {
        PayloadCompressionType operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
        PayloadCompressionType operator()(const ::vespalib::slime::Inspector & __inspector);
        PayloadCompressionType operator()(const ::vespalib::slime::Inspector & __inspector, PayloadCompressionType __eDefault);
    };
    enum class ServerNodeType { config, controller };
    typedef std::vector<ServerNodeType> ServerNodeTypeVector;
    typedef std::map<vespalib::string, ServerNodeType> ServerNodeTypeMap;
    static ServerNodeType getServerNodeType(const vespalib::string&);
    static vespalib::string getServerNodeTypeName(ServerNodeType e);

    struct InternalServerNodeTypeConverter {
        ServerNodeType operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
        ServerNodeType operator()(const ::vespalib::slime::Inspector & __inspector);
        ServerNodeType operator()(const ::vespalib::slime::Inspector & __inspector, ServerNodeType __eDefault);
    };
    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalConfigserverType(const ::config::ConfigValue & __value);
    InternalConfigserverType(const ::config::ConfigDataBuffer & __value);
    InternalConfigserverType(const ::config::ConfigPayload & __payload);
    InternalConfigserverType() ;
    InternalConfigserverType(const InternalConfigserverType & __rhs);
    InternalConfigserverType & operator = (const InternalConfigserverType & __rhs);
    InternalConfigserverType(InternalConfigserverType && __rhs) noexcept;
    InternalConfigserverType & operator = (InternalConfigserverType && __rhs) noexcept;
    ~InternalConfigserverType();

    bool operator==(const InternalConfigserverType& __rhs) const noexcept;
    bool operator!=(const InternalConfigserverType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /** Ports */
    int32_t rpcport; // Default: 19070
    int32_t httpport; // Default: 19071

    /** 0 means use the number of CPU cores available */
    int32_t numRpcThreads; // Default: 0
    ZookeeperserverVector zookeeperserver;
    Zookeeper zookeeper;
    bool zookeeperLocalhostAffinity; // Default: true
    int64_t sessionLifetime; // Default: 3600

    /**
     * How long to wait for all ZooKeeper servers to reach barrier after quorum
     * has reached barrier. In seconds
     */
    int64_t barrierWaitForAllTimeout; // Default: 5

    /** Directories */
    ::config::StringVector configModelPluginDir;
    vespalib::string configServerDBDir; // Default: "var/db/vespa/config_server/serverdb/"
    vespalib::string configDefinitionsDir; // Default: "share/vespa/configdefinitions/"
    vespalib::string fileReferencesDir; // Default: "var/db/vespa/filedistribution/"

    /**
     * Application package
     * The maximum decompressed size of an application package, in bytes.
     * Defaults to 8 GB
     */
    int64_t maxApplicationPackageSize; // Default: 8589934592

    /** Misc */
    int64_t masterGeneration; // Default: 0
    bool multitenant; // Default: false
    int32_t numDelayedResponseThreads; // Default: 1
    vespalib::string serverId; // Default: "localhost"
    bool hostedVespa; // Default: false
    int32_t numParallelTenantLoaders; // Default: 4
    int32_t numRedeploymentThreads; // Default: 4

    /** Configserver app */
    vespalib::string applicationDirectory; // Default: "conf/configserver-app"

    /** Zone information */
    vespalib::string cloud; // Default: "default"
    vespalib::string environment; // Default: "prod"
    vespalib::string region; // Default: "default"
    vespalib::string system; // Default: "main"
    ::config::StringVector zoneDnsSuffixes;

    /** RPC protocol */
    int32_t maxgetconfigclients; // Default: 1000000
    int32_t maxoutputbuffersize; // Default: 65536
    bool useVespaVersionInRequest; // Default: true
    PayloadCompressionType payloadCompressionType; // Default: LZ4

    /** Athenz config */
    vespalib::string loadBalancerAddress; // Default: ""
    vespalib::string athenzDnsSuffix; // Default: ""
    vespalib::string ztsUrl; // Default: ""

    /** Maintenance settings */
    int32_t maintainerIntervalMinutes; // Default: 30
    int32_t keepUnusedFileReferencesMinutes; // Default: 300
    int32_t keepSessionsWithUnknownStatusHours; // Default: 24

    /**
     * Bootstrapping
     * How long bootstrapping can take before giving up (in seconds)
     */
    int64_t maxDurationOfBootstrap; // Default: 7200

    /**
     * Initial time for how long to sleep before redeploying again if it fails
     * (in seconds)
     * Code uses backoff, so wait time will increase for every iteration
     */
    int64_t sleepTimeWhenRedeployingFails; // Default: 15
    bool canReturnEmptySentinelConfig; // Default: false
    ServerNodeType serverNodeType; // Default: config
};

} // namespace internal

typedef internal::InternalConfigserverType ConfigserverConfigBuilder;
typedef const internal::InternalConfigserverType ConfigserverConfig;

} // namespace cloud::config
#endif // VESPA_config_CLOUD_CONFIG_CONFIGSERVER_H
