#include "config-curator.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace cloud::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalCuratorType::CONFIG_DEF_MD5("be2bcf0a0f417062a033594379fddb22");
const vespalib::string InternalCuratorType::CONFIG_DEF_NAME("curator");
const vespalib::string InternalCuratorType::CONFIG_DEF_NAMESPACE("cloud.config");
const int64_t InternalCuratorType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=cloud.config",
"server[].hostname string",
"server[].port int default=2181",
"zookeeperLocalhostAffinity bool default=false",
"zookeeperSessionTimeoutSeconds int default=120",
};
const ::config::StringVector InternalCuratorType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalCuratorType::Server::Server() noexcept
    : hostname(),
      port(2181)
{
}

InternalCuratorType::Server::Server(const Server & __rhs) noexcept = default;
InternalCuratorType::Server & InternalCuratorType::Server::operator = (const Server & __rhs) noexcept = default;
InternalCuratorType::Server::Server(Server && __rhs) noexcept = default;
InternalCuratorType::Server & InternalCuratorType::Server::operator = (Server && __rhs) noexcept = default;
InternalCuratorType::Server::~Server() = default; 
InternalCuratorType::Server::Server(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    hostname = ConfigParser::parse<vespalib::string>("hostname", __lines);
    ConfigParser::stripLinesForKey("hostname", __remainingValuesToParse);
    port = ConfigParser::parse<int32_t>("port", __lines, 2181);
    ConfigParser::stripLinesForKey("port", __remainingValuesToParse);
}

bool
InternalCuratorType::Server::operator==(const Server& __rhs) const noexcept
{
    return (hostname == __rhs.hostname &&
            port == __rhs.port);
}

bool
InternalCuratorType::Server::operator!=(const Server& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalCuratorType::Server::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("hostname");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(hostname));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("port");
        __c.setString("type", "int");
        __c.setLong("value", port);
    }
}

InternalCuratorType::Server::Server(const vespalib::slime::Inspector & __inspector)
{
    hostname = __inspector["hostname"]["value"].asString().make_string();
    port = __inspector["port"]["value"].asLong();
}

InternalCuratorType::Server::Server(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    hostname = ::config::internal::ValueConverter<vespalib::string>()("hostname", __inspector["hostname"]);
    port = ::config::internal::ValueConverter<int32_t>()(__inspector["port"], 2181);
}

InternalCuratorType::InternalCuratorType()
    : server(),
      zookeeperLocalhostAffinity(false),
      zookeeperSessionTimeoutSeconds(120)
{
}

InternalCuratorType::InternalCuratorType(const InternalCuratorType & __rhs) = default;
InternalCuratorType & InternalCuratorType::operator =(const InternalCuratorType & __rhs) = default;
InternalCuratorType::InternalCuratorType(InternalCuratorType && __rhs) noexcept = default;
InternalCuratorType & InternalCuratorType::operator =(InternalCuratorType && __rhs) noexcept = default;
InternalCuratorType::~InternalCuratorType() = default; 
InternalCuratorType::InternalCuratorType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        server = ConfigParser::parseArray<ServerVector>("server", __lines);
        ConfigParser::stripLinesForKey("server", __remainingValuesToParse);
        zookeeperLocalhostAffinity = ConfigParser::parse<bool>("zookeeperLocalhostAffinity", __lines, false);
        ConfigParser::stripLinesForKey("zookeeperLocalhostAffinity", __remainingValuesToParse);
        zookeeperSessionTimeoutSeconds = ConfigParser::parse<int32_t>("zookeeperSessionTimeoutSeconds", __lines, 120);
        ConfigParser::stripLinesForKey("zookeeperSessionTimeoutSeconds", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalCuratorType::operator==(const InternalCuratorType& __rhs) const noexcept
{
    return (server == __rhs.server &&
            zookeeperLocalhostAffinity == __rhs.zookeeperLocalhostAffinity &&
            zookeeperSessionTimeoutSeconds == __rhs.zookeeperSessionTimeoutSeconds);
}

bool
InternalCuratorType::operator!=(const InternalCuratorType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalCuratorType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("server");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : server) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("zookeeperLocalhostAffinity");
        __c.setString("type", "bool");
        __c.setBool("value", zookeeperLocalhostAffinity);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("zookeeperSessionTimeoutSeconds");
        __c.setString("type", "int");
        __c.setLong("value", zookeeperSessionTimeoutSeconds);
    }
}

InternalCuratorType::InternalCuratorType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["server"]["value"].children(); __i++) {
        server.push_back(Server(__inspector["server"]["value"][__i]["value"]));
    }
    zookeeperLocalhostAffinity = __inspector["zookeeperLocalhostAffinity"]["value"].asBool();
    zookeeperSessionTimeoutSeconds = __inspector["zookeeperSessionTimeoutSeconds"]["value"].asLong();
}

InternalCuratorType::InternalCuratorType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<ServerVector> __serverInserter(server);
    __inspector["server"].traverse(__serverInserter);
    zookeeperLocalhostAffinity = ::config::internal::ValueConverter<bool>()(__inspector["zookeeperLocalhostAffinity"], false);
    zookeeperSessionTimeoutSeconds = ::config::internal::ValueConverter<int32_t>()(__inspector["zookeeperSessionTimeoutSeconds"], 120);
}

} // namespace internal

} // namespace cloud::config
