/**
 * @class cloud::config::Curator
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_CLOUD_CONFIG_CURATOR_H
#define CLOUD_CONFIG_CLOUD_CONFIG_CURATOR_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace cloud::config {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalCuratorType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalCuratorType> UP;
    class Server {
    public:
        Server(const ::config::StringVector & __lines);
        Server(const vespalib::slime::Inspector & __inspector);
        Server(const ::config::ConfigPayload & __payload);
        Server()  noexcept;
        Server(const Server & __rhs) noexcept;
        Server & operator = (const Server & __rhs) noexcept;
        Server(Server && __rhs) noexcept;
        Server & operator = (Server && __rhs) noexcept;
        ~Server();

        bool operator==(const Server& __rhs) const noexcept;
        bool operator!=(const Server& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** hostname and port of servers that curator will connect to */
        vespalib::string hostname;
        int32_t port; // Default: 2181
    };
    typedef std::vector<Server> ServerVector;

    typedef std::map<vespalib::string, Server> ServerMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalCuratorType(const ::config::ConfigValue & __value);
    InternalCuratorType(const ::config::ConfigDataBuffer & __value);
    InternalCuratorType(const ::config::ConfigPayload & __payload);
    InternalCuratorType() ;
    InternalCuratorType(const InternalCuratorType & __rhs);
    InternalCuratorType & operator = (const InternalCuratorType & __rhs);
    InternalCuratorType(InternalCuratorType && __rhs) noexcept;
    InternalCuratorType & operator = (InternalCuratorType && __rhs) noexcept;
    ~InternalCuratorType();

    bool operator==(const InternalCuratorType& __rhs) const noexcept;
    bool operator!=(const InternalCuratorType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    ServerVector server;

    /**
     * if true, only connect to server on localhost (must be in one of the
     * servers above)
     */
    bool zookeeperLocalhostAffinity; // Default: false

    /** session timeout, the high default is used by config servers */
    int32_t zookeeperSessionTimeoutSeconds; // Default: 120
};

} // namespace internal

typedef internal::InternalCuratorType CuratorConfigBuilder;
typedef const internal::InternalCuratorType CuratorConfig;

} // namespace cloud::config
#endif // VESPA_config_CLOUD_CONFIG_CURATOR_H
