#include "config-dataplane-proxy.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace cloud::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalDataplaneProxyType::CONFIG_DEF_MD5("e0b5fd391f1f63ae96c755f6f0776341");
const vespalib::string InternalDataplaneProxyType::CONFIG_DEF_NAME("dataplane-proxy");
const vespalib::string InternalDataplaneProxyType::CONFIG_DEF_NAMESPACE("cloud.config");
const int64_t InternalDataplaneProxyType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=cloud.config",
"tokenPort int",
"mtlsPort int",
"serverCertificate string",
"serverKey string",
"tokenEndpoints[] string",
};
const ::config::StringVector InternalDataplaneProxyType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalDataplaneProxyType::InternalDataplaneProxyType()
    : tokenPort(0),
      mtlsPort(0),
      serverCertificate(),
      serverKey(),
      tokenEndpoints()
{
}

InternalDataplaneProxyType::InternalDataplaneProxyType(const InternalDataplaneProxyType & __rhs) = default;
InternalDataplaneProxyType & InternalDataplaneProxyType::operator =(const InternalDataplaneProxyType & __rhs) = default;
InternalDataplaneProxyType::InternalDataplaneProxyType(InternalDataplaneProxyType && __rhs) noexcept = default;
InternalDataplaneProxyType & InternalDataplaneProxyType::operator =(InternalDataplaneProxyType && __rhs) noexcept = default;
InternalDataplaneProxyType::~InternalDataplaneProxyType() = default; 
InternalDataplaneProxyType::InternalDataplaneProxyType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        tokenPort = ConfigParser::parse<int32_t>("tokenPort", __lines);
        ConfigParser::stripLinesForKey("tokenPort", __remainingValuesToParse);
        mtlsPort = ConfigParser::parse<int32_t>("mtlsPort", __lines);
        ConfigParser::stripLinesForKey("mtlsPort", __remainingValuesToParse);
        serverCertificate = ConfigParser::parse<vespalib::string>("serverCertificate", __lines);
        ConfigParser::stripLinesForKey("serverCertificate", __remainingValuesToParse);
        serverKey = ConfigParser::parse<vespalib::string>("serverKey", __lines);
        ConfigParser::stripLinesForKey("serverKey", __remainingValuesToParse);
        tokenEndpoints = ConfigParser::parseArray<::config::StringVector>("tokenEndpoints", __lines);
        ConfigParser::stripLinesForKey("tokenEndpoints", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalDataplaneProxyType::
operator==(const InternalDataplaneProxyType& __rhs) const noexcept
{
    return (tokenPort == __rhs.tokenPort &&
            mtlsPort == __rhs.mtlsPort &&
            serverCertificate == __rhs.serverCertificate &&
            serverKey == __rhs.serverKey &&
            tokenEndpoints == __rhs.tokenEndpoints);
}

bool
InternalDataplaneProxyType::
operator!=(const InternalDataplaneProxyType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDataplaneProxyType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tokenPort");
        __c.setString("type", "int");
        __c.setLong("value", tokenPort);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("mtlsPort");
        __c.setString("type", "int");
        __c.setLong("value", mtlsPort);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("serverCertificate");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(serverCertificate));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("serverKey");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(serverKey));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tokenEndpoints");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : tokenEndpoints) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
}

InternalDataplaneProxyType::InternalDataplaneProxyType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    tokenPort = __inspector["tokenPort"]["value"].asLong();
    mtlsPort = __inspector["mtlsPort"]["value"].asLong();
    serverCertificate = __inspector["serverCertificate"]["value"].asString().make_string();
    serverKey = __inspector["serverKey"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["tokenEndpoints"]["value"].children(); __i++) {
        tokenEndpoints.push_back(__inspector["tokenEndpoints"]["value"][__i]["value"].asString().make_string());
    }
}

InternalDataplaneProxyType::InternalDataplaneProxyType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    tokenPort = ::config::internal::ValueConverter<int32_t>()("tokenPort", __inspector["tokenPort"]);
    mtlsPort = ::config::internal::ValueConverter<int32_t>()("mtlsPort", __inspector["mtlsPort"]);
    serverCertificate = ::config::internal::ValueConverter<vespalib::string>()("serverCertificate", __inspector["serverCertificate"]);
    serverKey = ::config::internal::ValueConverter<vespalib::string>()("serverKey", __inspector["serverKey"]);
    ::config::internal::VectorInserter<::config::StringVector> __tokenEndpointsInserter(tokenEndpoints);
    __inspector["tokenEndpoints"].traverse(__tokenEndpointsInserter);
}

} // namespace internal

} // namespace cloud::config
