#include "config-dispatch-nodes.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalDispatchNodesType::CONFIG_DEF_MD5("de348ed663f532c8bfb79fa82ab85566");
const vespalib::string InternalDispatchNodesType::CONFIG_DEF_NAME("dispatch-nodes");
const vespalib::string InternalDispatchNodesType::CONFIG_DEF_NAMESPACE("vespa.config.search");
const int64_t InternalDispatchNodesType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search",
"node[].key int",
"node[].group int default=0",
"node[].host string",
"node[].port int",
};
const ::config::StringVector InternalDispatchNodesType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalDispatchNodesType::Node::Node() noexcept
    : key(0),
      group(0),
      host(),
      port(0)
{
}

InternalDispatchNodesType::Node::Node(const Node & __rhs) noexcept = default;
InternalDispatchNodesType::Node & InternalDispatchNodesType::Node::operator = (const Node & __rhs) noexcept = default;
InternalDispatchNodesType::Node::Node(Node && __rhs) noexcept = default;
InternalDispatchNodesType::Node & InternalDispatchNodesType::Node::operator = (Node && __rhs) noexcept = default;
InternalDispatchNodesType::Node::~Node() = default; 
InternalDispatchNodesType::Node::Node(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    key = ConfigParser::parse<int32_t>("key", __lines);
    ConfigParser::stripLinesForKey("key", __remainingValuesToParse);
    group = ConfigParser::parse<int32_t>("group", __lines, 0);
    ConfigParser::stripLinesForKey("group", __remainingValuesToParse);
    host = ConfigParser::parse<vespalib::string>("host", __lines);
    ConfigParser::stripLinesForKey("host", __remainingValuesToParse);
    port = ConfigParser::parse<int32_t>("port", __lines);
    ConfigParser::stripLinesForKey("port", __remainingValuesToParse);
}

bool
InternalDispatchNodesType::Node::operator==(const Node& __rhs) const noexcept
{
    return (key == __rhs.key &&
            group == __rhs.group &&
            host == __rhs.host &&
            port == __rhs.port);
}

bool
InternalDispatchNodesType::Node::operator!=(const Node& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDispatchNodesType::Node::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("key");
        __c.setString("type", "int");
        __c.setLong("value", key);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("group");
        __c.setString("type", "int");
        __c.setLong("value", group);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("host");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(host));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("port");
        __c.setString("type", "int");
        __c.setLong("value", port);
    }
}

InternalDispatchNodesType::Node::Node(const vespalib::slime::Inspector & __inspector)
{
    key = __inspector["key"]["value"].asLong();
    group = __inspector["group"]["value"].asLong();
    host = __inspector["host"]["value"].asString().make_string();
    port = __inspector["port"]["value"].asLong();
}

InternalDispatchNodesType::Node::Node(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    key = ::config::internal::ValueConverter<int32_t>()("key", __inspector["key"]);
    group = ::config::internal::ValueConverter<int32_t>()(__inspector["group"], 0);
    host = ::config::internal::ValueConverter<vespalib::string>()("host", __inspector["host"]);
    port = ::config::internal::ValueConverter<int32_t>()("port", __inspector["port"]);
}

InternalDispatchNodesType::InternalDispatchNodesType()
    : node()
{
}

InternalDispatchNodesType::InternalDispatchNodesType(const InternalDispatchNodesType & __rhs) = default;
InternalDispatchNodesType & InternalDispatchNodesType::operator =(const InternalDispatchNodesType & __rhs) = default;
InternalDispatchNodesType::InternalDispatchNodesType(InternalDispatchNodesType && __rhs) noexcept = default;
InternalDispatchNodesType & InternalDispatchNodesType::operator =(InternalDispatchNodesType && __rhs) noexcept = default;
InternalDispatchNodesType::~InternalDispatchNodesType() = default; 
InternalDispatchNodesType::InternalDispatchNodesType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        node = ConfigParser::parseArray<NodeVector>("node", __lines);
        ConfigParser::stripLinesForKey("node", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalDispatchNodesType::
operator==(const InternalDispatchNodesType& __rhs) const noexcept
{
    return (node == __rhs.node);
}

bool
InternalDispatchNodesType::
operator!=(const InternalDispatchNodesType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDispatchNodesType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("node");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : node) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalDispatchNodesType::InternalDispatchNodesType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["node"]["value"].children(); __i++) {
        node.push_back(Node(__inspector["node"]["value"][__i]["value"]));
    }
}

InternalDispatchNodesType::InternalDispatchNodesType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<NodeVector> __nodeInserter(node);
    __inspector["node"].traverse(__nodeInserter);
}

} // namespace internal

} // namespace vespa::config::search
