/**
 * @class vespa::config::search::DispatchNodes
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 * List of nodes in a search cluster to dispatch to.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_SEARCH_DISPATCHNODES_H
#define CLOUD_CONFIG_VESPA_CONFIG_SEARCH_DISPATCHNODES_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::search {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalDispatchNodesType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalDispatchNodesType> UP;
    class Node {
    public:
        Node(const ::config::StringVector & __lines);
        Node(const vespalib::slime::Inspector & __inspector);
        Node(const ::config::ConfigPayload & __payload);
        Node()  noexcept;
        Node(const Node & __rhs) noexcept;
        Node & operator = (const Node & __rhs) noexcept;
        Node(Node && __rhs) noexcept;
        Node & operator = (Node && __rhs) noexcept;
        ~Node();

        bool operator==(const Node& __rhs) const noexcept;
        bool operator!=(const Node& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** The unique key of a search node */
        int32_t key;

        /** The index of the group this search node belongs to */
        int32_t group; // Default: 0

        /** The host name of this search node */
        vespalib::string host;

        /** The rpc port of this search node */
        int32_t port;
    };
    typedef std::vector<Node> NodeVector;

    typedef std::map<vespalib::string, Node> NodeMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalDispatchNodesType(const ::config::ConfigValue & __value);
    InternalDispatchNodesType(const ::config::ConfigDataBuffer & __value);
    InternalDispatchNodesType(const ::config::ConfigPayload & __payload);
    InternalDispatchNodesType() ;
    InternalDispatchNodesType(const InternalDispatchNodesType & __rhs);
    InternalDispatchNodesType & operator = (const InternalDispatchNodesType & __rhs);
    InternalDispatchNodesType(InternalDispatchNodesType && __rhs) noexcept;
    InternalDispatchNodesType & operator = (InternalDispatchNodesType && __rhs) noexcept;
    ~InternalDispatchNodesType();

    bool operator==(const InternalDispatchNodesType& __rhs) const noexcept;
    bool operator!=(const InternalDispatchNodesType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    NodeVector node;
};

} // namespace internal

typedef internal::InternalDispatchNodesType DispatchNodesConfigBuilder;
typedef const internal::InternalDispatchNodesType DispatchNodesConfig;

} // namespace vespa::config::search
#endif // VESPA_config_VESPA_CONFIG_SEARCH_DISPATCHNODES_H
