#include "config-dispatch.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalDispatchType::CONFIG_DEF_MD5("4f573838db27bb53baffbad20b9f1b94");
const vespalib::string InternalDispatchType::CONFIG_DEF_NAME("dispatch");
const vespalib::string InternalDispatchType::CONFIG_DEF_NAMESPACE("vespa.config.search");
const int64_t InternalDispatchType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search",
"minActivedocsPercentage double default=97.0",
"distributionPolicy enum { ROUNDROBIN, BEST_OF_RANDOM_2, LATENCY_AMORTIZED_OVER_REQUESTS, LATENCY_AMORTIZED_OVER_TIME, ADAPTIVE } default=ADAPTIVE",
"maxHitsPerNode int default=2147483647",
"topKProbability double default=0.9999",
"redundancy long default=1",
"minSearchCoverage double default=100",
"minWaitAfterCoverageFactor double default=0",
"maxWaitAfterCoverageFactor double default=1",
"numJrtTransportThreads int default=8",
"numJrtConnectionsPerNode int default=8",
"warmuptime double default=0.1",
"summaryDecodePolicy enum {EAGER, ONDEMAND} default=EAGER",
};
const ::config::StringVector InternalDispatchType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalDispatchType::DistributionPolicy
InternalDispatchType::getDistributionPolicy(const vespalib::string& name)
{
    if (name == "ROUNDROBIN") {
        return DistributionPolicy::ROUNDROBIN;
    } else if (name == "BEST_OF_RANDOM_2") {
        return DistributionPolicy::BEST_OF_RANDOM_2;
    } else if (name == "LATENCY_AMORTIZED_OVER_REQUESTS") {
        return DistributionPolicy::LATENCY_AMORTIZED_OVER_REQUESTS;
    } else if (name == "LATENCY_AMORTIZED_OVER_TIME") {
        return DistributionPolicy::LATENCY_AMORTIZED_OVER_TIME;
    } else if (name == "ADAPTIVE") {
        return DistributionPolicy::ADAPTIVE;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalDispatchType::getDistributionPolicyName(DistributionPolicy t)
{
    switch (t) {
        case DistributionPolicy::ROUNDROBIN: return "ROUNDROBIN";
        case DistributionPolicy::BEST_OF_RANDOM_2: return "BEST_OF_RANDOM_2";
        case DistributionPolicy::LATENCY_AMORTIZED_OVER_REQUESTS: return "LATENCY_AMORTIZED_OVER_REQUESTS";
        case DistributionPolicy::LATENCY_AMORTIZED_OVER_TIME: return "LATENCY_AMORTIZED_OVER_TIME";
        case DistributionPolicy::ADAPTIVE: return "ADAPTIVE";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalDispatchType::DistributionPolicy InternalDispatchType::InternalDistributionPolicyConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalDispatchType::getDistributionPolicy(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalDispatchType::DistributionPolicy InternalDispatchType::InternalDistributionPolicyConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalDispatchType::getDistributionPolicy(__inspector.asString().make_string());
}
InternalDispatchType::DistributionPolicy InternalDispatchType::InternalDistributionPolicyConverter::operator()(const ::vespalib::slime::Inspector & __inspector, DistributionPolicy __eDefault) {
    if (__inspector.valid()) {
        return InternalDispatchType::getDistributionPolicy(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalDispatchType::SummaryDecodePolicy
InternalDispatchType::getSummaryDecodePolicy(const vespalib::string& name)
{
    if (name == "EAGER") {
        return SummaryDecodePolicy::EAGER;
    } else if (name == "ONDEMAND") {
        return SummaryDecodePolicy::ONDEMAND;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalDispatchType::getSummaryDecodePolicyName(SummaryDecodePolicy t)
{
    switch (t) {
        case SummaryDecodePolicy::EAGER: return "EAGER";
        case SummaryDecodePolicy::ONDEMAND: return "ONDEMAND";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalDispatchType::SummaryDecodePolicy InternalDispatchType::InternalSummaryDecodePolicyConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalDispatchType::getSummaryDecodePolicy(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalDispatchType::SummaryDecodePolicy InternalDispatchType::InternalSummaryDecodePolicyConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalDispatchType::getSummaryDecodePolicy(__inspector.asString().make_string());
}
InternalDispatchType::SummaryDecodePolicy InternalDispatchType::InternalSummaryDecodePolicyConverter::operator()(const ::vespalib::slime::Inspector & __inspector, SummaryDecodePolicy __eDefault) {
    if (__inspector.valid()) {
        return InternalDispatchType::getSummaryDecodePolicy(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalDispatchType::InternalDispatchType() noexcept
    : minActivedocsPercentage(97.0),
      distributionPolicy(DistributionPolicy::ADAPTIVE),
      maxHitsPerNode(2147483647),
      topKProbability(0.9999),
      redundancy(1),
      minSearchCoverage(100),
      minWaitAfterCoverageFactor(0),
      maxWaitAfterCoverageFactor(1),
      numJrtTransportThreads(8),
      numJrtConnectionsPerNode(8),
      warmuptime(0.1),
      summaryDecodePolicy(SummaryDecodePolicy::EAGER)
{
}

InternalDispatchType::InternalDispatchType(const InternalDispatchType & __rhs) noexcept = default;
InternalDispatchType & InternalDispatchType::operator =(const InternalDispatchType & __rhs) noexcept = default;
InternalDispatchType::InternalDispatchType(InternalDispatchType && __rhs) noexcept = default;
InternalDispatchType & InternalDispatchType::operator =(InternalDispatchType && __rhs) noexcept = default;
InternalDispatchType::~InternalDispatchType() = default; 
InternalDispatchType::InternalDispatchType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        minActivedocsPercentage = ConfigParser::parse<double>("minActivedocsPercentage", __lines, 97.0);
        ConfigParser::stripLinesForKey("minActivedocsPercentage", __remainingValuesToParse);
        distributionPolicy = getDistributionPolicy(ConfigParser::parse<vespalib::string>("distributionPolicy", __lines, "ADAPTIVE"));
        ConfigParser::stripLinesForKey("distributionPolicy", __remainingValuesToParse);
        maxHitsPerNode = ConfigParser::parse<int32_t>("maxHitsPerNode", __lines, 2147483647);
        ConfigParser::stripLinesForKey("maxHitsPerNode", __remainingValuesToParse);
        topKProbability = ConfigParser::parse<double>("topKProbability", __lines, 0.9999);
        ConfigParser::stripLinesForKey("topKProbability", __remainingValuesToParse);
        redundancy = ConfigParser::parse<int64_t>("redundancy", __lines, 1);
        ConfigParser::stripLinesForKey("redundancy", __remainingValuesToParse);
        minSearchCoverage = ConfigParser::parse<double>("minSearchCoverage", __lines, 100);
        ConfigParser::stripLinesForKey("minSearchCoverage", __remainingValuesToParse);
        minWaitAfterCoverageFactor = ConfigParser::parse<double>("minWaitAfterCoverageFactor", __lines, 0);
        ConfigParser::stripLinesForKey("minWaitAfterCoverageFactor", __remainingValuesToParse);
        maxWaitAfterCoverageFactor = ConfigParser::parse<double>("maxWaitAfterCoverageFactor", __lines, 1);
        ConfigParser::stripLinesForKey("maxWaitAfterCoverageFactor", __remainingValuesToParse);
        numJrtTransportThreads = ConfigParser::parse<int32_t>("numJrtTransportThreads", __lines, 8);
        ConfigParser::stripLinesForKey("numJrtTransportThreads", __remainingValuesToParse);
        numJrtConnectionsPerNode = ConfigParser::parse<int32_t>("numJrtConnectionsPerNode", __lines, 8);
        ConfigParser::stripLinesForKey("numJrtConnectionsPerNode", __remainingValuesToParse);
        warmuptime = ConfigParser::parse<double>("warmuptime", __lines, 0.1);
        ConfigParser::stripLinesForKey("warmuptime", __remainingValuesToParse);
        summaryDecodePolicy = getSummaryDecodePolicy(ConfigParser::parse<vespalib::string>("summaryDecodePolicy", __lines, "EAGER"));
        ConfigParser::stripLinesForKey("summaryDecodePolicy", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalDispatchType::operator==(const InternalDispatchType& __rhs) const noexcept
{
    return (minActivedocsPercentage == __rhs.minActivedocsPercentage &&
            distributionPolicy == __rhs.distributionPolicy &&
            maxHitsPerNode == __rhs.maxHitsPerNode &&
            topKProbability == __rhs.topKProbability &&
            redundancy == __rhs.redundancy &&
            minSearchCoverage == __rhs.minSearchCoverage &&
            minWaitAfterCoverageFactor == __rhs.minWaitAfterCoverageFactor &&
            maxWaitAfterCoverageFactor == __rhs.maxWaitAfterCoverageFactor &&
            numJrtTransportThreads == __rhs.numJrtTransportThreads &&
            numJrtConnectionsPerNode == __rhs.numJrtConnectionsPerNode &&
            warmuptime == __rhs.warmuptime &&
            summaryDecodePolicy == __rhs.summaryDecodePolicy);
}

bool
InternalDispatchType::operator!=(const InternalDispatchType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDispatchType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("minActivedocsPercentage");
        __c.setString("type", "double");
        __c.setDouble("value", minActivedocsPercentage);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("distributionPolicy");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getDistributionPolicyName(distributionPolicy)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxHitsPerNode");
        __c.setString("type", "int");
        __c.setLong("value", maxHitsPerNode);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("topKProbability");
        __c.setString("type", "double");
        __c.setDouble("value", topKProbability);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("redundancy");
        __c.setString("type", "long");
        __c.setLong("value", redundancy);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("minSearchCoverage");
        __c.setString("type", "double");
        __c.setDouble("value", minSearchCoverage);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("minWaitAfterCoverageFactor");
        __c.setString("type", "double");
        __c.setDouble("value", minWaitAfterCoverageFactor);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxWaitAfterCoverageFactor");
        __c.setString("type", "double");
        __c.setDouble("value", maxWaitAfterCoverageFactor);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("numJrtTransportThreads");
        __c.setString("type", "int");
        __c.setLong("value", numJrtTransportThreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("numJrtConnectionsPerNode");
        __c.setString("type", "int");
        __c.setLong("value", numJrtConnectionsPerNode);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("warmuptime");
        __c.setString("type", "double");
        __c.setDouble("value", warmuptime);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("summaryDecodePolicy");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getSummaryDecodePolicyName(summaryDecodePolicy)));
    }
}

InternalDispatchType::InternalDispatchType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    minActivedocsPercentage = __inspector["minActivedocsPercentage"]["value"].asDouble();
    distributionPolicy = getDistributionPolicy(__inspector["distributionPolicy"]["value"].asString().make_string());
    maxHitsPerNode = __inspector["maxHitsPerNode"]["value"].asLong();
    topKProbability = __inspector["topKProbability"]["value"].asDouble();
    redundancy = __inspector["redundancy"]["value"].asLong();
    minSearchCoverage = __inspector["minSearchCoverage"]["value"].asDouble();
    minWaitAfterCoverageFactor = __inspector["minWaitAfterCoverageFactor"]["value"].asDouble();
    maxWaitAfterCoverageFactor = __inspector["maxWaitAfterCoverageFactor"]["value"].asDouble();
    numJrtTransportThreads = __inspector["numJrtTransportThreads"]["value"].asLong();
    numJrtConnectionsPerNode = __inspector["numJrtConnectionsPerNode"]["value"].asLong();
    warmuptime = __inspector["warmuptime"]["value"].asDouble();
    summaryDecodePolicy = getSummaryDecodePolicy(__inspector["summaryDecodePolicy"]["value"].asString().make_string());
}

InternalDispatchType::InternalDispatchType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    minActivedocsPercentage = ::config::internal::ValueConverter<double>()(__inspector["minActivedocsPercentage"], 97.0);
    distributionPolicy = InternalDistributionPolicyConverter()(__inspector["distributionPolicy"], DistributionPolicy::ADAPTIVE);
    maxHitsPerNode = ::config::internal::ValueConverter<int32_t>()(__inspector["maxHitsPerNode"], 2147483647);
    topKProbability = ::config::internal::ValueConverter<double>()(__inspector["topKProbability"], 0.9999);
    redundancy = ::config::internal::ValueConverter<int64_t>()(__inspector["redundancy"], 1);
    minSearchCoverage = ::config::internal::ValueConverter<double>()(__inspector["minSearchCoverage"], 100);
    minWaitAfterCoverageFactor = ::config::internal::ValueConverter<double>()(__inspector["minWaitAfterCoverageFactor"], 0);
    maxWaitAfterCoverageFactor = ::config::internal::ValueConverter<double>()(__inspector["maxWaitAfterCoverageFactor"], 1);
    numJrtTransportThreads = ::config::internal::ValueConverter<int32_t>()(__inspector["numJrtTransportThreads"], 8);
    numJrtConnectionsPerNode = ::config::internal::ValueConverter<int32_t>()(__inspector["numJrtConnectionsPerNode"], 8);
    warmuptime = ::config::internal::ValueConverter<double>()(__inspector["warmuptime"], 0.1);
    summaryDecodePolicy = InternalSummaryDecodePolicyConverter()(__inspector["summaryDecodePolicy"], SummaryDecodePolicy::EAGER);
}

} // namespace internal

} // namespace vespa::config::search
