/**
 * @class vespa::config::search::Dispatch
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 * Configuration of dispatch from container nodes to search clusters
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_SEARCH_DISPATCH_H
#define CLOUD_CONFIG_VESPA_CONFIG_SEARCH_DISPATCH_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::search {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalDispatchType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalDispatchType> UP;
    enum class DistributionPolicy { ROUNDROBIN, BEST_OF_RANDOM_2, LATENCY_AMORTIZED_OVER_REQUESTS, LATENCY_AMORTIZED_OVER_TIME, ADAPTIVE };
    typedef std::vector<DistributionPolicy> DistributionPolicyVector;
    typedef std::map<vespalib::string, DistributionPolicy> DistributionPolicyMap;
    static DistributionPolicy getDistributionPolicy(const vespalib::string&);
    static vespalib::string getDistributionPolicyName(DistributionPolicy e);

    struct InternalDistributionPolicyConverter {
        DistributionPolicy operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
        DistributionPolicy operator()(const ::vespalib::slime::Inspector & __inspector);
        DistributionPolicy operator()(const ::vespalib::slime::Inspector & __inspector, DistributionPolicy __eDefault);
    };
    enum class SummaryDecodePolicy { EAGER, ONDEMAND };
    typedef std::vector<SummaryDecodePolicy> SummaryDecodePolicyVector;
    typedef std::map<vespalib::string, SummaryDecodePolicy> SummaryDecodePolicyMap;
    static SummaryDecodePolicy getSummaryDecodePolicy(const vespalib::string&);
    static vespalib::string getSummaryDecodePolicyName(SummaryDecodePolicy e);

    struct InternalSummaryDecodePolicyConverter {
        SummaryDecodePolicy operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
        SummaryDecodePolicy operator()(const ::vespalib::slime::Inspector & __inspector);
        SummaryDecodePolicy operator()(const ::vespalib::slime::Inspector & __inspector, SummaryDecodePolicy __eDefault);
    };
    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalDispatchType(const ::config::ConfigValue & __value);
    InternalDispatchType(const ::config::ConfigDataBuffer & __value);
    InternalDispatchType(const ::config::ConfigPayload & __payload);
    InternalDispatchType()  noexcept;
    InternalDispatchType(const InternalDispatchType & __rhs) noexcept;
    InternalDispatchType & operator = (const InternalDispatchType & __rhs) noexcept;
    InternalDispatchType(InternalDispatchType && __rhs) noexcept;
    InternalDispatchType & operator = (InternalDispatchType && __rhs) noexcept;
    ~InternalDispatchType();

    bool operator==(const InternalDispatchType& __rhs) const noexcept;
    bool operator!=(const InternalDispatchType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /**
     * The active docs a group must have as a % of the average active docs of
     * all other groups,
     * for that group to be included in queries
     */
    double minActivedocsPercentage; // Default: 97.0

    /** Distribution policy for group selection */
    DistributionPolicy distributionPolicy; // Default: ADAPTIVE

    /**
     * Maximum number of hits that will be requested from a single node
     * in this dataset. If not set, there is no limit. Using this option
     * may help reduce network traffic when searching in datasets with big
     * fan-out, but it will also result in incorrect and incomplete results;
     * don't use it if you don't (really) mean it.
     */
    int32_t maxHitsPerNode; // Default: 2147483647

    /**
     * Probability for getting the K best hits (topK).
     * A value of 1.0 will ask all N partitions for K hits.
     * Any value between <0, 1> will use a Student T with 30 degrees freedom and
     * compute a value Q that
     * will give you the globally K best hits according to this formula with the
     * desired probability.
     * q = k/n + qT (p',30) x ???(k ?? (1/n) ?? (1 ??? 1/n))
     * With a probability of 0.999 and K=200 and N=10 will give a Q of 38,
     * meaning that you only need to fetch 19% compared to
     * a setting of 1.0. This is a significant optimisation with with very
     * little loss in presicion.
     */
    double topKProbability; // Default: 0.9999

    /**
     * Number of document replicas _per group_ that will be present in a stable
     * cluster.
     * Should always be >= searchableCopies.
     */
    int64_t redundancy; // Default: 1

    /**
     * Minimum search coverage required before returning the results of a query
     */
    double minSearchCoverage; // Default: 100

    /**
     * Minimum wait time for full coverage after minimum coverage is achieved,
     * factored based on time left at minimum coverage
     */
    double minWaitAfterCoverageFactor; // Default: 0

    /**
     * Maximum wait time for full coverage after minimum coverage is achieved,
     * factored based on time left at minimum coverage
     */
    double maxWaitAfterCoverageFactor; // Default: 1

    /** Number of JRT transport threads */
    int32_t numJrtTransportThreads; // Default: 8

    /** Number of JRT connections per backend node */
    int32_t numJrtConnectionsPerNode; // Default: 8

    /**
     * Number of seconds to spend warming up code to prevent JIT cold start
     * issues.
     */
    double warmuptime; // Default: 0.1

    /**
     * Specifies how summary data are decoded
     * Eager will build java objects immediately, while ONDEMAND will do so when
     * it is needed
     */
    SummaryDecodePolicy summaryDecodePolicy; // Default: EAGER
};

} // namespace internal

typedef internal::InternalDispatchType DispatchConfigBuilder;
typedef const internal::InternalDispatchType DispatchConfig;

} // namespace vespa::config::search
#endif // VESPA_config_VESPA_CONFIG_SEARCH_DISPATCH_H
