#include "config-distribution.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalDistributionType::CONFIG_DEF_MD5("38063abe32cb8050720cc9487f733ba0");
const vespalib::string InternalDistributionType::CONFIG_DEF_NAME("distribution");
const vespalib::string InternalDistributionType::CONFIG_DEF_NAMESPACE("vespa.config.content");
const int64_t InternalDistributionType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content",
"cluster{}.active_per_leaf_group bool default=false",
"cluster{}.ready_copies int default=0",
"cluster{}.redundancy int default=3",
"cluster{}.initial_redundancy int default=0",
"cluster{}.group[].index string",
"cluster{}.group[].name string",
"cluster{}.group[].capacity double default=1",
"cluster{}.group[].partitions string default=\"\"",
"cluster{}.group[].nodes[].index int",
"cluster{}.group[].nodes[].retired bool default=false",
};
const ::config::StringVector InternalDistributionType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalDistributionType::Cluster::Group::Nodes::Nodes() noexcept
    : index(0),
      retired(false)
{
}

InternalDistributionType::Cluster::Group::Nodes::Nodes(const Nodes & __rhs) noexcept = default;
InternalDistributionType::Cluster::Group::Nodes & InternalDistributionType::Cluster::Group::Nodes::operator = (const Nodes & __rhs) noexcept = default;
InternalDistributionType::Cluster::Group::Nodes::Nodes(Nodes && __rhs) noexcept = default;
InternalDistributionType::Cluster::Group::Nodes & InternalDistributionType::Cluster::Group::Nodes::operator = (Nodes && __rhs) noexcept = default;
InternalDistributionType::Cluster::Group::Nodes::~Nodes() = default; 
InternalDistributionType::Cluster::Group::Nodes::Nodes(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    index = ConfigParser::parse<int32_t>("index", __lines);
    ConfigParser::stripLinesForKey("index", __remainingValuesToParse);
    retired = ConfigParser::parse<bool>("retired", __lines, false);
    ConfigParser::stripLinesForKey("retired", __remainingValuesToParse);
}

bool
InternalDistributionType::Cluster::Group::Nodes::
operator==(const Nodes& __rhs) const noexcept
{
    return (index == __rhs.index &&
            retired == __rhs.retired);
}

bool
InternalDistributionType::Cluster::Group::Nodes::
operator!=(const Nodes& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDistributionType::Cluster::Group::Nodes::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("index");
        __c.setString("type", "int");
        __c.setLong("value", index);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("retired");
        __c.setString("type", "bool");
        __c.setBool("value", retired);
    }
}

InternalDistributionType::Cluster::Group::Nodes::Nodes(const vespalib::slime::Inspector & __inspector)
{
    index = __inspector["index"]["value"].asLong();
    retired = __inspector["retired"]["value"].asBool();
}

InternalDistributionType::Cluster::Group::Nodes::Nodes(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    index = ::config::internal::ValueConverter<int32_t>()("index", __inspector["index"]);
    retired = ::config::internal::ValueConverter<bool>()(__inspector["retired"], false);
}

InternalDistributionType::Cluster::Group::Group()
    : index(),
      name(),
      capacity(1),
      partitions(""),
      nodes()
{
}

InternalDistributionType::Cluster::Group::Group(const Group & __rhs) = default;
InternalDistributionType::Cluster::Group & InternalDistributionType::Cluster::Group::operator = (const Group & __rhs) = default;
InternalDistributionType::Cluster::Group::Group(Group && __rhs) noexcept = default;
InternalDistributionType::Cluster::Group & InternalDistributionType::Cluster::Group::operator = (Group && __rhs) noexcept = default;
InternalDistributionType::Cluster::Group::~Group() = default; 
InternalDistributionType::Cluster::Group::Group(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    index = ConfigParser::parse<vespalib::string>("index", __lines);
    ConfigParser::stripLinesForKey("index", __remainingValuesToParse);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    capacity = ConfigParser::parse<double>("capacity", __lines, 1);
    ConfigParser::stripLinesForKey("capacity", __remainingValuesToParse);
    partitions = ConfigParser::parse<vespalib::string>("partitions", __lines, "");
    ConfigParser::stripLinesForKey("partitions", __remainingValuesToParse);
    nodes = ConfigParser::parseArray<NodesVector>("nodes", __lines);
    ConfigParser::stripLinesForKey("nodes", __remainingValuesToParse);
}

bool
InternalDistributionType::Cluster::Group::operator==(const Group& __rhs) const noexcept
{
    return (index == __rhs.index &&
            name == __rhs.name &&
            capacity == __rhs.capacity &&
            partitions == __rhs.partitions &&
            nodes == __rhs.nodes);
}

bool
InternalDistributionType::Cluster::Group::operator!=(const Group& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDistributionType::Cluster::Group::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("index");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(index));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("capacity");
        __c.setString("type", "double");
        __c.setDouble("value", capacity);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("partitions");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(partitions));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("nodes");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : nodes) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalDistributionType::Cluster::Group::Group(const vespalib::slime::Inspector & __inspector)
{
    index = __inspector["index"]["value"].asString().make_string();
    name = __inspector["name"]["value"].asString().make_string();
    capacity = __inspector["capacity"]["value"].asDouble();
    partitions = __inspector["partitions"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["nodes"]["value"].children(); __i++) {
        nodes.push_back(Nodes(__inspector["nodes"]["value"][__i]["value"]));
    }
}

InternalDistributionType::Cluster::Group::Group(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    index = ::config::internal::ValueConverter<vespalib::string>()("index", __inspector["index"]);
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    capacity = ::config::internal::ValueConverter<double>()(__inspector["capacity"], 1);
    partitions = ::config::internal::ValueConverter<vespalib::string>()(__inspector["partitions"], "");
    ::config::internal::VectorInserter<NodesVector> __nodesInserter(nodes);
    __inspector["nodes"].traverse(__nodesInserter);
}

InternalDistributionType::Cluster::Cluster()
    : activePerLeafGroup(false),
      readyCopies(0),
      redundancy(3),
      initialRedundancy(0),
      group()
{
}

InternalDistributionType::Cluster::Cluster(const Cluster & __rhs) = default;
InternalDistributionType::Cluster & InternalDistributionType::Cluster::operator = (const Cluster & __rhs) = default;
InternalDistributionType::Cluster::Cluster(Cluster && __rhs) noexcept = default;
InternalDistributionType::Cluster & InternalDistributionType::Cluster::operator = (Cluster && __rhs) noexcept = default;
InternalDistributionType::Cluster::~Cluster() = default; 
InternalDistributionType::Cluster::Cluster(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    activePerLeafGroup = ConfigParser::parse<bool>("active_per_leaf_group", __lines, false);
    ConfigParser::stripLinesForKey("active_per_leaf_group", __remainingValuesToParse);
    readyCopies = ConfigParser::parse<int32_t>("ready_copies", __lines, 0);
    ConfigParser::stripLinesForKey("ready_copies", __remainingValuesToParse);
    redundancy = ConfigParser::parse<int32_t>("redundancy", __lines, 3);
    ConfigParser::stripLinesForKey("redundancy", __remainingValuesToParse);
    initialRedundancy = ConfigParser::parse<int32_t>("initial_redundancy", __lines, 0);
    ConfigParser::stripLinesForKey("initial_redundancy", __remainingValuesToParse);
    group = ConfigParser::parseArray<GroupVector>("group", __lines);
    ConfigParser::stripLinesForKey("group", __remainingValuesToParse);
}

bool
InternalDistributionType::Cluster::operator==(const Cluster& __rhs) const noexcept
{
    return (activePerLeafGroup == __rhs.activePerLeafGroup &&
            readyCopies == __rhs.readyCopies &&
            redundancy == __rhs.redundancy &&
            initialRedundancy == __rhs.initialRedundancy &&
            group == __rhs.group);
}

bool
InternalDistributionType::Cluster::operator!=(const Cluster& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDistributionType::Cluster::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("active_per_leaf_group");
        __c.setString("type", "bool");
        __c.setBool("value", activePerLeafGroup);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ready_copies");
        __c.setString("type", "int");
        __c.setLong("value", readyCopies);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("redundancy");
        __c.setString("type", "int");
        __c.setLong("value", redundancy);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("initial_redundancy");
        __c.setString("type", "int");
        __c.setLong("value", initialRedundancy);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("group");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : group) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalDistributionType::Cluster::Cluster(const vespalib::slime::Inspector & __inspector)
{
    activePerLeafGroup = __inspector["active_per_leaf_group"]["value"].asBool();
    readyCopies = __inspector["ready_copies"]["value"].asLong();
    redundancy = __inspector["redundancy"]["value"].asLong();
    initialRedundancy = __inspector["initial_redundancy"]["value"].asLong();
    for (size_t __i = 0; __i < __inspector["group"]["value"].children(); __i++) {
        group.push_back(Group(__inspector["group"]["value"][__i]["value"]));
    }
}

InternalDistributionType::Cluster::Cluster(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    activePerLeafGroup = ::config::internal::ValueConverter<bool>()(__inspector["active_per_leaf_group"], false);
    readyCopies = ::config::internal::ValueConverter<int32_t>()(__inspector["ready_copies"], 0);
    redundancy = ::config::internal::ValueConverter<int32_t>()(__inspector["redundancy"], 3);
    initialRedundancy = ::config::internal::ValueConverter<int32_t>()(__inspector["initial_redundancy"], 0);
    ::config::internal::VectorInserter<GroupVector> __groupInserter(group);
    __inspector["group"].traverse(__groupInserter);
}

InternalDistributionType::InternalDistributionType()
    : cluster()
{
}

InternalDistributionType::InternalDistributionType(const InternalDistributionType & __rhs) = default;
InternalDistributionType & InternalDistributionType::operator =(const InternalDistributionType & __rhs) = default;
InternalDistributionType::InternalDistributionType(InternalDistributionType && __rhs) noexcept = default;
InternalDistributionType & InternalDistributionType::operator =(InternalDistributionType && __rhs) noexcept = default;
InternalDistributionType::~InternalDistributionType() = default; 
InternalDistributionType::InternalDistributionType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        cluster = ConfigParser::parseMap<Cluster>("cluster", __lines);
        ConfigParser::stripLinesForKey("cluster", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalDistributionType::
operator==(const InternalDistributionType& __rhs) const noexcept
{
    return (cluster == __rhs.cluster);
}

bool
InternalDistributionType::
operator!=(const InternalDistributionType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDistributionType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("cluster");
        __c.setString("type", "map");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & entry : cluster) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("key", vespalib::Memory(entry.first));
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            entry.second.serialize(__c4);
        }
    }
}

InternalDistributionType::InternalDistributionType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["cluster"]["value"].children(); __i++) {
        cluster[__inspector["cluster"]["value"][__i]["key"].asString().make_string()] = Cluster(__inspector["cluster"]["value"][__i]["value"]);
    }
}

InternalDistributionType::InternalDistributionType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::MapInserter<Cluster> __clusterInserter(cluster);
    __inspector["cluster"].traverse(__clusterInserter);
}

} // namespace internal

} // namespace vespa::config::content
