/**
 * @class vespa::config::content::Distribution
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_CONTENT_DISTRIBUTION_H
#define CLOUD_CONFIG_VESPA_CONFIG_CONTENT_DISTRIBUTION_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::content {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalDistributionType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalDistributionType> UP;
    class Cluster {
    public:
        class Group {
        public:
            class Nodes {
            public:
                Nodes(const ::config::StringVector & __lines);
                Nodes(const vespalib::slime::Inspector & __inspector);
                Nodes(const ::config::ConfigPayload & __payload);
                Nodes()  noexcept;
                Nodes(const Nodes & __rhs) noexcept;
                Nodes & operator = (const Nodes & __rhs) noexcept;
                Nodes(Nodes && __rhs) noexcept;
                Nodes & operator = (Nodes && __rhs) noexcept;
                ~Nodes();

                bool operator==(const Nodes& __rhs) const noexcept;
                bool operator!=(const Nodes& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /**
                 * Leaf groups will have a set of nodes within them. Branch
                 * groups will have none.
                 */
                int32_t index;

                /**
                 * Whether this node is retired, and data should migrate out of
                 * it.
                 */
                bool retired; // Default: false
            };
            typedef std::vector<Nodes> NodesVector;

            typedef std::map<vespalib::string, Nodes> NodesMap;

            Group(const ::config::StringVector & __lines);
            Group(const vespalib::slime::Inspector & __inspector);
            Group(const ::config::ConfigPayload & __payload);
            Group() ;
            Group(const Group & __rhs);
            Group & operator = (const Group & __rhs);
            Group(Group && __rhs) noexcept;
            Group & operator = (Group && __rhs) noexcept;
            ~Group();

            bool operator==(const Group& __rhs) const noexcept;
            bool operator!=(const Group& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * Hierarchical grouping divides the nodes into a tree of groups.
             * The index is the
             * string representation of a path from the root node in this tree,
             * e.g., "1.2.1".
             */
            vespalib::string index;

            /** Each group needs to have a name. Obviously. Duh. */
            vespalib::string name;

            /** Capacity of the given group. */
            double capacity; // Default: 1

            /**
             * Partitions define how copies are divided among child
             * groups/nodes.
             */
            vespalib::string partitions; // Default: ""
            NodesVector nodes;
        };
        typedef std::vector<Group> GroupVector;

        typedef std::map<vespalib::string, Group> GroupMap;

        Cluster(const ::config::StringVector & __lines);
        Cluster(const vespalib::slime::Inspector & __inspector);
        Cluster(const ::config::ConfigPayload & __payload);
        Cluster() ;
        Cluster(const Cluster & __rhs);
        Cluster & operator = (const Cluster & __rhs);
        Cluster(Cluster && __rhs) noexcept;
        Cluster & operator = (Cluster && __rhs) noexcept;
        ~Cluster();

        bool operator==(const Cluster& __rhs) const noexcept;
        bool operator!=(const Cluster& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        //  
        // Super config for distribution in each content cluster, keyed by
        // cluster id.
        //  

        /**
         * If this is set to true, the distributor will try to enforce one
         * active copy of
         * buckets per hierarchical leaf group. This lets the top level
         * dispatcher send
         * queries only to the nodes of one group, saving computational effort.
         * If used, hierarchical grouping can not be used for other purposes.
         * Using this option implies that:
         * - ready_copies == redundancy
         * - Only one level of hierarchical grouping may be defined.
         * - That level distributes copies to all defined groups.
         */
        bool activePerLeafGroup; // Default: false

        /**
         * The number of copies that should be "ready" to be active. Maximum is
         * redundancy.
         */
        int32_t readyCopies; // Default: 0

        /** How many copies of a document are stored, across nodes. */
        int32_t redundancy; // Default: 3

        /**
         * Initial redundancy allows put-operations to return as completed after
         * a subset of all copies have been stored.
         * A value of 0 disable this, and causes normal redundancy behavior
         * instead.
         */
        int32_t initialRedundancy; // Default: 0
        GroupVector group;
    };
    typedef std::vector<Cluster> ClusterVector;

    typedef std::map<vespalib::string, Cluster> ClusterMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalDistributionType(const ::config::ConfigValue & __value);
    InternalDistributionType(const ::config::ConfigDataBuffer & __value);
    InternalDistributionType(const ::config::ConfigPayload & __payload);
    InternalDistributionType() ;
    InternalDistributionType(const InternalDistributionType & __rhs);
    InternalDistributionType & operator = (const InternalDistributionType & __rhs);
    InternalDistributionType(InternalDistributionType && __rhs) noexcept;
    InternalDistributionType & operator = (InternalDistributionType && __rhs) noexcept;
    ~InternalDistributionType();

    bool operator==(const InternalDistributionType& __rhs) const noexcept;
    bool operator!=(const InternalDistributionType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    ClusterMap cluster;
};

} // namespace internal

typedef internal::InternalDistributionType DistributionConfigBuilder;
typedef const internal::InternalDistributionType DistributionConfig;

} // namespace vespa::config::content
#endif // VESPA_config_VESPA_CONFIG_CONTENT_DISTRIBUTION_H
