#include "config-fleetcontroller.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalFleetcontrollerType::CONFIG_DEF_MD5("b8121c8b457d5f750e235c61972fbc43");
const vespalib::string InternalFleetcontrollerType::CONFIG_DEF_NAME("fleetcontroller");
const vespalib::string InternalFleetcontrollerType::CONFIG_DEF_NAMESPACE("vespa.config.content");
const int64_t InternalFleetcontrollerType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content",
"cluster_name string restart",
"index int restart",
"fleet_controller_count int default=1",
"zookeeper_session_timeout double default=30.0",
"master_zookeeper_cooldown_period double default=60.0",
"state_gather_count int default=1",
"zookeeper_server string restart",
"rpc_port int default=6500 restart",
"http_port int default=0 restart",
"storage_transition_time int default=30000",
"distributor_transition_time int default=0",
"init_progress_time int default=0",
"min_time_between_new_systemstates int default=10000",
"max_premature_crashes int default=100000",
"stable_state_time_period int default=7200000",
"event_log_max_size int default=1024",
"event_node_log_max_size int default=1024",
"total_distributor_count int default=0",
"total_storage_count int default=0",
"min_distributors_up_count int default=1",
"min_storage_up_count int default=1",
"min_distributor_up_ratio double default=0.01",
"min_storage_up_ratio double default=0.01",
"cycle_wait_time double default=0.1",
"min_time_before_first_system_state_broadcast double default=30.0",
"get_node_state_request_timeout double default=120.0",
"max_slobrok_disconnect_grace_period double default=60.0",
"show_local_systemstates_in_event_log bool default=true",
"ideal_distribution_bits int default=16",
"min_node_ratio_per_group double default=0.0",
"max_deferred_task_version_wait_time_sec double default=30.0",
"cluster_has_global_document_types bool default=false",
"min_merge_completion_ratio double default=1.0",
"enable_two_phase_cluster_state_transitions bool default=false",
"enable_cluster_feed_block bool default=true",
"cluster_feed_block_limit{} double",
"cluster_feed_block_noise_level double default=0.0",
"max_number_of_groups_allowed_to_be_down int default=-1",
};
const ::config::StringVector InternalFleetcontrollerType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalFleetcontrollerType::InternalFleetcontrollerType()
    : clusterName(),
      index(0),
      fleetControllerCount(1),
      zookeeperSessionTimeout(30.0),
      masterZookeeperCooldownPeriod(60.0),
      stateGatherCount(1),
      zookeeperServer(),
      rpcPort(6500),
      httpPort(0),
      storageTransitionTime(30000),
      distributorTransitionTime(0),
      initProgressTime(0),
      minTimeBetweenNewSystemstates(10000),
      maxPrematureCrashes(100000),
      stableStateTimePeriod(7200000),
      eventLogMaxSize(1024),
      eventNodeLogMaxSize(1024),
      totalDistributorCount(0),
      totalStorageCount(0),
      minDistributorsUpCount(1),
      minStorageUpCount(1),
      minDistributorUpRatio(0.01),
      minStorageUpRatio(0.01),
      cycleWaitTime(0.1),
      minTimeBeforeFirstSystemStateBroadcast(30.0),
      getNodeStateRequestTimeout(120.0),
      maxSlobrokDisconnectGracePeriod(60.0),
      showLocalSystemstatesInEventLog(true),
      idealDistributionBits(16),
      minNodeRatioPerGroup(0.0),
      maxDeferredTaskVersionWaitTimeSec(30.0),
      clusterHasGlobalDocumentTypes(false),
      minMergeCompletionRatio(1.0),
      enableTwoPhaseClusterStateTransitions(false),
      enableClusterFeedBlock(true),
      clusterFeedBlockLimit(),
      clusterFeedBlockNoiseLevel(0.0),
      maxNumberOfGroupsAllowedToBeDown(-1)
{
}

InternalFleetcontrollerType::InternalFleetcontrollerType(const InternalFleetcontrollerType & __rhs) = default;
InternalFleetcontrollerType & InternalFleetcontrollerType::operator =(const InternalFleetcontrollerType & __rhs) = default;
InternalFleetcontrollerType::InternalFleetcontrollerType(InternalFleetcontrollerType && __rhs) noexcept = default;
InternalFleetcontrollerType & InternalFleetcontrollerType::operator =(InternalFleetcontrollerType && __rhs) noexcept = default;
InternalFleetcontrollerType::~InternalFleetcontrollerType() = default; 
InternalFleetcontrollerType::InternalFleetcontrollerType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        clusterName = ConfigParser::parse<vespalib::string>("cluster_name", __lines);
        ConfigParser::stripLinesForKey("cluster_name", __remainingValuesToParse);
        index = ConfigParser::parse<int32_t>("index", __lines);
        ConfigParser::stripLinesForKey("index", __remainingValuesToParse);
        fleetControllerCount = ConfigParser::parse<int32_t>("fleet_controller_count", __lines, 1);
        ConfigParser::stripLinesForKey("fleet_controller_count", __remainingValuesToParse);
        zookeeperSessionTimeout = ConfigParser::parse<double>("zookeeper_session_timeout", __lines, 30.0);
        ConfigParser::stripLinesForKey("zookeeper_session_timeout", __remainingValuesToParse);
        masterZookeeperCooldownPeriod = ConfigParser::parse<double>("master_zookeeper_cooldown_period", __lines, 60.0);
        ConfigParser::stripLinesForKey("master_zookeeper_cooldown_period", __remainingValuesToParse);
        stateGatherCount = ConfigParser::parse<int32_t>("state_gather_count", __lines, 1);
        ConfigParser::stripLinesForKey("state_gather_count", __remainingValuesToParse);
        zookeeperServer = ConfigParser::parse<vespalib::string>("zookeeper_server", __lines);
        ConfigParser::stripLinesForKey("zookeeper_server", __remainingValuesToParse);
        rpcPort = ConfigParser::parse<int32_t>("rpc_port", __lines, 6500);
        ConfigParser::stripLinesForKey("rpc_port", __remainingValuesToParse);
        httpPort = ConfigParser::parse<int32_t>("http_port", __lines, 0);
        ConfigParser::stripLinesForKey("http_port", __remainingValuesToParse);
        storageTransitionTime = ConfigParser::parse<int32_t>("storage_transition_time", __lines, 30000);
        ConfigParser::stripLinesForKey("storage_transition_time", __remainingValuesToParse);
        distributorTransitionTime = ConfigParser::parse<int32_t>("distributor_transition_time", __lines, 0);
        ConfigParser::stripLinesForKey("distributor_transition_time", __remainingValuesToParse);
        initProgressTime = ConfigParser::parse<int32_t>("init_progress_time", __lines, 0);
        ConfigParser::stripLinesForKey("init_progress_time", __remainingValuesToParse);
        minTimeBetweenNewSystemstates = ConfigParser::parse<int32_t>("min_time_between_new_systemstates", __lines, 10000);
        ConfigParser::stripLinesForKey("min_time_between_new_systemstates", __remainingValuesToParse);
        maxPrematureCrashes = ConfigParser::parse<int32_t>("max_premature_crashes", __lines, 100000);
        ConfigParser::stripLinesForKey("max_premature_crashes", __remainingValuesToParse);
        stableStateTimePeriod = ConfigParser::parse<int32_t>("stable_state_time_period", __lines, 7200000);
        ConfigParser::stripLinesForKey("stable_state_time_period", __remainingValuesToParse);
        eventLogMaxSize = ConfigParser::parse<int32_t>("event_log_max_size", __lines, 1024);
        ConfigParser::stripLinesForKey("event_log_max_size", __remainingValuesToParse);
        eventNodeLogMaxSize = ConfigParser::parse<int32_t>("event_node_log_max_size", __lines, 1024);
        ConfigParser::stripLinesForKey("event_node_log_max_size", __remainingValuesToParse);
        totalDistributorCount = ConfigParser::parse<int32_t>("total_distributor_count", __lines, 0);
        ConfigParser::stripLinesForKey("total_distributor_count", __remainingValuesToParse);
        totalStorageCount = ConfigParser::parse<int32_t>("total_storage_count", __lines, 0);
        ConfigParser::stripLinesForKey("total_storage_count", __remainingValuesToParse);
        minDistributorsUpCount = ConfigParser::parse<int32_t>("min_distributors_up_count", __lines, 1);
        ConfigParser::stripLinesForKey("min_distributors_up_count", __remainingValuesToParse);
        minStorageUpCount = ConfigParser::parse<int32_t>("min_storage_up_count", __lines, 1);
        ConfigParser::stripLinesForKey("min_storage_up_count", __remainingValuesToParse);
        minDistributorUpRatio = ConfigParser::parse<double>("min_distributor_up_ratio", __lines, 0.01);
        ConfigParser::stripLinesForKey("min_distributor_up_ratio", __remainingValuesToParse);
        minStorageUpRatio = ConfigParser::parse<double>("min_storage_up_ratio", __lines, 0.01);
        ConfigParser::stripLinesForKey("min_storage_up_ratio", __remainingValuesToParse);
        cycleWaitTime = ConfigParser::parse<double>("cycle_wait_time", __lines, 0.1);
        ConfigParser::stripLinesForKey("cycle_wait_time", __remainingValuesToParse);
        minTimeBeforeFirstSystemStateBroadcast = ConfigParser::parse<double>("min_time_before_first_system_state_broadcast", __lines, 30.0);
        ConfigParser::stripLinesForKey("min_time_before_first_system_state_broadcast", __remainingValuesToParse);
        getNodeStateRequestTimeout = ConfigParser::parse<double>("get_node_state_request_timeout", __lines, 120.0);
        ConfigParser::stripLinesForKey("get_node_state_request_timeout", __remainingValuesToParse);
        maxSlobrokDisconnectGracePeriod = ConfigParser::parse<double>("max_slobrok_disconnect_grace_period", __lines, 60.0);
        ConfigParser::stripLinesForKey("max_slobrok_disconnect_grace_period", __remainingValuesToParse);
        showLocalSystemstatesInEventLog = ConfigParser::parse<bool>("show_local_systemstates_in_event_log", __lines, true);
        ConfigParser::stripLinesForKey("show_local_systemstates_in_event_log", __remainingValuesToParse);
        idealDistributionBits = ConfigParser::parse<int32_t>("ideal_distribution_bits", __lines, 16);
        ConfigParser::stripLinesForKey("ideal_distribution_bits", __remainingValuesToParse);
        minNodeRatioPerGroup = ConfigParser::parse<double>("min_node_ratio_per_group", __lines, 0.0);
        ConfigParser::stripLinesForKey("min_node_ratio_per_group", __remainingValuesToParse);
        maxDeferredTaskVersionWaitTimeSec = ConfigParser::parse<double>("max_deferred_task_version_wait_time_sec", __lines, 30.0);
        ConfigParser::stripLinesForKey("max_deferred_task_version_wait_time_sec", __remainingValuesToParse);
        clusterHasGlobalDocumentTypes = ConfigParser::parse<bool>("cluster_has_global_document_types", __lines, false);
        ConfigParser::stripLinesForKey("cluster_has_global_document_types", __remainingValuesToParse);
        minMergeCompletionRatio = ConfigParser::parse<double>("min_merge_completion_ratio", __lines, 1.0);
        ConfigParser::stripLinesForKey("min_merge_completion_ratio", __remainingValuesToParse);
        enableTwoPhaseClusterStateTransitions = ConfigParser::parse<bool>("enable_two_phase_cluster_state_transitions", __lines, false);
        ConfigParser::stripLinesForKey("enable_two_phase_cluster_state_transitions", __remainingValuesToParse);
        enableClusterFeedBlock = ConfigParser::parse<bool>("enable_cluster_feed_block", __lines, true);
        ConfigParser::stripLinesForKey("enable_cluster_feed_block", __remainingValuesToParse);
        clusterFeedBlockLimit = ConfigParser::parseMap<double>("cluster_feed_block_limit", __lines);
        ConfigParser::stripLinesForKey("cluster_feed_block_limit", __remainingValuesToParse);
        clusterFeedBlockNoiseLevel = ConfigParser::parse<double>("cluster_feed_block_noise_level", __lines, 0.0);
        ConfigParser::stripLinesForKey("cluster_feed_block_noise_level", __remainingValuesToParse);
        maxNumberOfGroupsAllowedToBeDown = ConfigParser::parse<int32_t>("max_number_of_groups_allowed_to_be_down", __lines, -1);
        ConfigParser::stripLinesForKey("max_number_of_groups_allowed_to_be_down", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalFleetcontrollerType::
operator==(const InternalFleetcontrollerType& __rhs) const noexcept
{
    return (clusterName == __rhs.clusterName &&
            index == __rhs.index &&
            fleetControllerCount == __rhs.fleetControllerCount &&
            zookeeperSessionTimeout == __rhs.zookeeperSessionTimeout &&
            masterZookeeperCooldownPeriod == __rhs.masterZookeeperCooldownPeriod &&
            stateGatherCount == __rhs.stateGatherCount &&
            zookeeperServer == __rhs.zookeeperServer &&
            rpcPort == __rhs.rpcPort &&
            httpPort == __rhs.httpPort &&
            storageTransitionTime == __rhs.storageTransitionTime &&
            distributorTransitionTime == __rhs.distributorTransitionTime &&
            initProgressTime == __rhs.initProgressTime &&
            minTimeBetweenNewSystemstates == __rhs.minTimeBetweenNewSystemstates &&
            maxPrematureCrashes == __rhs.maxPrematureCrashes &&
            stableStateTimePeriod == __rhs.stableStateTimePeriod &&
            eventLogMaxSize == __rhs.eventLogMaxSize &&
            eventNodeLogMaxSize == __rhs.eventNodeLogMaxSize &&
            totalDistributorCount == __rhs.totalDistributorCount &&
            totalStorageCount == __rhs.totalStorageCount &&
            minDistributorsUpCount == __rhs.minDistributorsUpCount &&
            minStorageUpCount == __rhs.minStorageUpCount &&
            minDistributorUpRatio == __rhs.minDistributorUpRatio &&
            minStorageUpRatio == __rhs.minStorageUpRatio &&
            cycleWaitTime == __rhs.cycleWaitTime &&
            minTimeBeforeFirstSystemStateBroadcast == __rhs.minTimeBeforeFirstSystemStateBroadcast &&
            getNodeStateRequestTimeout == __rhs.getNodeStateRequestTimeout &&
            maxSlobrokDisconnectGracePeriod == __rhs.maxSlobrokDisconnectGracePeriod &&
            showLocalSystemstatesInEventLog == __rhs.showLocalSystemstatesInEventLog &&
            idealDistributionBits == __rhs.idealDistributionBits &&
            minNodeRatioPerGroup == __rhs.minNodeRatioPerGroup &&
            maxDeferredTaskVersionWaitTimeSec == __rhs.maxDeferredTaskVersionWaitTimeSec &&
            clusterHasGlobalDocumentTypes == __rhs.clusterHasGlobalDocumentTypes &&
            minMergeCompletionRatio == __rhs.minMergeCompletionRatio &&
            enableTwoPhaseClusterStateTransitions == __rhs.enableTwoPhaseClusterStateTransitions &&
            enableClusterFeedBlock == __rhs.enableClusterFeedBlock &&
            clusterFeedBlockLimit == __rhs.clusterFeedBlockLimit &&
            clusterFeedBlockNoiseLevel == __rhs.clusterFeedBlockNoiseLevel &&
            maxNumberOfGroupsAllowedToBeDown == __rhs.maxNumberOfGroupsAllowedToBeDown);
}

bool
InternalFleetcontrollerType::
operator!=(const InternalFleetcontrollerType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalFleetcontrollerType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("cluster_name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(clusterName));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("index");
        __c.setString("type", "int");
        __c.setLong("value", index);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fleet_controller_count");
        __c.setString("type", "int");
        __c.setLong("value", fleetControllerCount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("zookeeper_session_timeout");
        __c.setString("type", "double");
        __c.setDouble("value", zookeeperSessionTimeout);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("master_zookeeper_cooldown_period");
        __c.setString("type", "double");
        __c.setDouble("value", masterZookeeperCooldownPeriod);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("state_gather_count");
        __c.setString("type", "int");
        __c.setLong("value", stateGatherCount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("zookeeper_server");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(zookeeperServer));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("rpc_port");
        __c.setString("type", "int");
        __c.setLong("value", rpcPort);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("http_port");
        __c.setString("type", "int");
        __c.setLong("value", httpPort);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("storage_transition_time");
        __c.setString("type", "int");
        __c.setLong("value", storageTransitionTime);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("distributor_transition_time");
        __c.setString("type", "int");
        __c.setLong("value", distributorTransitionTime);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("init_progress_time");
        __c.setString("type", "int");
        __c.setLong("value", initProgressTime);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("min_time_between_new_systemstates");
        __c.setString("type", "int");
        __c.setLong("value", minTimeBetweenNewSystemstates);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_premature_crashes");
        __c.setString("type", "int");
        __c.setLong("value", maxPrematureCrashes);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("stable_state_time_period");
        __c.setString("type", "int");
        __c.setLong("value", stableStateTimePeriod);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("event_log_max_size");
        __c.setString("type", "int");
        __c.setLong("value", eventLogMaxSize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("event_node_log_max_size");
        __c.setString("type", "int");
        __c.setLong("value", eventNodeLogMaxSize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("total_distributor_count");
        __c.setString("type", "int");
        __c.setLong("value", totalDistributorCount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("total_storage_count");
        __c.setString("type", "int");
        __c.setLong("value", totalStorageCount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("min_distributors_up_count");
        __c.setString("type", "int");
        __c.setLong("value", minDistributorsUpCount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("min_storage_up_count");
        __c.setString("type", "int");
        __c.setLong("value", minStorageUpCount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("min_distributor_up_ratio");
        __c.setString("type", "double");
        __c.setDouble("value", minDistributorUpRatio);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("min_storage_up_ratio");
        __c.setString("type", "double");
        __c.setDouble("value", minStorageUpRatio);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("cycle_wait_time");
        __c.setString("type", "double");
        __c.setDouble("value", cycleWaitTime);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("min_time_before_first_system_state_broadcast");
        __c.setString("type", "double");
        __c.setDouble("value", minTimeBeforeFirstSystemStateBroadcast);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("get_node_state_request_timeout");
        __c.setString("type", "double");
        __c.setDouble("value", getNodeStateRequestTimeout);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_slobrok_disconnect_grace_period");
        __c.setString("type", "double");
        __c.setDouble("value", maxSlobrokDisconnectGracePeriod);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("show_local_systemstates_in_event_log");
        __c.setString("type", "bool");
        __c.setBool("value", showLocalSystemstatesInEventLog);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ideal_distribution_bits");
        __c.setString("type", "int");
        __c.setLong("value", idealDistributionBits);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("min_node_ratio_per_group");
        __c.setString("type", "double");
        __c.setDouble("value", minNodeRatioPerGroup);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_deferred_task_version_wait_time_sec");
        __c.setString("type", "double");
        __c.setDouble("value", maxDeferredTaskVersionWaitTimeSec);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("cluster_has_global_document_types");
        __c.setString("type", "bool");
        __c.setBool("value", clusterHasGlobalDocumentTypes);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("min_merge_completion_ratio");
        __c.setString("type", "double");
        __c.setDouble("value", minMergeCompletionRatio);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_two_phase_cluster_state_transitions");
        __c.setString("type", "bool");
        __c.setBool("value", enableTwoPhaseClusterStateTransitions);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_cluster_feed_block");
        __c.setString("type", "bool");
        __c.setBool("value", enableClusterFeedBlock);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("cluster_feed_block_limit");
        __c.setString("type", "map");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & entry : clusterFeedBlockLimit) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("key", vespalib::Memory(entry.first));
            __c3.setString("type", "double");
            __c3.setDouble("value", entry.second);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("cluster_feed_block_noise_level");
        __c.setString("type", "double");
        __c.setDouble("value", clusterFeedBlockNoiseLevel);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_number_of_groups_allowed_to_be_down");
        __c.setString("type", "int");
        __c.setLong("value", maxNumberOfGroupsAllowedToBeDown);
    }
}

InternalFleetcontrollerType::InternalFleetcontrollerType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    clusterName = __inspector["cluster_name"]["value"].asString().make_string();
    index = __inspector["index"]["value"].asLong();
    fleetControllerCount = __inspector["fleet_controller_count"]["value"].asLong();
    zookeeperSessionTimeout = __inspector["zookeeper_session_timeout"]["value"].asDouble();
    masterZookeeperCooldownPeriod = __inspector["master_zookeeper_cooldown_period"]["value"].asDouble();
    stateGatherCount = __inspector["state_gather_count"]["value"].asLong();
    zookeeperServer = __inspector["zookeeper_server"]["value"].asString().make_string();
    rpcPort = __inspector["rpc_port"]["value"].asLong();
    httpPort = __inspector["http_port"]["value"].asLong();
    storageTransitionTime = __inspector["storage_transition_time"]["value"].asLong();
    distributorTransitionTime = __inspector["distributor_transition_time"]["value"].asLong();
    initProgressTime = __inspector["init_progress_time"]["value"].asLong();
    minTimeBetweenNewSystemstates = __inspector["min_time_between_new_systemstates"]["value"].asLong();
    maxPrematureCrashes = __inspector["max_premature_crashes"]["value"].asLong();
    stableStateTimePeriod = __inspector["stable_state_time_period"]["value"].asLong();
    eventLogMaxSize = __inspector["event_log_max_size"]["value"].asLong();
    eventNodeLogMaxSize = __inspector["event_node_log_max_size"]["value"].asLong();
    totalDistributorCount = __inspector["total_distributor_count"]["value"].asLong();
    totalStorageCount = __inspector["total_storage_count"]["value"].asLong();
    minDistributorsUpCount = __inspector["min_distributors_up_count"]["value"].asLong();
    minStorageUpCount = __inspector["min_storage_up_count"]["value"].asLong();
    minDistributorUpRatio = __inspector["min_distributor_up_ratio"]["value"].asDouble();
    minStorageUpRatio = __inspector["min_storage_up_ratio"]["value"].asDouble();
    cycleWaitTime = __inspector["cycle_wait_time"]["value"].asDouble();
    minTimeBeforeFirstSystemStateBroadcast = __inspector["min_time_before_first_system_state_broadcast"]["value"].asDouble();
    getNodeStateRequestTimeout = __inspector["get_node_state_request_timeout"]["value"].asDouble();
    maxSlobrokDisconnectGracePeriod = __inspector["max_slobrok_disconnect_grace_period"]["value"].asDouble();
    showLocalSystemstatesInEventLog = __inspector["show_local_systemstates_in_event_log"]["value"].asBool();
    idealDistributionBits = __inspector["ideal_distribution_bits"]["value"].asLong();
    minNodeRatioPerGroup = __inspector["min_node_ratio_per_group"]["value"].asDouble();
    maxDeferredTaskVersionWaitTimeSec = __inspector["max_deferred_task_version_wait_time_sec"]["value"].asDouble();
    clusterHasGlobalDocumentTypes = __inspector["cluster_has_global_document_types"]["value"].asBool();
    minMergeCompletionRatio = __inspector["min_merge_completion_ratio"]["value"].asDouble();
    enableTwoPhaseClusterStateTransitions = __inspector["enable_two_phase_cluster_state_transitions"]["value"].asBool();
    enableClusterFeedBlock = __inspector["enable_cluster_feed_block"]["value"].asBool();
    for (size_t __i = 0; __i < __inspector["cluster_feed_block_limit"]["value"].children(); __i++) {
        clusterFeedBlockLimit[__inspector["cluster_feed_block_limit"]["value"][__i]["key"].asString().make_string()] = __inspector["cluster_feed_block_limit"]["value"][__i]["value"].asDouble();
    }
    clusterFeedBlockNoiseLevel = __inspector["cluster_feed_block_noise_level"]["value"].asDouble();
    maxNumberOfGroupsAllowedToBeDown = __inspector["max_number_of_groups_allowed_to_be_down"]["value"].asLong();
}

InternalFleetcontrollerType::InternalFleetcontrollerType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    clusterName = ::config::internal::ValueConverter<vespalib::string>()("cluster_name", __inspector["cluster_name"]);
    index = ::config::internal::ValueConverter<int32_t>()("index", __inspector["index"]);
    fleetControllerCount = ::config::internal::ValueConverter<int32_t>()(__inspector["fleet_controller_count"], 1);
    zookeeperSessionTimeout = ::config::internal::ValueConverter<double>()(__inspector["zookeeper_session_timeout"], 30.0);
    masterZookeeperCooldownPeriod = ::config::internal::ValueConverter<double>()(__inspector["master_zookeeper_cooldown_period"], 60.0);
    stateGatherCount = ::config::internal::ValueConverter<int32_t>()(__inspector["state_gather_count"], 1);
    zookeeperServer = ::config::internal::ValueConverter<vespalib::string>()("zookeeper_server", __inspector["zookeeper_server"]);
    rpcPort = ::config::internal::ValueConverter<int32_t>()(__inspector["rpc_port"], 6500);
    httpPort = ::config::internal::ValueConverter<int32_t>()(__inspector["http_port"], 0);
    storageTransitionTime = ::config::internal::ValueConverter<int32_t>()(__inspector["storage_transition_time"], 30000);
    distributorTransitionTime = ::config::internal::ValueConverter<int32_t>()(__inspector["distributor_transition_time"], 0);
    initProgressTime = ::config::internal::ValueConverter<int32_t>()(__inspector["init_progress_time"], 0);
    minTimeBetweenNewSystemstates = ::config::internal::ValueConverter<int32_t>()(__inspector["min_time_between_new_systemstates"], 10000);
    maxPrematureCrashes = ::config::internal::ValueConverter<int32_t>()(__inspector["max_premature_crashes"], 100000);
    stableStateTimePeriod = ::config::internal::ValueConverter<int32_t>()(__inspector["stable_state_time_period"], 7200000);
    eventLogMaxSize = ::config::internal::ValueConverter<int32_t>()(__inspector["event_log_max_size"], 1024);
    eventNodeLogMaxSize = ::config::internal::ValueConverter<int32_t>()(__inspector["event_node_log_max_size"], 1024);
    totalDistributorCount = ::config::internal::ValueConverter<int32_t>()(__inspector["total_distributor_count"], 0);
    totalStorageCount = ::config::internal::ValueConverter<int32_t>()(__inspector["total_storage_count"], 0);
    minDistributorsUpCount = ::config::internal::ValueConverter<int32_t>()(__inspector["min_distributors_up_count"], 1);
    minStorageUpCount = ::config::internal::ValueConverter<int32_t>()(__inspector["min_storage_up_count"], 1);
    minDistributorUpRatio = ::config::internal::ValueConverter<double>()(__inspector["min_distributor_up_ratio"], 0.01);
    minStorageUpRatio = ::config::internal::ValueConverter<double>()(__inspector["min_storage_up_ratio"], 0.01);
    cycleWaitTime = ::config::internal::ValueConverter<double>()(__inspector["cycle_wait_time"], 0.1);
    minTimeBeforeFirstSystemStateBroadcast = ::config::internal::ValueConverter<double>()(__inspector["min_time_before_first_system_state_broadcast"], 30.0);
    getNodeStateRequestTimeout = ::config::internal::ValueConverter<double>()(__inspector["get_node_state_request_timeout"], 120.0);
    maxSlobrokDisconnectGracePeriod = ::config::internal::ValueConverter<double>()(__inspector["max_slobrok_disconnect_grace_period"], 60.0);
    showLocalSystemstatesInEventLog = ::config::internal::ValueConverter<bool>()(__inspector["show_local_systemstates_in_event_log"], true);
    idealDistributionBits = ::config::internal::ValueConverter<int32_t>()(__inspector["ideal_distribution_bits"], 16);
    minNodeRatioPerGroup = ::config::internal::ValueConverter<double>()(__inspector["min_node_ratio_per_group"], 0.0);
    maxDeferredTaskVersionWaitTimeSec = ::config::internal::ValueConverter<double>()(__inspector["max_deferred_task_version_wait_time_sec"], 30.0);
    clusterHasGlobalDocumentTypes = ::config::internal::ValueConverter<bool>()(__inspector["cluster_has_global_document_types"], false);
    minMergeCompletionRatio = ::config::internal::ValueConverter<double>()(__inspector["min_merge_completion_ratio"], 1.0);
    enableTwoPhaseClusterStateTransitions = ::config::internal::ValueConverter<bool>()(__inspector["enable_two_phase_cluster_state_transitions"], false);
    enableClusterFeedBlock = ::config::internal::ValueConverter<bool>()(__inspector["enable_cluster_feed_block"], true);
    ::config::internal::MapInserter<double> __clusterFeedBlockLimitInserter(clusterFeedBlockLimit);
    __inspector["cluster_feed_block_limit"].traverse(__clusterFeedBlockLimitInserter);
    clusterFeedBlockNoiseLevel = ::config::internal::ValueConverter<double>()(__inspector["cluster_feed_block_noise_level"], 0.0);
    maxNumberOfGroupsAllowedToBeDown = ::config::internal::ValueConverter<int32_t>()(__inspector["max_number_of_groups_allowed_to_be_down"], -1);
}

} // namespace internal

} // namespace vespa::config::content
