/**
 * @class vespa::config::content::Fleetcontroller
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_CONTENT_FLEETCONTROLLER_H
#define CLOUD_CONFIG_VESPA_CONFIG_CONTENT_FLEETCONTROLLER_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::content {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalFleetcontrollerType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalFleetcontrollerType> UP;
    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalFleetcontrollerType(const ::config::ConfigValue & __value);
    InternalFleetcontrollerType(const ::config::ConfigDataBuffer & __value);
    InternalFleetcontrollerType(const ::config::ConfigPayload & __payload);
    InternalFleetcontrollerType() ;
    InternalFleetcontrollerType(const InternalFleetcontrollerType & __rhs);
    InternalFleetcontrollerType & operator = (const InternalFleetcontrollerType & __rhs);
    InternalFleetcontrollerType(InternalFleetcontrollerType && __rhs) noexcept;
    InternalFleetcontrollerType & operator = (InternalFleetcontrollerType && __rhs) noexcept;
    ~InternalFleetcontrollerType();

    bool operator==(const InternalFleetcontrollerType& __rhs) const noexcept;
    bool operator!=(const InternalFleetcontrollerType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /** Name of VDS cluster */
    vespalib::string clusterName;

    /**
     * The fleetcontroller index. Each fleetcontroller should have a unique
     * index
     * which can be used to identify them.
     */
    int32_t index;

    /**
     * Number of fleet controllers. If more than one, the fleetcontroller needs
     * to
     * do master election in order to know which one is active. It then needs to
     * know how many exist.
     */
    int32_t fleetControllerCount; // Default: 1

    /**
     * The number of seconds we can attempt to talk to zookeeper before our
     * session
     * time out and we lose our connection.
     */
    double zookeeperSessionTimeout; // Default: 30.0

    /**
     * When a master candidate see the master disappearing from ZooKeeper, it
     * wants
     * to take over as master. But, before taking over, the master should be
     * aware
     * that it has lost its zookeeper connection, so it will stop reacting as
     * master. Suggests setting this to 2-3 times the zookeeper session timeout.
     * (Set in number of seconds)
     */
    double masterZookeeperCooldownPeriod; // Default: 60.0

    /**
     * Sets how many fleetcontrollers will gather state. A fleetcontroller
     * gathering state can take over quicker should the master fail.
     * If set to 1, only master will gather state. If set higher, others will
     * also do so, prioritizing those fleetcontrollers likely to be the ones to
     * take over if the master fails.
     * TODO: Deprecated, not used anymore, remove in Vespa 9
     */
    int32_t stateGatherCount; // Default: 1

    /** Location of ZooKeeper servers */
    vespalib::string zookeeperServer;

    /** RPC Port used by fleetcontroller */
    int32_t rpcPort; // Default: 6500

    /** Port where fleetcontroller listens for HTTP status request */
    int32_t httpPort; // Default: 0

    /**
     * Maximum number of milliseconds a storage node will be automatically
     * reported
     * in maintenance due to node recently being available. (If 0, no time at
     * all)
     */
    int32_t storageTransitionTime; // Default: 30000

    /**
     * Maximum number of milliseconds a distributor will be automatically
     * reported
     * in maintenance due to node recently being available. (If 0, no time at
     * all)
     *  
     * Currently default is 0.. Should probably be more when we know it is
     * working
     * correctly
     */
    int32_t distributorTransitionTime; // Default: 0

    /**
     * Maximum number of milliseconds allowed between progress increase during
     * initialization. If no progress have been made during this time period,
     * the
     * node will be considered down.
     *  
     * Currently disabled as 5.0 nodes gets load while initializing which may be
     * higher pri than initializing, so nodes can stay in init for a long time.
     */
    int32_t initProgressTime; // Default: 0

    /**
     * Minimum time (in ms) between system state updates. To limit updates in a
     * system where a lot is happening at the same time, this value will make
     * sure
     * we dont change the state too often.
     */
    int32_t minTimeBetweenNewSystemstates; // Default: 10000

    /**
     * The maximum amount of premature crashes a node is allowed to have in a
     * row
     * before the fleetcontroller disables that node.
     */
    int32_t maxPrematureCrashes; // Default: 100000

    /**
     * If a node has been down or up this many milliseconds, clear the premature
     * crash count of a node and consider the node as stable
     */
    int32_t stableStateTimePeriod; // Default: 7200000

    /** The maximum number of events to keep in the event log */
    int32_t eventLogMaxSize; // Default: 1024

    /**
     * The maximum number of node events to keep in the node event log per node
     */
    int32_t eventNodeLogMaxSize; // Default: 1024

    /**
     * The total number of distributor nodes that can exist. If 0, we dont know
     * and
     * will use the highest distributor index number we have ever seen + 1.
     */
    int32_t totalDistributorCount; // Default: 0

    /**
     * The total number of storage nodes that can exist. If 0, we dont know and
     * will use the highest storage index number we have ever seen + 1.
     */
    int32_t totalStorageCount; // Default: 0

    /**
     * The minimum number of distributor nodes that should be up for the cluster
     * state to be up. (Retired nodes counts as up in this case)
     */
    int32_t minDistributorsUpCount; // Default: 1

    /**
     * The minimum number of storage nodes that should be up for the cluster
     * state
     * to be up (Retired nodes counts as up in this case)
     */
    int32_t minStorageUpCount; // Default: 1

    /**
     * The minimum ratio of known distributor nodes that should be up (or
     * retired)
     * for the cluster state to stay up.
     */
    double minDistributorUpRatio; // Default: 0.01

    /**
     * The minimum ratio of known storage nodes that should be up (or retired)
     * for
     * the cluster state to stay up.
     */
    double minStorageUpRatio; // Default: 0.01

    /**
     * Seconds to sleep after doing a work cycle where we did no work. Some
     * events do not interrupt the sleeping, such as slobrok changes, so
     * shouldn't
     * set this too high
     */
    double cycleWaitTime; // Default: 0.1

    /**
     * Minimum time to pass in seconds before broadcasting our first systemstate
     * as
     * a new fleetcontroller. Will broadcast earlier than this if we have
     * gathered
     * state from all before this. To prevent disturbance when taking over as
     * fleetcontroller, give nodes a bit of time to answer so we dont
     * temporarily
     * report nodes as down. See also max_slobrok_disconnect_grace_period and
     * get_node_state_request_timeout.
     */
    double minTimeBeforeFirstSystemStateBroadcast; // Default: 30.0

    /**
     * Request timeout of node state requests. Keeping a high timeout allows us
     * to
     * always have a pending operation with very low cost. Keeping a low timeout
     * is
     * good to detect issues like packet loss. The default tries to balance the
     * two
     * by not resending too often, but detecting packet loss within a minute at
     * least. If we can guarantee RPC layer to fail on packet loss within
     * reasonable time we should increase this default.
     */
    double getNodeStateRequestTimeout; // Default: 120.0

    /**
     * If a node is out of slobrok longer than this time period, assume the node
     * is down, even if we have a pending node state request to it. Slobrok does
     * a bit more keep alive checking than fleetcontroller, so it is possible
     * that
     * the node disappears from slobrok while it still looks ok in
     * fleetcontroller.
     */
    double maxSlobrokDisconnectGracePeriod; // Default: 60.0

    /**
     * Whether to show system states that have never been sent to storage nodes
     * in
     * the event log.
     */
    bool showLocalSystemstatesInEventLog; // Default: true

    /** The ideal number of distribution bits this system should have */
    int32_t idealDistributionBits; // Default: 16

    /**
     * Minimum ratio of nodes that have to be available (i.e. not Down) in any
     * hierarchic content cluster group. If a higher ratio than this is Down at
     * any point, the remaning nodes in the group will be automatically marked
     * as down. Group nodes will automatically be taken back up as soon as node
     * availability has been restored above the given threshold.
     * Default is 0, i.e. functionality is for all intents and purposes
     * disabled.
     */
    double minNodeRatioPerGroup; // Default: 0.0

    /**
     * If a cluster controller task has a dependency on a given cluster state
     * version being published and ACKed by the cluster, it will be put on a
     * wait
     * queue while holding up the container thread associated with the task.
     * This config specifies the maximum time a task can be held in this queue
     * before being automatically failed out, if a version has not been ACKed
     * within this duration.
     */
    double maxDeferredTaskVersionWaitTimeSec; // Default: 30.0

    /**
     * Whether or not the content cluster the controller has responsibility for
     * contains any document types that are tagged as global. If this is true,
     * global document-specific behavior is enabled that marks nodes down in the
     * default space if they have merges pending in the global bucket space.
     */
    bool clusterHasGlobalDocumentTypes; // Default: false

    /**
     * The minimum merge completion ratio of buckets in a bucket space before it
     * is considered complete.
     *  
     * Bucket merges are considered complete when:
     * ((buckets_total - buckets_pending) / buckets_total)) >= min_merge_complet-
     * ion_ratio
     */
    double minMergeCompletionRatio; // Default: 1.0

    /**
     * If enabled, cluster state transitions are performed as two distinct
     * phases:
     *  
     * 1) state bundle propagation and bucket info gathering phase
     * 2) state activation phase, which is not performed until all nodes have
     * completed phase 1
     *  
     * This is to enable read-only operations to pass through the system during
     * phase 1
     * while nodes await phase 2. If this feature is disabled, nodes will
     * implicitly do
     * phase 2 as part of phase 1 at their own leisure, which means that actual
     * state
     * activation may happen at wildly different times throughout the cluster.
     * The 2 phase
     * transition logic aims to minimize the window of time where active states
     * diverge.
     */
    bool enableTwoPhaseClusterStateTransitions; // Default: false

    /**
     * If enabled, the cluster controller observes reported (categorized)
     * resource usage from content nodes (via host info),
     * and decides whether external feed should be blocked (or unblocked) in the
     * entire cluster.
     *  
     * Each resource category has a limit, which is specified in cluster_feed_bl-
     * ock_limit.
     * If one resource category from one content node is above the configured
     * limit, feed is blocked.
     * This information is pushed to all distributor nodes via a new cluster
     * state bundle.
     * The distributor nodes handle the actual feed blocking based on this
     * information.
     */
    bool enableClusterFeedBlock; // Default: true

    /**
     * Contains all resource categories (key) with its corresponding feed block
     * limit (value)
     * used when the cluster controller decides whether external feed should be
     * blocked (or unblocked) in the entire cluster.
     *  
     * The keys used must match the similar keys in the host info JSON
     * structure.
     * All limits are numbers between 0.0 and 1.0.
     */
    ::config::DoubleMap clusterFeedBlockLimit;

    /**
     * To avoid having the cluster feed block state flip-flop from nodes that
     * are hovering
     * just around the feed block limits, this noise threshold implicitly makes
     * the
     * feed block limit value _lower_ for a resource that is already exhausted.
     * I.e. the
     * node must reach a lower resource usage than the limit for feed to be
     * unblocked.
     * This is in absolute numbers, so 0.01 implies that a block limit of 0.8
     * effectively
     * becomes 0.79 for an already blocked node.
     */
    double clusterFeedBlockNoiseLevel; // Default: 0.0

    /**
     * For apps that have several groups this controls how many groups are
     * allowed to
     * be down simultaneously in this cluster.
     */
    int32_t maxNumberOfGroupsAllowedToBeDown; // Default: -1
};

} // namespace internal

typedef internal::InternalFleetcontrollerType FleetcontrollerConfigBuilder;
typedef const internal::InternalFleetcontrollerType FleetcontrollerConfig;

} // namespace vespa::config::content
#endif // VESPA_config_VESPA_CONFIG_CONTENT_FLEETCONTROLLER_H
