#include "config-hwinfo.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalHwinfoType::CONFIG_DEF_MD5("5549a24b7fba7a1233af3b6f426c2d00");
const vespalib::string InternalHwinfoType::CONFIG_DEF_NAME("hwinfo");
const vespalib::string InternalHwinfoType::CONFIG_DEF_NAMESPACE("vespa.config.search.core");
const int64_t InternalHwinfoType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search.core",
"disk.writespeed double default = 0.0",
"disk.sampletime long default = 0",
};
const ::config::StringVector InternalHwinfoType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalHwinfoType::Disk::Disk() noexcept
    : writespeed(0.0),
      sampletime(0)
{
}

InternalHwinfoType::Disk::Disk(const Disk & __rhs) noexcept = default;
InternalHwinfoType::Disk & InternalHwinfoType::Disk::operator = (const Disk & __rhs) noexcept = default;
InternalHwinfoType::Disk::Disk(Disk && __rhs) noexcept = default;
InternalHwinfoType::Disk & InternalHwinfoType::Disk::operator = (Disk && __rhs) noexcept = default;
InternalHwinfoType::Disk::~Disk() = default; 
InternalHwinfoType::Disk::Disk(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    writespeed = ConfigParser::parse<double>("writespeed", __lines, 0.0);
    ConfigParser::stripLinesForKey("writespeed", __remainingValuesToParse);
    sampletime = ConfigParser::parse<int64_t>("sampletime", __lines, 0);
    ConfigParser::stripLinesForKey("sampletime", __remainingValuesToParse);
}

bool
InternalHwinfoType::Disk::operator==(const Disk& __rhs) const noexcept
{
    return (writespeed == __rhs.writespeed &&
            sampletime == __rhs.sampletime);
}

bool
InternalHwinfoType::Disk::operator!=(const Disk& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalHwinfoType::Disk::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("writespeed");
        __c.setString("type", "double");
        __c.setDouble("value", writespeed);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("sampletime");
        __c.setString("type", "long");
        __c.setLong("value", sampletime);
    }
}

InternalHwinfoType::Disk::Disk(const vespalib::slime::Inspector & __inspector)
{
    writespeed = __inspector["writespeed"]["value"].asDouble();
    sampletime = __inspector["sampletime"]["value"].asLong();
}

InternalHwinfoType::Disk::Disk(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    writespeed = ::config::internal::ValueConverter<double>()(__inspector["writespeed"], 0.0);
    sampletime = ::config::internal::ValueConverter<int64_t>()(__inspector["sampletime"], 0);
}

InternalHwinfoType::InternalHwinfoType() noexcept
    : disk()
{
}

InternalHwinfoType::InternalHwinfoType(const InternalHwinfoType & __rhs) noexcept = default;
InternalHwinfoType & InternalHwinfoType::operator =(const InternalHwinfoType & __rhs) noexcept = default;
InternalHwinfoType::InternalHwinfoType(InternalHwinfoType && __rhs) noexcept = default;
InternalHwinfoType & InternalHwinfoType::operator =(InternalHwinfoType && __rhs) noexcept = default;
InternalHwinfoType::~InternalHwinfoType() = default; 
InternalHwinfoType::InternalHwinfoType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        disk = ConfigParser::parseStruct<Disk>("disk", __lines);
        ConfigParser::stripLinesForKey("disk", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalHwinfoType::operator==(const InternalHwinfoType& __rhs) const noexcept
{
    return (disk == __rhs.disk);
}

bool
InternalHwinfoType::operator!=(const InternalHwinfoType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalHwinfoType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("disk");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        disk.serialize(__c2);
    }
}

InternalHwinfoType::InternalHwinfoType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    disk = Disk(__inspector["disk"]["value"]);
}

InternalHwinfoType::InternalHwinfoType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    disk = ::config::internal::ValueConverter<Disk>()(__inspector["disk"]);
}

} // namespace internal

} // namespace vespa::config::search::core
