#include "config-ilscripts.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::configdefinition {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalIlscriptsType::CONFIG_DEF_MD5("ca2476e0c561b541f90ce523620739b0");
const vespalib::string InternalIlscriptsType::CONFIG_DEF_NAME("ilscripts");
const vespalib::string InternalIlscriptsType::CONFIG_DEF_NAMESPACE("vespa.configdefinition");
const int64_t InternalIlscriptsType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.configdefinition",
"maxtermoccurrences int default=10000",
"maxtokenlength int default=1000",
"fieldmatchmaxlength int default=1000000",
"ilscript[].doctype string",
"ilscript[].docfield[] string",
"ilscript[].content[] string",
};
const ::config::StringVector InternalIlscriptsType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalIlscriptsType::Ilscript::Ilscript()
    : doctype(),
      docfield(),
      content()
{
}

InternalIlscriptsType::Ilscript::Ilscript(const Ilscript & __rhs) = default;
InternalIlscriptsType::Ilscript & InternalIlscriptsType::Ilscript::operator = (const Ilscript & __rhs) = default;
InternalIlscriptsType::Ilscript::Ilscript(Ilscript && __rhs) noexcept = default;
InternalIlscriptsType::Ilscript & InternalIlscriptsType::Ilscript::operator = (Ilscript && __rhs) noexcept = default;
InternalIlscriptsType::Ilscript::~Ilscript() = default; 
InternalIlscriptsType::Ilscript::Ilscript(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    doctype = ConfigParser::parse<vespalib::string>("doctype", __lines);
    ConfigParser::stripLinesForKey("doctype", __remainingValuesToParse);
    docfield = ConfigParser::parseArray<::config::StringVector>("docfield", __lines);
    ConfigParser::stripLinesForKey("docfield", __remainingValuesToParse);
    content = ConfigParser::parseArray<::config::StringVector>("content", __lines);
    ConfigParser::stripLinesForKey("content", __remainingValuesToParse);
}

bool
InternalIlscriptsType::Ilscript::operator==(const Ilscript& __rhs) const noexcept
{
    return (doctype == __rhs.doctype &&
            docfield == __rhs.docfield &&
            content == __rhs.content);
}

bool
InternalIlscriptsType::Ilscript::operator!=(const Ilscript& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalIlscriptsType::Ilscript::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("doctype");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(doctype));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("docfield");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : docfield) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("content");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : content) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
}

InternalIlscriptsType::Ilscript::Ilscript(const vespalib::slime::Inspector & __inspector)
{
    doctype = __inspector["doctype"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["docfield"]["value"].children(); __i++) {
        docfield.push_back(__inspector["docfield"]["value"][__i]["value"].asString().make_string());
    }
    for (size_t __i = 0; __i < __inspector["content"]["value"].children(); __i++) {
        content.push_back(__inspector["content"]["value"][__i]["value"].asString().make_string());
    }
}

InternalIlscriptsType::Ilscript::Ilscript(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    doctype = ::config::internal::ValueConverter<vespalib::string>()("doctype", __inspector["doctype"]);
    ::config::internal::VectorInserter<::config::StringVector> __docfieldInserter(docfield);
    __inspector["docfield"].traverse(__docfieldInserter);
    ::config::internal::VectorInserter<::config::StringVector> __contentInserter(content);
    __inspector["content"].traverse(__contentInserter);
}

InternalIlscriptsType::InternalIlscriptsType()
    : maxtermoccurrences(10000),
      maxtokenlength(1000),
      fieldmatchmaxlength(1000000),
      ilscript()
{
}

InternalIlscriptsType::InternalIlscriptsType(const InternalIlscriptsType & __rhs) = default;
InternalIlscriptsType & InternalIlscriptsType::operator =(const InternalIlscriptsType & __rhs) = default;
InternalIlscriptsType::InternalIlscriptsType(InternalIlscriptsType && __rhs) noexcept = default;
InternalIlscriptsType & InternalIlscriptsType::operator =(InternalIlscriptsType && __rhs) noexcept = default;
InternalIlscriptsType::~InternalIlscriptsType() = default; 
InternalIlscriptsType::InternalIlscriptsType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        maxtermoccurrences = ConfigParser::parse<int32_t>("maxtermoccurrences", __lines, 10000);
        ConfigParser::stripLinesForKey("maxtermoccurrences", __remainingValuesToParse);
        maxtokenlength = ConfigParser::parse<int32_t>("maxtokenlength", __lines, 1000);
        ConfigParser::stripLinesForKey("maxtokenlength", __remainingValuesToParse);
        fieldmatchmaxlength = ConfigParser::parse<int32_t>("fieldmatchmaxlength", __lines, 1000000);
        ConfigParser::stripLinesForKey("fieldmatchmaxlength", __remainingValuesToParse);
        ilscript = ConfigParser::parseArray<IlscriptVector>("ilscript", __lines);
        ConfigParser::stripLinesForKey("ilscript", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalIlscriptsType::operator==(const InternalIlscriptsType& __rhs) const noexcept
{
    return (maxtermoccurrences == __rhs.maxtermoccurrences &&
            maxtokenlength == __rhs.maxtokenlength &&
            fieldmatchmaxlength == __rhs.fieldmatchmaxlength &&
            ilscript == __rhs.ilscript);
}

bool
InternalIlscriptsType::operator!=(const InternalIlscriptsType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalIlscriptsType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxtermoccurrences");
        __c.setString("type", "int");
        __c.setLong("value", maxtermoccurrences);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxtokenlength");
        __c.setString("type", "int");
        __c.setLong("value", maxtokenlength);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fieldmatchmaxlength");
        __c.setString("type", "int");
        __c.setLong("value", fieldmatchmaxlength);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ilscript");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : ilscript) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalIlscriptsType::InternalIlscriptsType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    maxtermoccurrences = __inspector["maxtermoccurrences"]["value"].asLong();
    maxtokenlength = __inspector["maxtokenlength"]["value"].asLong();
    fieldmatchmaxlength = __inspector["fieldmatchmaxlength"]["value"].asLong();
    for (size_t __i = 0; __i < __inspector["ilscript"]["value"].children(); __i++) {
        ilscript.push_back(Ilscript(__inspector["ilscript"]["value"][__i]["value"]));
    }
}

InternalIlscriptsType::InternalIlscriptsType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    maxtermoccurrences = ::config::internal::ValueConverter<int32_t>()(__inspector["maxtermoccurrences"], 10000);
    maxtokenlength = ::config::internal::ValueConverter<int32_t>()(__inspector["maxtokenlength"], 1000);
    fieldmatchmaxlength = ::config::internal::ValueConverter<int32_t>()(__inspector["fieldmatchmaxlength"], 1000000);
    ::config::internal::VectorInserter<IlscriptVector> __ilscriptInserter(ilscript);
    __inspector["ilscript"].traverse(__ilscriptInserter);
}

} // namespace internal

} // namespace vespa::configdefinition
