#include "config-imported-fields.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalImportedFieldsType::CONFIG_DEF_MD5("baa648b23e7f979b8c809e5a5f2cd42b");
const vespalib::string InternalImportedFieldsType::CONFIG_DEF_NAME("imported-fields");
const vespalib::string InternalImportedFieldsType::CONFIG_DEF_NAMESPACE("vespa.config.search");
const int64_t InternalImportedFieldsType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search",
"attribute[].name string",
"attribute[].referencefield string",
"attribute[].targetfield string",
};
const ::config::StringVector InternalImportedFieldsType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalImportedFieldsType::Attribute::Attribute() noexcept
    : name(),
      referencefield(),
      targetfield()
{
}

InternalImportedFieldsType::Attribute::Attribute(const Attribute & __rhs) noexcept = default;
InternalImportedFieldsType::Attribute & InternalImportedFieldsType::Attribute::operator = (const Attribute & __rhs) noexcept = default;
InternalImportedFieldsType::Attribute::Attribute(Attribute && __rhs) noexcept = default;
InternalImportedFieldsType::Attribute & InternalImportedFieldsType::Attribute::operator = (Attribute && __rhs) noexcept = default;
InternalImportedFieldsType::Attribute::~Attribute() = default; 
InternalImportedFieldsType::Attribute::Attribute(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    referencefield = ConfigParser::parse<vespalib::string>("referencefield", __lines);
    ConfigParser::stripLinesForKey("referencefield", __remainingValuesToParse);
    targetfield = ConfigParser::parse<vespalib::string>("targetfield", __lines);
    ConfigParser::stripLinesForKey("targetfield", __remainingValuesToParse);
}

bool
InternalImportedFieldsType::Attribute::operator==(const Attribute& __rhs) const noexcept
{
    return (name == __rhs.name &&
            referencefield == __rhs.referencefield &&
            targetfield == __rhs.targetfield);
}

bool
InternalImportedFieldsType::Attribute::operator!=(const Attribute& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalImportedFieldsType::Attribute::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("referencefield");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(referencefield));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("targetfield");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(targetfield));
    }
}

InternalImportedFieldsType::Attribute::Attribute(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    referencefield = __inspector["referencefield"]["value"].asString().make_string();
    targetfield = __inspector["targetfield"]["value"].asString().make_string();
}

InternalImportedFieldsType::Attribute::Attribute(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    referencefield = ::config::internal::ValueConverter<vespalib::string>()("referencefield", __inspector["referencefield"]);
    targetfield = ::config::internal::ValueConverter<vespalib::string>()("targetfield", __inspector["targetfield"]);
}

InternalImportedFieldsType::InternalImportedFieldsType()
    : attribute()
{
}

InternalImportedFieldsType::InternalImportedFieldsType(const InternalImportedFieldsType & __rhs) = default;
InternalImportedFieldsType & InternalImportedFieldsType::operator =(const InternalImportedFieldsType & __rhs) = default;
InternalImportedFieldsType::InternalImportedFieldsType(InternalImportedFieldsType && __rhs) noexcept = default;
InternalImportedFieldsType & InternalImportedFieldsType::operator =(InternalImportedFieldsType && __rhs) noexcept = default;
InternalImportedFieldsType::~InternalImportedFieldsType() = default; 
InternalImportedFieldsType::InternalImportedFieldsType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        attribute = ConfigParser::parseArray<AttributeVector>("attribute", __lines);
        ConfigParser::stripLinesForKey("attribute", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalImportedFieldsType::
operator==(const InternalImportedFieldsType& __rhs) const noexcept
{
    return (attribute == __rhs.attribute);
}

bool
InternalImportedFieldsType::
operator!=(const InternalImportedFieldsType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalImportedFieldsType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("attribute");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : attribute) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalImportedFieldsType::InternalImportedFieldsType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["attribute"]["value"].children(); __i++) {
        attribute.push_back(Attribute(__inspector["attribute"]["value"][__i]["value"]));
    }
}

InternalImportedFieldsType::InternalImportedFieldsType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<AttributeVector> __attributeInserter(attribute);
    __inspector["attribute"].traverse(__attributeInserter);
}

} // namespace internal

} // namespace vespa::config::search
