/**
 * @class vespa::config::search::ImportedFields
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_SEARCH_IMPORTEDFIELDS_H
#define CLOUD_CONFIG_VESPA_CONFIG_SEARCH_IMPORTEDFIELDS_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::search {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalImportedFieldsType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalImportedFieldsType> UP;
    class Attribute {
    public:
        Attribute(const ::config::StringVector & __lines);
        Attribute(const vespalib::slime::Inspector & __inspector);
        Attribute(const ::config::ConfigPayload & __payload);
        Attribute()  noexcept;
        Attribute(const Attribute & __rhs) noexcept;
        Attribute & operator = (const Attribute & __rhs) noexcept;
        Attribute(Attribute && __rhs) noexcept;
        Attribute & operator = (Attribute && __rhs) noexcept;
        ~Attribute();

        bool operator==(const Attribute& __rhs) const noexcept;
        bool operator!=(const Attribute& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * The name of an imported attribute field in context of this document
         * type.
         */
        vespalib::string name;

        /**
         * The name of the field (in this document type) that refers to a parent
         * document type.
         */
        vespalib::string referencefield;

        /**
         * The name of the target attribute field in the parent document type
         * that is imported into this document type.
         */
        vespalib::string targetfield;
    };
    typedef std::vector<Attribute> AttributeVector;

    typedef std::map<vespalib::string, Attribute> AttributeMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalImportedFieldsType(const ::config::ConfigValue & __value);
    InternalImportedFieldsType(const ::config::ConfigDataBuffer & __value);
    InternalImportedFieldsType(const ::config::ConfigPayload & __payload);
    InternalImportedFieldsType() ;
    InternalImportedFieldsType(const InternalImportedFieldsType & __rhs);
    InternalImportedFieldsType & operator = (const InternalImportedFieldsType & __rhs);
    InternalImportedFieldsType(InternalImportedFieldsType && __rhs) noexcept;
    InternalImportedFieldsType & operator = (InternalImportedFieldsType && __rhs) noexcept;
    ~InternalImportedFieldsType();

    bool operator==(const InternalImportedFieldsType& __rhs) const noexcept;
    bool operator!=(const InternalImportedFieldsType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    AttributeVector attribute;
};

} // namespace internal

typedef internal::InternalImportedFieldsType ImportedFieldsConfigBuilder;
typedef const internal::InternalImportedFieldsType ImportedFieldsConfig;

} // namespace vespa::config::search
#endif // VESPA_config_VESPA_CONFIG_SEARCH_IMPORTEDFIELDS_H
