#include "config-indexschema.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalIndexschemaType::CONFIG_DEF_MD5("3f4a875b74c4ef2e4ffe929c7172fdeb");
const vespalib::string InternalIndexschemaType::CONFIG_DEF_NAME("indexschema");
const vespalib::string InternalIndexschemaType::CONFIG_DEF_NAMESPACE("vespa.config.search");
const int64_t InternalIndexschemaType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search",
"indexfield[].name string",
"indexfield[].datatype enum { STRING, INT64 } default=STRING",
"indexfield[].collectiontype enum { SINGLE, ARRAY, WEIGHTEDSET } default=SINGLE",
"indexfield[].prefix bool default=false",
"indexfield[].phrases bool default=false",
"indexfield[].positions bool default=true",
"indexfield[].averageelementlen int default=512",
"indexfield[].interleavedfeatures bool default=false",
"fieldset[].name string",
"fieldset[].field[].name string",
};
const ::config::StringVector InternalIndexschemaType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalIndexschemaType::Indexfield::Datatype
InternalIndexschemaType::Indexfield::getDatatype(const vespalib::string& name)
{
    if (name == "STRING") {
        return Datatype::STRING;
    } else if (name == "INT64") {
        return Datatype::INT64;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalIndexschemaType::Indexfield::getDatatypeName(Datatype t)
{
    switch (t) {
        case Datatype::STRING: return "STRING";
        case Datatype::INT64: return "INT64";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalIndexschemaType::Indexfield::Datatype InternalIndexschemaType::Indexfield::InternalDatatypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalIndexschemaType::Indexfield::getDatatype(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalIndexschemaType::Indexfield::Datatype InternalIndexschemaType::Indexfield::InternalDatatypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalIndexschemaType::Indexfield::getDatatype(__inspector.asString().make_string());
}
InternalIndexschemaType::Indexfield::Datatype InternalIndexschemaType::Indexfield::InternalDatatypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Datatype __eDefault) {
    if (__inspector.valid()) {
        return InternalIndexschemaType::Indexfield::getDatatype(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalIndexschemaType::Indexfield::Collectiontype
InternalIndexschemaType::Indexfield::getCollectiontype(const vespalib::string& name)
{
    if (name == "SINGLE") {
        return Collectiontype::SINGLE;
    } else if (name == "ARRAY") {
        return Collectiontype::ARRAY;
    } else if (name == "WEIGHTEDSET") {
        return Collectiontype::WEIGHTEDSET;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalIndexschemaType::Indexfield::getCollectiontypeName(Collectiontype t)
{
    switch (t) {
        case Collectiontype::SINGLE: return "SINGLE";
        case Collectiontype::ARRAY: return "ARRAY";
        case Collectiontype::WEIGHTEDSET: return "WEIGHTEDSET";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalIndexschemaType::Indexfield::Collectiontype InternalIndexschemaType::Indexfield::InternalCollectiontypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalIndexschemaType::Indexfield::getCollectiontype(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalIndexschemaType::Indexfield::Collectiontype InternalIndexschemaType::Indexfield::InternalCollectiontypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalIndexschemaType::Indexfield::getCollectiontype(__inspector.asString().make_string());
}
InternalIndexschemaType::Indexfield::Collectiontype InternalIndexschemaType::Indexfield::InternalCollectiontypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Collectiontype __eDefault) {
    if (__inspector.valid()) {
        return InternalIndexschemaType::Indexfield::getCollectiontype(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalIndexschemaType::Indexfield::Indexfield() noexcept
    : name(),
      datatype(Datatype::STRING),
      collectiontype(Collectiontype::SINGLE),
      prefix(false),
      phrases(false),
      positions(true),
      averageelementlen(512),
      interleavedfeatures(false)
{
}

InternalIndexschemaType::Indexfield::Indexfield(const Indexfield & __rhs) noexcept = default;
InternalIndexschemaType::Indexfield & InternalIndexschemaType::Indexfield::operator = (const Indexfield & __rhs) noexcept = default;
InternalIndexschemaType::Indexfield::Indexfield(Indexfield && __rhs) noexcept = default;
InternalIndexschemaType::Indexfield & InternalIndexschemaType::Indexfield::operator = (Indexfield && __rhs) noexcept = default;
InternalIndexschemaType::Indexfield::~Indexfield() = default; 
InternalIndexschemaType::Indexfield::Indexfield(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    datatype = getDatatype(ConfigParser::parse<vespalib::string>("datatype", __lines, "STRING"));
    ConfigParser::stripLinesForKey("datatype", __remainingValuesToParse);
    collectiontype = getCollectiontype(ConfigParser::parse<vespalib::string>("collectiontype", __lines, "SINGLE"));
    ConfigParser::stripLinesForKey("collectiontype", __remainingValuesToParse);
    prefix = ConfigParser::parse<bool>("prefix", __lines, false);
    ConfigParser::stripLinesForKey("prefix", __remainingValuesToParse);
    phrases = ConfigParser::parse<bool>("phrases", __lines, false);
    ConfigParser::stripLinesForKey("phrases", __remainingValuesToParse);
    positions = ConfigParser::parse<bool>("positions", __lines, true);
    ConfigParser::stripLinesForKey("positions", __remainingValuesToParse);
    averageelementlen = ConfigParser::parse<int32_t>("averageelementlen", __lines, 512);
    ConfigParser::stripLinesForKey("averageelementlen", __remainingValuesToParse);
    interleavedfeatures = ConfigParser::parse<bool>("interleavedfeatures", __lines, false);
    ConfigParser::stripLinesForKey("interleavedfeatures", __remainingValuesToParse);
}

bool
InternalIndexschemaType::Indexfield::operator==(const Indexfield& __rhs) const noexcept
{
    return (name == __rhs.name &&
            datatype == __rhs.datatype &&
            collectiontype == __rhs.collectiontype &&
            prefix == __rhs.prefix &&
            phrases == __rhs.phrases &&
            positions == __rhs.positions &&
            averageelementlen == __rhs.averageelementlen &&
            interleavedfeatures == __rhs.interleavedfeatures);
}

bool
InternalIndexschemaType::Indexfield::operator!=(const Indexfield& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalIndexschemaType::Indexfield::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("datatype");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getDatatypeName(datatype)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("collectiontype");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getCollectiontypeName(collectiontype)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("prefix");
        __c.setString("type", "bool");
        __c.setBool("value", prefix);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("phrases");
        __c.setString("type", "bool");
        __c.setBool("value", phrases);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("positions");
        __c.setString("type", "bool");
        __c.setBool("value", positions);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("averageelementlen");
        __c.setString("type", "int");
        __c.setLong("value", averageelementlen);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("interleavedfeatures");
        __c.setString("type", "bool");
        __c.setBool("value", interleavedfeatures);
    }
}

InternalIndexschemaType::Indexfield::Indexfield(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    datatype = getDatatype(__inspector["datatype"]["value"].asString().make_string());
    collectiontype = getCollectiontype(__inspector["collectiontype"]["value"].asString().make_string());
    prefix = __inspector["prefix"]["value"].asBool();
    phrases = __inspector["phrases"]["value"].asBool();
    positions = __inspector["positions"]["value"].asBool();
    averageelementlen = __inspector["averageelementlen"]["value"].asLong();
    interleavedfeatures = __inspector["interleavedfeatures"]["value"].asBool();
}

InternalIndexschemaType::Indexfield::Indexfield(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    datatype = InternalDatatypeConverter()(__inspector["datatype"], Datatype::STRING);
    collectiontype = InternalCollectiontypeConverter()(__inspector["collectiontype"], Collectiontype::SINGLE);
    prefix = ::config::internal::ValueConverter<bool>()(__inspector["prefix"], false);
    phrases = ::config::internal::ValueConverter<bool>()(__inspector["phrases"], false);
    positions = ::config::internal::ValueConverter<bool>()(__inspector["positions"], true);
    averageelementlen = ::config::internal::ValueConverter<int32_t>()(__inspector["averageelementlen"], 512);
    interleavedfeatures = ::config::internal::ValueConverter<bool>()(__inspector["interleavedfeatures"], false);
}

InternalIndexschemaType::Fieldset::Field::Field() noexcept
    : name()
{
}

InternalIndexschemaType::Fieldset::Field::Field(const Field & __rhs) noexcept = default;
InternalIndexschemaType::Fieldset::Field & InternalIndexschemaType::Fieldset::Field::operator = (const Field & __rhs) noexcept = default;
InternalIndexschemaType::Fieldset::Field::Field(Field && __rhs) noexcept = default;
InternalIndexschemaType::Fieldset::Field & InternalIndexschemaType::Fieldset::Field::operator = (Field && __rhs) noexcept = default;
InternalIndexschemaType::Fieldset::Field::~Field() = default; 
InternalIndexschemaType::Fieldset::Field::Field(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
}

bool
InternalIndexschemaType::Fieldset::Field::operator==(const Field& __rhs) const noexcept
{
    return (name == __rhs.name);
}

bool
InternalIndexschemaType::Fieldset::Field::operator!=(const Field& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalIndexschemaType::Fieldset::Field::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
}

InternalIndexschemaType::Fieldset::Field::Field(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
}

InternalIndexschemaType::Fieldset::Field::Field(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
}

InternalIndexschemaType::Fieldset::Fieldset()
    : name(),
      field()
{
}

InternalIndexschemaType::Fieldset::Fieldset(const Fieldset & __rhs) = default;
InternalIndexschemaType::Fieldset & InternalIndexschemaType::Fieldset::operator = (const Fieldset & __rhs) = default;
InternalIndexschemaType::Fieldset::Fieldset(Fieldset && __rhs) noexcept = default;
InternalIndexschemaType::Fieldset & InternalIndexschemaType::Fieldset::operator = (Fieldset && __rhs) noexcept = default;
InternalIndexschemaType::Fieldset::~Fieldset() = default; 
InternalIndexschemaType::Fieldset::Fieldset(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    field = ConfigParser::parseArray<FieldVector>("field", __lines);
    ConfigParser::stripLinesForKey("field", __remainingValuesToParse);
}

bool
InternalIndexschemaType::Fieldset::operator==(const Fieldset& __rhs) const noexcept
{
    return (name == __rhs.name &&
            field == __rhs.field);
}

bool
InternalIndexschemaType::Fieldset::operator!=(const Fieldset& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalIndexschemaType::Fieldset::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("field");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : field) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalIndexschemaType::Fieldset::Fieldset(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["field"]["value"].children(); __i++) {
        field.push_back(Field(__inspector["field"]["value"][__i]["value"]));
    }
}

InternalIndexschemaType::Fieldset::Fieldset(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    ::config::internal::VectorInserter<FieldVector> __fieldInserter(field);
    __inspector["field"].traverse(__fieldInserter);
}

InternalIndexschemaType::InternalIndexschemaType()
    : indexfield(),
      fieldset()
{
}

InternalIndexschemaType::InternalIndexschemaType(const InternalIndexschemaType & __rhs) = default;
InternalIndexschemaType & InternalIndexschemaType::operator =(const InternalIndexschemaType & __rhs) = default;
InternalIndexschemaType::InternalIndexschemaType(InternalIndexschemaType && __rhs) noexcept = default;
InternalIndexschemaType & InternalIndexschemaType::operator =(InternalIndexschemaType && __rhs) noexcept = default;
InternalIndexschemaType::~InternalIndexschemaType() = default; 
InternalIndexschemaType::InternalIndexschemaType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        indexfield = ConfigParser::parseArray<IndexfieldVector>("indexfield", __lines);
        ConfigParser::stripLinesForKey("indexfield", __remainingValuesToParse);
        fieldset = ConfigParser::parseArray<FieldsetVector>("fieldset", __lines);
        ConfigParser::stripLinesForKey("fieldset", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalIndexschemaType::operator==(const InternalIndexschemaType& __rhs) const noexcept
{
    return (indexfield == __rhs.indexfield &&
            fieldset == __rhs.fieldset);
}

bool
InternalIndexschemaType::operator!=(const InternalIndexschemaType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalIndexschemaType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("indexfield");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : indexfield) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fieldset");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : fieldset) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalIndexschemaType::InternalIndexschemaType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["indexfield"]["value"].children(); __i++) {
        indexfield.push_back(Indexfield(__inspector["indexfield"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["fieldset"]["value"].children(); __i++) {
        fieldset.push_back(Fieldset(__inspector["fieldset"]["value"][__i]["value"]));
    }
}

InternalIndexschemaType::InternalIndexschemaType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<IndexfieldVector> __indexfieldInserter(indexfield);
    __inspector["indexfield"].traverse(__indexfieldInserter);
    ::config::internal::VectorInserter<FieldsetVector> __fieldsetInserter(fieldset);
    __inspector["fieldset"].traverse(__fieldsetInserter);
}

} // namespace internal

} // namespace vespa::config::search
