/**
 * @class vespa::config::search::Indexschema
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 * Config specifying the index fields and field collections that are part of an index schema.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_SEARCH_INDEXSCHEMA_H
#define CLOUD_CONFIG_VESPA_CONFIG_SEARCH_INDEXSCHEMA_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::search {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalIndexschemaType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalIndexschemaType> UP;
    class Indexfield {
    public:
        enum class Datatype { STRING, INT64 };
        typedef std::vector<Datatype> DatatypeVector;
        typedef std::map<vespalib::string, Datatype> DatatypeMap;
        static Datatype getDatatype(const vespalib::string&);
        static vespalib::string getDatatypeName(Datatype e);

        struct InternalDatatypeConverter {
            Datatype operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Datatype operator()(const ::vespalib::slime::Inspector & __inspector);
            Datatype operator()(const ::vespalib::slime::Inspector & __inspector, Datatype __eDefault);
        };
        enum class Collectiontype { SINGLE, ARRAY, WEIGHTEDSET };
        typedef std::vector<Collectiontype> CollectiontypeVector;
        typedef std::map<vespalib::string, Collectiontype> CollectiontypeMap;
        static Collectiontype getCollectiontype(const vespalib::string&);
        static vespalib::string getCollectiontypeName(Collectiontype e);

        struct InternalCollectiontypeConverter {
            Collectiontype operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Collectiontype operator()(const ::vespalib::slime::Inspector & __inspector);
            Collectiontype operator()(const ::vespalib::slime::Inspector & __inspector, Collectiontype __eDefault);
        };
        Indexfield(const ::config::StringVector & __lines);
        Indexfield(const vespalib::slime::Inspector & __inspector);
        Indexfield(const ::config::ConfigPayload & __payload);
        Indexfield()  noexcept;
        Indexfield(const Indexfield & __rhs) noexcept;
        Indexfield & operator = (const Indexfield & __rhs) noexcept;
        Indexfield(Indexfield && __rhs) noexcept;
        Indexfield & operator = (Indexfield && __rhs) noexcept;
        ~Indexfield();

        bool operator==(const Indexfield& __rhs) const noexcept;
        bool operator!=(const Indexfield& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** The name of the index field. */
        vespalib::string name;

        /** The data type of the index field. */
        Datatype datatype; // Default: STRING

        /** The collection type of the index field. */
        Collectiontype collectiontype; // Default: SINGLE

        /** Whether the index should support prefix searches. */
        bool prefix; // Default: false

        /** Whether the index should support phrase searches. */
        bool phrases; // Default: false

        /** Whether the index should have posting lists with word positions. */
        bool positions; // Default: true

        /** Average element length */
        int32_t averageelementlen; // Default: 512

        /**
         * Whether the index field should use posting lists with interleaved
         * features or not.
         */
        bool interleavedfeatures; // Default: false
    };
    typedef std::vector<Indexfield> IndexfieldVector;

    typedef std::map<vespalib::string, Indexfield> IndexfieldMap;

    class Fieldset {
    public:
        class Field {
        public:
            Field(const ::config::StringVector & __lines);
            Field(const vespalib::slime::Inspector & __inspector);
            Field(const ::config::ConfigPayload & __payload);
            Field()  noexcept;
            Field(const Field & __rhs) noexcept;
            Field & operator = (const Field & __rhs) noexcept;
            Field(Field && __rhs) noexcept;
            Field & operator = (Field && __rhs) noexcept;
            ~Field();

            bool operator==(const Field& __rhs) const noexcept;
            bool operator!=(const Field& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** The name of the index field that are part of this collection. */
            vespalib::string name;
        };
        typedef std::vector<Field> FieldVector;

        typedef std::map<vespalib::string, Field> FieldMap;

        Fieldset(const ::config::StringVector & __lines);
        Fieldset(const vespalib::slime::Inspector & __inspector);
        Fieldset(const ::config::ConfigPayload & __payload);
        Fieldset() ;
        Fieldset(const Fieldset & __rhs);
        Fieldset & operator = (const Fieldset & __rhs);
        Fieldset(Fieldset && __rhs) noexcept;
        Fieldset & operator = (Fieldset && __rhs) noexcept;
        ~Fieldset();

        bool operator==(const Fieldset& __rhs) const noexcept;
        bool operator!=(const Fieldset& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** The name of the field collection (aka logical view). */
        vespalib::string name;
        FieldVector field;
    };
    typedef std::vector<Fieldset> FieldsetVector;

    typedef std::map<vespalib::string, Fieldset> FieldsetMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalIndexschemaType(const ::config::ConfigValue & __value);
    InternalIndexschemaType(const ::config::ConfigDataBuffer & __value);
    InternalIndexschemaType(const ::config::ConfigPayload & __payload);
    InternalIndexschemaType() ;
    InternalIndexschemaType(const InternalIndexschemaType & __rhs);
    InternalIndexschemaType & operator = (const InternalIndexschemaType & __rhs);
    InternalIndexschemaType(InternalIndexschemaType && __rhs) noexcept;
    InternalIndexschemaType & operator = (InternalIndexschemaType && __rhs) noexcept;
    ~InternalIndexschemaType();

    bool operator==(const InternalIndexschemaType& __rhs) const noexcept;
    bool operator!=(const InternalIndexschemaType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    IndexfieldVector indexfield;
    FieldsetVector fieldset;
};

} // namespace internal

typedef internal::InternalIndexschemaType IndexschemaConfigBuilder;
typedef const internal::InternalIndexschemaType IndexschemaConfig;

} // namespace vespa::config::search
#endif // VESPA_config_VESPA_CONFIG_SEARCH_INDEXSCHEMA_H
