#include "config-lb-services.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace cloud::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalLbServicesType::CONFIG_DEF_MD5("71d168615a8aa43c5ca2bbc6526adb7c");
const vespalib::string InternalLbServicesType::CONFIG_DEF_NAME("lb-services");
const vespalib::string InternalLbServicesType::CONFIG_DEF_NAMESPACE("cloud.config");
const int64_t InternalLbServicesType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=cloud.config",
"tenants{}.applications{}.activeRotation bool default=true",
"tenants{}.applications{}.endpoints[].dnsName string",
"tenants{}.applications{}.endpoints[].clusterId string",
"tenants{}.applications{}.endpoints[].scope enum {application, global, zone}",
"tenants{}.applications{}.endpoints[].routingMethod enum {shared, sharedLayer4}",
"tenants{}.applications{}.endpoints[].weight int default=1",
"tenants{}.applications{}.endpoints[].hosts[] string",
};
const ::config::StringVector InternalLbServicesType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalLbServicesType::Tenants::Applications::Endpoints::Scope
InternalLbServicesType::Tenants::Applications::Endpoints::getScope(const vespalib::string& name)
{
    if (name == "application") {
        return Scope::application;
    } else if (name == "global") {
        return Scope::global;
    } else if (name == "zone") {
        return Scope::zone;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalLbServicesType::Tenants::Applications::Endpoints::getScopeName(Scope t)
{
    switch (t) {
        case Scope::application: return "application";
        case Scope::global: return "global";
        case Scope::zone: return "zone";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalLbServicesType::Tenants::Applications::Endpoints::Scope InternalLbServicesType::Tenants::Applications::Endpoints::InternalScopeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalLbServicesType::Tenants::Applications::Endpoints::getScope(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalLbServicesType::Tenants::Applications::Endpoints::Scope InternalLbServicesType::Tenants::Applications::Endpoints::InternalScopeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalLbServicesType::Tenants::Applications::Endpoints::getScope(__inspector.asString().make_string());
}
InternalLbServicesType::Tenants::Applications::Endpoints::Scope InternalLbServicesType::Tenants::Applications::Endpoints::InternalScopeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Scope __eDefault) {
    if (__inspector.valid()) {
        return InternalLbServicesType::Tenants::Applications::Endpoints::getScope(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalLbServicesType::Tenants::Applications::Endpoints::RoutingMethod
InternalLbServicesType::Tenants::Applications::Endpoints::getRoutingMethod(const vespalib::string& name)
{
    if (name == "shared") {
        return RoutingMethod::shared;
    } else if (name == "sharedLayer4") {
        return RoutingMethod::sharedLayer4;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalLbServicesType::Tenants::Applications::Endpoints::getRoutingMethodName(RoutingMethod t)
{
    switch (t) {
        case RoutingMethod::shared: return "shared";
        case RoutingMethod::sharedLayer4: return "sharedLayer4";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalLbServicesType::Tenants::Applications::Endpoints::RoutingMethod InternalLbServicesType::Tenants::Applications::Endpoints::InternalRoutingMethodConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalLbServicesType::Tenants::Applications::Endpoints::getRoutingMethod(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalLbServicesType::Tenants::Applications::Endpoints::RoutingMethod InternalLbServicesType::Tenants::Applications::Endpoints::InternalRoutingMethodConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalLbServicesType::Tenants::Applications::Endpoints::getRoutingMethod(__inspector.asString().make_string());
}
InternalLbServicesType::Tenants::Applications::Endpoints::RoutingMethod InternalLbServicesType::Tenants::Applications::Endpoints::InternalRoutingMethodConverter::operator()(const ::vespalib::slime::Inspector & __inspector, RoutingMethod __eDefault) {
    if (__inspector.valid()) {
        return InternalLbServicesType::Tenants::Applications::Endpoints::getRoutingMethod(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalLbServicesType::Tenants::Applications::Endpoints::Endpoints()
    : dnsName(),
      clusterId(),
      scope(Scope::application),
      routingMethod(RoutingMethod::shared),
      weight(1),
      hosts()
{
}

InternalLbServicesType::Tenants::Applications::Endpoints::Endpoints(const Endpoints & __rhs) = default;
InternalLbServicesType::Tenants::Applications::Endpoints & InternalLbServicesType::Tenants::Applications::Endpoints::operator = (const Endpoints & __rhs) = default;
InternalLbServicesType::Tenants::Applications::Endpoints::Endpoints(Endpoints && __rhs) noexcept = default;
InternalLbServicesType::Tenants::Applications::Endpoints & InternalLbServicesType::Tenants::Applications::Endpoints::operator = (Endpoints && __rhs) noexcept = default;
InternalLbServicesType::Tenants::Applications::Endpoints::~Endpoints() = default; 
InternalLbServicesType::Tenants::Applications::Endpoints::Endpoints(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    dnsName = ConfigParser::parse<vespalib::string>("dnsName", __lines);
    ConfigParser::stripLinesForKey("dnsName", __remainingValuesToParse);
    clusterId = ConfigParser::parse<vespalib::string>("clusterId", __lines);
    ConfigParser::stripLinesForKey("clusterId", __remainingValuesToParse);
    scope = getScope(ConfigParser::parse<vespalib::string>("scope", __lines));
    ConfigParser::stripLinesForKey("scope", __remainingValuesToParse);
    routingMethod = getRoutingMethod(ConfigParser::parse<vespalib::string>("routingMethod", __lines));
    ConfigParser::stripLinesForKey("routingMethod", __remainingValuesToParse);
    weight = ConfigParser::parse<int32_t>("weight", __lines, 1);
    ConfigParser::stripLinesForKey("weight", __remainingValuesToParse);
    hosts = ConfigParser::parseArray<::config::StringVector>("hosts", __lines);
    ConfigParser::stripLinesForKey("hosts", __remainingValuesToParse);
}

bool
InternalLbServicesType::Tenants::Applications::Endpoints::
operator==(const Endpoints& __rhs) const noexcept
{
    return (dnsName == __rhs.dnsName &&
            clusterId == __rhs.clusterId &&
            scope == __rhs.scope &&
            routingMethod == __rhs.routingMethod &&
            weight == __rhs.weight &&
            hosts == __rhs.hosts);
}

bool
InternalLbServicesType::Tenants::Applications::Endpoints::
operator!=(const Endpoints& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLbServicesType::Tenants::Applications::Endpoints::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("dnsName");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(dnsName));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("clusterId");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(clusterId));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("scope");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getScopeName(scope)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("routingMethod");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getRoutingMethodName(routingMethod)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("weight");
        __c.setString("type", "int");
        __c.setLong("value", weight);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("hosts");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : hosts) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
}

InternalLbServicesType::Tenants::Applications::Endpoints::Endpoints(const vespalib::slime::Inspector & __inspector)
{
    dnsName = __inspector["dnsName"]["value"].asString().make_string();
    clusterId = __inspector["clusterId"]["value"].asString().make_string();
    scope = getScope(__inspector["scope"]["value"].asString().make_string());
    routingMethod = getRoutingMethod(__inspector["routingMethod"]["value"].asString().make_string());
    weight = __inspector["weight"]["value"].asLong();
    for (size_t __i = 0; __i < __inspector["hosts"]["value"].children(); __i++) {
        hosts.push_back(__inspector["hosts"]["value"][__i]["value"].asString().make_string());
    }
}

InternalLbServicesType::Tenants::Applications::Endpoints::Endpoints(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    dnsName = ::config::internal::ValueConverter<vespalib::string>()("dnsName", __inspector["dnsName"]);
    clusterId = ::config::internal::ValueConverter<vespalib::string>()("clusterId", __inspector["clusterId"]);
    scope = InternalScopeConverter()("scope", __inspector["scope"]);
    routingMethod = InternalRoutingMethodConverter()("routingMethod", __inspector["routingMethod"]);
    weight = ::config::internal::ValueConverter<int32_t>()(__inspector["weight"], 1);
    ::config::internal::VectorInserter<::config::StringVector> __hostsInserter(hosts);
    __inspector["hosts"].traverse(__hostsInserter);
}

InternalLbServicesType::Tenants::Applications::Applications()
    : activeRotation(true),
      endpoints()
{
}

InternalLbServicesType::Tenants::Applications::Applications(const Applications & __rhs) = default;
InternalLbServicesType::Tenants::Applications & InternalLbServicesType::Tenants::Applications::operator = (const Applications & __rhs) = default;
InternalLbServicesType::Tenants::Applications::Applications(Applications && __rhs) noexcept = default;
InternalLbServicesType::Tenants::Applications & InternalLbServicesType::Tenants::Applications::operator = (Applications && __rhs) noexcept = default;
InternalLbServicesType::Tenants::Applications::~Applications() = default; 
InternalLbServicesType::Tenants::Applications::Applications(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    activeRotation = ConfigParser::parse<bool>("activeRotation", __lines, true);
    ConfigParser::stripLinesForKey("activeRotation", __remainingValuesToParse);
    endpoints = ConfigParser::parseArray<EndpointsVector>("endpoints", __lines);
    ConfigParser::stripLinesForKey("endpoints", __remainingValuesToParse);
}

bool
InternalLbServicesType::Tenants::Applications::
operator==(const Applications& __rhs) const noexcept
{
    return (activeRotation == __rhs.activeRotation &&
            endpoints == __rhs.endpoints);
}

bool
InternalLbServicesType::Tenants::Applications::
operator!=(const Applications& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLbServicesType::Tenants::Applications::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("activeRotation");
        __c.setString("type", "bool");
        __c.setBool("value", activeRotation);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("endpoints");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : endpoints) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalLbServicesType::Tenants::Applications::Applications(const vespalib::slime::Inspector & __inspector)
{
    activeRotation = __inspector["activeRotation"]["value"].asBool();
    for (size_t __i = 0; __i < __inspector["endpoints"]["value"].children(); __i++) {
        endpoints.push_back(Endpoints(__inspector["endpoints"]["value"][__i]["value"]));
    }
}

InternalLbServicesType::Tenants::Applications::Applications(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    activeRotation = ::config::internal::ValueConverter<bool>()(__inspector["activeRotation"], true);
    ::config::internal::VectorInserter<EndpointsVector> __endpointsInserter(endpoints);
    __inspector["endpoints"].traverse(__endpointsInserter);
}

InternalLbServicesType::Tenants::Tenants()
    : applications()
{
}

InternalLbServicesType::Tenants::Tenants(const Tenants & __rhs) = default;
InternalLbServicesType::Tenants & InternalLbServicesType::Tenants::operator = (const Tenants & __rhs) = default;
InternalLbServicesType::Tenants::Tenants(Tenants && __rhs) noexcept = default;
InternalLbServicesType::Tenants & InternalLbServicesType::Tenants::operator = (Tenants && __rhs) noexcept = default;
InternalLbServicesType::Tenants::~Tenants() = default; 
InternalLbServicesType::Tenants::Tenants(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    applications = ConfigParser::parseMap<Applications>("applications", __lines);
    ConfigParser::stripLinesForKey("applications", __remainingValuesToParse);
}

bool
InternalLbServicesType::Tenants::operator==(const Tenants& __rhs) const noexcept
{
    return (applications == __rhs.applications);
}

bool
InternalLbServicesType::Tenants::operator!=(const Tenants& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLbServicesType::Tenants::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("applications");
        __c.setString("type", "map");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & entry : applications) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("key", vespalib::Memory(entry.first));
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            entry.second.serialize(__c4);
        }
    }
}

InternalLbServicesType::Tenants::Tenants(const vespalib::slime::Inspector & __inspector)
{
    for (size_t __i = 0; __i < __inspector["applications"]["value"].children(); __i++) {
        applications[__inspector["applications"]["value"][__i]["key"].asString().make_string()] = Applications(__inspector["applications"]["value"][__i]["value"]);
    }
}

InternalLbServicesType::Tenants::Tenants(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::MapInserter<Applications> __applicationsInserter(applications);
    __inspector["applications"].traverse(__applicationsInserter);
}

InternalLbServicesType::InternalLbServicesType()
    : tenants()
{
}

InternalLbServicesType::InternalLbServicesType(const InternalLbServicesType & __rhs) = default;
InternalLbServicesType & InternalLbServicesType::operator =(const InternalLbServicesType & __rhs) = default;
InternalLbServicesType::InternalLbServicesType(InternalLbServicesType && __rhs) noexcept = default;
InternalLbServicesType & InternalLbServicesType::operator =(InternalLbServicesType && __rhs) noexcept = default;
InternalLbServicesType::~InternalLbServicesType() = default; 
InternalLbServicesType::InternalLbServicesType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        tenants = ConfigParser::parseMap<Tenants>("tenants", __lines);
        ConfigParser::stripLinesForKey("tenants", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalLbServicesType::operator==(const InternalLbServicesType& __rhs) const noexcept
{
    return (tenants == __rhs.tenants);
}

bool
InternalLbServicesType::operator!=(const InternalLbServicesType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLbServicesType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tenants");
        __c.setString("type", "map");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & entry : tenants) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("key", vespalib::Memory(entry.first));
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            entry.second.serialize(__c4);
        }
    }
}

InternalLbServicesType::InternalLbServicesType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["tenants"]["value"].children(); __i++) {
        tenants[__inspector["tenants"]["value"][__i]["key"].asString().make_string()] = Tenants(__inspector["tenants"]["value"][__i]["value"]);
    }
}

InternalLbServicesType::InternalLbServicesType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::MapInserter<Tenants> __tenantsInserter(tenants);
    __inspector["tenants"].traverse(__tenantsInserter);
}

} // namespace internal

} // namespace cloud::config
