/**
 * @class cloud::config::LbServices
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 * Config for load balancer that needs to know about all services for
 * all tenants and applications in a config server cluster
 */
#ifndef CLOUD_CONFIG_CLOUD_CONFIG_LBSERVICES_H
#define CLOUD_CONFIG_CLOUD_CONFIG_LBSERVICES_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace cloud::config {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalLbServicesType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalLbServicesType> UP;
    class Tenants {
    public:
        class Applications {
        public:
            class Endpoints {
            public:
                enum class Scope { application, global, zone };
                typedef std::vector<Scope> ScopeVector;
                typedef std::map<vespalib::string, Scope> ScopeMap;
                static Scope getScope(const vespalib::string&);
                static vespalib::string getScopeName(Scope e);

                struct InternalScopeConverter {
                    Scope operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                    Scope operator()(const ::vespalib::slime::Inspector & __inspector);
                    Scope operator()(const ::vespalib::slime::Inspector & __inspector, Scope __eDefault);
                };
                enum class RoutingMethod { shared, sharedLayer4 };
                typedef std::vector<RoutingMethod> RoutingMethodVector;
                typedef std::map<vespalib::string, RoutingMethod> RoutingMethodMap;
                static RoutingMethod getRoutingMethod(const vespalib::string&);
                static vespalib::string getRoutingMethodName(RoutingMethod e);

                struct InternalRoutingMethodConverter {
                    RoutingMethod operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                    RoutingMethod operator()(const ::vespalib::slime::Inspector & __inspector);
                    RoutingMethod operator()(const ::vespalib::slime::Inspector & __inspector, RoutingMethod __eDefault);
                };
                Endpoints(const ::config::StringVector & __lines);
                Endpoints(const vespalib::slime::Inspector & __inspector);
                Endpoints(const ::config::ConfigPayload & __payload);
                Endpoints() ;
                Endpoints(const Endpoints & __rhs);
                Endpoints & operator = (const Endpoints & __rhs);
                Endpoints(Endpoints && __rhs) noexcept;
                Endpoints & operator = (Endpoints && __rhs) noexcept;
                ~Endpoints();

                bool operator==(const Endpoints& __rhs) const noexcept;
                bool operator!=(const Endpoints& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;
                vespalib::string dnsName;
                vespalib::string clusterId;
                Scope scope;
                RoutingMethod routingMethod;
                int32_t weight; // Default: 1
                ::config::StringVector hosts;
            };
            typedef std::vector<Endpoints> EndpointsVector;

            typedef std::map<vespalib::string, Endpoints> EndpointsMap;

            Applications(const ::config::StringVector & __lines);
            Applications(const vespalib::slime::Inspector & __inspector);
            Applications(const ::config::ConfigPayload & __payload);
            Applications() ;
            Applications(const Applications & __rhs);
            Applications & operator = (const Applications & __rhs);
            Applications(Applications && __rhs) noexcept;
            Applications & operator = (Applications && __rhs) noexcept;
            ~Applications();

            bool operator==(const Applications& __rhs) const noexcept;
            bool operator!=(const Applications& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * Active rotation given as flag 'active' for a prod region in
             * deployment.xml
             * Default true for now (since code in config-model to set it is not
             * ready yet), should have no default value
             */
            bool activeRotation; // Default: true
            EndpointsVector endpoints;
        };
        typedef std::vector<Applications> ApplicationsVector;

        typedef std::map<vespalib::string, Applications> ApplicationsMap;

        Tenants(const ::config::StringVector & __lines);
        Tenants(const vespalib::slime::Inspector & __inspector);
        Tenants(const ::config::ConfigPayload & __payload);
        Tenants() ;
        Tenants(const Tenants & __rhs);
        Tenants & operator = (const Tenants & __rhs);
        Tenants(Tenants && __rhs) noexcept;
        Tenants & operator = (Tenants && __rhs) noexcept;
        ~Tenants();

        bool operator==(const Tenants& __rhs) const noexcept;
        bool operator!=(const Tenants& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        ApplicationsMap applications;
    };
    typedef std::vector<Tenants> TenantsVector;

    typedef std::map<vespalib::string, Tenants> TenantsMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalLbServicesType(const ::config::ConfigValue & __value);
    InternalLbServicesType(const ::config::ConfigDataBuffer & __value);
    InternalLbServicesType(const ::config::ConfigPayload & __payload);
    InternalLbServicesType() ;
    InternalLbServicesType(const InternalLbServicesType & __rhs);
    InternalLbServicesType & operator = (const InternalLbServicesType & __rhs);
    InternalLbServicesType(InternalLbServicesType && __rhs) noexcept;
    InternalLbServicesType & operator = (InternalLbServicesType && __rhs) noexcept;
    ~InternalLbServicesType();

    bool operator==(const InternalLbServicesType& __rhs) const noexcept;
    bool operator!=(const InternalLbServicesType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    TenantsMap tenants;
};

} // namespace internal

typedef internal::InternalLbServicesType LbServicesConfigBuilder;
typedef const internal::InternalLbServicesType LbServicesConfig;

} // namespace cloud::config
#endif // VESPA_config_CLOUD_CONFIG_LBSERVICES_H
