#include "config-load-type.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalLoadTypeType::CONFIG_DEF_MD5("d05d64950b5e21ee32a19fd88de62dd6");
const vespalib::string InternalLoadTypeType::CONFIG_DEF_NAME("load-type");
const vespalib::string InternalLoadTypeType::CONFIG_DEF_NAMESPACE("vespa.config.content");
const int64_t InternalLoadTypeType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content",
"type[].id int restart",
"type[].name string restart",
"type[].priority string restart",
};
const ::config::StringVector InternalLoadTypeType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalLoadTypeType::Type::Type() noexcept
    : id(0),
      name(),
      priority()
{
}

InternalLoadTypeType::Type::Type(const Type & __rhs) noexcept = default;
InternalLoadTypeType::Type & InternalLoadTypeType::Type::operator = (const Type & __rhs) noexcept = default;
InternalLoadTypeType::Type::Type(Type && __rhs) noexcept = default;
InternalLoadTypeType::Type & InternalLoadTypeType::Type::operator = (Type && __rhs) noexcept = default;
InternalLoadTypeType::Type::~Type() = default; 
InternalLoadTypeType::Type::Type(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    id = ConfigParser::parse<int32_t>("id", __lines);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    priority = ConfigParser::parse<vespalib::string>("priority", __lines);
    ConfigParser::stripLinesForKey("priority", __remainingValuesToParse);
}

bool
InternalLoadTypeType::Type::operator==(const Type& __rhs) const noexcept
{
    return (id == __rhs.id &&
            name == __rhs.name &&
            priority == __rhs.priority);
}

bool
InternalLoadTypeType::Type::operator!=(const Type& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLoadTypeType::Type::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "int");
        __c.setLong("value", id);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("priority");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(priority));
    }
}

InternalLoadTypeType::Type::Type(const vespalib::slime::Inspector & __inspector)
{
    id = __inspector["id"]["value"].asLong();
    name = __inspector["name"]["value"].asString().make_string();
    priority = __inspector["priority"]["value"].asString().make_string();
}

InternalLoadTypeType::Type::Type(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    id = ::config::internal::ValueConverter<int32_t>()("id", __inspector["id"]);
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    priority = ::config::internal::ValueConverter<vespalib::string>()("priority", __inspector["priority"]);
}

InternalLoadTypeType::InternalLoadTypeType()
    : type()
{
}

InternalLoadTypeType::InternalLoadTypeType(const InternalLoadTypeType & __rhs) = default;
InternalLoadTypeType & InternalLoadTypeType::operator =(const InternalLoadTypeType & __rhs) = default;
InternalLoadTypeType::InternalLoadTypeType(InternalLoadTypeType && __rhs) noexcept = default;
InternalLoadTypeType & InternalLoadTypeType::operator =(InternalLoadTypeType && __rhs) noexcept = default;
InternalLoadTypeType::~InternalLoadTypeType() = default; 
InternalLoadTypeType::InternalLoadTypeType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        type = ConfigParser::parseArray<TypeVector>("type", __lines);
        ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalLoadTypeType::operator==(const InternalLoadTypeType& __rhs) const noexcept
{
    return (type == __rhs.type);
}

bool
InternalLoadTypeType::operator!=(const InternalLoadTypeType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLoadTypeType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : type) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalLoadTypeType::InternalLoadTypeType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["type"]["value"].children(); __i++) {
        type.push_back(Type(__inspector["type"]["value"][__i]["value"]));
    }
}

InternalLoadTypeType::InternalLoadTypeType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<TypeVector> __typeInserter(type);
    __inspector["type"].traverse(__typeInserter);
}

} // namespace internal

} // namespace vespa::config::content
