/**
 * @class vespa::config::content::LoadType
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 * This config is used to define load types in VESPA. Load types are merely
 * a way to distinguish types of load, as to present such types differently.
 * For instance, some metrics will be presented per load type, such that you
 * can see how that specific type is being handled. Load types are especially
 * useful for separating loads of different priority.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_CONTENT_LOADTYPE_H
#define CLOUD_CONFIG_VESPA_CONFIG_CONTENT_LOADTYPE_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::content {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalLoadTypeType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalLoadTypeType> UP;
    class Type {
    public:
        Type(const ::config::StringVector & __lines);
        Type(const vespalib::slime::Inspector & __inspector);
        Type(const ::config::ConfigPayload & __payload);
        Type()  noexcept;
        Type(const Type & __rhs) noexcept;
        Type & operator = (const Type & __rhs) noexcept;
        Type(Type && __rhs) noexcept;
        Type & operator = (Type && __rhs) noexcept;
        ~Type();

        bool operator==(const Type& __rhs) const noexcept;
        bool operator!=(const Type& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * The identifying index of this type. 0 cannot be used as it is
         * reserved for the default type. Other types should be
         * non-overlapping from 1 and upwards. No gaps are allowed in the
         * index range. (For performance reasons we want to use arrays
         * internally where the id is the index)
         *  
         * The identifier is used for efficient transfer of type information.
         * The types will be identified by names instead in more textual or
         * verbose interfaces.
         * restart flag was added automatically and needs to be verified.
         */
        int32_t id;

        /**
         * The name of a given load type. Each load type must have a unique
         * name. The name "default" is reserved for the default type of id 0.
         * restart flag was added automatically and needs to be verified.
         */
        vespalib::string name;

        /**
         * The default priority for the load type.
         * restart flag was added automatically and needs to be verified.
         */
        vespalib::string priority;
    };
    typedef std::vector<Type> TypeVector;

    typedef std::map<vespalib::string, Type> TypeMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalLoadTypeType(const ::config::ConfigValue & __value);
    InternalLoadTypeType(const ::config::ConfigDataBuffer & __value);
    InternalLoadTypeType(const ::config::ConfigPayload & __payload);
    InternalLoadTypeType() ;
    InternalLoadTypeType(const InternalLoadTypeType & __rhs);
    InternalLoadTypeType & operator = (const InternalLoadTypeType & __rhs);
    InternalLoadTypeType(InternalLoadTypeType && __rhs) noexcept;
    InternalLoadTypeType & operator = (InternalLoadTypeType && __rhs) noexcept;
    ~InternalLoadTypeType();

    bool operator==(const InternalLoadTypeType& __rhs) const noexcept;
    bool operator!=(const InternalLoadTypeType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    TypeVector type;
};

} // namespace internal

typedef internal::InternalLoadTypeType LoadTypeConfigBuilder;
typedef const internal::InternalLoadTypeType LoadTypeConfig;

} // namespace vespa::config::content
#endif // VESPA_config_VESPA_CONFIG_CONTENT_LOADTYPE_H
