#include "config-logforwarder.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace cloud::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalLogforwarderType::CONFIG_DEF_MD5("d5e5313aea917591fe88b9d777c75001");
const vespalib::string InternalLogforwarderType::CONFIG_DEF_NAME("logforwarder");
const vespalib::string InternalLogforwarderType::CONFIG_DEF_NAMESPACE("cloud.config");
const int64_t InternalLogforwarderType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=cloud.config",
"deploymentServer string default=\"\"",
"clientName string default=\"\"",
"splunkHome string default=\"/opt/splunkforwarder\"",
"phoneHomeInterval int default=60",
"role string default=\"\"",
};
const ::config::StringVector InternalLogforwarderType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalLogforwarderType::InternalLogforwarderType() noexcept
    : deploymentServer(""),
      clientName(""),
      splunkHome("/opt/splunkforwarder"),
      phoneHomeInterval(60),
      role("")
{
}

InternalLogforwarderType::InternalLogforwarderType(const InternalLogforwarderType & __rhs) noexcept = default;
InternalLogforwarderType & InternalLogforwarderType::operator =(const InternalLogforwarderType & __rhs) noexcept = default;
InternalLogforwarderType::InternalLogforwarderType(InternalLogforwarderType && __rhs) noexcept = default;
InternalLogforwarderType & InternalLogforwarderType::operator =(InternalLogforwarderType && __rhs) noexcept = default;
InternalLogforwarderType::~InternalLogforwarderType() = default; 
InternalLogforwarderType::InternalLogforwarderType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        deploymentServer = ConfigParser::parse<vespalib::string>("deploymentServer", __lines, "");
        ConfigParser::stripLinesForKey("deploymentServer", __remainingValuesToParse);
        clientName = ConfigParser::parse<vespalib::string>("clientName", __lines, "");
        ConfigParser::stripLinesForKey("clientName", __remainingValuesToParse);
        splunkHome = ConfigParser::parse<vespalib::string>("splunkHome", __lines, "/opt/splunkforwarder");
        ConfigParser::stripLinesForKey("splunkHome", __remainingValuesToParse);
        phoneHomeInterval = ConfigParser::parse<int32_t>("phoneHomeInterval", __lines, 60);
        ConfigParser::stripLinesForKey("phoneHomeInterval", __remainingValuesToParse);
        role = ConfigParser::parse<vespalib::string>("role", __lines, "");
        ConfigParser::stripLinesForKey("role", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalLogforwarderType::
operator==(const InternalLogforwarderType& __rhs) const noexcept
{
    return (deploymentServer == __rhs.deploymentServer &&
            clientName == __rhs.clientName &&
            splunkHome == __rhs.splunkHome &&
            phoneHomeInterval == __rhs.phoneHomeInterval &&
            role == __rhs.role);
}

bool
InternalLogforwarderType::
operator!=(const InternalLogforwarderType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLogforwarderType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("deploymentServer");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(deploymentServer));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("clientName");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(clientName));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("splunkHome");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(splunkHome));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("phoneHomeInterval");
        __c.setString("type", "int");
        __c.setLong("value", phoneHomeInterval);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("role");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(role));
    }
}

InternalLogforwarderType::InternalLogforwarderType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    deploymentServer = __inspector["deploymentServer"]["value"].asString().make_string();
    clientName = __inspector["clientName"]["value"].asString().make_string();
    splunkHome = __inspector["splunkHome"]["value"].asString().make_string();
    phoneHomeInterval = __inspector["phoneHomeInterval"]["value"].asLong();
    role = __inspector["role"]["value"].asString().make_string();
}

InternalLogforwarderType::InternalLogforwarderType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    deploymentServer = ::config::internal::ValueConverter<vespalib::string>()(__inspector["deploymentServer"], "");
    clientName = ::config::internal::ValueConverter<vespalib::string>()(__inspector["clientName"], "");
    splunkHome = ::config::internal::ValueConverter<vespalib::string>()(__inspector["splunkHome"], "/opt/splunkforwarder");
    phoneHomeInterval = ::config::internal::ValueConverter<int32_t>()(__inspector["phoneHomeInterval"], 60);
    role = ::config::internal::ValueConverter<vespalib::string>()(__inspector["role"], "");
}

} // namespace internal

} // namespace cloud::config
