#include "config-model.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace cloud::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalModelType::CONFIG_DEF_MD5("665f15b69fcddf6de75c32230ffdfc0e");
const vespalib::string InternalModelType::CONFIG_DEF_NAME("model");
const vespalib::string InternalModelType::CONFIG_DEF_NAMESPACE("cloud.config");
const int64_t InternalModelType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=cloud.config",
"vespaVersion string default=\"unknown\"",
"hosts[].name string default=\"(unknownhostname)\"",
"hosts[].services[].name string default=\"(noservicename)\"",
"hosts[].services[].type string default=\"(noservicetype)\"",
"hosts[].services[].configid string default=\"(noconfigid)\"",
"hosts[].services[].clustertype string default=\"(unknownclustertype)\"",
"hosts[].services[].clustername string default=\"(unknownclustername)\"",
"hosts[].services[].index int default=0",
"hosts[].services[].ports[].number int default=-1",
"hosts[].services[].ports[].tags string default=\"(notags)\"",
};
const ::config::StringVector InternalModelType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalModelType::Hosts::Services::Ports::Ports() noexcept
    : number(-1),
      tags("(notags)")
{
}

InternalModelType::Hosts::Services::Ports::Ports(const Ports & __rhs) noexcept = default;
InternalModelType::Hosts::Services::Ports & InternalModelType::Hosts::Services::Ports::operator = (const Ports & __rhs) noexcept = default;
InternalModelType::Hosts::Services::Ports::Ports(Ports && __rhs) noexcept = default;
InternalModelType::Hosts::Services::Ports & InternalModelType::Hosts::Services::Ports::operator = (Ports && __rhs) noexcept = default;
InternalModelType::Hosts::Services::Ports::~Ports() = default; 
InternalModelType::Hosts::Services::Ports::Ports(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    number = ConfigParser::parse<int32_t>("number", __lines, -1);
    ConfigParser::stripLinesForKey("number", __remainingValuesToParse);
    tags = ConfigParser::parse<vespalib::string>("tags", __lines, "(notags)");
    ConfigParser::stripLinesForKey("tags", __remainingValuesToParse);
}

bool
InternalModelType::Hosts::Services::Ports::operator==(const Ports& __rhs) const noexcept
{
    return (number == __rhs.number &&
            tags == __rhs.tags);
}

bool
InternalModelType::Hosts::Services::Ports::operator!=(const Ports& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalModelType::Hosts::Services::Ports::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("number");
        __c.setString("type", "int");
        __c.setLong("value", number);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tags");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(tags));
    }
}

InternalModelType::Hosts::Services::Ports::Ports(const vespalib::slime::Inspector & __inspector)
{
    number = __inspector["number"]["value"].asLong();
    tags = __inspector["tags"]["value"].asString().make_string();
}

InternalModelType::Hosts::Services::Ports::Ports(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    number = ::config::internal::ValueConverter<int32_t>()(__inspector["number"], -1);
    tags = ::config::internal::ValueConverter<vespalib::string>()(__inspector["tags"], "(notags)");
}

InternalModelType::Hosts::Services::Services()
    : name("(noservicename)"),
      type("(noservicetype)"),
      configid("(noconfigid)"),
      clustertype("(unknownclustertype)"),
      clustername("(unknownclustername)"),
      index(0),
      ports()
{
}

InternalModelType::Hosts::Services::Services(const Services & __rhs) = default;
InternalModelType::Hosts::Services & InternalModelType::Hosts::Services::operator = (const Services & __rhs) = default;
InternalModelType::Hosts::Services::Services(Services && __rhs) noexcept = default;
InternalModelType::Hosts::Services & InternalModelType::Hosts::Services::operator = (Services && __rhs) noexcept = default;
InternalModelType::Hosts::Services::~Services() = default; 
InternalModelType::Hosts::Services::Services(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines, "(noservicename)");
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    type = ConfigParser::parse<vespalib::string>("type", __lines, "(noservicetype)");
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
    configid = ConfigParser::parse<vespalib::string>("configid", __lines, "(noconfigid)");
    ConfigParser::stripLinesForKey("configid", __remainingValuesToParse);
    clustertype = ConfigParser::parse<vespalib::string>("clustertype", __lines, "(unknownclustertype)");
    ConfigParser::stripLinesForKey("clustertype", __remainingValuesToParse);
    clustername = ConfigParser::parse<vespalib::string>("clustername", __lines, "(unknownclustername)");
    ConfigParser::stripLinesForKey("clustername", __remainingValuesToParse);
    index = ConfigParser::parse<int32_t>("index", __lines, 0);
    ConfigParser::stripLinesForKey("index", __remainingValuesToParse);
    ports = ConfigParser::parseArray<PortsVector>("ports", __lines);
    ConfigParser::stripLinesForKey("ports", __remainingValuesToParse);
}

bool
InternalModelType::Hosts::Services::operator==(const Services& __rhs) const noexcept
{
    return (name == __rhs.name &&
            type == __rhs.type &&
            configid == __rhs.configid &&
            clustertype == __rhs.clustertype &&
            clustername == __rhs.clustername &&
            index == __rhs.index &&
            ports == __rhs.ports);
}

bool
InternalModelType::Hosts::Services::operator!=(const Services& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalModelType::Hosts::Services::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(type));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("configid");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(configid));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("clustertype");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(clustertype));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("clustername");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(clustername));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("index");
        __c.setString("type", "int");
        __c.setLong("value", index);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ports");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : ports) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalModelType::Hosts::Services::Services(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    type = __inspector["type"]["value"].asString().make_string();
    configid = __inspector["configid"]["value"].asString().make_string();
    clustertype = __inspector["clustertype"]["value"].asString().make_string();
    clustername = __inspector["clustername"]["value"].asString().make_string();
    index = __inspector["index"]["value"].asLong();
    for (size_t __i = 0; __i < __inspector["ports"]["value"].children(); __i++) {
        ports.push_back(Ports(__inspector["ports"]["value"][__i]["value"]));
    }
}

InternalModelType::Hosts::Services::Services(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()(__inspector["name"], "(noservicename)");
    type = ::config::internal::ValueConverter<vespalib::string>()(__inspector["type"], "(noservicetype)");
    configid = ::config::internal::ValueConverter<vespalib::string>()(__inspector["configid"], "(noconfigid)");
    clustertype = ::config::internal::ValueConverter<vespalib::string>()(__inspector["clustertype"], "(unknownclustertype)");
    clustername = ::config::internal::ValueConverter<vespalib::string>()(__inspector["clustername"], "(unknownclustername)");
    index = ::config::internal::ValueConverter<int32_t>()(__inspector["index"], 0);
    ::config::internal::VectorInserter<PortsVector> __portsInserter(ports);
    __inspector["ports"].traverse(__portsInserter);
}

InternalModelType::Hosts::Hosts()
    : name("(unknownhostname)"),
      services()
{
}

InternalModelType::Hosts::Hosts(const Hosts & __rhs) = default;
InternalModelType::Hosts & InternalModelType::Hosts::operator = (const Hosts & __rhs) = default;
InternalModelType::Hosts::Hosts(Hosts && __rhs) noexcept = default;
InternalModelType::Hosts & InternalModelType::Hosts::operator = (Hosts && __rhs) noexcept = default;
InternalModelType::Hosts::~Hosts() = default; 
InternalModelType::Hosts::Hosts(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines, "(unknownhostname)");
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    services = ConfigParser::parseArray<ServicesVector>("services", __lines);
    ConfigParser::stripLinesForKey("services", __remainingValuesToParse);
}

bool
InternalModelType::Hosts::operator==(const Hosts& __rhs) const noexcept
{
    return (name == __rhs.name &&
            services == __rhs.services);
}

bool
InternalModelType::Hosts::operator!=(const Hosts& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalModelType::Hosts::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("services");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : services) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalModelType::Hosts::Hosts(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["services"]["value"].children(); __i++) {
        services.push_back(Services(__inspector["services"]["value"][__i]["value"]));
    }
}

InternalModelType::Hosts::Hosts(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()(__inspector["name"], "(unknownhostname)");
    ::config::internal::VectorInserter<ServicesVector> __servicesInserter(services);
    __inspector["services"].traverse(__servicesInserter);
}

InternalModelType::InternalModelType()
    : vespaVersion("unknown"),
      hosts()
{
}

InternalModelType::InternalModelType(const InternalModelType & __rhs) = default;
InternalModelType & InternalModelType::operator =(const InternalModelType & __rhs) = default;
InternalModelType::InternalModelType(InternalModelType && __rhs) noexcept = default;
InternalModelType & InternalModelType::operator =(InternalModelType && __rhs) noexcept = default;
InternalModelType::~InternalModelType() = default; 
InternalModelType::InternalModelType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        vespaVersion = ConfigParser::parse<vespalib::string>("vespaVersion", __lines, "unknown");
        ConfigParser::stripLinesForKey("vespaVersion", __remainingValuesToParse);
        hosts = ConfigParser::parseArray<HostsVector>("hosts", __lines);
        ConfigParser::stripLinesForKey("hosts", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalModelType::operator==(const InternalModelType& __rhs) const noexcept
{
    return (vespaVersion == __rhs.vespaVersion &&
            hosts == __rhs.hosts);
}

bool
InternalModelType::operator!=(const InternalModelType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalModelType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("vespaVersion");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(vespaVersion));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("hosts");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : hosts) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalModelType::InternalModelType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    vespaVersion = __inspector["vespaVersion"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["hosts"]["value"].children(); __i++) {
        hosts.push_back(Hosts(__inspector["hosts"]["value"][__i]["value"]));
    }
}

InternalModelType::InternalModelType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    vespaVersion = ::config::internal::ValueConverter<vespalib::string>()(__inspector["vespaVersion"], "unknown");
    ::config::internal::VectorInserter<HostsVector> __hostsInserter(hosts);
    __inspector["hosts"].traverse(__hostsInserter);
}

} // namespace internal

} // namespace cloud::config
