/**
 * @class cloud::config::Model
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_CLOUD_CONFIG_MODEL_H
#define CLOUD_CONFIG_CLOUD_CONFIG_MODEL_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace cloud::config {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalModelType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalModelType> UP;
    class Hosts {
    public:
        class Services {
        public:
            class Ports {
            public:
                Ports(const ::config::StringVector & __lines);
                Ports(const vespalib::slime::Inspector & __inspector);
                Ports(const ::config::ConfigPayload & __payload);
                Ports()  noexcept;
                Ports(const Ports & __rhs) noexcept;
                Ports & operator = (const Ports & __rhs) noexcept;
                Ports(Ports && __rhs) noexcept;
                Ports & operator = (Ports && __rhs) noexcept;
                ~Ports();

                bool operator==(const Ports& __rhs) const noexcept;
                bool operator!=(const Ports& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;
                int32_t number; // Default: -1
                vespalib::string tags; // Default: "(notags)"
            };
            typedef std::vector<Ports> PortsVector;

            typedef std::map<vespalib::string, Ports> PortsMap;

            Services(const ::config::StringVector & __lines);
            Services(const vespalib::slime::Inspector & __inspector);
            Services(const ::config::ConfigPayload & __payload);
            Services() ;
            Services(const Services & __rhs);
            Services & operator = (const Services & __rhs);
            Services(Services && __rhs) noexcept;
            Services & operator = (Services && __rhs) noexcept;
            ~Services();

            bool operator==(const Services& __rhs) const noexcept;
            bool operator!=(const Services& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            vespalib::string name; // Default: "(noservicename)"
            vespalib::string type; // Default: "(noservicetype)"
            vespalib::string configid; // Default: "(noconfigid)"
            vespalib::string clustertype; // Default: "(unknownclustertype)"
            vespalib::string clustername; // Default: "(unknownclustername)"
            int32_t index; // Default: 0
            PortsVector ports;
        };
        typedef std::vector<Services> ServicesVector;

        typedef std::map<vespalib::string, Services> ServicesMap;

        Hosts(const ::config::StringVector & __lines);
        Hosts(const vespalib::slime::Inspector & __inspector);
        Hosts(const ::config::ConfigPayload & __payload);
        Hosts() ;
        Hosts(const Hosts & __rhs);
        Hosts & operator = (const Hosts & __rhs);
        Hosts(Hosts && __rhs) noexcept;
        Hosts & operator = (Hosts && __rhs) noexcept;
        ~Hosts();

        bool operator==(const Hosts& __rhs) const noexcept;
        bool operator!=(const Hosts& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        vespalib::string name; // Default: "(unknownhostname)"
        ServicesVector services;
    };
    typedef std::vector<Hosts> HostsVector;

    typedef std::map<vespalib::string, Hosts> HostsMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalModelType(const ::config::ConfigValue & __value);
    InternalModelType(const ::config::ConfigDataBuffer & __value);
    InternalModelType(const ::config::ConfigPayload & __payload);
    InternalModelType() ;
    InternalModelType(const InternalModelType & __rhs);
    InternalModelType & operator = (const InternalModelType & __rhs);
    InternalModelType(InternalModelType && __rhs) noexcept;
    InternalModelType & operator = (InternalModelType && __rhs) noexcept;
    ~InternalModelType();

    bool operator==(const InternalModelType& __rhs) const noexcept;
    bool operator!=(const InternalModelType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    vespalib::string vespaVersion; // Default: "unknown"
    HostsVector hosts;
};

} // namespace internal

typedef internal::InternalModelType ModelConfigBuilder;
typedef const internal::InternalModelType ModelConfig;

} // namespace cloud::config
#endif // VESPA_config_CLOUD_CONFIG_MODEL_H
