#include "config-onnx-models.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalOnnxModelsType::CONFIG_DEF_MD5("9913a57d64b34a429d24117926c02b1e");
const vespalib::string InternalOnnxModelsType::CONFIG_DEF_NAME("onnx-models");
const vespalib::string InternalOnnxModelsType::CONFIG_DEF_NAMESPACE("vespa.config.search.core");
const int64_t InternalOnnxModelsType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search.core",
"gpu.count int default=-1",
"model[].name string",
"model[].fileref file",
"model[].input[].name string",
"model[].input[].source string",
"model[].output[].name string",
"model[].output[].as string",
"model[].dry_run_on_setup bool default=false",
"model[].stateless_execution_mode string default=\"\"",
"model[].stateless_interop_threads int default=-1",
"model[].stateless_intraop_threads int default=-1",
"model[].gpu_device int default=-1",
"model[].gpu_device_required bool default=false",
};
const ::config::StringVector InternalOnnxModelsType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalOnnxModelsType::Gpu::Gpu() noexcept
    : count(-1)
{
}

InternalOnnxModelsType::Gpu::Gpu(const Gpu & __rhs) noexcept = default;
InternalOnnxModelsType::Gpu & InternalOnnxModelsType::Gpu::operator = (const Gpu & __rhs) noexcept = default;
InternalOnnxModelsType::Gpu::Gpu(Gpu && __rhs) noexcept = default;
InternalOnnxModelsType::Gpu & InternalOnnxModelsType::Gpu::operator = (Gpu && __rhs) noexcept = default;
InternalOnnxModelsType::Gpu::~Gpu() = default; 
InternalOnnxModelsType::Gpu::Gpu(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    count = ConfigParser::parse<int32_t>("count", __lines, -1);
    ConfigParser::stripLinesForKey("count", __remainingValuesToParse);
}

bool
InternalOnnxModelsType::Gpu::operator==(const Gpu& __rhs) const noexcept
{
    return (count == __rhs.count);
}

bool
InternalOnnxModelsType::Gpu::operator!=(const Gpu& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalOnnxModelsType::Gpu::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("count");
        __c.setString("type", "int");
        __c.setLong("value", count);
    }
}

InternalOnnxModelsType::Gpu::Gpu(const vespalib::slime::Inspector & __inspector)
{
    count = __inspector["count"]["value"].asLong();
}

InternalOnnxModelsType::Gpu::Gpu(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    count = ::config::internal::ValueConverter<int32_t>()(__inspector["count"], -1);
}

InternalOnnxModelsType::Model::Input::Input() noexcept
    : name(),
      source()
{
}

InternalOnnxModelsType::Model::Input::Input(const Input & __rhs) noexcept = default;
InternalOnnxModelsType::Model::Input & InternalOnnxModelsType::Model::Input::operator = (const Input & __rhs) noexcept = default;
InternalOnnxModelsType::Model::Input::Input(Input && __rhs) noexcept = default;
InternalOnnxModelsType::Model::Input & InternalOnnxModelsType::Model::Input::operator = (Input && __rhs) noexcept = default;
InternalOnnxModelsType::Model::Input::~Input() = default; 
InternalOnnxModelsType::Model::Input::Input(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    source = ConfigParser::parse<vespalib::string>("source", __lines);
    ConfigParser::stripLinesForKey("source", __remainingValuesToParse);
}

bool
InternalOnnxModelsType::Model::Input::operator==(const Input& __rhs) const noexcept
{
    return (name == __rhs.name &&
            source == __rhs.source);
}

bool
InternalOnnxModelsType::Model::Input::operator!=(const Input& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalOnnxModelsType::Model::Input::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("source");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(source));
    }
}

InternalOnnxModelsType::Model::Input::Input(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    source = __inspector["source"]["value"].asString().make_string();
}

InternalOnnxModelsType::Model::Input::Input(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    source = ::config::internal::ValueConverter<vespalib::string>()("source", __inspector["source"]);
}

InternalOnnxModelsType::Model::Output::Output() noexcept
    : name(),
      as()
{
}

InternalOnnxModelsType::Model::Output::Output(const Output & __rhs) noexcept = default;
InternalOnnxModelsType::Model::Output & InternalOnnxModelsType::Model::Output::operator = (const Output & __rhs) noexcept = default;
InternalOnnxModelsType::Model::Output::Output(Output && __rhs) noexcept = default;
InternalOnnxModelsType::Model::Output & InternalOnnxModelsType::Model::Output::operator = (Output && __rhs) noexcept = default;
InternalOnnxModelsType::Model::Output::~Output() = default; 
InternalOnnxModelsType::Model::Output::Output(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    as = ConfigParser::parse<vespalib::string>("as", __lines);
    ConfigParser::stripLinesForKey("as", __remainingValuesToParse);
}

bool
InternalOnnxModelsType::Model::Output::operator==(const Output& __rhs) const noexcept
{
    return (name == __rhs.name &&
            as == __rhs.as);
}

bool
InternalOnnxModelsType::Model::Output::operator!=(const Output& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalOnnxModelsType::Model::Output::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("as");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(as));
    }
}

InternalOnnxModelsType::Model::Output::Output(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    as = __inspector["as"]["value"].asString().make_string();
}

InternalOnnxModelsType::Model::Output::Output(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    as = ::config::internal::ValueConverter<vespalib::string>()("as", __inspector["as"]);
}

InternalOnnxModelsType::Model::Model()
    : name(),
      fileref(),
      input(),
      output(),
      dryRunOnSetup(false),
      statelessExecutionMode(""),
      statelessInteropThreads(-1),
      statelessIntraopThreads(-1),
      gpuDevice(-1),
      gpuDeviceRequired(false)
{
}

InternalOnnxModelsType::Model::Model(const Model & __rhs) = default;
InternalOnnxModelsType::Model & InternalOnnxModelsType::Model::operator = (const Model & __rhs) = default;
InternalOnnxModelsType::Model::Model(Model && __rhs) noexcept = default;
InternalOnnxModelsType::Model & InternalOnnxModelsType::Model::operator = (Model && __rhs) noexcept = default;
InternalOnnxModelsType::Model::~Model() = default; 
InternalOnnxModelsType::Model::Model(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    fileref = ConfigParser::parse<vespalib::string>("fileref", __lines);
    ConfigParser::stripLinesForKey("fileref", __remainingValuesToParse);
    input = ConfigParser::parseArray<InputVector>("input", __lines);
    ConfigParser::stripLinesForKey("input", __remainingValuesToParse);
    output = ConfigParser::parseArray<OutputVector>("output", __lines);
    ConfigParser::stripLinesForKey("output", __remainingValuesToParse);
    dryRunOnSetup = ConfigParser::parse<bool>("dry_run_on_setup", __lines, false);
    ConfigParser::stripLinesForKey("dry_run_on_setup", __remainingValuesToParse);
    statelessExecutionMode = ConfigParser::parse<vespalib::string>("stateless_execution_mode", __lines, "");
    ConfigParser::stripLinesForKey("stateless_execution_mode", __remainingValuesToParse);
    statelessInteropThreads = ConfigParser::parse<int32_t>("stateless_interop_threads", __lines, -1);
    ConfigParser::stripLinesForKey("stateless_interop_threads", __remainingValuesToParse);
    statelessIntraopThreads = ConfigParser::parse<int32_t>("stateless_intraop_threads", __lines, -1);
    ConfigParser::stripLinesForKey("stateless_intraop_threads", __remainingValuesToParse);
    gpuDevice = ConfigParser::parse<int32_t>("gpu_device", __lines, -1);
    ConfigParser::stripLinesForKey("gpu_device", __remainingValuesToParse);
    gpuDeviceRequired = ConfigParser::parse<bool>("gpu_device_required", __lines, false);
    ConfigParser::stripLinesForKey("gpu_device_required", __remainingValuesToParse);
}

bool
InternalOnnxModelsType::Model::operator==(const Model& __rhs) const noexcept
{
    return (name == __rhs.name &&
            fileref == __rhs.fileref &&
            input == __rhs.input &&
            output == __rhs.output &&
            dryRunOnSetup == __rhs.dryRunOnSetup &&
            statelessExecutionMode == __rhs.statelessExecutionMode &&
            statelessInteropThreads == __rhs.statelessInteropThreads &&
            statelessIntraopThreads == __rhs.statelessIntraopThreads &&
            gpuDevice == __rhs.gpuDevice &&
            gpuDeviceRequired == __rhs.gpuDeviceRequired);
}

bool
InternalOnnxModelsType::Model::operator!=(const Model& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalOnnxModelsType::Model::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fileref");
        __c.setString("type", "file");
        __c.setString("value", vespalib::Memory(fileref));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("input");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : input) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("output");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : output) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("dry_run_on_setup");
        __c.setString("type", "bool");
        __c.setBool("value", dryRunOnSetup);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("stateless_execution_mode");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(statelessExecutionMode));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("stateless_interop_threads");
        __c.setString("type", "int");
        __c.setLong("value", statelessInteropThreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("stateless_intraop_threads");
        __c.setString("type", "int");
        __c.setLong("value", statelessIntraopThreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("gpu_device");
        __c.setString("type", "int");
        __c.setLong("value", gpuDevice);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("gpu_device_required");
        __c.setString("type", "bool");
        __c.setBool("value", gpuDeviceRequired);
    }
}

InternalOnnxModelsType::Model::Model(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    fileref = __inspector["fileref"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["input"]["value"].children(); __i++) {
        input.push_back(Input(__inspector["input"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["output"]["value"].children(); __i++) {
        output.push_back(Output(__inspector["output"]["value"][__i]["value"]));
    }
    dryRunOnSetup = __inspector["dry_run_on_setup"]["value"].asBool();
    statelessExecutionMode = __inspector["stateless_execution_mode"]["value"].asString().make_string();
    statelessInteropThreads = __inspector["stateless_interop_threads"]["value"].asLong();
    statelessIntraopThreads = __inspector["stateless_intraop_threads"]["value"].asLong();
    gpuDevice = __inspector["gpu_device"]["value"].asLong();
    gpuDeviceRequired = __inspector["gpu_device_required"]["value"].asBool();
}

InternalOnnxModelsType::Model::Model(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    fileref = ::config::internal::ValueConverter<vespalib::string>()("fileref", __inspector["fileref"]);
    ::config::internal::VectorInserter<InputVector> __inputInserter(input);
    __inspector["input"].traverse(__inputInserter);
    ::config::internal::VectorInserter<OutputVector> __outputInserter(output);
    __inspector["output"].traverse(__outputInserter);
    dryRunOnSetup = ::config::internal::ValueConverter<bool>()(__inspector["dry_run_on_setup"], false);
    statelessExecutionMode = ::config::internal::ValueConverter<vespalib::string>()(__inspector["stateless_execution_mode"], "");
    statelessInteropThreads = ::config::internal::ValueConverter<int32_t>()(__inspector["stateless_interop_threads"], -1);
    statelessIntraopThreads = ::config::internal::ValueConverter<int32_t>()(__inspector["stateless_intraop_threads"], -1);
    gpuDevice = ::config::internal::ValueConverter<int32_t>()(__inspector["gpu_device"], -1);
    gpuDeviceRequired = ::config::internal::ValueConverter<bool>()(__inspector["gpu_device_required"], false);
}

InternalOnnxModelsType::InternalOnnxModelsType()
    : gpu(),
      model()
{
}

InternalOnnxModelsType::InternalOnnxModelsType(const InternalOnnxModelsType & __rhs) = default;
InternalOnnxModelsType & InternalOnnxModelsType::operator =(const InternalOnnxModelsType & __rhs) = default;
InternalOnnxModelsType::InternalOnnxModelsType(InternalOnnxModelsType && __rhs) noexcept = default;
InternalOnnxModelsType & InternalOnnxModelsType::operator =(InternalOnnxModelsType && __rhs) noexcept = default;
InternalOnnxModelsType::~InternalOnnxModelsType() = default; 
InternalOnnxModelsType::InternalOnnxModelsType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        gpu = ConfigParser::parseStruct<Gpu>("gpu", __lines);
        ConfigParser::stripLinesForKey("gpu", __remainingValuesToParse);
        model = ConfigParser::parseArray<ModelVector>("model", __lines);
        ConfigParser::stripLinesForKey("model", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalOnnxModelsType::operator==(const InternalOnnxModelsType& __rhs) const noexcept
{
    return (gpu == __rhs.gpu &&
            model == __rhs.model);
}

bool
InternalOnnxModelsType::operator!=(const InternalOnnxModelsType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalOnnxModelsType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("gpu");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        gpu.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("model");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : model) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalOnnxModelsType::InternalOnnxModelsType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    gpu = Gpu(__inspector["gpu"]["value"]);
    for (size_t __i = 0; __i < __inspector["model"]["value"].children(); __i++) {
        model.push_back(Model(__inspector["model"]["value"][__i]["value"]));
    }
}

InternalOnnxModelsType::InternalOnnxModelsType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    gpu = ::config::internal::ValueConverter<Gpu>()(__inspector["gpu"]);
    ::config::internal::VectorInserter<ModelVector> __modelInserter(model);
    __inspector["model"].traverse(__modelInserter);
}

} // namespace internal

} // namespace vespa::config::search::core
