/**
 * @class vespa::config::search::core::OnnxModels
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_SEARCH_CORE_ONNXMODELS_H
#define CLOUD_CONFIG_VESPA_CONFIG_SEARCH_CORE_ONNXMODELS_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::search::core {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalOnnxModelsType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalOnnxModelsType> UP;
    class Gpu {
    public:
        Gpu(const ::config::StringVector & __lines);
        Gpu(const vespalib::slime::Inspector & __inspector);
        Gpu(const ::config::ConfigPayload & __payload);
        Gpu()  noexcept;
        Gpu(const Gpu & __rhs) noexcept;
        Gpu & operator = (const Gpu & __rhs) noexcept;
        Gpu(Gpu && __rhs) noexcept;
        Gpu & operator = (Gpu && __rhs) noexcept;
        ~Gpu();

        bool operator==(const Gpu& __rhs) const noexcept;
        bool operator!=(const Gpu& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** Number of GPUs available for ONNX evaluation, or -1 if unknown. */
        int32_t count; // Default: -1
    };
    typedef std::vector<Gpu> GpuVector;

    typedef std::map<vespalib::string, Gpu> GpuMap;

    class Model {
    public:
        class Input {
        public:
            Input(const ::config::StringVector & __lines);
            Input(const vespalib::slime::Inspector & __inspector);
            Input(const ::config::ConfigPayload & __payload);
            Input()  noexcept;
            Input(const Input & __rhs) noexcept;
            Input & operator = (const Input & __rhs) noexcept;
            Input(Input && __rhs) noexcept;
            Input & operator = (Input && __rhs) noexcept;
            ~Input();

            bool operator==(const Input& __rhs) const noexcept;
            bool operator!=(const Input& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            vespalib::string name;
            vespalib::string source;
        };
        typedef std::vector<Input> InputVector;

        typedef std::map<vespalib::string, Input> InputMap;

        class Output {
        public:
            Output(const ::config::StringVector & __lines);
            Output(const vespalib::slime::Inspector & __inspector);
            Output(const ::config::ConfigPayload & __payload);
            Output()  noexcept;
            Output(const Output & __rhs) noexcept;
            Output & operator = (const Output & __rhs) noexcept;
            Output(Output && __rhs) noexcept;
            Output & operator = (Output && __rhs) noexcept;
            ~Output();

            bool operator==(const Output& __rhs) const noexcept;
            bool operator!=(const Output& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            vespalib::string name;
            vespalib::string as;
        };
        typedef std::vector<Output> OutputVector;

        typedef std::map<vespalib::string, Output> OutputMap;

        Model(const ::config::StringVector & __lines);
        Model(const vespalib::slime::Inspector & __inspector);
        Model(const ::config::ConfigPayload & __payload);
        Model() ;
        Model(const Model & __rhs);
        Model & operator = (const Model & __rhs);
        Model(Model && __rhs) noexcept;
        Model & operator = (Model && __rhs) noexcept;
        ~Model();

        bool operator==(const Model& __rhs) const noexcept;
        bool operator!=(const Model& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        vespalib::string name;
        vespalib::string fileref;
        InputVector input;
        OutputVector output;
        bool dryRunOnSetup; // Default: false
        vespalib::string statelessExecutionMode; // Default: ""
        int32_t statelessInteropThreads; // Default: -1
        int32_t statelessIntraopThreads; // Default: -1
        int32_t gpuDevice; // Default: -1
        bool gpuDeviceRequired; // Default: false
    };
    typedef std::vector<Model> ModelVector;

    typedef std::map<vespalib::string, Model> ModelMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalOnnxModelsType(const ::config::ConfigValue & __value);
    InternalOnnxModelsType(const ::config::ConfigDataBuffer & __value);
    InternalOnnxModelsType(const ::config::ConfigPayload & __payload);
    InternalOnnxModelsType() ;
    InternalOnnxModelsType(const InternalOnnxModelsType & __rhs);
    InternalOnnxModelsType & operator = (const InternalOnnxModelsType & __rhs);
    InternalOnnxModelsType(InternalOnnxModelsType && __rhs) noexcept;
    InternalOnnxModelsType & operator = (InternalOnnxModelsType && __rhs) noexcept;
    ~InternalOnnxModelsType();

    bool operator==(const InternalOnnxModelsType& __rhs) const noexcept;
    bool operator!=(const InternalOnnxModelsType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    Gpu gpu;
    ModelVector model;
};

} // namespace internal

typedef internal::InternalOnnxModelsType OnnxModelsConfigBuilder;
typedef const internal::InternalOnnxModelsType OnnxModelsConfig;

} // namespace vespa::config::search::core
#endif // VESPA_config_VESPA_CONFIG_SEARCH_CORE_ONNXMODELS_H
