#include "config-open-telemetry.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace cloud::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalOpenTelemetryType::CONFIG_DEF_MD5("7ca00a9409bc0071551d4f0335adacae");
const vespalib::string InternalOpenTelemetryType::CONFIG_DEF_NAME("open-telemetry");
const vespalib::string InternalOpenTelemetryType::CONFIG_DEF_NAMESPACE("cloud.config");
const int64_t InternalOpenTelemetryType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=cloud.config",
"yaml string",
"refPaths[] string",
};
const ::config::StringVector InternalOpenTelemetryType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalOpenTelemetryType::InternalOpenTelemetryType()
    : yaml(),
      refPaths()
{
}

InternalOpenTelemetryType::InternalOpenTelemetryType(const InternalOpenTelemetryType & __rhs) = default;
InternalOpenTelemetryType & InternalOpenTelemetryType::operator =(const InternalOpenTelemetryType & __rhs) = default;
InternalOpenTelemetryType::InternalOpenTelemetryType(InternalOpenTelemetryType && __rhs) noexcept = default;
InternalOpenTelemetryType & InternalOpenTelemetryType::operator =(InternalOpenTelemetryType && __rhs) noexcept = default;
InternalOpenTelemetryType::~InternalOpenTelemetryType() = default; 
InternalOpenTelemetryType::InternalOpenTelemetryType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        yaml = ConfigParser::parse<vespalib::string>("yaml", __lines);
        ConfigParser::stripLinesForKey("yaml", __remainingValuesToParse);
        refPaths = ConfigParser::parseArray<::config::StringVector>("refPaths", __lines);
        ConfigParser::stripLinesForKey("refPaths", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalOpenTelemetryType::
operator==(const InternalOpenTelemetryType& __rhs) const noexcept
{
    return (yaml == __rhs.yaml &&
            refPaths == __rhs.refPaths);
}

bool
InternalOpenTelemetryType::
operator!=(const InternalOpenTelemetryType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalOpenTelemetryType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("yaml");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(yaml));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("refPaths");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : refPaths) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
}

InternalOpenTelemetryType::InternalOpenTelemetryType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    yaml = __inspector["yaml"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["refPaths"]["value"].children(); __i++) {
        refPaths.push_back(__inspector["refPaths"]["value"][__i]["value"].asString().make_string());
    }
}

InternalOpenTelemetryType::InternalOpenTelemetryType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    yaml = ::config::internal::ValueConverter<vespalib::string>()("yaml", __inspector["yaml"]);
    ::config::internal::VectorInserter<::config::StringVector> __refPathsInserter(refPaths);
    __inspector["refPaths"].traverse(__refPathsInserter);
}

} // namespace internal

} // namespace cloud::config
