#include "config-orchestrator.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::orchestrator::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalOrchestratorType::CONFIG_DEF_MD5("6e219d782624697a8256471110590f04");
const vespalib::string InternalOrchestratorType::CONFIG_DEF_NAME("orchestrator");
const vespalib::string InternalOrchestratorType::CONFIG_DEF_NAMESPACE("vespa.orchestrator.config");
const int64_t InternalOrchestratorType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.orchestrator.config",
"serviceMonitorConvergenceLatencySeconds int default=0",
"numProxies int default=0",
"numProxiesAllowedDown int default=1",
"numProxiesAllowedDownRatio double default=0.1",
};
const ::config::StringVector InternalOrchestratorType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalOrchestratorType::InternalOrchestratorType() noexcept
    : serviceMonitorConvergenceLatencySeconds(0),
      numProxies(0),
      numProxiesAllowedDown(1),
      numProxiesAllowedDownRatio(0.1)
{
}

InternalOrchestratorType::InternalOrchestratorType(const InternalOrchestratorType & __rhs) noexcept = default;
InternalOrchestratorType & InternalOrchestratorType::operator =(const InternalOrchestratorType & __rhs) noexcept = default;
InternalOrchestratorType::InternalOrchestratorType(InternalOrchestratorType && __rhs) noexcept = default;
InternalOrchestratorType & InternalOrchestratorType::operator =(InternalOrchestratorType && __rhs) noexcept = default;
InternalOrchestratorType::~InternalOrchestratorType() = default; 
InternalOrchestratorType::InternalOrchestratorType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        serviceMonitorConvergenceLatencySeconds = ConfigParser::parse<int32_t>("serviceMonitorConvergenceLatencySeconds", __lines, 0);
        ConfigParser::stripLinesForKey("serviceMonitorConvergenceLatencySeconds", __remainingValuesToParse);
        numProxies = ConfigParser::parse<int32_t>("numProxies", __lines, 0);
        ConfigParser::stripLinesForKey("numProxies", __remainingValuesToParse);
        numProxiesAllowedDown = ConfigParser::parse<int32_t>("numProxiesAllowedDown", __lines, 1);
        ConfigParser::stripLinesForKey("numProxiesAllowedDown", __remainingValuesToParse);
        numProxiesAllowedDownRatio = ConfigParser::parse<double>("numProxiesAllowedDownRatio", __lines, 0.1);
        ConfigParser::stripLinesForKey("numProxiesAllowedDownRatio", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalOrchestratorType::
operator==(const InternalOrchestratorType& __rhs) const noexcept
{
    return (serviceMonitorConvergenceLatencySeconds == __rhs.serviceMonitorConvergenceLatencySeconds &&
            numProxies == __rhs.numProxies &&
            numProxiesAllowedDown == __rhs.numProxiesAllowedDown &&
            numProxiesAllowedDownRatio == __rhs.numProxiesAllowedDownRatio);
}

bool
InternalOrchestratorType::
operator!=(const InternalOrchestratorType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalOrchestratorType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("serviceMonitorConvergenceLatencySeconds");
        __c.setString("type", "int");
        __c.setLong("value", serviceMonitorConvergenceLatencySeconds);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("numProxies");
        __c.setString("type", "int");
        __c.setLong("value", numProxies);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("numProxiesAllowedDown");
        __c.setString("type", "int");
        __c.setLong("value", numProxiesAllowedDown);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("numProxiesAllowedDownRatio");
        __c.setString("type", "double");
        __c.setDouble("value", numProxiesAllowedDownRatio);
    }
}

InternalOrchestratorType::InternalOrchestratorType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    serviceMonitorConvergenceLatencySeconds = __inspector["serviceMonitorConvergenceLatencySeconds"]["value"].asLong();
    numProxies = __inspector["numProxies"]["value"].asLong();
    numProxiesAllowedDown = __inspector["numProxiesAllowedDown"]["value"].asLong();
    numProxiesAllowedDownRatio = __inspector["numProxiesAllowedDownRatio"]["value"].asDouble();
}

InternalOrchestratorType::InternalOrchestratorType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    serviceMonitorConvergenceLatencySeconds = ::config::internal::ValueConverter<int32_t>()(__inspector["serviceMonitorConvergenceLatencySeconds"], 0);
    numProxies = ::config::internal::ValueConverter<int32_t>()(__inspector["numProxies"], 0);
    numProxiesAllowedDown = ::config::internal::ValueConverter<int32_t>()(__inspector["numProxiesAllowedDown"], 1);
    numProxiesAllowedDownRatio = ::config::internal::ValueConverter<double>()(__inspector["numProxiesAllowedDownRatio"], 0.1);
}

} // namespace internal

} // namespace vespa::orchestrator::config
