/**
 * @class vespa::orchestrator::config::Orchestrator
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_ORCHESTRATOR_CONFIG_ORCHESTRATOR_H
#define CLOUD_CONFIG_VESPA_ORCHESTRATOR_CONFIG_ORCHESTRATOR_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::orchestrator::config {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalOrchestratorType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalOrchestratorType> UP;
    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalOrchestratorType(const ::config::ConfigValue & __value);
    InternalOrchestratorType(const ::config::ConfigDataBuffer & __value);
    InternalOrchestratorType(const ::config::ConfigPayload & __payload);
    InternalOrchestratorType()  noexcept;
    InternalOrchestratorType(const InternalOrchestratorType & __rhs) noexcept;
    InternalOrchestratorType & operator = (const InternalOrchestratorType & __rhs) noexcept;
    InternalOrchestratorType(InternalOrchestratorType && __rhs) noexcept;
    InternalOrchestratorType & operator = (InternalOrchestratorType && __rhs) noexcept;
    ~InternalOrchestratorType();

    bool operator==(const InternalOrchestratorType& __rhs) const noexcept;
    bool operator!=(const InternalOrchestratorType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /** TODO: Change the default to actual latency in real setup. */
    int32_t serviceMonitorConvergenceLatencySeconds; // Default: 0

    /**
     * The number of services in the routing/container cluster of
     * hosted-vespa.routing.default.
     */
    int32_t numProxies; // Default: 0

    /**
     * The number of of the services that are allowed to be down for the routing
     * container cluster
     * in the hosted-vespa.routing.default infrastructure application.
     */
    int32_t numProxiesAllowedDown; // Default: 1

    /**
     * See numProxiesAllowedDown for background. Allow up to this ratio of
     * services to be down,
     * unless already allowed by numProxiesAllowedDown.
     */
    double numProxiesAllowedDownRatio; // Default: 0.1
};

} // namespace internal

typedef internal::InternalOrchestratorType OrchestratorConfigBuilder;
typedef const internal::InternalOrchestratorType OrchestratorConfig;

} // namespace vespa::orchestrator::config
#endif // VESPA_config_VESPA_ORCHESTRATOR_CONFIG_ORCHESTRATOR_H
