#include "config-persistence.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalPersistenceType::CONFIG_DEF_MD5("e2ec2c9ef2aa6c99583a99ab1bcb5065");
const vespalib::string InternalPersistenceType::CONFIG_DEF_NAME("persistence");
const vespalib::string InternalPersistenceType::CONFIG_DEF_NAMESPACE("vespa.config.content");
const int64_t InternalPersistenceType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content",
"abort_operations_with_changed_bucket_ownership bool default=true",
"abort_outdated_mutating_ideal_state_ops bool default=true",
"abort_outdated_mutating_external_load_ops bool default=true",
"fail_partition_on_error bool default=true",
"revert_time_period int default=300",
"keep_remove_time_period int default=604800",
"maximum_versions_of_single_document_stored int default=5",
};
const ::config::StringVector InternalPersistenceType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalPersistenceType::InternalPersistenceType() noexcept
    : abortOperationsWithChangedBucketOwnership(true),
      abortOutdatedMutatingIdealStateOps(true),
      abortOutdatedMutatingExternalLoadOps(true),
      failPartitionOnError(true),
      revertTimePeriod(300),
      keepRemoveTimePeriod(604800),
      maximumVersionsOfSingleDocumentStored(5)
{
}

InternalPersistenceType::InternalPersistenceType(const InternalPersistenceType & __rhs) noexcept = default;
InternalPersistenceType & InternalPersistenceType::operator =(const InternalPersistenceType & __rhs) noexcept = default;
InternalPersistenceType::InternalPersistenceType(InternalPersistenceType && __rhs) noexcept = default;
InternalPersistenceType & InternalPersistenceType::operator =(InternalPersistenceType && __rhs) noexcept = default;
InternalPersistenceType::~InternalPersistenceType() = default; 
InternalPersistenceType::InternalPersistenceType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        abortOperationsWithChangedBucketOwnership = ConfigParser::parse<bool>("abort_operations_with_changed_bucket_ownership", __lines, true);
        ConfigParser::stripLinesForKey("abort_operations_with_changed_bucket_ownership", __remainingValuesToParse);
        abortOutdatedMutatingIdealStateOps = ConfigParser::parse<bool>("abort_outdated_mutating_ideal_state_ops", __lines, true);
        ConfigParser::stripLinesForKey("abort_outdated_mutating_ideal_state_ops", __remainingValuesToParse);
        abortOutdatedMutatingExternalLoadOps = ConfigParser::parse<bool>("abort_outdated_mutating_external_load_ops", __lines, true);
        ConfigParser::stripLinesForKey("abort_outdated_mutating_external_load_ops", __remainingValuesToParse);
        failPartitionOnError = ConfigParser::parse<bool>("fail_partition_on_error", __lines, true);
        ConfigParser::stripLinesForKey("fail_partition_on_error", __remainingValuesToParse);
        revertTimePeriod = ConfigParser::parse<int32_t>("revert_time_period", __lines, 300);
        ConfigParser::stripLinesForKey("revert_time_period", __remainingValuesToParse);
        keepRemoveTimePeriod = ConfigParser::parse<int32_t>("keep_remove_time_period", __lines, 604800);
        ConfigParser::stripLinesForKey("keep_remove_time_period", __remainingValuesToParse);
        maximumVersionsOfSingleDocumentStored = ConfigParser::parse<int32_t>("maximum_versions_of_single_document_stored", __lines, 5);
        ConfigParser::stripLinesForKey("maximum_versions_of_single_document_stored", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalPersistenceType::operator==(const InternalPersistenceType& __rhs) const noexcept
{
    return (abortOperationsWithChangedBucketOwnership == __rhs.abortOperationsWithChangedBucketOwnership &&
            abortOutdatedMutatingIdealStateOps == __rhs.abortOutdatedMutatingIdealStateOps &&
            abortOutdatedMutatingExternalLoadOps == __rhs.abortOutdatedMutatingExternalLoadOps &&
            failPartitionOnError == __rhs.failPartitionOnError &&
            revertTimePeriod == __rhs.revertTimePeriod &&
            keepRemoveTimePeriod == __rhs.keepRemoveTimePeriod &&
            maximumVersionsOfSingleDocumentStored == __rhs.maximumVersionsOfSingleDocumentStored);
}

bool
InternalPersistenceType::operator!=(const InternalPersistenceType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalPersistenceType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("abort_operations_with_changed_bucket_ownership");
        __c.setString("type", "bool");
        __c.setBool("value", abortOperationsWithChangedBucketOwnership);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("abort_outdated_mutating_ideal_state_ops");
        __c.setString("type", "bool");
        __c.setBool("value", abortOutdatedMutatingIdealStateOps);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("abort_outdated_mutating_external_load_ops");
        __c.setString("type", "bool");
        __c.setBool("value", abortOutdatedMutatingExternalLoadOps);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fail_partition_on_error");
        __c.setString("type", "bool");
        __c.setBool("value", failPartitionOnError);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("revert_time_period");
        __c.setString("type", "int");
        __c.setLong("value", revertTimePeriod);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("keep_remove_time_period");
        __c.setString("type", "int");
        __c.setLong("value", keepRemoveTimePeriod);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maximum_versions_of_single_document_stored");
        __c.setString("type", "int");
        __c.setLong("value", maximumVersionsOfSingleDocumentStored);
    }
}

InternalPersistenceType::InternalPersistenceType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    abortOperationsWithChangedBucketOwnership = __inspector["abort_operations_with_changed_bucket_ownership"]["value"].asBool();
    abortOutdatedMutatingIdealStateOps = __inspector["abort_outdated_mutating_ideal_state_ops"]["value"].asBool();
    abortOutdatedMutatingExternalLoadOps = __inspector["abort_outdated_mutating_external_load_ops"]["value"].asBool();
    failPartitionOnError = __inspector["fail_partition_on_error"]["value"].asBool();
    revertTimePeriod = __inspector["revert_time_period"]["value"].asLong();
    keepRemoveTimePeriod = __inspector["keep_remove_time_period"]["value"].asLong();
    maximumVersionsOfSingleDocumentStored = __inspector["maximum_versions_of_single_document_stored"]["value"].asLong();
}

InternalPersistenceType::InternalPersistenceType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    abortOperationsWithChangedBucketOwnership = ::config::internal::ValueConverter<bool>()(__inspector["abort_operations_with_changed_bucket_ownership"], true);
    abortOutdatedMutatingIdealStateOps = ::config::internal::ValueConverter<bool>()(__inspector["abort_outdated_mutating_ideal_state_ops"], true);
    abortOutdatedMutatingExternalLoadOps = ::config::internal::ValueConverter<bool>()(__inspector["abort_outdated_mutating_external_load_ops"], true);
    failPartitionOnError = ::config::internal::ValueConverter<bool>()(__inspector["fail_partition_on_error"], true);
    revertTimePeriod = ::config::internal::ValueConverter<int32_t>()(__inspector["revert_time_period"], 300);
    keepRemoveTimePeriod = ::config::internal::ValueConverter<int32_t>()(__inspector["keep_remove_time_period"], 604800);
    maximumVersionsOfSingleDocumentStored = ::config::internal::ValueConverter<int32_t>()(__inspector["maximum_versions_of_single_document_stored"], 5);
}

} // namespace internal

} // namespace vespa::config::content
