/**
 * @class vespa::config::content::Persistence
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_CONTENT_PERSISTENCE_H
#define CLOUD_CONFIG_VESPA_CONFIG_CONTENT_PERSISTENCE_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::content {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalPersistenceType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalPersistenceType> UP;
    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalPersistenceType(const ::config::ConfigValue & __value);
    InternalPersistenceType(const ::config::ConfigDataBuffer & __value);
    InternalPersistenceType(const ::config::ConfigPayload & __payload);
    InternalPersistenceType()  noexcept;
    InternalPersistenceType(const InternalPersistenceType & __rhs) noexcept;
    InternalPersistenceType & operator = (const InternalPersistenceType & __rhs) noexcept;
    InternalPersistenceType(InternalPersistenceType && __rhs) noexcept;
    InternalPersistenceType & operator = (InternalPersistenceType && __rhs) noexcept;
    ~InternalPersistenceType();

    bool operator==(const InternalPersistenceType& __rhs) const noexcept;
    bool operator!=(const InternalPersistenceType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /**
     * If set, causes the service layer to trigger an abort of any queued
     * operations for buckets whose ownership changes during a cluster state
     * change. This helps ensure consistency between the distributors and
     * the storage nodes by discarding operations that were not sent by the
     * new owning distributor. The service layer will also block any new
     * operations from happening until all operations currently executing for
     * buckets with changed ownership have completed.
     * Enabling this option has a potential of causing a minor performance
     * impact when a node goes up or down due to its blocking semantics, but
     * this should be on the order of tens to a few hundred milliseconds.
     */
    bool abortOperationsWithChangedBucketOwnership; // Default: true

    /**
     * If set, aborts any mutating ideal state operations that arrive to a node
     * that were scheduled and sent by distributors that no longer own the
     * buckets
     * the operations are bound for. This option complements the above config,
     * as this is checked for every arriving message and not when the state
     * changes.
     */
    bool abortOutdatedMutatingIdealStateOps; // Default: true

    /**
     * If set, aborts any external (client) operations that arrive to a node
     * from
     * a distributor that is no longer the owner of the bucket those operations
     * belong to. As with the abort_outdated_mutating_ideal_state_ops option,
     * this is a check that happens whenever a message arrives, not when the
     * state
     * changes.
     */
    bool abortOutdatedMutatingExternalLoadOps; // Default: true

    /** Whether or not to disable partitions when they report I/O errors */
    bool failPartitionOnError; // Default: true

    /**
     * Time period to keep all updates (given in seconds). One can revert any
     * operation done within this time.
     */
    int32_t revertTimePeriod; // Default: 300

    /**
     * Minimum number of seconds that document remove (tombstone) entries are
     * retained in a bucket before being automatically compacted away. In the
     * case
     * where a node has been down for a longer period of time than this value
     * and then comes back up, there exists a possibility for documents to be
     * resurrected. This happens when documents existing on both the downed node
     * and the online nodes are marked as removed on the online nodes and then
     * subsequently compacted away along with their remove entries. When the old
     * node comes back online, it will appear to the system as if the node has
     * documents that do not exist on other nodes (as no remove-entries can be
     * found for them) and they will subsequently re-appear.
     */
    int32_t keepRemoveTimePeriod; // Default: 604800

    /**
     * For providers supporting multiple document versions, this option
     * specifies
     * a soft limit on the maximum number of versions that can be stored for a
     * single document at any given time. This is considered a soft limit as it
     * is possible for more versions to exist concurrently than what this config
     * value specifies in a case where the pressure on the containing bucket is
     * not
     * yet high enough to warrant a compaction.
     * After a compaction has taken place, only the n _newest_ document versions
     * will be present in the compacted representation of the bucket.
     * This limitation takes precedence over what versions may be stored
     * according
     * to revert_time_period.
     * Configured value must be at least 1.
     */
    int32_t maximumVersionsOfSingleDocumentStored; // Default: 5
};

} // namespace internal

typedef internal::InternalPersistenceType PersistenceConfigBuilder;
typedef const internal::InternalPersistenceType PersistenceConfig;

} // namespace vespa::config::content
#endif // VESPA_config_VESPA_CONFIG_CONTENT_PERSISTENCE_H
