/**
 * @class vespa::config::search::core::Proton
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_SEARCH_CORE_PROTON_H
#define CLOUD_CONFIG_VESPA_CONFIG_SEARCH_CORE_PROTON_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::search::core {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalProtonType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalProtonType> UP;
    class Rpc {
    public:
        Rpc(const ::config::StringVector & __lines);
        Rpc(const vespalib::slime::Inspector & __inspector);
        Rpc(const ::config::ConfigPayload & __payload);
        Rpc()  noexcept;
        Rpc(const Rpc & __rhs) noexcept;
        Rpc & operator = (const Rpc & __rhs) noexcept;
        Rpc(Rpc && __rhs) noexcept;
        Rpc & operator = (Rpc && __rhs) noexcept;
        ~Rpc();

        bool operator==(const Rpc& __rhs) const noexcept;
        bool operator!=(const Rpc& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * Number of threads used for rpc transport threads
         * A zero value will make the backend smart about the number.
         */
        int32_t transportthreads; // Default: 0
    };
    typedef std::vector<Rpc> RpcVector;

    typedef std::map<vespalib::string, Rpc> RpcMap;

    class Search {
    public:
        class Mmap {
        public:
            enum class Options { POPULATE, HUGETLB };
            typedef std::vector<Options> OptionsVector;
            typedef std::map<vespalib::string, Options> OptionsMap;
            static Options getOptions(const vespalib::string&);
            static vespalib::string getOptionsName(Options e);

            struct InternalOptionsConverter {
                Options operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                Options operator()(const ::vespalib::slime::Inspector & __inspector);
                Options operator()(const ::vespalib::slime::Inspector & __inspector, Options __eDefault);
            };
            enum class Advise { NORMAL, RANDOM, SEQUENTIAL };
            typedef std::vector<Advise> AdviseVector;
            typedef std::map<vespalib::string, Advise> AdviseMap;
            static Advise getAdvise(const vespalib::string&);
            static vespalib::string getAdviseName(Advise e);

            struct InternalAdviseConverter {
                Advise operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                Advise operator()(const ::vespalib::slime::Inspector & __inspector);
                Advise operator()(const ::vespalib::slime::Inspector & __inspector, Advise __eDefault);
            };
            Mmap(const ::config::StringVector & __lines);
            Mmap(const vespalib::slime::Inspector & __inspector);
            Mmap(const ::config::ConfigPayload & __payload);
            Mmap() ;
            Mmap(const Mmap & __rhs);
            Mmap & operator = (const Mmap & __rhs);
            Mmap(Mmap && __rhs) noexcept;
            Mmap & operator = (Mmap && __rhs) noexcept;
            ~Mmap();

            bool operator==(const Mmap& __rhs) const noexcept;
            bool operator!=(const Mmap& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** Multiple optional options for use with mmap */
            OptionsVector options;

            /**
             * Advise to give to os when mapping memory.
             * TODO Check if default should be random
             */
            Advise advise; // Default: NORMAL
        };
        typedef std::vector<Mmap> MmapVector;

        typedef std::map<vespalib::string, Mmap> MmapMap;

        class Memory {
        public:
            class Limiter {
            public:
                Limiter(const ::config::StringVector & __lines);
                Limiter(const vespalib::slime::Inspector & __inspector);
                Limiter(const ::config::ConfigPayload & __payload);
                Limiter()  noexcept;
                Limiter(const Limiter & __rhs) noexcept;
                Limiter & operator = (const Limiter & __rhs) noexcept;
                Limiter(Limiter && __rhs) noexcept;
                Limiter & operator = (Limiter && __rhs) noexcept;
                ~Limiter();

                bool operator==(const Limiter& __rhs) const noexcept;
                bool operator!=(const Limiter& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /**
                 * Max number of threads allowed to handle large queries
                 * concurrently
                 * Positive number means there is a limit, 0 or negative means
                 * no limit.
                 * TODO Check if ever used in config.
                 */
                int32_t maxthreads; // Default: 0

                /**
                 * Minimum coverage of corpus to postprocess before applying
                 * above concurrency limit.
                 */
                double mincoverage; // Default: 1.0

                /**
                 * Minimum number of hits to postprocess before applying above
                 * concurrency limit.
                 * Both must be covered before applying limiter.
                 */
                int32_t minhits; // Default: 1000000
            };
            typedef std::vector<Limiter> LimiterVector;

            typedef std::map<vespalib::string, Limiter> LimiterMap;

            Memory(const ::config::StringVector & __lines);
            Memory(const vespalib::slime::Inspector & __inspector);
            Memory(const ::config::ConfigPayload & __payload);
            Memory()  noexcept;
            Memory(const Memory & __rhs) noexcept;
            Memory & operator = (const Memory & __rhs) noexcept;
            Memory(Memory && __rhs) noexcept;
            Memory & operator = (Memory && __rhs) noexcept;
            ~Memory();

            bool operator==(const Memory& __rhs) const noexcept;
            bool operator!=(const Memory& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            Limiter limiter;
        };
        typedef std::vector<Memory> MemoryVector;

        typedef std::map<vespalib::string, Memory> MemoryMap;

        Search(const ::config::StringVector & __lines);
        Search(const vespalib::slime::Inspector & __inspector);
        Search(const ::config::ConfigPayload & __payload);
        Search() ;
        Search(const Search & __rhs);
        Search & operator = (const Search & __rhs);
        Search(Search && __rhs) noexcept;
        Search & operator = (Search && __rhs) noexcept;
        ~Search();

        bool operator==(const Search& __rhs) const noexcept;
        bool operator!=(const Search& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** Dispatch search requests to threadpool */
        bool async; // Default: true
        Mmap mmap;
        Memory memory;
    };
    typedef std::vector<Search> SearchVector;

    typedef std::map<vespalib::string, Search> SearchMap;

    class Docsum {
    public:
        Docsum(const ::config::StringVector & __lines);
        Docsum(const vespalib::slime::Inspector & __inspector);
        Docsum(const ::config::ConfigPayload & __payload);
        Docsum()  noexcept;
        Docsum(const Docsum & __rhs) noexcept;
        Docsum & operator = (const Docsum & __rhs) noexcept;
        Docsum(Docsum && __rhs) noexcept;
        Docsum & operator = (Docsum && __rhs) noexcept;
        ~Docsum();

        bool operator==(const Docsum& __rhs) const noexcept;
        bool operator!=(const Docsum& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** Dispatch docsum requests to threadpool */
        bool async; // Default: true
    };
    typedef std::vector<Docsum> DocsumVector;

    typedef std::map<vespalib::string, Docsum> DocsumMap;

    enum class ValidateAndSanitizeDocstore { NO, YES };
    typedef std::vector<ValidateAndSanitizeDocstore> ValidateAndSanitizeDocstoreVector;
    typedef std::map<vespalib::string, ValidateAndSanitizeDocstore> ValidateAndSanitizeDocstoreMap;
    static ValidateAndSanitizeDocstore getValidateAndSanitizeDocstore(const vespalib::string&);
    static vespalib::string getValidateAndSanitizeDocstoreName(ValidateAndSanitizeDocstore e);

    struct InternalValidateAndSanitizeDocstoreConverter {
        ValidateAndSanitizeDocstore operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
        ValidateAndSanitizeDocstore operator()(const ::vespalib::slime::Inspector & __inspector);
        ValidateAndSanitizeDocstore operator()(const ::vespalib::slime::Inspector & __inspector, ValidateAndSanitizeDocstore __eDefault);
    };
    class Flush {
    public:
        enum class Strategy { SIMPLE, MEMORY };
        typedef std::vector<Strategy> StrategyVector;
        typedef std::map<vespalib::string, Strategy> StrategyMap;
        static Strategy getStrategy(const vespalib::string&);
        static vespalib::string getStrategyName(Strategy e);

        struct InternalStrategyConverter {
            Strategy operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Strategy operator()(const ::vespalib::slime::Inspector & __inspector);
            Strategy operator()(const ::vespalib::slime::Inspector & __inspector, Strategy __eDefault);
        };
        class Memory {
        public:
            class Each {
            public:
                Each(const ::config::StringVector & __lines);
                Each(const vespalib::slime::Inspector & __inspector);
                Each(const ::config::ConfigPayload & __payload);
                Each()  noexcept;
                Each(const Each & __rhs) noexcept;
                Each & operator = (const Each & __rhs) noexcept;
                Each(Each && __rhs) noexcept;
                Each & operator = (Each && __rhs) noexcept;
                ~Each();

                bool operator==(const Each& __rhs) const noexcept;
                bool operator!=(const Each& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /**
                 * The maximum memory (in bytes) used by a single FLUSH
                 * component before running flush.
                 * A FLUSH component will free memory when flushed (e.g. memory
                 * index).
                 */
                int64_t maxmemory; // Default: 1073741824

                /**
                 * Maximum disk bloat factor per component before forcing flush.
                 */
                double diskbloatfactor; // Default: 0.25
            };
            typedef std::vector<Each> EachVector;

            typedef std::map<vespalib::string, Each> EachMap;

            class Maxage {
            public:
                Maxage(const ::config::StringVector & __lines);
                Maxage(const vespalib::slime::Inspector & __inspector);
                Maxage(const ::config::ConfigPayload & __payload);
                Maxage()  noexcept;
                Maxage(const Maxage & __rhs) noexcept;
                Maxage & operator = (const Maxage & __rhs) noexcept;
                Maxage(Maxage && __rhs) noexcept;
                Maxage & operator = (Maxage && __rhs) noexcept;
                ~Maxage();

                bool operator==(const Maxage& __rhs) const noexcept;
                bool operator!=(const Maxage& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /**
                 * Age of unflushed content before forcing age prioritization.
                 * Unit is seconds with 31 hours being the default.
                 * 31 is selected as it is both a prime and (31-24=7) is a prime
                 * and hence it will
                 * not be closer to a multiple of a day for a month, and it will
                 * be at least one hour away.
                 */
                double time; // Default: 111600.0
            };
            typedef std::vector<Maxage> MaxageVector;

            typedef std::map<vespalib::string, Maxage> MaxageMap;

            class Conservative {
            public:
                Conservative(const ::config::StringVector & __lines);
                Conservative(const vespalib::slime::Inspector & __inspector);
                Conservative(const ::config::ConfigPayload & __payload);
                Conservative()  noexcept;
                Conservative(const Conservative & __rhs) noexcept;
                Conservative & operator = (const Conservative & __rhs) noexcept;
                Conservative(Conservative && __rhs) noexcept;
                Conservative & operator = (Conservative && __rhs) noexcept;
                ~Conservative();

                bool operator==(const Conservative& __rhs) const noexcept;
                bool operator!=(const Conservative& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /**
                 * When resource limit for memory is reached we choose a
                 * conservative mode for the flush strategy.
                 * In this case this factor is multiplied with 'maxmemory' and
                 * 'each.maxmemory' to calculate conservative values to use
                 * instead.
                 */
                double memorylimitfactor; // Default: 0.5

                /**
                 * When resource limit for disk is reached we choose a
                 * conservative mode for the flush strategy.
                 * In this case this factor is multiplied with 'maxtlssize' to
                 * calculate a conservative value to use instead.
                 */
                double disklimitfactor; // Default: 0.5

                /**
                 * The factor used to multiply with the resource limits for disk
                 * / memory to find the high
                 * watermark indicating when to from normal into conservative
                 * mode for the flush strategy.
                 */
                double highwatermarkfactor; // Default: 0.95

                /**
                 * The factor used to multiply with the resource limits for disk
                 * / memory to find the low
                 * watermark indicating when to go back from conservative to
                 * normal mode for the flush strategy.
                 */
                double lowwatermarkfactor; // Default: 0.9
            };
            typedef std::vector<Conservative> ConservativeVector;

            typedef std::map<vespalib::string, Conservative> ConservativeMap;

            Memory(const ::config::StringVector & __lines);
            Memory(const vespalib::slime::Inspector & __inspector);
            Memory(const ::config::ConfigPayload & __payload);
            Memory()  noexcept;
            Memory(const Memory & __rhs) noexcept;
            Memory & operator = (const Memory & __rhs) noexcept;
            Memory(Memory && __rhs) noexcept;
            Memory & operator = (Memory && __rhs) noexcept;
            ~Memory();

            bool operator==(const Memory& __rhs) const noexcept;
            bool operator!=(const Memory& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * The total maximum memory (in bytes) used by FLUSH components
             * before running flush.
             * A FLUSH component will free memory when flushed (e.g. memory
             * index).
             */
            int64_t maxmemory; // Default: 4294967296

            /** Maximum total disk bloat factor before forcing flush. */
            double diskbloatfactor; // Default: 0.25

            /**
             * Max disk usage (in bytes) for all transaction logs before running
             * flush.
             * In this case the oldest component is flushed such that
             * transaction log can be pruned and disk freed.
             */
            int64_t maxtlssize; // Default: 21474836480
            Each each;
            Maxage maxage;
            Conservative conservative;
        };
        typedef std::vector<Memory> MemoryVector;

        typedef std::map<vespalib::string, Memory> MemoryMap;

        class Preparerestart {
        public:
            Preparerestart(const ::config::StringVector & __lines);
            Preparerestart(const vespalib::slime::Inspector & __inspector);
            Preparerestart(const ::config::ConfigPayload & __payload);
            Preparerestart()  noexcept;
            Preparerestart(const Preparerestart & __rhs) noexcept;
            Preparerestart & operator = (const Preparerestart & __rhs) noexcept;
            Preparerestart(Preparerestart && __rhs) noexcept;
            Preparerestart & operator = (Preparerestart && __rhs) noexcept;
            ~Preparerestart();

            bool operator==(const Preparerestart& __rhs) const noexcept;
            bool operator!=(const Preparerestart& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * The cost of replaying a byte when replaying the transaction log.
             *  
             * The estimate of the total cost of replaying the transaction log:
             * (number of bytes to replay) * replaycost + (number of operations
             * to replay) * replayoperationcost
             *  
             * The prepare for restart flush strategy will choose a set of
             * components to flush
             * such that the cost of flushing these + the cost of replaying the
             * transaction log
             * is as low as possible.
             */
            double replaycost; // Default: 8.0

            /**
             * The cost of replaying an operation when replaying the transaction
             * log.
             *  
             * The estimate of the total cost of replaying the transaction log:
             * (number of bytes to replay) * replaycost + (number of operations
             * to replay) * replayoperationcost
             *  
             * The default value is chosen based on the following example:
             * Assume we can replay 9 MB/s and this corresponds to 24000 ops/s.
             * replayoperationcost = (bytes to replay) * replaycost /
             * (operations to replay) = 9 MB * 8.0 / 24000 = 3000
             *  
             * The prepare for restart flush strategy will choose a set of
             * components to flush
             * such that the cost of flushing these + the cost of replaying the
             * transaction log
             * is as low as possible.
             */
            double replayoperationcost; // Default: 3000.0

            /**
             * The cost of writing a byte when flushing components to disk.
             *  
             * The number of bytes to write (for a set of flushed components) *
             * writecost
             * gives an estimate of the total cost of flushing this set of
             * components.
             *  
             * The prepare for restart flush strategy will choose a set of
             * components to flush
             * such that the cost of flushing these + the cost of replaying the
             * transaction log
             * is as low as possible.
             */
            double writecost; // Default: 1.0
        };
        typedef std::vector<Preparerestart> PreparerestartVector;

        typedef std::map<vespalib::string, Preparerestart> PreparerestartMap;

        Flush(const ::config::StringVector & __lines);
        Flush(const vespalib::slime::Inspector & __inspector);
        Flush(const ::config::ConfigPayload & __payload);
        Flush()  noexcept;
        Flush(const Flush & __rhs) noexcept;
        Flush & operator = (const Flush & __rhs) noexcept;
        Flush(Flush && __rhs) noexcept;
        Flush & operator = (Flush && __rhs) noexcept;
        ~Flush();

        bool operator==(const Flush& __rhs) const noexcept;
        bool operator!=(const Flush& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** Maximum number of concurrent flushes outstanding. */
        int32_t maxconcurrent; // Default: 2

        /**
         * Number of seconds between checking for stuff to flush when the system
         * is idling.
         */
        double idleinterval; // Default: 10.0

        /** Which flushstrategy to use. */
        Strategy strategy; // Default: MEMORY
        Memory memory;
        Preparerestart preparerestart;
    };
    typedef std::vector<Flush> FlushVector;

    typedef std::map<vespalib::string, Flush> FlushMap;

    class Indexing {
    public:
        class Write {
        public:
            enum class Io { NORMAL, OSYNC, DIRECTIO };
            typedef std::vector<Io> IoVector;
            typedef std::map<vespalib::string, Io> IoMap;
            static Io getIo(const vespalib::string&);
            static vespalib::string getIoName(Io e);

            struct InternalIoConverter {
                Io operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                Io operator()(const ::vespalib::slime::Inspector & __inspector);
                Io operator()(const ::vespalib::slime::Inspector & __inspector, Io __eDefault);
            };
            Write(const ::config::StringVector & __lines);
            Write(const vespalib::slime::Inspector & __inspector);
            Write(const ::config::ConfigPayload & __payload);
            Write()  noexcept;
            Write(const Write & __rhs) noexcept;
            Write & operator = (const Write & __rhs) noexcept;
            Write(Write && __rhs) noexcept;
            Write & operator = (Write && __rhs) noexcept;
            ~Write();

            bool operator==(const Write& __rhs) const noexcept;
            bool operator!=(const Write& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** Control io options during write both under dump and fusion. */
            Io io; // Default: DIRECTIO
        };
        typedef std::vector<Write> WriteVector;

        typedef std::map<vespalib::string, Write> WriteMap;

        class Read {
        public:
            enum class Io { NORMAL, DIRECTIO };
            typedef std::vector<Io> IoVector;
            typedef std::map<vespalib::string, Io> IoMap;
            static Io getIo(const vespalib::string&);
            static vespalib::string getIoName(Io e);

            struct InternalIoConverter {
                Io operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                Io operator()(const ::vespalib::slime::Inspector & __inspector);
                Io operator()(const ::vespalib::slime::Inspector & __inspector, Io __eDefault);
            };
            Read(const ::config::StringVector & __lines);
            Read(const vespalib::slime::Inspector & __inspector);
            Read(const ::config::ConfigPayload & __payload);
            Read()  noexcept;
            Read(const Read & __rhs) noexcept;
            Read & operator = (const Read & __rhs) noexcept;
            Read(Read && __rhs) noexcept;
            Read & operator = (Read && __rhs) noexcept;
            ~Read();

            bool operator==(const Read& __rhs) const noexcept;
            bool operator!=(const Read& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** Control io options during read both under dump and fusion. */
            Io io; // Default: DIRECTIO
        };
        typedef std::vector<Read> ReadVector;

        typedef std::map<vespalib::string, Read> ReadMap;

        enum class Optimize { LATENCY, THROUGHPUT, ADAPTIVE };
        typedef std::vector<Optimize> OptimizeVector;
        typedef std::map<vespalib::string, Optimize> OptimizeMap;
        static Optimize getOptimize(const vespalib::string&);
        static vespalib::string getOptimizeName(Optimize e);

        struct InternalOptimizeConverter {
            Optimize operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Optimize operator()(const ::vespalib::slime::Inspector & __inspector);
            Optimize operator()(const ::vespalib::slime::Inspector & __inspector, Optimize __eDefault);
        };
        Indexing(const ::config::StringVector & __lines);
        Indexing(const vespalib::slime::Inspector & __inspector);
        Indexing(const ::config::ConfigPayload & __payload);
        Indexing()  noexcept;
        Indexing(const Indexing & __rhs) noexcept;
        Indexing & operator = (const Indexing & __rhs) noexcept;
        Indexing(Indexing && __rhs) noexcept;
        Indexing & operator = (Indexing && __rhs) noexcept;
        ~Indexing();

        bool operator==(const Indexing& __rhs) const noexcept;
        bool operator!=(const Indexing& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        Write write;
        Read read;

        /**
         * Option to specify what is most important during indexing.
         * This is experimental and will most likely be temporary.
         */
        Optimize optimize; // Default: THROUGHPUT

        /**
         * Maximum number of pending operations for each of the internal
         * indexing threads. Only used when visibility delay is zero.
         */
        int32_t tasklimit; // Default: -1000

        /**
         * Kind of watermark for when to activate extra manpower
         * Utilized if optimize is set to either THROUGHPUT or ADAPTIVE
         */
        int32_t kindOfWatermark; // Default: 0

        /** Controls minimum reaction time in seconds if using THROUGHPUT */
        double reactiontime; // Default: 0.001

        // Everything below are deprecated and ignored. Will go away at any
        // time.

        /** Deprecated and ignored, will soon go away */
        int32_t semiunboundtasklimit; // Default: 1000

        /**
         * Overrides the number of threads used for writing fields across all
         * document dbs.
         * See feeding.concurrency for details.
         * DEPRECATED - Remove usage
         */
        int32_t threads; // Default: 1
    };
    typedef std::vector<Indexing> IndexingVector;

    typedef std::map<vespalib::string, Indexing> IndexingMap;

    class Index {
    public:
        class Warmup {
        public:
            Warmup(const ::config::StringVector & __lines);
            Warmup(const vespalib::slime::Inspector & __inspector);
            Warmup(const ::config::ConfigPayload & __payload);
            Warmup()  noexcept;
            Warmup(const Warmup & __rhs) noexcept;
            Warmup & operator = (const Warmup & __rhs) noexcept;
            Warmup(Warmup && __rhs) noexcept;
            Warmup & operator = (Warmup && __rhs) noexcept;
            ~Warmup();

            bool operator==(const Warmup& __rhs) const noexcept;
            bool operator!=(const Warmup& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * How long a freshly loaded index shall be warmed up
             * before being used for serving
             */
            double time; // Default: 0.0

            /**
             * Indicate if we also want warm up with full unpack, instead of
             * only cheaper seek.
             */
            bool unpack; // Default: false
        };
        typedef std::vector<Warmup> WarmupVector;

        typedef std::map<vespalib::string, Warmup> WarmupMap;

        class Cache {
        public:
            Cache(const ::config::StringVector & __lines);
            Cache(const vespalib::slime::Inspector & __inspector);
            Cache(const ::config::ConfigPayload & __payload);
            Cache()  noexcept;
            Cache(const Cache & __rhs) noexcept;
            Cache & operator = (const Cache & __rhs) noexcept;
            Cache(Cache && __rhs) noexcept;
            Cache & operator = (Cache && __rhs) noexcept;
            ~Cache();

            bool operator==(const Cache& __rhs) const noexcept;
            bool operator!=(const Cache& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * How much memory is set aside for caching.
             * Now only used for caching of dictionary lookups.
             * TODO Still relevant, check config model, seems unused.
             */
            int64_t size; // Default: 0
        };
        typedef std::vector<Cache> CacheVector;

        typedef std::map<vespalib::string, Cache> CacheMap;

        Index(const ::config::StringVector & __lines);
        Index(const vespalib::slime::Inspector & __inspector);
        Index(const ::config::ConfigPayload & __payload);
        Index()  noexcept;
        Index(const Index & __rhs) noexcept;
        Index & operator = (const Index & __rhs) noexcept;
        Index(Index && __rhs) noexcept;
        Index & operator = (Index && __rhs) noexcept;
        ~Index();

        bool operator==(const Index& __rhs) const noexcept;
        bool operator!=(const Index& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        Warmup warmup;

        /**
         * How many flushed indexes there can be before fusion is forced while
         * node is
         * not in retired state.
         * Setting to 1 will force an immediate fusion.
         */
        int32_t maxflushed; // Default: 2

        /**
         * How many flushed indexes there can be before fusion is forced while
         * node is
         * in retired state.
         * Setting to 1 will force an immediate fusion.
         */
        int32_t maxflushedretired; // Default: 20
        Cache cache;
    };
    typedef std::vector<Index> IndexVector;

    typedef std::map<vespalib::string, Index> IndexMap;

    class Attribute {
    public:
        class Write {
        public:
            enum class Io { NORMAL, OSYNC, DIRECTIO };
            typedef std::vector<Io> IoVector;
            typedef std::map<vespalib::string, Io> IoMap;
            static Io getIo(const vespalib::string&);
            static vespalib::string getIoName(Io e);

            struct InternalIoConverter {
                Io operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                Io operator()(const ::vespalib::slime::Inspector & __inspector);
                Io operator()(const ::vespalib::slime::Inspector & __inspector, Io __eDefault);
            };
            Write(const ::config::StringVector & __lines);
            Write(const vespalib::slime::Inspector & __inspector);
            Write(const ::config::ConfigPayload & __payload);
            Write()  noexcept;
            Write(const Write & __rhs) noexcept;
            Write & operator = (const Write & __rhs) noexcept;
            Write(Write && __rhs) noexcept;
            Write & operator = (Write && __rhs) noexcept;
            ~Write();

            bool operator==(const Write& __rhs) const noexcept;
            bool operator!=(const Write& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** Control io options during flushing of attributes. */
            Io io; // Default: DIRECTIO
        };
        typedef std::vector<Write> WriteVector;

        typedef std::map<vespalib::string, Write> WriteMap;

        Attribute(const ::config::StringVector & __lines);
        Attribute(const vespalib::slime::Inspector & __inspector);
        Attribute(const ::config::ConfigPayload & __payload);
        Attribute()  noexcept;
        Attribute(const Attribute & __rhs) noexcept;
        Attribute & operator = (const Attribute & __rhs) noexcept;
        Attribute(Attribute && __rhs) noexcept;
        Attribute & operator = (Attribute && __rhs) noexcept;
        ~Attribute();

        bool operator==(const Attribute& __rhs) const noexcept;
        bool operator!=(const Attribute& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        Write write;
    };
    typedef std::vector<Attribute> AttributeVector;

    typedef std::map<vespalib::string, Attribute> AttributeMap;

    class Grouping {
    public:
        class Sessionmanager {
        public:
            class Pruning {
            public:
                Pruning(const ::config::StringVector & __lines);
                Pruning(const vespalib::slime::Inspector & __inspector);
                Pruning(const ::config::ConfigPayload & __payload);
                Pruning()  noexcept;
                Pruning(const Pruning & __rhs) noexcept;
                Pruning & operator = (const Pruning & __rhs) noexcept;
                Pruning(Pruning && __rhs) noexcept;
                Pruning & operator = (Pruning && __rhs) noexcept;
                ~Pruning();

                bool operator==(const Pruning& __rhs) const noexcept;
                bool operator!=(const Pruning& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /**
                 * Control of pruning interval to remove sessions that have
                 * timed out
                 */
                double interval; // Default: 1.0
            };
            typedef std::vector<Pruning> PruningVector;

            typedef std::map<vespalib::string, Pruning> PruningMap;

            Sessionmanager(const ::config::StringVector & __lines);
            Sessionmanager(const vespalib::slime::Inspector & __inspector);
            Sessionmanager(const ::config::ConfigPayload & __payload);
            Sessionmanager()  noexcept;
            Sessionmanager(const Sessionmanager & __rhs) noexcept;
            Sessionmanager & operator = (const Sessionmanager & __rhs) noexcept;
            Sessionmanager(Sessionmanager && __rhs) noexcept;
            Sessionmanager & operator = (Sessionmanager && __rhs) noexcept;
            ~Sessionmanager();

            bool operator==(const Sessionmanager& __rhs) const noexcept;
            bool operator!=(const Sessionmanager& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** Control of grouping session manager entries */
            int32_t maxentries; // Default: 500
            Pruning pruning;
        };
        typedef std::vector<Sessionmanager> SessionmanagerVector;

        typedef std::map<vespalib::string, Sessionmanager> SessionmanagerMap;

        Grouping(const ::config::StringVector & __lines);
        Grouping(const vespalib::slime::Inspector & __inspector);
        Grouping(const ::config::ConfigPayload & __payload);
        Grouping()  noexcept;
        Grouping(const Grouping & __rhs) noexcept;
        Grouping & operator = (const Grouping & __rhs) noexcept;
        Grouping(Grouping && __rhs) noexcept;
        Grouping & operator = (Grouping && __rhs) noexcept;
        ~Grouping();

        bool operator==(const Grouping& __rhs) const noexcept;
        bool operator!=(const Grouping& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        Sessionmanager sessionmanager;
    };
    typedef std::vector<Grouping> GroupingVector;

    typedef std::map<vespalib::string, Grouping> GroupingMap;

    class Distribution {
    public:
        Distribution(const ::config::StringVector & __lines);
        Distribution(const vespalib::slime::Inspector & __inspector);
        Distribution(const ::config::ConfigPayload & __payload);
        Distribution()  noexcept;
        Distribution(const Distribution & __rhs) noexcept;
        Distribution & operator = (const Distribution & __rhs) noexcept;
        Distribution(Distribution && __rhs) noexcept;
        Distribution & operator = (Distribution && __rhs) noexcept;
        ~Distribution();

        bool operator==(const Distribution& __rhs) const noexcept;
        bool operator!=(const Distribution& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** Redundancy of documents. */
        int64_t redundancy; // Default: 1

        /** Searchable copies of the documents. */
        int64_t searchablecopies; // Default: 1
    };
    typedef std::vector<Distribution> DistributionVector;

    typedef std::map<vespalib::string, Distribution> DistributionMap;

    class Summary {
    public:
        class Cache {
        public:
            class Compression {
            public:
                enum class Type { NONE, LZ4, ZSTD };
                typedef std::vector<Type> TypeVector;
                typedef std::map<vespalib::string, Type> TypeMap;
                static Type getType(const vespalib::string&);
                static vespalib::string getTypeName(Type e);

                struct InternalTypeConverter {
                    Type operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                    Type operator()(const ::vespalib::slime::Inspector & __inspector);
                    Type operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault);
                };
                Compression(const ::config::StringVector & __lines);
                Compression(const vespalib::slime::Inspector & __inspector);
                Compression(const ::config::ConfigPayload & __payload);
                Compression()  noexcept;
                Compression(const Compression & __rhs) noexcept;
                Compression & operator = (const Compression & __rhs) noexcept;
                Compression(Compression && __rhs) noexcept;
                Compression & operator = (Compression && __rhs) noexcept;
                ~Compression();

                bool operator==(const Compression& __rhs) const noexcept;
                bool operator!=(const Compression& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /**
                 * Control compression type of the summary while in the cache.
                 */
                Type type; // Default: LZ4

                /**
                 * Control compression level of the summary while in cache.
                 * LZ4 has normal range 1..9 while ZSTD has range 1..19
                 * 6 is a default for lz4 to prioritize speed.
                 */
                int32_t level; // Default: 6
            };
            typedef std::vector<Compression> CompressionVector;

            typedef std::map<vespalib::string, Compression> CompressionMap;

            enum class UpdateStrategy { INVALIDATE, UPDATE };
            typedef std::vector<UpdateStrategy> UpdateStrategyVector;
            typedef std::map<vespalib::string, UpdateStrategy> UpdateStrategyMap;
            static UpdateStrategy getUpdateStrategy(const vespalib::string&);
            static vespalib::string getUpdateStrategyName(UpdateStrategy e);

            struct InternalUpdateStrategyConverter {
                UpdateStrategy operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                UpdateStrategy operator()(const ::vespalib::slime::Inspector & __inspector);
                UpdateStrategy operator()(const ::vespalib::slime::Inspector & __inspector, UpdateStrategy __eDefault);
            };
            Cache(const ::config::StringVector & __lines);
            Cache(const vespalib::slime::Inspector & __inspector);
            Cache(const ::config::ConfigPayload & __payload);
            Cache()  noexcept;
            Cache(const Cache & __rhs) noexcept;
            Cache & operator = (const Cache & __rhs) noexcept;
            Cache(Cache && __rhs) noexcept;
            Cache & operator = (Cache && __rhs) noexcept;
            ~Cache();

            bool operator==(const Cache& __rhs) const noexcept;
            bool operator!=(const Cache& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * Control cache size in bytes.
             * Postive numbers are absolute in bytes.
             * Negative numbers are a percentage of memory.
             */
            int64_t maxbytes; // Default: -4
            Compression compression;

            /** Control if cache entry is updated or ivalidated when changed. */
            UpdateStrategy updateStrategy; // Default: INVALIDATE

            /**
             * Include visits in the cache, if the visitoperation allows it.
             * This will enable another separate cache of summary.cache.maxbytes
             * size.
             * IGNORED and DEPRECATED Will go away soon
             */
            bool allowvisitcaching; // Default: true

            /**
             * Control number of cache entries preallocated.
             * Default is no preallocation.
             * Can be set to a higher number to avoid resizing.
             * IGNORED and DEPRECATED Will go away soon
             */
            int64_t initialentries; // Default: 0
        };
        typedef std::vector<Cache> CacheVector;

        typedef std::map<vespalib::string, Cache> CacheMap;

        class Log {
        public:
            class Compact {
            public:
                class Compression {
                public:
                    enum class Type { NONE, LZ4, ZSTD };
                    typedef std::vector<Type> TypeVector;
                    typedef std::map<vespalib::string, Type> TypeMap;
                    static Type getType(const vespalib::string&);
                    static vespalib::string getTypeName(Type e);

                    struct InternalTypeConverter {
                        Type operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                        Type operator()(const ::vespalib::slime::Inspector & __inspector);
                        Type operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault);
                    };
                    Compression(const ::config::StringVector & __lines);
                    Compression(const vespalib::slime::Inspector & __inspector);
                    Compression(const ::config::ConfigPayload & __payload);
                    Compression()  noexcept;
                    Compression(const Compression & __rhs) noexcept;
                    Compression & operator = (const Compression & __rhs) noexcept;
                    Compression(Compression && __rhs) noexcept;
                    Compression & operator = (Compression && __rhs) noexcept;
                    ~Compression();

                    bool operator==(const Compression& __rhs) const noexcept;
                    bool operator!=(const Compression& __rhs) const noexcept;

                    void serialize(vespalib::slime::Cursor & __cursor) const;

                    /**
                     * Control compression type of the summary while in memory
                     * during compaction
                     * NB So far only stragey=LOG honours it.
                     * TODO Use same as for store (chunk.compression).
                     */
                    Type type; // Default: ZSTD

                    /**
                     * Control compression level of the summary while in memory
                     * during compaction
                     * LZ4 has normal range 1..9 while ZSTD has range 1..19
                     * 9 is a reasonable default for both
                     */
                    int32_t level; // Default: 9
                };
                typedef std::vector<Compression> CompressionVector;

                typedef std::map<vespalib::string, Compression> CompressionMap;

                Compact(const ::config::StringVector & __lines);
                Compact(const vespalib::slime::Inspector & __inspector);
                Compact(const ::config::ConfigPayload & __payload);
                Compact()  noexcept;
                Compact(const Compact & __rhs) noexcept;
                Compact & operator = (const Compact & __rhs) noexcept;
                Compact(Compact && __rhs) noexcept;
                Compact & operator = (Compact && __rhs) noexcept;
                ~Compact();

                bool operator==(const Compact& __rhs) const noexcept;
                bool operator!=(const Compact& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;
                Compression compression;
            };
            typedef std::vector<Compact> CompactVector;

            typedef std::map<vespalib::string, Compact> CompactMap;

            class Chunk {
            public:
                class Compression {
                public:
                    enum class Type { NONE, LZ4, ZSTD };
                    typedef std::vector<Type> TypeVector;
                    typedef std::map<vespalib::string, Type> TypeMap;
                    static Type getType(const vespalib::string&);
                    static vespalib::string getTypeName(Type e);

                    struct InternalTypeConverter {
                        Type operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                        Type operator()(const ::vespalib::slime::Inspector & __inspector);
                        Type operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault);
                    };
                    Compression(const ::config::StringVector & __lines);
                    Compression(const vespalib::slime::Inspector & __inspector);
                    Compression(const ::config::ConfigPayload & __payload);
                    Compression()  noexcept;
                    Compression(const Compression & __rhs) noexcept;
                    Compression & operator = (const Compression & __rhs) noexcept;
                    Compression(Compression && __rhs) noexcept;
                    Compression & operator = (Compression && __rhs) noexcept;
                    ~Compression();

                    bool operator==(const Compression& __rhs) const noexcept;
                    bool operator!=(const Compression& __rhs) const noexcept;

                    void serialize(vespalib::slime::Cursor & __cursor) const;

                    /** Control compression type of the summary */
                    Type type; // Default: ZSTD

                    /**
                     * Control compression level of the summary
                     * LZ4 has normal range 1..9 while ZSTD has range 1..19
                     * 9 is a reasonable default for both. Going above for ZSTD
                     * can give an improvement,
                     * but is better done in conjunction with increasing chunk
                     * size.
                     */
                    int32_t level; // Default: 9
                };
                typedef std::vector<Compression> CompressionVector;

                typedef std::map<vespalib::string, Compression> CompressionMap;

                Chunk(const ::config::StringVector & __lines);
                Chunk(const vespalib::slime::Inspector & __inspector);
                Chunk(const ::config::ConfigPayload & __payload);
                Chunk()  noexcept;
                Chunk(const Chunk & __rhs) noexcept;
                Chunk & operator = (const Chunk & __rhs) noexcept;
                Chunk(Chunk && __rhs) noexcept;
                Chunk & operator = (Chunk && __rhs) noexcept;
                ~Chunk();

                bool operator==(const Chunk& __rhs) const noexcept;
                bool operator!=(const Chunk& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;
                Compression compression;

                /** Max size in bytes per chunk. */
                int32_t maxbytes; // Default: 65536

                /**
                 * Skip crc32 check on read.
                 * IGNORED and DEPRECATED Will go away soon
                 */
                bool skipcrconread; // Default: false
            };
            typedef std::vector<Chunk> ChunkVector;

            typedef std::map<vespalib::string, Chunk> ChunkMap;

            Log(const ::config::StringVector & __lines);
            Log(const vespalib::slime::Inspector & __inspector);
            Log(const ::config::ConfigPayload & __payload);
            Log()  noexcept;
            Log(const Log & __rhs) noexcept;
            Log & operator = (const Log & __rhs) noexcept;
            Log(Log && __rhs) noexcept;
            Log & operator = (Log && __rhs) noexcept;
            ~Log();

            bool operator==(const Log& __rhs) const noexcept;
            bool operator!=(const Log& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            Compact compact;
            Chunk chunk;

            /** Max size per summary file. */
            int64_t maxfilesize; // Default: 1000000000

            /**
             * Max number of lid entries per file
             * TODO Decide based on memory on node.
             */
            int32_t maxnumlids; // Default: 8388608

            /** Max disk bloat factor. This will trigger compacting. */
            double maxdiskbloatfactor; // Default: 0.1

            /**
             * Max bucket spread within a single summary file. This will trigger
             * bucket order compacting.
             */
            double maxbucketspread; // Default: 2.5

            /**
             * If a file goes below this ratio compared to allowed max size it
             * will be joined to the front.
             * Value in the range [0.0, 1.0]
             */
            double minfilesizefactor; // Default: 0.2
        };
        typedef std::vector<Log> LogVector;

        typedef std::map<vespalib::string, Log> LogMap;

        class Write {
        public:
            enum class Io { NORMAL, OSYNC, DIRECTIO };
            typedef std::vector<Io> IoVector;
            typedef std::map<vespalib::string, Io> IoMap;
            static Io getIo(const vespalib::string&);
            static vespalib::string getIoName(Io e);

            struct InternalIoConverter {
                Io operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                Io operator()(const ::vespalib::slime::Inspector & __inspector);
                Io operator()(const ::vespalib::slime::Inspector & __inspector, Io __eDefault);
            };
            Write(const ::config::StringVector & __lines);
            Write(const vespalib::slime::Inspector & __inspector);
            Write(const ::config::ConfigPayload & __payload);
            Write()  noexcept;
            Write(const Write & __rhs) noexcept;
            Write & operator = (const Write & __rhs) noexcept;
            Write(Write && __rhs) noexcept;
            Write & operator = (Write && __rhs) noexcept;
            ~Write();

            bool operator==(const Write& __rhs) const noexcept;
            bool operator!=(const Write& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** Control io options during flush of stored documents. */
            Io io; // Default: DIRECTIO
        };
        typedef std::vector<Write> WriteVector;

        typedef std::map<vespalib::string, Write> WriteMap;

        class Read {
        public:
            enum class Io { NORMAL, DIRECTIO, MMAP };
            typedef std::vector<Io> IoVector;
            typedef std::map<vespalib::string, Io> IoMap;
            static Io getIo(const vespalib::string&);
            static vespalib::string getIoName(Io e);

            struct InternalIoConverter {
                Io operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                Io operator()(const ::vespalib::slime::Inspector & __inspector);
                Io operator()(const ::vespalib::slime::Inspector & __inspector, Io __eDefault);
            };
            class Mmap {
            public:
                enum class Options { POPULATE, HUGETLB };
                typedef std::vector<Options> OptionsVector;
                typedef std::map<vespalib::string, Options> OptionsMap;
                static Options getOptions(const vespalib::string&);
                static vespalib::string getOptionsName(Options e);

                struct InternalOptionsConverter {
                    Options operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                    Options operator()(const ::vespalib::slime::Inspector & __inspector);
                    Options operator()(const ::vespalib::slime::Inspector & __inspector, Options __eDefault);
                };
                enum class Advise { NORMAL, RANDOM, SEQUENTIAL };
                typedef std::vector<Advise> AdviseVector;
                typedef std::map<vespalib::string, Advise> AdviseMap;
                static Advise getAdvise(const vespalib::string&);
                static vespalib::string getAdviseName(Advise e);

                struct InternalAdviseConverter {
                    Advise operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                    Advise operator()(const ::vespalib::slime::Inspector & __inspector);
                    Advise operator()(const ::vespalib::slime::Inspector & __inspector, Advise __eDefault);
                };
                Mmap(const ::config::StringVector & __lines);
                Mmap(const vespalib::slime::Inspector & __inspector);
                Mmap(const ::config::ConfigPayload & __payload);
                Mmap() ;
                Mmap(const Mmap & __rhs);
                Mmap & operator = (const Mmap & __rhs);
                Mmap(Mmap && __rhs) noexcept;
                Mmap & operator = (Mmap && __rhs) noexcept;
                ~Mmap();

                bool operator==(const Mmap& __rhs) const noexcept;
                bool operator!=(const Mmap& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /** Multiple optional options for use with mmap */
                OptionsVector options;

                /** Advise to give to os when mapping memory. */
                Advise advise; // Default: NORMAL
            };
            typedef std::vector<Mmap> MmapVector;

            typedef std::map<vespalib::string, Mmap> MmapMap;

            Read(const ::config::StringVector & __lines);
            Read(const vespalib::slime::Inspector & __inspector);
            Read(const ::config::ConfigPayload & __payload);
            Read() ;
            Read(const Read & __rhs);
            Read & operator = (const Read & __rhs);
            Read(Read && __rhs) noexcept;
            Read & operator = (Read && __rhs) noexcept;
            ~Read();

            bool operator==(const Read& __rhs) const noexcept;
            bool operator!=(const Read& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * Control io options during read of stored documents.
             * All summary.read options will take effect immediately on new
             * files written.
             * On old files it will take effect either upon compact or on
             * restart.
             * TODO Default is probably DIRECTIO
             */
            Io io; // Default: MMAP
            Mmap mmap;
        };
        typedef std::vector<Read> ReadVector;

        typedef std::map<vespalib::string, Read> ReadMap;

        Summary(const ::config::StringVector & __lines);
        Summary(const vespalib::slime::Inspector & __inspector);
        Summary(const ::config::ConfigPayload & __payload);
        Summary() ;
        Summary(const Summary & __rhs);
        Summary & operator = (const Summary & __rhs);
        Summary(Summary && __rhs) noexcept;
        Summary & operator = (Summary && __rhs) noexcept;
        ~Summary();

        bool operator==(const Summary& __rhs) const noexcept;
        bool operator!=(const Summary& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        Cache cache;
        Log log;
        Write write;
        Read read;
    };
    typedef std::vector<Summary> SummaryVector;

    typedef std::map<vespalib::string, Summary> SummaryMap;

    class Documentdb {
    public:
        enum class Mode { INDEX, STREAMING, STORE_ONLY };
        typedef std::vector<Mode> ModeVector;
        typedef std::map<vespalib::string, Mode> ModeMap;
        static Mode getMode(const vespalib::string&);
        static vespalib::string getModeName(Mode e);

        struct InternalModeConverter {
            Mode operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Mode operator()(const ::vespalib::slime::Inspector & __inspector);
            Mode operator()(const ::vespalib::slime::Inspector & __inspector, Mode __eDefault);
        };
        class Allocation {
        public:
            Allocation(const ::config::StringVector & __lines);
            Allocation(const vespalib::slime::Inspector & __inspector);
            Allocation(const ::config::ConfigPayload & __payload);
            Allocation()  noexcept;
            Allocation(const Allocation & __rhs) noexcept;
            Allocation & operator = (const Allocation & __rhs) noexcept;
            Allocation(Allocation && __rhs) noexcept;
            Allocation & operator = (Allocation && __rhs) noexcept;
            ~Allocation();

            bool operator==(const Allocation& __rhs) const noexcept;
            bool operator!=(const Allocation& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** Minimum initial size for any per document tables. */
            int64_t initialnumdocs; // Default: 1024

            /** Grow factor for any per document tables. */
            double growfactor; // Default: 0.2

            /** Constant added when growing any per document tables. */
            int32_t growbias; // Default: 1

            /** The number of documents to amortize memory spike cost over */
            int32_t amortizecount; // Default: 10000

            /**
             * The grow factor used when allocating buffers in the array store
             * used in multi-value attribute vectors to store underlying values.
             */
            double multivaluegrowfactor; // Default: 0.2

            /**
             * The ratio of used bytes that can be dead before attempting to
             * perform compaction.
             */
            double maxDeadBytesRatio; // Default: 0.05

            /**
             * The ratio of used address space that can be dead before
             * attempting to perform compaction.
             */
            double maxDeadAddressSpaceRatio; // Default: 0.2

            /**
             * Upper bound for number of buffers that can be compacted at the
             * same time in
             * a data store for a specific reason (e.g. memory used, address
             * space used).
             */
            int32_t maxCompactBuffers; // Default: 1

            /**
             * Ratio of active buffers that can be compacted at the same time in
             * a data
             * store for a specific reason (e.g. memory used, address space
             * used).
             * Effective limit is ceil(active_buffers * active_buffers_ratio).
             */
            double activeBuffersRatio; // Default: 0.1
        };
        typedef std::vector<Allocation> AllocationVector;

        typedef std::map<vespalib::string, Allocation> AllocationMap;

        Documentdb(const ::config::StringVector & __lines);
        Documentdb(const vespalib::slime::Inspector & __inspector);
        Documentdb(const ::config::ConfigPayload & __payload);
        Documentdb()  noexcept;
        Documentdb(const Documentdb & __rhs) noexcept;
        Documentdb & operator = (const Documentdb & __rhs) noexcept;
        Documentdb(Documentdb && __rhs) noexcept;
        Documentdb & operator = (Documentdb && __rhs) noexcept;
        ~Documentdb();

        bool operator==(const Documentdb& __rhs) const noexcept;
        bool operator!=(const Documentdb& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** The name of the input document type */
        vespalib::string inputdoctypename;

        /** The type of the documentdb */
        Mode mode; // Default: INDEX

        /** The configid used to subscribe to config for this database. */
        vespalib::string configid;

        /**
         * How many seconds is allowed from document is received to it is
         * visible in the index.
         */
        double visibilitydelay; // Default: 0.0

        /** Whether this document type is globally distributed or not. */
        bool global; // Default: false
        Allocation allocation;
    };
    typedef std::vector<Documentdb> DocumentdbVector;

    typedef std::map<vespalib::string, Documentdb> DocumentdbMap;

    class Periodic {
    public:
        Periodic(const ::config::StringVector & __lines);
        Periodic(const vespalib::slime::Inspector & __inspector);
        Periodic(const ::config::ConfigPayload & __payload);
        Periodic()  noexcept;
        Periodic(const Periodic & __rhs) noexcept;
        Periodic & operator = (const Periodic & __rhs) noexcept;
        Periodic(Periodic && __rhs) noexcept;
        Periodic & operator = (Periodic && __rhs) noexcept;
        ~Periodic();

        bool operator==(const Periodic& __rhs) const noexcept;
        bool operator!=(const Periodic& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** The interval of when periodic tasks should be run */
        double interval; // Default: 3600.0
    };
    typedef std::vector<Periodic> PeriodicVector;

    typedef std::map<vespalib::string, Periodic> PeriodicMap;

    class Filedistributor {
    public:
        Filedistributor(const ::config::StringVector & __lines);
        Filedistributor(const vespalib::slime::Inspector & __inspector);
        Filedistributor(const ::config::ConfigPayload & __payload);
        Filedistributor()  noexcept;
        Filedistributor(const Filedistributor & __rhs) noexcept;
        Filedistributor & operator = (const Filedistributor & __rhs) noexcept;
        Filedistributor(Filedistributor && __rhs) noexcept;
        Filedistributor & operator = (Filedistributor && __rhs) noexcept;
        ~Filedistributor();

        bool operator==(const Filedistributor& __rhs) const noexcept;
        bool operator!=(const Filedistributor& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** filedistributor rpc configuration */
        vespalib::string configid; // Default: ""
    };
    typedef std::vector<Filedistributor> FiledistributorVector;

    typedef std::map<vespalib::string, Filedistributor> FiledistributorMap;

    enum class Packetcompresstype { NONE, LZ4 };
    typedef std::vector<Packetcompresstype> PacketcompresstypeVector;
    typedef std::map<vespalib::string, Packetcompresstype> PacketcompresstypeMap;
    static Packetcompresstype getPacketcompresstype(const vespalib::string&);
    static vespalib::string getPacketcompresstypeName(Packetcompresstype e);

    struct InternalPacketcompresstypeConverter {
        Packetcompresstype operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
        Packetcompresstype operator()(const ::vespalib::slime::Inspector & __inspector);
        Packetcompresstype operator()(const ::vespalib::slime::Inspector & __inspector, Packetcompresstype __eDefault);
    };
    class Lidspacecompaction {
    public:
        Lidspacecompaction(const ::config::StringVector & __lines);
        Lidspacecompaction(const vespalib::slime::Inspector & __inspector);
        Lidspacecompaction(const ::config::ConfigPayload & __payload);
        Lidspacecompaction()  noexcept;
        Lidspacecompaction(const Lidspacecompaction & __rhs) noexcept;
        Lidspacecompaction & operator = (const Lidspacecompaction & __rhs) noexcept;
        Lidspacecompaction(Lidspacecompaction && __rhs) noexcept;
        Lidspacecompaction & operator = (Lidspacecompaction && __rhs) noexcept;
        ~Lidspacecompaction();

        bool operator==(const Lidspacecompaction& __rhs) const noexcept;
        bool operator!=(const Lidspacecompaction& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * Interval between considering if lid space compaction should be done
         * (in seconds).
         *  
         * Default value is 10 seconds.
         */
        double interval; // Default: 10.0

        /**
         * The allowed lid bloat (in docs) before considering lid space
         * compaction.
         *  
         * When considering compaction the lid bloat is calculated as
         * (docIdLimit - numDocs).
         * The lid bloat must be >= allowedlidbloat before considering
         * compaction.
         */
        int32_t allowedlidbloat; // Default: 1

        /**
         * The allowed lid bloat factor (relative) before considering lid space
         * compaction.
         *  
         * When considering compaction the lid bloat factor is calculated as
         * (docIdLimit - numDocs)/docIdLimit.
         * The lid bloat factor must be >= allowedlidbloatfactor before
         * considering compaction.
         */
        double allowedlidbloatfactor; // Default: 0.01

        /**
         * The rate (ops / second) of remove batch operations for when to block
         * lid space compaction.
         *  
         * When considering compaction, if the current observed rate of remove
         * batch operations
         * is higher than the given block rate, the lid space compaction job is
         * blocked.
         * It is considered again at the next regular interval (see above).
         *  
         * Remove batch operations are used when deleting buckets on a content
         * node.
         * This functionality ensures that during massive deleting of buckets
         * (e.g. as part of redistribution of data to a new node),
         * lid space compaction do not interfere, but instead is applied after
         * deleting of buckets is complete.
         */
        double removebatchblockrate; // Default: 0.5

        /**
         * The rate (ops / second) of remove operations for when to block lid
         * space compaction.
         *  
         * When considering compaction, if the current observed rate of remove
         * operations
         * is higher than the given block rate, the lid space compaction job is
         * blocked.
         * It is considered again at the next regular interval (see above).
         */
        double removeblockrate; // Default: 100.0
    };
    typedef std::vector<Lidspacecompaction> LidspacecompactionVector;

    typedef std::map<vespalib::string, Lidspacecompaction> LidspacecompactionMap;

    class Bucketmove {
    public:
        Bucketmove(const ::config::StringVector & __lines);
        Bucketmove(const vespalib::slime::Inspector & __inspector);
        Bucketmove(const ::config::ConfigPayload & __payload);
        Bucketmove()  noexcept;
        Bucketmove(const Bucketmove & __rhs) noexcept;
        Bucketmove & operator = (const Bucketmove & __rhs) noexcept;
        Bucketmove(Bucketmove && __rhs) noexcept;
        Bucketmove & operator = (Bucketmove && __rhs) noexcept;
        ~Bucketmove();

        bool operator==(const Bucketmove& __rhs) const noexcept;
        bool operator!=(const Bucketmove& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** Maximum docs to move in single operation per bucket */
        int32_t maxdocstomoveperbucket; // Default: 1
    };
    typedef std::vector<Bucketmove> BucketmoveVector;

    typedef std::map<vespalib::string, Bucketmove> BucketmoveMap;

    class Visit {
    public:
        Visit(const ::config::StringVector & __lines);
        Visit(const vespalib::slime::Inspector & __inspector);
        Visit(const ::config::ConfigPayload & __payload);
        Visit()  noexcept;
        Visit(const Visit & __rhs) noexcept;
        Visit & operator = (const Visit & __rhs) noexcept;
        Visit(Visit && __rhs) noexcept;
        Visit & operator = (Visit && __rhs) noexcept;
        ~Visit();

        bool operator==(const Visit& __rhs) const noexcept;
        bool operator!=(const Visit& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * You can set this to a number above zero for visit to shortcut
         * expensive serialize size computation.
         * This value will be provided instead.
         * negative number will compute it accurately.
         * This is only used for weakly consistent visiting, like streaming
         * search.
         */
        int64_t defaultserializedsize; // Default: 1

        /**
         * This will ignore the maxbytes limit advised from above.
         * This is only used for weakly consistent visiting, like streaming
         * search.
         */
        bool ignoremaxbytes; // Default: true
    };
    typedef std::vector<Visit> VisitVector;

    typedef std::map<vespalib::string, Visit> VisitMap;

    class Initialize {
    public:
        Initialize(const ::config::StringVector & __lines);
        Initialize(const vespalib::slime::Inspector & __inspector);
        Initialize(const ::config::ConfigPayload & __payload);
        Initialize()  noexcept;
        Initialize(const Initialize & __rhs) noexcept;
        Initialize & operator = (const Initialize & __rhs) noexcept;
        Initialize(Initialize && __rhs) noexcept;
        Initialize & operator = (Initialize && __rhs) noexcept;
        ~Initialize();

        bool operator==(const Initialize& __rhs) const noexcept;
        bool operator!=(const Initialize& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * Number of initializer threads used for loading structures from disk
         * at proton startup.
         * The threads are shared between document databases when value is
         * larger than 0.
         * When set to 0 (default) we use 1 separate thread per document
         * database.
         * TODO Consider if really necessary, could be automatic.
         */
        int32_t threads; // Default: 0
    };
    typedef std::vector<Initialize> InitializeVector;

    typedef std::map<vespalib::string, Initialize> InitializeMap;

    class Writefilter {
    public:
        class Attribute {
        public:
            Attribute(const ::config::StringVector & __lines);
            Attribute(const vespalib::slime::Inspector & __inspector);
            Attribute(const ::config::ConfigPayload & __payload);
            Attribute()  noexcept;
            Attribute(const Attribute & __rhs) noexcept;
            Attribute & operator = (const Attribute & __rhs) noexcept;
            Attribute(Attribute && __rhs) noexcept;
            Attribute & operator = (Attribute && __rhs) noexcept;
            ~Attribute();

            bool operator==(const Attribute& __rhs) const noexcept;
            bool operator!=(const Attribute& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * Portion of max address space used in components in attribute
             * vectors
             * before put and update operations in feed is blocked.
             */
            double addressSpaceLimit; // Default: 0.92
        };
        typedef std::vector<Attribute> AttributeVector;

        typedef std::map<vespalib::string, Attribute> AttributeMap;

        Writefilter(const ::config::StringVector & __lines);
        Writefilter(const vespalib::slime::Inspector & __inspector);
        Writefilter(const ::config::ConfigPayload & __payload);
        Writefilter()  noexcept;
        Writefilter(const Writefilter & __rhs) noexcept;
        Writefilter & operator = (const Writefilter & __rhs) noexcept;
        Writefilter(Writefilter && __rhs) noexcept;
        Writefilter & operator = (Writefilter && __rhs) noexcept;
        ~Writefilter();

        bool operator==(const Writefilter& __rhs) const noexcept;
        bool operator!=(const Writefilter& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        Attribute attribute;

        /**
         * Portion of physical memory that can be resident memory in anonymous
         * mapping
         * by the proton process before put and update portion of feed is
         * blocked.
         */
        double memorylimit; // Default: 0.8

        /**
         * Portion of space on disk partition that can be used or reserved
         * before
         * put and update portion of feed is blocked.
         */
        double disklimit; // Default: 0.8

        /**
         * Interval between sampling of disk and memory usage. Default is 10
         * seconds.
         */
        double sampleinterval; // Default: 10.0
    };
    typedef std::vector<Writefilter> WritefilterVector;

    typedef std::map<vespalib::string, Writefilter> WritefilterMap;

    class Hwinfo {
    public:
        class Disk {
        public:
            Disk(const ::config::StringVector & __lines);
            Disk(const vespalib::slime::Inspector & __inspector);
            Disk(const ::config::ConfigPayload & __payload);
            Disk()  noexcept;
            Disk(const Disk & __rhs) noexcept;
            Disk & operator = (const Disk & __rhs) noexcept;
            Disk(Disk && __rhs) noexcept;
            Disk & operator = (Disk && __rhs) noexcept;
            ~Disk();

            bool operator==(const Disk& __rhs) const noexcept;
            bool operator!=(const Disk& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * The size of the disk partition (in bytes) on which proton basedir
             * is located.
             * If set to 0, the disk size is sampled by looking at the
             * filesystem space info.
             * The disk size is used when determining if feed should be blocked
             * in writefilter.
             */
            int64_t size; // Default: 0

            /**
             * Whether the disk partition is shared among several instances of
             * proton (e.g. when using docker).
             * If shared, disk usage is sampled by doing a recursive directory
             * scan in proton basedir.
             * If not, disk usage is sampled by looking at the filesystem space
             * info.
             */
            bool shared; // Default: false

            /**
             * Override for disk write speed, measured in MiB/s. When zero, the
             * actual disk write speed is sampled by writing data to a temporary
             * file.
             */
            double writespeed; // Default: 200.0

            /**
             * Amount of data to write to temporary file when sampling disk
             * write speed.
             * Default is 1 GiB.
             * TODO Check if still in use
             */
            int64_t samplewritesize; // Default: 1073741824

            /**
             * Minimum write speed needed to avoid disk being considered slow.
             * Unit is MiB/s, default is 100.0 MiB/s.
             * TODO Check if still in use
             */
            double slowwritespeedlimit; // Default: 100.0
        };
        typedef std::vector<Disk> DiskVector;

        typedef std::map<vespalib::string, Disk> DiskMap;

        class Memory {
        public:
            Memory(const ::config::StringVector & __lines);
            Memory(const vespalib::slime::Inspector & __inspector);
            Memory(const ::config::ConfigPayload & __payload);
            Memory()  noexcept;
            Memory(const Memory & __rhs) noexcept;
            Memory & operator = (const Memory & __rhs) noexcept;
            Memory(Memory && __rhs) noexcept;
            Memory & operator = (Memory && __rhs) noexcept;
            ~Memory();

            bool operator==(const Memory& __rhs) const noexcept;
            bool operator!=(const Memory& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * The size of physical memory (in bytes) available to proton.
             * If set to 0, the memory size is sampled as _SC_PHYS_PAGES *
             * _SC_PAGESIZE by using sysconf to do the lookup.
             * The memory size is used when determining if feed should be
             * blocked in writefilter.
             */
            int64_t size; // Default: 0
        };
        typedef std::vector<Memory> MemoryVector;

        typedef std::map<vespalib::string, Memory> MemoryMap;

        class Cpu {
        public:
            Cpu(const ::config::StringVector & __lines);
            Cpu(const vespalib::slime::Inspector & __inspector);
            Cpu(const ::config::ConfigPayload & __payload);
            Cpu()  noexcept;
            Cpu(const Cpu & __rhs) noexcept;
            Cpu & operator = (const Cpu & __rhs) noexcept;
            Cpu(Cpu && __rhs) noexcept;
            Cpu & operator = (Cpu && __rhs) noexcept;
            ~Cpu();

            bool operator==(const Cpu& __rhs) const noexcept;
            bool operator!=(const Cpu& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * The number of cores on the cpu.
             * If set to 0, this is sampled by using std::thread::hardware_concu-
             * rrency().
             */
            int32_t cores; // Default: 0
        };
        typedef std::vector<Cpu> CpuVector;

        typedef std::map<vespalib::string, Cpu> CpuMap;

        Hwinfo(const ::config::StringVector & __lines);
        Hwinfo(const vespalib::slime::Inspector & __inspector);
        Hwinfo(const ::config::ConfigPayload & __payload);
        Hwinfo()  noexcept;
        Hwinfo(const Hwinfo & __rhs) noexcept;
        Hwinfo & operator = (const Hwinfo & __rhs) noexcept;
        Hwinfo(Hwinfo && __rhs) noexcept;
        Hwinfo & operator = (Hwinfo && __rhs) noexcept;
        ~Hwinfo();

        bool operator==(const Hwinfo& __rhs) const noexcept;
        bool operator!=(const Hwinfo& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        Disk disk;
        Memory memory;
        Cpu cpu;
    };
    typedef std::vector<Hwinfo> HwinfoVector;

    typedef std::map<vespalib::string, Hwinfo> HwinfoMap;

    class Feeding {
    public:
        Feeding(const ::config::StringVector & __lines);
        Feeding(const vespalib::slime::Inspector & __inspector);
        Feeding(const ::config::ConfigPayload & __payload);
        Feeding()  noexcept;
        Feeding(const Feeding & __rhs) noexcept;
        Feeding & operator = (const Feeding & __rhs) noexcept;
        Feeding(Feeding && __rhs) noexcept;
        Feeding & operator = (Feeding && __rhs) noexcept;
        ~Feeding();

        bool operator==(const Feeding& __rhs) const noexcept;
        bool operator!=(const Feeding& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * A number between 0.0 and 1.0 that specifies the concurrency when
         * handling feed operations.
         * When set to 1.0 all cores on the cpu is utilized.
         *  
         * 3 thread pools used for various aspect of feeding are configured
         * based on this setting:
         * 1) Basic shared thread pool. E.g. used for compressing and compacting
         * documents.
         * 2) Warmup thread pool. Used for disk index warmup.
         * 3) Field writer thread pool. Used for writing data to document
         * fields:
         * - Inverting index fields
         * - Writing changes to index fields
         * - Writing changes to attribute fields
         *  
         * See shared_threading_service_config.cpp for details on how the thread
         * pool sizes are calculated.
         */
        double concurrency; // Default: 0.2

        /**
         * A number between 0.0 and 1.0 telling how nice the feed and background
         * threads shall be.
         * A value of 0.0, which is default, means 'not any nicer than anyone
         * else'.
         * The scale from 0.0 to 1.0 is not linear. It is OS specific.
         */
        double niceness; // Default: 0.0

        /**
         * Maximum number of pending tasks for the master thread in each
         * document db.
         *  
         * This limit is only considered when executing tasks for handling
         * external feed operations.
         * In that case the calling thread (persistence thread) is blocked until
         * the master thread has capacity to handle more tasks.
         * When this limit is set to 0 it is ignored.
         * TODO Check if still in use
         */
        int32_t masterTaskLimit; // Default: 0
    };
    typedef std::vector<Feeding> FeedingVector;

    typedef std::map<vespalib::string, Feeding> FeedingMap;

    class Maintenancejobs {
    public:
        Maintenancejobs(const ::config::StringVector & __lines);
        Maintenancejobs(const vespalib::slime::Inspector & __inspector);
        Maintenancejobs(const ::config::ConfigPayload & __payload);
        Maintenancejobs()  noexcept;
        Maintenancejobs(const Maintenancejobs & __rhs) noexcept;
        Maintenancejobs & operator = (const Maintenancejobs & __rhs) noexcept;
        Maintenancejobs(Maintenancejobs && __rhs) noexcept;
        Maintenancejobs & operator = (Maintenancejobs && __rhs) noexcept;
        ~Maintenancejobs();

        bool operator==(const Maintenancejobs& __rhs) const noexcept;
        bool operator!=(const Maintenancejobs& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * Adjustment to resource limit when determining if maintenance jobs can
         * run.
         *  
         * Currently used by 'lid_space_compaction' and 'move_buckets' jobs.
         */
        double resourcelimitfactor; // Default: 1.05

        /**
         * The max outstanding move operations a maintenance job can have before
         * being blocked.
         *  
         * The job is unblocked (and executed again) when this goes under the
         * limit again.
         * Currently used by 'lid_space_compaction' job.
         */
        int32_t maxoutstandingmoveops; // Default: 100
    };
    typedef std::vector<Maintenancejobs> MaintenancejobsVector;

    typedef std::map<vespalib::string, Maintenancejobs> MaintenancejobsMap;

    class Bucketdb {
    public:
        enum class Checksumtype { LEGACY, XXHASH64 };
        typedef std::vector<Checksumtype> ChecksumtypeVector;
        typedef std::map<vespalib::string, Checksumtype> ChecksumtypeMap;
        static Checksumtype getChecksumtype(const vespalib::string&);
        static vespalib::string getChecksumtypeName(Checksumtype e);

        struct InternalChecksumtypeConverter {
            Checksumtype operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Checksumtype operator()(const ::vespalib::slime::Inspector & __inspector);
            Checksumtype operator()(const ::vespalib::slime::Inspector & __inspector, Checksumtype __eDefault);
        };
        Bucketdb(const ::config::StringVector & __lines);
        Bucketdb(const vespalib::slime::Inspector & __inspector);
        Bucketdb(const ::config::ConfigPayload & __payload);
        Bucketdb()  noexcept;
        Bucketdb(const Bucketdb & __rhs) noexcept;
        Bucketdb & operator = (const Bucketdb & __rhs) noexcept;
        Bucketdb(Bucketdb && __rhs) noexcept;
        Bucketdb & operator = (Bucketdb && __rhs) noexcept;
        ~Bucketdb();

        bool operator==(const Bucketdb& __rhs) const noexcept;
        bool operator!=(const Bucketdb& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * Controls the type of bucket checksum used. Do not change unless
         * in depth understanding is present.
         */
        Checksumtype checksumtype; // Default: LEGACY
    };
    typedef std::vector<Bucketdb> BucketdbVector;

    typedef std::map<vespalib::string, Bucketdb> BucketdbMap;

    class ReplayThrottlingPolicy {
    public:
        enum class Type { UNLIMITED, DYNAMIC };
        typedef std::vector<Type> TypeVector;
        typedef std::map<vespalib::string, Type> TypeMap;
        static Type getType(const vespalib::string&);
        static vespalib::string getTypeName(Type e);

        struct InternalTypeConverter {
            Type operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Type operator()(const ::vespalib::slime::Inspector & __inspector);
            Type operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault);
        };
        ReplayThrottlingPolicy(const ::config::StringVector & __lines);
        ReplayThrottlingPolicy(const vespalib::slime::Inspector & __inspector);
        ReplayThrottlingPolicy(const ::config::ConfigPayload & __payload);
        ReplayThrottlingPolicy()  noexcept;
        ReplayThrottlingPolicy(const ReplayThrottlingPolicy & __rhs) noexcept;
        ReplayThrottlingPolicy & operator = (const ReplayThrottlingPolicy & __rhs) noexcept;
        ReplayThrottlingPolicy(ReplayThrottlingPolicy && __rhs) noexcept;
        ReplayThrottlingPolicy & operator = (ReplayThrottlingPolicy && __rhs) noexcept;
        ~ReplayThrottlingPolicy();

        bool operator==(const ReplayThrottlingPolicy& __rhs) const noexcept;
        bool operator!=(const ReplayThrottlingPolicy& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * Chooses the throttling policy used to control the window size
         * of the SharedOperationThrottler component used by the transaction log
         * replay feed state.
         */
        Type type; // Default: DYNAMIC

        /**
         * Only used if replay_throttling_policy.type == DYNAMIC:
         * TODO consider just hardcoding values as they have never been tuned.
         */
        int32_t minWindowSize; // Default: 100
        int32_t maxWindowSize; // Default: 10000
        int32_t windowSizeIncrement; // Default: 20
    };
    typedef std::vector<ReplayThrottlingPolicy> ReplayThrottlingPolicyVector;

    typedef std::map<vespalib::string, ReplayThrottlingPolicy> ReplayThrottlingPolicyMap;

    enum class TensorImplementation { TENSOR_ENGINE, FAST_VALUE };
    typedef std::vector<TensorImplementation> TensorImplementationVector;
    typedef std::map<vespalib::string, TensorImplementation> TensorImplementationMap;
    static TensorImplementation getTensorImplementation(const vespalib::string&);
    static vespalib::string getTensorImplementationName(TensorImplementation e);

    struct InternalTensorImplementationConverter {
        TensorImplementation operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
        TensorImplementation operator()(const ::vespalib::slime::Inspector & __inspector);
        TensorImplementation operator()(const ::vespalib::slime::Inspector & __inspector, TensorImplementation __eDefault);
    };
    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalProtonType(const ::config::ConfigValue & __value);
    InternalProtonType(const ::config::ConfigDataBuffer & __value);
    InternalProtonType(const ::config::ConfigPayload & __payload);
    InternalProtonType() ;
    InternalProtonType(const InternalProtonType & __rhs);
    InternalProtonType & operator = (const InternalProtonType & __rhs);
    InternalProtonType(InternalProtonType && __rhs) noexcept;
    InternalProtonType & operator = (InternalProtonType && __rhs) noexcept;
    ~InternalProtonType();

    bool operator==(const InternalProtonType& __rhs) const noexcept;
    bool operator!=(const InternalProtonType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /** Base directory. The default is ignored as it is assigned by the model */
    vespalib::string basedir; // Default: "."

    /** Port to use for the rpcserver. */
    int32_t rpcport; // Default: 8004

    /** Port to use for the web server */
    int32_t httpport; // Default: 0

    /** Cluster name */
    vespalib::string clustername; // Default: ""

    /** Distribution key */
    int32_t distributionkey; // Default: -1
    Rpc rpc;
    Search search;
    Docsum docsum;

    /** Num searcher threads */
    int32_t numsearcherthreads; // Default: 64

    /** Number of threads used per search */
    int32_t numthreadspersearch; // Default: 1

    /** Num summary threads */
    int32_t numsummarythreads; // Default: 16

    /**
     * Perform extra validation of stored data on startup
     * It requires a restart to enable, but no restart to disable.
     * Hence it must always be followed by a manual restart when enabled.
     */
    ValidateAndSanitizeDocstore validateAndSanitizeDocstore; // Default: NO
    Flush flush;
    Indexing indexing;
    Index index;
    Attribute attribute;
    Grouping grouping;
    Distribution distribution;
    Summary summary;
    DocumentdbVector documentdb;
    Periodic periodic;

    /**
     * Connect spec for transactionlog server.
     * TODO Consider not using RPC at all
     */
    vespalib::string tlsspec; // Default: "tcp/localhost:13700"

    /** ConfigId for transactionlogserver */
    vespalib::string tlsconfigid; // Default: ""

    /** Slobrok configid */
    vespalib::string slobrokconfigid; // Default: ""

    /** Routing configid */
    vespalib::string routingconfigid; // Default: ""
    Filedistributor filedistributor;

    /**
     * Interval between pruning of old removed documents.
     *  
     * If set to 0 (default) the value is calculated as (pruneremoveddocumentsag-
     * e / 100) (default 3.36 hours).
     */
    double pruneremoveddocumentsinterval; // Default: 0.0

    /**
     * Age of removed document before it can be pruned.
     *  
     * Default value is 2 weeks (1209600 seconds).
     */
    double pruneremoveddocumentsage; // Default: 1209600.0

    /**
     * Minimum size of packets to compress (0 means no compression)
     *  
     */
    int32_t packetcompresslimit; // Default: 1024

    /**
     * Compression level for packets
     *  
     * Default value is 3
     */
    int32_t packetcompresslevel; // Default: 3

    /**
     * Compression type for packets
     *  
     * Default is LZ4
     */
    Packetcompresstype packetcompresstype; // Default: LZ4
    Lidspacecompaction lidspacecompaction;
    Bucketmove bucketmove;

    /**
     * This is the maximum value visibilitydelay you can have.
     * A to higher value here will cost more memory while not improving too
     * much.
     */
    double maxvisibilitydelay; // Default: 1.0
    Visit visit;
    Initialize initialize;
    Writefilter writefilter;
    Hwinfo hwinfo;
    Feeding feeding;
    Maintenancejobs maintenancejobs;
    Bucketdb bucketdb;
    ReplayThrottlingPolicy replayThrottlingPolicy;

    /**
     * Specifies which tensor implementation to use for all backend code.
     *  
     * TENSOR_ENGINE (default) uses DefaultTensorEngine, which has been the
     * production implementation for years.
     * FAST_VALUE uses the new and optimized FastValueBuilderFactory instead.
     * TODO: Remove when default has been switched to FAST_VALUE.
     */
    TensorImplementation tensorImplementation; // Default: FAST_VALUE

    /**
     * Whether to report issues back to the container via protobuf field
     * TODO Remove always on
     */
    bool forwardIssues; // Default: true
};

} // namespace internal

typedef internal::InternalProtonType ProtonConfigBuilder;
typedef const internal::InternalProtonType ProtonConfig;

} // namespace vespa::config::search::core
#endif // VESPA_config_VESPA_CONFIG_SEARCH_CORE_PROTON_H
