#include "config-rank-profiles.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalRankProfilesType::CONFIG_DEF_MD5("0fdf364a07547ac907946467ad09ef82");
const vespalib::string InternalRankProfilesType::CONFIG_DEF_NAME("rank-profiles");
const vespalib::string InternalRankProfilesType::CONFIG_DEF_NAMESPACE("vespa.config.search");
const int64_t InternalRankProfilesType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search",
"rankprofile[].name string",
"rankprofile[].fef.property[].name string",
"rankprofile[].fef.property[].value string",
"rankprofile[].normalizer[].name string",
"rankprofile[].normalizer[].input string",
"rankprofile[].normalizer[].algo enum { LINEAR, RRANK } default=LINEAR",
"rankprofile[].normalizer[].kparam double default=60.0",
};
const ::config::StringVector InternalRankProfilesType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalRankProfilesType::Rankprofile::Fef::Property::Property() noexcept
    : name(),
      value()
{
}

InternalRankProfilesType::Rankprofile::Fef::Property::Property(const Property & __rhs) noexcept = default;
InternalRankProfilesType::Rankprofile::Fef::Property & InternalRankProfilesType::Rankprofile::Fef::Property::operator = (const Property & __rhs) noexcept = default;
InternalRankProfilesType::Rankprofile::Fef::Property::Property(Property && __rhs) noexcept = default;
InternalRankProfilesType::Rankprofile::Fef::Property & InternalRankProfilesType::Rankprofile::Fef::Property::operator = (Property && __rhs) noexcept = default;
InternalRankProfilesType::Rankprofile::Fef::Property::~Property() = default; 
InternalRankProfilesType::Rankprofile::Fef::Property::Property(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    value = ConfigParser::parse<vespalib::string>("value", __lines);
    ConfigParser::stripLinesForKey("value", __remainingValuesToParse);
}

bool
InternalRankProfilesType::Rankprofile::Fef::Property::
operator==(const Property& __rhs) const noexcept
{
    return (name == __rhs.name &&
            value == __rhs.value);
}

bool
InternalRankProfilesType::Rankprofile::Fef::Property::
operator!=(const Property& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalRankProfilesType::Rankprofile::Fef::Property::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("value");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(value));
    }
}

InternalRankProfilesType::Rankprofile::Fef::Property::Property(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    value = __inspector["value"]["value"].asString().make_string();
}

InternalRankProfilesType::Rankprofile::Fef::Property::Property(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    value = ::config::internal::ValueConverter<vespalib::string>()("value", __inspector["value"]);
}

InternalRankProfilesType::Rankprofile::Fef::Fef()
    : property()
{
}

InternalRankProfilesType::Rankprofile::Fef::Fef(const Fef & __rhs) = default;
InternalRankProfilesType::Rankprofile::Fef & InternalRankProfilesType::Rankprofile::Fef::operator = (const Fef & __rhs) = default;
InternalRankProfilesType::Rankprofile::Fef::Fef(Fef && __rhs) noexcept = default;
InternalRankProfilesType::Rankprofile::Fef & InternalRankProfilesType::Rankprofile::Fef::operator = (Fef && __rhs) noexcept = default;
InternalRankProfilesType::Rankprofile::Fef::~Fef() = default; 
InternalRankProfilesType::Rankprofile::Fef::Fef(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    property = ConfigParser::parseArray<PropertyVector>("property", __lines);
    ConfigParser::stripLinesForKey("property", __remainingValuesToParse);
}

bool
InternalRankProfilesType::Rankprofile::Fef::operator==(const Fef& __rhs) const noexcept
{
    return (property == __rhs.property);
}

bool
InternalRankProfilesType::Rankprofile::Fef::operator!=(const Fef& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalRankProfilesType::Rankprofile::Fef::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("property");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : property) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalRankProfilesType::Rankprofile::Fef::Fef(const vespalib::slime::Inspector & __inspector)
{
    for (size_t __i = 0; __i < __inspector["property"]["value"].children(); __i++) {
        property.push_back(Property(__inspector["property"]["value"][__i]["value"]));
    }
}

InternalRankProfilesType::Rankprofile::Fef::Fef(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<PropertyVector> __propertyInserter(property);
    __inspector["property"].traverse(__propertyInserter);
}

InternalRankProfilesType::Rankprofile::Normalizer::Algo
InternalRankProfilesType::Rankprofile::Normalizer::getAlgo(const vespalib::string& name)
{
    if (name == "LINEAR") {
        return Algo::LINEAR;
    } else if (name == "RRANK") {
        return Algo::RRANK;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalRankProfilesType::Rankprofile::Normalizer::getAlgoName(Algo t)
{
    switch (t) {
        case Algo::LINEAR: return "LINEAR";
        case Algo::RRANK: return "RRANK";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalRankProfilesType::Rankprofile::Normalizer::Algo InternalRankProfilesType::Rankprofile::Normalizer::InternalAlgoConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalRankProfilesType::Rankprofile::Normalizer::getAlgo(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalRankProfilesType::Rankprofile::Normalizer::Algo InternalRankProfilesType::Rankprofile::Normalizer::InternalAlgoConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalRankProfilesType::Rankprofile::Normalizer::getAlgo(__inspector.asString().make_string());
}
InternalRankProfilesType::Rankprofile::Normalizer::Algo InternalRankProfilesType::Rankprofile::Normalizer::InternalAlgoConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Algo __eDefault) {
    if (__inspector.valid()) {
        return InternalRankProfilesType::Rankprofile::Normalizer::getAlgo(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalRankProfilesType::Rankprofile::Normalizer::Normalizer() noexcept
    : name(),
      input(),
      algo(Algo::LINEAR),
      kparam(60.0)
{
}

InternalRankProfilesType::Rankprofile::Normalizer::Normalizer(const Normalizer & __rhs) noexcept = default;
InternalRankProfilesType::Rankprofile::Normalizer & InternalRankProfilesType::Rankprofile::Normalizer::operator = (const Normalizer & __rhs) noexcept = default;
InternalRankProfilesType::Rankprofile::Normalizer::Normalizer(Normalizer && __rhs) noexcept = default;
InternalRankProfilesType::Rankprofile::Normalizer & InternalRankProfilesType::Rankprofile::Normalizer::operator = (Normalizer && __rhs) noexcept = default;
InternalRankProfilesType::Rankprofile::Normalizer::~Normalizer() = default; 
InternalRankProfilesType::Rankprofile::Normalizer::Normalizer(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    input = ConfigParser::parse<vespalib::string>("input", __lines);
    ConfigParser::stripLinesForKey("input", __remainingValuesToParse);
    algo = getAlgo(ConfigParser::parse<vespalib::string>("algo", __lines, "LINEAR"));
    ConfigParser::stripLinesForKey("algo", __remainingValuesToParse);
    kparam = ConfigParser::parse<double>("kparam", __lines, 60.0);
    ConfigParser::stripLinesForKey("kparam", __remainingValuesToParse);
}

bool
InternalRankProfilesType::Rankprofile::Normalizer::
operator==(const Normalizer& __rhs) const noexcept
{
    return (name == __rhs.name &&
            input == __rhs.input &&
            algo == __rhs.algo &&
            kparam == __rhs.kparam);
}

bool
InternalRankProfilesType::Rankprofile::Normalizer::
operator!=(const Normalizer& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalRankProfilesType::Rankprofile::Normalizer::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("input");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(input));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("algo");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getAlgoName(algo)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("kparam");
        __c.setString("type", "double");
        __c.setDouble("value", kparam);
    }
}

InternalRankProfilesType::Rankprofile::Normalizer::Normalizer(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    input = __inspector["input"]["value"].asString().make_string();
    algo = getAlgo(__inspector["algo"]["value"].asString().make_string());
    kparam = __inspector["kparam"]["value"].asDouble();
}

InternalRankProfilesType::Rankprofile::Normalizer::Normalizer(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    input = ::config::internal::ValueConverter<vespalib::string>()("input", __inspector["input"]);
    algo = InternalAlgoConverter()(__inspector["algo"], Algo::LINEAR);
    kparam = ::config::internal::ValueConverter<double>()(__inspector["kparam"], 60.0);
}

InternalRankProfilesType::Rankprofile::Rankprofile()
    : name(),
      fef(),
      normalizer()
{
}

InternalRankProfilesType::Rankprofile::Rankprofile(const Rankprofile & __rhs) = default;
InternalRankProfilesType::Rankprofile & InternalRankProfilesType::Rankprofile::operator = (const Rankprofile & __rhs) = default;
InternalRankProfilesType::Rankprofile::Rankprofile(Rankprofile && __rhs) noexcept = default;
InternalRankProfilesType::Rankprofile & InternalRankProfilesType::Rankprofile::operator = (Rankprofile && __rhs) noexcept = default;
InternalRankProfilesType::Rankprofile::~Rankprofile() = default; 
InternalRankProfilesType::Rankprofile::Rankprofile(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    fef = ConfigParser::parseStruct<Fef>("fef", __lines);
    ConfigParser::stripLinesForKey("fef", __remainingValuesToParse);
    normalizer = ConfigParser::parseArray<NormalizerVector>("normalizer", __lines);
    ConfigParser::stripLinesForKey("normalizer", __remainingValuesToParse);
}

bool
InternalRankProfilesType::Rankprofile::
operator==(const Rankprofile& __rhs) const noexcept
{
    return (name == __rhs.name &&
            fef == __rhs.fef &&
            normalizer == __rhs.normalizer);
}

bool
InternalRankProfilesType::Rankprofile::
operator!=(const Rankprofile& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalRankProfilesType::Rankprofile::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fef");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        fef.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("normalizer");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : normalizer) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalRankProfilesType::Rankprofile::Rankprofile(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    fef = Fef(__inspector["fef"]["value"]);
    for (size_t __i = 0; __i < __inspector["normalizer"]["value"].children(); __i++) {
        normalizer.push_back(Normalizer(__inspector["normalizer"]["value"][__i]["value"]));
    }
}

InternalRankProfilesType::Rankprofile::Rankprofile(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    fef = ::config::internal::ValueConverter<Fef>()(__inspector["fef"]);
    ::config::internal::VectorInserter<NormalizerVector> __normalizerInserter(normalizer);
    __inspector["normalizer"].traverse(__normalizerInserter);
}

InternalRankProfilesType::InternalRankProfilesType()
    : rankprofile()
{
}

InternalRankProfilesType::InternalRankProfilesType(const InternalRankProfilesType & __rhs) = default;
InternalRankProfilesType & InternalRankProfilesType::operator =(const InternalRankProfilesType & __rhs) = default;
InternalRankProfilesType::InternalRankProfilesType(InternalRankProfilesType && __rhs) noexcept = default;
InternalRankProfilesType & InternalRankProfilesType::operator =(InternalRankProfilesType && __rhs) noexcept = default;
InternalRankProfilesType::~InternalRankProfilesType() = default; 
InternalRankProfilesType::InternalRankProfilesType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        rankprofile = ConfigParser::parseArray<RankprofileVector>("rankprofile", __lines);
        ConfigParser::stripLinesForKey("rankprofile", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalRankProfilesType::
operator==(const InternalRankProfilesType& __rhs) const noexcept
{
    return (rankprofile == __rhs.rankprofile);
}

bool
InternalRankProfilesType::
operator!=(const InternalRankProfilesType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalRankProfilesType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("rankprofile");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : rankprofile) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalRankProfilesType::InternalRankProfilesType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["rankprofile"]["value"].children(); __i++) {
        rankprofile.push_back(Rankprofile(__inspector["rankprofile"]["value"][__i]["value"]));
    }
}

InternalRankProfilesType::InternalRankProfilesType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<RankprofileVector> __rankprofileInserter(rankprofile);
    __inspector["rankprofile"].traverse(__rankprofileInserter);
}

} // namespace internal

} // namespace vespa::config::search
