/**
 * @class vespa::config::search::RankProfiles
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_SEARCH_RANKPROFILES_H
#define CLOUD_CONFIG_VESPA_CONFIG_SEARCH_RANKPROFILES_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::search {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalRankProfilesType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalRankProfilesType> UP;
    class Rankprofile {
    public:
        class Fef {
        public:
            class Property {
            public:
                Property(const ::config::StringVector & __lines);
                Property(const vespalib::slime::Inspector & __inspector);
                Property(const ::config::ConfigPayload & __payload);
                Property()  noexcept;
                Property(const Property & __rhs) noexcept;
                Property & operator = (const Property & __rhs) noexcept;
                Property(Property && __rhs) noexcept;
                Property & operator = (Property && __rhs) noexcept;
                ~Property();

                bool operator==(const Property& __rhs) const noexcept;
                bool operator!=(const Property& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /**
                 * the name of a generic property available to the feature
                 * execution framework and feature plugins
                 */
                vespalib::string name;

                /**
                 * the value of a generic property available to feature plugins
                 */
                vespalib::string value;
            };
            typedef std::vector<Property> PropertyVector;

            typedef std::map<vespalib::string, Property> PropertyMap;

            Fef(const ::config::StringVector & __lines);
            Fef(const vespalib::slime::Inspector & __inspector);
            Fef(const ::config::ConfigPayload & __payload);
            Fef() ;
            Fef(const Fef & __rhs);
            Fef & operator = (const Fef & __rhs);
            Fef(Fef && __rhs) noexcept;
            Fef & operator = (Fef && __rhs) noexcept;
            ~Fef();

            bool operator==(const Fef& __rhs) const noexcept;
            bool operator!=(const Fef& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            PropertyVector property;
        };
        typedef std::vector<Fef> FefVector;

        typedef std::map<vespalib::string, Fef> FefMap;

        class Normalizer {
        public:
            enum class Algo { LINEAR, RRANK };
            typedef std::vector<Algo> AlgoVector;
            typedef std::map<vespalib::string, Algo> AlgoMap;
            static Algo getAlgo(const vespalib::string&);
            static vespalib::string getAlgoName(Algo e);

            struct InternalAlgoConverter {
                Algo operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                Algo operator()(const ::vespalib::slime::Inspector & __inspector);
                Algo operator()(const ::vespalib::slime::Inspector & __inspector, Algo __eDefault);
            };
            Normalizer(const ::config::StringVector & __lines);
            Normalizer(const vespalib::slime::Inspector & __inspector);
            Normalizer(const ::config::ConfigPayload & __payload);
            Normalizer()  noexcept;
            Normalizer(const Normalizer & __rhs) noexcept;
            Normalizer & operator = (const Normalizer & __rhs) noexcept;
            Normalizer(Normalizer && __rhs) noexcept;
            Normalizer & operator = (Normalizer && __rhs) noexcept;
            ~Normalizer();

            bool operator==(const Normalizer& __rhs) const noexcept;
            bool operator!=(const Normalizer& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** output from cross-hits normalizing function */
            vespalib::string name;

            /** input to cross-hits normalizing function */
            vespalib::string input;

            /** type of cross-hits normalizing function */
            Algo algo; // Default: LINEAR

            /** extra "k" param (if applicable) */
            double kparam; // Default: 60.0
        };
        typedef std::vector<Normalizer> NormalizerVector;

        typedef std::map<vespalib::string, Normalizer> NormalizerMap;

        Rankprofile(const ::config::StringVector & __lines);
        Rankprofile(const vespalib::slime::Inspector & __inspector);
        Rankprofile(const ::config::ConfigPayload & __payload);
        Rankprofile() ;
        Rankprofile(const Rankprofile & __rhs);
        Rankprofile & operator = (const Rankprofile & __rhs);
        Rankprofile(Rankprofile && __rhs) noexcept;
        Rankprofile & operator = (Rankprofile && __rhs) noexcept;
        ~Rankprofile();

        bool operator==(const Rankprofile& __rhs) const noexcept;
        bool operator!=(const Rankprofile& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** name of this rank profile. maps to table index for internal use. */
        vespalib::string name;
        Fef fef;
        NormalizerVector normalizer;
    };
    typedef std::vector<Rankprofile> RankprofileVector;

    typedef std::map<vespalib::string, Rankprofile> RankprofileMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalRankProfilesType(const ::config::ConfigValue & __value);
    InternalRankProfilesType(const ::config::ConfigDataBuffer & __value);
    InternalRankProfilesType(const ::config::ConfigPayload & __payload);
    InternalRankProfilesType() ;
    InternalRankProfilesType(const InternalRankProfilesType & __rhs);
    InternalRankProfilesType & operator = (const InternalRankProfilesType & __rhs);
    InternalRankProfilesType(InternalRankProfilesType && __rhs) noexcept;
    InternalRankProfilesType & operator = (InternalRankProfilesType && __rhs) noexcept;
    ~InternalRankProfilesType();

    bool operator==(const InternalRankProfilesType& __rhs) const noexcept;
    bool operator!=(const InternalRankProfilesType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    RankprofileVector rankprofile;
};

} // namespace internal

typedef internal::InternalRankProfilesType RankProfilesConfigBuilder;
typedef const internal::InternalRankProfilesType RankProfilesConfig;

} // namespace vespa::config::search
#endif // VESPA_config_VESPA_CONFIG_SEARCH_RANKPROFILES_H
