#include "config-ranking-constants.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalRankingConstantsType::CONFIG_DEF_MD5("c285ed1fda604d6dd53e814bae81a008");
const vespalib::string InternalRankingConstantsType::CONFIG_DEF_NAME("ranking-constants");
const vespalib::string InternalRankingConstantsType::CONFIG_DEF_NAMESPACE("vespa.config.search.core");
const int64_t InternalRankingConstantsType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search.core",
"constant[].name string",
"constant[].fileref file",
"constant[].type string",
};
const ::config::StringVector InternalRankingConstantsType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalRankingConstantsType::Constant::Constant() noexcept
    : name(),
      fileref(),
      type()
{
}

InternalRankingConstantsType::Constant::Constant(const Constant & __rhs) noexcept = default;
InternalRankingConstantsType::Constant & InternalRankingConstantsType::Constant::operator = (const Constant & __rhs) noexcept = default;
InternalRankingConstantsType::Constant::Constant(Constant && __rhs) noexcept = default;
InternalRankingConstantsType::Constant & InternalRankingConstantsType::Constant::operator = (Constant && __rhs) noexcept = default;
InternalRankingConstantsType::Constant::~Constant() = default; 
InternalRankingConstantsType::Constant::Constant(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    fileref = ConfigParser::parse<vespalib::string>("fileref", __lines);
    ConfigParser::stripLinesForKey("fileref", __remainingValuesToParse);
    type = ConfigParser::parse<vespalib::string>("type", __lines);
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
}

bool
InternalRankingConstantsType::Constant::operator==(const Constant& __rhs) const noexcept
{
    return (name == __rhs.name &&
            fileref == __rhs.fileref &&
            type == __rhs.type);
}

bool
InternalRankingConstantsType::Constant::operator!=(const Constant& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalRankingConstantsType::Constant::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fileref");
        __c.setString("type", "file");
        __c.setString("value", vespalib::Memory(fileref));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(type));
    }
}

InternalRankingConstantsType::Constant::Constant(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    fileref = __inspector["fileref"]["value"].asString().make_string();
    type = __inspector["type"]["value"].asString().make_string();
}

InternalRankingConstantsType::Constant::Constant(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    fileref = ::config::internal::ValueConverter<vespalib::string>()("fileref", __inspector["fileref"]);
    type = ::config::internal::ValueConverter<vespalib::string>()("type", __inspector["type"]);
}

InternalRankingConstantsType::InternalRankingConstantsType()
    : constant()
{
}

InternalRankingConstantsType::InternalRankingConstantsType(const InternalRankingConstantsType & __rhs) = default;
InternalRankingConstantsType & InternalRankingConstantsType::operator =(const InternalRankingConstantsType & __rhs) = default;
InternalRankingConstantsType::InternalRankingConstantsType(InternalRankingConstantsType && __rhs) noexcept = default;
InternalRankingConstantsType & InternalRankingConstantsType::operator =(InternalRankingConstantsType && __rhs) noexcept = default;
InternalRankingConstantsType::~InternalRankingConstantsType() = default; 
InternalRankingConstantsType::InternalRankingConstantsType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        constant = ConfigParser::parseArray<ConstantVector>("constant", __lines);
        ConfigParser::stripLinesForKey("constant", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalRankingConstantsType::
operator==(const InternalRankingConstantsType& __rhs) const noexcept
{
    return (constant == __rhs.constant);
}

bool
InternalRankingConstantsType::
operator!=(const InternalRankingConstantsType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalRankingConstantsType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("constant");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : constant) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalRankingConstantsType::InternalRankingConstantsType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["constant"]["value"].children(); __i++) {
        constant.push_back(Constant(__inspector["constant"]["value"][__i]["value"]));
    }
}

InternalRankingConstantsType::InternalRankingConstantsType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<ConstantVector> __constantInserter(constant);
    __inspector["constant"].traverse(__constantInserter);
}

} // namespace internal

} // namespace vespa::config::search::core
