#include "config-ranking-expressions.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalRankingExpressionsType::CONFIG_DEF_MD5("ff17c6b7529b1bccd49409790771212b");
const vespalib::string InternalRankingExpressionsType::CONFIG_DEF_NAME("ranking-expressions");
const vespalib::string InternalRankingExpressionsType::CONFIG_DEF_NAMESPACE("vespa.config.search.core");
const int64_t InternalRankingExpressionsType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search.core",
"expression[].name string",
"expression[].fileref file",
};
const ::config::StringVector InternalRankingExpressionsType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalRankingExpressionsType::Expression::Expression() noexcept
    : name(),
      fileref()
{
}

InternalRankingExpressionsType::Expression::Expression(const Expression & __rhs) noexcept = default;
InternalRankingExpressionsType::Expression & InternalRankingExpressionsType::Expression::operator = (const Expression & __rhs) noexcept = default;
InternalRankingExpressionsType::Expression::Expression(Expression && __rhs) noexcept = default;
InternalRankingExpressionsType::Expression & InternalRankingExpressionsType::Expression::operator = (Expression && __rhs) noexcept = default;
InternalRankingExpressionsType::Expression::~Expression() = default; 
InternalRankingExpressionsType::Expression::Expression(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    fileref = ConfigParser::parse<vespalib::string>("fileref", __lines);
    ConfigParser::stripLinesForKey("fileref", __remainingValuesToParse);
}

bool
InternalRankingExpressionsType::Expression::
operator==(const Expression& __rhs) const noexcept
{
    return (name == __rhs.name &&
            fileref == __rhs.fileref);
}

bool
InternalRankingExpressionsType::Expression::
operator!=(const Expression& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalRankingExpressionsType::Expression::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fileref");
        __c.setString("type", "file");
        __c.setString("value", vespalib::Memory(fileref));
    }
}

InternalRankingExpressionsType::Expression::Expression(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    fileref = __inspector["fileref"]["value"].asString().make_string();
}

InternalRankingExpressionsType::Expression::Expression(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    fileref = ::config::internal::ValueConverter<vespalib::string>()("fileref", __inspector["fileref"]);
}

InternalRankingExpressionsType::InternalRankingExpressionsType()
    : expression()
{
}

InternalRankingExpressionsType::InternalRankingExpressionsType(const InternalRankingExpressionsType & __rhs) = default;
InternalRankingExpressionsType & InternalRankingExpressionsType::operator =(const InternalRankingExpressionsType & __rhs) = default;
InternalRankingExpressionsType::InternalRankingExpressionsType(InternalRankingExpressionsType && __rhs) noexcept = default;
InternalRankingExpressionsType & InternalRankingExpressionsType::operator =(InternalRankingExpressionsType && __rhs) noexcept = default;
InternalRankingExpressionsType::~InternalRankingExpressionsType() = default; 
InternalRankingExpressionsType::InternalRankingExpressionsType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        expression = ConfigParser::parseArray<ExpressionVector>("expression", __lines);
        ConfigParser::stripLinesForKey("expression", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalRankingExpressionsType::
operator==(const InternalRankingExpressionsType& __rhs) const noexcept
{
    return (expression == __rhs.expression);
}

bool
InternalRankingExpressionsType::
operator!=(const InternalRankingExpressionsType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalRankingExpressionsType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("expression");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : expression) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalRankingExpressionsType::InternalRankingExpressionsType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["expression"]["value"].children(); __i++) {
        expression.push_back(Expression(__inspector["expression"]["value"][__i]["value"]));
    }
}

InternalRankingExpressionsType::InternalRankingExpressionsType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<ExpressionVector> __expressionInserter(expression);
    __inspector["expression"].traverse(__expressionInserter);
}

} // namespace internal

} // namespace vespa::config::search::core
