#include "config-reindexing.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content::reindexing {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalReindexingType::CONFIG_DEF_MD5("163d32a177d2bad9278217cbb6c0d042");
const vespalib::string InternalReindexingType::CONFIG_DEF_NAME("reindexing");
const vespalib::string InternalReindexingType::CONFIG_DEF_NAMESPACE("vespa.config.content.reindexing");
const int64_t InternalReindexingType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content.reindexing",
"enabled bool default=false",
"clusters{}.documentTypes{}.readyAtMillis long",
"clusters{}.documentTypes{}.speed double default=0.2",
};
const ::config::StringVector InternalReindexingType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalReindexingType::Clusters::DocumentTypes::DocumentTypes() noexcept
    : readyAtMillis(),
      speed(0.2)
{
}

InternalReindexingType::Clusters::DocumentTypes::DocumentTypes(const DocumentTypes & __rhs) noexcept = default;
InternalReindexingType::Clusters::DocumentTypes & InternalReindexingType::Clusters::DocumentTypes::operator = (const DocumentTypes & __rhs) noexcept = default;
InternalReindexingType::Clusters::DocumentTypes::DocumentTypes(DocumentTypes && __rhs) noexcept = default;
InternalReindexingType::Clusters::DocumentTypes & InternalReindexingType::Clusters::DocumentTypes::operator = (DocumentTypes && __rhs) noexcept = default;
InternalReindexingType::Clusters::DocumentTypes::~DocumentTypes() = default; 
InternalReindexingType::Clusters::DocumentTypes::DocumentTypes(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    readyAtMillis = ConfigParser::parse<int64_t>("readyAtMillis", __lines);
    ConfigParser::stripLinesForKey("readyAtMillis", __remainingValuesToParse);
    speed = ConfigParser::parse<double>("speed", __lines, 0.2);
    ConfigParser::stripLinesForKey("speed", __remainingValuesToParse);
}

bool
InternalReindexingType::Clusters::DocumentTypes::
operator==(const DocumentTypes& __rhs) const noexcept
{
    return (readyAtMillis == __rhs.readyAtMillis &&
            speed == __rhs.speed);
}

bool
InternalReindexingType::Clusters::DocumentTypes::
operator!=(const DocumentTypes& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalReindexingType::Clusters::DocumentTypes::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("readyAtMillis");
        __c.setString("type", "long");
        __c.setLong("value", readyAtMillis);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("speed");
        __c.setString("type", "double");
        __c.setDouble("value", speed);
    }
}

InternalReindexingType::Clusters::DocumentTypes::DocumentTypes(const vespalib::slime::Inspector & __inspector)
{
    readyAtMillis = __inspector["readyAtMillis"]["value"].asLong();
    speed = __inspector["speed"]["value"].asDouble();
}

InternalReindexingType::Clusters::DocumentTypes::DocumentTypes(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    readyAtMillis = ::config::internal::ValueConverter<int64_t>()("readyAtMillis", __inspector["readyAtMillis"]);
    speed = ::config::internal::ValueConverter<double>()(__inspector["speed"], 0.2);
}

InternalReindexingType::Clusters::Clusters()
    : documentTypes()
{
}

InternalReindexingType::Clusters::Clusters(const Clusters & __rhs) = default;
InternalReindexingType::Clusters & InternalReindexingType::Clusters::operator = (const Clusters & __rhs) = default;
InternalReindexingType::Clusters::Clusters(Clusters && __rhs) noexcept = default;
InternalReindexingType::Clusters & InternalReindexingType::Clusters::operator = (Clusters && __rhs) noexcept = default;
InternalReindexingType::Clusters::~Clusters() = default; 
InternalReindexingType::Clusters::Clusters(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    documentTypes = ConfigParser::parseMap<DocumentTypes>("documentTypes", __lines);
    ConfigParser::stripLinesForKey("documentTypes", __remainingValuesToParse);
}

bool
InternalReindexingType::Clusters::operator==(const Clusters& __rhs) const noexcept
{
    return (documentTypes == __rhs.documentTypes);
}

bool
InternalReindexingType::Clusters::operator!=(const Clusters& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalReindexingType::Clusters::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("documentTypes");
        __c.setString("type", "map");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & entry : documentTypes) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("key", vespalib::Memory(entry.first));
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            entry.second.serialize(__c4);
        }
    }
}

InternalReindexingType::Clusters::Clusters(const vespalib::slime::Inspector & __inspector)
{
    for (size_t __i = 0; __i < __inspector["documentTypes"]["value"].children(); __i++) {
        documentTypes[__inspector["documentTypes"]["value"][__i]["key"].asString().make_string()] = DocumentTypes(__inspector["documentTypes"]["value"][__i]["value"]);
    }
}

InternalReindexingType::Clusters::Clusters(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::MapInserter<DocumentTypes> __documentTypesInserter(documentTypes);
    __inspector["documentTypes"].traverse(__documentTypesInserter);
}

InternalReindexingType::InternalReindexingType()
    : enabled(false),
      clusters()
{
}

InternalReindexingType::InternalReindexingType(const InternalReindexingType & __rhs) = default;
InternalReindexingType & InternalReindexingType::operator =(const InternalReindexingType & __rhs) = default;
InternalReindexingType::InternalReindexingType(InternalReindexingType && __rhs) noexcept = default;
InternalReindexingType & InternalReindexingType::operator =(InternalReindexingType && __rhs) noexcept = default;
InternalReindexingType::~InternalReindexingType() = default; 
InternalReindexingType::InternalReindexingType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        enabled = ConfigParser::parse<bool>("enabled", __lines, false);
        ConfigParser::stripLinesForKey("enabled", __remainingValuesToParse);
        clusters = ConfigParser::parseMap<Clusters>("clusters", __lines);
        ConfigParser::stripLinesForKey("clusters", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalReindexingType::operator==(const InternalReindexingType& __rhs) const noexcept
{
    return (enabled == __rhs.enabled &&
            clusters == __rhs.clusters);
}

bool
InternalReindexingType::operator!=(const InternalReindexingType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalReindexingType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enabled");
        __c.setString("type", "bool");
        __c.setBool("value", enabled);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("clusters");
        __c.setString("type", "map");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & entry : clusters) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("key", vespalib::Memory(entry.first));
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            entry.second.serialize(__c4);
        }
    }
}

InternalReindexingType::InternalReindexingType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    enabled = __inspector["enabled"]["value"].asBool();
    for (size_t __i = 0; __i < __inspector["clusters"]["value"].children(); __i++) {
        clusters[__inspector["clusters"]["value"][__i]["key"].asString().make_string()] = Clusters(__inspector["clusters"]["value"][__i]["value"]);
    }
}

InternalReindexingType::InternalReindexingType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    enabled = ::config::internal::ValueConverter<bool>()(__inspector["enabled"], false);
    ::config::internal::MapInserter<Clusters> __clustersInserter(clusters);
    __inspector["clusters"].traverse(__clustersInserter);
}

} // namespace internal

} // namespace vespa::config::content::reindexing
