/**
 * @class vespa::config::content::reindexing::Reindexing
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 * Reindexing status per document type, for a Vespa application
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_CONTENT_REINDEXING_REINDEXING_H
#define CLOUD_CONFIG_VESPA_CONFIG_CONTENT_REINDEXING_REINDEXING_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::content::reindexing {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalReindexingType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalReindexingType> UP;
    class Clusters {
    public:
        class DocumentTypes {
        public:
            DocumentTypes(const ::config::StringVector & __lines);
            DocumentTypes(const vespalib::slime::Inspector & __inspector);
            DocumentTypes(const ::config::ConfigPayload & __payload);
            DocumentTypes()  noexcept;
            DocumentTypes(const DocumentTypes & __rhs) noexcept;
            DocumentTypes & operator = (const DocumentTypes & __rhs) noexcept;
            DocumentTypes(DocumentTypes && __rhs) noexcept;
            DocumentTypes & operator = (DocumentTypes && __rhs) noexcept;
            ~DocumentTypes();

            bool operator==(const DocumentTypes& __rhs) const noexcept;
            bool operator!=(const DocumentTypes& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * Epoch millis after which latest reprocessing may begin, per
             * document type, per cluster
             */
            int64_t readyAtMillis;

            /**
             * Window size increment used for the dynamic throttling policy of
             * the reindexing visitor session for each type
             */
            double speed; // Default: 0.2
        };
        typedef std::vector<DocumentTypes> DocumentTypesVector;

        typedef std::map<vespalib::string, DocumentTypes> DocumentTypesMap;

        Clusters(const ::config::StringVector & __lines);
        Clusters(const vespalib::slime::Inspector & __inspector);
        Clusters(const ::config::ConfigPayload & __payload);
        Clusters() ;
        Clusters(const Clusters & __rhs);
        Clusters & operator = (const Clusters & __rhs);
        Clusters(Clusters && __rhs) noexcept;
        Clusters & operator = (Clusters && __rhs) noexcept;
        ~Clusters();

        bool operator==(const Clusters& __rhs) const noexcept;
        bool operator!=(const Clusters& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        DocumentTypesMap documentTypes;
    };
    typedef std::vector<Clusters> ClustersVector;

    typedef std::map<vespalib::string, Clusters> ClustersMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalReindexingType(const ::config::ConfigValue & __value);
    InternalReindexingType(const ::config::ConfigDataBuffer & __value);
    InternalReindexingType(const ::config::ConfigPayload & __payload);
    InternalReindexingType() ;
    InternalReindexingType(const InternalReindexingType & __rhs);
    InternalReindexingType & operator = (const InternalReindexingType & __rhs);
    InternalReindexingType(InternalReindexingType && __rhs) noexcept;
    InternalReindexingType & operator = (InternalReindexingType && __rhs) noexcept;
    ~InternalReindexingType();

    bool operator==(const InternalReindexingType& __rhs) const noexcept;
    bool operator!=(const InternalReindexingType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /** Whether reindexing should run at all */
    bool enabled; // Default: false
    ClustersMap clusters;
};

} // namespace internal

typedef internal::InternalReindexingType ReindexingConfigBuilder;
typedef const internal::InternalReindexingType ReindexingConfig;

} // namespace vespa::config::content::reindexing
#endif // VESPA_config_VESPA_CONFIG_CONTENT_REINDEXING_REINDEXING_H
