#include "config-sentinel.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace cloud::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalSentinelType::CONFIG_DEF_MD5("aa9115ebf63cfc6721f75ada21d7cdfa");
const vespalib::string InternalSentinelType::CONFIG_DEF_NAME("sentinel");
const vespalib::string InternalSentinelType::CONFIG_DEF_NAMESPACE("cloud.config");
const int64_t InternalSentinelType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=cloud.config",
"port.telnet int default=19098",
"port.rpc int default=19097",
"application.tenant string default=\"default\"",
"application.name string default=\"default\"",
"application.environment string default=\"default\"",
"application.instance string default=\"default\"",
"application.region string default=\"default\"",
"connectivity.minOkPercent int default=50",
"connectivity.maxBadCount int default=1",
"service[].command string",
"service[].environ[].varname string",
"service[].environ[].varvalue string",
"service[].logctl[].componentSpec string",
"service[].logctl[].levelsModSpec string",
"service[].preShutdownCommand string default=\"\"",
"service[].name string",
"service[].autostart bool default=false",
"service[].autorestart bool default=true",
"service[].id reference",
"service[].affinity.cpuSocket int default=-1",
};
const ::config::StringVector InternalSentinelType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalSentinelType::Port::Port() noexcept
    : telnet(19098),
      rpc(19097)
{
}

InternalSentinelType::Port::Port(const Port & __rhs) noexcept = default;
InternalSentinelType::Port & InternalSentinelType::Port::operator = (const Port & __rhs) noexcept = default;
InternalSentinelType::Port::Port(Port && __rhs) noexcept = default;
InternalSentinelType::Port & InternalSentinelType::Port::operator = (Port && __rhs) noexcept = default;
InternalSentinelType::Port::~Port() = default; 
InternalSentinelType::Port::Port(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    telnet = ConfigParser::parse<int32_t>("telnet", __lines, 19098);
    ConfigParser::stripLinesForKey("telnet", __remainingValuesToParse);
    rpc = ConfigParser::parse<int32_t>("rpc", __lines, 19097);
    ConfigParser::stripLinesForKey("rpc", __remainingValuesToParse);
}

bool
InternalSentinelType::Port::operator==(const Port& __rhs) const noexcept
{
    return (telnet == __rhs.telnet &&
            rpc == __rhs.rpc);
}

bool
InternalSentinelType::Port::operator!=(const Port& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSentinelType::Port::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("telnet");
        __c.setString("type", "int");
        __c.setLong("value", telnet);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("rpc");
        __c.setString("type", "int");
        __c.setLong("value", rpc);
    }
}

InternalSentinelType::Port::Port(const vespalib::slime::Inspector & __inspector)
{
    telnet = __inspector["telnet"]["value"].asLong();
    rpc = __inspector["rpc"]["value"].asLong();
}

InternalSentinelType::Port::Port(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    telnet = ::config::internal::ValueConverter<int32_t>()(__inspector["telnet"], 19098);
    rpc = ::config::internal::ValueConverter<int32_t>()(__inspector["rpc"], 19097);
}

InternalSentinelType::Application::Application() noexcept
    : tenant("default"),
      name("default"),
      environment("default"),
      instance("default"),
      region("default")
{
}

InternalSentinelType::Application::Application(const Application & __rhs) noexcept = default;
InternalSentinelType::Application & InternalSentinelType::Application::operator = (const Application & __rhs) noexcept = default;
InternalSentinelType::Application::Application(Application && __rhs) noexcept = default;
InternalSentinelType::Application & InternalSentinelType::Application::operator = (Application && __rhs) noexcept = default;
InternalSentinelType::Application::~Application() = default; 
InternalSentinelType::Application::Application(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    tenant = ConfigParser::parse<vespalib::string>("tenant", __lines, "default");
    ConfigParser::stripLinesForKey("tenant", __remainingValuesToParse);
    name = ConfigParser::parse<vespalib::string>("name", __lines, "default");
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    environment = ConfigParser::parse<vespalib::string>("environment", __lines, "default");
    ConfigParser::stripLinesForKey("environment", __remainingValuesToParse);
    instance = ConfigParser::parse<vespalib::string>("instance", __lines, "default");
    ConfigParser::stripLinesForKey("instance", __remainingValuesToParse);
    region = ConfigParser::parse<vespalib::string>("region", __lines, "default");
    ConfigParser::stripLinesForKey("region", __remainingValuesToParse);
}

bool
InternalSentinelType::Application::operator==(const Application& __rhs) const noexcept
{
    return (tenant == __rhs.tenant &&
            name == __rhs.name &&
            environment == __rhs.environment &&
            instance == __rhs.instance &&
            region == __rhs.region);
}

bool
InternalSentinelType::Application::operator!=(const Application& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSentinelType::Application::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tenant");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(tenant));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("environment");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(environment));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("instance");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(instance));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("region");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(region));
    }
}

InternalSentinelType::Application::Application(const vespalib::slime::Inspector & __inspector)
{
    tenant = __inspector["tenant"]["value"].asString().make_string();
    name = __inspector["name"]["value"].asString().make_string();
    environment = __inspector["environment"]["value"].asString().make_string();
    instance = __inspector["instance"]["value"].asString().make_string();
    region = __inspector["region"]["value"].asString().make_string();
}

InternalSentinelType::Application::Application(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    tenant = ::config::internal::ValueConverter<vespalib::string>()(__inspector["tenant"], "default");
    name = ::config::internal::ValueConverter<vespalib::string>()(__inspector["name"], "default");
    environment = ::config::internal::ValueConverter<vespalib::string>()(__inspector["environment"], "default");
    instance = ::config::internal::ValueConverter<vespalib::string>()(__inspector["instance"], "default");
    region = ::config::internal::ValueConverter<vespalib::string>()(__inspector["region"], "default");
}

InternalSentinelType::Connectivity::Connectivity() noexcept
    : minOkPercent(50),
      maxBadCount(1)
{
}

InternalSentinelType::Connectivity::Connectivity(const Connectivity & __rhs) noexcept = default;
InternalSentinelType::Connectivity & InternalSentinelType::Connectivity::operator = (const Connectivity & __rhs) noexcept = default;
InternalSentinelType::Connectivity::Connectivity(Connectivity && __rhs) noexcept = default;
InternalSentinelType::Connectivity & InternalSentinelType::Connectivity::operator = (Connectivity && __rhs) noexcept = default;
InternalSentinelType::Connectivity::~Connectivity() = default; 
InternalSentinelType::Connectivity::Connectivity(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    minOkPercent = ConfigParser::parse<int32_t>("minOkPercent", __lines, 50);
    ConfigParser::stripLinesForKey("minOkPercent", __remainingValuesToParse);
    maxBadCount = ConfigParser::parse<int32_t>("maxBadCount", __lines, 1);
    ConfigParser::stripLinesForKey("maxBadCount", __remainingValuesToParse);
}

bool
InternalSentinelType::Connectivity::operator==(const Connectivity& __rhs) const noexcept
{
    return (minOkPercent == __rhs.minOkPercent &&
            maxBadCount == __rhs.maxBadCount);
}

bool
InternalSentinelType::Connectivity::operator!=(const Connectivity& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSentinelType::Connectivity::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("minOkPercent");
        __c.setString("type", "int");
        __c.setLong("value", minOkPercent);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxBadCount");
        __c.setString("type", "int");
        __c.setLong("value", maxBadCount);
    }
}

InternalSentinelType::Connectivity::Connectivity(const vespalib::slime::Inspector & __inspector)
{
    minOkPercent = __inspector["minOkPercent"]["value"].asLong();
    maxBadCount = __inspector["maxBadCount"]["value"].asLong();
}

InternalSentinelType::Connectivity::Connectivity(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    minOkPercent = ::config::internal::ValueConverter<int32_t>()(__inspector["minOkPercent"], 50);
    maxBadCount = ::config::internal::ValueConverter<int32_t>()(__inspector["maxBadCount"], 1);
}

InternalSentinelType::Service::Environ::Environ() noexcept
    : varname(),
      varvalue()
{
}

InternalSentinelType::Service::Environ::Environ(const Environ & __rhs) noexcept = default;
InternalSentinelType::Service::Environ & InternalSentinelType::Service::Environ::operator = (const Environ & __rhs) noexcept = default;
InternalSentinelType::Service::Environ::Environ(Environ && __rhs) noexcept = default;
InternalSentinelType::Service::Environ & InternalSentinelType::Service::Environ::operator = (Environ && __rhs) noexcept = default;
InternalSentinelType::Service::Environ::~Environ() = default; 
InternalSentinelType::Service::Environ::Environ(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    varname = ConfigParser::parse<vespalib::string>("varname", __lines);
    ConfigParser::stripLinesForKey("varname", __remainingValuesToParse);
    varvalue = ConfigParser::parse<vespalib::string>("varvalue", __lines);
    ConfigParser::stripLinesForKey("varvalue", __remainingValuesToParse);
}

bool
InternalSentinelType::Service::Environ::operator==(const Environ& __rhs) const noexcept
{
    return (varname == __rhs.varname &&
            varvalue == __rhs.varvalue);
}

bool
InternalSentinelType::Service::Environ::operator!=(const Environ& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSentinelType::Service::Environ::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("varname");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(varname));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("varvalue");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(varvalue));
    }
}

InternalSentinelType::Service::Environ::Environ(const vespalib::slime::Inspector & __inspector)
{
    varname = __inspector["varname"]["value"].asString().make_string();
    varvalue = __inspector["varvalue"]["value"].asString().make_string();
}

InternalSentinelType::Service::Environ::Environ(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    varname = ::config::internal::ValueConverter<vespalib::string>()("varname", __inspector["varname"]);
    varvalue = ::config::internal::ValueConverter<vespalib::string>()("varvalue", __inspector["varvalue"]);
}

InternalSentinelType::Service::Logctl::Logctl() noexcept
    : componentSpec(),
      levelsModSpec()
{
}

InternalSentinelType::Service::Logctl::Logctl(const Logctl & __rhs) noexcept = default;
InternalSentinelType::Service::Logctl & InternalSentinelType::Service::Logctl::operator = (const Logctl & __rhs) noexcept = default;
InternalSentinelType::Service::Logctl::Logctl(Logctl && __rhs) noexcept = default;
InternalSentinelType::Service::Logctl & InternalSentinelType::Service::Logctl::operator = (Logctl && __rhs) noexcept = default;
InternalSentinelType::Service::Logctl::~Logctl() = default; 
InternalSentinelType::Service::Logctl::Logctl(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    componentSpec = ConfigParser::parse<vespalib::string>("componentSpec", __lines);
    ConfigParser::stripLinesForKey("componentSpec", __remainingValuesToParse);
    levelsModSpec = ConfigParser::parse<vespalib::string>("levelsModSpec", __lines);
    ConfigParser::stripLinesForKey("levelsModSpec", __remainingValuesToParse);
}

bool
InternalSentinelType::Service::Logctl::operator==(const Logctl& __rhs) const noexcept
{
    return (componentSpec == __rhs.componentSpec &&
            levelsModSpec == __rhs.levelsModSpec);
}

bool
InternalSentinelType::Service::Logctl::operator!=(const Logctl& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSentinelType::Service::Logctl::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("componentSpec");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(componentSpec));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("levelsModSpec");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(levelsModSpec));
    }
}

InternalSentinelType::Service::Logctl::Logctl(const vespalib::slime::Inspector & __inspector)
{
    componentSpec = __inspector["componentSpec"]["value"].asString().make_string();
    levelsModSpec = __inspector["levelsModSpec"]["value"].asString().make_string();
}

InternalSentinelType::Service::Logctl::Logctl(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    componentSpec = ::config::internal::ValueConverter<vespalib::string>()("componentSpec", __inspector["componentSpec"]);
    levelsModSpec = ::config::internal::ValueConverter<vespalib::string>()("levelsModSpec", __inspector["levelsModSpec"]);
}

InternalSentinelType::Service::Affinity::Affinity() noexcept
    : cpuSocket(-1)
{
}

InternalSentinelType::Service::Affinity::Affinity(const Affinity & __rhs) noexcept = default;
InternalSentinelType::Service::Affinity & InternalSentinelType::Service::Affinity::operator = (const Affinity & __rhs) noexcept = default;
InternalSentinelType::Service::Affinity::Affinity(Affinity && __rhs) noexcept = default;
InternalSentinelType::Service::Affinity & InternalSentinelType::Service::Affinity::operator = (Affinity && __rhs) noexcept = default;
InternalSentinelType::Service::Affinity::~Affinity() = default; 
InternalSentinelType::Service::Affinity::Affinity(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    cpuSocket = ConfigParser::parse<int32_t>("cpuSocket", __lines, -1);
    ConfigParser::stripLinesForKey("cpuSocket", __remainingValuesToParse);
}

bool
InternalSentinelType::Service::Affinity::operator==(const Affinity& __rhs) const noexcept
{
    return (cpuSocket == __rhs.cpuSocket);
}

bool
InternalSentinelType::Service::Affinity::operator!=(const Affinity& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSentinelType::Service::Affinity::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("cpuSocket");
        __c.setString("type", "int");
        __c.setLong("value", cpuSocket);
    }
}

InternalSentinelType::Service::Affinity::Affinity(const vespalib::slime::Inspector & __inspector)
{
    cpuSocket = __inspector["cpuSocket"]["value"].asLong();
}

InternalSentinelType::Service::Affinity::Affinity(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    cpuSocket = ::config::internal::ValueConverter<int32_t>()(__inspector["cpuSocket"], -1);
}

InternalSentinelType::Service::Service()
    : command(),
      environ(),
      logctl(),
      preShutdownCommand(""),
      name(),
      autostart(false),
      autorestart(true),
      id(),
      affinity()
{
}

InternalSentinelType::Service::Service(const Service & __rhs) = default;
InternalSentinelType::Service & InternalSentinelType::Service::operator = (const Service & __rhs) = default;
InternalSentinelType::Service::Service(Service && __rhs) noexcept = default;
InternalSentinelType::Service & InternalSentinelType::Service::operator = (Service && __rhs) noexcept = default;
InternalSentinelType::Service::~Service() = default; 
InternalSentinelType::Service::Service(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    command = ConfigParser::parse<vespalib::string>("command", __lines);
    ConfigParser::stripLinesForKey("command", __remainingValuesToParse);
    environ = ConfigParser::parseArray<EnvironVector>("environ", __lines);
    ConfigParser::stripLinesForKey("environ", __remainingValuesToParse);
    logctl = ConfigParser::parseArray<LogctlVector>("logctl", __lines);
    ConfigParser::stripLinesForKey("logctl", __remainingValuesToParse);
    preShutdownCommand = ConfigParser::parse<vespalib::string>("preShutdownCommand", __lines, "");
    ConfigParser::stripLinesForKey("preShutdownCommand", __remainingValuesToParse);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    autostart = ConfigParser::parse<bool>("autostart", __lines, false);
    ConfigParser::stripLinesForKey("autostart", __remainingValuesToParse);
    autorestart = ConfigParser::parse<bool>("autorestart", __lines, true);
    ConfigParser::stripLinesForKey("autorestart", __remainingValuesToParse);
    id = ConfigParser::parse<vespalib::string>("id", __lines);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
    affinity = ConfigParser::parseStruct<Affinity>("affinity", __lines);
    ConfigParser::stripLinesForKey("affinity", __remainingValuesToParse);
}

bool
InternalSentinelType::Service::operator==(const Service& __rhs) const noexcept
{
    return (command == __rhs.command &&
            environ == __rhs.environ &&
            logctl == __rhs.logctl &&
            preShutdownCommand == __rhs.preShutdownCommand &&
            name == __rhs.name &&
            autostart == __rhs.autostart &&
            autorestart == __rhs.autorestart &&
            id == __rhs.id &&
            affinity == __rhs.affinity);
}

bool
InternalSentinelType::Service::operator!=(const Service& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSentinelType::Service::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("command");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(command));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("environ");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : environ) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("logctl");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : logctl) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("preShutdownCommand");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(preShutdownCommand));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("autostart");
        __c.setString("type", "bool");
        __c.setBool("value", autostart);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("autorestart");
        __c.setString("type", "bool");
        __c.setBool("value", autorestart);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "reference");
        __c.setString("value", vespalib::Memory(id));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("affinity");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        affinity.serialize(__c2);
    }
}

InternalSentinelType::Service::Service(const vespalib::slime::Inspector & __inspector)
{
    command = __inspector["command"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["environ"]["value"].children(); __i++) {
        environ.push_back(Environ(__inspector["environ"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["logctl"]["value"].children(); __i++) {
        logctl.push_back(Logctl(__inspector["logctl"]["value"][__i]["value"]));
    }
    preShutdownCommand = __inspector["preShutdownCommand"]["value"].asString().make_string();
    name = __inspector["name"]["value"].asString().make_string();
    autostart = __inspector["autostart"]["value"].asBool();
    autorestart = __inspector["autorestart"]["value"].asBool();
    id = __inspector["id"]["value"].asString().make_string();
    affinity = Affinity(__inspector["affinity"]["value"]);
}

InternalSentinelType::Service::Service(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    command = ::config::internal::ValueConverter<vespalib::string>()("command", __inspector["command"]);
    ::config::internal::VectorInserter<EnvironVector> __environInserter(environ);
    __inspector["environ"].traverse(__environInserter);
    ::config::internal::VectorInserter<LogctlVector> __logctlInserter(logctl);
    __inspector["logctl"].traverse(__logctlInserter);
    preShutdownCommand = ::config::internal::ValueConverter<vespalib::string>()(__inspector["preShutdownCommand"], "");
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    autostart = ::config::internal::ValueConverter<bool>()(__inspector["autostart"], false);
    autorestart = ::config::internal::ValueConverter<bool>()(__inspector["autorestart"], true);
    id = ::config::internal::ValueConverter<vespalib::string>()("id", __inspector["id"]);
    affinity = ::config::internal::ValueConverter<Affinity>()(__inspector["affinity"]);
}

InternalSentinelType::InternalSentinelType()
    : port(),
      application(),
      connectivity(),
      service()
{
}

InternalSentinelType::InternalSentinelType(const InternalSentinelType & __rhs) = default;
InternalSentinelType & InternalSentinelType::operator =(const InternalSentinelType & __rhs) = default;
InternalSentinelType::InternalSentinelType(InternalSentinelType && __rhs) noexcept = default;
InternalSentinelType & InternalSentinelType::operator =(InternalSentinelType && __rhs) noexcept = default;
InternalSentinelType::~InternalSentinelType() = default; 
InternalSentinelType::InternalSentinelType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        port = ConfigParser::parseStruct<Port>("port", __lines);
        ConfigParser::stripLinesForKey("port", __remainingValuesToParse);
        application = ConfigParser::parseStruct<Application>("application", __lines);
        ConfigParser::stripLinesForKey("application", __remainingValuesToParse);
        connectivity = ConfigParser::parseStruct<Connectivity>("connectivity", __lines);
        ConfigParser::stripLinesForKey("connectivity", __remainingValuesToParse);
        service = ConfigParser::parseArray<ServiceVector>("service", __lines);
        ConfigParser::stripLinesForKey("service", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalSentinelType::operator==(const InternalSentinelType& __rhs) const noexcept
{
    return (port == __rhs.port &&
            application == __rhs.application &&
            connectivity == __rhs.connectivity &&
            service == __rhs.service);
}

bool
InternalSentinelType::operator!=(const InternalSentinelType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSentinelType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("port");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        port.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("application");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        application.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("connectivity");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        connectivity.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("service");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : service) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalSentinelType::InternalSentinelType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    port = Port(__inspector["port"]["value"]);
    application = Application(__inspector["application"]["value"]);
    connectivity = Connectivity(__inspector["connectivity"]["value"]);
    for (size_t __i = 0; __i < __inspector["service"]["value"].children(); __i++) {
        service.push_back(Service(__inspector["service"]["value"][__i]["value"]));
    }
}

InternalSentinelType::InternalSentinelType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    port = ::config::internal::ValueConverter<Port>()(__inspector["port"]);
    application = ::config::internal::ValueConverter<Application>()(__inspector["application"]);
    connectivity = ::config::internal::ValueConverter<Connectivity>()(__inspector["connectivity"]);
    ::config::internal::VectorInserter<ServiceVector> __serviceInserter(service);
    __inspector["service"].traverse(__serviceInserter);
}

} // namespace internal

} // namespace cloud::config
