/**
 * @class cloud::config::Sentinel
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_CLOUD_CONFIG_SENTINEL_H
#define CLOUD_CONFIG_CLOUD_CONFIG_SENTINEL_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace cloud::config {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalSentinelType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalSentinelType> UP;
    class Port {
    public:
        Port(const ::config::StringVector & __lines);
        Port(const vespalib::slime::Inspector & __inspector);
        Port(const ::config::ConfigPayload & __payload);
        Port()  noexcept;
        Port(const Port & __rhs) noexcept;
        Port & operator = (const Port & __rhs) noexcept;
        Port(Port && __rhs) noexcept;
        Port & operator = (Port && __rhs) noexcept;
        ~Port();

        bool operator==(const Port& __rhs) const noexcept;
        bool operator!=(const Port& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** The port where sentinel should run its telnet interface */
        int32_t telnet; // Default: 19098

        /** The port where sentinel should run its RPC interface */
        int32_t rpc; // Default: 19097
    };
    typedef std::vector<Port> PortVector;

    typedef std::map<vespalib::string, Port> PortMap;

    class Application {
    public:
        Application(const ::config::StringVector & __lines);
        Application(const vespalib::slime::Inspector & __inspector);
        Application(const ::config::ConfigPayload & __payload);
        Application()  noexcept;
        Application(const Application & __rhs) noexcept;
        Application & operator = (const Application & __rhs) noexcept;
        Application(Application && __rhs) noexcept;
        Application & operator = (Application && __rhs) noexcept;
        ~Application();

        bool operator==(const Application& __rhs) const noexcept;
        bool operator!=(const Application& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * Meta info about the application running on this node. All values are
         * exported to child processes as $VESPA_APPLICATION_TENANT,
         * $VESPA_APPLICATION_NAME, $VESPA_APPLICATION_ENVIRONMENT,
         * $VESPA_APPLICATION_REGION
         * and $VESPA_APPLICATION_INSTANCE
         */
        vespalib::string tenant; // Default: "default"
        vespalib::string name; // Default: "default"
        vespalib::string environment; // Default: "default"
        vespalib::string instance; // Default: "default"
        vespalib::string region; // Default: "default"
    };
    typedef std::vector<Application> ApplicationVector;

    typedef std::map<vespalib::string, Application> ApplicationMap;

    class Connectivity {
    public:
        Connectivity(const ::config::StringVector & __lines);
        Connectivity(const vespalib::slime::Inspector & __inspector);
        Connectivity(const ::config::ConfigPayload & __payload);
        Connectivity()  noexcept;
        Connectivity(const Connectivity & __rhs) noexcept;
        Connectivity & operator = (const Connectivity & __rhs) noexcept;
        Connectivity(Connectivity && __rhs) noexcept;
        Connectivity & operator = (Connectivity && __rhs) noexcept;
        ~Connectivity();

        bool operator==(const Connectivity& __rhs) const noexcept;
        bool operator!=(const Connectivity& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        // Connectivity checks run before starting services and measure how
        // many nodes in the Vespa cluster we can connect to and how many of
        // those that can connect back to us. We delay starting services
        // if we have more problems than the following limits allow:

        /** Percentage of nodes that must be up and fully OK, minimum */
        int32_t minOkPercent; // Default: 50

        /**
         * Absolute number of nodes with confirmed network connectivity
         * problems, maximum
         */
        int32_t maxBadCount; // Default: 1
    };
    typedef std::vector<Connectivity> ConnectivityVector;

    typedef std::map<vespalib::string, Connectivity> ConnectivityMap;

    class Service {
    public:
        class Environ {
        public:
            Environ(const ::config::StringVector & __lines);
            Environ(const vespalib::slime::Inspector & __inspector);
            Environ(const ::config::ConfigPayload & __payload);
            Environ()  noexcept;
            Environ(const Environ & __rhs) noexcept;
            Environ & operator = (const Environ & __rhs) noexcept;
            Environ(Environ && __rhs) noexcept;
            Environ & operator = (Environ && __rhs) noexcept;
            ~Environ();

            bool operator==(const Environ& __rhs) const noexcept;
            bool operator!=(const Environ& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * Extra environment variables that will be exposed for this service
             */
            vespalib::string varname;
            vespalib::string varvalue;
        };
        typedef std::vector<Environ> EnvironVector;

        typedef std::map<vespalib::string, Environ> EnvironMap;

        class Logctl {
        public:
            Logctl(const ::config::StringVector & __lines);
            Logctl(const vespalib::slime::Inspector & __inspector);
            Logctl(const ::config::ConfigPayload & __payload);
            Logctl()  noexcept;
            Logctl(const Logctl & __rhs) noexcept;
            Logctl & operator = (const Logctl & __rhs) noexcept;
            Logctl(Logctl && __rhs) noexcept;
            Logctl & operator = (Logctl && __rhs) noexcept;
            ~Logctl();

            bool operator==(const Logctl& __rhs) const noexcept;
            bool operator!=(const Logctl& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** Tune log-level settings for specific components */
            vespalib::string componentSpec;
            vespalib::string levelsModSpec;
        };
        typedef std::vector<Logctl> LogctlVector;

        typedef std::map<vespalib::string, Logctl> LogctlMap;

        class Affinity {
        public:
            Affinity(const ::config::StringVector & __lines);
            Affinity(const vespalib::slime::Inspector & __inspector);
            Affinity(const ::config::ConfigPayload & __payload);
            Affinity()  noexcept;
            Affinity(const Affinity & __rhs) noexcept;
            Affinity & operator = (const Affinity & __rhs) noexcept;
            Affinity(Affinity && __rhs) noexcept;
            Affinity & operator = (Affinity && __rhs) noexcept;
            ~Affinity();

            bool operator==(const Affinity& __rhs) const noexcept;
            bool operator!=(const Affinity& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** Affinity of this service towards a CPU socket */
            int32_t cpuSocket; // Default: -1
        };
        typedef std::vector<Affinity> AffinityVector;

        typedef std::map<vespalib::string, Affinity> AffinityMap;

        Service(const ::config::StringVector & __lines);
        Service(const vespalib::slime::Inspector & __inspector);
        Service(const ::config::ConfigPayload & __payload);
        Service() ;
        Service(const Service & __rhs);
        Service & operator = (const Service & __rhs);
        Service(Service && __rhs) noexcept;
        Service & operator = (Service && __rhs) noexcept;
        ~Service();

        bool operator==(const Service& __rhs) const noexcept;
        bool operator!=(const Service& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * The command to run. This will be run by sh -c, and the following
         * environment variables are defined: $ROOT, $VESPA_SERVICE_NAME,
         * $VESPA_CONFIG_ID
         *  
         * stdin points to /dev/null
         * stdout and stderr connected to sentinel via pipe on startup.
         */
        vespalib::string command;
        EnvironVector environ;
        LogctlVector logctl;

        /**
         * The command to run before stopping service. The same properties as
         * for
         * startup command holds.
         */
        vespalib::string preShutdownCommand; // Default: ""

        /**
         * The service name for this service. Exported in VESPA_SERVICE_NAME
         * environment variable. All services must have unique names.
         * The service name is used by to start/stop services, and to list
         * them. It is also logged in the service field in every log message.
         */
        vespalib::string name;

        /**
         * Should the service be auto-started by configd as soon as it starts
         * up,
         * or must it be started manually? Almost always true.
         */
        bool autostart; // Default: false

        /** Should the service automatically be restarted if it exits? */
        bool autorestart; // Default: true

        /**
         * Config reference to the service being started.
         * Exported in the VESPA_CONFIG_ID environment variable.
         */
        vespalib::string id;
        Affinity affinity;
    };
    typedef std::vector<Service> ServiceVector;

    typedef std::map<vespalib::string, Service> ServiceMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalSentinelType(const ::config::ConfigValue & __value);
    InternalSentinelType(const ::config::ConfigDataBuffer & __value);
    InternalSentinelType(const ::config::ConfigPayload & __payload);
    InternalSentinelType() ;
    InternalSentinelType(const InternalSentinelType & __rhs);
    InternalSentinelType & operator = (const InternalSentinelType & __rhs);
    InternalSentinelType(InternalSentinelType && __rhs) noexcept;
    InternalSentinelType & operator = (InternalSentinelType && __rhs) noexcept;
    ~InternalSentinelType();

    bool operator==(const InternalSentinelType& __rhs) const noexcept;
    bool operator!=(const InternalSentinelType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    Port port;
    Application application;
    Connectivity connectivity;
    ServiceVector service;
};

} // namespace internal

typedef internal::InternalSentinelType SentinelConfigBuilder;
typedef const internal::InternalSentinelType SentinelConfig;

} // namespace cloud::config
#endif // VESPA_config_CLOUD_CONFIG_SENTINEL_H
