#include "config-slobroks.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace cloud::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalSlobroksType::CONFIG_DEF_MD5("4d375b91d30ffdbbe45de52ee5aa9fe0");
const vespalib::string InternalSlobroksType::CONFIG_DEF_NAME("slobroks");
const vespalib::string InternalSlobroksType::CONFIG_DEF_NAMESPACE("cloud.config");
const int64_t InternalSlobroksType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=cloud.config",
"slobrok[].connectionspec string restart",
};
const ::config::StringVector InternalSlobroksType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalSlobroksType::Slobrok::Slobrok() noexcept
    : connectionspec()
{
}

InternalSlobroksType::Slobrok::Slobrok(const Slobrok & __rhs) noexcept = default;
InternalSlobroksType::Slobrok & InternalSlobroksType::Slobrok::operator = (const Slobrok & __rhs) noexcept = default;
InternalSlobroksType::Slobrok::Slobrok(Slobrok && __rhs) noexcept = default;
InternalSlobroksType::Slobrok & InternalSlobroksType::Slobrok::operator = (Slobrok && __rhs) noexcept = default;
InternalSlobroksType::Slobrok::~Slobrok() = default; 
InternalSlobroksType::Slobrok::Slobrok(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    connectionspec = ConfigParser::parse<vespalib::string>("connectionspec", __lines);
    ConfigParser::stripLinesForKey("connectionspec", __remainingValuesToParse);
}

bool
InternalSlobroksType::Slobrok::operator==(const Slobrok& __rhs) const noexcept
{
    return (connectionspec == __rhs.connectionspec);
}

bool
InternalSlobroksType::Slobrok::operator!=(const Slobrok& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSlobroksType::Slobrok::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("connectionspec");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(connectionspec));
    }
}

InternalSlobroksType::Slobrok::Slobrok(const vespalib::slime::Inspector & __inspector)
{
    connectionspec = __inspector["connectionspec"]["value"].asString().make_string();
}

InternalSlobroksType::Slobrok::Slobrok(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    connectionspec = ::config::internal::ValueConverter<vespalib::string>()("connectionspec", __inspector["connectionspec"]);
}

InternalSlobroksType::InternalSlobroksType()
    : slobrok()
{
}

InternalSlobroksType::InternalSlobroksType(const InternalSlobroksType & __rhs) = default;
InternalSlobroksType & InternalSlobroksType::operator =(const InternalSlobroksType & __rhs) = default;
InternalSlobroksType::InternalSlobroksType(InternalSlobroksType && __rhs) noexcept = default;
InternalSlobroksType & InternalSlobroksType::operator =(InternalSlobroksType && __rhs) noexcept = default;
InternalSlobroksType::~InternalSlobroksType() = default; 
InternalSlobroksType::InternalSlobroksType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        slobrok = ConfigParser::parseArray<SlobrokVector>("slobrok", __lines);
        ConfigParser::stripLinesForKey("slobrok", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalSlobroksType::operator==(const InternalSlobroksType& __rhs) const noexcept
{
    return (slobrok == __rhs.slobrok);
}

bool
InternalSlobroksType::operator!=(const InternalSlobroksType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSlobroksType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("slobrok");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : slobrok) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalSlobroksType::InternalSlobroksType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["slobrok"]["value"].children(); __i++) {
        slobrok.push_back(Slobrok(__inspector["slobrok"]["value"][__i]["value"]));
    }
}

InternalSlobroksType::InternalSlobroksType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<SlobrokVector> __slobrokInserter(slobrok);
    __inspector["slobrok"].traverse(__slobrokInserter);
}

} // namespace internal

} // namespace cloud::config
