#include "config-specialtokens.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::configdefinition {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalSpecialtokensType::CONFIG_DEF_MD5("385d8b4566e898e31fd9fbde4e9c8b65");
const vespalib::string InternalSpecialtokensType::CONFIG_DEF_NAME("specialtokens");
const vespalib::string InternalSpecialtokensType::CONFIG_DEF_NAMESPACE("vespa.configdefinition");
const int64_t InternalSpecialtokensType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.configdefinition",
"makefsaPath string default=\"bin/vespa-makefsa\"",
"tokenlist[].name string",
"tokenlist[].tokens[].token string",
"tokenlist[].tokens[].replace string default=\"\"",
};
const ::config::StringVector InternalSpecialtokensType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalSpecialtokensType::Tokenlist::Tokens::Tokens() noexcept
    : token(),
      replace("")
{
}

InternalSpecialtokensType::Tokenlist::Tokens::Tokens(const Tokens & __rhs) noexcept = default;
InternalSpecialtokensType::Tokenlist::Tokens & InternalSpecialtokensType::Tokenlist::Tokens::operator = (const Tokens & __rhs) noexcept = default;
InternalSpecialtokensType::Tokenlist::Tokens::Tokens(Tokens && __rhs) noexcept = default;
InternalSpecialtokensType::Tokenlist::Tokens & InternalSpecialtokensType::Tokenlist::Tokens::operator = (Tokens && __rhs) noexcept = default;
InternalSpecialtokensType::Tokenlist::Tokens::~Tokens() = default; 
InternalSpecialtokensType::Tokenlist::Tokens::Tokens(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    token = ConfigParser::parse<vespalib::string>("token", __lines);
    ConfigParser::stripLinesForKey("token", __remainingValuesToParse);
    replace = ConfigParser::parse<vespalib::string>("replace", __lines, "");
    ConfigParser::stripLinesForKey("replace", __remainingValuesToParse);
}

bool
InternalSpecialtokensType::Tokenlist::Tokens::
operator==(const Tokens& __rhs) const noexcept
{
    return (token == __rhs.token &&
            replace == __rhs.replace);
}

bool
InternalSpecialtokensType::Tokenlist::Tokens::
operator!=(const Tokens& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSpecialtokensType::Tokenlist::Tokens::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("token");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(token));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("replace");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(replace));
    }
}

InternalSpecialtokensType::Tokenlist::Tokens::Tokens(const vespalib::slime::Inspector & __inspector)
{
    token = __inspector["token"]["value"].asString().make_string();
    replace = __inspector["replace"]["value"].asString().make_string();
}

InternalSpecialtokensType::Tokenlist::Tokens::Tokens(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    token = ::config::internal::ValueConverter<vespalib::string>()("token", __inspector["token"]);
    replace = ::config::internal::ValueConverter<vespalib::string>()(__inspector["replace"], "");
}

InternalSpecialtokensType::Tokenlist::Tokenlist()
    : name(),
      tokens()
{
}

InternalSpecialtokensType::Tokenlist::Tokenlist(const Tokenlist & __rhs) = default;
InternalSpecialtokensType::Tokenlist & InternalSpecialtokensType::Tokenlist::operator = (const Tokenlist & __rhs) = default;
InternalSpecialtokensType::Tokenlist::Tokenlist(Tokenlist && __rhs) noexcept = default;
InternalSpecialtokensType::Tokenlist & InternalSpecialtokensType::Tokenlist::operator = (Tokenlist && __rhs) noexcept = default;
InternalSpecialtokensType::Tokenlist::~Tokenlist() = default; 
InternalSpecialtokensType::Tokenlist::Tokenlist(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    tokens = ConfigParser::parseArray<TokensVector>("tokens", __lines);
    ConfigParser::stripLinesForKey("tokens", __remainingValuesToParse);
}

bool
InternalSpecialtokensType::Tokenlist::operator==(const Tokenlist& __rhs) const noexcept
{
    return (name == __rhs.name &&
            tokens == __rhs.tokens);
}

bool
InternalSpecialtokensType::Tokenlist::operator!=(const Tokenlist& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSpecialtokensType::Tokenlist::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tokens");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : tokens) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalSpecialtokensType::Tokenlist::Tokenlist(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["tokens"]["value"].children(); __i++) {
        tokens.push_back(Tokens(__inspector["tokens"]["value"][__i]["value"]));
    }
}

InternalSpecialtokensType::Tokenlist::Tokenlist(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    ::config::internal::VectorInserter<TokensVector> __tokensInserter(tokens);
    __inspector["tokens"].traverse(__tokensInserter);
}

InternalSpecialtokensType::InternalSpecialtokensType()
    : makefsaPath("bin/vespa-makefsa"),
      tokenlist()
{
}

InternalSpecialtokensType::InternalSpecialtokensType(const InternalSpecialtokensType & __rhs) = default;
InternalSpecialtokensType & InternalSpecialtokensType::operator =(const InternalSpecialtokensType & __rhs) = default;
InternalSpecialtokensType::InternalSpecialtokensType(InternalSpecialtokensType && __rhs) noexcept = default;
InternalSpecialtokensType & InternalSpecialtokensType::operator =(InternalSpecialtokensType && __rhs) noexcept = default;
InternalSpecialtokensType::~InternalSpecialtokensType() = default; 
InternalSpecialtokensType::InternalSpecialtokensType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        makefsaPath = ConfigParser::parse<vespalib::string>("makefsaPath", __lines, "bin/vespa-makefsa");
        ConfigParser::stripLinesForKey("makefsaPath", __remainingValuesToParse);
        tokenlist = ConfigParser::parseArray<TokenlistVector>("tokenlist", __lines);
        ConfigParser::stripLinesForKey("tokenlist", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalSpecialtokensType::
operator==(const InternalSpecialtokensType& __rhs) const noexcept
{
    return (makefsaPath == __rhs.makefsaPath &&
            tokenlist == __rhs.tokenlist);
}

bool
InternalSpecialtokensType::
operator!=(const InternalSpecialtokensType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSpecialtokensType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("makefsaPath");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(makefsaPath));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tokenlist");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : tokenlist) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalSpecialtokensType::InternalSpecialtokensType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    makefsaPath = __inspector["makefsaPath"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["tokenlist"]["value"].children(); __i++) {
        tokenlist.push_back(Tokenlist(__inspector["tokenlist"]["value"][__i]["value"]));
    }
}

InternalSpecialtokensType::InternalSpecialtokensType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    makefsaPath = ::config::internal::ValueConverter<vespalib::string>()(__inspector["makefsaPath"], "bin/vespa-makefsa");
    ::config::internal::VectorInserter<TokenlistVector> __tokenlistInserter(tokenlist);
    __inspector["tokenlist"].traverse(__tokenlistInserter);
}

} // namespace internal

} // namespace vespa::configdefinition
