/**
 * @class vespa::configdefinition::Specialtokens
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 * Named lists of special tokens - string which should be
 * treated as words no matter what characters they consist of.
 * A special token can also optionally be replaced by another
 * token.
 *  
 * Special tokens are case sensitive, since queries and tokens
 * are lowercased they should normally be written in lower case.
 *  
 * The list named default is the only list which is used currently
 *  
 * Order matters in token lists. If one special token is a
 * prefix of another token, Vespa will prefer to return the first
 * matching token in the list when encountering the longest
 * special token.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIGDEFINITION_SPECIALTOKENS_H
#define CLOUD_CONFIG_VESPA_CONFIGDEFINITION_SPECIALTOKENS_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::configdefinition {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalSpecialtokensType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalSpecialtokensType> UP;
    class Tokenlist {
    public:
        class Tokens {
        public:
            Tokens(const ::config::StringVector & __lines);
            Tokens(const vespalib::slime::Inspector & __inspector);
            Tokens(const ::config::ConfigPayload & __payload);
            Tokens()  noexcept;
            Tokens(const Tokens & __rhs) noexcept;
            Tokens & operator = (const Tokens & __rhs) noexcept;
            Tokens(Tokens && __rhs) noexcept;
            Tokens & operator = (Tokens && __rhs) noexcept;
            ~Tokens();

            bool operator==(const Tokens& __rhs) const noexcept;
            bool operator!=(const Tokens& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** Special tokens, any string is permitted */
            vespalib::string token;

            /**
             * The token which should replace the special token
             * If no replace is set for a token, the word token
             * will be the special token string itself
             */
            vespalib::string replace; // Default: ""
        };
        typedef std::vector<Tokens> TokensVector;

        typedef std::map<vespalib::string, Tokens> TokensMap;

        Tokenlist(const ::config::StringVector & __lines);
        Tokenlist(const vespalib::slime::Inspector & __inspector);
        Tokenlist(const ::config::ConfigPayload & __payload);
        Tokenlist() ;
        Tokenlist(const Tokenlist & __rhs);
        Tokenlist & operator = (const Tokenlist & __rhs);
        Tokenlist(Tokenlist && __rhs) noexcept;
        Tokenlist & operator = (Tokenlist && __rhs) noexcept;
        ~Tokenlist();

        bool operator==(const Tokenlist& __rhs) const noexcept;
        bool operator!=(const Tokenlist& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * Use 'default' as the name of the list to use in regular
         * indexing and queries
         */
        vespalib::string name;
        TokensVector tokens;
    };
    typedef std::vector<Tokenlist> TokenlistVector;

    typedef std::map<vespalib::string, Tokenlist> TokenlistMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalSpecialtokensType(const ::config::ConfigValue & __value);
    InternalSpecialtokensType(const ::config::ConfigDataBuffer & __value);
    InternalSpecialtokensType(const ::config::ConfigPayload & __payload);
    InternalSpecialtokensType() ;
    InternalSpecialtokensType(const InternalSpecialtokensType & __rhs);
    InternalSpecialtokensType & operator = (const InternalSpecialtokensType & __rhs);
    InternalSpecialtokensType(InternalSpecialtokensType && __rhs) noexcept;
    InternalSpecialtokensType & operator = (InternalSpecialtokensType && __rhs) noexcept;
    ~InternalSpecialtokensType();

    bool operator==(const InternalSpecialtokensType& __rhs) const noexcept;
    bool operator!=(const InternalSpecialtokensType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /**
     * Path to makefsa binary, needed to create specialtokens-dictionary
     * If the path is relative, vespa home is prepended
     */
    vespalib::string makefsaPath; // Default: "bin/vespa-makefsa"
    TokenlistVector tokenlist;
};

} // namespace internal

typedef internal::InternalSpecialtokensType SpecialtokensConfigBuilder;
typedef const internal::InternalSpecialtokensType SpecialtokensConfig;

} // namespace vespa::configdefinition
#endif // VESPA_config_VESPA_CONFIGDEFINITION_SPECIALTOKENS_H
