#include "config-stateserver.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalStateserverType::CONFIG_DEF_MD5("75d37986bb1dc22067b1f95fbe3ec009");
const vespalib::string InternalStateserverType::CONFIG_DEF_NAME("stateserver");
const vespalib::string InternalStateserverType::CONFIG_DEF_NAMESPACE("vespa.config.core");
const int64_t InternalStateserverType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.core",
"httpport int default = 0",
};
const ::config::StringVector InternalStateserverType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalStateserverType::InternalStateserverType() noexcept
    : httpport(0)
{
}

InternalStateserverType::InternalStateserverType(const InternalStateserverType & __rhs) noexcept = default;
InternalStateserverType & InternalStateserverType::operator =(const InternalStateserverType & __rhs) noexcept = default;
InternalStateserverType::InternalStateserverType(InternalStateserverType && __rhs) noexcept = default;
InternalStateserverType & InternalStateserverType::operator =(InternalStateserverType && __rhs) noexcept = default;
InternalStateserverType::~InternalStateserverType() = default; 
InternalStateserverType::InternalStateserverType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        httpport = ConfigParser::parse<int32_t>("httpport", __lines, 0);
        ConfigParser::stripLinesForKey("httpport", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalStateserverType::operator==(const InternalStateserverType& __rhs) const noexcept
{
    return (httpport == __rhs.httpport);
}

bool
InternalStateserverType::operator!=(const InternalStateserverType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStateserverType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("httpport");
        __c.setString("type", "int");
        __c.setLong("value", httpport);
    }
}

InternalStateserverType::InternalStateserverType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    httpport = __inspector["httpport"]["value"].asLong();
}

InternalStateserverType::InternalStateserverType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    httpport = ::config::internal::ValueConverter<int32_t>()(__inspector["httpport"], 0);
}

} // namespace internal

} // namespace vespa::config::core
