#include "config-stor-distribution.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalStorDistributionType::CONFIG_DEF_MD5("c92d9731f286f2746a74fd1350b7c568");
const vespalib::string InternalStorDistributionType::CONFIG_DEF_NAME("stor-distribution");
const vespalib::string InternalStorDistributionType::CONFIG_DEF_NAMESPACE("vespa.config.content");
const int64_t InternalStorDistributionType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content",
"redundancy int default=3",
"initial_redundancy int default=0",
"ensure_primary_persisted bool default=true",
"ready_copies int default=0",
"active_per_leaf_group bool default=false",
"group[].index string",
"group[].name string",
"group[].capacity double default=1",
"group[].partitions string default=\"\"",
"group[].nodes[].index int",
"group[].nodes[].retired bool default=false",
};
const ::config::StringVector InternalStorDistributionType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalStorDistributionType::Group::Nodes::Nodes() noexcept
    : index(0),
      retired(false)
{
}

InternalStorDistributionType::Group::Nodes::Nodes(const Nodes & __rhs) noexcept = default;
InternalStorDistributionType::Group::Nodes & InternalStorDistributionType::Group::Nodes::operator = (const Nodes & __rhs) noexcept = default;
InternalStorDistributionType::Group::Nodes::Nodes(Nodes && __rhs) noexcept = default;
InternalStorDistributionType::Group::Nodes & InternalStorDistributionType::Group::Nodes::operator = (Nodes && __rhs) noexcept = default;
InternalStorDistributionType::Group::Nodes::~Nodes() = default; 
InternalStorDistributionType::Group::Nodes::Nodes(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    index = ConfigParser::parse<int32_t>("index", __lines);
    ConfigParser::stripLinesForKey("index", __remainingValuesToParse);
    retired = ConfigParser::parse<bool>("retired", __lines, false);
    ConfigParser::stripLinesForKey("retired", __remainingValuesToParse);
}

bool
InternalStorDistributionType::Group::Nodes::operator==(const Nodes& __rhs) const noexcept
{
    return (index == __rhs.index &&
            retired == __rhs.retired);
}

bool
InternalStorDistributionType::Group::Nodes::operator!=(const Nodes& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorDistributionType::Group::Nodes::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("index");
        __c.setString("type", "int");
        __c.setLong("value", index);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("retired");
        __c.setString("type", "bool");
        __c.setBool("value", retired);
    }
}

InternalStorDistributionType::Group::Nodes::Nodes(const vespalib::slime::Inspector & __inspector)
{
    index = __inspector["index"]["value"].asLong();
    retired = __inspector["retired"]["value"].asBool();
}

InternalStorDistributionType::Group::Nodes::Nodes(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    index = ::config::internal::ValueConverter<int32_t>()("index", __inspector["index"]);
    retired = ::config::internal::ValueConverter<bool>()(__inspector["retired"], false);
}

InternalStorDistributionType::Group::Group()
    : index(),
      name(),
      capacity(1),
      partitions(""),
      nodes()
{
}

InternalStorDistributionType::Group::Group(const Group & __rhs) = default;
InternalStorDistributionType::Group & InternalStorDistributionType::Group::operator = (const Group & __rhs) = default;
InternalStorDistributionType::Group::Group(Group && __rhs) noexcept = default;
InternalStorDistributionType::Group & InternalStorDistributionType::Group::operator = (Group && __rhs) noexcept = default;
InternalStorDistributionType::Group::~Group() = default; 
InternalStorDistributionType::Group::Group(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    index = ConfigParser::parse<vespalib::string>("index", __lines);
    ConfigParser::stripLinesForKey("index", __remainingValuesToParse);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    capacity = ConfigParser::parse<double>("capacity", __lines, 1);
    ConfigParser::stripLinesForKey("capacity", __remainingValuesToParse);
    partitions = ConfigParser::parse<vespalib::string>("partitions", __lines, "");
    ConfigParser::stripLinesForKey("partitions", __remainingValuesToParse);
    nodes = ConfigParser::parseArray<NodesVector>("nodes", __lines);
    ConfigParser::stripLinesForKey("nodes", __remainingValuesToParse);
}

bool
InternalStorDistributionType::Group::operator==(const Group& __rhs) const noexcept
{
    return (index == __rhs.index &&
            name == __rhs.name &&
            capacity == __rhs.capacity &&
            partitions == __rhs.partitions &&
            nodes == __rhs.nodes);
}

bool
InternalStorDistributionType::Group::operator!=(const Group& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorDistributionType::Group::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("index");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(index));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("capacity");
        __c.setString("type", "double");
        __c.setDouble("value", capacity);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("partitions");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(partitions));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("nodes");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : nodes) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalStorDistributionType::Group::Group(const vespalib::slime::Inspector & __inspector)
{
    index = __inspector["index"]["value"].asString().make_string();
    name = __inspector["name"]["value"].asString().make_string();
    capacity = __inspector["capacity"]["value"].asDouble();
    partitions = __inspector["partitions"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["nodes"]["value"].children(); __i++) {
        nodes.push_back(Nodes(__inspector["nodes"]["value"][__i]["value"]));
    }
}

InternalStorDistributionType::Group::Group(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    index = ::config::internal::ValueConverter<vespalib::string>()("index", __inspector["index"]);
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    capacity = ::config::internal::ValueConverter<double>()(__inspector["capacity"], 1);
    partitions = ::config::internal::ValueConverter<vespalib::string>()(__inspector["partitions"], "");
    ::config::internal::VectorInserter<NodesVector> __nodesInserter(nodes);
    __inspector["nodes"].traverse(__nodesInserter);
}

InternalStorDistributionType::InternalStorDistributionType()
    : redundancy(3),
      initialRedundancy(0),
      ensurePrimaryPersisted(true),
      readyCopies(0),
      activePerLeafGroup(false),
      group()
{
}

InternalStorDistributionType::InternalStorDistributionType(const InternalStorDistributionType & __rhs) = default;
InternalStorDistributionType & InternalStorDistributionType::operator =(const InternalStorDistributionType & __rhs) = default;
InternalStorDistributionType::InternalStorDistributionType(InternalStorDistributionType && __rhs) noexcept = default;
InternalStorDistributionType & InternalStorDistributionType::operator =(InternalStorDistributionType && __rhs) noexcept = default;
InternalStorDistributionType::~InternalStorDistributionType() = default; 
InternalStorDistributionType::InternalStorDistributionType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        redundancy = ConfigParser::parse<int32_t>("redundancy", __lines, 3);
        ConfigParser::stripLinesForKey("redundancy", __remainingValuesToParse);
        initialRedundancy = ConfigParser::parse<int32_t>("initial_redundancy", __lines, 0);
        ConfigParser::stripLinesForKey("initial_redundancy", __remainingValuesToParse);
        ensurePrimaryPersisted = ConfigParser::parse<bool>("ensure_primary_persisted", __lines, true);
        ConfigParser::stripLinesForKey("ensure_primary_persisted", __remainingValuesToParse);
        readyCopies = ConfigParser::parse<int32_t>("ready_copies", __lines, 0);
        ConfigParser::stripLinesForKey("ready_copies", __remainingValuesToParse);
        activePerLeafGroup = ConfigParser::parse<bool>("active_per_leaf_group", __lines, false);
        ConfigParser::stripLinesForKey("active_per_leaf_group", __remainingValuesToParse);
        group = ConfigParser::parseArray<GroupVector>("group", __lines);
        ConfigParser::stripLinesForKey("group", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalStorDistributionType::
operator==(const InternalStorDistributionType& __rhs) const noexcept
{
    return (redundancy == __rhs.redundancy &&
            initialRedundancy == __rhs.initialRedundancy &&
            ensurePrimaryPersisted == __rhs.ensurePrimaryPersisted &&
            readyCopies == __rhs.readyCopies &&
            activePerLeafGroup == __rhs.activePerLeafGroup &&
            group == __rhs.group);
}

bool
InternalStorDistributionType::
operator!=(const InternalStorDistributionType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorDistributionType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("redundancy");
        __c.setString("type", "int");
        __c.setLong("value", redundancy);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("initial_redundancy");
        __c.setString("type", "int");
        __c.setLong("value", initialRedundancy);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ensure_primary_persisted");
        __c.setString("type", "bool");
        __c.setBool("value", ensurePrimaryPersisted);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ready_copies");
        __c.setString("type", "int");
        __c.setLong("value", readyCopies);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("active_per_leaf_group");
        __c.setString("type", "bool");
        __c.setBool("value", activePerLeafGroup);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("group");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : group) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalStorDistributionType::InternalStorDistributionType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    redundancy = __inspector["redundancy"]["value"].asLong();
    initialRedundancy = __inspector["initial_redundancy"]["value"].asLong();
    ensurePrimaryPersisted = __inspector["ensure_primary_persisted"]["value"].asBool();
    readyCopies = __inspector["ready_copies"]["value"].asLong();
    activePerLeafGroup = __inspector["active_per_leaf_group"]["value"].asBool();
    for (size_t __i = 0; __i < __inspector["group"]["value"].children(); __i++) {
        group.push_back(Group(__inspector["group"]["value"][__i]["value"]));
    }
}

InternalStorDistributionType::InternalStorDistributionType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    redundancy = ::config::internal::ValueConverter<int32_t>()(__inspector["redundancy"], 3);
    initialRedundancy = ::config::internal::ValueConverter<int32_t>()(__inspector["initial_redundancy"], 0);
    ensurePrimaryPersisted = ::config::internal::ValueConverter<bool>()(__inspector["ensure_primary_persisted"], true);
    readyCopies = ::config::internal::ValueConverter<int32_t>()(__inspector["ready_copies"], 0);
    activePerLeafGroup = ::config::internal::ValueConverter<bool>()(__inspector["active_per_leaf_group"], false);
    ::config::internal::VectorInserter<GroupVector> __groupInserter(group);
    __inspector["group"].traverse(__groupInserter);
}

} // namespace internal

} // namespace vespa::config::content
