/**
 * @class vespa::config::content::StorDistribution
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_CONTENT_STORDISTRIBUTION_H
#define CLOUD_CONFIG_VESPA_CONFIG_CONTENT_STORDISTRIBUTION_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::content {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalStorDistributionType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalStorDistributionType> UP;
    class Group {
    public:
        class Nodes {
        public:
            Nodes(const ::config::StringVector & __lines);
            Nodes(const vespalib::slime::Inspector & __inspector);
            Nodes(const ::config::ConfigPayload & __payload);
            Nodes()  noexcept;
            Nodes(const Nodes & __rhs) noexcept;
            Nodes & operator = (const Nodes & __rhs) noexcept;
            Nodes(Nodes && __rhs) noexcept;
            Nodes & operator = (Nodes && __rhs) noexcept;
            ~Nodes();

            bool operator==(const Nodes& __rhs) const noexcept;
            bool operator!=(const Nodes& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * Leaf groups will have a set of nodes within them. Branch groups
             * will have
             * none.
             */
            int32_t index;

            /**
             * Whether this node is retired.
             * The system will migrate all data away from retired nodes such
             * that they can
             * eventually be removed without partial data loss
             */
            bool retired; // Default: false
        };
        typedef std::vector<Nodes> NodesVector;

        typedef std::map<vespalib::string, Nodes> NodesMap;

        Group(const ::config::StringVector & __lines);
        Group(const vespalib::slime::Inspector & __inspector);
        Group(const ::config::ConfigPayload & __payload);
        Group() ;
        Group(const Group & __rhs);
        Group & operator = (const Group & __rhs);
        Group(Group && __rhs) noexcept;
        Group & operator = (Group && __rhs) noexcept;
        ~Group();

        bool operator==(const Group& __rhs) const noexcept;
        bool operator!=(const Group& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * Hierarchical grouping divides the nodes into a tree of groups. Due to
         * config
         * liking flat structures. The tree of groups is represented by a single
         * array
         * of groups defined here, where index is a string that can have a form
         * like
         * "1.2.1" in order to tell where in the tree structure the group is.
         */
        vespalib::string index;

        /**
         * Each group needs to have a name. Makes it easier to operations that
         * they can
         * name their groups according to what they want to call them.
         */
        vespalib::string name;

        /** Capacity of the given group */
        double capacity; // Default: 1

        /**
         * Partitions define how copies are divided among child groups/nodes.
         */
        vespalib::string partitions; // Default: ""
        NodesVector nodes;
    };
    typedef std::vector<Group> GroupVector;

    typedef std::map<vespalib::string, Group> GroupMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalStorDistributionType(const ::config::ConfigValue & __value);
    InternalStorDistributionType(const ::config::ConfigDataBuffer & __value);
    InternalStorDistributionType(const ::config::ConfigPayload & __payload);
    InternalStorDistributionType() ;
    InternalStorDistributionType(const InternalStorDistributionType & __rhs);
    InternalStorDistributionType & operator = (const InternalStorDistributionType & __rhs);
    InternalStorDistributionType(InternalStorDistributionType && __rhs) noexcept;
    InternalStorDistributionType & operator = (InternalStorDistributionType && __rhs) noexcept;
    ~InternalStorDistributionType();

    bool operator==(const InternalStorDistributionType& __rhs) const noexcept;
    bool operator!=(const InternalStorDistributionType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /**
     * Redundancy decides how many copies of documents will be stored. Documents
     * will store copies on enough nodes, such that the sum of the node
     * reliability
     * factors is equal or above the redundancy.
     */
    int32_t redundancy; // Default: 3

    /**
     * Initial redundancy allows put-operations to return as completed after
     * a subset of all copies have been persisted (known as initial persistence
     * in BCP terms).
     * Set to 0 to disable and use normal redundancy behavior instead.
     */
    int32_t initialRedundancy; // Default: 0

    /**
     * If set, requires that the primary nodes copy should always be persisted
     * before an n-of-m initial persistence operation can complete.
     */
    bool ensurePrimaryPersisted; // Default: true

    /**
     * The number of copies that should be "ready" to be active.
     * Maximum is redundancy.
     */
    int32_t readyCopies; // Default: 0

    /**
     * If this option is set true, the distributor will try to enforce one
     * active copy
     * of buckets per leaf hierarchical group. This is a simple implementation
     * for
     * search to be able to setup top level dispatcher to only send search to
     * all
     * nodes in one group as they have a static cost per node used. If used,
     * hierarchical grouping can not be used for other purposes. Using this
     * option
     * implies that:
     * - ready_copies == redundancy
     * - Only one level of hierarchical grouping defined.
     * - That level distributes copies to all defined groups.
     */
    bool activePerLeafGroup; // Default: false
    GroupVector group;
};

} // namespace internal

typedef internal::InternalStorDistributionType StorDistributionConfigBuilder;
typedef const internal::InternalStorDistributionType StorDistributionConfig;

} // namespace vespa::config::content
#endif // VESPA_config_VESPA_CONFIG_CONTENT_STORDISTRIBUTION_H
