#include "config-summary.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalSummaryType::CONFIG_DEF_MD5("063ea02634d07cabfe7b4e1b408659b2");
const vespalib::string InternalSummaryType::CONFIG_DEF_NAME("summary");
const vespalib::string InternalSummaryType::CONFIG_DEF_NAMESPACE("vespa.config.search");
const int64_t InternalSummaryType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search",
"defaultsummaryid int default=-1",
"usev8geopositions bool default=false",
"classes[].id int",
"classes[].name string",
"classes[].omitsummaryfeatures bool default=false",
"classes[].fields[].name string",
"classes[].fields[].command string default=\"\"",
"classes[].fields[].source string default=\"\"",
};
const ::config::StringVector InternalSummaryType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalSummaryType::Classes::Fields::Fields() noexcept
    : name(),
      command(""),
      source("")
{
}

InternalSummaryType::Classes::Fields::Fields(const Fields & __rhs) noexcept = default;
InternalSummaryType::Classes::Fields & InternalSummaryType::Classes::Fields::operator = (const Fields & __rhs) noexcept = default;
InternalSummaryType::Classes::Fields::Fields(Fields && __rhs) noexcept = default;
InternalSummaryType::Classes::Fields & InternalSummaryType::Classes::Fields::operator = (Fields && __rhs) noexcept = default;
InternalSummaryType::Classes::Fields::~Fields() = default; 
InternalSummaryType::Classes::Fields::Fields(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    command = ConfigParser::parse<vespalib::string>("command", __lines, "");
    ConfigParser::stripLinesForKey("command", __remainingValuesToParse);
    source = ConfigParser::parse<vespalib::string>("source", __lines, "");
    ConfigParser::stripLinesForKey("source", __remainingValuesToParse);
}

bool
InternalSummaryType::Classes::Fields::operator==(const Fields& __rhs) const noexcept
{
    return (name == __rhs.name &&
            command == __rhs.command &&
            source == __rhs.source);
}

bool
InternalSummaryType::Classes::Fields::operator!=(const Fields& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSummaryType::Classes::Fields::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("command");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(command));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("source");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(source));
    }
}

InternalSummaryType::Classes::Fields::Fields(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    command = __inspector["command"]["value"].asString().make_string();
    source = __inspector["source"]["value"].asString().make_string();
}

InternalSummaryType::Classes::Fields::Fields(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    command = ::config::internal::ValueConverter<vespalib::string>()(__inspector["command"], "");
    source = ::config::internal::ValueConverter<vespalib::string>()(__inspector["source"], "");
}

InternalSummaryType::Classes::Classes()
    : id(0),
      name(),
      omitsummaryfeatures(false),
      fields()
{
}

InternalSummaryType::Classes::Classes(const Classes & __rhs) = default;
InternalSummaryType::Classes & InternalSummaryType::Classes::operator = (const Classes & __rhs) = default;
InternalSummaryType::Classes::Classes(Classes && __rhs) noexcept = default;
InternalSummaryType::Classes & InternalSummaryType::Classes::operator = (Classes && __rhs) noexcept = default;
InternalSummaryType::Classes::~Classes() = default; 
InternalSummaryType::Classes::Classes(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    id = ConfigParser::parse<int32_t>("id", __lines);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    omitsummaryfeatures = ConfigParser::parse<bool>("omitsummaryfeatures", __lines, false);
    ConfigParser::stripLinesForKey("omitsummaryfeatures", __remainingValuesToParse);
    fields = ConfigParser::parseArray<FieldsVector>("fields", __lines);
    ConfigParser::stripLinesForKey("fields", __remainingValuesToParse);
}

bool
InternalSummaryType::Classes::operator==(const Classes& __rhs) const noexcept
{
    return (id == __rhs.id &&
            name == __rhs.name &&
            omitsummaryfeatures == __rhs.omitsummaryfeatures &&
            fields == __rhs.fields);
}

bool
InternalSummaryType::Classes::operator!=(const Classes& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSummaryType::Classes::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "int");
        __c.setLong("value", id);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("omitsummaryfeatures");
        __c.setString("type", "bool");
        __c.setBool("value", omitsummaryfeatures);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fields");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : fields) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalSummaryType::Classes::Classes(const vespalib::slime::Inspector & __inspector)
{
    id = __inspector["id"]["value"].asLong();
    name = __inspector["name"]["value"].asString().make_string();
    omitsummaryfeatures = __inspector["omitsummaryfeatures"]["value"].asBool();
    for (size_t __i = 0; __i < __inspector["fields"]["value"].children(); __i++) {
        fields.push_back(Fields(__inspector["fields"]["value"][__i]["value"]));
    }
}

InternalSummaryType::Classes::Classes(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    id = ::config::internal::ValueConverter<int32_t>()("id", __inspector["id"]);
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    omitsummaryfeatures = ::config::internal::ValueConverter<bool>()(__inspector["omitsummaryfeatures"], false);
    ::config::internal::VectorInserter<FieldsVector> __fieldsInserter(fields);
    __inspector["fields"].traverse(__fieldsInserter);
}

InternalSummaryType::InternalSummaryType()
    : defaultsummaryid(-1),
      usev8geopositions(false),
      classes()
{
}

InternalSummaryType::InternalSummaryType(const InternalSummaryType & __rhs) = default;
InternalSummaryType & InternalSummaryType::operator =(const InternalSummaryType & __rhs) = default;
InternalSummaryType::InternalSummaryType(InternalSummaryType && __rhs) noexcept = default;
InternalSummaryType & InternalSummaryType::operator =(InternalSummaryType && __rhs) noexcept = default;
InternalSummaryType::~InternalSummaryType() = default; 
InternalSummaryType::InternalSummaryType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        defaultsummaryid = ConfigParser::parse<int32_t>("defaultsummaryid", __lines, -1);
        ConfigParser::stripLinesForKey("defaultsummaryid", __remainingValuesToParse);
        usev8geopositions = ConfigParser::parse<bool>("usev8geopositions", __lines, false);
        ConfigParser::stripLinesForKey("usev8geopositions", __remainingValuesToParse);
        classes = ConfigParser::parseArray<ClassesVector>("classes", __lines);
        ConfigParser::stripLinesForKey("classes", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalSummaryType::operator==(const InternalSummaryType& __rhs) const noexcept
{
    return (defaultsummaryid == __rhs.defaultsummaryid &&
            usev8geopositions == __rhs.usev8geopositions &&
            classes == __rhs.classes);
}

bool
InternalSummaryType::operator!=(const InternalSummaryType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalSummaryType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("defaultsummaryid");
        __c.setString("type", "int");
        __c.setLong("value", defaultsummaryid);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("usev8geopositions");
        __c.setString("type", "bool");
        __c.setBool("value", usev8geopositions);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("classes");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : classes) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalSummaryType::InternalSummaryType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    defaultsummaryid = __inspector["defaultsummaryid"]["value"].asLong();
    usev8geopositions = __inspector["usev8geopositions"]["value"].asBool();
    for (size_t __i = 0; __i < __inspector["classes"]["value"].children(); __i++) {
        classes.push_back(Classes(__inspector["classes"]["value"][__i]["value"]));
    }
}

InternalSummaryType::InternalSummaryType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    defaultsummaryid = ::config::internal::ValueConverter<int32_t>()(__inspector["defaultsummaryid"], -1);
    usev8geopositions = ::config::internal::ValueConverter<bool>()(__inspector["usev8geopositions"], false);
    ::config::internal::VectorInserter<ClassesVector> __classesInserter(classes);
    __inspector["classes"].traverse(__classesInserter);
}

} // namespace internal

} // namespace vespa::config::search
