/**
 * @class vespa::config::search::Summary
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_SEARCH_SUMMARY_H
#define CLOUD_CONFIG_VESPA_CONFIG_SEARCH_SUMMARY_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::search {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalSummaryType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalSummaryType> UP;
    class Classes {
    public:
        class Fields {
        public:
            Fields(const ::config::StringVector & __lines);
            Fields(const vespalib::slime::Inspector & __inspector);
            Fields(const ::config::ConfigPayload & __payload);
            Fields()  noexcept;
            Fields(const Fields & __rhs) noexcept;
            Fields & operator = (const Fields & __rhs) noexcept;
            Fields(Fields && __rhs) noexcept;
            Fields & operator = (Fields && __rhs) noexcept;
            ~Fields();

            bool operator==(const Fields& __rhs) const noexcept;
            bool operator!=(const Fields& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            vespalib::string name;

            /**
             * The name of the command that is writing this field.
             * See docsumconfig.cpp for all supported commands (DocsumFieldWrite-
             * r implementations).
             */
            vespalib::string command; // Default: ""

            /** The name of the source field used by the command. */
            vespalib::string source; // Default: ""
        };
        typedef std::vector<Fields> FieldsVector;

        typedef std::map<vespalib::string, Fields> FieldsMap;

        Classes(const ::config::StringVector & __lines);
        Classes(const vespalib::slime::Inspector & __inspector);
        Classes(const ::config::ConfigPayload & __payload);
        Classes() ;
        Classes(const Classes & __rhs);
        Classes & operator = (const Classes & __rhs);
        Classes(Classes && __rhs) noexcept;
        Classes & operator = (Classes && __rhs) noexcept;
        ~Classes();

        bool operator==(const Classes& __rhs) const noexcept;
        bool operator!=(const Classes& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        int32_t id;
        vespalib::string name;
        bool omitsummaryfeatures; // Default: false
        FieldsVector fields;
    };
    typedef std::vector<Classes> ClassesVector;

    typedef std::map<vespalib::string, Classes> ClassesMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalSummaryType(const ::config::ConfigValue & __value);
    InternalSummaryType(const ::config::ConfigDataBuffer & __value);
    InternalSummaryType(const ::config::ConfigPayload & __payload);
    InternalSummaryType() ;
    InternalSummaryType(const InternalSummaryType & __rhs);
    InternalSummaryType & operator = (const InternalSummaryType & __rhs);
    InternalSummaryType(InternalSummaryType && __rhs) noexcept;
    InternalSummaryType & operator = (InternalSummaryType && __rhs) noexcept;
    ~InternalSummaryType();

    bool operator==(const InternalSummaryType& __rhs) const noexcept;
    bool operator!=(const InternalSummaryType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    int32_t defaultsummaryid; // Default: -1
    bool usev8geopositions; // Default: false
    ClassesVector classes;
};

} // namespace internal

typedef internal::InternalSummaryType SummaryConfigBuilder;
typedef const internal::InternalSummaryType SummaryConfig;

} // namespace vespa::config::search
#endif // VESPA_config_VESPA_CONFIG_SEARCH_SUMMARY_H
