#include "config-upgrading.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalUpgradingType::CONFIG_DEF_MD5("06b2267e0e1998f0ffeafc53decef552");
const vespalib::string InternalUpgradingType::CONFIG_DEF_NAME("upgrading");
const vespalib::string InternalUpgradingType::CONFIG_DEF_NAMESPACE("vespa.config.content");
const int64_t InternalUpgradingType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content",
"upgrading_major_to bool default=false",
"upgrading_major_from bool default=false",
"upgrading_minor_to bool default=false",
"upgrading_minor_from bool default=false",
};
const ::config::StringVector InternalUpgradingType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalUpgradingType::InternalUpgradingType() noexcept
    : upgradingMajorTo(false),
      upgradingMajorFrom(false),
      upgradingMinorTo(false),
      upgradingMinorFrom(false)
{
}

InternalUpgradingType::InternalUpgradingType(const InternalUpgradingType & __rhs) noexcept = default;
InternalUpgradingType & InternalUpgradingType::operator =(const InternalUpgradingType & __rhs) noexcept = default;
InternalUpgradingType::InternalUpgradingType(InternalUpgradingType && __rhs) noexcept = default;
InternalUpgradingType & InternalUpgradingType::operator =(InternalUpgradingType && __rhs) noexcept = default;
InternalUpgradingType::~InternalUpgradingType() = default; 
InternalUpgradingType::InternalUpgradingType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        upgradingMajorTo = ConfigParser::parse<bool>("upgrading_major_to", __lines, false);
        ConfigParser::stripLinesForKey("upgrading_major_to", __remainingValuesToParse);
        upgradingMajorFrom = ConfigParser::parse<bool>("upgrading_major_from", __lines, false);
        ConfigParser::stripLinesForKey("upgrading_major_from", __remainingValuesToParse);
        upgradingMinorTo = ConfigParser::parse<bool>("upgrading_minor_to", __lines, false);
        ConfigParser::stripLinesForKey("upgrading_minor_to", __remainingValuesToParse);
        upgradingMinorFrom = ConfigParser::parse<bool>("upgrading_minor_from", __lines, false);
        ConfigParser::stripLinesForKey("upgrading_minor_from", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalUpgradingType::operator==(const InternalUpgradingType& __rhs) const noexcept
{
    return (upgradingMajorTo == __rhs.upgradingMajorTo &&
            upgradingMajorFrom == __rhs.upgradingMajorFrom &&
            upgradingMinorTo == __rhs.upgradingMinorTo &&
            upgradingMinorFrom == __rhs.upgradingMinorFrom);
}

bool
InternalUpgradingType::operator!=(const InternalUpgradingType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalUpgradingType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("upgrading_major_to");
        __c.setString("type", "bool");
        __c.setBool("value", upgradingMajorTo);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("upgrading_major_from");
        __c.setString("type", "bool");
        __c.setBool("value", upgradingMajorFrom);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("upgrading_minor_to");
        __c.setString("type", "bool");
        __c.setBool("value", upgradingMinorTo);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("upgrading_minor_from");
        __c.setString("type", "bool");
        __c.setBool("value", upgradingMinorFrom);
    }
}

InternalUpgradingType::InternalUpgradingType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    upgradingMajorTo = __inspector["upgrading_major_to"]["value"].asBool();
    upgradingMajorFrom = __inspector["upgrading_major_from"]["value"].asBool();
    upgradingMinorTo = __inspector["upgrading_minor_to"]["value"].asBool();
    upgradingMinorFrom = __inspector["upgrading_minor_from"]["value"].asBool();
}

InternalUpgradingType::InternalUpgradingType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    upgradingMajorTo = ::config::internal::ValueConverter<bool>()(__inspector["upgrading_major_to"], false);
    upgradingMajorFrom = ::config::internal::ValueConverter<bool>()(__inspector["upgrading_major_from"], false);
    upgradingMinorTo = ::config::internal::ValueConverter<bool>()(__inspector["upgrading_minor_to"], false);
    upgradingMinorFrom = ::config::internal::ValueConverter<bool>()(__inspector["upgrading_minor_from"], false);
}

} // namespace internal

} // namespace vespa::config::content
