/**
 * @class vespa::config::content::Upgrading
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_CONTENT_UPGRADING_H
#define CLOUD_CONFIG_VESPA_CONFIG_CONTENT_UPGRADING_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::content {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalUpgradingType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalUpgradingType> UP;
    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalUpgradingType(const ::config::ConfigValue & __value);
    InternalUpgradingType(const ::config::ConfigDataBuffer & __value);
    InternalUpgradingType(const ::config::ConfigPayload & __payload);
    InternalUpgradingType()  noexcept;
    InternalUpgradingType(const InternalUpgradingType & __rhs) noexcept;
    InternalUpgradingType & operator = (const InternalUpgradingType & __rhs) noexcept;
    InternalUpgradingType(InternalUpgradingType && __rhs) noexcept;
    InternalUpgradingType & operator = (InternalUpgradingType && __rhs) noexcept;
    ~InternalUpgradingType();

    bool operator==(const InternalUpgradingType& __rhs) const noexcept;
    bool operator!=(const InternalUpgradingType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    // Config specifying cluster global parameters for a storage cluster.

    /**
     * Set to true when there exist nodes of previous major version in cluster.
     * This will enable major version backward compability mode in the cluster.
     */
    bool upgradingMajorTo; // Default: false

    /**
     * Set to true when there exist nodes of the next major version in the
     * cluster. This will usually not be needed, unless specified in the upgrade
     * docs. Sometimes though, it may be easier to let old version be forward
     * compatable than having new version backwards compatable.
     */
    bool upgradingMajorFrom; // Default: false

    /**
     * Set to true when there exist nodes of previous minor version in cluster.
     * This should not be needed, but settings exist in case we end up having
     * no other choice than to have a compatability mode between minor versions.
     */
    bool upgradingMinorTo; // Default: false

    /**
     * Set to true when there exist nodes of next minor version in cluster.
     * This should not be needed, but settings exist in case we end up having
     * no other choice than to have a compatability mode between minor versions.
     */
    bool upgradingMinorFrom; // Default: false
};

} // namespace internal

typedef internal::InternalUpgradingType UpgradingConfigBuilder;
typedef const internal::InternalUpgradingType UpgradingConfig;

} // namespace vespa::config::content
#endif // VESPA_config_VESPA_CONFIG_CONTENT_UPGRADING_H
