#include "config-zookeeper-server.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace cloud::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalZookeeperServerType::CONFIG_DEF_MD5("f5092671b1e70c7439e7b9fa0fd0c5fb");
const vespalib::string InternalZookeeperServerType::CONFIG_DEF_NAME("zookeeper-server");
const vespalib::string InternalZookeeperServerType::CONFIG_DEF_NAMESPACE("cloud.config");
const int64_t InternalZookeeperServerType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=cloud.config",
"zooKeeperConfigFile string default=\"var/zookeeper/conf/zookeeper.cfg\"",
"tickTime int default=6000",
"initLimit int default=20",
"syncLimit int default=15",
"maxClientConnections int default=0",
"dataDir string default=\"var/zookeeper\"",
"clientPort int default=2181",
"snapshotCount int default=50000",
"autopurge.purgeInterval int default=1",
"autopurge.snapRetainCount int default=15",
"myidFile string default=\"var/zookeeper/myid\"",
"juteMaxBuffer int default=104857600",
"myid int restart",
"server[].id int",
"server[].hostname string",
"server[].clientPort int default=2181",
"server[].quorumPort int default=2182",
"server[].electionPort int default=2183",
"server[].joining bool default=false",
"server[].retired bool default=false",
"trustEmptySnapshot bool default=true",
"dynamicReconfiguration bool default=false",
"snapshotMethod string default=\"\"",
"vespaTlsConfigFile string default=\"\"",
"leaderCloseSocketAsync bool default=false",
"learnerAsyncSending bool default=false",
};
const ::config::StringVector InternalZookeeperServerType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalZookeeperServerType::Autopurge::Autopurge() noexcept
    : purgeInterval(1),
      snapRetainCount(15)
{
}

InternalZookeeperServerType::Autopurge::Autopurge(const Autopurge & __rhs) noexcept = default;
InternalZookeeperServerType::Autopurge & InternalZookeeperServerType::Autopurge::operator = (const Autopurge & __rhs) noexcept = default;
InternalZookeeperServerType::Autopurge::Autopurge(Autopurge && __rhs) noexcept = default;
InternalZookeeperServerType::Autopurge & InternalZookeeperServerType::Autopurge::operator = (Autopurge && __rhs) noexcept = default;
InternalZookeeperServerType::Autopurge::~Autopurge() = default; 
InternalZookeeperServerType::Autopurge::Autopurge(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    purgeInterval = ConfigParser::parse<int32_t>("purgeInterval", __lines, 1);
    ConfigParser::stripLinesForKey("purgeInterval", __remainingValuesToParse);
    snapRetainCount = ConfigParser::parse<int32_t>("snapRetainCount", __lines, 15);
    ConfigParser::stripLinesForKey("snapRetainCount", __remainingValuesToParse);
}

bool
InternalZookeeperServerType::Autopurge::operator==(const Autopurge& __rhs) const noexcept
{
    return (purgeInterval == __rhs.purgeInterval &&
            snapRetainCount == __rhs.snapRetainCount);
}

bool
InternalZookeeperServerType::Autopurge::operator!=(const Autopurge& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalZookeeperServerType::Autopurge::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("purgeInterval");
        __c.setString("type", "int");
        __c.setLong("value", purgeInterval);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("snapRetainCount");
        __c.setString("type", "int");
        __c.setLong("value", snapRetainCount);
    }
}

InternalZookeeperServerType::Autopurge::Autopurge(const vespalib::slime::Inspector & __inspector)
{
    purgeInterval = __inspector["purgeInterval"]["value"].asLong();
    snapRetainCount = __inspector["snapRetainCount"]["value"].asLong();
}

InternalZookeeperServerType::Autopurge::Autopurge(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    purgeInterval = ::config::internal::ValueConverter<int32_t>()(__inspector["purgeInterval"], 1);
    snapRetainCount = ::config::internal::ValueConverter<int32_t>()(__inspector["snapRetainCount"], 15);
}

InternalZookeeperServerType::Server::Server() noexcept
    : id(0),
      hostname(),
      clientPort(2181),
      quorumPort(2182),
      electionPort(2183),
      joining(false),
      retired(false)
{
}

InternalZookeeperServerType::Server::Server(const Server & __rhs) noexcept = default;
InternalZookeeperServerType::Server & InternalZookeeperServerType::Server::operator = (const Server & __rhs) noexcept = default;
InternalZookeeperServerType::Server::Server(Server && __rhs) noexcept = default;
InternalZookeeperServerType::Server & InternalZookeeperServerType::Server::operator = (Server && __rhs) noexcept = default;
InternalZookeeperServerType::Server::~Server() = default; 
InternalZookeeperServerType::Server::Server(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    id = ConfigParser::parse<int32_t>("id", __lines);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
    hostname = ConfigParser::parse<vespalib::string>("hostname", __lines);
    ConfigParser::stripLinesForKey("hostname", __remainingValuesToParse);
    clientPort = ConfigParser::parse<int32_t>("clientPort", __lines, 2181);
    ConfigParser::stripLinesForKey("clientPort", __remainingValuesToParse);
    quorumPort = ConfigParser::parse<int32_t>("quorumPort", __lines, 2182);
    ConfigParser::stripLinesForKey("quorumPort", __remainingValuesToParse);
    electionPort = ConfigParser::parse<int32_t>("electionPort", __lines, 2183);
    ConfigParser::stripLinesForKey("electionPort", __remainingValuesToParse);
    joining = ConfigParser::parse<bool>("joining", __lines, false);
    ConfigParser::stripLinesForKey("joining", __remainingValuesToParse);
    retired = ConfigParser::parse<bool>("retired", __lines, false);
    ConfigParser::stripLinesForKey("retired", __remainingValuesToParse);
}

bool
InternalZookeeperServerType::Server::operator==(const Server& __rhs) const noexcept
{
    return (id == __rhs.id &&
            hostname == __rhs.hostname &&
            clientPort == __rhs.clientPort &&
            quorumPort == __rhs.quorumPort &&
            electionPort == __rhs.electionPort &&
            joining == __rhs.joining &&
            retired == __rhs.retired);
}

bool
InternalZookeeperServerType::Server::operator!=(const Server& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalZookeeperServerType::Server::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "int");
        __c.setLong("value", id);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("hostname");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(hostname));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("clientPort");
        __c.setString("type", "int");
        __c.setLong("value", clientPort);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("quorumPort");
        __c.setString("type", "int");
        __c.setLong("value", quorumPort);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("electionPort");
        __c.setString("type", "int");
        __c.setLong("value", electionPort);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("joining");
        __c.setString("type", "bool");
        __c.setBool("value", joining);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("retired");
        __c.setString("type", "bool");
        __c.setBool("value", retired);
    }
}

InternalZookeeperServerType::Server::Server(const vespalib::slime::Inspector & __inspector)
{
    id = __inspector["id"]["value"].asLong();
    hostname = __inspector["hostname"]["value"].asString().make_string();
    clientPort = __inspector["clientPort"]["value"].asLong();
    quorumPort = __inspector["quorumPort"]["value"].asLong();
    electionPort = __inspector["electionPort"]["value"].asLong();
    joining = __inspector["joining"]["value"].asBool();
    retired = __inspector["retired"]["value"].asBool();
}

InternalZookeeperServerType::Server::Server(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    id = ::config::internal::ValueConverter<int32_t>()("id", __inspector["id"]);
    hostname = ::config::internal::ValueConverter<vespalib::string>()("hostname", __inspector["hostname"]);
    clientPort = ::config::internal::ValueConverter<int32_t>()(__inspector["clientPort"], 2181);
    quorumPort = ::config::internal::ValueConverter<int32_t>()(__inspector["quorumPort"], 2182);
    electionPort = ::config::internal::ValueConverter<int32_t>()(__inspector["electionPort"], 2183);
    joining = ::config::internal::ValueConverter<bool>()(__inspector["joining"], false);
    retired = ::config::internal::ValueConverter<bool>()(__inspector["retired"], false);
}

InternalZookeeperServerType::InternalZookeeperServerType()
    : zooKeeperConfigFile("var/zookeeper/conf/zookeeper.cfg"),
      tickTime(6000),
      initLimit(20),
      syncLimit(15),
      maxClientConnections(0),
      dataDir("var/zookeeper"),
      clientPort(2181),
      snapshotCount(50000),
      autopurge(),
      myidFile("var/zookeeper/myid"),
      juteMaxBuffer(104857600),
      myid(0),
      server(),
      trustEmptySnapshot(true),
      dynamicReconfiguration(false),
      snapshotMethod(""),
      vespaTlsConfigFile(""),
      leaderCloseSocketAsync(false),
      learnerAsyncSending(false)
{
}

InternalZookeeperServerType::InternalZookeeperServerType(const InternalZookeeperServerType & __rhs) = default;
InternalZookeeperServerType & InternalZookeeperServerType::operator =(const InternalZookeeperServerType & __rhs) = default;
InternalZookeeperServerType::InternalZookeeperServerType(InternalZookeeperServerType && __rhs) noexcept = default;
InternalZookeeperServerType & InternalZookeeperServerType::operator =(InternalZookeeperServerType && __rhs) noexcept = default;
InternalZookeeperServerType::~InternalZookeeperServerType() = default; 
InternalZookeeperServerType::InternalZookeeperServerType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        zooKeeperConfigFile = ConfigParser::parse<vespalib::string>("zooKeeperConfigFile", __lines, "var/zookeeper/conf/zookeeper.cfg");
        ConfigParser::stripLinesForKey("zooKeeperConfigFile", __remainingValuesToParse);
        tickTime = ConfigParser::parse<int32_t>("tickTime", __lines, 6000);
        ConfigParser::stripLinesForKey("tickTime", __remainingValuesToParse);
        initLimit = ConfigParser::parse<int32_t>("initLimit", __lines, 20);
        ConfigParser::stripLinesForKey("initLimit", __remainingValuesToParse);
        syncLimit = ConfigParser::parse<int32_t>("syncLimit", __lines, 15);
        ConfigParser::stripLinesForKey("syncLimit", __remainingValuesToParse);
        maxClientConnections = ConfigParser::parse<int32_t>("maxClientConnections", __lines, 0);
        ConfigParser::stripLinesForKey("maxClientConnections", __remainingValuesToParse);
        dataDir = ConfigParser::parse<vespalib::string>("dataDir", __lines, "var/zookeeper");
        ConfigParser::stripLinesForKey("dataDir", __remainingValuesToParse);
        clientPort = ConfigParser::parse<int32_t>("clientPort", __lines, 2181);
        ConfigParser::stripLinesForKey("clientPort", __remainingValuesToParse);
        snapshotCount = ConfigParser::parse<int32_t>("snapshotCount", __lines, 50000);
        ConfigParser::stripLinesForKey("snapshotCount", __remainingValuesToParse);
        autopurge = ConfigParser::parseStruct<Autopurge>("autopurge", __lines);
        ConfigParser::stripLinesForKey("autopurge", __remainingValuesToParse);
        myidFile = ConfigParser::parse<vespalib::string>("myidFile", __lines, "var/zookeeper/myid");
        ConfigParser::stripLinesForKey("myidFile", __remainingValuesToParse);
        juteMaxBuffer = ConfigParser::parse<int32_t>("juteMaxBuffer", __lines, 104857600);
        ConfigParser::stripLinesForKey("juteMaxBuffer", __remainingValuesToParse);
        myid = ConfigParser::parse<int32_t>("myid", __lines);
        ConfigParser::stripLinesForKey("myid", __remainingValuesToParse);
        server = ConfigParser::parseArray<ServerVector>("server", __lines);
        ConfigParser::stripLinesForKey("server", __remainingValuesToParse);
        trustEmptySnapshot = ConfigParser::parse<bool>("trustEmptySnapshot", __lines, true);
        ConfigParser::stripLinesForKey("trustEmptySnapshot", __remainingValuesToParse);
        dynamicReconfiguration = ConfigParser::parse<bool>("dynamicReconfiguration", __lines, false);
        ConfigParser::stripLinesForKey("dynamicReconfiguration", __remainingValuesToParse);
        snapshotMethod = ConfigParser::parse<vespalib::string>("snapshotMethod", __lines, "");
        ConfigParser::stripLinesForKey("snapshotMethod", __remainingValuesToParse);
        vespaTlsConfigFile = ConfigParser::parse<vespalib::string>("vespaTlsConfigFile", __lines, "");
        ConfigParser::stripLinesForKey("vespaTlsConfigFile", __remainingValuesToParse);
        leaderCloseSocketAsync = ConfigParser::parse<bool>("leaderCloseSocketAsync", __lines, false);
        ConfigParser::stripLinesForKey("leaderCloseSocketAsync", __remainingValuesToParse);
        learnerAsyncSending = ConfigParser::parse<bool>("learnerAsyncSending", __lines, false);
        ConfigParser::stripLinesForKey("learnerAsyncSending", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalZookeeperServerType::
operator==(const InternalZookeeperServerType& __rhs) const noexcept
{
    return (zooKeeperConfigFile == __rhs.zooKeeperConfigFile &&
            tickTime == __rhs.tickTime &&
            initLimit == __rhs.initLimit &&
            syncLimit == __rhs.syncLimit &&
            maxClientConnections == __rhs.maxClientConnections &&
            dataDir == __rhs.dataDir &&
            clientPort == __rhs.clientPort &&
            snapshotCount == __rhs.snapshotCount &&
            autopurge == __rhs.autopurge &&
            myidFile == __rhs.myidFile &&
            juteMaxBuffer == __rhs.juteMaxBuffer &&
            myid == __rhs.myid &&
            server == __rhs.server &&
            trustEmptySnapshot == __rhs.trustEmptySnapshot &&
            dynamicReconfiguration == __rhs.dynamicReconfiguration &&
            snapshotMethod == __rhs.snapshotMethod &&
            vespaTlsConfigFile == __rhs.vespaTlsConfigFile &&
            leaderCloseSocketAsync == __rhs.leaderCloseSocketAsync &&
            learnerAsyncSending == __rhs.learnerAsyncSending);
}

bool
InternalZookeeperServerType::
operator!=(const InternalZookeeperServerType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalZookeeperServerType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("zooKeeperConfigFile");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(zooKeeperConfigFile));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tickTime");
        __c.setString("type", "int");
        __c.setLong("value", tickTime);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("initLimit");
        __c.setString("type", "int");
        __c.setLong("value", initLimit);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("syncLimit");
        __c.setString("type", "int");
        __c.setLong("value", syncLimit);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxClientConnections");
        __c.setString("type", "int");
        __c.setLong("value", maxClientConnections);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("dataDir");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(dataDir));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("clientPort");
        __c.setString("type", "int");
        __c.setLong("value", clientPort);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("snapshotCount");
        __c.setString("type", "int");
        __c.setLong("value", snapshotCount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("autopurge");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        autopurge.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("myidFile");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(myidFile));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("juteMaxBuffer");
        __c.setString("type", "int");
        __c.setLong("value", juteMaxBuffer);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("myid");
        __c.setString("type", "int");
        __c.setLong("value", myid);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("server");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : server) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("trustEmptySnapshot");
        __c.setString("type", "bool");
        __c.setBool("value", trustEmptySnapshot);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("dynamicReconfiguration");
        __c.setString("type", "bool");
        __c.setBool("value", dynamicReconfiguration);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("snapshotMethod");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(snapshotMethod));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("vespaTlsConfigFile");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(vespaTlsConfigFile));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("leaderCloseSocketAsync");
        __c.setString("type", "bool");
        __c.setBool("value", leaderCloseSocketAsync);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("learnerAsyncSending");
        __c.setString("type", "bool");
        __c.setBool("value", learnerAsyncSending);
    }
}

InternalZookeeperServerType::InternalZookeeperServerType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    zooKeeperConfigFile = __inspector["zooKeeperConfigFile"]["value"].asString().make_string();
    tickTime = __inspector["tickTime"]["value"].asLong();
    initLimit = __inspector["initLimit"]["value"].asLong();
    syncLimit = __inspector["syncLimit"]["value"].asLong();
    maxClientConnections = __inspector["maxClientConnections"]["value"].asLong();
    dataDir = __inspector["dataDir"]["value"].asString().make_string();
    clientPort = __inspector["clientPort"]["value"].asLong();
    snapshotCount = __inspector["snapshotCount"]["value"].asLong();
    autopurge = Autopurge(__inspector["autopurge"]["value"]);
    myidFile = __inspector["myidFile"]["value"].asString().make_string();
    juteMaxBuffer = __inspector["juteMaxBuffer"]["value"].asLong();
    myid = __inspector["myid"]["value"].asLong();
    for (size_t __i = 0; __i < __inspector["server"]["value"].children(); __i++) {
        server.push_back(Server(__inspector["server"]["value"][__i]["value"]));
    }
    trustEmptySnapshot = __inspector["trustEmptySnapshot"]["value"].asBool();
    dynamicReconfiguration = __inspector["dynamicReconfiguration"]["value"].asBool();
    snapshotMethod = __inspector["snapshotMethod"]["value"].asString().make_string();
    vespaTlsConfigFile = __inspector["vespaTlsConfigFile"]["value"].asString().make_string();
    leaderCloseSocketAsync = __inspector["leaderCloseSocketAsync"]["value"].asBool();
    learnerAsyncSending = __inspector["learnerAsyncSending"]["value"].asBool();
}

InternalZookeeperServerType::InternalZookeeperServerType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    zooKeeperConfigFile = ::config::internal::ValueConverter<vespalib::string>()(__inspector["zooKeeperConfigFile"], "var/zookeeper/conf/zookeeper.cfg");
    tickTime = ::config::internal::ValueConverter<int32_t>()(__inspector["tickTime"], 6000);
    initLimit = ::config::internal::ValueConverter<int32_t>()(__inspector["initLimit"], 20);
    syncLimit = ::config::internal::ValueConverter<int32_t>()(__inspector["syncLimit"], 15);
    maxClientConnections = ::config::internal::ValueConverter<int32_t>()(__inspector["maxClientConnections"], 0);
    dataDir = ::config::internal::ValueConverter<vespalib::string>()(__inspector["dataDir"], "var/zookeeper");
    clientPort = ::config::internal::ValueConverter<int32_t>()(__inspector["clientPort"], 2181);
    snapshotCount = ::config::internal::ValueConverter<int32_t>()(__inspector["snapshotCount"], 50000);
    autopurge = ::config::internal::ValueConverter<Autopurge>()(__inspector["autopurge"]);
    myidFile = ::config::internal::ValueConverter<vespalib::string>()(__inspector["myidFile"], "var/zookeeper/myid");
    juteMaxBuffer = ::config::internal::ValueConverter<int32_t>()(__inspector["juteMaxBuffer"], 104857600);
    myid = ::config::internal::ValueConverter<int32_t>()("myid", __inspector["myid"]);
    ::config::internal::VectorInserter<ServerVector> __serverInserter(server);
    __inspector["server"].traverse(__serverInserter);
    trustEmptySnapshot = ::config::internal::ValueConverter<bool>()(__inspector["trustEmptySnapshot"], true);
    dynamicReconfiguration = ::config::internal::ValueConverter<bool>()(__inspector["dynamicReconfiguration"], false);
    snapshotMethod = ::config::internal::ValueConverter<vespalib::string>()(__inspector["snapshotMethod"], "");
    vespaTlsConfigFile = ::config::internal::ValueConverter<vespalib::string>()(__inspector["vespaTlsConfigFile"], "");
    leaderCloseSocketAsync = ::config::internal::ValueConverter<bool>()(__inspector["leaderCloseSocketAsync"], false);
    learnerAsyncSending = ::config::internal::ValueConverter<bool>()(__inspector["learnerAsyncSending"], false);
}

} // namespace internal

} // namespace cloud::config
