/**
 * @class cloud::config::ZookeeperServer
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_CLOUD_CONFIG_ZOOKEEPERSERVER_H
#define CLOUD_CONFIG_CLOUD_CONFIG_ZOOKEEPERSERVER_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace cloud::config {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalZookeeperServerType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalZookeeperServerType> UP;
    class Autopurge {
    public:
        Autopurge(const ::config::StringVector & __lines);
        Autopurge(const vespalib::slime::Inspector & __inspector);
        Autopurge(const ::config::ConfigPayload & __payload);
        Autopurge()  noexcept;
        Autopurge(const Autopurge & __rhs) noexcept;
        Autopurge & operator = (const Autopurge & __rhs) noexcept;
        Autopurge(Autopurge && __rhs) noexcept;
        Autopurge & operator = (Autopurge && __rhs) noexcept;
        ~Autopurge();

        bool operator==(const Autopurge& __rhs) const noexcept;
        bool operator!=(const Autopurge& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** Purge interval in hours */
        int32_t purgeInterval; // Default: 1
        int32_t snapRetainCount; // Default: 15
    };
    typedef std::vector<Autopurge> AutopurgeVector;

    typedef std::map<vespalib::string, Autopurge> AutopurgeMap;

    class Server {
    public:
        Server(const ::config::StringVector & __lines);
        Server(const vespalib::slime::Inspector & __inspector);
        Server(const ::config::ConfigPayload & __payload);
        Server()  noexcept;
        Server(const Server & __rhs) noexcept;
        Server & operator = (const Server & __rhs) noexcept;
        Server(Server && __rhs) noexcept;
        Server & operator = (Server && __rhs) noexcept;
        ~Server();

        bool operator==(const Server& __rhs) const noexcept;
        bool operator!=(const Server& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        int32_t id;
        vespalib::string hostname;
        int32_t clientPort; // Default: 2181
        int32_t quorumPort; // Default: 2182
        int32_t electionPort; // Default: 2183

        /** Whether this server is joining an existing cluster */
        bool joining; // Default: false

        /** Whether this server is retired, and about to be removed */
        bool retired; // Default: false
    };
    typedef std::vector<Server> ServerVector;

    typedef std::map<vespalib::string, Server> ServerMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalZookeeperServerType(const ::config::ConfigValue & __value);
    InternalZookeeperServerType(const ::config::ConfigDataBuffer & __value);
    InternalZookeeperServerType(const ::config::ConfigPayload & __payload);
    InternalZookeeperServerType() ;
    InternalZookeeperServerType(const InternalZookeeperServerType & __rhs);
    InternalZookeeperServerType & operator = (const InternalZookeeperServerType & __rhs);
    InternalZookeeperServerType(InternalZookeeperServerType && __rhs) noexcept;
    InternalZookeeperServerType & operator = (InternalZookeeperServerType && __rhs) noexcept;
    ~InternalZookeeperServerType();

    bool operator==(const InternalZookeeperServerType& __rhs) const noexcept;
    bool operator!=(const InternalZookeeperServerType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /** Vespa home is prepended if the file is relative */
    vespalib::string zooKeeperConfigFile; // Default: "var/zookeeper/conf/zookeeper.cfg"

    // For more info about the values below, see ZooKeeper documentation

    /**
     * tick time in milliseconds, min and max session timeout are 2 and 20 times
     * this value
     */
    int32_t tickTime; // Default: 6000
    int32_t initLimit; // Default: 20
    int32_t syncLimit; // Default: 15
    int32_t maxClientConnections; // Default: 0

    /** Vespa home is prepended if the file is relative */
    vespalib::string dataDir; // Default: "var/zookeeper"
    int32_t clientPort; // Default: 2181
    int32_t snapshotCount; // Default: 50000
    Autopurge autopurge;

    /** Vespa home is prepended if the file is relative */
    vespalib::string myidFile; // Default: "var/zookeeper/myid"

    /** Change from default of 1 Mb in zookeeper to 100 Mb */
    int32_t juteMaxBuffer; // Default: 104857600
    int32_t myid;
    ServerVector server;

    /**
     * Needed when upgrading from ZooKeeper 3.4 to 3.5, see https://issues.apach-
     * e.org/jira/browse/ZOOKEEPER-3056,
     * and in general where there is a zookeeper ensemble running that has had
     * few transactions.
     * TODO: Consider setting this to false by default (and override where
     * appropriate)
     */
    bool trustEmptySnapshot; // Default: true
    bool dynamicReconfiguration; // Default: false

    /**
     * Set the snapshot compression method: "gz" for GZIP or "snappy" for
     * Snappy. Any other value disables compression.
     */
    vespalib::string snapshotMethod; // Default: ""

    /** Uses default Vespa mTLS config if empty string */
    vespalib::string vespaTlsConfigFile; // Default: ""
    bool leaderCloseSocketAsync; // Default: false
    bool learnerAsyncSending; // Default: false
};

} // namespace internal

typedef internal::InternalZookeeperServerType ZookeeperServerConfigBuilder;
typedef const internal::InternalZookeeperServerType ZookeeperServerConfig;

} // namespace cloud::config
#endif // VESPA_config_CLOUD_CONFIG_ZOOKEEPERSERVER_H
