#include "config-zookeepers.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace cloud::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalZookeepersType::CONFIG_DEF_MD5("0e5b7849e3e6b63b8b7691fdef9e34bb");
const vespalib::string InternalZookeepersType::CONFIG_DEF_NAME("zookeepers");
const vespalib::string InternalZookeepersType::CONFIG_DEF_NAMESPACE("cloud.config");
const int64_t InternalZookeepersType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=cloud.config",
"zookeeperserverlist string",
};
const ::config::StringVector InternalZookeepersType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalZookeepersType::InternalZookeepersType() noexcept
    : zookeeperserverlist()
{
}

InternalZookeepersType::InternalZookeepersType(const InternalZookeepersType & __rhs) noexcept = default;
InternalZookeepersType & InternalZookeepersType::operator =(const InternalZookeepersType & __rhs) noexcept = default;
InternalZookeepersType::InternalZookeepersType(InternalZookeepersType && __rhs) noexcept = default;
InternalZookeepersType & InternalZookeepersType::operator =(InternalZookeepersType && __rhs) noexcept = default;
InternalZookeepersType::~InternalZookeepersType() = default; 
InternalZookeepersType::InternalZookeepersType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        zookeeperserverlist = ConfigParser::parse<vespalib::string>("zookeeperserverlist", __lines);
        ConfigParser::stripLinesForKey("zookeeperserverlist", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalZookeepersType::operator==(const InternalZookeepersType& __rhs) const noexcept
{
    return (zookeeperserverlist == __rhs.zookeeperserverlist);
}

bool
InternalZookeepersType::operator!=(const InternalZookeepersType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalZookeepersType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("zookeeperserverlist");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(zookeeperserverlist));
    }
}

InternalZookeepersType::InternalZookeepersType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    zookeeperserverlist = __inspector["zookeeperserverlist"]["value"].asString().make_string();
}

InternalZookeepersType::InternalZookeepersType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    zookeeperserverlist = ::config::internal::ValueConverter<vespalib::string>()("zookeeperserverlist", __inspector["zookeeperserverlist"]);
}

} // namespace internal

} // namespace cloud::config
