#include "config-documenttypes.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace document::config {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalDocumenttypesType::CONFIG_DEF_MD5("ae358370c3b44ec6e365d44dff202ae4");
const vespalib::string InternalDocumenttypesType::CONFIG_DEF_NAME("documenttypes");
const vespalib::string InternalDocumenttypesType::CONFIG_DEF_NAMESPACE("document.config");
const int64_t InternalDocumenttypesType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=document.config",
"ignoreundefinedfields bool default=false",
"usev8geopositions bool default=false",
"documenttype[].id int",
"documenttype[].name string",
"documenttype[].version int default=0",
"documenttype[].headerstruct int",
"documenttype[].bodystruct int default=0",
"documenttype[].inherits[].id int",
"documenttype[].datatype[].id int",
"documenttype[].datatype[].type enum {STRUCT, ARRAY, WSET, MAP, ANNOTATIONREF, PRIMITIVE, TENSOR}",
"documenttype[].datatype[].array.element.id int default=0",
"documenttype[].datatype[].map.key.id int default=0",
"documenttype[].datatype[].map.value.id int default=0",
"documenttype[].datatype[].wset.key.id int default=0",
"documenttype[].datatype[].wset.createifnonexistent bool default=false",
"documenttype[].datatype[].wset.removeifzero bool default=false",
"documenttype[].datatype[].annotationref.annotation.id int default=0",
"documenttype[].datatype[].sstruct.name string default=\"\"",
"documenttype[].datatype[].sstruct.version int default=0",
"documenttype[].datatype[].sstruct.compression.type enum {NONE, LZ4} default=NONE",
"documenttype[].datatype[].sstruct.compression.level int default=0",
"documenttype[].datatype[].sstruct.compression.threshold int default=95",
"documenttype[].datatype[].sstruct.compression.minsize int default=200",
"documenttype[].datatype[].sstruct.field[].name string",
"documenttype[].datatype[].sstruct.field[].id int",
"documenttype[].datatype[].sstruct.field[].datatype int",
"documenttype[].datatype[].sstruct.field[].detailedtype string default=\"\"",
"documenttype[].annotationtype[].id int",
"documenttype[].annotationtype[].name string",
"documenttype[].annotationtype[].datatype int default=-1",
"documenttype[].annotationtype[].inherits[].id int",
"documenttype[].fieldsets{}.fields[] string",
"documenttype[].referencetype[].id int",
"documenttype[].referencetype[].target_type_id int",
"documenttype[].importedfield[].name string",
"doctype[].name string",
"doctype[].idx int",
"doctype[].internalid int",
"doctype[].inherits[].idx int",
"doctype[].contentstruct int",
"doctype[].fieldsets{}.fields[] string",
"doctype[].importedfield[].name string",
"doctype[].primitivetype[].idx int",
"doctype[].primitivetype[].name string",
"doctype[].arraytype[].idx int",
"doctype[].arraytype[].elementtype int",
"doctype[].arraytype[].internalid int",
"doctype[].maptype[].idx int",
"doctype[].maptype[].keytype int",
"doctype[].maptype[].valuetype int",
"doctype[].maptype[].internalid int",
"doctype[].wsettype[].idx int",
"doctype[].wsettype[].elementtype int",
"doctype[].wsettype[].createifnonexistent bool default=false",
"doctype[].wsettype[].removeifzero bool default=false",
"doctype[].wsettype[].internalid int",
"doctype[].tensortype[].idx int",
"doctype[].tensortype[].detailedtype string",
"doctype[].documentref[].idx int",
"doctype[].documentref[].targettype int",
"doctype[].documentref[].internalid int",
"doctype[].annotationtype[].idx int",
"doctype[].annotationtype[].name string",
"doctype[].annotationtype[].internalid int",
"doctype[].annotationtype[].datatype int default=-1",
"doctype[].annotationtype[].inherits[].idx int",
"doctype[].annotationref[].idx int",
"doctype[].annotationref[].annotationtype int",
"doctype[].annotationref[].internalid int",
"doctype[].structtype[].idx int",
"doctype[].structtype[].name string",
"doctype[].structtype[].inherits[].type int",
"doctype[].structtype[].field[].name string",
"doctype[].structtype[].field[].internalid int",
"doctype[].structtype[].field[].type int",
"doctype[].structtype[].internalid int",
};
const ::config::StringVector InternalDocumenttypesType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalDocumenttypesType::Documenttype::Inherits::Inherits() noexcept
    : id(0)
{
}

InternalDocumenttypesType::Documenttype::Inherits::Inherits(const Inherits & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Inherits & InternalDocumenttypesType::Documenttype::Inherits::operator = (const Inherits & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Inherits::Inherits(Inherits && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Inherits & InternalDocumenttypesType::Documenttype::Inherits::operator = (Inherits && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Inherits::~Inherits() = default; 
InternalDocumenttypesType::Documenttype::Inherits::Inherits(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    id = ConfigParser::parse<int32_t>("id", __lines);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Inherits::
operator==(const Inherits& __rhs) const noexcept
{
    return (id == __rhs.id);
}

bool
InternalDocumenttypesType::Documenttype::Inherits::
operator!=(const Inherits& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Inherits::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "int");
        __c.setLong("value", id);
    }
}

InternalDocumenttypesType::Documenttype::Inherits::Inherits(const vespalib::slime::Inspector & __inspector)
{
    id = __inspector["id"]["value"].asLong();
}

InternalDocumenttypesType::Documenttype::Inherits::Inherits(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    id = ::config::internal::ValueConverter<int32_t>()("id", __inspector["id"]);
}

InternalDocumenttypesType::Documenttype::Datatype::Type
InternalDocumenttypesType::Documenttype::Datatype::getType(const vespalib::string& name)
{
    if (name == "STRUCT") {
        return Type::STRUCT;
    } else if (name == "ARRAY") {
        return Type::ARRAY;
    } else if (name == "WSET") {
        return Type::WSET;
    } else if (name == "MAP") {
        return Type::MAP;
    } else if (name == "ANNOTATIONREF") {
        return Type::ANNOTATIONREF;
    } else if (name == "PRIMITIVE") {
        return Type::PRIMITIVE;
    } else if (name == "TENSOR") {
        return Type::TENSOR;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalDocumenttypesType::Documenttype::Datatype::getTypeName(Type t)
{
    switch (t) {
        case Type::STRUCT: return "STRUCT";
        case Type::ARRAY: return "ARRAY";
        case Type::WSET: return "WSET";
        case Type::MAP: return "MAP";
        case Type::ANNOTATIONREF: return "ANNOTATIONREF";
        case Type::PRIMITIVE: return "PRIMITIVE";
        case Type::TENSOR: return "TENSOR";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Type InternalDocumenttypesType::Documenttype::Datatype::InternalTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalDocumenttypesType::Documenttype::Datatype::getType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalDocumenttypesType::Documenttype::Datatype::Type InternalDocumenttypesType::Documenttype::Datatype::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalDocumenttypesType::Documenttype::Datatype::getType(__inspector.asString().make_string());
}
InternalDocumenttypesType::Documenttype::Datatype::Type InternalDocumenttypesType::Documenttype::Datatype::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault) {
    if (__inspector.valid()) {
        return InternalDocumenttypesType::Documenttype::Datatype::getType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalDocumenttypesType::Documenttype::Datatype::Array::Element::Element() noexcept
    : id(0)
{
}

InternalDocumenttypesType::Documenttype::Datatype::Array::Element::Element(const Element & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Array::Element & InternalDocumenttypesType::Documenttype::Datatype::Array::Element::operator = (const Element & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Array::Element::Element(Element && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Array::Element & InternalDocumenttypesType::Documenttype::Datatype::Array::Element::operator = (Element && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Array::Element::~Element() = default; 
InternalDocumenttypesType::Documenttype::Datatype::Array::Element::Element(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    id = ConfigParser::parse<int32_t>("id", __lines, 0);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Array::Element::
operator==(const Element& __rhs) const noexcept
{
    return (id == __rhs.id);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Array::Element::
operator!=(const Element& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Datatype::Array::Element::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "int");
        __c.setLong("value", id);
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Array::Element::Element(const vespalib::slime::Inspector & __inspector)
{
    id = __inspector["id"]["value"].asLong();
}

InternalDocumenttypesType::Documenttype::Datatype::Array::Element::Element(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    id = ::config::internal::ValueConverter<int32_t>()(__inspector["id"], 0);
}

InternalDocumenttypesType::Documenttype::Datatype::Array::Array() noexcept
    : element()
{
}

InternalDocumenttypesType::Documenttype::Datatype::Array::Array(const Array & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Array & InternalDocumenttypesType::Documenttype::Datatype::Array::operator = (const Array & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Array::Array(Array && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Array & InternalDocumenttypesType::Documenttype::Datatype::Array::operator = (Array && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Array::~Array() = default; 
InternalDocumenttypesType::Documenttype::Datatype::Array::Array(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    element = ConfigParser::parseStruct<Element>("element", __lines);
    ConfigParser::stripLinesForKey("element", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Array::
operator==(const Array& __rhs) const noexcept
{
    return (element == __rhs.element);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Array::
operator!=(const Array& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Datatype::Array::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("element");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        element.serialize(__c2);
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Array::Array(const vespalib::slime::Inspector & __inspector)
{
    element = Element(__inspector["element"]["value"]);
}

InternalDocumenttypesType::Documenttype::Datatype::Array::Array(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    element = ::config::internal::ValueConverter<Element>()(__inspector["element"]);
}

InternalDocumenttypesType::Documenttype::Datatype::Map::Key::Key() noexcept
    : id(0)
{
}

InternalDocumenttypesType::Documenttype::Datatype::Map::Key::Key(const Key & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Map::Key & InternalDocumenttypesType::Documenttype::Datatype::Map::Key::operator = (const Key & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Map::Key::Key(Key && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Map::Key & InternalDocumenttypesType::Documenttype::Datatype::Map::Key::operator = (Key && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Map::Key::~Key() = default; 
InternalDocumenttypesType::Documenttype::Datatype::Map::Key::Key(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    id = ConfigParser::parse<int32_t>("id", __lines, 0);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Map::Key::
operator==(const Key& __rhs) const noexcept
{
    return (id == __rhs.id);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Map::Key::
operator!=(const Key& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Datatype::Map::Key::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "int");
        __c.setLong("value", id);
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Map::Key::Key(const vespalib::slime::Inspector & __inspector)
{
    id = __inspector["id"]["value"].asLong();
}

InternalDocumenttypesType::Documenttype::Datatype::Map::Key::Key(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    id = ::config::internal::ValueConverter<int32_t>()(__inspector["id"], 0);
}

InternalDocumenttypesType::Documenttype::Datatype::Map::Value::Value() noexcept
    : id(0)
{
}

InternalDocumenttypesType::Documenttype::Datatype::Map::Value::Value(const Value & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Map::Value & InternalDocumenttypesType::Documenttype::Datatype::Map::Value::operator = (const Value & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Map::Value::Value(Value && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Map::Value & InternalDocumenttypesType::Documenttype::Datatype::Map::Value::operator = (Value && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Map::Value::~Value() = default; 
InternalDocumenttypesType::Documenttype::Datatype::Map::Value::Value(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    id = ConfigParser::parse<int32_t>("id", __lines, 0);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Map::Value::
operator==(const Value& __rhs) const noexcept
{
    return (id == __rhs.id);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Map::Value::
operator!=(const Value& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Datatype::Map::Value::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "int");
        __c.setLong("value", id);
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Map::Value::Value(const vespalib::slime::Inspector & __inspector)
{
    id = __inspector["id"]["value"].asLong();
}

InternalDocumenttypesType::Documenttype::Datatype::Map::Value::Value(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    id = ::config::internal::ValueConverter<int32_t>()(__inspector["id"], 0);
}

InternalDocumenttypesType::Documenttype::Datatype::Map::Map() noexcept
    : key(),
      value()
{
}

InternalDocumenttypesType::Documenttype::Datatype::Map::Map(const Map & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Map & InternalDocumenttypesType::Documenttype::Datatype::Map::operator = (const Map & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Map::Map(Map && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Map & InternalDocumenttypesType::Documenttype::Datatype::Map::operator = (Map && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Map::~Map() = default; 
InternalDocumenttypesType::Documenttype::Datatype::Map::Map(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    key = ConfigParser::parseStruct<Key>("key", __lines);
    ConfigParser::stripLinesForKey("key", __remainingValuesToParse);
    value = ConfigParser::parseStruct<Value>("value", __lines);
    ConfigParser::stripLinesForKey("value", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Map::
operator==(const Map& __rhs) const noexcept
{
    return (key == __rhs.key &&
            value == __rhs.value);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Map::
operator!=(const Map& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Datatype::Map::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("key");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        key.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("value");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        value.serialize(__c2);
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Map::Map(const vespalib::slime::Inspector & __inspector)
{
    key = Key(__inspector["key"]["value"]);
    value = Value(__inspector["value"]["value"]);
}

InternalDocumenttypesType::Documenttype::Datatype::Map::Map(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    key = ::config::internal::ValueConverter<Key>()(__inspector["key"]);
    value = ::config::internal::ValueConverter<Value>()(__inspector["value"]);
}

InternalDocumenttypesType::Documenttype::Datatype::Wset::Key::Key() noexcept
    : id(0)
{
}

InternalDocumenttypesType::Documenttype::Datatype::Wset::Key::Key(const Key & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Wset::Key & InternalDocumenttypesType::Documenttype::Datatype::Wset::Key::operator = (const Key & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Wset::Key::Key(Key && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Wset::Key & InternalDocumenttypesType::Documenttype::Datatype::Wset::Key::operator = (Key && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Wset::Key::~Key() = default; 
InternalDocumenttypesType::Documenttype::Datatype::Wset::Key::Key(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    id = ConfigParser::parse<int32_t>("id", __lines, 0);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Wset::Key::
operator==(const Key& __rhs) const noexcept
{
    return (id == __rhs.id);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Wset::Key::
operator!=(const Key& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Datatype::Wset::Key::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "int");
        __c.setLong("value", id);
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Wset::Key::Key(const vespalib::slime::Inspector & __inspector)
{
    id = __inspector["id"]["value"].asLong();
}

InternalDocumenttypesType::Documenttype::Datatype::Wset::Key::Key(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    id = ::config::internal::ValueConverter<int32_t>()(__inspector["id"], 0);
}

InternalDocumenttypesType::Documenttype::Datatype::Wset::Wset() noexcept
    : key(),
      createifnonexistent(false),
      removeifzero(false)
{
}

InternalDocumenttypesType::Documenttype::Datatype::Wset::Wset(const Wset & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Wset & InternalDocumenttypesType::Documenttype::Datatype::Wset::operator = (const Wset & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Wset::Wset(Wset && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Wset & InternalDocumenttypesType::Documenttype::Datatype::Wset::operator = (Wset && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Wset::~Wset() = default; 
InternalDocumenttypesType::Documenttype::Datatype::Wset::Wset(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    key = ConfigParser::parseStruct<Key>("key", __lines);
    ConfigParser::stripLinesForKey("key", __remainingValuesToParse);
    createifnonexistent = ConfigParser::parse<bool>("createifnonexistent", __lines, false);
    ConfigParser::stripLinesForKey("createifnonexistent", __remainingValuesToParse);
    removeifzero = ConfigParser::parse<bool>("removeifzero", __lines, false);
    ConfigParser::stripLinesForKey("removeifzero", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Wset::
operator==(const Wset& __rhs) const noexcept
{
    return (key == __rhs.key &&
            createifnonexistent == __rhs.createifnonexistent &&
            removeifzero == __rhs.removeifzero);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Wset::
operator!=(const Wset& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Datatype::Wset::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("key");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        key.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("createifnonexistent");
        __c.setString("type", "bool");
        __c.setBool("value", createifnonexistent);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("removeifzero");
        __c.setString("type", "bool");
        __c.setBool("value", removeifzero);
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Wset::Wset(const vespalib::slime::Inspector & __inspector)
{
    key = Key(__inspector["key"]["value"]);
    createifnonexistent = __inspector["createifnonexistent"]["value"].asBool();
    removeifzero = __inspector["removeifzero"]["value"].asBool();
}

InternalDocumenttypesType::Documenttype::Datatype::Wset::Wset(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    key = ::config::internal::ValueConverter<Key>()(__inspector["key"]);
    createifnonexistent = ::config::internal::ValueConverter<bool>()(__inspector["createifnonexistent"], false);
    removeifzero = ::config::internal::ValueConverter<bool>()(__inspector["removeifzero"], false);
}

InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotation::Annotation() noexcept
    : id(0)
{
}

InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotation::Annotation(const Annotation & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotation & InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotation::operator = (const Annotation & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotation::Annotation(Annotation && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotation & InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotation::operator = (Annotation && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotation::~Annotation() = default; 
InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotation::Annotation(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    id = ConfigParser::parse<int32_t>("id", __lines, 0);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotation::
operator==(const Annotation& __rhs) const noexcept
{
    return (id == __rhs.id);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotation::
operator!=(const Annotation& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotation::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "int");
        __c.setLong("value", id);
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotation::Annotation(const vespalib::slime::Inspector & __inspector)
{
    id = __inspector["id"]["value"].asLong();
}

InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotation::Annotation(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    id = ::config::internal::ValueConverter<int32_t>()(__inspector["id"], 0);
}

InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotationref() noexcept
    : annotation()
{
}

InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotationref(const Annotationref & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Annotationref & InternalDocumenttypesType::Documenttype::Datatype::Annotationref::operator = (const Annotationref & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotationref(Annotationref && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Annotationref & InternalDocumenttypesType::Documenttype::Datatype::Annotationref::operator = (Annotationref && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Annotationref::~Annotationref() = default; 
InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotationref(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    annotation = ConfigParser::parseStruct<Annotation>("annotation", __lines);
    ConfigParser::stripLinesForKey("annotation", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Annotationref::
operator==(const Annotationref& __rhs) const noexcept
{
    return (annotation == __rhs.annotation);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Annotationref::
operator!=(const Annotationref& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Datatype::Annotationref::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("annotation");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        annotation.serialize(__c2);
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotationref(const vespalib::slime::Inspector & __inspector)
{
    annotation = Annotation(__inspector["annotation"]["value"]);
}

InternalDocumenttypesType::Documenttype::Datatype::Annotationref::Annotationref(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    annotation = ::config::internal::ValueConverter<Annotation>()(__inspector["annotation"]);
}

InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::Type
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::getType(const vespalib::string& name)
{
    if (name == "NONE") {
        return Type::NONE;
    } else if (name == "LZ4") {
        return Type::LZ4;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::getTypeName(Type t)
{
    switch (t) {
        case Type::NONE: return "NONE";
        case Type::LZ4: return "LZ4";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::Type InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::InternalTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::getType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::Type InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::getType(__inspector.asString().make_string());
}
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::Type InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault) {
    if (__inspector.valid()) {
        return InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::getType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::Compression() noexcept
    : type(Type::NONE),
      level(0),
      threshold(95),
      minsize(200)
{
}

InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::Compression(const Compression & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression & InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::operator = (const Compression & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::Compression(Compression && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression & InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::operator = (Compression && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::~Compression() = default; 
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::Compression(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    type = getType(ConfigParser::parse<vespalib::string>("type", __lines, "NONE"));
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
    level = ConfigParser::parse<int32_t>("level", __lines, 0);
    ConfigParser::stripLinesForKey("level", __remainingValuesToParse);
    threshold = ConfigParser::parse<int32_t>("threshold", __lines, 95);
    ConfigParser::stripLinesForKey("threshold", __remainingValuesToParse);
    minsize = ConfigParser::parse<int32_t>("minsize", __lines, 200);
    ConfigParser::stripLinesForKey("minsize", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::
operator==(const Compression& __rhs) const noexcept
{
    return (type == __rhs.type &&
            level == __rhs.level &&
            threshold == __rhs.threshold &&
            minsize == __rhs.minsize);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::
operator!=(const Compression& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getTypeName(type)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("level");
        __c.setString("type", "int");
        __c.setLong("value", level);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("threshold");
        __c.setString("type", "int");
        __c.setLong("value", threshold);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("minsize");
        __c.setString("type", "int");
        __c.setLong("value", minsize);
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::Compression(const vespalib::slime::Inspector & __inspector)
{
    type = getType(__inspector["type"]["value"].asString().make_string());
    level = __inspector["level"]["value"].asLong();
    threshold = __inspector["threshold"]["value"].asLong();
    minsize = __inspector["minsize"]["value"].asLong();
}

InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Compression::Compression(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    type = InternalTypeConverter()(__inspector["type"], Type::NONE);
    level = ::config::internal::ValueConverter<int32_t>()(__inspector["level"], 0);
    threshold = ::config::internal::ValueConverter<int32_t>()(__inspector["threshold"], 95);
    minsize = ::config::internal::ValueConverter<int32_t>()(__inspector["minsize"], 200);
}

InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Field::Field() noexcept
    : name(),
      id(0),
      datatype(0),
      detailedtype("")
{
}

InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Field::Field(const Field & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Field & InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Field::operator = (const Field & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Field::Field(Field && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Field & InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Field::operator = (Field && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Field::~Field() = default; 
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Field::Field(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    id = ConfigParser::parse<int32_t>("id", __lines);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
    datatype = ConfigParser::parse<int32_t>("datatype", __lines);
    ConfigParser::stripLinesForKey("datatype", __remainingValuesToParse);
    detailedtype = ConfigParser::parse<vespalib::string>("detailedtype", __lines, "");
    ConfigParser::stripLinesForKey("detailedtype", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Field::
operator==(const Field& __rhs) const noexcept
{
    return (name == __rhs.name &&
            id == __rhs.id &&
            datatype == __rhs.datatype &&
            detailedtype == __rhs.detailedtype);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Field::
operator!=(const Field& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Field::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "int");
        __c.setLong("value", id);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("datatype");
        __c.setString("type", "int");
        __c.setLong("value", datatype);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("detailedtype");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(detailedtype));
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Field::Field(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    id = __inspector["id"]["value"].asLong();
    datatype = __inspector["datatype"]["value"].asLong();
    detailedtype = __inspector["detailedtype"]["value"].asString().make_string();
}

InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Field::Field(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    id = ::config::internal::ValueConverter<int32_t>()("id", __inspector["id"]);
    datatype = ::config::internal::ValueConverter<int32_t>()("datatype", __inspector["datatype"]);
    detailedtype = ::config::internal::ValueConverter<vespalib::string>()(__inspector["detailedtype"], "");
}

InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Sstruct()
    : name(""),
      version(0),
      compression(),
      field()
{
}

InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Sstruct(const Sstruct & __rhs) = default;
InternalDocumenttypesType::Documenttype::Datatype::Sstruct & InternalDocumenttypesType::Documenttype::Datatype::Sstruct::operator = (const Sstruct & __rhs) = default;
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Sstruct(Sstruct && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Sstruct & InternalDocumenttypesType::Documenttype::Datatype::Sstruct::operator = (Sstruct && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::~Sstruct() = default; 
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Sstruct(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines, "");
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    version = ConfigParser::parse<int32_t>("version", __lines, 0);
    ConfigParser::stripLinesForKey("version", __remainingValuesToParse);
    compression = ConfigParser::parseStruct<Compression>("compression", __lines);
    ConfigParser::stripLinesForKey("compression", __remainingValuesToParse);
    field = ConfigParser::parseArray<FieldVector>("field", __lines);
    ConfigParser::stripLinesForKey("field", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::
operator==(const Sstruct& __rhs) const noexcept
{
    return (name == __rhs.name &&
            version == __rhs.version &&
            compression == __rhs.compression &&
            field == __rhs.field);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::
operator!=(const Sstruct& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Datatype::Sstruct::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("version");
        __c.setString("type", "int");
        __c.setLong("value", version);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("compression");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        compression.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("field");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : field) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Sstruct(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    version = __inspector["version"]["value"].asLong();
    compression = Compression(__inspector["compression"]["value"]);
    for (size_t __i = 0; __i < __inspector["field"]["value"].children(); __i++) {
        field.push_back(Field(__inspector["field"]["value"][__i]["value"]));
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Sstruct::Sstruct(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()(__inspector["name"], "");
    version = ::config::internal::ValueConverter<int32_t>()(__inspector["version"], 0);
    compression = ::config::internal::ValueConverter<Compression>()(__inspector["compression"]);
    ::config::internal::VectorInserter<FieldVector> __fieldInserter(field);
    __inspector["field"].traverse(__fieldInserter);
}

InternalDocumenttypesType::Documenttype::Datatype::Datatype()
    : id(0),
      type(Type::STRUCT),
      array(),
      map(),
      wset(),
      annotationref(),
      sstruct()
{
}

InternalDocumenttypesType::Documenttype::Datatype::Datatype(const Datatype & __rhs) = default;
InternalDocumenttypesType::Documenttype::Datatype & InternalDocumenttypesType::Documenttype::Datatype::operator = (const Datatype & __rhs) = default;
InternalDocumenttypesType::Documenttype::Datatype::Datatype(Datatype && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype & InternalDocumenttypesType::Documenttype::Datatype::operator = (Datatype && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Datatype::~Datatype() = default; 
InternalDocumenttypesType::Documenttype::Datatype::Datatype(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    id = ConfigParser::parse<int32_t>("id", __lines);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
    type = getType(ConfigParser::parse<vespalib::string>("type", __lines));
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
    array = ConfigParser::parseStruct<Array>("array", __lines);
    ConfigParser::stripLinesForKey("array", __remainingValuesToParse);
    map = ConfigParser::parseStruct<Map>("map", __lines);
    ConfigParser::stripLinesForKey("map", __remainingValuesToParse);
    wset = ConfigParser::parseStruct<Wset>("wset", __lines);
    ConfigParser::stripLinesForKey("wset", __remainingValuesToParse);
    annotationref = ConfigParser::parseStruct<Annotationref>("annotationref", __lines);
    ConfigParser::stripLinesForKey("annotationref", __remainingValuesToParse);
    sstruct = ConfigParser::parseStruct<Sstruct>("sstruct", __lines);
    ConfigParser::stripLinesForKey("sstruct", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::
operator==(const Datatype& __rhs) const noexcept
{
    return (id == __rhs.id &&
            type == __rhs.type &&
            array == __rhs.array &&
            map == __rhs.map &&
            wset == __rhs.wset &&
            annotationref == __rhs.annotationref &&
            sstruct == __rhs.sstruct);
}

bool
InternalDocumenttypesType::Documenttype::Datatype::
operator!=(const Datatype& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Datatype::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "int");
        __c.setLong("value", id);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getTypeName(type)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("array");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        array.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("map");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        map.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("wset");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        wset.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("annotationref");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        annotationref.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("sstruct");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        sstruct.serialize(__c2);
    }
}

InternalDocumenttypesType::Documenttype::Datatype::Datatype(const vespalib::slime::Inspector & __inspector)
{
    id = __inspector["id"]["value"].asLong();
    type = getType(__inspector["type"]["value"].asString().make_string());
    array = Array(__inspector["array"]["value"]);
    map = Map(__inspector["map"]["value"]);
    wset = Wset(__inspector["wset"]["value"]);
    annotationref = Annotationref(__inspector["annotationref"]["value"]);
    sstruct = Sstruct(__inspector["sstruct"]["value"]);
}

InternalDocumenttypesType::Documenttype::Datatype::Datatype(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    id = ::config::internal::ValueConverter<int32_t>()("id", __inspector["id"]);
    type = InternalTypeConverter()("type", __inspector["type"]);
    array = ::config::internal::ValueConverter<Array>()(__inspector["array"]);
    map = ::config::internal::ValueConverter<Map>()(__inspector["map"]);
    wset = ::config::internal::ValueConverter<Wset>()(__inspector["wset"]);
    annotationref = ::config::internal::ValueConverter<Annotationref>()(__inspector["annotationref"]);
    sstruct = ::config::internal::ValueConverter<Sstruct>()(__inspector["sstruct"]);
}

InternalDocumenttypesType::Documenttype::Annotationtype::Inherits::Inherits() noexcept
    : id(0)
{
}

InternalDocumenttypesType::Documenttype::Annotationtype::Inherits::Inherits(const Inherits & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Annotationtype::Inherits & InternalDocumenttypesType::Documenttype::Annotationtype::Inherits::operator = (const Inherits & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Annotationtype::Inherits::Inherits(Inherits && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Annotationtype::Inherits & InternalDocumenttypesType::Documenttype::Annotationtype::Inherits::operator = (Inherits && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Annotationtype::Inherits::~Inherits() = default; 
InternalDocumenttypesType::Documenttype::Annotationtype::Inherits::Inherits(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    id = ConfigParser::parse<int32_t>("id", __lines);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Annotationtype::Inherits::
operator==(const Inherits& __rhs) const noexcept
{
    return (id == __rhs.id);
}

bool
InternalDocumenttypesType::Documenttype::Annotationtype::Inherits::
operator!=(const Inherits& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Annotationtype::Inherits::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "int");
        __c.setLong("value", id);
    }
}

InternalDocumenttypesType::Documenttype::Annotationtype::Inherits::Inherits(const vespalib::slime::Inspector & __inspector)
{
    id = __inspector["id"]["value"].asLong();
}

InternalDocumenttypesType::Documenttype::Annotationtype::Inherits::Inherits(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    id = ::config::internal::ValueConverter<int32_t>()("id", __inspector["id"]);
}

InternalDocumenttypesType::Documenttype::Annotationtype::Annotationtype()
    : id(0),
      name(),
      datatype(-1),
      inherits()
{
}

InternalDocumenttypesType::Documenttype::Annotationtype::Annotationtype(const Annotationtype & __rhs) = default;
InternalDocumenttypesType::Documenttype::Annotationtype & InternalDocumenttypesType::Documenttype::Annotationtype::operator = (const Annotationtype & __rhs) = default;
InternalDocumenttypesType::Documenttype::Annotationtype::Annotationtype(Annotationtype && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Annotationtype & InternalDocumenttypesType::Documenttype::Annotationtype::operator = (Annotationtype && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Annotationtype::~Annotationtype() = default; 
InternalDocumenttypesType::Documenttype::Annotationtype::Annotationtype(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    id = ConfigParser::parse<int32_t>("id", __lines);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    datatype = ConfigParser::parse<int32_t>("datatype", __lines, -1);
    ConfigParser::stripLinesForKey("datatype", __remainingValuesToParse);
    inherits = ConfigParser::parseArray<InheritsVector>("inherits", __lines);
    ConfigParser::stripLinesForKey("inherits", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Annotationtype::
operator==(const Annotationtype& __rhs) const noexcept
{
    return (id == __rhs.id &&
            name == __rhs.name &&
            datatype == __rhs.datatype &&
            inherits == __rhs.inherits);
}

bool
InternalDocumenttypesType::Documenttype::Annotationtype::
operator!=(const Annotationtype& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Annotationtype::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "int");
        __c.setLong("value", id);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("datatype");
        __c.setString("type", "int");
        __c.setLong("value", datatype);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("inherits");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : inherits) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalDocumenttypesType::Documenttype::Annotationtype::Annotationtype(const vespalib::slime::Inspector & __inspector)
{
    id = __inspector["id"]["value"].asLong();
    name = __inspector["name"]["value"].asString().make_string();
    datatype = __inspector["datatype"]["value"].asLong();
    for (size_t __i = 0; __i < __inspector["inherits"]["value"].children(); __i++) {
        inherits.push_back(Inherits(__inspector["inherits"]["value"][__i]["value"]));
    }
}

InternalDocumenttypesType::Documenttype::Annotationtype::Annotationtype(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    id = ::config::internal::ValueConverter<int32_t>()("id", __inspector["id"]);
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    datatype = ::config::internal::ValueConverter<int32_t>()(__inspector["datatype"], -1);
    ::config::internal::VectorInserter<InheritsVector> __inheritsInserter(inherits);
    __inspector["inherits"].traverse(__inheritsInserter);
}

InternalDocumenttypesType::Documenttype::Fieldsets::Fieldsets()
    : fields()
{
}

InternalDocumenttypesType::Documenttype::Fieldsets::Fieldsets(const Fieldsets & __rhs) = default;
InternalDocumenttypesType::Documenttype::Fieldsets & InternalDocumenttypesType::Documenttype::Fieldsets::operator = (const Fieldsets & __rhs) = default;
InternalDocumenttypesType::Documenttype::Fieldsets::Fieldsets(Fieldsets && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Fieldsets & InternalDocumenttypesType::Documenttype::Fieldsets::operator = (Fieldsets && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Fieldsets::~Fieldsets() = default; 
InternalDocumenttypesType::Documenttype::Fieldsets::Fieldsets(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    fields = ConfigParser::parseArray<::config::StringVector>("fields", __lines);
    ConfigParser::stripLinesForKey("fields", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Fieldsets::
operator==(const Fieldsets& __rhs) const noexcept
{
    return (fields == __rhs.fields);
}

bool
InternalDocumenttypesType::Documenttype::Fieldsets::
operator!=(const Fieldsets& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Fieldsets::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fields");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : fields) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
}

InternalDocumenttypesType::Documenttype::Fieldsets::Fieldsets(const vespalib::slime::Inspector & __inspector)
{
    for (size_t __i = 0; __i < __inspector["fields"]["value"].children(); __i++) {
        fields.push_back(__inspector["fields"]["value"][__i]["value"].asString().make_string());
    }
}

InternalDocumenttypesType::Documenttype::Fieldsets::Fieldsets(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<::config::StringVector> __fieldsInserter(fields);
    __inspector["fields"].traverse(__fieldsInserter);
}

InternalDocumenttypesType::Documenttype::Referencetype::Referencetype() noexcept
    : id(0),
      targetTypeId(0)
{
}

InternalDocumenttypesType::Documenttype::Referencetype::Referencetype(const Referencetype & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Referencetype & InternalDocumenttypesType::Documenttype::Referencetype::operator = (const Referencetype & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Referencetype::Referencetype(Referencetype && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Referencetype & InternalDocumenttypesType::Documenttype::Referencetype::operator = (Referencetype && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Referencetype::~Referencetype() = default; 
InternalDocumenttypesType::Documenttype::Referencetype::Referencetype(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    id = ConfigParser::parse<int32_t>("id", __lines);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
    targetTypeId = ConfigParser::parse<int32_t>("target_type_id", __lines);
    ConfigParser::stripLinesForKey("target_type_id", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Referencetype::
operator==(const Referencetype& __rhs) const noexcept
{
    return (id == __rhs.id &&
            targetTypeId == __rhs.targetTypeId);
}

bool
InternalDocumenttypesType::Documenttype::Referencetype::
operator!=(const Referencetype& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Referencetype::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "int");
        __c.setLong("value", id);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("target_type_id");
        __c.setString("type", "int");
        __c.setLong("value", targetTypeId);
    }
}

InternalDocumenttypesType::Documenttype::Referencetype::Referencetype(const vespalib::slime::Inspector & __inspector)
{
    id = __inspector["id"]["value"].asLong();
    targetTypeId = __inspector["target_type_id"]["value"].asLong();
}

InternalDocumenttypesType::Documenttype::Referencetype::Referencetype(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    id = ::config::internal::ValueConverter<int32_t>()("id", __inspector["id"]);
    targetTypeId = ::config::internal::ValueConverter<int32_t>()("target_type_id", __inspector["target_type_id"]);
}

InternalDocumenttypesType::Documenttype::Importedfield::Importedfield() noexcept
    : name()
{
}

InternalDocumenttypesType::Documenttype::Importedfield::Importedfield(const Importedfield & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Importedfield & InternalDocumenttypesType::Documenttype::Importedfield::operator = (const Importedfield & __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Importedfield::Importedfield(Importedfield && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Importedfield & InternalDocumenttypesType::Documenttype::Importedfield::operator = (Importedfield && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::Importedfield::~Importedfield() = default; 
InternalDocumenttypesType::Documenttype::Importedfield::Importedfield(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::Importedfield::
operator==(const Importedfield& __rhs) const noexcept
{
    return (name == __rhs.name);
}

bool
InternalDocumenttypesType::Documenttype::Importedfield::
operator!=(const Importedfield& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::Importedfield::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
}

InternalDocumenttypesType::Documenttype::Importedfield::Importedfield(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
}

InternalDocumenttypesType::Documenttype::Importedfield::Importedfield(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
}

InternalDocumenttypesType::Documenttype::Documenttype()
    : id(0),
      name(),
      version(0),
      headerstruct(0),
      bodystruct(0),
      inherits(),
      datatype(),
      annotationtype(),
      fieldsets(),
      referencetype(),
      importedfield()
{
}

InternalDocumenttypesType::Documenttype::Documenttype(const Documenttype & __rhs) = default;
InternalDocumenttypesType::Documenttype & InternalDocumenttypesType::Documenttype::operator = (const Documenttype & __rhs) = default;
InternalDocumenttypesType::Documenttype::Documenttype(Documenttype && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype & InternalDocumenttypesType::Documenttype::operator = (Documenttype && __rhs) noexcept = default;
InternalDocumenttypesType::Documenttype::~Documenttype() = default; 
InternalDocumenttypesType::Documenttype::Documenttype(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    id = ConfigParser::parse<int32_t>("id", __lines);
    ConfigParser::stripLinesForKey("id", __remainingValuesToParse);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    version = ConfigParser::parse<int32_t>("version", __lines, 0);
    ConfigParser::stripLinesForKey("version", __remainingValuesToParse);
    headerstruct = ConfigParser::parse<int32_t>("headerstruct", __lines);
    ConfigParser::stripLinesForKey("headerstruct", __remainingValuesToParse);
    bodystruct = ConfigParser::parse<int32_t>("bodystruct", __lines, 0);
    ConfigParser::stripLinesForKey("bodystruct", __remainingValuesToParse);
    inherits = ConfigParser::parseArray<InheritsVector>("inherits", __lines);
    ConfigParser::stripLinesForKey("inherits", __remainingValuesToParse);
    datatype = ConfigParser::parseArray<DatatypeVector>("datatype", __lines);
    ConfigParser::stripLinesForKey("datatype", __remainingValuesToParse);
    annotationtype = ConfigParser::parseArray<AnnotationtypeVector>("annotationtype", __lines);
    ConfigParser::stripLinesForKey("annotationtype", __remainingValuesToParse);
    fieldsets = ConfigParser::parseMap<Fieldsets>("fieldsets", __lines);
    ConfigParser::stripLinesForKey("fieldsets", __remainingValuesToParse);
    referencetype = ConfigParser::parseArray<ReferencetypeVector>("referencetype", __lines);
    ConfigParser::stripLinesForKey("referencetype", __remainingValuesToParse);
    importedfield = ConfigParser::parseArray<ImportedfieldVector>("importedfield", __lines);
    ConfigParser::stripLinesForKey("importedfield", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Documenttype::
operator==(const Documenttype& __rhs) const noexcept
{
    return (id == __rhs.id &&
            name == __rhs.name &&
            version == __rhs.version &&
            headerstruct == __rhs.headerstruct &&
            bodystruct == __rhs.bodystruct &&
            inherits == __rhs.inherits &&
            datatype == __rhs.datatype &&
            annotationtype == __rhs.annotationtype &&
            fieldsets == __rhs.fieldsets &&
            referencetype == __rhs.referencetype &&
            importedfield == __rhs.importedfield);
}

bool
InternalDocumenttypesType::Documenttype::
operator!=(const Documenttype& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Documenttype::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("id");
        __c.setString("type", "int");
        __c.setLong("value", id);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("version");
        __c.setString("type", "int");
        __c.setLong("value", version);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("headerstruct");
        __c.setString("type", "int");
        __c.setLong("value", headerstruct);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("bodystruct");
        __c.setString("type", "int");
        __c.setLong("value", bodystruct);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("inherits");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : inherits) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("datatype");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : datatype) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("annotationtype");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : annotationtype) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fieldsets");
        __c.setString("type", "map");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & entry : fieldsets) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("key", vespalib::Memory(entry.first));
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            entry.second.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("referencetype");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : referencetype) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("importedfield");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : importedfield) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalDocumenttypesType::Documenttype::Documenttype(const vespalib::slime::Inspector & __inspector)
{
    id = __inspector["id"]["value"].asLong();
    name = __inspector["name"]["value"].asString().make_string();
    version = __inspector["version"]["value"].asLong();
    headerstruct = __inspector["headerstruct"]["value"].asLong();
    bodystruct = __inspector["bodystruct"]["value"].asLong();
    for (size_t __i = 0; __i < __inspector["inherits"]["value"].children(); __i++) {
        inherits.push_back(Inherits(__inspector["inherits"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["datatype"]["value"].children(); __i++) {
        datatype.push_back(Datatype(__inspector["datatype"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["annotationtype"]["value"].children(); __i++) {
        annotationtype.push_back(Annotationtype(__inspector["annotationtype"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["fieldsets"]["value"].children(); __i++) {
        fieldsets[__inspector["fieldsets"]["value"][__i]["key"].asString().make_string()] = Fieldsets(__inspector["fieldsets"]["value"][__i]["value"]);
    }
    for (size_t __i = 0; __i < __inspector["referencetype"]["value"].children(); __i++) {
        referencetype.push_back(Referencetype(__inspector["referencetype"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["importedfield"]["value"].children(); __i++) {
        importedfield.push_back(Importedfield(__inspector["importedfield"]["value"][__i]["value"]));
    }
}

InternalDocumenttypesType::Documenttype::Documenttype(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    id = ::config::internal::ValueConverter<int32_t>()("id", __inspector["id"]);
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    version = ::config::internal::ValueConverter<int32_t>()(__inspector["version"], 0);
    headerstruct = ::config::internal::ValueConverter<int32_t>()("headerstruct", __inspector["headerstruct"]);
    bodystruct = ::config::internal::ValueConverter<int32_t>()(__inspector["bodystruct"], 0);
    ::config::internal::VectorInserter<InheritsVector> __inheritsInserter(inherits);
    __inspector["inherits"].traverse(__inheritsInserter);
    ::config::internal::VectorInserter<DatatypeVector> __datatypeInserter(datatype);
    __inspector["datatype"].traverse(__datatypeInserter);
    ::config::internal::VectorInserter<AnnotationtypeVector> __annotationtypeInserter(annotationtype);
    __inspector["annotationtype"].traverse(__annotationtypeInserter);
    ::config::internal::MapInserter<Fieldsets> __fieldsetsInserter(fieldsets);
    __inspector["fieldsets"].traverse(__fieldsetsInserter);
    ::config::internal::VectorInserter<ReferencetypeVector> __referencetypeInserter(referencetype);
    __inspector["referencetype"].traverse(__referencetypeInserter);
    ::config::internal::VectorInserter<ImportedfieldVector> __importedfieldInserter(importedfield);
    __inspector["importedfield"].traverse(__importedfieldInserter);
}

InternalDocumenttypesType::Doctype::Inherits::Inherits() noexcept
    : idx(0)
{
}

InternalDocumenttypesType::Doctype::Inherits::Inherits(const Inherits & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Inherits & InternalDocumenttypesType::Doctype::Inherits::operator = (const Inherits & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Inherits::Inherits(Inherits && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Inherits & InternalDocumenttypesType::Doctype::Inherits::operator = (Inherits && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Inherits::~Inherits() = default; 
InternalDocumenttypesType::Doctype::Inherits::Inherits(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    idx = ConfigParser::parse<int32_t>("idx", __lines);
    ConfigParser::stripLinesForKey("idx", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::Inherits::
operator==(const Inherits& __rhs) const noexcept
{
    return (idx == __rhs.idx);
}

bool
InternalDocumenttypesType::Doctype::Inherits::
operator!=(const Inherits& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::Inherits::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("idx");
        __c.setString("type", "int");
        __c.setLong("value", idx);
    }
}

InternalDocumenttypesType::Doctype::Inherits::Inherits(const vespalib::slime::Inspector & __inspector)
{
    idx = __inspector["idx"]["value"].asLong();
}

InternalDocumenttypesType::Doctype::Inherits::Inherits(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    idx = ::config::internal::ValueConverter<int32_t>()("idx", __inspector["idx"]);
}

InternalDocumenttypesType::Doctype::Fieldsets::Fieldsets()
    : fields()
{
}

InternalDocumenttypesType::Doctype::Fieldsets::Fieldsets(const Fieldsets & __rhs) = default;
InternalDocumenttypesType::Doctype::Fieldsets & InternalDocumenttypesType::Doctype::Fieldsets::operator = (const Fieldsets & __rhs) = default;
InternalDocumenttypesType::Doctype::Fieldsets::Fieldsets(Fieldsets && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Fieldsets & InternalDocumenttypesType::Doctype::Fieldsets::operator = (Fieldsets && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Fieldsets::~Fieldsets() = default; 
InternalDocumenttypesType::Doctype::Fieldsets::Fieldsets(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    fields = ConfigParser::parseArray<::config::StringVector>("fields", __lines);
    ConfigParser::stripLinesForKey("fields", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::Fieldsets::
operator==(const Fieldsets& __rhs) const noexcept
{
    return (fields == __rhs.fields);
}

bool
InternalDocumenttypesType::Doctype::Fieldsets::
operator!=(const Fieldsets& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::Fieldsets::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fields");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : fields) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
}

InternalDocumenttypesType::Doctype::Fieldsets::Fieldsets(const vespalib::slime::Inspector & __inspector)
{
    for (size_t __i = 0; __i < __inspector["fields"]["value"].children(); __i++) {
        fields.push_back(__inspector["fields"]["value"][__i]["value"].asString().make_string());
    }
}

InternalDocumenttypesType::Doctype::Fieldsets::Fieldsets(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<::config::StringVector> __fieldsInserter(fields);
    __inspector["fields"].traverse(__fieldsInserter);
}

InternalDocumenttypesType::Doctype::Importedfield::Importedfield() noexcept
    : name()
{
}

InternalDocumenttypesType::Doctype::Importedfield::Importedfield(const Importedfield & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Importedfield & InternalDocumenttypesType::Doctype::Importedfield::operator = (const Importedfield & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Importedfield::Importedfield(Importedfield && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Importedfield & InternalDocumenttypesType::Doctype::Importedfield::operator = (Importedfield && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Importedfield::~Importedfield() = default; 
InternalDocumenttypesType::Doctype::Importedfield::Importedfield(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::Importedfield::
operator==(const Importedfield& __rhs) const noexcept
{
    return (name == __rhs.name);
}

bool
InternalDocumenttypesType::Doctype::Importedfield::
operator!=(const Importedfield& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::Importedfield::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
}

InternalDocumenttypesType::Doctype::Importedfield::Importedfield(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
}

InternalDocumenttypesType::Doctype::Importedfield::Importedfield(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
}

InternalDocumenttypesType::Doctype::Primitivetype::Primitivetype() noexcept
    : idx(0),
      name()
{
}

InternalDocumenttypesType::Doctype::Primitivetype::Primitivetype(const Primitivetype & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Primitivetype & InternalDocumenttypesType::Doctype::Primitivetype::operator = (const Primitivetype & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Primitivetype::Primitivetype(Primitivetype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Primitivetype & InternalDocumenttypesType::Doctype::Primitivetype::operator = (Primitivetype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Primitivetype::~Primitivetype() = default; 
InternalDocumenttypesType::Doctype::Primitivetype::Primitivetype(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    idx = ConfigParser::parse<int32_t>("idx", __lines);
    ConfigParser::stripLinesForKey("idx", __remainingValuesToParse);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::Primitivetype::
operator==(const Primitivetype& __rhs) const noexcept
{
    return (idx == __rhs.idx &&
            name == __rhs.name);
}

bool
InternalDocumenttypesType::Doctype::Primitivetype::
operator!=(const Primitivetype& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::Primitivetype::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("idx");
        __c.setString("type", "int");
        __c.setLong("value", idx);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
}

InternalDocumenttypesType::Doctype::Primitivetype::Primitivetype(const vespalib::slime::Inspector & __inspector)
{
    idx = __inspector["idx"]["value"].asLong();
    name = __inspector["name"]["value"].asString().make_string();
}

InternalDocumenttypesType::Doctype::Primitivetype::Primitivetype(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    idx = ::config::internal::ValueConverter<int32_t>()("idx", __inspector["idx"]);
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
}

InternalDocumenttypesType::Doctype::Arraytype::Arraytype() noexcept
    : idx(0),
      elementtype(0),
      internalid(0)
{
}

InternalDocumenttypesType::Doctype::Arraytype::Arraytype(const Arraytype & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Arraytype & InternalDocumenttypesType::Doctype::Arraytype::operator = (const Arraytype & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Arraytype::Arraytype(Arraytype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Arraytype & InternalDocumenttypesType::Doctype::Arraytype::operator = (Arraytype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Arraytype::~Arraytype() = default; 
InternalDocumenttypesType::Doctype::Arraytype::Arraytype(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    idx = ConfigParser::parse<int32_t>("idx", __lines);
    ConfigParser::stripLinesForKey("idx", __remainingValuesToParse);
    elementtype = ConfigParser::parse<int32_t>("elementtype", __lines);
    ConfigParser::stripLinesForKey("elementtype", __remainingValuesToParse);
    internalid = ConfigParser::parse<int32_t>("internalid", __lines);
    ConfigParser::stripLinesForKey("internalid", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::Arraytype::
operator==(const Arraytype& __rhs) const noexcept
{
    return (idx == __rhs.idx &&
            elementtype == __rhs.elementtype &&
            internalid == __rhs.internalid);
}

bool
InternalDocumenttypesType::Doctype::Arraytype::
operator!=(const Arraytype& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::Arraytype::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("idx");
        __c.setString("type", "int");
        __c.setLong("value", idx);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("elementtype");
        __c.setString("type", "int");
        __c.setLong("value", elementtype);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("internalid");
        __c.setString("type", "int");
        __c.setLong("value", internalid);
    }
}

InternalDocumenttypesType::Doctype::Arraytype::Arraytype(const vespalib::slime::Inspector & __inspector)
{
    idx = __inspector["idx"]["value"].asLong();
    elementtype = __inspector["elementtype"]["value"].asLong();
    internalid = __inspector["internalid"]["value"].asLong();
}

InternalDocumenttypesType::Doctype::Arraytype::Arraytype(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    idx = ::config::internal::ValueConverter<int32_t>()("idx", __inspector["idx"]);
    elementtype = ::config::internal::ValueConverter<int32_t>()("elementtype", __inspector["elementtype"]);
    internalid = ::config::internal::ValueConverter<int32_t>()("internalid", __inspector["internalid"]);
}

InternalDocumenttypesType::Doctype::Maptype::Maptype() noexcept
    : idx(0),
      keytype(0),
      valuetype(0),
      internalid(0)
{
}

InternalDocumenttypesType::Doctype::Maptype::Maptype(const Maptype & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Maptype & InternalDocumenttypesType::Doctype::Maptype::operator = (const Maptype & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Maptype::Maptype(Maptype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Maptype & InternalDocumenttypesType::Doctype::Maptype::operator = (Maptype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Maptype::~Maptype() = default; 
InternalDocumenttypesType::Doctype::Maptype::Maptype(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    idx = ConfigParser::parse<int32_t>("idx", __lines);
    ConfigParser::stripLinesForKey("idx", __remainingValuesToParse);
    keytype = ConfigParser::parse<int32_t>("keytype", __lines);
    ConfigParser::stripLinesForKey("keytype", __remainingValuesToParse);
    valuetype = ConfigParser::parse<int32_t>("valuetype", __lines);
    ConfigParser::stripLinesForKey("valuetype", __remainingValuesToParse);
    internalid = ConfigParser::parse<int32_t>("internalid", __lines);
    ConfigParser::stripLinesForKey("internalid", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::Maptype::
operator==(const Maptype& __rhs) const noexcept
{
    return (idx == __rhs.idx &&
            keytype == __rhs.keytype &&
            valuetype == __rhs.valuetype &&
            internalid == __rhs.internalid);
}

bool
InternalDocumenttypesType::Doctype::Maptype::
operator!=(const Maptype& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::Maptype::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("idx");
        __c.setString("type", "int");
        __c.setLong("value", idx);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("keytype");
        __c.setString("type", "int");
        __c.setLong("value", keytype);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("valuetype");
        __c.setString("type", "int");
        __c.setLong("value", valuetype);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("internalid");
        __c.setString("type", "int");
        __c.setLong("value", internalid);
    }
}

InternalDocumenttypesType::Doctype::Maptype::Maptype(const vespalib::slime::Inspector & __inspector)
{
    idx = __inspector["idx"]["value"].asLong();
    keytype = __inspector["keytype"]["value"].asLong();
    valuetype = __inspector["valuetype"]["value"].asLong();
    internalid = __inspector["internalid"]["value"].asLong();
}

InternalDocumenttypesType::Doctype::Maptype::Maptype(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    idx = ::config::internal::ValueConverter<int32_t>()("idx", __inspector["idx"]);
    keytype = ::config::internal::ValueConverter<int32_t>()("keytype", __inspector["keytype"]);
    valuetype = ::config::internal::ValueConverter<int32_t>()("valuetype", __inspector["valuetype"]);
    internalid = ::config::internal::ValueConverter<int32_t>()("internalid", __inspector["internalid"]);
}

InternalDocumenttypesType::Doctype::Wsettype::Wsettype() noexcept
    : idx(0),
      elementtype(0),
      createifnonexistent(false),
      removeifzero(false),
      internalid(0)
{
}

InternalDocumenttypesType::Doctype::Wsettype::Wsettype(const Wsettype & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Wsettype & InternalDocumenttypesType::Doctype::Wsettype::operator = (const Wsettype & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Wsettype::Wsettype(Wsettype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Wsettype & InternalDocumenttypesType::Doctype::Wsettype::operator = (Wsettype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Wsettype::~Wsettype() = default; 
InternalDocumenttypesType::Doctype::Wsettype::Wsettype(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    idx = ConfigParser::parse<int32_t>("idx", __lines);
    ConfigParser::stripLinesForKey("idx", __remainingValuesToParse);
    elementtype = ConfigParser::parse<int32_t>("elementtype", __lines);
    ConfigParser::stripLinesForKey("elementtype", __remainingValuesToParse);
    createifnonexistent = ConfigParser::parse<bool>("createifnonexistent", __lines, false);
    ConfigParser::stripLinesForKey("createifnonexistent", __remainingValuesToParse);
    removeifzero = ConfigParser::parse<bool>("removeifzero", __lines, false);
    ConfigParser::stripLinesForKey("removeifzero", __remainingValuesToParse);
    internalid = ConfigParser::parse<int32_t>("internalid", __lines);
    ConfigParser::stripLinesForKey("internalid", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::Wsettype::
operator==(const Wsettype& __rhs) const noexcept
{
    return (idx == __rhs.idx &&
            elementtype == __rhs.elementtype &&
            createifnonexistent == __rhs.createifnonexistent &&
            removeifzero == __rhs.removeifzero &&
            internalid == __rhs.internalid);
}

bool
InternalDocumenttypesType::Doctype::Wsettype::
operator!=(const Wsettype& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::Wsettype::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("idx");
        __c.setString("type", "int");
        __c.setLong("value", idx);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("elementtype");
        __c.setString("type", "int");
        __c.setLong("value", elementtype);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("createifnonexistent");
        __c.setString("type", "bool");
        __c.setBool("value", createifnonexistent);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("removeifzero");
        __c.setString("type", "bool");
        __c.setBool("value", removeifzero);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("internalid");
        __c.setString("type", "int");
        __c.setLong("value", internalid);
    }
}

InternalDocumenttypesType::Doctype::Wsettype::Wsettype(const vespalib::slime::Inspector & __inspector)
{
    idx = __inspector["idx"]["value"].asLong();
    elementtype = __inspector["elementtype"]["value"].asLong();
    createifnonexistent = __inspector["createifnonexistent"]["value"].asBool();
    removeifzero = __inspector["removeifzero"]["value"].asBool();
    internalid = __inspector["internalid"]["value"].asLong();
}

InternalDocumenttypesType::Doctype::Wsettype::Wsettype(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    idx = ::config::internal::ValueConverter<int32_t>()("idx", __inspector["idx"]);
    elementtype = ::config::internal::ValueConverter<int32_t>()("elementtype", __inspector["elementtype"]);
    createifnonexistent = ::config::internal::ValueConverter<bool>()(__inspector["createifnonexistent"], false);
    removeifzero = ::config::internal::ValueConverter<bool>()(__inspector["removeifzero"], false);
    internalid = ::config::internal::ValueConverter<int32_t>()("internalid", __inspector["internalid"]);
}

InternalDocumenttypesType::Doctype::Tensortype::Tensortype() noexcept
    : idx(0),
      detailedtype()
{
}

InternalDocumenttypesType::Doctype::Tensortype::Tensortype(const Tensortype & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Tensortype & InternalDocumenttypesType::Doctype::Tensortype::operator = (const Tensortype & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Tensortype::Tensortype(Tensortype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Tensortype & InternalDocumenttypesType::Doctype::Tensortype::operator = (Tensortype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Tensortype::~Tensortype() = default; 
InternalDocumenttypesType::Doctype::Tensortype::Tensortype(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    idx = ConfigParser::parse<int32_t>("idx", __lines);
    ConfigParser::stripLinesForKey("idx", __remainingValuesToParse);
    detailedtype = ConfigParser::parse<vespalib::string>("detailedtype", __lines);
    ConfigParser::stripLinesForKey("detailedtype", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::Tensortype::
operator==(const Tensortype& __rhs) const noexcept
{
    return (idx == __rhs.idx &&
            detailedtype == __rhs.detailedtype);
}

bool
InternalDocumenttypesType::Doctype::Tensortype::
operator!=(const Tensortype& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::Tensortype::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("idx");
        __c.setString("type", "int");
        __c.setLong("value", idx);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("detailedtype");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(detailedtype));
    }
}

InternalDocumenttypesType::Doctype::Tensortype::Tensortype(const vespalib::slime::Inspector & __inspector)
{
    idx = __inspector["idx"]["value"].asLong();
    detailedtype = __inspector["detailedtype"]["value"].asString().make_string();
}

InternalDocumenttypesType::Doctype::Tensortype::Tensortype(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    idx = ::config::internal::ValueConverter<int32_t>()("idx", __inspector["idx"]);
    detailedtype = ::config::internal::ValueConverter<vespalib::string>()("detailedtype", __inspector["detailedtype"]);
}

InternalDocumenttypesType::Doctype::Documentref::Documentref() noexcept
    : idx(0),
      targettype(0),
      internalid(0)
{
}

InternalDocumenttypesType::Doctype::Documentref::Documentref(const Documentref & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Documentref & InternalDocumenttypesType::Doctype::Documentref::operator = (const Documentref & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Documentref::Documentref(Documentref && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Documentref & InternalDocumenttypesType::Doctype::Documentref::operator = (Documentref && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Documentref::~Documentref() = default; 
InternalDocumenttypesType::Doctype::Documentref::Documentref(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    idx = ConfigParser::parse<int32_t>("idx", __lines);
    ConfigParser::stripLinesForKey("idx", __remainingValuesToParse);
    targettype = ConfigParser::parse<int32_t>("targettype", __lines);
    ConfigParser::stripLinesForKey("targettype", __remainingValuesToParse);
    internalid = ConfigParser::parse<int32_t>("internalid", __lines);
    ConfigParser::stripLinesForKey("internalid", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::Documentref::
operator==(const Documentref& __rhs) const noexcept
{
    return (idx == __rhs.idx &&
            targettype == __rhs.targettype &&
            internalid == __rhs.internalid);
}

bool
InternalDocumenttypesType::Doctype::Documentref::
operator!=(const Documentref& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::Documentref::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("idx");
        __c.setString("type", "int");
        __c.setLong("value", idx);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("targettype");
        __c.setString("type", "int");
        __c.setLong("value", targettype);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("internalid");
        __c.setString("type", "int");
        __c.setLong("value", internalid);
    }
}

InternalDocumenttypesType::Doctype::Documentref::Documentref(const vespalib::slime::Inspector & __inspector)
{
    idx = __inspector["idx"]["value"].asLong();
    targettype = __inspector["targettype"]["value"].asLong();
    internalid = __inspector["internalid"]["value"].asLong();
}

InternalDocumenttypesType::Doctype::Documentref::Documentref(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    idx = ::config::internal::ValueConverter<int32_t>()("idx", __inspector["idx"]);
    targettype = ::config::internal::ValueConverter<int32_t>()("targettype", __inspector["targettype"]);
    internalid = ::config::internal::ValueConverter<int32_t>()("internalid", __inspector["internalid"]);
}

InternalDocumenttypesType::Doctype::Annotationtype::Inherits::Inherits() noexcept
    : idx(0)
{
}

InternalDocumenttypesType::Doctype::Annotationtype::Inherits::Inherits(const Inherits & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Annotationtype::Inherits & InternalDocumenttypesType::Doctype::Annotationtype::Inherits::operator = (const Inherits & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Annotationtype::Inherits::Inherits(Inherits && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Annotationtype::Inherits & InternalDocumenttypesType::Doctype::Annotationtype::Inherits::operator = (Inherits && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Annotationtype::Inherits::~Inherits() = default; 
InternalDocumenttypesType::Doctype::Annotationtype::Inherits::Inherits(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    idx = ConfigParser::parse<int32_t>("idx", __lines);
    ConfigParser::stripLinesForKey("idx", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::Annotationtype::Inherits::
operator==(const Inherits& __rhs) const noexcept
{
    return (idx == __rhs.idx);
}

bool
InternalDocumenttypesType::Doctype::Annotationtype::Inherits::
operator!=(const Inherits& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::Annotationtype::Inherits::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("idx");
        __c.setString("type", "int");
        __c.setLong("value", idx);
    }
}

InternalDocumenttypesType::Doctype::Annotationtype::Inherits::Inherits(const vespalib::slime::Inspector & __inspector)
{
    idx = __inspector["idx"]["value"].asLong();
}

InternalDocumenttypesType::Doctype::Annotationtype::Inherits::Inherits(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    idx = ::config::internal::ValueConverter<int32_t>()("idx", __inspector["idx"]);
}

InternalDocumenttypesType::Doctype::Annotationtype::Annotationtype()
    : idx(0),
      name(),
      internalid(0),
      datatype(-1),
      inherits()
{
}

InternalDocumenttypesType::Doctype::Annotationtype::Annotationtype(const Annotationtype & __rhs) = default;
InternalDocumenttypesType::Doctype::Annotationtype & InternalDocumenttypesType::Doctype::Annotationtype::operator = (const Annotationtype & __rhs) = default;
InternalDocumenttypesType::Doctype::Annotationtype::Annotationtype(Annotationtype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Annotationtype & InternalDocumenttypesType::Doctype::Annotationtype::operator = (Annotationtype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Annotationtype::~Annotationtype() = default; 
InternalDocumenttypesType::Doctype::Annotationtype::Annotationtype(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    idx = ConfigParser::parse<int32_t>("idx", __lines);
    ConfigParser::stripLinesForKey("idx", __remainingValuesToParse);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    internalid = ConfigParser::parse<int32_t>("internalid", __lines);
    ConfigParser::stripLinesForKey("internalid", __remainingValuesToParse);
    datatype = ConfigParser::parse<int32_t>("datatype", __lines, -1);
    ConfigParser::stripLinesForKey("datatype", __remainingValuesToParse);
    inherits = ConfigParser::parseArray<InheritsVector>("inherits", __lines);
    ConfigParser::stripLinesForKey("inherits", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::Annotationtype::
operator==(const Annotationtype& __rhs) const noexcept
{
    return (idx == __rhs.idx &&
            name == __rhs.name &&
            internalid == __rhs.internalid &&
            datatype == __rhs.datatype &&
            inherits == __rhs.inherits);
}

bool
InternalDocumenttypesType::Doctype::Annotationtype::
operator!=(const Annotationtype& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::Annotationtype::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("idx");
        __c.setString("type", "int");
        __c.setLong("value", idx);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("internalid");
        __c.setString("type", "int");
        __c.setLong("value", internalid);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("datatype");
        __c.setString("type", "int");
        __c.setLong("value", datatype);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("inherits");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : inherits) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalDocumenttypesType::Doctype::Annotationtype::Annotationtype(const vespalib::slime::Inspector & __inspector)
{
    idx = __inspector["idx"]["value"].asLong();
    name = __inspector["name"]["value"].asString().make_string();
    internalid = __inspector["internalid"]["value"].asLong();
    datatype = __inspector["datatype"]["value"].asLong();
    for (size_t __i = 0; __i < __inspector["inherits"]["value"].children(); __i++) {
        inherits.push_back(Inherits(__inspector["inherits"]["value"][__i]["value"]));
    }
}

InternalDocumenttypesType::Doctype::Annotationtype::Annotationtype(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    idx = ::config::internal::ValueConverter<int32_t>()("idx", __inspector["idx"]);
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    internalid = ::config::internal::ValueConverter<int32_t>()("internalid", __inspector["internalid"]);
    datatype = ::config::internal::ValueConverter<int32_t>()(__inspector["datatype"], -1);
    ::config::internal::VectorInserter<InheritsVector> __inheritsInserter(inherits);
    __inspector["inherits"].traverse(__inheritsInserter);
}

InternalDocumenttypesType::Doctype::Annotationref::Annotationref() noexcept
    : idx(0),
      annotationtype(0),
      internalid(0)
{
}

InternalDocumenttypesType::Doctype::Annotationref::Annotationref(const Annotationref & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Annotationref & InternalDocumenttypesType::Doctype::Annotationref::operator = (const Annotationref & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Annotationref::Annotationref(Annotationref && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Annotationref & InternalDocumenttypesType::Doctype::Annotationref::operator = (Annotationref && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Annotationref::~Annotationref() = default; 
InternalDocumenttypesType::Doctype::Annotationref::Annotationref(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    idx = ConfigParser::parse<int32_t>("idx", __lines);
    ConfigParser::stripLinesForKey("idx", __remainingValuesToParse);
    annotationtype = ConfigParser::parse<int32_t>("annotationtype", __lines);
    ConfigParser::stripLinesForKey("annotationtype", __remainingValuesToParse);
    internalid = ConfigParser::parse<int32_t>("internalid", __lines);
    ConfigParser::stripLinesForKey("internalid", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::Annotationref::
operator==(const Annotationref& __rhs) const noexcept
{
    return (idx == __rhs.idx &&
            annotationtype == __rhs.annotationtype &&
            internalid == __rhs.internalid);
}

bool
InternalDocumenttypesType::Doctype::Annotationref::
operator!=(const Annotationref& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::Annotationref::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("idx");
        __c.setString("type", "int");
        __c.setLong("value", idx);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("annotationtype");
        __c.setString("type", "int");
        __c.setLong("value", annotationtype);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("internalid");
        __c.setString("type", "int");
        __c.setLong("value", internalid);
    }
}

InternalDocumenttypesType::Doctype::Annotationref::Annotationref(const vespalib::slime::Inspector & __inspector)
{
    idx = __inspector["idx"]["value"].asLong();
    annotationtype = __inspector["annotationtype"]["value"].asLong();
    internalid = __inspector["internalid"]["value"].asLong();
}

InternalDocumenttypesType::Doctype::Annotationref::Annotationref(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    idx = ::config::internal::ValueConverter<int32_t>()("idx", __inspector["idx"]);
    annotationtype = ::config::internal::ValueConverter<int32_t>()("annotationtype", __inspector["annotationtype"]);
    internalid = ::config::internal::ValueConverter<int32_t>()("internalid", __inspector["internalid"]);
}

InternalDocumenttypesType::Doctype::Structtype::Inherits::Inherits() noexcept
    : type(0)
{
}

InternalDocumenttypesType::Doctype::Structtype::Inherits::Inherits(const Inherits & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Structtype::Inherits & InternalDocumenttypesType::Doctype::Structtype::Inherits::operator = (const Inherits & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Structtype::Inherits::Inherits(Inherits && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Structtype::Inherits & InternalDocumenttypesType::Doctype::Structtype::Inherits::operator = (Inherits && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Structtype::Inherits::~Inherits() = default; 
InternalDocumenttypesType::Doctype::Structtype::Inherits::Inherits(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    type = ConfigParser::parse<int32_t>("type", __lines);
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::Structtype::Inherits::
operator==(const Inherits& __rhs) const noexcept
{
    return (type == __rhs.type);
}

bool
InternalDocumenttypesType::Doctype::Structtype::Inherits::
operator!=(const Inherits& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::Structtype::Inherits::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "int");
        __c.setLong("value", type);
    }
}

InternalDocumenttypesType::Doctype::Structtype::Inherits::Inherits(const vespalib::slime::Inspector & __inspector)
{
    type = __inspector["type"]["value"].asLong();
}

InternalDocumenttypesType::Doctype::Structtype::Inherits::Inherits(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    type = ::config::internal::ValueConverter<int32_t>()("type", __inspector["type"]);
}

InternalDocumenttypesType::Doctype::Structtype::Field::Field() noexcept
    : name(),
      internalid(0),
      type(0)
{
}

InternalDocumenttypesType::Doctype::Structtype::Field::Field(const Field & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Structtype::Field & InternalDocumenttypesType::Doctype::Structtype::Field::operator = (const Field & __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Structtype::Field::Field(Field && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Structtype::Field & InternalDocumenttypesType::Doctype::Structtype::Field::operator = (Field && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Structtype::Field::~Field() = default; 
InternalDocumenttypesType::Doctype::Structtype::Field::Field(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    internalid = ConfigParser::parse<int32_t>("internalid", __lines);
    ConfigParser::stripLinesForKey("internalid", __remainingValuesToParse);
    type = ConfigParser::parse<int32_t>("type", __lines);
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::Structtype::Field::
operator==(const Field& __rhs) const noexcept
{
    return (name == __rhs.name &&
            internalid == __rhs.internalid &&
            type == __rhs.type);
}

bool
InternalDocumenttypesType::Doctype::Structtype::Field::
operator!=(const Field& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::Structtype::Field::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("internalid");
        __c.setString("type", "int");
        __c.setLong("value", internalid);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "int");
        __c.setLong("value", type);
    }
}

InternalDocumenttypesType::Doctype::Structtype::Field::Field(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    internalid = __inspector["internalid"]["value"].asLong();
    type = __inspector["type"]["value"].asLong();
}

InternalDocumenttypesType::Doctype::Structtype::Field::Field(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    internalid = ::config::internal::ValueConverter<int32_t>()("internalid", __inspector["internalid"]);
    type = ::config::internal::ValueConverter<int32_t>()("type", __inspector["type"]);
}

InternalDocumenttypesType::Doctype::Structtype::Structtype()
    : idx(0),
      name(),
      inherits(),
      field(),
      internalid(0)
{
}

InternalDocumenttypesType::Doctype::Structtype::Structtype(const Structtype & __rhs) = default;
InternalDocumenttypesType::Doctype::Structtype & InternalDocumenttypesType::Doctype::Structtype::operator = (const Structtype & __rhs) = default;
InternalDocumenttypesType::Doctype::Structtype::Structtype(Structtype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Structtype & InternalDocumenttypesType::Doctype::Structtype::operator = (Structtype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::Structtype::~Structtype() = default; 
InternalDocumenttypesType::Doctype::Structtype::Structtype(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    idx = ConfigParser::parse<int32_t>("idx", __lines);
    ConfigParser::stripLinesForKey("idx", __remainingValuesToParse);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    inherits = ConfigParser::parseArray<InheritsVector>("inherits", __lines);
    ConfigParser::stripLinesForKey("inherits", __remainingValuesToParse);
    field = ConfigParser::parseArray<FieldVector>("field", __lines);
    ConfigParser::stripLinesForKey("field", __remainingValuesToParse);
    internalid = ConfigParser::parse<int32_t>("internalid", __lines);
    ConfigParser::stripLinesForKey("internalid", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::Structtype::
operator==(const Structtype& __rhs) const noexcept
{
    return (idx == __rhs.idx &&
            name == __rhs.name &&
            inherits == __rhs.inherits &&
            field == __rhs.field &&
            internalid == __rhs.internalid);
}

bool
InternalDocumenttypesType::Doctype::Structtype::
operator!=(const Structtype& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::Structtype::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("idx");
        __c.setString("type", "int");
        __c.setLong("value", idx);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("inherits");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : inherits) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("field");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : field) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("internalid");
        __c.setString("type", "int");
        __c.setLong("value", internalid);
    }
}

InternalDocumenttypesType::Doctype::Structtype::Structtype(const vespalib::slime::Inspector & __inspector)
{
    idx = __inspector["idx"]["value"].asLong();
    name = __inspector["name"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["inherits"]["value"].children(); __i++) {
        inherits.push_back(Inherits(__inspector["inherits"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["field"]["value"].children(); __i++) {
        field.push_back(Field(__inspector["field"]["value"][__i]["value"]));
    }
    internalid = __inspector["internalid"]["value"].asLong();
}

InternalDocumenttypesType::Doctype::Structtype::Structtype(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    idx = ::config::internal::ValueConverter<int32_t>()("idx", __inspector["idx"]);
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    ::config::internal::VectorInserter<InheritsVector> __inheritsInserter(inherits);
    __inspector["inherits"].traverse(__inheritsInserter);
    ::config::internal::VectorInserter<FieldVector> __fieldInserter(field);
    __inspector["field"].traverse(__fieldInserter);
    internalid = ::config::internal::ValueConverter<int32_t>()("internalid", __inspector["internalid"]);
}

InternalDocumenttypesType::Doctype::Doctype()
    : name(),
      idx(0),
      internalid(0),
      inherits(),
      contentstruct(0),
      fieldsets(),
      importedfield(),
      primitivetype(),
      arraytype(),
      maptype(),
      wsettype(),
      tensortype(),
      documentref(),
      annotationtype(),
      annotationref(),
      structtype()
{
}

InternalDocumenttypesType::Doctype::Doctype(const Doctype & __rhs) = default;
InternalDocumenttypesType::Doctype & InternalDocumenttypesType::Doctype::operator = (const Doctype & __rhs) = default;
InternalDocumenttypesType::Doctype::Doctype(Doctype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype & InternalDocumenttypesType::Doctype::operator = (Doctype && __rhs) noexcept = default;
InternalDocumenttypesType::Doctype::~Doctype() = default; 
InternalDocumenttypesType::Doctype::Doctype(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    idx = ConfigParser::parse<int32_t>("idx", __lines);
    ConfigParser::stripLinesForKey("idx", __remainingValuesToParse);
    internalid = ConfigParser::parse<int32_t>("internalid", __lines);
    ConfigParser::stripLinesForKey("internalid", __remainingValuesToParse);
    inherits = ConfigParser::parseArray<InheritsVector>("inherits", __lines);
    ConfigParser::stripLinesForKey("inherits", __remainingValuesToParse);
    contentstruct = ConfigParser::parse<int32_t>("contentstruct", __lines);
    ConfigParser::stripLinesForKey("contentstruct", __remainingValuesToParse);
    fieldsets = ConfigParser::parseMap<Fieldsets>("fieldsets", __lines);
    ConfigParser::stripLinesForKey("fieldsets", __remainingValuesToParse);
    importedfield = ConfigParser::parseArray<ImportedfieldVector>("importedfield", __lines);
    ConfigParser::stripLinesForKey("importedfield", __remainingValuesToParse);
    primitivetype = ConfigParser::parseArray<PrimitivetypeVector>("primitivetype", __lines);
    ConfigParser::stripLinesForKey("primitivetype", __remainingValuesToParse);
    arraytype = ConfigParser::parseArray<ArraytypeVector>("arraytype", __lines);
    ConfigParser::stripLinesForKey("arraytype", __remainingValuesToParse);
    maptype = ConfigParser::parseArray<MaptypeVector>("maptype", __lines);
    ConfigParser::stripLinesForKey("maptype", __remainingValuesToParse);
    wsettype = ConfigParser::parseArray<WsettypeVector>("wsettype", __lines);
    ConfigParser::stripLinesForKey("wsettype", __remainingValuesToParse);
    tensortype = ConfigParser::parseArray<TensortypeVector>("tensortype", __lines);
    ConfigParser::stripLinesForKey("tensortype", __remainingValuesToParse);
    documentref = ConfigParser::parseArray<DocumentrefVector>("documentref", __lines);
    ConfigParser::stripLinesForKey("documentref", __remainingValuesToParse);
    annotationtype = ConfigParser::parseArray<AnnotationtypeVector>("annotationtype", __lines);
    ConfigParser::stripLinesForKey("annotationtype", __remainingValuesToParse);
    annotationref = ConfigParser::parseArray<AnnotationrefVector>("annotationref", __lines);
    ConfigParser::stripLinesForKey("annotationref", __remainingValuesToParse);
    structtype = ConfigParser::parseArray<StructtypeVector>("structtype", __lines);
    ConfigParser::stripLinesForKey("structtype", __remainingValuesToParse);
}

bool
InternalDocumenttypesType::Doctype::operator==(const Doctype& __rhs) const noexcept
{
    return (name == __rhs.name &&
            idx == __rhs.idx &&
            internalid == __rhs.internalid &&
            inherits == __rhs.inherits &&
            contentstruct == __rhs.contentstruct &&
            fieldsets == __rhs.fieldsets &&
            importedfield == __rhs.importedfield &&
            primitivetype == __rhs.primitivetype &&
            arraytype == __rhs.arraytype &&
            maptype == __rhs.maptype &&
            wsettype == __rhs.wsettype &&
            tensortype == __rhs.tensortype &&
            documentref == __rhs.documentref &&
            annotationtype == __rhs.annotationtype &&
            annotationref == __rhs.annotationref &&
            structtype == __rhs.structtype);
}

bool
InternalDocumenttypesType::Doctype::operator!=(const Doctype& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::Doctype::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("idx");
        __c.setString("type", "int");
        __c.setLong("value", idx);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("internalid");
        __c.setString("type", "int");
        __c.setLong("value", internalid);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("inherits");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : inherits) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("contentstruct");
        __c.setString("type", "int");
        __c.setLong("value", contentstruct);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fieldsets");
        __c.setString("type", "map");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & entry : fieldsets) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("key", vespalib::Memory(entry.first));
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            entry.second.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("importedfield");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : importedfield) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("primitivetype");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : primitivetype) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("arraytype");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : arraytype) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maptype");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : maptype) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("wsettype");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : wsettype) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tensortype");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : tensortype) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("documentref");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : documentref) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("annotationtype");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : annotationtype) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("annotationref");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : annotationref) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("structtype");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : structtype) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalDocumenttypesType::Doctype::Doctype(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    idx = __inspector["idx"]["value"].asLong();
    internalid = __inspector["internalid"]["value"].asLong();
    for (size_t __i = 0; __i < __inspector["inherits"]["value"].children(); __i++) {
        inherits.push_back(Inherits(__inspector["inherits"]["value"][__i]["value"]));
    }
    contentstruct = __inspector["contentstruct"]["value"].asLong();
    for (size_t __i = 0; __i < __inspector["fieldsets"]["value"].children(); __i++) {
        fieldsets[__inspector["fieldsets"]["value"][__i]["key"].asString().make_string()] = Fieldsets(__inspector["fieldsets"]["value"][__i]["value"]);
    }
    for (size_t __i = 0; __i < __inspector["importedfield"]["value"].children(); __i++) {
        importedfield.push_back(Importedfield(__inspector["importedfield"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["primitivetype"]["value"].children(); __i++) {
        primitivetype.push_back(Primitivetype(__inspector["primitivetype"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["arraytype"]["value"].children(); __i++) {
        arraytype.push_back(Arraytype(__inspector["arraytype"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["maptype"]["value"].children(); __i++) {
        maptype.push_back(Maptype(__inspector["maptype"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["wsettype"]["value"].children(); __i++) {
        wsettype.push_back(Wsettype(__inspector["wsettype"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["tensortype"]["value"].children(); __i++) {
        tensortype.push_back(Tensortype(__inspector["tensortype"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["documentref"]["value"].children(); __i++) {
        documentref.push_back(Documentref(__inspector["documentref"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["annotationtype"]["value"].children(); __i++) {
        annotationtype.push_back(Annotationtype(__inspector["annotationtype"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["annotationref"]["value"].children(); __i++) {
        annotationref.push_back(Annotationref(__inspector["annotationref"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["structtype"]["value"].children(); __i++) {
        structtype.push_back(Structtype(__inspector["structtype"]["value"][__i]["value"]));
    }
}

InternalDocumenttypesType::Doctype::Doctype(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    idx = ::config::internal::ValueConverter<int32_t>()("idx", __inspector["idx"]);
    internalid = ::config::internal::ValueConverter<int32_t>()("internalid", __inspector["internalid"]);
    ::config::internal::VectorInserter<InheritsVector> __inheritsInserter(inherits);
    __inspector["inherits"].traverse(__inheritsInserter);
    contentstruct = ::config::internal::ValueConverter<int32_t>()("contentstruct", __inspector["contentstruct"]);
    ::config::internal::MapInserter<Fieldsets> __fieldsetsInserter(fieldsets);
    __inspector["fieldsets"].traverse(__fieldsetsInserter);
    ::config::internal::VectorInserter<ImportedfieldVector> __importedfieldInserter(importedfield);
    __inspector["importedfield"].traverse(__importedfieldInserter);
    ::config::internal::VectorInserter<PrimitivetypeVector> __primitivetypeInserter(primitivetype);
    __inspector["primitivetype"].traverse(__primitivetypeInserter);
    ::config::internal::VectorInserter<ArraytypeVector> __arraytypeInserter(arraytype);
    __inspector["arraytype"].traverse(__arraytypeInserter);
    ::config::internal::VectorInserter<MaptypeVector> __maptypeInserter(maptype);
    __inspector["maptype"].traverse(__maptypeInserter);
    ::config::internal::VectorInserter<WsettypeVector> __wsettypeInserter(wsettype);
    __inspector["wsettype"].traverse(__wsettypeInserter);
    ::config::internal::VectorInserter<TensortypeVector> __tensortypeInserter(tensortype);
    __inspector["tensortype"].traverse(__tensortypeInserter);
    ::config::internal::VectorInserter<DocumentrefVector> __documentrefInserter(documentref);
    __inspector["documentref"].traverse(__documentrefInserter);
    ::config::internal::VectorInserter<AnnotationtypeVector> __annotationtypeInserter(annotationtype);
    __inspector["annotationtype"].traverse(__annotationtypeInserter);
    ::config::internal::VectorInserter<AnnotationrefVector> __annotationrefInserter(annotationref);
    __inspector["annotationref"].traverse(__annotationrefInserter);
    ::config::internal::VectorInserter<StructtypeVector> __structtypeInserter(structtype);
    __inspector["structtype"].traverse(__structtypeInserter);
}

InternalDocumenttypesType::InternalDocumenttypesType()
    : ignoreundefinedfields(false),
      usev8geopositions(false),
      documenttype(),
      doctype()
{
}

InternalDocumenttypesType::InternalDocumenttypesType(const InternalDocumenttypesType & __rhs) = default;
InternalDocumenttypesType & InternalDocumenttypesType::operator =(const InternalDocumenttypesType & __rhs) = default;
InternalDocumenttypesType::InternalDocumenttypesType(InternalDocumenttypesType && __rhs) noexcept = default;
InternalDocumenttypesType & InternalDocumenttypesType::operator =(InternalDocumenttypesType && __rhs) noexcept = default;
InternalDocumenttypesType::~InternalDocumenttypesType() = default; 
InternalDocumenttypesType::InternalDocumenttypesType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        ignoreundefinedfields = ConfigParser::parse<bool>("ignoreundefinedfields", __lines, false);
        ConfigParser::stripLinesForKey("ignoreundefinedfields", __remainingValuesToParse);
        usev8geopositions = ConfigParser::parse<bool>("usev8geopositions", __lines, false);
        ConfigParser::stripLinesForKey("usev8geopositions", __remainingValuesToParse);
        documenttype = ConfigParser::parseArray<DocumenttypeVector>("documenttype", __lines);
        ConfigParser::stripLinesForKey("documenttype", __remainingValuesToParse);
        doctype = ConfigParser::parseArray<DoctypeVector>("doctype", __lines);
        ConfigParser::stripLinesForKey("doctype", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalDocumenttypesType::
operator==(const InternalDocumenttypesType& __rhs) const noexcept
{
    return (ignoreundefinedfields == __rhs.ignoreundefinedfields &&
            usev8geopositions == __rhs.usev8geopositions &&
            documenttype == __rhs.documenttype &&
            doctype == __rhs.doctype);
}

bool
InternalDocumenttypesType::
operator!=(const InternalDocumenttypesType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumenttypesType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ignoreundefinedfields");
        __c.setString("type", "bool");
        __c.setBool("value", ignoreundefinedfields);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("usev8geopositions");
        __c.setString("type", "bool");
        __c.setBool("value", usev8geopositions);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("documenttype");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : documenttype) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("doctype");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : doctype) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalDocumenttypesType::InternalDocumenttypesType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    ignoreundefinedfields = __inspector["ignoreundefinedfields"]["value"].asBool();
    usev8geopositions = __inspector["usev8geopositions"]["value"].asBool();
    for (size_t __i = 0; __i < __inspector["documenttype"]["value"].children(); __i++) {
        documenttype.push_back(Documenttype(__inspector["documenttype"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["doctype"]["value"].children(); __i++) {
        doctype.push_back(Doctype(__inspector["doctype"]["value"][__i]["value"]));
    }
}

InternalDocumenttypesType::InternalDocumenttypesType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ignoreundefinedfields = ::config::internal::ValueConverter<bool>()(__inspector["ignoreundefinedfields"], false);
    usev8geopositions = ::config::internal::ValueConverter<bool>()(__inspector["usev8geopositions"], false);
    ::config::internal::VectorInserter<DocumenttypeVector> __documenttypeInserter(documenttype);
    __inspector["documenttype"].traverse(__documenttypeInserter);
    ::config::internal::VectorInserter<DoctypeVector> __doctypeInserter(doctype);
    __inspector["doctype"].traverse(__doctypeInserter);
}

} // namespace internal

} // namespace document::config
