/**
 * @class document::config::Documenttypes
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_DOCUMENT_CONFIG_DOCUMENTTYPES_H
#define CLOUD_CONFIG_DOCUMENT_CONFIG_DOCUMENTTYPES_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace document::config {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalDocumenttypesType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalDocumenttypesType> UP;
    class Documenttype {
    public:
        class Inherits {
        public:
            Inherits(const ::config::StringVector & __lines);
            Inherits(const vespalib::slime::Inspector & __inspector);
            Inherits(const ::config::ConfigPayload & __payload);
            Inherits()  noexcept;
            Inherits(const Inherits & __rhs) noexcept;
            Inherits & operator = (const Inherits & __rhs) noexcept;
            Inherits(Inherits && __rhs) noexcept;
            Inherits & operator = (Inherits && __rhs) noexcept;
            ~Inherits();

            bool operator==(const Inherits& __rhs) const noexcept;
            bool operator!=(const Inherits& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** Specify a document type to inherit (id) */
            int32_t id;
        };
        typedef std::vector<Inherits> InheritsVector;

        typedef std::map<vespalib::string, Inherits> InheritsMap;

        class Datatype {
        public:
            enum class Type { STRUCT, ARRAY, WSET, MAP, ANNOTATIONREF, PRIMITIVE, TENSOR };
            typedef std::vector<Type> TypeVector;
            typedef std::map<vespalib::string, Type> TypeMap;
            static Type getType(const vespalib::string&);
            static vespalib::string getTypeName(Type e);

            struct InternalTypeConverter {
                Type operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                Type operator()(const ::vespalib::slime::Inspector & __inspector);
                Type operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault);
            };
            class Array {
            public:
                class Element {
                public:
                    Element(const ::config::StringVector & __lines);
                    Element(const vespalib::slime::Inspector & __inspector);
                    Element(const ::config::ConfigPayload & __payload);
                    Element()  noexcept;
                    Element(const Element & __rhs) noexcept;
                    Element & operator = (const Element & __rhs) noexcept;
                    Element(Element && __rhs) noexcept;
                    Element & operator = (Element && __rhs) noexcept;
                    ~Element();

                    bool operator==(const Element& __rhs) const noexcept;
                    bool operator!=(const Element& __rhs) const noexcept;

                    void serialize(vespalib::slime::Cursor & __cursor) const;

                    /**
                     * This is the id of the datatype of the element in the
                     * array.
                     */
                    int32_t id; // Default: 0
                };
                typedef std::vector<Element> ElementVector;

                typedef std::map<vespalib::string, Element> ElementMap;

                Array(const ::config::StringVector & __lines);
                Array(const vespalib::slime::Inspector & __inspector);
                Array(const ::config::ConfigPayload & __payload);
                Array()  noexcept;
                Array(const Array & __rhs) noexcept;
                Array & operator = (const Array & __rhs) noexcept;
                Array(Array && __rhs) noexcept;
                Array & operator = (Array && __rhs) noexcept;
                ~Array();

                bool operator==(const Array& __rhs) const noexcept;
                bool operator!=(const Array& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;
                Element element;
            };
            typedef std::vector<Array> ArrayVector;

            typedef std::map<vespalib::string, Array> ArrayMap;

            class Map {
            public:
                class Key {
                public:
                    Key(const ::config::StringVector & __lines);
                    Key(const vespalib::slime::Inspector & __inspector);
                    Key(const ::config::ConfigPayload & __payload);
                    Key()  noexcept;
                    Key(const Key & __rhs) noexcept;
                    Key & operator = (const Key & __rhs) noexcept;
                    Key(Key && __rhs) noexcept;
                    Key & operator = (Key && __rhs) noexcept;
                    ~Key();

                    bool operator==(const Key& __rhs) const noexcept;
                    bool operator!=(const Key& __rhs) const noexcept;

                    void serialize(vespalib::slime::Cursor & __cursor) const;

                    /** This is the id of the datatype of the key in the map. */
                    int32_t id; // Default: 0
                };
                typedef std::vector<Key> KeyVector;

                typedef std::map<vespalib::string, Key> KeyMap;

                class Value {
                public:
                    Value(const ::config::StringVector & __lines);
                    Value(const vespalib::slime::Inspector & __inspector);
                    Value(const ::config::ConfigPayload & __payload);
                    Value()  noexcept;
                    Value(const Value & __rhs) noexcept;
                    Value & operator = (const Value & __rhs) noexcept;
                    Value(Value && __rhs) noexcept;
                    Value & operator = (Value && __rhs) noexcept;
                    ~Value();

                    bool operator==(const Value& __rhs) const noexcept;
                    bool operator!=(const Value& __rhs) const noexcept;

                    void serialize(vespalib::slime::Cursor & __cursor) const;

                    /**
                     * This is the id of the datatype of the value in the map.
                     */
                    int32_t id; // Default: 0
                };
                typedef std::vector<Value> ValueVector;

                typedef std::map<vespalib::string, Value> ValueMap;

                Map(const ::config::StringVector & __lines);
                Map(const vespalib::slime::Inspector & __inspector);
                Map(const ::config::ConfigPayload & __payload);
                Map()  noexcept;
                Map(const Map & __rhs) noexcept;
                Map & operator = (const Map & __rhs) noexcept;
                Map(Map && __rhs) noexcept;
                Map & operator = (Map && __rhs) noexcept;
                ~Map();

                bool operator==(const Map& __rhs) const noexcept;
                bool operator!=(const Map& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;
                Key key;
                Value value;
            };
            typedef std::vector<Map> MapVector;

            typedef std::map<vespalib::string, Map> MapMap;

            class Wset {
            public:
                class Key {
                public:
                    Key(const ::config::StringVector & __lines);
                    Key(const vespalib::slime::Inspector & __inspector);
                    Key(const ::config::ConfigPayload & __payload);
                    Key()  noexcept;
                    Key(const Key & __rhs) noexcept;
                    Key & operator = (const Key & __rhs) noexcept;
                    Key(Key && __rhs) noexcept;
                    Key & operator = (Key && __rhs) noexcept;
                    ~Key();

                    bool operator==(const Key& __rhs) const noexcept;
                    bool operator!=(const Key& __rhs) const noexcept;

                    void serialize(vespalib::slime::Cursor & __cursor) const;

                    /**
                     * This is the id of the datatype of the key in the wset.
                     */
                    int32_t id; // Default: 0
                };
                typedef std::vector<Key> KeyVector;

                typedef std::map<vespalib::string, Key> KeyMap;

                Wset(const ::config::StringVector & __lines);
                Wset(const vespalib::slime::Inspector & __inspector);
                Wset(const ::config::ConfigPayload & __payload);
                Wset()  noexcept;
                Wset(const Wset & __rhs) noexcept;
                Wset & operator = (const Wset & __rhs) noexcept;
                Wset(Wset && __rhs) noexcept;
                Wset & operator = (Wset && __rhs) noexcept;
                ~Wset();

                bool operator==(const Wset& __rhs) const noexcept;
                bool operator!=(const Wset& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;
                Key key;

                /**
                 * Should an update to a nonexistent element cause it to be
                 * created
                 */
                bool createifnonexistent; // Default: false

                /**
                 * Should an element in a weighted set be removed if an update
                 * changes the weight to 0
                 */
                bool removeifzero; // Default: false
            };
            typedef std::vector<Wset> WsetVector;

            typedef std::map<vespalib::string, Wset> WsetMap;

            class Annotationref {
            public:
                class Annotation {
                public:
                    Annotation(const ::config::StringVector & __lines);
                    Annotation(const vespalib::slime::Inspector & __inspector);
                    Annotation(const ::config::ConfigPayload & __payload);
                    Annotation()  noexcept;
                    Annotation(const Annotation & __rhs) noexcept;
                    Annotation & operator = (const Annotation & __rhs) noexcept;
                    Annotation(Annotation && __rhs) noexcept;
                    Annotation & operator = (Annotation && __rhs) noexcept;
                    ~Annotation();

                    bool operator==(const Annotation& __rhs) const noexcept;
                    bool operator!=(const Annotation& __rhs) const noexcept;

                    void serialize(vespalib::slime::Cursor & __cursor) const;

                    /** This is the id of the referenced annotation. */
                    int32_t id; // Default: 0
                };
                typedef std::vector<Annotation> AnnotationVector;

                typedef std::map<vespalib::string, Annotation> AnnotationMap;

                Annotationref(const ::config::StringVector & __lines);
                Annotationref(const vespalib::slime::Inspector & __inspector);
                Annotationref(const ::config::ConfigPayload & __payload);
                Annotationref()  noexcept;
                Annotationref(const Annotationref & __rhs) noexcept;
                Annotationref & operator = (const Annotationref & __rhs) noexcept;
                Annotationref(Annotationref && __rhs) noexcept;
                Annotationref & operator = (Annotationref && __rhs) noexcept;
                ~Annotationref();

                bool operator==(const Annotationref& __rhs) const noexcept;
                bool operator!=(const Annotationref& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;
                Annotation annotation;
            };
            typedef std::vector<Annotationref> AnnotationrefVector;

            typedef std::map<vespalib::string, Annotationref> AnnotationrefMap;

            class Sstruct {
            public:
                class Compression {
                public:
                    enum class Type { NONE, LZ4 };
                    typedef std::vector<Type> TypeVector;
                    typedef std::map<vespalib::string, Type> TypeMap;
                    static Type getType(const vespalib::string&);
                    static vespalib::string getTypeName(Type e);

                    struct InternalTypeConverter {
                        Type operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                        Type operator()(const ::vespalib::slime::Inspector & __inspector);
                        Type operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault);
                    };
                    Compression(const ::config::StringVector & __lines);
                    Compression(const vespalib::slime::Inspector & __inspector);
                    Compression(const ::config::ConfigPayload & __payload);
                    Compression()  noexcept;
                    Compression(const Compression & __rhs) noexcept;
                    Compression & operator = (const Compression & __rhs) noexcept;
                    Compression(Compression && __rhs) noexcept;
                    Compression & operator = (Compression && __rhs) noexcept;
                    ~Compression();

                    bool operator==(const Compression& __rhs) const noexcept;
                    bool operator!=(const Compression& __rhs) const noexcept;

                    void serialize(vespalib::slime::Cursor & __cursor) const;

                    /**
                     * Specify which compression to use if compression is
                     * enabled above (0 = NONE, 6 = LZ4)
                     */
                    Type type; // Default: NONE

                    /**
                     * Specify the compression level to use if compression is
                     * enabled
                     */
                    int32_t level; // Default: 0

                    /**
                     * Specify the minimum reduction required from compression
                     * in order to keep the compressed version (maximum
                     * percentage of original size)
                     */
                    int32_t threshold; // Default: 95

                    /** Specify the minimum size before trying compression. */
                    int32_t minsize; // Default: 200
                };
                typedef std::vector<Compression> CompressionVector;

                typedef std::map<vespalib::string, Compression> CompressionMap;

                class Field {
                public:
                    Field(const ::config::StringVector & __lines);
                    Field(const vespalib::slime::Inspector & __inspector);
                    Field(const ::config::ConfigPayload & __payload);
                    Field()  noexcept;
                    Field(const Field & __rhs) noexcept;
                    Field & operator = (const Field & __rhs) noexcept;
                    Field(Field && __rhs) noexcept;
                    Field & operator = (Field && __rhs) noexcept;
                    ~Field();

                    bool operator==(const Field& __rhs) const noexcept;
                    bool operator!=(const Field& __rhs) const noexcept;

                    void serialize(vespalib::slime::Cursor & __cursor) const;

                    /**
                     * Specify a struct field name. Must be unique within the
                     * struct type.
                     */
                    vespalib::string name;

                    /**
                     * Specify a document field id. If not specified, this is
                     * generated by a hash function
                     */
                    int32_t id;

                    /**
                     * Specify the datatype of the field. Can only be a type
                     * defined in document
                     * or one of its inherited document types.
                     */
                    int32_t datatype;

                    /**
                     * Additional, optional type information which can be
                     * changed without
                     * (necessarily) causing field incompatibility
                     */
                    vespalib::string detailedtype; // Default: ""
                };
                typedef std::vector<Field> FieldVector;

                typedef std::map<vespalib::string, Field> FieldMap;

                Sstruct(const ::config::StringVector & __lines);
                Sstruct(const vespalib::slime::Inspector & __inspector);
                Sstruct(const ::config::ConfigPayload & __payload);
                Sstruct() ;
                Sstruct(const Sstruct & __rhs);
                Sstruct & operator = (const Sstruct & __rhs);
                Sstruct(Sstruct && __rhs) noexcept;
                Sstruct & operator = (Sstruct && __rhs) noexcept;
                ~Sstruct();

                bool operator==(const Sstruct& __rhs) const noexcept;
                bool operator!=(const Sstruct& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /**
                 * Specify the name of the struct type. Must be unique within
                 * documenttype.
                 */
                vespalib::string name; // Default: ""

                /** Version is not used */
                int32_t version; // Default: 0
                Compression compression;
                FieldVector field;
            };
            typedef std::vector<Sstruct> SstructVector;

            typedef std::map<vespalib::string, Sstruct> SstructMap;

            Datatype(const ::config::StringVector & __lines);
            Datatype(const vespalib::slime::Inspector & __inspector);
            Datatype(const ::config::ConfigPayload & __payload);
            Datatype() ;
            Datatype(const Datatype & __rhs);
            Datatype & operator = (const Datatype & __rhs);
            Datatype(Datatype && __rhs) noexcept;
            Datatype & operator = (Datatype && __rhs) noexcept;
            ~Datatype();

            bool operator==(const Datatype& __rhs) const noexcept;
            bool operator!=(const Datatype& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** This is the id of a datatype defined in the document. */
            int32_t id;

            /** This is the type of the datatype. */
            Type type;
            Array array;
            Map map;
            Wset wset;
            Annotationref annotationref;
            Sstruct sstruct;
        };
        typedef std::vector<Datatype> DatatypeVector;

        typedef std::map<vespalib::string, Datatype> DatatypeMap;

        class Annotationtype {
        public:
            class Inherits {
            public:
                Inherits(const ::config::StringVector & __lines);
                Inherits(const vespalib::slime::Inspector & __inspector);
                Inherits(const ::config::ConfigPayload & __payload);
                Inherits()  noexcept;
                Inherits(const Inherits & __rhs) noexcept;
                Inherits & operator = (const Inherits & __rhs) noexcept;
                Inherits(Inherits && __rhs) noexcept;
                Inherits & operator = (Inherits && __rhs) noexcept;
                ~Inherits();

                bool operator==(const Inherits& __rhs) const noexcept;
                bool operator!=(const Inherits& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /**
                 * The annotation type that this type inherits. (0 or 1 type)
                 */
                int32_t id;
            };
            typedef std::vector<Inherits> InheritsVector;

            typedef std::map<vespalib::string, Inherits> InheritsMap;

            Annotationtype(const ::config::StringVector & __lines);
            Annotationtype(const vespalib::slime::Inspector & __inspector);
            Annotationtype(const ::config::ConfigPayload & __payload);
            Annotationtype() ;
            Annotationtype(const Annotationtype & __rhs);
            Annotationtype & operator = (const Annotationtype & __rhs);
            Annotationtype(Annotationtype && __rhs) noexcept;
            Annotationtype & operator = (Annotationtype && __rhs) noexcept;
            ~Annotationtype();

            bool operator==(const Annotationtype& __rhs) const noexcept;
            bool operator!=(const Annotationtype& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** The id of the annotation type. */
            int32_t id;

            /** The name of the annotation type. */
            vespalib::string name;

            /** The contained datatype of the annotation type. */
            int32_t datatype; // Default: -1
            InheritsVector inherits;
        };
        typedef std::vector<Annotationtype> AnnotationtypeVector;

        typedef std::map<vespalib::string, Annotationtype> AnnotationtypeMap;

        class Fieldsets {
        public:
            Fieldsets(const ::config::StringVector & __lines);
            Fieldsets(const vespalib::slime::Inspector & __inspector);
            Fieldsets(const ::config::ConfigPayload & __payload);
            Fieldsets() ;
            Fieldsets(const Fieldsets & __rhs);
            Fieldsets & operator = (const Fieldsets & __rhs);
            Fieldsets(Fieldsets && __rhs) noexcept;
            Fieldsets & operator = (Fieldsets && __rhs) noexcept;
            ~Fieldsets();

            bool operator==(const Fieldsets& __rhs) const noexcept;
            bool operator!=(const Fieldsets& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** Field sets */
            ::config::StringVector fields;
        };
        typedef std::vector<Fieldsets> FieldsetsVector;

        typedef std::map<vespalib::string, Fieldsets> FieldsetsMap;

        class Referencetype {
        public:
            Referencetype(const ::config::StringVector & __lines);
            Referencetype(const vespalib::slime::Inspector & __inspector);
            Referencetype(const ::config::ConfigPayload & __payload);
            Referencetype()  noexcept;
            Referencetype(const Referencetype & __rhs) noexcept;
            Referencetype & operator = (const Referencetype & __rhs) noexcept;
            Referencetype(Referencetype && __rhs) noexcept;
            Referencetype & operator = (Referencetype && __rhs) noexcept;
            ~Referencetype();

            bool operator==(const Referencetype& __rhs) const noexcept;
            bool operator!=(const Referencetype& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * ID of reference type. This is a regular data type, but is kept in
             * its own
             * array to avoid polluting the existing datatype array with a new
             * default
             * field value.
             */
            int32_t id;

            /**
             * Numeric ID of the document type instances of the reference point
             * to.
             */
            int32_t targetTypeId;
        };
        typedef std::vector<Referencetype> ReferencetypeVector;

        typedef std::map<vespalib::string, Referencetype> ReferencetypeMap;

        class Importedfield {
        public:
            Importedfield(const ::config::StringVector & __lines);
            Importedfield(const vespalib::slime::Inspector & __inspector);
            Importedfield(const ::config::ConfigPayload & __payload);
            Importedfield()  noexcept;
            Importedfield(const Importedfield & __rhs) noexcept;
            Importedfield & operator = (const Importedfield & __rhs) noexcept;
            Importedfield(Importedfield && __rhs) noexcept;
            Importedfield & operator = (Importedfield && __rhs) noexcept;
            ~Importedfield();

            bool operator==(const Importedfield& __rhs) const noexcept;
            bool operator!=(const Importedfield& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * Imported fields (specified outside the document block in the
             * schema)
             */
            vespalib::string name;
        };
        typedef std::vector<Importedfield> ImportedfieldVector;

        typedef std::map<vespalib::string, Importedfield> ImportedfieldMap;

        Documenttype(const ::config::StringVector & __lines);
        Documenttype(const vespalib::slime::Inspector & __inspector);
        Documenttype(const ::config::ConfigPayload & __payload);
        Documenttype() ;
        Documenttype(const Documenttype & __rhs);
        Documenttype & operator = (const Documenttype & __rhs);
        Documenttype(Documenttype && __rhs) noexcept;
        Documenttype & operator = (Documenttype && __rhs) noexcept;
        ~Documenttype();

        bool operator==(const Documenttype& __rhs) const noexcept;
        bool operator!=(const Documenttype& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * The Id of the documenttype. Must be unique among all document types.
         */
        int32_t id;

        /** Specify the name of the document type. Must be unique. */
        vespalib::string name;

        /** Version is not used */
        int32_t version; // Default: 0

        /** Name of header struct defining document header. */
        int32_t headerstruct;

        /**
         * Specify a document field id. Must be unique within the document type.
         */
        int32_t bodystruct; // Default: 0
        InheritsVector inherits;
        DatatypeVector datatype;
        AnnotationtypeVector annotationtype;
        FieldsetsMap fieldsets;
        ReferencetypeVector referencetype;
        ImportedfieldVector importedfield;
    };
    typedef std::vector<Documenttype> DocumenttypeVector;

    typedef std::map<vespalib::string, Documenttype> DocumenttypeMap;

    class Doctype {
    public:
        class Inherits {
        public:
            Inherits(const ::config::StringVector & __lines);
            Inherits(const vespalib::slime::Inspector & __inspector);
            Inherits(const ::config::ConfigPayload & __payload);
            Inherits()  noexcept;
            Inherits(const Inherits & __rhs) noexcept;
            Inherits & operator = (const Inherits & __rhs) noexcept;
            Inherits(Inherits && __rhs) noexcept;
            Inherits & operator = (Inherits && __rhs) noexcept;
            ~Inherits();

            bool operator==(const Inherits& __rhs) const noexcept;
            bool operator!=(const Inherits& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** Specify document types to inherit */
            int32_t idx;
        };
        typedef std::vector<Inherits> InheritsVector;

        typedef std::map<vespalib::string, Inherits> InheritsMap;

        class Fieldsets {
        public:
            Fieldsets(const ::config::StringVector & __lines);
            Fieldsets(const vespalib::slime::Inspector & __inspector);
            Fieldsets(const ::config::ConfigPayload & __payload);
            Fieldsets() ;
            Fieldsets(const Fieldsets & __rhs);
            Fieldsets & operator = (const Fieldsets & __rhs);
            Fieldsets(Fieldsets && __rhs) noexcept;
            Fieldsets & operator = (Fieldsets && __rhs) noexcept;
            ~Fieldsets();

            bool operator==(const Fieldsets& __rhs) const noexcept;
            bool operator!=(const Fieldsets& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** Field sets available for this document type */
            ::config::StringVector fields;
        };
        typedef std::vector<Fieldsets> FieldsetsVector;

        typedef std::map<vespalib::string, Fieldsets> FieldsetsMap;

        class Importedfield {
        public:
            Importedfield(const ::config::StringVector & __lines);
            Importedfield(const vespalib::slime::Inspector & __inspector);
            Importedfield(const ::config::ConfigPayload & __payload);
            Importedfield()  noexcept;
            Importedfield(const Importedfield & __rhs) noexcept;
            Importedfield & operator = (const Importedfield & __rhs) noexcept;
            Importedfield(Importedfield && __rhs) noexcept;
            Importedfield & operator = (Importedfield && __rhs) noexcept;
            ~Importedfield();

            bool operator==(const Importedfield& __rhs) const noexcept;
            bool operator!=(const Importedfield& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * Imported fields (specified outside the document block in the
             * schema)
             */
            vespalib::string name;
        };
        typedef std::vector<Importedfield> ImportedfieldVector;

        typedef std::map<vespalib::string, Importedfield> ImportedfieldMap;

        class Primitivetype {
        public:
            Primitivetype(const ::config::StringVector & __lines);
            Primitivetype(const vespalib::slime::Inspector & __inspector);
            Primitivetype(const ::config::ConfigPayload & __payload);
            Primitivetype()  noexcept;
            Primitivetype(const Primitivetype & __rhs) noexcept;
            Primitivetype & operator = (const Primitivetype & __rhs) noexcept;
            Primitivetype(Primitivetype && __rhs) noexcept;
            Primitivetype & operator = (Primitivetype && __rhs) noexcept;
            ~Primitivetype();

            bool operator==(const Primitivetype& __rhs) const noexcept;
            bool operator!=(const Primitivetype& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            // Everything below here is configuration of data types defined by
            // this document type.

            // Primitive types must be present as built-in static members.

            /** Index of primitive type */
            int32_t idx;

            /** The name of this primitive type */
            vespalib::string name;
        };
        typedef std::vector<Primitivetype> PrimitivetypeVector;

        typedef std::map<vespalib::string, Primitivetype> PrimitivetypeMap;

        class Arraytype {
        public:
            Arraytype(const ::config::StringVector & __lines);
            Arraytype(const vespalib::slime::Inspector & __inspector);
            Arraytype(const ::config::ConfigPayload & __payload);
            Arraytype()  noexcept;
            Arraytype(const Arraytype & __rhs) noexcept;
            Arraytype & operator = (const Arraytype & __rhs) noexcept;
            Arraytype(Arraytype && __rhs) noexcept;
            Arraytype & operator = (Arraytype && __rhs) noexcept;
            ~Arraytype();

            bool operator==(const Arraytype& __rhs) const noexcept;
            bool operator!=(const Arraytype& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            // Arrays are the simplest collection type:

            /** Index of this array type */
            int32_t idx;

            /** Index of the element type this array type contains */
            int32_t elementtype;

            /** Internal ID of this datatype */
            int32_t internalid;
        };
        typedef std::vector<Arraytype> ArraytypeVector;

        typedef std::map<vespalib::string, Arraytype> ArraytypeMap;

        class Maptype {
        public:
            Maptype(const ::config::StringVector & __lines);
            Maptype(const vespalib::slime::Inspector & __inspector);
            Maptype(const ::config::ConfigPayload & __payload);
            Maptype()  noexcept;
            Maptype(const Maptype & __rhs) noexcept;
            Maptype & operator = (const Maptype & __rhs) noexcept;
            Maptype(Maptype && __rhs) noexcept;
            Maptype & operator = (Maptype && __rhs) noexcept;
            ~Maptype();

            bool operator==(const Maptype& __rhs) const noexcept;
            bool operator!=(const Maptype& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            // Maps are another collection type:

            /** Index of this map type */
            int32_t idx;

            /** Index of the key type used by this map type */
            int32_t keytype;

            /** Index of the key type used by this map type */
            int32_t valuetype;

            /** Internal ID of this datatype */
            int32_t internalid;
        };
        typedef std::vector<Maptype> MaptypeVector;

        typedef std::map<vespalib::string, Maptype> MaptypeMap;

        class Wsettype {
        public:
            Wsettype(const ::config::StringVector & __lines);
            Wsettype(const vespalib::slime::Inspector & __inspector);
            Wsettype(const ::config::ConfigPayload & __payload);
            Wsettype()  noexcept;
            Wsettype(const Wsettype & __rhs) noexcept;
            Wsettype & operator = (const Wsettype & __rhs) noexcept;
            Wsettype(Wsettype && __rhs) noexcept;
            Wsettype & operator = (Wsettype && __rhs) noexcept;
            ~Wsettype();

            bool operator==(const Wsettype& __rhs) const noexcept;
            bool operator!=(const Wsettype& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            // Weighted sets are more complicated;
            // they can be considered as an collection
            // of unique elements where each element has
            // an associated weight:

            /** Index of this weighted set type */
            int32_t idx;

            /**
             * Index of the element types contained in this weighted set type
             */
            int32_t elementtype;

            /**
             * Should an update to a nonexistent element cause it to be created
             */
            bool createifnonexistent; // Default: false

            /**
             * Should an element in a weighted set be removed if an update
             * changes the weight to 0
             */
            bool removeifzero; // Default: false

            /** Internal ID of this datatype */
            int32_t internalid;
        };
        typedef std::vector<Wsettype> WsettypeVector;

        typedef std::map<vespalib::string, Wsettype> WsettypeMap;

        class Tensortype {
        public:
            Tensortype(const ::config::StringVector & __lines);
            Tensortype(const vespalib::slime::Inspector & __inspector);
            Tensortype(const ::config::ConfigPayload & __payload);
            Tensortype()  noexcept;
            Tensortype(const Tensortype & __rhs) noexcept;
            Tensortype & operator = (const Tensortype & __rhs) noexcept;
            Tensortype(Tensortype && __rhs) noexcept;
            Tensortype & operator = (Tensortype && __rhs) noexcept;
            ~Tensortype();

            bool operator==(const Tensortype& __rhs) const noexcept;
            bool operator!=(const Tensortype& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            // Tensors have their own type system

            /** Index of this tensor type */
            int32_t idx;

            /** Description of the type of the actual tensors contained */
            vespalib::string detailedtype;
        };
        typedef std::vector<Tensortype> TensortypeVector;

        typedef std::map<vespalib::string, Tensortype> TensortypeMap;

        class Documentref {
        public:
            Documentref(const ::config::StringVector & __lines);
            Documentref(const vespalib::slime::Inspector & __inspector);
            Documentref(const ::config::ConfigPayload & __payload);
            Documentref()  noexcept;
            Documentref(const Documentref & __rhs) noexcept;
            Documentref & operator = (const Documentref & __rhs) noexcept;
            Documentref(Documentref && __rhs) noexcept;
            Documentref & operator = (Documentref && __rhs) noexcept;
            ~Documentref();

            bool operator==(const Documentref& __rhs) const noexcept;
            bool operator!=(const Documentref& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            // Document references refer to parent documents that a document can
            // import fields from:

            /** Index of this reference data type: */
            int32_t idx;

            /** Index of the document type this reference type refers to: */
            int32_t targettype;

            /** Internal ID of this datatype */
            int32_t internalid;
        };
        typedef std::vector<Documentref> DocumentrefVector;

        typedef std::map<vespalib::string, Documentref> DocumentrefMap;

        class Annotationtype {
        public:
            class Inherits {
            public:
                Inherits(const ::config::StringVector & __lines);
                Inherits(const vespalib::slime::Inspector & __inspector);
                Inherits(const ::config::ConfigPayload & __payload);
                Inherits()  noexcept;
                Inherits(const Inherits & __rhs) noexcept;
                Inherits & operator = (const Inherits & __rhs) noexcept;
                Inherits(Inherits && __rhs) noexcept;
                Inherits & operator = (Inherits && __rhs) noexcept;
                ~Inherits();

                bool operator==(const Inherits& __rhs) const noexcept;
                bool operator!=(const Inherits& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /** Index of annotation type that this type inherits. */
                int32_t idx;
            };
            typedef std::vector<Inherits> InheritsVector;

            typedef std::map<vespalib::string, Inherits> InheritsMap;

            Annotationtype(const ::config::StringVector & __lines);
            Annotationtype(const vespalib::slime::Inspector & __inspector);
            Annotationtype(const ::config::ConfigPayload & __payload);
            Annotationtype() ;
            Annotationtype(const Annotationtype & __rhs);
            Annotationtype & operator = (const Annotationtype & __rhs);
            Annotationtype(Annotationtype && __rhs) noexcept;
            Annotationtype & operator = (Annotationtype && __rhs) noexcept;
            ~Annotationtype();

            bool operator==(const Annotationtype& __rhs) const noexcept;
            bool operator!=(const Annotationtype& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            // Annotation types are another world, but are modeled here
            // as if they were also datatypes contained inside document types:

            /** Index of an annotation type. */
            int32_t idx;

            /** Name of the annotation type. */
            vespalib::string name;

            /** Internal id of this annotation type */
            int32_t internalid;

            /** Index of contained datatype of the annotation type, if any */
            int32_t datatype; // Default: -1
            InheritsVector inherits;
        };
        typedef std::vector<Annotationtype> AnnotationtypeVector;

        typedef std::map<vespalib::string, Annotationtype> AnnotationtypeMap;

        class Annotationref {
        public:
            Annotationref(const ::config::StringVector & __lines);
            Annotationref(const vespalib::slime::Inspector & __inspector);
            Annotationref(const ::config::ConfigPayload & __payload);
            Annotationref()  noexcept;
            Annotationref(const Annotationref & __rhs) noexcept;
            Annotationref & operator = (const Annotationref & __rhs) noexcept;
            Annotationref(Annotationref && __rhs) noexcept;
            Annotationref & operator = (Annotationref && __rhs) noexcept;
            ~Annotationref();

            bool operator==(const Annotationref& __rhs) const noexcept;
            bool operator!=(const Annotationref& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            // Annotation references are field values referring to
            // an annotation of a certain annotation type.

            /** Index of this annotation reference type */
            int32_t idx;

            /**
             * Index of the annotation type this annotation reference type
             * refers to
             */
            int32_t annotationtype;

            /** Internal ID of this datatype */
            int32_t internalid;
        };
        typedef std::vector<Annotationref> AnnotationrefVector;

        typedef std::map<vespalib::string, Annotationref> AnnotationrefMap;

        class Structtype {
        public:
            class Inherits {
            public:
                Inherits(const ::config::StringVector & __lines);
                Inherits(const vespalib::slime::Inspector & __inspector);
                Inherits(const ::config::ConfigPayload & __payload);
                Inherits()  noexcept;
                Inherits(const Inherits & __rhs) noexcept;
                Inherits & operator = (const Inherits & __rhs) noexcept;
                Inherits(Inherits && __rhs) noexcept;
                Inherits & operator = (Inherits && __rhs) noexcept;
                ~Inherits();

                bool operator==(const Inherits& __rhs) const noexcept;
                bool operator!=(const Inherits& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /** Index of another struct type to inherit */
                int32_t type;
            };
            typedef std::vector<Inherits> InheritsVector;

            typedef std::map<vespalib::string, Inherits> InheritsMap;

            class Field {
            public:
                Field(const ::config::StringVector & __lines);
                Field(const vespalib::slime::Inspector & __inspector);
                Field(const ::config::ConfigPayload & __payload);
                Field()  noexcept;
                Field(const Field & __rhs) noexcept;
                Field & operator = (const Field & __rhs) noexcept;
                Field(Field && __rhs) noexcept;
                Field & operator = (Field && __rhs) noexcept;
                ~Field();

                bool operator==(const Field& __rhs) const noexcept;
                bool operator!=(const Field& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /**
                 * Name of a struct field. Must be unique within the struct
                 * type.
                 */
                vespalib::string name;

                /** The "field id" - used in serialized format! */
                int32_t internalid;

                /** Index of the type of this field */
                int32_t type;
            };
            typedef std::vector<Field> FieldVector;

            typedef std::map<vespalib::string, Field> FieldMap;

            Structtype(const ::config::StringVector & __lines);
            Structtype(const vespalib::slime::Inspector & __inspector);
            Structtype(const ::config::ConfigPayload & __payload);
            Structtype() ;
            Structtype(const Structtype & __rhs);
            Structtype & operator = (const Structtype & __rhs);
            Structtype(Structtype && __rhs) noexcept;
            Structtype & operator = (Structtype && __rhs) noexcept;
            ~Structtype();

            bool operator==(const Structtype& __rhs) const noexcept;
            bool operator!=(const Structtype& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            // A struct is just a named collections of fields:

            /** Index of this struct type */
            int32_t idx;

            /** Name of the struct type. Must be unique within documenttype. */
            vespalib::string name;
            InheritsVector inherits;
            FieldVector field;

            /** Internal ID of this datatype */
            int32_t internalid;
        };
        typedef std::vector<Structtype> StructtypeVector;

        typedef std::map<vespalib::string, Structtype> StructtypeMap;

        Doctype(const ::config::StringVector & __lines);
        Doctype(const vespalib::slime::Inspector & __inspector);
        Doctype(const ::config::ConfigPayload & __payload);
        Doctype() ;
        Doctype(const Doctype & __rhs);
        Doctype & operator = (const Doctype & __rhs);
        Doctype(Doctype && __rhs) noexcept;
        Doctype & operator = (Doctype && __rhs) noexcept;
        ~Doctype();

        bool operator==(const Doctype& __rhs) const noexcept;
        bool operator!=(const Doctype& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        // Here starts a new model for how datatypes are configured, where
        // everything is per document-type, and each documenttype contains the
        // datatypes it defines.

        // Note: we will include the built-in "document" document
        // type that all other doctypes inherit from also, in order
        // to get all the primitive and built-in types declared
        // with an idx we can refer to.

        // Note: indexes are only meaningful as internal references in this
        // config; they will typically be sequential (1,2,3,...) in the order
        // that they are generated (but nothing should depend on that).

        /**
         * 
         * Name of the document type. Must be unique.
         */
        vespalib::string name;

        /** Index of this type (as a datatype which can be referred to). */
        int32_t idx;

        /** Internal ID of this datatype */
        int32_t internalid;
        InheritsVector inherits;

        /** Index of struct defining document fields */
        int32_t contentstruct;
        FieldsetsMap fieldsets;
        ImportedfieldVector importedfield;
        PrimitivetypeVector primitivetype;
        ArraytypeVector arraytype;
        MaptypeVector maptype;
        WsettypeVector wsettype;
        TensortypeVector tensortype;
        DocumentrefVector documentref;
        AnnotationtypeVector annotationtype;
        AnnotationrefVector annotationref;
        StructtypeVector structtype;
    };
    typedef std::vector<Doctype> DoctypeVector;

    typedef std::map<vespalib::string, Doctype> DoctypeMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalDocumenttypesType(const ::config::ConfigValue & __value);
    InternalDocumenttypesType(const ::config::ConfigDataBuffer & __value);
    InternalDocumenttypesType(const ::config::ConfigPayload & __payload);
    InternalDocumenttypesType() ;
    InternalDocumenttypesType(const InternalDocumenttypesType & __rhs);
    InternalDocumenttypesType & operator = (const InternalDocumenttypesType & __rhs);
    InternalDocumenttypesType(InternalDocumenttypesType && __rhs) noexcept;
    InternalDocumenttypesType & operator = (InternalDocumenttypesType && __rhs) noexcept;
    ~InternalDocumenttypesType();

    bool operator==(const InternalDocumenttypesType& __rhs) const noexcept;
    bool operator!=(const InternalDocumenttypesType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /**
     * Whether attempts to set an undefined field should be ignored rather than
     * causing an error
     */
    bool ignoreundefinedfields; // Default: false

    /** Prefer "Vespa 8" format for the "position" type */
    bool usev8geopositions; // Default: false
    DocumenttypeVector documenttype;
    DoctypeVector doctype;
};

} // namespace internal

typedef internal::InternalDocumenttypesType DocumenttypesConfigBuilder;
typedef const internal::InternalDocumenttypesType DocumenttypesConfig;

} // namespace document::config
#endif // VESPA_config_DOCUMENT_CONFIG_DOCUMENTTYPES_H
