#include "config-document-protocol-policies.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace documentapi::messagebus::protocol {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalDocumentProtocolPoliciesType::CONFIG_DEF_MD5("b97766709a49f349f355d4379d6f9afa");
const vespalib::string InternalDocumentProtocolPoliciesType::CONFIG_DEF_NAME("document-protocol-policies");
const vespalib::string InternalDocumentProtocolPoliciesType::CONFIG_DEF_NAMESPACE("documentapi.messagebus.protocol");
const int64_t InternalDocumentProtocolPoliciesType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=documentapi.messagebus.protocol",
"cluster{}.defaultRoute string",
"cluster{}.route[].name string",
"cluster{}.route[].messageType int",
"cluster{}.selector string",
};
const ::config::StringVector InternalDocumentProtocolPoliciesType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalDocumentProtocolPoliciesType::Cluster::Route::Route() noexcept
    : name(),
      messageType(0)
{
}

InternalDocumentProtocolPoliciesType::Cluster::Route::Route(const Route & __rhs) noexcept = default;
InternalDocumentProtocolPoliciesType::Cluster::Route & InternalDocumentProtocolPoliciesType::Cluster::Route::operator = (const Route & __rhs) noexcept = default;
InternalDocumentProtocolPoliciesType::Cluster::Route::Route(Route && __rhs) noexcept = default;
InternalDocumentProtocolPoliciesType::Cluster::Route & InternalDocumentProtocolPoliciesType::Cluster::Route::operator = (Route && __rhs) noexcept = default;
InternalDocumentProtocolPoliciesType::Cluster::Route::~Route() = default; 
InternalDocumentProtocolPoliciesType::Cluster::Route::Route(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    messageType = ConfigParser::parse<int32_t>("messageType", __lines);
    ConfigParser::stripLinesForKey("messageType", __remainingValuesToParse);
}

bool
InternalDocumentProtocolPoliciesType::Cluster::Route::
operator==(const Route& __rhs) const noexcept
{
    return (name == __rhs.name &&
            messageType == __rhs.messageType);
}

bool
InternalDocumentProtocolPoliciesType::Cluster::Route::
operator!=(const Route& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumentProtocolPoliciesType::Cluster::Route::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("messageType");
        __c.setString("type", "int");
        __c.setLong("value", messageType);
    }
}

InternalDocumentProtocolPoliciesType::Cluster::Route::Route(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    messageType = __inspector["messageType"]["value"].asLong();
}

InternalDocumentProtocolPoliciesType::Cluster::Route::Route(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    messageType = ::config::internal::ValueConverter<int32_t>()("messageType", __inspector["messageType"]);
}

InternalDocumentProtocolPoliciesType::Cluster::Cluster()
    : defaultRoute(),
      route(),
      selector()
{
}

InternalDocumentProtocolPoliciesType::Cluster::Cluster(const Cluster & __rhs) = default;
InternalDocumentProtocolPoliciesType::Cluster & InternalDocumentProtocolPoliciesType::Cluster::operator = (const Cluster & __rhs) = default;
InternalDocumentProtocolPoliciesType::Cluster::Cluster(Cluster && __rhs) noexcept = default;
InternalDocumentProtocolPoliciesType::Cluster & InternalDocumentProtocolPoliciesType::Cluster::operator = (Cluster && __rhs) noexcept = default;
InternalDocumentProtocolPoliciesType::Cluster::~Cluster() = default; 
InternalDocumentProtocolPoliciesType::Cluster::Cluster(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    defaultRoute = ConfigParser::parse<vespalib::string>("defaultRoute", __lines);
    ConfigParser::stripLinesForKey("defaultRoute", __remainingValuesToParse);
    route = ConfigParser::parseArray<RouteVector>("route", __lines);
    ConfigParser::stripLinesForKey("route", __remainingValuesToParse);
    selector = ConfigParser::parse<vespalib::string>("selector", __lines);
    ConfigParser::stripLinesForKey("selector", __remainingValuesToParse);
}

bool
InternalDocumentProtocolPoliciesType::Cluster::
operator==(const Cluster& __rhs) const noexcept
{
    return (defaultRoute == __rhs.defaultRoute &&
            route == __rhs.route &&
            selector == __rhs.selector);
}

bool
InternalDocumentProtocolPoliciesType::Cluster::
operator!=(const Cluster& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumentProtocolPoliciesType::Cluster::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("defaultRoute");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(defaultRoute));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("route");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : route) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("selector");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(selector));
    }
}

InternalDocumentProtocolPoliciesType::Cluster::Cluster(const vespalib::slime::Inspector & __inspector)
{
    defaultRoute = __inspector["defaultRoute"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["route"]["value"].children(); __i++) {
        route.push_back(Route(__inspector["route"]["value"][__i]["value"]));
    }
    selector = __inspector["selector"]["value"].asString().make_string();
}

InternalDocumentProtocolPoliciesType::Cluster::Cluster(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    defaultRoute = ::config::internal::ValueConverter<vespalib::string>()("defaultRoute", __inspector["defaultRoute"]);
    ::config::internal::VectorInserter<RouteVector> __routeInserter(route);
    __inspector["route"].traverse(__routeInserter);
    selector = ::config::internal::ValueConverter<vespalib::string>()("selector", __inspector["selector"]);
}

InternalDocumentProtocolPoliciesType::InternalDocumentProtocolPoliciesType()
    : cluster()
{
}

InternalDocumentProtocolPoliciesType::InternalDocumentProtocolPoliciesType(const InternalDocumentProtocolPoliciesType & __rhs) = default;
InternalDocumentProtocolPoliciesType & InternalDocumentProtocolPoliciesType::operator =(const InternalDocumentProtocolPoliciesType & __rhs) = default;
InternalDocumentProtocolPoliciesType::InternalDocumentProtocolPoliciesType(InternalDocumentProtocolPoliciesType && __rhs) noexcept = default;
InternalDocumentProtocolPoliciesType & InternalDocumentProtocolPoliciesType::operator =(InternalDocumentProtocolPoliciesType && __rhs) noexcept = default;
InternalDocumentProtocolPoliciesType::~InternalDocumentProtocolPoliciesType() = default; 
InternalDocumentProtocolPoliciesType::InternalDocumentProtocolPoliciesType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        cluster = ConfigParser::parseMap<Cluster>("cluster", __lines);
        ConfigParser::stripLinesForKey("cluster", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalDocumentProtocolPoliciesType::
operator==(const InternalDocumentProtocolPoliciesType& __rhs) const noexcept
{
    return (cluster == __rhs.cluster);
}

bool
InternalDocumentProtocolPoliciesType::
operator!=(const InternalDocumentProtocolPoliciesType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumentProtocolPoliciesType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("cluster");
        __c.setString("type", "map");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & entry : cluster) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("key", vespalib::Memory(entry.first));
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            entry.second.serialize(__c4);
        }
    }
}

InternalDocumentProtocolPoliciesType::InternalDocumentProtocolPoliciesType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["cluster"]["value"].children(); __i++) {
        cluster[__inspector["cluster"]["value"][__i]["key"].asString().make_string()] = Cluster(__inspector["cluster"]["value"][__i]["value"]);
    }
}

InternalDocumentProtocolPoliciesType::InternalDocumentProtocolPoliciesType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::MapInserter<Cluster> __clusterInserter(cluster);
    __inspector["cluster"].traverse(__clusterInserter);
}

} // namespace internal

} // namespace documentapi::messagebus::protocol
