/**
 * @class documentapi::messagebus::protocol::DocumentProtocolPolicies
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_DOCUMENTAPI_MESSAGEBUS_PROTOCOL_DOCUMENTPROTOCOLPOLICIES_H
#define CLOUD_CONFIG_DOCUMENTAPI_MESSAGEBUS_PROTOCOL_DOCUMENTPROTOCOLPOLICIES_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace documentapi::messagebus::protocol {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalDocumentProtocolPoliciesType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalDocumentProtocolPoliciesType> UP;
    class Cluster {
    public:
        class Route {
        public:
            Route(const ::config::StringVector & __lines);
            Route(const vespalib::slime::Inspector & __inspector);
            Route(const ::config::ConfigPayload & __payload);
            Route()  noexcept;
            Route(const Route & __rhs) noexcept;
            Route & operator = (const Route & __rhs) noexcept;
            Route(Route && __rhs) noexcept;
            Route & operator = (Route && __rhs) noexcept;
            ~Route();

            bool operator==(const Route& __rhs) const noexcept;
            bool operator!=(const Route& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** The name of the route. */
            vespalib::string name;

            /** The document protocol message type triggering this route. */
            int32_t messageType;
        };
        typedef std::vector<Route> RouteVector;

        typedef std::map<vespalib::string, Route> RouteMap;

        Cluster(const ::config::StringVector & __lines);
        Cluster(const vespalib::slime::Inspector & __inspector);
        Cluster(const ::config::ConfigPayload & __payload);
        Cluster() ;
        Cluster(const Cluster & __rhs);
        Cluster & operator = (const Cluster & __rhs);
        Cluster(Cluster && __rhs) noexcept;
        Cluster & operator = (Cluster && __rhs) noexcept;
        ~Cluster();

        bool operator==(const Cluster& __rhs) const noexcept;
        bool operator!=(const Cluster& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        //  
        // Super config for all policies in the document protocol, keyed by
        // content cluster ids.
        // Note: ContentPolicy also uses the "distribution" config.
        //  

        /**
         *  
         * Config used by MessageTypeRouteSelectorPolicy
         *  
         * Default route if no override is set for a type.
         */
        vespalib::string defaultRoute;
        RouteVector route;

        /**
         *  
         * Config used by DocumentRouteSelectorPolicy
         *  
         * The document selector for this cluster route.
         */
        vespalib::string selector;
    };
    typedef std::vector<Cluster> ClusterVector;

    typedef std::map<vespalib::string, Cluster> ClusterMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalDocumentProtocolPoliciesType(const ::config::ConfigValue & __value);
    InternalDocumentProtocolPoliciesType(const ::config::ConfigDataBuffer & __value);
    InternalDocumentProtocolPoliciesType(const ::config::ConfigPayload & __payload);
    InternalDocumentProtocolPoliciesType() ;
    InternalDocumentProtocolPoliciesType(const InternalDocumentProtocolPoliciesType & __rhs);
    InternalDocumentProtocolPoliciesType & operator = (const InternalDocumentProtocolPoliciesType & __rhs);
    InternalDocumentProtocolPoliciesType(InternalDocumentProtocolPoliciesType && __rhs) noexcept;
    InternalDocumentProtocolPoliciesType & operator = (InternalDocumentProtocolPoliciesType && __rhs) noexcept;
    ~InternalDocumentProtocolPoliciesType();

    bool operator==(const InternalDocumentProtocolPoliciesType& __rhs) const noexcept;
    bool operator!=(const InternalDocumentProtocolPoliciesType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    ClusterMap cluster;
};

} // namespace internal

typedef internal::InternalDocumentProtocolPoliciesType DocumentProtocolPoliciesConfigBuilder;
typedef const internal::InternalDocumentProtocolPoliciesType DocumentProtocolPoliciesConfig;

} // namespace documentapi::messagebus::protocol
#endif // VESPA_config_DOCUMENTAPI_MESSAGEBUS_PROTOCOL_DOCUMENTPROTOCOLPOLICIES_H
