#include "config-documentrouteselectorpolicy.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace documentapi::messagebus::protocol {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalDocumentrouteselectorpolicyType::CONFIG_DEF_MD5("8c9f66e2e55e8863aea07acdf1ab654e");
const vespalib::string InternalDocumentrouteselectorpolicyType::CONFIG_DEF_NAME("documentrouteselectorpolicy");
const vespalib::string InternalDocumentrouteselectorpolicyType::CONFIG_DEF_NAMESPACE("documentapi.messagebus.protocol");
const int64_t InternalDocumentrouteselectorpolicyType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=documentapi.messagebus.protocol",
"route[].name string",
"route[].selector string",
"route[].feed string default=\"\"",
};
const ::config::StringVector InternalDocumentrouteselectorpolicyType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalDocumentrouteselectorpolicyType::Route::Route() noexcept
    : name(),
      selector(),
      feed("")
{
}

InternalDocumentrouteselectorpolicyType::Route::Route(const Route & __rhs) noexcept = default;
InternalDocumentrouteselectorpolicyType::Route & InternalDocumentrouteselectorpolicyType::Route::operator = (const Route & __rhs) noexcept = default;
InternalDocumentrouteselectorpolicyType::Route::Route(Route && __rhs) noexcept = default;
InternalDocumentrouteselectorpolicyType::Route & InternalDocumentrouteselectorpolicyType::Route::operator = (Route && __rhs) noexcept = default;
InternalDocumentrouteselectorpolicyType::Route::~Route() = default; 
InternalDocumentrouteselectorpolicyType::Route::Route(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    selector = ConfigParser::parse<vespalib::string>("selector", __lines);
    ConfigParser::stripLinesForKey("selector", __remainingValuesToParse);
    feed = ConfigParser::parse<vespalib::string>("feed", __lines, "");
    ConfigParser::stripLinesForKey("feed", __remainingValuesToParse);
}

bool
InternalDocumentrouteselectorpolicyType::Route::
operator==(const Route& __rhs) const noexcept
{
    return (name == __rhs.name &&
            selector == __rhs.selector &&
            feed == __rhs.feed);
}

bool
InternalDocumentrouteselectorpolicyType::Route::
operator!=(const Route& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumentrouteselectorpolicyType::Route::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("selector");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(selector));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("feed");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(feed));
    }
}

InternalDocumentrouteselectorpolicyType::Route::Route(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    selector = __inspector["selector"]["value"].asString().make_string();
    feed = __inspector["feed"]["value"].asString().make_string();
}

InternalDocumentrouteselectorpolicyType::Route::Route(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    selector = ::config::internal::ValueConverter<vespalib::string>()("selector", __inspector["selector"]);
    feed = ::config::internal::ValueConverter<vespalib::string>()(__inspector["feed"], "");
}

InternalDocumentrouteselectorpolicyType::InternalDocumentrouteselectorpolicyType()
    : route()
{
}

InternalDocumentrouteselectorpolicyType::InternalDocumentrouteselectorpolicyType(const InternalDocumentrouteselectorpolicyType & __rhs) = default;
InternalDocumentrouteselectorpolicyType & InternalDocumentrouteselectorpolicyType::operator =(const InternalDocumentrouteselectorpolicyType & __rhs) = default;
InternalDocumentrouteselectorpolicyType::InternalDocumentrouteselectorpolicyType(InternalDocumentrouteselectorpolicyType && __rhs) noexcept = default;
InternalDocumentrouteselectorpolicyType & InternalDocumentrouteselectorpolicyType::operator =(InternalDocumentrouteselectorpolicyType && __rhs) noexcept = default;
InternalDocumentrouteselectorpolicyType::~InternalDocumentrouteselectorpolicyType() = default; 
InternalDocumentrouteselectorpolicyType::InternalDocumentrouteselectorpolicyType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        route = ConfigParser::parseArray<RouteVector>("route", __lines);
        ConfigParser::stripLinesForKey("route", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalDocumentrouteselectorpolicyType::
operator==(const InternalDocumentrouteselectorpolicyType& __rhs) const noexcept
{
    return (route == __rhs.route);
}

bool
InternalDocumentrouteselectorpolicyType::
operator!=(const InternalDocumentrouteselectorpolicyType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalDocumentrouteselectorpolicyType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("route");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : route) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalDocumentrouteselectorpolicyType::InternalDocumentrouteselectorpolicyType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["route"]["value"].children(); __i++) {
        route.push_back(Route(__inspector["route"]["value"][__i]["value"]));
    }
}

InternalDocumentrouteselectorpolicyType::InternalDocumentrouteselectorpolicyType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<RouteVector> __routeInserter(route);
    __inspector["route"].traverse(__routeInserter);
}

} // namespace internal

} // namespace documentapi::messagebus::protocol
