#include "config-logd.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace cloud::config::log {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalLogdType::CONFIG_DEF_MD5("d990b8d5b58421a1bd40bcba0079621c");
const vespalib::string InternalLogdType::CONFIG_DEF_NAME("logd");
const vespalib::string InternalLogdType::CONFIG_DEF_NAMESPACE("cloud.config.log");
const int64_t InternalLogdType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=cloud.config.log",
"stateport int default=0",
"logserver.host string default=\"localhost\"",
"logserver.rpcport int default=5822",
"logserver.use bool default=true",
"loglevel.fatal.forward bool default=true",
"loglevel.error.forward bool default=true",
"loglevel.warning.forward bool default=true",
"loglevel.config.forward bool default=true",
"loglevel.info.forward bool default=true",
"loglevel.event.forward bool default=true",
"loglevel.debug.forward bool default=false",
"loglevel.spam.forward bool default=false",
"rotate.size int default=10000000",
"rotate.age int default=86400",
"remove.totalmegabytes int default=1000",
"remove.age int default=30",
};
const ::config::StringVector InternalLogdType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalLogdType::Logserver::Logserver() noexcept
    : host("localhost"),
      rpcport(5822),
      use(true)
{
}

InternalLogdType::Logserver::Logserver(const Logserver & __rhs) noexcept = default;
InternalLogdType::Logserver & InternalLogdType::Logserver::operator = (const Logserver & __rhs) noexcept = default;
InternalLogdType::Logserver::Logserver(Logserver && __rhs) noexcept = default;
InternalLogdType::Logserver & InternalLogdType::Logserver::operator = (Logserver && __rhs) noexcept = default;
InternalLogdType::Logserver::~Logserver() = default; 
InternalLogdType::Logserver::Logserver(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    host = ConfigParser::parse<vespalib::string>("host", __lines, "localhost");
    ConfigParser::stripLinesForKey("host", __remainingValuesToParse);
    rpcport = ConfigParser::parse<int32_t>("rpcport", __lines, 5822);
    ConfigParser::stripLinesForKey("rpcport", __remainingValuesToParse);
    use = ConfigParser::parse<bool>("use", __lines, true);
    ConfigParser::stripLinesForKey("use", __remainingValuesToParse);
}

bool
InternalLogdType::Logserver::operator==(const Logserver& __rhs) const noexcept
{
    return (host == __rhs.host &&
            rpcport == __rhs.rpcport &&
            use == __rhs.use);
}

bool
InternalLogdType::Logserver::operator!=(const Logserver& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLogdType::Logserver::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("host");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(host));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("rpcport");
        __c.setString("type", "int");
        __c.setLong("value", rpcport);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("use");
        __c.setString("type", "bool");
        __c.setBool("value", use);
    }
}

InternalLogdType::Logserver::Logserver(const vespalib::slime::Inspector & __inspector)
{
    host = __inspector["host"]["value"].asString().make_string();
    rpcport = __inspector["rpcport"]["value"].asLong();
    use = __inspector["use"]["value"].asBool();
}

InternalLogdType::Logserver::Logserver(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    host = ::config::internal::ValueConverter<vespalib::string>()(__inspector["host"], "localhost");
    rpcport = ::config::internal::ValueConverter<int32_t>()(__inspector["rpcport"], 5822);
    use = ::config::internal::ValueConverter<bool>()(__inspector["use"], true);
}

InternalLogdType::Loglevel::Fatal::Fatal() noexcept
    : forward(true)
{
}

InternalLogdType::Loglevel::Fatal::Fatal(const Fatal & __rhs) noexcept = default;
InternalLogdType::Loglevel::Fatal & InternalLogdType::Loglevel::Fatal::operator = (const Fatal & __rhs) noexcept = default;
InternalLogdType::Loglevel::Fatal::Fatal(Fatal && __rhs) noexcept = default;
InternalLogdType::Loglevel::Fatal & InternalLogdType::Loglevel::Fatal::operator = (Fatal && __rhs) noexcept = default;
InternalLogdType::Loglevel::Fatal::~Fatal() = default; 
InternalLogdType::Loglevel::Fatal::Fatal(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    forward = ConfigParser::parse<bool>("forward", __lines, true);
    ConfigParser::stripLinesForKey("forward", __remainingValuesToParse);
}

bool
InternalLogdType::Loglevel::Fatal::operator==(const Fatal& __rhs) const noexcept
{
    return (forward == __rhs.forward);
}

bool
InternalLogdType::Loglevel::Fatal::operator!=(const Fatal& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLogdType::Loglevel::Fatal::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("forward");
        __c.setString("type", "bool");
        __c.setBool("value", forward);
    }
}

InternalLogdType::Loglevel::Fatal::Fatal(const vespalib::slime::Inspector & __inspector)
{
    forward = __inspector["forward"]["value"].asBool();
}

InternalLogdType::Loglevel::Fatal::Fatal(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    forward = ::config::internal::ValueConverter<bool>()(__inspector["forward"], true);
}

InternalLogdType::Loglevel::Error::Error() noexcept
    : forward(true)
{
}

InternalLogdType::Loglevel::Error::Error(const Error & __rhs) noexcept = default;
InternalLogdType::Loglevel::Error & InternalLogdType::Loglevel::Error::operator = (const Error & __rhs) noexcept = default;
InternalLogdType::Loglevel::Error::Error(Error && __rhs) noexcept = default;
InternalLogdType::Loglevel::Error & InternalLogdType::Loglevel::Error::operator = (Error && __rhs) noexcept = default;
InternalLogdType::Loglevel::Error::~Error() = default; 
InternalLogdType::Loglevel::Error::Error(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    forward = ConfigParser::parse<bool>("forward", __lines, true);
    ConfigParser::stripLinesForKey("forward", __remainingValuesToParse);
}

bool
InternalLogdType::Loglevel::Error::operator==(const Error& __rhs) const noexcept
{
    return (forward == __rhs.forward);
}

bool
InternalLogdType::Loglevel::Error::operator!=(const Error& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLogdType::Loglevel::Error::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("forward");
        __c.setString("type", "bool");
        __c.setBool("value", forward);
    }
}

InternalLogdType::Loglevel::Error::Error(const vespalib::slime::Inspector & __inspector)
{
    forward = __inspector["forward"]["value"].asBool();
}

InternalLogdType::Loglevel::Error::Error(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    forward = ::config::internal::ValueConverter<bool>()(__inspector["forward"], true);
}

InternalLogdType::Loglevel::Warning::Warning() noexcept
    : forward(true)
{
}

InternalLogdType::Loglevel::Warning::Warning(const Warning & __rhs) noexcept = default;
InternalLogdType::Loglevel::Warning & InternalLogdType::Loglevel::Warning::operator = (const Warning & __rhs) noexcept = default;
InternalLogdType::Loglevel::Warning::Warning(Warning && __rhs) noexcept = default;
InternalLogdType::Loglevel::Warning & InternalLogdType::Loglevel::Warning::operator = (Warning && __rhs) noexcept = default;
InternalLogdType::Loglevel::Warning::~Warning() = default; 
InternalLogdType::Loglevel::Warning::Warning(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    forward = ConfigParser::parse<bool>("forward", __lines, true);
    ConfigParser::stripLinesForKey("forward", __remainingValuesToParse);
}

bool
InternalLogdType::Loglevel::Warning::operator==(const Warning& __rhs) const noexcept
{
    return (forward == __rhs.forward);
}

bool
InternalLogdType::Loglevel::Warning::operator!=(const Warning& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLogdType::Loglevel::Warning::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("forward");
        __c.setString("type", "bool");
        __c.setBool("value", forward);
    }
}

InternalLogdType::Loglevel::Warning::Warning(const vespalib::slime::Inspector & __inspector)
{
    forward = __inspector["forward"]["value"].asBool();
}

InternalLogdType::Loglevel::Warning::Warning(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    forward = ::config::internal::ValueConverter<bool>()(__inspector["forward"], true);
}

InternalLogdType::Loglevel::Config::Config() noexcept
    : forward(true)
{
}

InternalLogdType::Loglevel::Config::Config(const Config & __rhs) noexcept = default;
InternalLogdType::Loglevel::Config & InternalLogdType::Loglevel::Config::operator = (const Config & __rhs) noexcept = default;
InternalLogdType::Loglevel::Config::Config(Config && __rhs) noexcept = default;
InternalLogdType::Loglevel::Config & InternalLogdType::Loglevel::Config::operator = (Config && __rhs) noexcept = default;
InternalLogdType::Loglevel::Config::~Config() = default; 
InternalLogdType::Loglevel::Config::Config(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    forward = ConfigParser::parse<bool>("forward", __lines, true);
    ConfigParser::stripLinesForKey("forward", __remainingValuesToParse);
}

bool
InternalLogdType::Loglevel::Config::operator==(const Config& __rhs) const noexcept
{
    return (forward == __rhs.forward);
}

bool
InternalLogdType::Loglevel::Config::operator!=(const Config& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLogdType::Loglevel::Config::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("forward");
        __c.setString("type", "bool");
        __c.setBool("value", forward);
    }
}

InternalLogdType::Loglevel::Config::Config(const vespalib::slime::Inspector & __inspector)
{
    forward = __inspector["forward"]["value"].asBool();
}

InternalLogdType::Loglevel::Config::Config(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    forward = ::config::internal::ValueConverter<bool>()(__inspector["forward"], true);
}

InternalLogdType::Loglevel::Info::Info() noexcept
    : forward(true)
{
}

InternalLogdType::Loglevel::Info::Info(const Info & __rhs) noexcept = default;
InternalLogdType::Loglevel::Info & InternalLogdType::Loglevel::Info::operator = (const Info & __rhs) noexcept = default;
InternalLogdType::Loglevel::Info::Info(Info && __rhs) noexcept = default;
InternalLogdType::Loglevel::Info & InternalLogdType::Loglevel::Info::operator = (Info && __rhs) noexcept = default;
InternalLogdType::Loglevel::Info::~Info() = default; 
InternalLogdType::Loglevel::Info::Info(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    forward = ConfigParser::parse<bool>("forward", __lines, true);
    ConfigParser::stripLinesForKey("forward", __remainingValuesToParse);
}

bool
InternalLogdType::Loglevel::Info::operator==(const Info& __rhs) const noexcept
{
    return (forward == __rhs.forward);
}

bool
InternalLogdType::Loglevel::Info::operator!=(const Info& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLogdType::Loglevel::Info::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("forward");
        __c.setString("type", "bool");
        __c.setBool("value", forward);
    }
}

InternalLogdType::Loglevel::Info::Info(const vespalib::slime::Inspector & __inspector)
{
    forward = __inspector["forward"]["value"].asBool();
}

InternalLogdType::Loglevel::Info::Info(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    forward = ::config::internal::ValueConverter<bool>()(__inspector["forward"], true);
}

InternalLogdType::Loglevel::Event::Event() noexcept
    : forward(true)
{
}

InternalLogdType::Loglevel::Event::Event(const Event & __rhs) noexcept = default;
InternalLogdType::Loglevel::Event & InternalLogdType::Loglevel::Event::operator = (const Event & __rhs) noexcept = default;
InternalLogdType::Loglevel::Event::Event(Event && __rhs) noexcept = default;
InternalLogdType::Loglevel::Event & InternalLogdType::Loglevel::Event::operator = (Event && __rhs) noexcept = default;
InternalLogdType::Loglevel::Event::~Event() = default; 
InternalLogdType::Loglevel::Event::Event(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    forward = ConfigParser::parse<bool>("forward", __lines, true);
    ConfigParser::stripLinesForKey("forward", __remainingValuesToParse);
}

bool
InternalLogdType::Loglevel::Event::operator==(const Event& __rhs) const noexcept
{
    return (forward == __rhs.forward);
}

bool
InternalLogdType::Loglevel::Event::operator!=(const Event& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLogdType::Loglevel::Event::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("forward");
        __c.setString("type", "bool");
        __c.setBool("value", forward);
    }
}

InternalLogdType::Loglevel::Event::Event(const vespalib::slime::Inspector & __inspector)
{
    forward = __inspector["forward"]["value"].asBool();
}

InternalLogdType::Loglevel::Event::Event(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    forward = ::config::internal::ValueConverter<bool>()(__inspector["forward"], true);
}

InternalLogdType::Loglevel::Debug::Debug() noexcept
    : forward(false)
{
}

InternalLogdType::Loglevel::Debug::Debug(const Debug & __rhs) noexcept = default;
InternalLogdType::Loglevel::Debug & InternalLogdType::Loglevel::Debug::operator = (const Debug & __rhs) noexcept = default;
InternalLogdType::Loglevel::Debug::Debug(Debug && __rhs) noexcept = default;
InternalLogdType::Loglevel::Debug & InternalLogdType::Loglevel::Debug::operator = (Debug && __rhs) noexcept = default;
InternalLogdType::Loglevel::Debug::~Debug() = default; 
InternalLogdType::Loglevel::Debug::Debug(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    forward = ConfigParser::parse<bool>("forward", __lines, false);
    ConfigParser::stripLinesForKey("forward", __remainingValuesToParse);
}

bool
InternalLogdType::Loglevel::Debug::operator==(const Debug& __rhs) const noexcept
{
    return (forward == __rhs.forward);
}

bool
InternalLogdType::Loglevel::Debug::operator!=(const Debug& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLogdType::Loglevel::Debug::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("forward");
        __c.setString("type", "bool");
        __c.setBool("value", forward);
    }
}

InternalLogdType::Loglevel::Debug::Debug(const vespalib::slime::Inspector & __inspector)
{
    forward = __inspector["forward"]["value"].asBool();
}

InternalLogdType::Loglevel::Debug::Debug(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    forward = ::config::internal::ValueConverter<bool>()(__inspector["forward"], false);
}

InternalLogdType::Loglevel::Spam::Spam() noexcept
    : forward(false)
{
}

InternalLogdType::Loglevel::Spam::Spam(const Spam & __rhs) noexcept = default;
InternalLogdType::Loglevel::Spam & InternalLogdType::Loglevel::Spam::operator = (const Spam & __rhs) noexcept = default;
InternalLogdType::Loglevel::Spam::Spam(Spam && __rhs) noexcept = default;
InternalLogdType::Loglevel::Spam & InternalLogdType::Loglevel::Spam::operator = (Spam && __rhs) noexcept = default;
InternalLogdType::Loglevel::Spam::~Spam() = default; 
InternalLogdType::Loglevel::Spam::Spam(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    forward = ConfigParser::parse<bool>("forward", __lines, false);
    ConfigParser::stripLinesForKey("forward", __remainingValuesToParse);
}

bool
InternalLogdType::Loglevel::Spam::operator==(const Spam& __rhs) const noexcept
{
    return (forward == __rhs.forward);
}

bool
InternalLogdType::Loglevel::Spam::operator!=(const Spam& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLogdType::Loglevel::Spam::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("forward");
        __c.setString("type", "bool");
        __c.setBool("value", forward);
    }
}

InternalLogdType::Loglevel::Spam::Spam(const vespalib::slime::Inspector & __inspector)
{
    forward = __inspector["forward"]["value"].asBool();
}

InternalLogdType::Loglevel::Spam::Spam(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    forward = ::config::internal::ValueConverter<bool>()(__inspector["forward"], false);
}

InternalLogdType::Loglevel::Loglevel() noexcept
    : fatal(),
      error(),
      warning(),
      config(),
      info(),
      event(),
      debug(),
      spam()
{
}

InternalLogdType::Loglevel::Loglevel(const Loglevel & __rhs) noexcept = default;
InternalLogdType::Loglevel & InternalLogdType::Loglevel::operator = (const Loglevel & __rhs) noexcept = default;
InternalLogdType::Loglevel::Loglevel(Loglevel && __rhs) noexcept = default;
InternalLogdType::Loglevel & InternalLogdType::Loglevel::operator = (Loglevel && __rhs) noexcept = default;
InternalLogdType::Loglevel::~Loglevel() = default; 
InternalLogdType::Loglevel::Loglevel(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    fatal = ConfigParser::parseStruct<Fatal>("fatal", __lines);
    ConfigParser::stripLinesForKey("fatal", __remainingValuesToParse);
    error = ConfigParser::parseStruct<Error>("error", __lines);
    ConfigParser::stripLinesForKey("error", __remainingValuesToParse);
    warning = ConfigParser::parseStruct<Warning>("warning", __lines);
    ConfigParser::stripLinesForKey("warning", __remainingValuesToParse);
    config = ConfigParser::parseStruct<Config>("config", __lines);
    ConfigParser::stripLinesForKey("config", __remainingValuesToParse);
    info = ConfigParser::parseStruct<Info>("info", __lines);
    ConfigParser::stripLinesForKey("info", __remainingValuesToParse);
    event = ConfigParser::parseStruct<Event>("event", __lines);
    ConfigParser::stripLinesForKey("event", __remainingValuesToParse);
    debug = ConfigParser::parseStruct<Debug>("debug", __lines);
    ConfigParser::stripLinesForKey("debug", __remainingValuesToParse);
    spam = ConfigParser::parseStruct<Spam>("spam", __lines);
    ConfigParser::stripLinesForKey("spam", __remainingValuesToParse);
}

bool
InternalLogdType::Loglevel::operator==(const Loglevel& __rhs) const noexcept
{
    return (fatal == __rhs.fatal &&
            error == __rhs.error &&
            warning == __rhs.warning &&
            config == __rhs.config &&
            info == __rhs.info &&
            event == __rhs.event &&
            debug == __rhs.debug &&
            spam == __rhs.spam);
}

bool
InternalLogdType::Loglevel::operator!=(const Loglevel& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLogdType::Loglevel::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fatal");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        fatal.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("error");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        error.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("warning");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        warning.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("config");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        config.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("info");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        info.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("event");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        event.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("debug");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        debug.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("spam");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        spam.serialize(__c2);
    }
}

InternalLogdType::Loglevel::Loglevel(const vespalib::slime::Inspector & __inspector)
{
    fatal = Fatal(__inspector["fatal"]["value"]);
    error = Error(__inspector["error"]["value"]);
    warning = Warning(__inspector["warning"]["value"]);
    config = Config(__inspector["config"]["value"]);
    info = Info(__inspector["info"]["value"]);
    event = Event(__inspector["event"]["value"]);
    debug = Debug(__inspector["debug"]["value"]);
    spam = Spam(__inspector["spam"]["value"]);
}

InternalLogdType::Loglevel::Loglevel(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    fatal = ::config::internal::ValueConverter<Fatal>()(__inspector["fatal"]);
    error = ::config::internal::ValueConverter<Error>()(__inspector["error"]);
    warning = ::config::internal::ValueConverter<Warning>()(__inspector["warning"]);
    config = ::config::internal::ValueConverter<Config>()(__inspector["config"]);
    info = ::config::internal::ValueConverter<Info>()(__inspector["info"]);
    event = ::config::internal::ValueConverter<Event>()(__inspector["event"]);
    debug = ::config::internal::ValueConverter<Debug>()(__inspector["debug"]);
    spam = ::config::internal::ValueConverter<Spam>()(__inspector["spam"]);
}

InternalLogdType::Rotate::Rotate() noexcept
    : size(10000000),
      age(86400)
{
}

InternalLogdType::Rotate::Rotate(const Rotate & __rhs) noexcept = default;
InternalLogdType::Rotate & InternalLogdType::Rotate::operator = (const Rotate & __rhs) noexcept = default;
InternalLogdType::Rotate::Rotate(Rotate && __rhs) noexcept = default;
InternalLogdType::Rotate & InternalLogdType::Rotate::operator = (Rotate && __rhs) noexcept = default;
InternalLogdType::Rotate::~Rotate() = default; 
InternalLogdType::Rotate::Rotate(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    size = ConfigParser::parse<int32_t>("size", __lines, 10000000);
    ConfigParser::stripLinesForKey("size", __remainingValuesToParse);
    age = ConfigParser::parse<int32_t>("age", __lines, 86400);
    ConfigParser::stripLinesForKey("age", __remainingValuesToParse);
}

bool
InternalLogdType::Rotate::operator==(const Rotate& __rhs) const noexcept
{
    return (size == __rhs.size &&
            age == __rhs.age);
}

bool
InternalLogdType::Rotate::operator!=(const Rotate& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLogdType::Rotate::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("size");
        __c.setString("type", "int");
        __c.setLong("value", size);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("age");
        __c.setString("type", "int");
        __c.setLong("value", age);
    }
}

InternalLogdType::Rotate::Rotate(const vespalib::slime::Inspector & __inspector)
{
    size = __inspector["size"]["value"].asLong();
    age = __inspector["age"]["value"].asLong();
}

InternalLogdType::Rotate::Rotate(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    size = ::config::internal::ValueConverter<int32_t>()(__inspector["size"], 10000000);
    age = ::config::internal::ValueConverter<int32_t>()(__inspector["age"], 86400);
}

InternalLogdType::Remove::Remove() noexcept
    : totalmegabytes(1000),
      age(30)
{
}

InternalLogdType::Remove::Remove(const Remove & __rhs) noexcept = default;
InternalLogdType::Remove & InternalLogdType::Remove::operator = (const Remove & __rhs) noexcept = default;
InternalLogdType::Remove::Remove(Remove && __rhs) noexcept = default;
InternalLogdType::Remove & InternalLogdType::Remove::operator = (Remove && __rhs) noexcept = default;
InternalLogdType::Remove::~Remove() = default; 
InternalLogdType::Remove::Remove(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    totalmegabytes = ConfigParser::parse<int32_t>("totalmegabytes", __lines, 1000);
    ConfigParser::stripLinesForKey("totalmegabytes", __remainingValuesToParse);
    age = ConfigParser::parse<int32_t>("age", __lines, 30);
    ConfigParser::stripLinesForKey("age", __remainingValuesToParse);
}

bool
InternalLogdType::Remove::operator==(const Remove& __rhs) const noexcept
{
    return (totalmegabytes == __rhs.totalmegabytes &&
            age == __rhs.age);
}

bool
InternalLogdType::Remove::operator!=(const Remove& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLogdType::Remove::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("totalmegabytes");
        __c.setString("type", "int");
        __c.setLong("value", totalmegabytes);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("age");
        __c.setString("type", "int");
        __c.setLong("value", age);
    }
}

InternalLogdType::Remove::Remove(const vespalib::slime::Inspector & __inspector)
{
    totalmegabytes = __inspector["totalmegabytes"]["value"].asLong();
    age = __inspector["age"]["value"].asLong();
}

InternalLogdType::Remove::Remove(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    totalmegabytes = ::config::internal::ValueConverter<int32_t>()(__inspector["totalmegabytes"], 1000);
    age = ::config::internal::ValueConverter<int32_t>()(__inspector["age"], 30);
}

InternalLogdType::InternalLogdType() noexcept
    : stateport(0),
      logserver(),
      loglevel(),
      rotate(),
      remove()
{
}

InternalLogdType::InternalLogdType(const InternalLogdType & __rhs) noexcept = default;
InternalLogdType & InternalLogdType::operator =(const InternalLogdType & __rhs) noexcept = default;
InternalLogdType::InternalLogdType(InternalLogdType && __rhs) noexcept = default;
InternalLogdType & InternalLogdType::operator =(InternalLogdType && __rhs) noexcept = default;
InternalLogdType::~InternalLogdType() = default; 
InternalLogdType::InternalLogdType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        stateport = ConfigParser::parse<int32_t>("stateport", __lines, 0);
        ConfigParser::stripLinesForKey("stateport", __remainingValuesToParse);
        logserver = ConfigParser::parseStruct<Logserver>("logserver", __lines);
        ConfigParser::stripLinesForKey("logserver", __remainingValuesToParse);
        loglevel = ConfigParser::parseStruct<Loglevel>("loglevel", __lines);
        ConfigParser::stripLinesForKey("loglevel", __remainingValuesToParse);
        rotate = ConfigParser::parseStruct<Rotate>("rotate", __lines);
        ConfigParser::stripLinesForKey("rotate", __remainingValuesToParse);
        remove = ConfigParser::parseStruct<Remove>("remove", __lines);
        ConfigParser::stripLinesForKey("remove", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalLogdType::operator==(const InternalLogdType& __rhs) const noexcept
{
    return (stateport == __rhs.stateport &&
            logserver == __rhs.logserver &&
            loglevel == __rhs.loglevel &&
            rotate == __rhs.rotate &&
            remove == __rhs.remove);
}

bool
InternalLogdType::operator!=(const InternalLogdType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalLogdType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("stateport");
        __c.setString("type", "int");
        __c.setLong("value", stateport);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("logserver");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        logserver.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("loglevel");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        loglevel.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("rotate");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        rotate.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("remove");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        remove.serialize(__c2);
    }
}

InternalLogdType::InternalLogdType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    stateport = __inspector["stateport"]["value"].asLong();
    logserver = Logserver(__inspector["logserver"]["value"]);
    loglevel = Loglevel(__inspector["loglevel"]["value"]);
    rotate = Rotate(__inspector["rotate"]["value"]);
    remove = Remove(__inspector["remove"]["value"]);
}

InternalLogdType::InternalLogdType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    stateport = ::config::internal::ValueConverter<int32_t>()(__inspector["stateport"], 0);
    logserver = ::config::internal::ValueConverter<Logserver>()(__inspector["logserver"]);
    loglevel = ::config::internal::ValueConverter<Loglevel>()(__inspector["loglevel"]);
    rotate = ::config::internal::ValueConverter<Rotate>()(__inspector["rotate"]);
    remove = ::config::internal::ValueConverter<Remove>()(__inspector["remove"]);
}

} // namespace internal

} // namespace cloud::config::log
