/**
 * @class cloud::config::log::Logd
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_CLOUD_CONFIG_LOG_LOGD_H
#define CLOUD_CONFIG_CLOUD_CONFIG_LOG_LOGD_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace cloud::config::log {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalLogdType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalLogdType> UP;
    class Logserver {
    public:
        Logserver(const ::config::StringVector & __lines);
        Logserver(const vespalib::slime::Inspector & __inspector);
        Logserver(const ::config::ConfigPayload & __payload);
        Logserver()  noexcept;
        Logserver(const Logserver & __rhs) noexcept;
        Logserver & operator = (const Logserver & __rhs) noexcept;
        Logserver(Logserver && __rhs) noexcept;
        Logserver & operator = (Logserver && __rhs) noexcept;
        ~Logserver();

        bool operator==(const Logserver& __rhs) const noexcept;
        bool operator!=(const Logserver& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** Host to contact the logserver on. */
        vespalib::string host; // Default: "localhost"

        /** RPC port of logserver. */
        int32_t rpcport; // Default: 5822

        /**
         * Forward to a logserver. Other logserver configuration is irrelevant
         * if false.
         */
        bool use; // Default: true
    };
    typedef std::vector<Logserver> LogserverVector;

    typedef std::map<vespalib::string, Logserver> LogserverMap;

    class Loglevel {
    public:
        class Fatal {
        public:
            Fatal(const ::config::StringVector & __lines);
            Fatal(const vespalib::slime::Inspector & __inspector);
            Fatal(const ::config::ConfigPayload & __payload);
            Fatal()  noexcept;
            Fatal(const Fatal & __rhs) noexcept;
            Fatal & operator = (const Fatal & __rhs) noexcept;
            Fatal(Fatal && __rhs) noexcept;
            Fatal & operator = (Fatal && __rhs) noexcept;
            ~Fatal();

            bool operator==(const Fatal& __rhs) const noexcept;
            bool operator!=(const Fatal& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * Loglevel config whether they should be stored and/or forwarded
             */
            bool forward; // Default: true
        };
        typedef std::vector<Fatal> FatalVector;

        typedef std::map<vespalib::string, Fatal> FatalMap;

        class Error {
        public:
            Error(const ::config::StringVector & __lines);
            Error(const vespalib::slime::Inspector & __inspector);
            Error(const ::config::ConfigPayload & __payload);
            Error()  noexcept;
            Error(const Error & __rhs) noexcept;
            Error & operator = (const Error & __rhs) noexcept;
            Error(Error && __rhs) noexcept;
            Error & operator = (Error && __rhs) noexcept;
            ~Error();

            bool operator==(const Error& __rhs) const noexcept;
            bool operator!=(const Error& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            bool forward; // Default: true
        };
        typedef std::vector<Error> ErrorVector;

        typedef std::map<vespalib::string, Error> ErrorMap;

        class Warning {
        public:
            Warning(const ::config::StringVector & __lines);
            Warning(const vespalib::slime::Inspector & __inspector);
            Warning(const ::config::ConfigPayload & __payload);
            Warning()  noexcept;
            Warning(const Warning & __rhs) noexcept;
            Warning & operator = (const Warning & __rhs) noexcept;
            Warning(Warning && __rhs) noexcept;
            Warning & operator = (Warning && __rhs) noexcept;
            ~Warning();

            bool operator==(const Warning& __rhs) const noexcept;
            bool operator!=(const Warning& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            bool forward; // Default: true
        };
        typedef std::vector<Warning> WarningVector;

        typedef std::map<vespalib::string, Warning> WarningMap;

        class Config {
        public:
            Config(const ::config::StringVector & __lines);
            Config(const vespalib::slime::Inspector & __inspector);
            Config(const ::config::ConfigPayload & __payload);
            Config()  noexcept;
            Config(const Config & __rhs) noexcept;
            Config & operator = (const Config & __rhs) noexcept;
            Config(Config && __rhs) noexcept;
            Config & operator = (Config && __rhs) noexcept;
            ~Config();

            bool operator==(const Config& __rhs) const noexcept;
            bool operator!=(const Config& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            bool forward; // Default: true
        };
        typedef std::vector<Config> ConfigVector;

        typedef std::map<vespalib::string, Config> ConfigMap;

        class Info {
        public:
            Info(const ::config::StringVector & __lines);
            Info(const vespalib::slime::Inspector & __inspector);
            Info(const ::config::ConfigPayload & __payload);
            Info()  noexcept;
            Info(const Info & __rhs) noexcept;
            Info & operator = (const Info & __rhs) noexcept;
            Info(Info && __rhs) noexcept;
            Info & operator = (Info && __rhs) noexcept;
            ~Info();

            bool operator==(const Info& __rhs) const noexcept;
            bool operator!=(const Info& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            bool forward; // Default: true
        };
        typedef std::vector<Info> InfoVector;

        typedef std::map<vespalib::string, Info> InfoMap;

        class Event {
        public:
            Event(const ::config::StringVector & __lines);
            Event(const vespalib::slime::Inspector & __inspector);
            Event(const ::config::ConfigPayload & __payload);
            Event()  noexcept;
            Event(const Event & __rhs) noexcept;
            Event & operator = (const Event & __rhs) noexcept;
            Event(Event && __rhs) noexcept;
            Event & operator = (Event && __rhs) noexcept;
            ~Event();

            bool operator==(const Event& __rhs) const noexcept;
            bool operator!=(const Event& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            bool forward; // Default: true
        };
        typedef std::vector<Event> EventVector;

        typedef std::map<vespalib::string, Event> EventMap;

        class Debug {
        public:
            Debug(const ::config::StringVector & __lines);
            Debug(const vespalib::slime::Inspector & __inspector);
            Debug(const ::config::ConfigPayload & __payload);
            Debug()  noexcept;
            Debug(const Debug & __rhs) noexcept;
            Debug & operator = (const Debug & __rhs) noexcept;
            Debug(Debug && __rhs) noexcept;
            Debug & operator = (Debug && __rhs) noexcept;
            ~Debug();

            bool operator==(const Debug& __rhs) const noexcept;
            bool operator!=(const Debug& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            bool forward; // Default: false
        };
        typedef std::vector<Debug> DebugVector;

        typedef std::map<vespalib::string, Debug> DebugMap;

        class Spam {
        public:
            Spam(const ::config::StringVector & __lines);
            Spam(const vespalib::slime::Inspector & __inspector);
            Spam(const ::config::ConfigPayload & __payload);
            Spam()  noexcept;
            Spam(const Spam & __rhs) noexcept;
            Spam & operator = (const Spam & __rhs) noexcept;
            Spam(Spam && __rhs) noexcept;
            Spam & operator = (Spam && __rhs) noexcept;
            ~Spam();

            bool operator==(const Spam& __rhs) const noexcept;
            bool operator!=(const Spam& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            bool forward; // Default: false
        };
        typedef std::vector<Spam> SpamVector;

        typedef std::map<vespalib::string, Spam> SpamMap;

        Loglevel(const ::config::StringVector & __lines);
        Loglevel(const vespalib::slime::Inspector & __inspector);
        Loglevel(const ::config::ConfigPayload & __payload);
        Loglevel()  noexcept;
        Loglevel(const Loglevel & __rhs) noexcept;
        Loglevel & operator = (const Loglevel & __rhs) noexcept;
        Loglevel(Loglevel && __rhs) noexcept;
        Loglevel & operator = (Loglevel && __rhs) noexcept;
        ~Loglevel();

        bool operator==(const Loglevel& __rhs) const noexcept;
        bool operator!=(const Loglevel& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        Fatal fatal;
        Error error;
        Warning warning;
        Config config;
        Info info;
        Event event;
        Debug debug;
        Spam spam;
    };
    typedef std::vector<Loglevel> LoglevelVector;

    typedef std::map<vespalib::string, Loglevel> LoglevelMap;

    class Rotate {
    public:
        Rotate(const ::config::StringVector & __lines);
        Rotate(const vespalib::slime::Inspector & __inspector);
        Rotate(const ::config::ConfigPayload & __payload);
        Rotate()  noexcept;
        Rotate(const Rotate & __rhs) noexcept;
        Rotate & operator = (const Rotate & __rhs) noexcept;
        Rotate(Rotate && __rhs) noexcept;
        Rotate & operator = (Rotate && __rhs) noexcept;
        ~Rotate();

        bool operator==(const Rotate& __rhs) const noexcept;
        bool operator!=(const Rotate& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** when the logfile reaches this size, rotate it */
        int32_t size; // Default: 10000000

        /** when the logfile reaches this age (in seconds), rotate it */
        int32_t age; // Default: 86400
    };
    typedef std::vector<Rotate> RotateVector;

    typedef std::map<vespalib::string, Rotate> RotateMap;

    class Remove {
    public:
        Remove(const ::config::StringVector & __lines);
        Remove(const vespalib::slime::Inspector & __inspector);
        Remove(const ::config::ConfigPayload & __payload);
        Remove()  noexcept;
        Remove(const Remove & __rhs) noexcept;
        Remove & operator = (const Remove & __rhs) noexcept;
        Remove(Remove && __rhs) noexcept;
        Remove & operator = (Remove && __rhs) noexcept;
        ~Remove();

        bool operator==(const Remove& __rhs) const noexcept;
        bool operator!=(const Remove& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * remove old logfiles so total size does not exceed this (in megabytes)
         */
        int32_t totalmegabytes; // Default: 1000

        /** remove old logfiles older than this (in days) */
        int32_t age; // Default: 30
    };
    typedef std::vector<Remove> RemoveVector;

    typedef std::map<vespalib::string, Remove> RemoveMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalLogdType(const ::config::ConfigValue & __value);
    InternalLogdType(const ::config::ConfigDataBuffer & __value);
    InternalLogdType(const ::config::ConfigPayload & __payload);
    InternalLogdType()  noexcept;
    InternalLogdType(const InternalLogdType & __rhs) noexcept;
    InternalLogdType & operator = (const InternalLogdType & __rhs) noexcept;
    InternalLogdType(InternalLogdType && __rhs) noexcept;
    InternalLogdType & operator = (InternalLogdType && __rhs) noexcept;
    ~InternalLogdType();

    bool operator==(const InternalLogdType& __rhs) const noexcept;
    bool operator!=(const InternalLogdType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /** Port to serve status and metrics on */
    int32_t stateport; // Default: 0
    Logserver logserver;
    Loglevel loglevel;
    Rotate rotate;
    Remove remove;
};

} // namespace internal

typedef internal::InternalLogdType LogdConfigBuilder;
typedef const internal::InternalLogdType LogdConfig;

} // namespace cloud::config::log
#endif // VESPA_config_CLOUD_CONFIG_LOG_LOGD_H
