#include "config-messagebus.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace messagebus {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalMessagebusType::CONFIG_DEF_MD5("04a33fc5c97314d4c98f3d743fbd25c4");
const vespalib::string InternalMessagebusType::CONFIG_DEF_NAME("messagebus");
const vespalib::string InternalMessagebusType::CONFIG_DEF_NAMESPACE("messagebus");
const int64_t InternalMessagebusType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=messagebus",
"routingtable[].protocol string",
"routingtable[].hop[].name string",
"routingtable[].hop[].selector string",
"routingtable[].hop[].recipient[] string",
"routingtable[].hop[].ignoreresult bool default=false",
"routingtable[].route[].name string",
"routingtable[].route[].hop[] string",
};
const ::config::StringVector InternalMessagebusType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalMessagebusType::Routingtable::Hop::Hop()
    : name(),
      selector(),
      recipient(),
      ignoreresult(false)
{
}

InternalMessagebusType::Routingtable::Hop::Hop(const Hop & __rhs) = default;
InternalMessagebusType::Routingtable::Hop & InternalMessagebusType::Routingtable::Hop::operator = (const Hop & __rhs) = default;
InternalMessagebusType::Routingtable::Hop::Hop(Hop && __rhs) noexcept = default;
InternalMessagebusType::Routingtable::Hop & InternalMessagebusType::Routingtable::Hop::operator = (Hop && __rhs) noexcept = default;
InternalMessagebusType::Routingtable::Hop::~Hop() = default; 
InternalMessagebusType::Routingtable::Hop::Hop(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    selector = ConfigParser::parse<vespalib::string>("selector", __lines);
    ConfigParser::stripLinesForKey("selector", __remainingValuesToParse);
    recipient = ConfigParser::parseArray<::config::StringVector>("recipient", __lines);
    ConfigParser::stripLinesForKey("recipient", __remainingValuesToParse);
    ignoreresult = ConfigParser::parse<bool>("ignoreresult", __lines, false);
    ConfigParser::stripLinesForKey("ignoreresult", __remainingValuesToParse);
}

bool
InternalMessagebusType::Routingtable::Hop::operator==(const Hop& __rhs) const noexcept
{
    return (name == __rhs.name &&
            selector == __rhs.selector &&
            recipient == __rhs.recipient &&
            ignoreresult == __rhs.ignoreresult);
}

bool
InternalMessagebusType::Routingtable::Hop::operator!=(const Hop& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalMessagebusType::Routingtable::Hop::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("selector");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(selector));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("recipient");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : recipient) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ignoreresult");
        __c.setString("type", "bool");
        __c.setBool("value", ignoreresult);
    }
}

InternalMessagebusType::Routingtable::Hop::Hop(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    selector = __inspector["selector"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["recipient"]["value"].children(); __i++) {
        recipient.push_back(__inspector["recipient"]["value"][__i]["value"].asString().make_string());
    }
    ignoreresult = __inspector["ignoreresult"]["value"].asBool();
}

InternalMessagebusType::Routingtable::Hop::Hop(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    selector = ::config::internal::ValueConverter<vespalib::string>()("selector", __inspector["selector"]);
    ::config::internal::VectorInserter<::config::StringVector> __recipientInserter(recipient);
    __inspector["recipient"].traverse(__recipientInserter);
    ignoreresult = ::config::internal::ValueConverter<bool>()(__inspector["ignoreresult"], false);
}

InternalMessagebusType::Routingtable::Route::Route()
    : name(),
      hop()
{
}

InternalMessagebusType::Routingtable::Route::Route(const Route & __rhs) = default;
InternalMessagebusType::Routingtable::Route & InternalMessagebusType::Routingtable::Route::operator = (const Route & __rhs) = default;
InternalMessagebusType::Routingtable::Route::Route(Route && __rhs) noexcept = default;
InternalMessagebusType::Routingtable::Route & InternalMessagebusType::Routingtable::Route::operator = (Route && __rhs) noexcept = default;
InternalMessagebusType::Routingtable::Route::~Route() = default; 
InternalMessagebusType::Routingtable::Route::Route(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    hop = ConfigParser::parseArray<::config::StringVector>("hop", __lines);
    ConfigParser::stripLinesForKey("hop", __remainingValuesToParse);
}

bool
InternalMessagebusType::Routingtable::Route::
operator==(const Route& __rhs) const noexcept
{
    return (name == __rhs.name &&
            hop == __rhs.hop);
}

bool
InternalMessagebusType::Routingtable::Route::
operator!=(const Route& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalMessagebusType::Routingtable::Route::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("hop");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : hop) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
}

InternalMessagebusType::Routingtable::Route::Route(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["hop"]["value"].children(); __i++) {
        hop.push_back(__inspector["hop"]["value"][__i]["value"].asString().make_string());
    }
}

InternalMessagebusType::Routingtable::Route::Route(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    ::config::internal::VectorInserter<::config::StringVector> __hopInserter(hop);
    __inspector["hop"].traverse(__hopInserter);
}

InternalMessagebusType::Routingtable::Routingtable()
    : protocol(),
      hop(),
      route()
{
}

InternalMessagebusType::Routingtable::Routingtable(const Routingtable & __rhs) = default;
InternalMessagebusType::Routingtable & InternalMessagebusType::Routingtable::operator = (const Routingtable & __rhs) = default;
InternalMessagebusType::Routingtable::Routingtable(Routingtable && __rhs) noexcept = default;
InternalMessagebusType::Routingtable & InternalMessagebusType::Routingtable::operator = (Routingtable && __rhs) noexcept = default;
InternalMessagebusType::Routingtable::~Routingtable() = default; 
InternalMessagebusType::Routingtable::Routingtable(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    protocol = ConfigParser::parse<vespalib::string>("protocol", __lines);
    ConfigParser::stripLinesForKey("protocol", __remainingValuesToParse);
    hop = ConfigParser::parseArray<HopVector>("hop", __lines);
    ConfigParser::stripLinesForKey("hop", __remainingValuesToParse);
    route = ConfigParser::parseArray<RouteVector>("route", __lines);
    ConfigParser::stripLinesForKey("route", __remainingValuesToParse);
}

bool
InternalMessagebusType::Routingtable::
operator==(const Routingtable& __rhs) const noexcept
{
    return (protocol == __rhs.protocol &&
            hop == __rhs.hop &&
            route == __rhs.route);
}

bool
InternalMessagebusType::Routingtable::
operator!=(const Routingtable& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalMessagebusType::Routingtable::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("protocol");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(protocol));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("hop");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : hop) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("route");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : route) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalMessagebusType::Routingtable::Routingtable(const vespalib::slime::Inspector & __inspector)
{
    protocol = __inspector["protocol"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["hop"]["value"].children(); __i++) {
        hop.push_back(Hop(__inspector["hop"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["route"]["value"].children(); __i++) {
        route.push_back(Route(__inspector["route"]["value"][__i]["value"]));
    }
}

InternalMessagebusType::Routingtable::Routingtable(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    protocol = ::config::internal::ValueConverter<vespalib::string>()("protocol", __inspector["protocol"]);
    ::config::internal::VectorInserter<HopVector> __hopInserter(hop);
    __inspector["hop"].traverse(__hopInserter);
    ::config::internal::VectorInserter<RouteVector> __routeInserter(route);
    __inspector["route"].traverse(__routeInserter);
}

InternalMessagebusType::InternalMessagebusType()
    : routingtable()
{
}

InternalMessagebusType::InternalMessagebusType(const InternalMessagebusType & __rhs) = default;
InternalMessagebusType & InternalMessagebusType::operator =(const InternalMessagebusType & __rhs) = default;
InternalMessagebusType::InternalMessagebusType(InternalMessagebusType && __rhs) noexcept = default;
InternalMessagebusType & InternalMessagebusType::operator =(InternalMessagebusType && __rhs) noexcept = default;
InternalMessagebusType::~InternalMessagebusType() = default; 
InternalMessagebusType::InternalMessagebusType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        routingtable = ConfigParser::parseArray<RoutingtableVector>("routingtable", __lines);
        ConfigParser::stripLinesForKey("routingtable", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalMessagebusType::operator==(const InternalMessagebusType& __rhs) const noexcept
{
    return (routingtable == __rhs.routingtable);
}

bool
InternalMessagebusType::operator!=(const InternalMessagebusType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalMessagebusType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("routingtable");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : routingtable) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalMessagebusType::InternalMessagebusType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["routingtable"]["value"].children(); __i++) {
        routingtable.push_back(Routingtable(__inspector["routingtable"]["value"][__i]["value"]));
    }
}

InternalMessagebusType::InternalMessagebusType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<RoutingtableVector> __routingtableInserter(routingtable);
    __inspector["routingtable"].traverse(__routingtableInserter);
}

} // namespace internal

} // namespace messagebus
