/**
 * @class messagebus::Messagebus
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_MESSAGEBUS_MESSAGEBUS_H
#define CLOUD_CONFIG_MESSAGEBUS_MESSAGEBUS_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace messagebus {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalMessagebusType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalMessagebusType> UP;
    class Routingtable {
    public:
        class Hop {
        public:
            Hop(const ::config::StringVector & __lines);
            Hop(const vespalib::slime::Inspector & __inspector);
            Hop(const ::config::ConfigPayload & __payload);
            Hop() ;
            Hop(const Hop & __rhs);
            Hop & operator = (const Hop & __rhs);
            Hop(Hop && __rhs) noexcept;
            Hop & operator = (Hop && __rhs) noexcept;
            ~Hop();

            bool operator==(const Hop& __rhs) const noexcept;
            bool operator!=(const Hop& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** A protocol-unique name for a hop. */
            vespalib::string name;

            /**
             * The selector string of a hop, this string typically contains
             * routing policy references on the form [policy-name:parameter].
             * The protocol for the routing table must support all named
             * policies.
             */
            vespalib::string selector;

            /**
             * List of recipients for a hop. These strings may contain
             * wildcards to allow the network layer to choose any single
             * matching service.
             */
            ::config::StringVector recipient;

            /** Whether or not to ignore the result from this hop. */
            bool ignoreresult; // Default: false
        };
        typedef std::vector<Hop> HopVector;

        typedef std::map<vespalib::string, Hop> HopMap;

        class Route {
        public:
            Route(const ::config::StringVector & __lines);
            Route(const vespalib::slime::Inspector & __inspector);
            Route(const ::config::ConfigPayload & __payload);
            Route() ;
            Route(const Route & __rhs);
            Route & operator = (const Route & __rhs);
            Route(Route && __rhs) noexcept;
            Route & operator = (Route && __rhs) noexcept;
            ~Route();

            bool operator==(const Route& __rhs) const noexcept;
            bool operator!=(const Route& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** A protocol-unique name for a route. */
            vespalib::string name;

            /** An array of hop names that together make up the route. */
            ::config::StringVector hop;
        };
        typedef std::vector<Route> RouteVector;

        typedef std::map<vespalib::string, Route> RouteMap;

        Routingtable(const ::config::StringVector & __lines);
        Routingtable(const vespalib::slime::Inspector & __inspector);
        Routingtable(const ::config::ConfigPayload & __payload);
        Routingtable() ;
        Routingtable(const Routingtable & __rhs);
        Routingtable & operator = (const Routingtable & __rhs);
        Routingtable(Routingtable && __rhs) noexcept;
        Routingtable & operator = (Routingtable && __rhs) noexcept;
        ~Routingtable();

        bool operator==(const Routingtable& __rhs) const noexcept;
        bool operator!=(const Routingtable& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * Name of the protocol that uses this routing table. All
         * instances of message bus must support all named protocols.
         */
        vespalib::string protocol;
        HopVector hop;
        RouteVector route;
    };
    typedef std::vector<Routingtable> RoutingtableVector;

    typedef std::map<vespalib::string, Routingtable> RoutingtableMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalMessagebusType(const ::config::ConfigValue & __value);
    InternalMessagebusType(const ::config::ConfigDataBuffer & __value);
    InternalMessagebusType(const ::config::ConfigPayload & __payload);
    InternalMessagebusType() ;
    InternalMessagebusType(const InternalMessagebusType & __rhs);
    InternalMessagebusType & operator = (const InternalMessagebusType & __rhs);
    InternalMessagebusType(InternalMessagebusType && __rhs) noexcept;
    InternalMessagebusType & operator = (InternalMessagebusType && __rhs) noexcept;
    ~InternalMessagebusType();

    bool operator==(const InternalMessagebusType& __rhs) const noexcept;
    bool operator!=(const InternalMessagebusType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    RoutingtableVector routingtable;
};

} // namespace internal

typedef internal::InternalMessagebusType MessagebusConfigBuilder;
typedef const internal::InternalMessagebusType MessagebusConfig;

} // namespace messagebus
#endif // VESPA_config_MESSAGEBUS_MESSAGEBUS_H
