#include "config-metricsmanager.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace metrics {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalMetricsmanagerType::CONFIG_DEF_MD5("ea7f889d45f793408ebcd4d946050902");
const vespalib::string InternalMetricsmanagerType::CONFIG_DEF_NAME("metricsmanager");
const vespalib::string InternalMetricsmanagerType::CONFIG_DEF_NAMESPACE("metrics");
const int64_t InternalMetricsmanagerType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=metrics",
"snapshot.periods[] int restart",
"consumer[].name string restart",
"consumer[].tags[] string restart",
"consumer[].removedtags[] string restart",
"consumer[].addedmetrics[] string restart",
"consumer[].removedmetrics[] string restart",
};
const ::config::StringVector InternalMetricsmanagerType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalMetricsmanagerType::Snapshot::Snapshot()
    : periods()
{
}

InternalMetricsmanagerType::Snapshot::Snapshot(const Snapshot & __rhs) = default;
InternalMetricsmanagerType::Snapshot & InternalMetricsmanagerType::Snapshot::operator = (const Snapshot & __rhs) = default;
InternalMetricsmanagerType::Snapshot::Snapshot(Snapshot && __rhs) noexcept = default;
InternalMetricsmanagerType::Snapshot & InternalMetricsmanagerType::Snapshot::operator = (Snapshot && __rhs) noexcept = default;
InternalMetricsmanagerType::Snapshot::~Snapshot() = default; 
InternalMetricsmanagerType::Snapshot::Snapshot(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    periods = ConfigParser::parseArray<::config::IntVector>("periods", __lines);
    ConfigParser::stripLinesForKey("periods", __remainingValuesToParse);
}

bool
InternalMetricsmanagerType::Snapshot::operator==(const Snapshot& __rhs) const noexcept
{
    return (periods == __rhs.periods);
}

bool
InternalMetricsmanagerType::Snapshot::operator!=(const Snapshot& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalMetricsmanagerType::Snapshot::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("periods");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : periods) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "int");
            __c3.setLong("value", child);
        }
    }
}

InternalMetricsmanagerType::Snapshot::Snapshot(const vespalib::slime::Inspector & __inspector)
{
    for (size_t __i = 0; __i < __inspector["periods"]["value"].children(); __i++) {
        periods.push_back(__inspector["periods"]["value"][__i]["value"].asLong());
    }
}

InternalMetricsmanagerType::Snapshot::Snapshot(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<::config::IntVector> __periodsInserter(periods);
    __inspector["periods"].traverse(__periodsInserter);
}

InternalMetricsmanagerType::Consumer::Consumer()
    : name(),
      tags(),
      removedtags(),
      addedmetrics(),
      removedmetrics()
{
}

InternalMetricsmanagerType::Consumer::Consumer(const Consumer & __rhs) = default;
InternalMetricsmanagerType::Consumer & InternalMetricsmanagerType::Consumer::operator = (const Consumer & __rhs) = default;
InternalMetricsmanagerType::Consumer::Consumer(Consumer && __rhs) noexcept = default;
InternalMetricsmanagerType::Consumer & InternalMetricsmanagerType::Consumer::operator = (Consumer && __rhs) noexcept = default;
InternalMetricsmanagerType::Consumer::~Consumer() = default; 
InternalMetricsmanagerType::Consumer::Consumer(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    tags = ConfigParser::parseArray<::config::StringVector>("tags", __lines);
    ConfigParser::stripLinesForKey("tags", __remainingValuesToParse);
    removedtags = ConfigParser::parseArray<::config::StringVector>("removedtags", __lines);
    ConfigParser::stripLinesForKey("removedtags", __remainingValuesToParse);
    addedmetrics = ConfigParser::parseArray<::config::StringVector>("addedmetrics", __lines);
    ConfigParser::stripLinesForKey("addedmetrics", __remainingValuesToParse);
    removedmetrics = ConfigParser::parseArray<::config::StringVector>("removedmetrics", __lines);
    ConfigParser::stripLinesForKey("removedmetrics", __remainingValuesToParse);
}

bool
InternalMetricsmanagerType::Consumer::operator==(const Consumer& __rhs) const noexcept
{
    return (name == __rhs.name &&
            tags == __rhs.tags &&
            removedtags == __rhs.removedtags &&
            addedmetrics == __rhs.addedmetrics &&
            removedmetrics == __rhs.removedmetrics);
}

bool
InternalMetricsmanagerType::Consumer::operator!=(const Consumer& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalMetricsmanagerType::Consumer::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tags");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : tags) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("removedtags");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : removedtags) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("addedmetrics");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : addedmetrics) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("removedmetrics");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : removedmetrics) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "string");
            __c3.setString("value", vespalib::Memory(child));
        }
    }
}

InternalMetricsmanagerType::Consumer::Consumer(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["tags"]["value"].children(); __i++) {
        tags.push_back(__inspector["tags"]["value"][__i]["value"].asString().make_string());
    }
    for (size_t __i = 0; __i < __inspector["removedtags"]["value"].children(); __i++) {
        removedtags.push_back(__inspector["removedtags"]["value"][__i]["value"].asString().make_string());
    }
    for (size_t __i = 0; __i < __inspector["addedmetrics"]["value"].children(); __i++) {
        addedmetrics.push_back(__inspector["addedmetrics"]["value"][__i]["value"].asString().make_string());
    }
    for (size_t __i = 0; __i < __inspector["removedmetrics"]["value"].children(); __i++) {
        removedmetrics.push_back(__inspector["removedmetrics"]["value"][__i]["value"].asString().make_string());
    }
}

InternalMetricsmanagerType::Consumer::Consumer(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    ::config::internal::VectorInserter<::config::StringVector> __tagsInserter(tags);
    __inspector["tags"].traverse(__tagsInserter);
    ::config::internal::VectorInserter<::config::StringVector> __removedtagsInserter(removedtags);
    __inspector["removedtags"].traverse(__removedtagsInserter);
    ::config::internal::VectorInserter<::config::StringVector> __addedmetricsInserter(addedmetrics);
    __inspector["addedmetrics"].traverse(__addedmetricsInserter);
    ::config::internal::VectorInserter<::config::StringVector> __removedmetricsInserter(removedmetrics);
    __inspector["removedmetrics"].traverse(__removedmetricsInserter);
}

InternalMetricsmanagerType::InternalMetricsmanagerType()
    : snapshot(),
      consumer()
{
}

InternalMetricsmanagerType::InternalMetricsmanagerType(const InternalMetricsmanagerType & __rhs) = default;
InternalMetricsmanagerType & InternalMetricsmanagerType::operator =(const InternalMetricsmanagerType & __rhs) = default;
InternalMetricsmanagerType::InternalMetricsmanagerType(InternalMetricsmanagerType && __rhs) noexcept = default;
InternalMetricsmanagerType & InternalMetricsmanagerType::operator =(InternalMetricsmanagerType && __rhs) noexcept = default;
InternalMetricsmanagerType::~InternalMetricsmanagerType() = default; 
InternalMetricsmanagerType::InternalMetricsmanagerType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        snapshot = ConfigParser::parseStruct<Snapshot>("snapshot", __lines);
        ConfigParser::stripLinesForKey("snapshot", __remainingValuesToParse);
        consumer = ConfigParser::parseArray<ConsumerVector>("consumer", __lines);
        ConfigParser::stripLinesForKey("consumer", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalMetricsmanagerType::
operator==(const InternalMetricsmanagerType& __rhs) const noexcept
{
    return (snapshot == __rhs.snapshot &&
            consumer == __rhs.consumer);
}

bool
InternalMetricsmanagerType::
operator!=(const InternalMetricsmanagerType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalMetricsmanagerType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("snapshot");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        snapshot.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("consumer");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : consumer) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalMetricsmanagerType::InternalMetricsmanagerType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    snapshot = Snapshot(__inspector["snapshot"]["value"]);
    for (size_t __i = 0; __i < __inspector["consumer"]["value"].children(); __i++) {
        consumer.push_back(Consumer(__inspector["consumer"]["value"][__i]["value"]));
    }
}

InternalMetricsmanagerType::InternalMetricsmanagerType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    snapshot = ::config::internal::ValueConverter<Snapshot>()(__inspector["snapshot"]);
    ::config::internal::VectorInserter<ConsumerVector> __consumerInserter(consumer);
    __inspector["consumer"].traverse(__consumerInserter);
}

} // namespace internal

} // namespace metrics
