/**
 * @class metrics::Metricsmanager
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_METRICS_METRICSMANAGER_H
#define CLOUD_CONFIG_METRICS_METRICSMANAGER_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace metrics {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalMetricsmanagerType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalMetricsmanagerType> UP;
    class Snapshot {
    public:
        Snapshot(const ::config::StringVector & __lines);
        Snapshot(const vespalib::slime::Inspector & __inspector);
        Snapshot(const ::config::ConfigPayload & __payload);
        Snapshot() ;
        Snapshot(const Snapshot & __rhs);
        Snapshot & operator = (const Snapshot & __rhs);
        Snapshot(Snapshot && __rhs) noexcept;
        Snapshot & operator = (Snapshot && __rhs) noexcept;
        ~Snapshot();

        bool operator==(const Snapshot& __rhs) const noexcept;
        bool operator!=(const Snapshot& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * If any snapshot periods is set, these override all the default ones.
         * The array sets the snapshot periods in number of seconds. Note that
         * when
         * sorted in rising time length order, it is required that the snapshot
         * length of
         * the next snapshot is a multiplum of the previous snapshot.
         * restart flag was added automatically and needs to be verified.
         */
        ::config::IntVector periods;
    };
    typedef std::vector<Snapshot> SnapshotVector;

    typedef std::map<vespalib::string, Snapshot> SnapshotMap;

    class Consumer {
    public:
        Consumer(const ::config::StringVector & __lines);
        Consumer(const vespalib::slime::Inspector & __inspector);
        Consumer(const ::config::ConfigPayload & __payload);
        Consumer() ;
        Consumer(const Consumer & __rhs);
        Consumer & operator = (const Consumer & __rhs);
        Consumer(Consumer && __rhs) noexcept;
        Consumer & operator = (Consumer && __rhs) noexcept;
        ~Consumer();

        bool operator==(const Consumer& __rhs) const noexcept;
        bool operator!=(const Consumer& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * The name of the consumer that should include the given metrics
         * specified.
         * restart flag was added automatically and needs to be verified.
         */
        vespalib::string name;

        /**
         * Include metrics that match the given tags.
         *  
         * A tag specification either adds a single tag by given the full tag
         * name, or
         * any tag, by specifying an asterix as wildcard ('*'). An asterix will
         * not match
         * metrics without any tags set, but you can match these by specifying
         * an empty
         * string.
         *  
         * A metric set matched by tags, will also include every metric
         * contained in it
         * regardless of tags.
         * restart flag was added automatically and needs to be verified.
         */
        ::config::StringVector tags;

        /**
         * Do not include metrics that match the specific tags. This has higher
         * presedence than the adding of tags, but lower presedence than name
         * patterns.
         *  
         * Removed tags are specified as added tags, with a small exception.
         * Removing
         * wildcard or untagged metrics does not make sense, as metrics are not
         * added by
         * default. If you don not want to include these metrics, just not
         * specify them
         * as tags to be added. Thus, these are not allowed in remove tags spec.
         *  
         * A metric set matched by remove tag will remove all metrics below that
         * tag.
         * restart flag was added automatically and needs to be verified.
         */
        ::config::StringVector removedtags;

        /**
         * Include all metrics that matches the names given here.
         * Metrics added here have presedence above tag specifications.
         *  
         * Names are specified as a complete dot separated path to metric.
         * Asterix can
         * be used to match any part. An asterix will only match a complete name
         * without
         * a dot. So if you have an average metric called 'queuesize' in your
         * top level
         * metric set called 'myapp', you can add this metric with the spec
         * 'myapp.queuesize'. You can also add non-default average metric
         * entries by
         * specifying which, such as 'myapp.queuesize.last' or 'myapp.queuesize.-
         * max'.
         * Patterns can be used, to for instance specify '*.queuesize',
         * 'myapp.*', '*.*',
         * 'myapp.*.max' or similar.
         *  
         * A metric set added with specific name will add all metrics within it.
         * restart flag was added automatically and needs to be verified.
         */
        ::config::StringVector addedmetrics;

        /**
         * Do not include metrics with the below names.
         * This has highest presedence. Metrics removed like this will override
         * any
         * specification otherwise that would include them.
         *  
         * The remove metrics are specified exactly the same as the added
         * metrics.
         *  
         * A metric set removed with specific name will remove all metrics
         * within it.
         * restart flag was added automatically and needs to be verified.
         */
        ::config::StringVector removedmetrics;
    };
    typedef std::vector<Consumer> ConsumerVector;

    typedef std::map<vespalib::string, Consumer> ConsumerMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalMetricsmanagerType(const ::config::ConfigValue & __value);
    InternalMetricsmanagerType(const ::config::ConfigDataBuffer & __value);
    InternalMetricsmanagerType(const ::config::ConfigPayload & __payload);
    InternalMetricsmanagerType() ;
    InternalMetricsmanagerType(const InternalMetricsmanagerType & __rhs);
    InternalMetricsmanagerType & operator = (const InternalMetricsmanagerType & __rhs);
    InternalMetricsmanagerType(InternalMetricsmanagerType && __rhs) noexcept;
    InternalMetricsmanagerType & operator = (InternalMetricsmanagerType && __rhs) noexcept;
    ~InternalMetricsmanagerType();

    bool operator==(const InternalMetricsmanagerType& __rhs) const noexcept;
    bool operator!=(const InternalMetricsmanagerType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    Snapshot snapshot;
    ConsumerVector consumer;
};

} // namespace internal

typedef internal::InternalMetricsmanagerType MetricsmanagerConfigBuilder;
typedef const internal::InternalMetricsmanagerType MetricsmanagerConfig;

} // namespace metrics
#endif // VESPA_config_METRICS_METRICSMANAGER_H
