#include "config-verify-ranksetup.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalVerifyRanksetupType::CONFIG_DEF_MD5("6c239ecc2ac2b06b54f411d5cd4d59a8");
const vespalib::string InternalVerifyRanksetupType::CONFIG_DEF_NAME("verify-ranksetup");
const vespalib::string InternalVerifyRanksetupType::CONFIG_DEF_NAMESPACE("vespa.config.search.core");
const int64_t InternalVerifyRanksetupType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search.core",
"file[].ref string",
"file[].path string",
};
const ::config::StringVector InternalVerifyRanksetupType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalVerifyRanksetupType::File::File() noexcept
    : ref(),
      path()
{
}

InternalVerifyRanksetupType::File::File(const File & __rhs) noexcept = default;
InternalVerifyRanksetupType::File & InternalVerifyRanksetupType::File::operator = (const File & __rhs) noexcept = default;
InternalVerifyRanksetupType::File::File(File && __rhs) noexcept = default;
InternalVerifyRanksetupType::File & InternalVerifyRanksetupType::File::operator = (File && __rhs) noexcept = default;
InternalVerifyRanksetupType::File::~File() = default; 
InternalVerifyRanksetupType::File::File(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    ref = ConfigParser::parse<vespalib::string>("ref", __lines);
    ConfigParser::stripLinesForKey("ref", __remainingValuesToParse);
    path = ConfigParser::parse<vespalib::string>("path", __lines);
    ConfigParser::stripLinesForKey("path", __remainingValuesToParse);
}

bool
InternalVerifyRanksetupType::File::operator==(const File& __rhs) const noexcept
{
    return (ref == __rhs.ref &&
            path == __rhs.path);
}

bool
InternalVerifyRanksetupType::File::operator!=(const File& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalVerifyRanksetupType::File::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ref");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(ref));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("path");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(path));
    }
}

InternalVerifyRanksetupType::File::File(const vespalib::slime::Inspector & __inspector)
{
    ref = __inspector["ref"]["value"].asString().make_string();
    path = __inspector["path"]["value"].asString().make_string();
}

InternalVerifyRanksetupType::File::File(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ref = ::config::internal::ValueConverter<vespalib::string>()("ref", __inspector["ref"]);
    path = ::config::internal::ValueConverter<vespalib::string>()("path", __inspector["path"]);
}

InternalVerifyRanksetupType::InternalVerifyRanksetupType()
    : file()
{
}

InternalVerifyRanksetupType::InternalVerifyRanksetupType(const InternalVerifyRanksetupType & __rhs) = default;
InternalVerifyRanksetupType & InternalVerifyRanksetupType::operator =(const InternalVerifyRanksetupType & __rhs) = default;
InternalVerifyRanksetupType::InternalVerifyRanksetupType(InternalVerifyRanksetupType && __rhs) noexcept = default;
InternalVerifyRanksetupType & InternalVerifyRanksetupType::operator =(InternalVerifyRanksetupType && __rhs) noexcept = default;
InternalVerifyRanksetupType::~InternalVerifyRanksetupType() = default; 
InternalVerifyRanksetupType::InternalVerifyRanksetupType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        file = ConfigParser::parseArray<FileVector>("file", __lines);
        ConfigParser::stripLinesForKey("file", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalVerifyRanksetupType::
operator==(const InternalVerifyRanksetupType& __rhs) const noexcept
{
    return (file == __rhs.file);
}

bool
InternalVerifyRanksetupType::
operator!=(const InternalVerifyRanksetupType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalVerifyRanksetupType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("file");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : file) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalVerifyRanksetupType::InternalVerifyRanksetupType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    for (size_t __i = 0; __i < __inspector["file"]["value"].children(); __i++) {
        file.push_back(File(__inspector["file"]["value"][__i]["value"]));
    }
}

InternalVerifyRanksetupType::InternalVerifyRanksetupType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<FileVector> __fileInserter(file);
    __inspector["file"].traverse(__fileInserter);
}

} // namespace internal

} // namespace vespa::config::search::core
