#include "config-translogserver.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace searchlib {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalTranslogserverType::CONFIG_DEF_MD5("c1ce57179c37c70725be8631e6ab0ef0");
const vespalib::string InternalTranslogserverType::CONFIG_DEF_NAME("translogserver");
const vespalib::string InternalTranslogserverType::CONFIG_DEF_NAMESPACE("searchlib");
const int64_t InternalTranslogserverType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=searchlib",
"listenport int default=13700 restart",
"filesizemax int default=50000000",
"servername string default=\"tls\" restart",
"basedir string default=\"tmp\" restart",
"usefsync bool default=true",
"maxthreads int default=0 restart",
"crcmethod enum {ccitt_crc32, xxh64} default=xxh64",
"compression.type enum {NONE, NONE_MULTI, LZ4, ZSTD} default=ZSTD",
"compression.level int default=3",
"chunk.sizelimit int default = 256000",
};
const ::config::StringVector InternalTranslogserverType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalTranslogserverType::Crcmethod
InternalTranslogserverType::getCrcmethod(const vespalib::string& name)
{
    if (name == "ccitt_crc32") {
        return Crcmethod::ccitt_crc32;
    } else if (name == "xxh64") {
        return Crcmethod::xxh64;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalTranslogserverType::getCrcmethodName(Crcmethod t)
{
    switch (t) {
        case Crcmethod::ccitt_crc32: return "ccitt_crc32";
        case Crcmethod::xxh64: return "xxh64";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalTranslogserverType::Crcmethod InternalTranslogserverType::InternalCrcmethodConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalTranslogserverType::getCrcmethod(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalTranslogserverType::Crcmethod InternalTranslogserverType::InternalCrcmethodConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalTranslogserverType::getCrcmethod(__inspector.asString().make_string());
}
InternalTranslogserverType::Crcmethod InternalTranslogserverType::InternalCrcmethodConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Crcmethod __eDefault) {
    if (__inspector.valid()) {
        return InternalTranslogserverType::getCrcmethod(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalTranslogserverType::Compression::Type
InternalTranslogserverType::Compression::getType(const vespalib::string& name)
{
    if (name == "NONE") {
        return Type::NONE;
    } else if (name == "NONE_MULTI") {
        return Type::NONE_MULTI;
    } else if (name == "LZ4") {
        return Type::LZ4;
    } else if (name == "ZSTD") {
        return Type::ZSTD;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalTranslogserverType::Compression::getTypeName(Type t)
{
    switch (t) {
        case Type::NONE: return "NONE";
        case Type::NONE_MULTI: return "NONE_MULTI";
        case Type::LZ4: return "LZ4";
        case Type::ZSTD: return "ZSTD";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalTranslogserverType::Compression::Type InternalTranslogserverType::Compression::InternalTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalTranslogserverType::Compression::getType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalTranslogserverType::Compression::Type InternalTranslogserverType::Compression::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalTranslogserverType::Compression::getType(__inspector.asString().make_string());
}
InternalTranslogserverType::Compression::Type InternalTranslogserverType::Compression::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault) {
    if (__inspector.valid()) {
        return InternalTranslogserverType::Compression::getType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalTranslogserverType::Compression::Compression() noexcept
    : type(Type::ZSTD),
      level(3)
{
}

InternalTranslogserverType::Compression::Compression(const Compression & __rhs) noexcept = default;
InternalTranslogserverType::Compression & InternalTranslogserverType::Compression::operator = (const Compression & __rhs) noexcept = default;
InternalTranslogserverType::Compression::Compression(Compression && __rhs) noexcept = default;
InternalTranslogserverType::Compression & InternalTranslogserverType::Compression::operator = (Compression && __rhs) noexcept = default;
InternalTranslogserverType::Compression::~Compression() = default; 
InternalTranslogserverType::Compression::Compression(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    type = getType(ConfigParser::parse<vespalib::string>("type", __lines, "ZSTD"));
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
    level = ConfigParser::parse<int32_t>("level", __lines, 3);
    ConfigParser::stripLinesForKey("level", __remainingValuesToParse);
}

bool
InternalTranslogserverType::Compression::
operator==(const Compression& __rhs) const noexcept
{
    return (type == __rhs.type &&
            level == __rhs.level);
}

bool
InternalTranslogserverType::Compression::
operator!=(const Compression& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalTranslogserverType::Compression::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getTypeName(type)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("level");
        __c.setString("type", "int");
        __c.setLong("value", level);
    }
}

InternalTranslogserverType::Compression::Compression(const vespalib::slime::Inspector & __inspector)
{
    type = getType(__inspector["type"]["value"].asString().make_string());
    level = __inspector["level"]["value"].asLong();
}

InternalTranslogserverType::Compression::Compression(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    type = InternalTypeConverter()(__inspector["type"], Type::ZSTD);
    level = ::config::internal::ValueConverter<int32_t>()(__inspector["level"], 3);
}

InternalTranslogserverType::Chunk::Chunk() noexcept
    : sizelimit(256000)
{
}

InternalTranslogserverType::Chunk::Chunk(const Chunk & __rhs) noexcept = default;
InternalTranslogserverType::Chunk & InternalTranslogserverType::Chunk::operator = (const Chunk & __rhs) noexcept = default;
InternalTranslogserverType::Chunk::Chunk(Chunk && __rhs) noexcept = default;
InternalTranslogserverType::Chunk & InternalTranslogserverType::Chunk::operator = (Chunk && __rhs) noexcept = default;
InternalTranslogserverType::Chunk::~Chunk() = default; 
InternalTranslogserverType::Chunk::Chunk(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    sizelimit = ConfigParser::parse<int32_t>("sizelimit", __lines, 256000);
    ConfigParser::stripLinesForKey("sizelimit", __remainingValuesToParse);
}

bool
InternalTranslogserverType::Chunk::operator==(const Chunk& __rhs) const noexcept
{
    return (sizelimit == __rhs.sizelimit);
}

bool
InternalTranslogserverType::Chunk::operator!=(const Chunk& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalTranslogserverType::Chunk::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("sizelimit");
        __c.setString("type", "int");
        __c.setLong("value", sizelimit);
    }
}

InternalTranslogserverType::Chunk::Chunk(const vespalib::slime::Inspector & __inspector)
{
    sizelimit = __inspector["sizelimit"]["value"].asLong();
}

InternalTranslogserverType::Chunk::Chunk(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    sizelimit = ::config::internal::ValueConverter<int32_t>()(__inspector["sizelimit"], 256000);
}

InternalTranslogserverType::InternalTranslogserverType() noexcept
    : listenport(13700),
      filesizemax(50000000),
      servername("tls"),
      basedir("tmp"),
      usefsync(true),
      maxthreads(0),
      crcmethod(Crcmethod::xxh64),
      compression(),
      chunk()
{
}

InternalTranslogserverType::InternalTranslogserverType(const InternalTranslogserverType & __rhs) noexcept = default;
InternalTranslogserverType & InternalTranslogserverType::operator =(const InternalTranslogserverType & __rhs) noexcept = default;
InternalTranslogserverType::InternalTranslogserverType(InternalTranslogserverType && __rhs) noexcept = default;
InternalTranslogserverType & InternalTranslogserverType::operator =(InternalTranslogserverType && __rhs) noexcept = default;
InternalTranslogserverType::~InternalTranslogserverType() = default; 
InternalTranslogserverType::InternalTranslogserverType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        listenport = ConfigParser::parse<int32_t>("listenport", __lines, 13700);
        ConfigParser::stripLinesForKey("listenport", __remainingValuesToParse);
        filesizemax = ConfigParser::parse<int32_t>("filesizemax", __lines, 50000000);
        ConfigParser::stripLinesForKey("filesizemax", __remainingValuesToParse);
        servername = ConfigParser::parse<vespalib::string>("servername", __lines, "tls");
        ConfigParser::stripLinesForKey("servername", __remainingValuesToParse);
        basedir = ConfigParser::parse<vespalib::string>("basedir", __lines, "tmp");
        ConfigParser::stripLinesForKey("basedir", __remainingValuesToParse);
        usefsync = ConfigParser::parse<bool>("usefsync", __lines, true);
        ConfigParser::stripLinesForKey("usefsync", __remainingValuesToParse);
        maxthreads = ConfigParser::parse<int32_t>("maxthreads", __lines, 0);
        ConfigParser::stripLinesForKey("maxthreads", __remainingValuesToParse);
        crcmethod = getCrcmethod(ConfigParser::parse<vespalib::string>("crcmethod", __lines, "xxh64"));
        ConfigParser::stripLinesForKey("crcmethod", __remainingValuesToParse);
        compression = ConfigParser::parseStruct<Compression>("compression", __lines);
        ConfigParser::stripLinesForKey("compression", __remainingValuesToParse);
        chunk = ConfigParser::parseStruct<Chunk>("chunk", __lines);
        ConfigParser::stripLinesForKey("chunk", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalTranslogserverType::
operator==(const InternalTranslogserverType& __rhs) const noexcept
{
    return (listenport == __rhs.listenport &&
            filesizemax == __rhs.filesizemax &&
            servername == __rhs.servername &&
            basedir == __rhs.basedir &&
            usefsync == __rhs.usefsync &&
            maxthreads == __rhs.maxthreads &&
            crcmethod == __rhs.crcmethod &&
            compression == __rhs.compression &&
            chunk == __rhs.chunk);
}

bool
InternalTranslogserverType::
operator!=(const InternalTranslogserverType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalTranslogserverType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("listenport");
        __c.setString("type", "int");
        __c.setLong("value", listenport);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("filesizemax");
        __c.setString("type", "int");
        __c.setLong("value", filesizemax);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("servername");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(servername));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("basedir");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(basedir));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("usefsync");
        __c.setString("type", "bool");
        __c.setBool("value", usefsync);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxthreads");
        __c.setString("type", "int");
        __c.setLong("value", maxthreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("crcmethod");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getCrcmethodName(crcmethod)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("compression");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        compression.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("chunk");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        chunk.serialize(__c2);
    }
}

InternalTranslogserverType::InternalTranslogserverType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    listenport = __inspector["listenport"]["value"].asLong();
    filesizemax = __inspector["filesizemax"]["value"].asLong();
    servername = __inspector["servername"]["value"].asString().make_string();
    basedir = __inspector["basedir"]["value"].asString().make_string();
    usefsync = __inspector["usefsync"]["value"].asBool();
    maxthreads = __inspector["maxthreads"]["value"].asLong();
    crcmethod = getCrcmethod(__inspector["crcmethod"]["value"].asString().make_string());
    compression = Compression(__inspector["compression"]["value"]);
    chunk = Chunk(__inspector["chunk"]["value"]);
}

InternalTranslogserverType::InternalTranslogserverType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    listenport = ::config::internal::ValueConverter<int32_t>()(__inspector["listenport"], 13700);
    filesizemax = ::config::internal::ValueConverter<int32_t>()(__inspector["filesizemax"], 50000000);
    servername = ::config::internal::ValueConverter<vespalib::string>()(__inspector["servername"], "tls");
    basedir = ::config::internal::ValueConverter<vespalib::string>()(__inspector["basedir"], "tmp");
    usefsync = ::config::internal::ValueConverter<bool>()(__inspector["usefsync"], true);
    maxthreads = ::config::internal::ValueConverter<int32_t>()(__inspector["maxthreads"], 0);
    crcmethod = InternalCrcmethodConverter()(__inspector["crcmethod"], Crcmethod::xxh64);
    compression = ::config::internal::ValueConverter<Compression>()(__inspector["compression"]);
    chunk = ::config::internal::ValueConverter<Chunk>()(__inspector["chunk"]);
}

} // namespace internal

} // namespace searchlib
