/**
 * @class searchlib::Translogserver
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_SEARCHLIB_TRANSLOGSERVER_H
#define CLOUD_CONFIG_SEARCHLIB_TRANSLOGSERVER_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace searchlib {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalTranslogserverType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalTranslogserverType> UP;
    enum class Crcmethod { ccitt_crc32, xxh64 };
    typedef std::vector<Crcmethod> CrcmethodVector;
    typedef std::map<vespalib::string, Crcmethod> CrcmethodMap;
    static Crcmethod getCrcmethod(const vespalib::string&);
    static vespalib::string getCrcmethodName(Crcmethod e);

    struct InternalCrcmethodConverter {
        Crcmethod operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
        Crcmethod operator()(const ::vespalib::slime::Inspector & __inspector);
        Crcmethod operator()(const ::vespalib::slime::Inspector & __inspector, Crcmethod __eDefault);
    };
    class Compression {
    public:
        enum class Type { NONE, NONE_MULTI, LZ4, ZSTD };
        typedef std::vector<Type> TypeVector;
        typedef std::map<vespalib::string, Type> TypeMap;
        static Type getType(const vespalib::string&);
        static vespalib::string getTypeName(Type e);

        struct InternalTypeConverter {
            Type operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Type operator()(const ::vespalib::slime::Inspector & __inspector);
            Type operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault);
        };
        Compression(const ::config::StringVector & __lines);
        Compression(const vespalib::slime::Inspector & __inspector);
        Compression(const ::config::ConfigPayload & __payload);
        Compression()  noexcept;
        Compression(const Compression & __rhs) noexcept;
        Compression & operator = (const Compression & __rhs) noexcept;
        Compression(Compression && __rhs) noexcept;
        Compression & operator = (Compression && __rhs) noexcept;
        ~Compression();

        bool operator==(const Compression& __rhs) const noexcept;
        bool operator!=(const Compression& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** Control compression type. */
        Type type; // Default: ZSTD

        /**
         * Control compression level
         * LZ4 has normal range 1..9 while ZSTD has range 1..19
         * 9 is a reasonable default for both
         */
        int32_t level; // Default: 3
    };
    typedef std::vector<Compression> CompressionVector;

    typedef std::map<vespalib::string, Compression> CompressionMap;

    class Chunk {
    public:
        Chunk(const ::config::StringVector & __lines);
        Chunk(const vespalib::slime::Inspector & __inspector);
        Chunk(const ::config::ConfigPayload & __payload);
        Chunk()  noexcept;
        Chunk(const Chunk & __rhs) noexcept;
        Chunk & operator = (const Chunk & __rhs) noexcept;
        Chunk(Chunk && __rhs) noexcept;
        Chunk & operator = (Chunk && __rhs) noexcept;
        ~Chunk();

        bool operator==(const Chunk& __rhs) const noexcept;
        bool operator!=(const Chunk& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** How large a chunk can grow in memory before beeing flushed */
        int32_t sizelimit; // Default: 256000
    };
    typedef std::vector<Chunk> ChunkVector;

    typedef std::map<vespalib::string, Chunk> ChunkMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalTranslogserverType(const ::config::ConfigValue & __value);
    InternalTranslogserverType(const ::config::ConfigDataBuffer & __value);
    InternalTranslogserverType(const ::config::ConfigPayload & __payload);
    InternalTranslogserverType()  noexcept;
    InternalTranslogserverType(const InternalTranslogserverType & __rhs) noexcept;
    InternalTranslogserverType & operator = (const InternalTranslogserverType & __rhs) noexcept;
    InternalTranslogserverType(InternalTranslogserverType && __rhs) noexcept;
    InternalTranslogserverType & operator = (InternalTranslogserverType && __rhs) noexcept;
    ~InternalTranslogserverType();

    bool operator==(const InternalTranslogserverType& __rhs) const noexcept;
    bool operator!=(const InternalTranslogserverType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /** Port number to use for listening. */
    int32_t listenport; // Default: 13700

    /** Max file size (50M) */
    int32_t filesizemax; // Default: 50000000

    /** Server name to identify server. */
    vespalib::string servername; // Default: "tls"

    /**
     * Base directory. The default is not used as it is decided by the model.
     */
    vespalib::string basedir; // Default: "tmp"

    /**
     * Use fsync after each commit.
     * If not the below interval is used.
     */
    bool usefsync; // Default: true

    /** Number of threads available for visiting/subscription. */
    int32_t maxthreads; // Default: 0

    /** Default crc method used */
    Crcmethod crcmethod; // Default: xxh64
    Compression compression;
    Chunk chunk;
};

} // namespace internal

typedef internal::InternalTranslogserverType TranslogserverConfigBuilder;
typedef const internal::InternalTranslogserverType TranslogserverConfig;

} // namespace searchlib
#endif // VESPA_config_SEARCHLIB_TRANSLOGSERVER_H
