#include "config-juniperrc.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search::summary {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalJuniperrcType::CONFIG_DEF_MD5("a16512198c0f1d91f8390f1827386205");
const vespalib::string InternalJuniperrcType::CONFIG_DEF_NAME("juniperrc");
const vespalib::string InternalJuniperrcType::CONFIG_DEF_NAMESPACE("vespa.config.search.summary");
const int64_t InternalJuniperrcType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search.summary",
"length int default=256",
"max_matches int default=3",
"min_length int default=128",
"prefix bool default=true",
"surround_max int default=128",
"winsize int default=200",
"winsize_fallback_multiplier double default=10.0",
"max_match_candidates int default=1000",
"stem_min_length int default=5",
"stem_max_extend int default=3",
"override[].fieldname string",
"override[].length int default=256",
"override[].max_matches int default=3",
"override[].min_length int default=128",
"override[].prefix bool default=true",
"override[].surround_max int default=128",
"override[].winsize int default=200",
"override[].winsize_fallback_multiplier double default=10.0",
"override[].max_match_candidates int default=1000",
"override[].stem_min_length int default=5",
"override[].stem_max_extend int default=3",
};
const ::config::StringVector InternalJuniperrcType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalJuniperrcType::Override::Override() noexcept
    : fieldname(),
      length(256),
      maxMatches(3),
      minLength(128),
      prefix(true),
      surroundMax(128),
      winsize(200),
      winsizeFallbackMultiplier(10.0),
      maxMatchCandidates(1000),
      stemMinLength(5),
      stemMaxExtend(3)
{
}

InternalJuniperrcType::Override::Override(const Override & __rhs) noexcept = default;
InternalJuniperrcType::Override & InternalJuniperrcType::Override::operator = (const Override & __rhs) noexcept = default;
InternalJuniperrcType::Override::Override(Override && __rhs) noexcept = default;
InternalJuniperrcType::Override & InternalJuniperrcType::Override::operator = (Override && __rhs) noexcept = default;
InternalJuniperrcType::Override::~Override() = default; 
InternalJuniperrcType::Override::Override(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    fieldname = ConfigParser::parse<vespalib::string>("fieldname", __lines);
    ConfigParser::stripLinesForKey("fieldname", __remainingValuesToParse);
    length = ConfigParser::parse<int32_t>("length", __lines, 256);
    ConfigParser::stripLinesForKey("length", __remainingValuesToParse);
    maxMatches = ConfigParser::parse<int32_t>("max_matches", __lines, 3);
    ConfigParser::stripLinesForKey("max_matches", __remainingValuesToParse);
    minLength = ConfigParser::parse<int32_t>("min_length", __lines, 128);
    ConfigParser::stripLinesForKey("min_length", __remainingValuesToParse);
    prefix = ConfigParser::parse<bool>("prefix", __lines, true);
    ConfigParser::stripLinesForKey("prefix", __remainingValuesToParse);
    surroundMax = ConfigParser::parse<int32_t>("surround_max", __lines, 128);
    ConfigParser::stripLinesForKey("surround_max", __remainingValuesToParse);
    winsize = ConfigParser::parse<int32_t>("winsize", __lines, 200);
    ConfigParser::stripLinesForKey("winsize", __remainingValuesToParse);
    winsizeFallbackMultiplier = ConfigParser::parse<double>("winsize_fallback_multiplier", __lines, 10.0);
    ConfigParser::stripLinesForKey("winsize_fallback_multiplier", __remainingValuesToParse);
    maxMatchCandidates = ConfigParser::parse<int32_t>("max_match_candidates", __lines, 1000);
    ConfigParser::stripLinesForKey("max_match_candidates", __remainingValuesToParse);
    stemMinLength = ConfigParser::parse<int32_t>("stem_min_length", __lines, 5);
    ConfigParser::stripLinesForKey("stem_min_length", __remainingValuesToParse);
    stemMaxExtend = ConfigParser::parse<int32_t>("stem_max_extend", __lines, 3);
    ConfigParser::stripLinesForKey("stem_max_extend", __remainingValuesToParse);
}

bool
InternalJuniperrcType::Override::operator==(const Override& __rhs) const noexcept
{
    return (fieldname == __rhs.fieldname &&
            length == __rhs.length &&
            maxMatches == __rhs.maxMatches &&
            minLength == __rhs.minLength &&
            prefix == __rhs.prefix &&
            surroundMax == __rhs.surroundMax &&
            winsize == __rhs.winsize &&
            winsizeFallbackMultiplier == __rhs.winsizeFallbackMultiplier &&
            maxMatchCandidates == __rhs.maxMatchCandidates &&
            stemMinLength == __rhs.stemMinLength &&
            stemMaxExtend == __rhs.stemMaxExtend);
}

bool
InternalJuniperrcType::Override::operator!=(const Override& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalJuniperrcType::Override::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fieldname");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(fieldname));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("length");
        __c.setString("type", "int");
        __c.setLong("value", length);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_matches");
        __c.setString("type", "int");
        __c.setLong("value", maxMatches);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("min_length");
        __c.setString("type", "int");
        __c.setLong("value", minLength);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("prefix");
        __c.setString("type", "bool");
        __c.setBool("value", prefix);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("surround_max");
        __c.setString("type", "int");
        __c.setLong("value", surroundMax);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("winsize");
        __c.setString("type", "int");
        __c.setLong("value", winsize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("winsize_fallback_multiplier");
        __c.setString("type", "double");
        __c.setDouble("value", winsizeFallbackMultiplier);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_match_candidates");
        __c.setString("type", "int");
        __c.setLong("value", maxMatchCandidates);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("stem_min_length");
        __c.setString("type", "int");
        __c.setLong("value", stemMinLength);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("stem_max_extend");
        __c.setString("type", "int");
        __c.setLong("value", stemMaxExtend);
    }
}

InternalJuniperrcType::Override::Override(const vespalib::slime::Inspector & __inspector)
{
    fieldname = __inspector["fieldname"]["value"].asString().make_string();
    length = __inspector["length"]["value"].asLong();
    maxMatches = __inspector["max_matches"]["value"].asLong();
    minLength = __inspector["min_length"]["value"].asLong();
    prefix = __inspector["prefix"]["value"].asBool();
    surroundMax = __inspector["surround_max"]["value"].asLong();
    winsize = __inspector["winsize"]["value"].asLong();
    winsizeFallbackMultiplier = __inspector["winsize_fallback_multiplier"]["value"].asDouble();
    maxMatchCandidates = __inspector["max_match_candidates"]["value"].asLong();
    stemMinLength = __inspector["stem_min_length"]["value"].asLong();
    stemMaxExtend = __inspector["stem_max_extend"]["value"].asLong();
}

InternalJuniperrcType::Override::Override(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    fieldname = ::config::internal::ValueConverter<vespalib::string>()("fieldname", __inspector["fieldname"]);
    length = ::config::internal::ValueConverter<int32_t>()(__inspector["length"], 256);
    maxMatches = ::config::internal::ValueConverter<int32_t>()(__inspector["max_matches"], 3);
    minLength = ::config::internal::ValueConverter<int32_t>()(__inspector["min_length"], 128);
    prefix = ::config::internal::ValueConverter<bool>()(__inspector["prefix"], true);
    surroundMax = ::config::internal::ValueConverter<int32_t>()(__inspector["surround_max"], 128);
    winsize = ::config::internal::ValueConverter<int32_t>()(__inspector["winsize"], 200);
    winsizeFallbackMultiplier = ::config::internal::ValueConverter<double>()(__inspector["winsize_fallback_multiplier"], 10.0);
    maxMatchCandidates = ::config::internal::ValueConverter<int32_t>()(__inspector["max_match_candidates"], 1000);
    stemMinLength = ::config::internal::ValueConverter<int32_t>()(__inspector["stem_min_length"], 5);
    stemMaxExtend = ::config::internal::ValueConverter<int32_t>()(__inspector["stem_max_extend"], 3);
}

InternalJuniperrcType::InternalJuniperrcType()
    : length(256),
      maxMatches(3),
      minLength(128),
      prefix(true),
      surroundMax(128),
      winsize(200),
      winsizeFallbackMultiplier(10.0),
      maxMatchCandidates(1000),
      stemMinLength(5),
      stemMaxExtend(3),
      override()
{
}

InternalJuniperrcType::InternalJuniperrcType(const InternalJuniperrcType & __rhs) = default;
InternalJuniperrcType & InternalJuniperrcType::operator =(const InternalJuniperrcType & __rhs) = default;
InternalJuniperrcType::InternalJuniperrcType(InternalJuniperrcType && __rhs) noexcept = default;
InternalJuniperrcType & InternalJuniperrcType::operator =(InternalJuniperrcType && __rhs) noexcept = default;
InternalJuniperrcType::~InternalJuniperrcType() = default; 
InternalJuniperrcType::InternalJuniperrcType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        length = ConfigParser::parse<int32_t>("length", __lines, 256);
        ConfigParser::stripLinesForKey("length", __remainingValuesToParse);
        maxMatches = ConfigParser::parse<int32_t>("max_matches", __lines, 3);
        ConfigParser::stripLinesForKey("max_matches", __remainingValuesToParse);
        minLength = ConfigParser::parse<int32_t>("min_length", __lines, 128);
        ConfigParser::stripLinesForKey("min_length", __remainingValuesToParse);
        prefix = ConfigParser::parse<bool>("prefix", __lines, true);
        ConfigParser::stripLinesForKey("prefix", __remainingValuesToParse);
        surroundMax = ConfigParser::parse<int32_t>("surround_max", __lines, 128);
        ConfigParser::stripLinesForKey("surround_max", __remainingValuesToParse);
        winsize = ConfigParser::parse<int32_t>("winsize", __lines, 200);
        ConfigParser::stripLinesForKey("winsize", __remainingValuesToParse);
        winsizeFallbackMultiplier = ConfigParser::parse<double>("winsize_fallback_multiplier", __lines, 10.0);
        ConfigParser::stripLinesForKey("winsize_fallback_multiplier", __remainingValuesToParse);
        maxMatchCandidates = ConfigParser::parse<int32_t>("max_match_candidates", __lines, 1000);
        ConfigParser::stripLinesForKey("max_match_candidates", __remainingValuesToParse);
        stemMinLength = ConfigParser::parse<int32_t>("stem_min_length", __lines, 5);
        ConfigParser::stripLinesForKey("stem_min_length", __remainingValuesToParse);
        stemMaxExtend = ConfigParser::parse<int32_t>("stem_max_extend", __lines, 3);
        ConfigParser::stripLinesForKey("stem_max_extend", __remainingValuesToParse);
        override = ConfigParser::parseArray<OverrideVector>("override", __lines);
        ConfigParser::stripLinesForKey("override", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalJuniperrcType::operator==(const InternalJuniperrcType& __rhs) const noexcept
{
    return (length == __rhs.length &&
            maxMatches == __rhs.maxMatches &&
            minLength == __rhs.minLength &&
            prefix == __rhs.prefix &&
            surroundMax == __rhs.surroundMax &&
            winsize == __rhs.winsize &&
            winsizeFallbackMultiplier == __rhs.winsizeFallbackMultiplier &&
            maxMatchCandidates == __rhs.maxMatchCandidates &&
            stemMinLength == __rhs.stemMinLength &&
            stemMaxExtend == __rhs.stemMaxExtend &&
            override == __rhs.override);
}

bool
InternalJuniperrcType::operator!=(const InternalJuniperrcType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalJuniperrcType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("length");
        __c.setString("type", "int");
        __c.setLong("value", length);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_matches");
        __c.setString("type", "int");
        __c.setLong("value", maxMatches);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("min_length");
        __c.setString("type", "int");
        __c.setLong("value", minLength);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("prefix");
        __c.setString("type", "bool");
        __c.setBool("value", prefix);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("surround_max");
        __c.setString("type", "int");
        __c.setLong("value", surroundMax);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("winsize");
        __c.setString("type", "int");
        __c.setLong("value", winsize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("winsize_fallback_multiplier");
        __c.setString("type", "double");
        __c.setDouble("value", winsizeFallbackMultiplier);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_match_candidates");
        __c.setString("type", "int");
        __c.setLong("value", maxMatchCandidates);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("stem_min_length");
        __c.setString("type", "int");
        __c.setLong("value", stemMinLength);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("stem_max_extend");
        __c.setString("type", "int");
        __c.setLong("value", stemMaxExtend);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("override");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : override) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalJuniperrcType::InternalJuniperrcType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    length = __inspector["length"]["value"].asLong();
    maxMatches = __inspector["max_matches"]["value"].asLong();
    minLength = __inspector["min_length"]["value"].asLong();
    prefix = __inspector["prefix"]["value"].asBool();
    surroundMax = __inspector["surround_max"]["value"].asLong();
    winsize = __inspector["winsize"]["value"].asLong();
    winsizeFallbackMultiplier = __inspector["winsize_fallback_multiplier"]["value"].asDouble();
    maxMatchCandidates = __inspector["max_match_candidates"]["value"].asLong();
    stemMinLength = __inspector["stem_min_length"]["value"].asLong();
    stemMaxExtend = __inspector["stem_max_extend"]["value"].asLong();
    for (size_t __i = 0; __i < __inspector["override"]["value"].children(); __i++) {
        override.push_back(Override(__inspector["override"]["value"][__i]["value"]));
    }
}

InternalJuniperrcType::InternalJuniperrcType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    length = ::config::internal::ValueConverter<int32_t>()(__inspector["length"], 256);
    maxMatches = ::config::internal::ValueConverter<int32_t>()(__inspector["max_matches"], 3);
    minLength = ::config::internal::ValueConverter<int32_t>()(__inspector["min_length"], 128);
    prefix = ::config::internal::ValueConverter<bool>()(__inspector["prefix"], true);
    surroundMax = ::config::internal::ValueConverter<int32_t>()(__inspector["surround_max"], 128);
    winsize = ::config::internal::ValueConverter<int32_t>()(__inspector["winsize"], 200);
    winsizeFallbackMultiplier = ::config::internal::ValueConverter<double>()(__inspector["winsize_fallback_multiplier"], 10.0);
    maxMatchCandidates = ::config::internal::ValueConverter<int32_t>()(__inspector["max_match_candidates"], 1000);
    stemMinLength = ::config::internal::ValueConverter<int32_t>()(__inspector["stem_min_length"], 5);
    stemMaxExtend = ::config::internal::ValueConverter<int32_t>()(__inspector["stem_max_extend"], 3);
    ::config::internal::VectorInserter<OverrideVector> __overrideInserter(override);
    __inspector["override"].traverse(__overrideInserter);
}

} // namespace internal

} // namespace vespa::config::search::summary
