/**
 * @class vespa::config::search::summary::Juniperrc
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_SEARCH_SUMMARY_JUNIPERRC_H
#define CLOUD_CONFIG_VESPA_CONFIG_SEARCH_SUMMARY_JUNIPERRC_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::search::summary {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalJuniperrcType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalJuniperrcType> UP;
    class Override {
    public:
        Override(const ::config::StringVector & __lines);
        Override(const vespalib::slime::Inspector & __inspector);
        Override(const ::config::ConfigPayload & __payload);
        Override()  noexcept;
        Override(const Override & __rhs) noexcept;
        Override & operator = (const Override & __rhs) noexcept;
        Override(Override && __rhs) noexcept;
        Override & operator = (Override && __rhs) noexcept;
        ~Override();

        bool operator==(const Override& __rhs) const noexcept;
        bool operator!=(const Override& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * The parameters above may also be overriden on a per-field basis
         * using the following array.
         */
        vespalib::string fieldname;
        int32_t length; // Default: 256
        int32_t maxMatches; // Default: 3
        int32_t minLength; // Default: 128
        bool prefix; // Default: true
        int32_t surroundMax; // Default: 128
        int32_t winsize; // Default: 200
        double winsizeFallbackMultiplier; // Default: 10.0
        int32_t maxMatchCandidates; // Default: 1000
        int32_t stemMinLength; // Default: 5
        int32_t stemMaxExtend; // Default: 3
    };
    typedef std::vector<Override> OverrideVector;

    typedef std::map<vespalib::string, Override> OverrideMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalJuniperrcType(const ::config::ConfigValue & __value);
    InternalJuniperrcType(const ::config::ConfigDataBuffer & __value);
    InternalJuniperrcType(const ::config::ConfigPayload & __payload);
    InternalJuniperrcType() ;
    InternalJuniperrcType(const InternalJuniperrcType & __rhs);
    InternalJuniperrcType & operator = (const InternalJuniperrcType & __rhs);
    InternalJuniperrcType(InternalJuniperrcType && __rhs) noexcept;
    InternalJuniperrcType & operator = (InternalJuniperrcType && __rhs) noexcept;
    ~InternalJuniperrcType();

    bool operator==(const InternalJuniperrcType& __rhs) const noexcept;
    bool operator!=(const InternalJuniperrcType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /**
     * Set the length (in #characters) of the dynamically generated
     * summaries. This is a hint to the module that generates the
     * dynamic summary - the actual size will depend on the available text
     * and the query. You may find that you need to set this length
     * substantially longer than your actual desired average length -
     * setting this about twice as long is often appropriate.
     */
    int32_t length; // Default: 256

    /**
     * The number of (possibly partial) set of keywords
     * matching the query, to attempt to include in the summary. The larger this
     * value compared is set relative to the length parameter, the more
     * dense the keywords may appear in the summary.
     */
    int32_t maxMatches; // Default: 3

    /**
     * Minimal desired length of the generated summary in
     * bytes. This is the shortest summary length for which the number of
     * matches will be respected. Eg. if
     * a summary appear to become shorter than min_length bytes with
     * max_matches matches, then additional matches will be used if available.
     */
    int32_t minLength; // Default: 128

    /**
     * Make sure the prefix (length controlled by 'juniper.dynsum.length')
     * of all fields with summary: dynamic are returned in the dynamic
     * summary if a query does not hit in those fields
     */
    bool prefix; // Default: true

    /**
     * The maximal number of bytes of context to prepend and append to
     * each of the selected query keyword hits. This parameter defines the
     * max size a summary would become if there are few keyword hits
     * (max_matches set low or document contained few matches of the keywords).
     */
    int32_t surroundMax; // Default: 128

    /**
     * The size of the sliding window used to determine if
     * multiple query terms occur together. The larger the value, the more
     * likely the system will find (and present in dynamic summary) complete
     * matches containing all the search terms. The downside is a potential
     * performance overhead of keeping candidates for matches longer during
     * matching, and consequently updating more candidates that eventually
     * gets thrown.
     */
    int32_t winsize; // Default: 200

    /**
     * This value multiplied with the winsize gives the size of a fallback
     * window used to break out when searching for phrase term matches.
     */
    double winsizeFallbackMultiplier; // Default: 10.0

    /**
     * This value specifies the maximum number of match candidates that are
     * managed for a non-leaf query node when matching the query against the
     * input text.
     */
    int32_t maxMatchCandidates; // Default: 1000

    /**
     * The minimal number of bytes in a query keyword for
     * it to be subject to the simple Juniper stemming algorithm. Keywords
     * that are shorter than or equal to this limit will only yield exact
     * matches in the dynamic summaries.
     */
    int32_t stemMinLength; // Default: 5

    /**
     * The maximal number of bytes that a word in the document
     * can be longer than the keyword itself to yield a match. Eg. for
     * the default values, if the keyword is 7 bytes long, it will match any
     * word with length less than or equal to 10 for which the keyword is a
     * prefix.
     */
    int32_t stemMaxExtend; // Default: 3
    OverrideVector override;
};

} // namespace internal

typedef internal::InternalJuniperrcType JuniperrcConfigBuilder;
typedef const internal::InternalJuniperrcType JuniperrcConfig;

} // namespace vespa::config::search::summary
#endif // VESPA_config_VESPA_CONFIG_SEARCH_SUMMARY_JUNIPERRC_H
