#include "config-rpc-provider.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalRpcProviderType::CONFIG_DEF_MD5("e0c721199ca1c8a9aef454289df8eab2");
const vespalib::string InternalRpcProviderType::CONFIG_DEF_NAME("rpc-provider");
const vespalib::string InternalRpcProviderType::CONFIG_DEF_NAMESPACE("vespa.config.content.core");
const int64_t InternalRpcProviderType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content.core",
"connectspec string default=\"tcp/localhost:17777\" restart",
};
const ::config::StringVector InternalRpcProviderType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalRpcProviderType::InternalRpcProviderType() noexcept
    : connectspec("tcp/localhost:17777")
{
}

InternalRpcProviderType::InternalRpcProviderType(const InternalRpcProviderType & __rhs) noexcept = default;
InternalRpcProviderType & InternalRpcProviderType::operator =(const InternalRpcProviderType & __rhs) noexcept = default;
InternalRpcProviderType::InternalRpcProviderType(InternalRpcProviderType && __rhs) noexcept = default;
InternalRpcProviderType & InternalRpcProviderType::operator =(InternalRpcProviderType && __rhs) noexcept = default;
InternalRpcProviderType::~InternalRpcProviderType() = default; 
InternalRpcProviderType::InternalRpcProviderType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        connectspec = ConfigParser::parse<vespalib::string>("connectspec", __lines, "tcp/localhost:17777");
        ConfigParser::stripLinesForKey("connectspec", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalRpcProviderType::operator==(const InternalRpcProviderType& __rhs) const noexcept
{
    return (connectspec == __rhs.connectspec);
}

bool
InternalRpcProviderType::operator!=(const InternalRpcProviderType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalRpcProviderType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("connectspec");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(connectspec));
    }
}

InternalRpcProviderType::InternalRpcProviderType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    connectspec = __inspector["connectspec"]["value"].asString().make_string();
}

InternalRpcProviderType::InternalRpcProviderType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    connectspec = ::config::internal::ValueConverter<vespalib::string>()(__inspector["connectspec"], "tcp/localhost:17777");
}

} // namespace internal

} // namespace vespa::config::content::core
