#include "config-stor-bouncer.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalStorBouncerType::CONFIG_DEF_MD5("d0a19f7c56d6355f3f629baa35d46fdb");
const vespalib::string InternalStorBouncerType::CONFIG_DEF_NAME("stor-bouncer");
const vespalib::string InternalStorBouncerType::CONFIG_DEF_NAMESPACE("vespa.config.content.core");
const int64_t InternalStorBouncerType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content.core",
"max_clock_skew_seconds int default=5",
};
const ::config::StringVector InternalStorBouncerType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalStorBouncerType::InternalStorBouncerType() noexcept
    : maxClockSkewSeconds(5)
{
}

InternalStorBouncerType::InternalStorBouncerType(const InternalStorBouncerType & __rhs) noexcept = default;
InternalStorBouncerType & InternalStorBouncerType::operator =(const InternalStorBouncerType & __rhs) noexcept = default;
InternalStorBouncerType::InternalStorBouncerType(InternalStorBouncerType && __rhs) noexcept = default;
InternalStorBouncerType & InternalStorBouncerType::operator =(InternalStorBouncerType && __rhs) noexcept = default;
InternalStorBouncerType::~InternalStorBouncerType() = default; 
InternalStorBouncerType::InternalStorBouncerType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        maxClockSkewSeconds = ConfigParser::parse<int32_t>("max_clock_skew_seconds", __lines, 5);
        ConfigParser::stripLinesForKey("max_clock_skew_seconds", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalStorBouncerType::operator==(const InternalStorBouncerType& __rhs) const noexcept
{
    return (maxClockSkewSeconds == __rhs.maxClockSkewSeconds);
}

bool
InternalStorBouncerType::operator!=(const InternalStorBouncerType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorBouncerType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_clock_skew_seconds");
        __c.setString("type", "int");
        __c.setLong("value", maxClockSkewSeconds);
    }
}

InternalStorBouncerType::InternalStorBouncerType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    maxClockSkewSeconds = __inspector["max_clock_skew_seconds"]["value"].asLong();
}

InternalStorBouncerType::InternalStorBouncerType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    maxClockSkewSeconds = ::config::internal::ValueConverter<int32_t>()(__inspector["max_clock_skew_seconds"], 5);
}

} // namespace internal

} // namespace vespa::config::content::core
