#include "config-stor-communicationmanager.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalStorCommunicationmanagerType::CONFIG_DEF_MD5("10b9c60647237b2c3c3905fd45cbae83");
const vespalib::string InternalStorCommunicationmanagerType::CONFIG_DEF_NAME("stor-communicationmanager");
const vespalib::string InternalStorCommunicationmanagerType::CONFIG_DEF_NAMESPACE("vespa.config.content.core");
const int64_t InternalStorCommunicationmanagerType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content.core",
"mbusport int default=-1 restart",
"rpcport int default=6000 restart",
"mbus_distributor_node_max_pending_count int default=5000",
"mbus_content_node_max_pending_count int default=0",
"mbus_distributor_node_max_pending_size int default=0",
"mbus_content_node_max_pending_size int default=0",
"mbus.compress.limit int default=1024 restart",
"mbus.compress.level int default=3 restart",
"mbus.compress.type enum {NONE, LZ4, ZSTD} default=LZ4 restart",
"mbus.rpctargetcache.ttl double default = 600 restart",
"mbus.num_rpc_targets int default=2 restart",
"mbus.tcp_no_delay bool default=true restart",
"mbus.num_network_threads int default=1 restart",
"mbus.events_before_wakeup int default=1 restart",
"rpc.num_network_threads int default=2 restart",
"rpc.events_before_wakeup int default=1 restart",
"rpc.num_targets_per_node int default=2 restart",
"rpc.compress.limit int default=1024 restart",
"rpc.compress.level int default=3 restart",
"rpc.compress.type enum {NONE, LZ4, ZSTD} default=LZ4 restart",
};
const ::config::StringVector InternalStorCommunicationmanagerType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalStorCommunicationmanagerType::Mbus::Compress::Type
InternalStorCommunicationmanagerType::Mbus::Compress::getType(const vespalib::string& name)
{
    if (name == "NONE") {
        return Type::NONE;
    } else if (name == "LZ4") {
        return Type::LZ4;
    } else if (name == "ZSTD") {
        return Type::ZSTD;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalStorCommunicationmanagerType::Mbus::Compress::getTypeName(Type t)
{
    switch (t) {
        case Type::NONE: return "NONE";
        case Type::LZ4: return "LZ4";
        case Type::ZSTD: return "ZSTD";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalStorCommunicationmanagerType::Mbus::Compress::Type InternalStorCommunicationmanagerType::Mbus::Compress::InternalTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalStorCommunicationmanagerType::Mbus::Compress::getType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalStorCommunicationmanagerType::Mbus::Compress::Type InternalStorCommunicationmanagerType::Mbus::Compress::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalStorCommunicationmanagerType::Mbus::Compress::getType(__inspector.asString().make_string());
}
InternalStorCommunicationmanagerType::Mbus::Compress::Type InternalStorCommunicationmanagerType::Mbus::Compress::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault) {
    if (__inspector.valid()) {
        return InternalStorCommunicationmanagerType::Mbus::Compress::getType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalStorCommunicationmanagerType::Mbus::Compress::Compress() noexcept
    : limit(1024),
      level(3),
      type(Type::LZ4)
{
}

InternalStorCommunicationmanagerType::Mbus::Compress::Compress(const Compress & __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Mbus::Compress & InternalStorCommunicationmanagerType::Mbus::Compress::operator = (const Compress & __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Mbus::Compress::Compress(Compress && __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Mbus::Compress & InternalStorCommunicationmanagerType::Mbus::Compress::operator = (Compress && __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Mbus::Compress::~Compress() = default; 
InternalStorCommunicationmanagerType::Mbus::Compress::Compress(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    limit = ConfigParser::parse<int32_t>("limit", __lines, 1024);
    ConfigParser::stripLinesForKey("limit", __remainingValuesToParse);
    level = ConfigParser::parse<int32_t>("level", __lines, 3);
    ConfigParser::stripLinesForKey("level", __remainingValuesToParse);
    type = getType(ConfigParser::parse<vespalib::string>("type", __lines, "LZ4"));
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
}

bool
InternalStorCommunicationmanagerType::Mbus::Compress::
operator==(const Compress& __rhs) const noexcept
{
    return (limit == __rhs.limit &&
            level == __rhs.level &&
            type == __rhs.type);
}

bool
InternalStorCommunicationmanagerType::Mbus::Compress::
operator!=(const Compress& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorCommunicationmanagerType::Mbus::Compress::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("limit");
        __c.setString("type", "int");
        __c.setLong("value", limit);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("level");
        __c.setString("type", "int");
        __c.setLong("value", level);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getTypeName(type)));
    }
}

InternalStorCommunicationmanagerType::Mbus::Compress::Compress(const vespalib::slime::Inspector & __inspector)
{
    limit = __inspector["limit"]["value"].asLong();
    level = __inspector["level"]["value"].asLong();
    type = getType(__inspector["type"]["value"].asString().make_string());
}

InternalStorCommunicationmanagerType::Mbus::Compress::Compress(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    limit = ::config::internal::ValueConverter<int32_t>()(__inspector["limit"], 1024);
    level = ::config::internal::ValueConverter<int32_t>()(__inspector["level"], 3);
    type = InternalTypeConverter()(__inspector["type"], Type::LZ4);
}

InternalStorCommunicationmanagerType::Mbus::Rpctargetcache::Rpctargetcache() noexcept
    : ttl(600)
{
}

InternalStorCommunicationmanagerType::Mbus::Rpctargetcache::Rpctargetcache(const Rpctargetcache & __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Mbus::Rpctargetcache & InternalStorCommunicationmanagerType::Mbus::Rpctargetcache::operator = (const Rpctargetcache & __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Mbus::Rpctargetcache::Rpctargetcache(Rpctargetcache && __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Mbus::Rpctargetcache & InternalStorCommunicationmanagerType::Mbus::Rpctargetcache::operator = (Rpctargetcache && __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Mbus::Rpctargetcache::~Rpctargetcache() = default; 
InternalStorCommunicationmanagerType::Mbus::Rpctargetcache::Rpctargetcache(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    ttl = ConfigParser::parse<double>("ttl", __lines, 600);
    ConfigParser::stripLinesForKey("ttl", __remainingValuesToParse);
}

bool
InternalStorCommunicationmanagerType::Mbus::Rpctargetcache::
operator==(const Rpctargetcache& __rhs) const noexcept
{
    return (ttl == __rhs.ttl);
}

bool
InternalStorCommunicationmanagerType::Mbus::Rpctargetcache::
operator!=(const Rpctargetcache& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorCommunicationmanagerType::Mbus::Rpctargetcache::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ttl");
        __c.setString("type", "double");
        __c.setDouble("value", ttl);
    }
}

InternalStorCommunicationmanagerType::Mbus::Rpctargetcache::Rpctargetcache(const vespalib::slime::Inspector & __inspector)
{
    ttl = __inspector["ttl"]["value"].asDouble();
}

InternalStorCommunicationmanagerType::Mbus::Rpctargetcache::Rpctargetcache(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ttl = ::config::internal::ValueConverter<double>()(__inspector["ttl"], 600);
}

InternalStorCommunicationmanagerType::Mbus::Mbus() noexcept
    : compress(),
      rpctargetcache(),
      numRpcTargets(2),
      tcpNoDelay(true),
      numNetworkThreads(1),
      eventsBeforeWakeup(1)
{
}

InternalStorCommunicationmanagerType::Mbus::Mbus(const Mbus & __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Mbus & InternalStorCommunicationmanagerType::Mbus::operator = (const Mbus & __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Mbus::Mbus(Mbus && __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Mbus & InternalStorCommunicationmanagerType::Mbus::operator = (Mbus && __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Mbus::~Mbus() = default; 
InternalStorCommunicationmanagerType::Mbus::Mbus(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    compress = ConfigParser::parseStruct<Compress>("compress", __lines);
    ConfigParser::stripLinesForKey("compress", __remainingValuesToParse);
    rpctargetcache = ConfigParser::parseStruct<Rpctargetcache>("rpctargetcache", __lines);
    ConfigParser::stripLinesForKey("rpctargetcache", __remainingValuesToParse);
    numRpcTargets = ConfigParser::parse<int32_t>("num_rpc_targets", __lines, 2);
    ConfigParser::stripLinesForKey("num_rpc_targets", __remainingValuesToParse);
    tcpNoDelay = ConfigParser::parse<bool>("tcp_no_delay", __lines, true);
    ConfigParser::stripLinesForKey("tcp_no_delay", __remainingValuesToParse);
    numNetworkThreads = ConfigParser::parse<int32_t>("num_network_threads", __lines, 1);
    ConfigParser::stripLinesForKey("num_network_threads", __remainingValuesToParse);
    eventsBeforeWakeup = ConfigParser::parse<int32_t>("events_before_wakeup", __lines, 1);
    ConfigParser::stripLinesForKey("events_before_wakeup", __remainingValuesToParse);
}

bool
InternalStorCommunicationmanagerType::Mbus::operator==(const Mbus& __rhs) const noexcept
{
    return (compress == __rhs.compress &&
            rpctargetcache == __rhs.rpctargetcache &&
            numRpcTargets == __rhs.numRpcTargets &&
            tcpNoDelay == __rhs.tcpNoDelay &&
            numNetworkThreads == __rhs.numNetworkThreads &&
            eventsBeforeWakeup == __rhs.eventsBeforeWakeup);
}

bool
InternalStorCommunicationmanagerType::Mbus::operator!=(const Mbus& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorCommunicationmanagerType::Mbus::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("compress");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        compress.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("rpctargetcache");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        rpctargetcache.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("num_rpc_targets");
        __c.setString("type", "int");
        __c.setLong("value", numRpcTargets);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tcp_no_delay");
        __c.setString("type", "bool");
        __c.setBool("value", tcpNoDelay);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("num_network_threads");
        __c.setString("type", "int");
        __c.setLong("value", numNetworkThreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("events_before_wakeup");
        __c.setString("type", "int");
        __c.setLong("value", eventsBeforeWakeup);
    }
}

InternalStorCommunicationmanagerType::Mbus::Mbus(const vespalib::slime::Inspector & __inspector)
{
    compress = Compress(__inspector["compress"]["value"]);
    rpctargetcache = Rpctargetcache(__inspector["rpctargetcache"]["value"]);
    numRpcTargets = __inspector["num_rpc_targets"]["value"].asLong();
    tcpNoDelay = __inspector["tcp_no_delay"]["value"].asBool();
    numNetworkThreads = __inspector["num_network_threads"]["value"].asLong();
    eventsBeforeWakeup = __inspector["events_before_wakeup"]["value"].asLong();
}

InternalStorCommunicationmanagerType::Mbus::Mbus(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    compress = ::config::internal::ValueConverter<Compress>()(__inspector["compress"]);
    rpctargetcache = ::config::internal::ValueConverter<Rpctargetcache>()(__inspector["rpctargetcache"]);
    numRpcTargets = ::config::internal::ValueConverter<int32_t>()(__inspector["num_rpc_targets"], 2);
    tcpNoDelay = ::config::internal::ValueConverter<bool>()(__inspector["tcp_no_delay"], true);
    numNetworkThreads = ::config::internal::ValueConverter<int32_t>()(__inspector["num_network_threads"], 1);
    eventsBeforeWakeup = ::config::internal::ValueConverter<int32_t>()(__inspector["events_before_wakeup"], 1);
}

InternalStorCommunicationmanagerType::Rpc::Compress::Type
InternalStorCommunicationmanagerType::Rpc::Compress::getType(const vespalib::string& name)
{
    if (name == "NONE") {
        return Type::NONE;
    } else if (name == "LZ4") {
        return Type::LZ4;
    } else if (name == "ZSTD") {
        return Type::ZSTD;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalStorCommunicationmanagerType::Rpc::Compress::getTypeName(Type t)
{
    switch (t) {
        case Type::NONE: return "NONE";
        case Type::LZ4: return "LZ4";
        case Type::ZSTD: return "ZSTD";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalStorCommunicationmanagerType::Rpc::Compress::Type InternalStorCommunicationmanagerType::Rpc::Compress::InternalTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalStorCommunicationmanagerType::Rpc::Compress::getType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalStorCommunicationmanagerType::Rpc::Compress::Type InternalStorCommunicationmanagerType::Rpc::Compress::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalStorCommunicationmanagerType::Rpc::Compress::getType(__inspector.asString().make_string());
}
InternalStorCommunicationmanagerType::Rpc::Compress::Type InternalStorCommunicationmanagerType::Rpc::Compress::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault) {
    if (__inspector.valid()) {
        return InternalStorCommunicationmanagerType::Rpc::Compress::getType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalStorCommunicationmanagerType::Rpc::Compress::Compress() noexcept
    : limit(1024),
      level(3),
      type(Type::LZ4)
{
}

InternalStorCommunicationmanagerType::Rpc::Compress::Compress(const Compress & __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Rpc::Compress & InternalStorCommunicationmanagerType::Rpc::Compress::operator = (const Compress & __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Rpc::Compress::Compress(Compress && __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Rpc::Compress & InternalStorCommunicationmanagerType::Rpc::Compress::operator = (Compress && __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Rpc::Compress::~Compress() = default; 
InternalStorCommunicationmanagerType::Rpc::Compress::Compress(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    limit = ConfigParser::parse<int32_t>("limit", __lines, 1024);
    ConfigParser::stripLinesForKey("limit", __remainingValuesToParse);
    level = ConfigParser::parse<int32_t>("level", __lines, 3);
    ConfigParser::stripLinesForKey("level", __remainingValuesToParse);
    type = getType(ConfigParser::parse<vespalib::string>("type", __lines, "LZ4"));
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
}

bool
InternalStorCommunicationmanagerType::Rpc::Compress::
operator==(const Compress& __rhs) const noexcept
{
    return (limit == __rhs.limit &&
            level == __rhs.level &&
            type == __rhs.type);
}

bool
InternalStorCommunicationmanagerType::Rpc::Compress::
operator!=(const Compress& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorCommunicationmanagerType::Rpc::Compress::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("limit");
        __c.setString("type", "int");
        __c.setLong("value", limit);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("level");
        __c.setString("type", "int");
        __c.setLong("value", level);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getTypeName(type)));
    }
}

InternalStorCommunicationmanagerType::Rpc::Compress::Compress(const vespalib::slime::Inspector & __inspector)
{
    limit = __inspector["limit"]["value"].asLong();
    level = __inspector["level"]["value"].asLong();
    type = getType(__inspector["type"]["value"].asString().make_string());
}

InternalStorCommunicationmanagerType::Rpc::Compress::Compress(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    limit = ::config::internal::ValueConverter<int32_t>()(__inspector["limit"], 1024);
    level = ::config::internal::ValueConverter<int32_t>()(__inspector["level"], 3);
    type = InternalTypeConverter()(__inspector["type"], Type::LZ4);
}

InternalStorCommunicationmanagerType::Rpc::Rpc() noexcept
    : numNetworkThreads(2),
      eventsBeforeWakeup(1),
      numTargetsPerNode(2),
      compress()
{
}

InternalStorCommunicationmanagerType::Rpc::Rpc(const Rpc & __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Rpc & InternalStorCommunicationmanagerType::Rpc::operator = (const Rpc & __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Rpc::Rpc(Rpc && __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Rpc & InternalStorCommunicationmanagerType::Rpc::operator = (Rpc && __rhs) noexcept = default;
InternalStorCommunicationmanagerType::Rpc::~Rpc() = default; 
InternalStorCommunicationmanagerType::Rpc::Rpc(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    numNetworkThreads = ConfigParser::parse<int32_t>("num_network_threads", __lines, 2);
    ConfigParser::stripLinesForKey("num_network_threads", __remainingValuesToParse);
    eventsBeforeWakeup = ConfigParser::parse<int32_t>("events_before_wakeup", __lines, 1);
    ConfigParser::stripLinesForKey("events_before_wakeup", __remainingValuesToParse);
    numTargetsPerNode = ConfigParser::parse<int32_t>("num_targets_per_node", __lines, 2);
    ConfigParser::stripLinesForKey("num_targets_per_node", __remainingValuesToParse);
    compress = ConfigParser::parseStruct<Compress>("compress", __lines);
    ConfigParser::stripLinesForKey("compress", __remainingValuesToParse);
}

bool
InternalStorCommunicationmanagerType::Rpc::operator==(const Rpc& __rhs) const noexcept
{
    return (numNetworkThreads == __rhs.numNetworkThreads &&
            eventsBeforeWakeup == __rhs.eventsBeforeWakeup &&
            numTargetsPerNode == __rhs.numTargetsPerNode &&
            compress == __rhs.compress);
}

bool
InternalStorCommunicationmanagerType::Rpc::operator!=(const Rpc& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorCommunicationmanagerType::Rpc::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("num_network_threads");
        __c.setString("type", "int");
        __c.setLong("value", numNetworkThreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("events_before_wakeup");
        __c.setString("type", "int");
        __c.setLong("value", eventsBeforeWakeup);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("num_targets_per_node");
        __c.setString("type", "int");
        __c.setLong("value", numTargetsPerNode);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("compress");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        compress.serialize(__c2);
    }
}

InternalStorCommunicationmanagerType::Rpc::Rpc(const vespalib::slime::Inspector & __inspector)
{
    numNetworkThreads = __inspector["num_network_threads"]["value"].asLong();
    eventsBeforeWakeup = __inspector["events_before_wakeup"]["value"].asLong();
    numTargetsPerNode = __inspector["num_targets_per_node"]["value"].asLong();
    compress = Compress(__inspector["compress"]["value"]);
}

InternalStorCommunicationmanagerType::Rpc::Rpc(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    numNetworkThreads = ::config::internal::ValueConverter<int32_t>()(__inspector["num_network_threads"], 2);
    eventsBeforeWakeup = ::config::internal::ValueConverter<int32_t>()(__inspector["events_before_wakeup"], 1);
    numTargetsPerNode = ::config::internal::ValueConverter<int32_t>()(__inspector["num_targets_per_node"], 2);
    compress = ::config::internal::ValueConverter<Compress>()(__inspector["compress"]);
}

InternalStorCommunicationmanagerType::InternalStorCommunicationmanagerType() noexcept
    : mbusport(-1),
      rpcport(6000),
      mbusDistributorNodeMaxPendingCount(5000),
      mbusContentNodeMaxPendingCount(0),
      mbusDistributorNodeMaxPendingSize(0),
      mbusContentNodeMaxPendingSize(0),
      mbus(),
      rpc()
{
}

InternalStorCommunicationmanagerType::InternalStorCommunicationmanagerType(const InternalStorCommunicationmanagerType & __rhs) noexcept = default;
InternalStorCommunicationmanagerType & InternalStorCommunicationmanagerType::operator =(const InternalStorCommunicationmanagerType & __rhs) noexcept = default;
InternalStorCommunicationmanagerType::InternalStorCommunicationmanagerType(InternalStorCommunicationmanagerType && __rhs) noexcept = default;
InternalStorCommunicationmanagerType & InternalStorCommunicationmanagerType::operator =(InternalStorCommunicationmanagerType && __rhs) noexcept = default;
InternalStorCommunicationmanagerType::~InternalStorCommunicationmanagerType() = default; 
InternalStorCommunicationmanagerType::InternalStorCommunicationmanagerType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        mbusport = ConfigParser::parse<int32_t>("mbusport", __lines, -1);
        ConfigParser::stripLinesForKey("mbusport", __remainingValuesToParse);
        rpcport = ConfigParser::parse<int32_t>("rpcport", __lines, 6000);
        ConfigParser::stripLinesForKey("rpcport", __remainingValuesToParse);
        mbusDistributorNodeMaxPendingCount = ConfigParser::parse<int32_t>("mbus_distributor_node_max_pending_count", __lines, 5000);
        ConfigParser::stripLinesForKey("mbus_distributor_node_max_pending_count", __remainingValuesToParse);
        mbusContentNodeMaxPendingCount = ConfigParser::parse<int32_t>("mbus_content_node_max_pending_count", __lines, 0);
        ConfigParser::stripLinesForKey("mbus_content_node_max_pending_count", __remainingValuesToParse);
        mbusDistributorNodeMaxPendingSize = ConfigParser::parse<int32_t>("mbus_distributor_node_max_pending_size", __lines, 0);
        ConfigParser::stripLinesForKey("mbus_distributor_node_max_pending_size", __remainingValuesToParse);
        mbusContentNodeMaxPendingSize = ConfigParser::parse<int32_t>("mbus_content_node_max_pending_size", __lines, 0);
        ConfigParser::stripLinesForKey("mbus_content_node_max_pending_size", __remainingValuesToParse);
        mbus = ConfigParser::parseStruct<Mbus>("mbus", __lines);
        ConfigParser::stripLinesForKey("mbus", __remainingValuesToParse);
        rpc = ConfigParser::parseStruct<Rpc>("rpc", __lines);
        ConfigParser::stripLinesForKey("rpc", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalStorCommunicationmanagerType::
operator==(const InternalStorCommunicationmanagerType& __rhs) const noexcept
{
    return (mbusport == __rhs.mbusport &&
            rpcport == __rhs.rpcport &&
            mbusDistributorNodeMaxPendingCount == __rhs.mbusDistributorNodeMaxPendingCount &&
            mbusContentNodeMaxPendingCount == __rhs.mbusContentNodeMaxPendingCount &&
            mbusDistributorNodeMaxPendingSize == __rhs.mbusDistributorNodeMaxPendingSize &&
            mbusContentNodeMaxPendingSize == __rhs.mbusContentNodeMaxPendingSize &&
            mbus == __rhs.mbus &&
            rpc == __rhs.rpc);
}

bool
InternalStorCommunicationmanagerType::
operator!=(const InternalStorCommunicationmanagerType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorCommunicationmanagerType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("mbusport");
        __c.setString("type", "int");
        __c.setLong("value", mbusport);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("rpcport");
        __c.setString("type", "int");
        __c.setLong("value", rpcport);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("mbus_distributor_node_max_pending_count");
        __c.setString("type", "int");
        __c.setLong("value", mbusDistributorNodeMaxPendingCount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("mbus_content_node_max_pending_count");
        __c.setString("type", "int");
        __c.setLong("value", mbusContentNodeMaxPendingCount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("mbus_distributor_node_max_pending_size");
        __c.setString("type", "int");
        __c.setLong("value", mbusDistributorNodeMaxPendingSize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("mbus_content_node_max_pending_size");
        __c.setString("type", "int");
        __c.setLong("value", mbusContentNodeMaxPendingSize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("mbus");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        mbus.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("rpc");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        rpc.serialize(__c2);
    }
}

InternalStorCommunicationmanagerType::InternalStorCommunicationmanagerType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    mbusport = __inspector["mbusport"]["value"].asLong();
    rpcport = __inspector["rpcport"]["value"].asLong();
    mbusDistributorNodeMaxPendingCount = __inspector["mbus_distributor_node_max_pending_count"]["value"].asLong();
    mbusContentNodeMaxPendingCount = __inspector["mbus_content_node_max_pending_count"]["value"].asLong();
    mbusDistributorNodeMaxPendingSize = __inspector["mbus_distributor_node_max_pending_size"]["value"].asLong();
    mbusContentNodeMaxPendingSize = __inspector["mbus_content_node_max_pending_size"]["value"].asLong();
    mbus = Mbus(__inspector["mbus"]["value"]);
    rpc = Rpc(__inspector["rpc"]["value"]);
}

InternalStorCommunicationmanagerType::InternalStorCommunicationmanagerType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    mbusport = ::config::internal::ValueConverter<int32_t>()(__inspector["mbusport"], -1);
    rpcport = ::config::internal::ValueConverter<int32_t>()(__inspector["rpcport"], 6000);
    mbusDistributorNodeMaxPendingCount = ::config::internal::ValueConverter<int32_t>()(__inspector["mbus_distributor_node_max_pending_count"], 5000);
    mbusContentNodeMaxPendingCount = ::config::internal::ValueConverter<int32_t>()(__inspector["mbus_content_node_max_pending_count"], 0);
    mbusDistributorNodeMaxPendingSize = ::config::internal::ValueConverter<int32_t>()(__inspector["mbus_distributor_node_max_pending_size"], 0);
    mbusContentNodeMaxPendingSize = ::config::internal::ValueConverter<int32_t>()(__inspector["mbus_content_node_max_pending_size"], 0);
    mbus = ::config::internal::ValueConverter<Mbus>()(__inspector["mbus"]);
    rpc = ::config::internal::ValueConverter<Rpc>()(__inspector["rpc"]);
}

} // namespace internal

} // namespace vespa::config::content::core
