/**
 * @class vespa::config::content::core::StorCommunicationmanager
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_CONTENT_CORE_STORCOMMUNICATIONMANAGER_H
#define CLOUD_CONFIG_VESPA_CONFIG_CONTENT_CORE_STORCOMMUNICATIONMANAGER_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::content::core {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalStorCommunicationmanagerType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalStorCommunicationmanagerType> UP;
    class Mbus {
    public:
        class Compress {
        public:
            enum class Type { NONE, LZ4, ZSTD };
            typedef std::vector<Type> TypeVector;
            typedef std::map<vespalib::string, Type> TypeMap;
            static Type getType(const vespalib::string&);
            static vespalib::string getTypeName(Type e);

            struct InternalTypeConverter {
                Type operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                Type operator()(const ::vespalib::slime::Inspector & __inspector);
                Type operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault);
            };
            Compress(const ::config::StringVector & __lines);
            Compress(const vespalib::slime::Inspector & __inspector);
            Compress(const ::config::ConfigPayload & __payload);
            Compress()  noexcept;
            Compress(const Compress & __rhs) noexcept;
            Compress & operator = (const Compress & __rhs) noexcept;
            Compress(Compress && __rhs) noexcept;
            Compress & operator = (Compress && __rhs) noexcept;
            ~Compress();

            bool operator==(const Compress& __rhs) const noexcept;
            bool operator!=(const Compress& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * Minimum size of packets to compress (0 means no compression)
             * TODO Common compression config for mbus and rpc, and consider
             * ZSTD as default
             */
            int32_t limit; // Default: 1024

            /** Compression level for packets */
            int32_t level; // Default: 3

            /** Compression type for packets. */
            Type type; // Default: LZ4
        };
        typedef std::vector<Compress> CompressVector;

        typedef std::map<vespalib::string, Compress> CompressMap;

        class Rpctargetcache {
        public:
            Rpctargetcache(const ::config::StringVector & __lines);
            Rpctargetcache(const vespalib::slime::Inspector & __inspector);
            Rpctargetcache(const ::config::ConfigPayload & __payload);
            Rpctargetcache()  noexcept;
            Rpctargetcache(const Rpctargetcache & __rhs) noexcept;
            Rpctargetcache & operator = (const Rpctargetcache & __rhs) noexcept;
            Rpctargetcache(Rpctargetcache && __rhs) noexcept;
            Rpctargetcache & operator = (Rpctargetcache && __rhs) noexcept;
            ~Rpctargetcache();

            bool operator==(const Rpctargetcache& __rhs) const noexcept;
            bool operator!=(const Rpctargetcache& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** TTL for rpc target cache */
            double ttl; // Default: 600
        };
        typedef std::vector<Rpctargetcache> RpctargetcacheVector;

        typedef std::map<vespalib::string, Rpctargetcache> RpctargetcacheMap;

        Mbus(const ::config::StringVector & __lines);
        Mbus(const vespalib::slime::Inspector & __inspector);
        Mbus(const ::config::ConfigPayload & __payload);
        Mbus()  noexcept;
        Mbus(const Mbus & __rhs) noexcept;
        Mbus & operator = (const Mbus & __rhs) noexcept;
        Mbus(Mbus && __rhs) noexcept;
        Mbus & operator = (Mbus && __rhs) noexcept;
        ~Mbus();

        bool operator==(const Mbus& __rhs) const noexcept;
        bool operator!=(const Mbus& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;
        Compress compress;
        Rpctargetcache rpctargetcache;

        /** Number of rpc targets per spec */
        int32_t numRpcTargets; // Default: 2

        /** Use tcpNoDelay for mbus network writes */
        bool tcpNoDelay; // Default: true

        /** Number of threads for network. */
        int32_t numNetworkThreads; // Default: 1

        /**
         * The number of events in the queue of a network (FNET) thread before
         * it is woken up.
         */
        int32_t eventsBeforeWakeup; // Default: 1
    };
    typedef std::vector<Mbus> MbusVector;

    typedef std::map<vespalib::string, Mbus> MbusMap;

    class Rpc {
    public:
        class Compress {
        public:
            enum class Type { NONE, LZ4, ZSTD };
            typedef std::vector<Type> TypeVector;
            typedef std::map<vespalib::string, Type> TypeMap;
            static Type getType(const vespalib::string&);
            static vespalib::string getTypeName(Type e);

            struct InternalTypeConverter {
                Type operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
                Type operator()(const ::vespalib::slime::Inspector & __inspector);
                Type operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault);
            };
            Compress(const ::config::StringVector & __lines);
            Compress(const vespalib::slime::Inspector & __inspector);
            Compress(const ::config::ConfigPayload & __payload);
            Compress()  noexcept;
            Compress(const Compress & __rhs) noexcept;
            Compress & operator = (const Compress & __rhs) noexcept;
            Compress(Compress && __rhs) noexcept;
            Compress & operator = (Compress && __rhs) noexcept;
            ~Compress();

            bool operator==(const Compress& __rhs) const noexcept;
            bool operator!=(const Compress& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /** Minimum size of packets to compress (0 means no compression) */
            int32_t limit; // Default: 1024

            /** Compression level for packets */
            int32_t level; // Default: 3

            /** Compression type for packets. */
            Type type; // Default: LZ4
        };
        typedef std::vector<Compress> CompressVector;

        typedef std::map<vespalib::string, Compress> CompressMap;

        Rpc(const ::config::StringVector & __lines);
        Rpc(const vespalib::slime::Inspector & __inspector);
        Rpc(const ::config::ConfigPayload & __payload);
        Rpc()  noexcept;
        Rpc(const Rpc & __rhs) noexcept;
        Rpc & operator = (const Rpc & __rhs) noexcept;
        Rpc(Rpc && __rhs) noexcept;
        Rpc & operator = (Rpc && __rhs) noexcept;
        ~Rpc();

        bool operator==(const Rpc& __rhs) const noexcept;
        bool operator!=(const Rpc& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * The number of network (FNET) threads used by the shared rpc resource.
         */
        int32_t numNetworkThreads; // Default: 2

        /**
         * The number of events in the queue of a network (FNET) thread before
         * it is woken up.
         */
        int32_t eventsBeforeWakeup; // Default: 1

        /**
         * The number of (FNET) RPC targets to use per node in the cluster.
         *  
         * The bucket id associated with a message is used to select the RPC
         * target.
         * This ensures the same RPC target is used for all messages to the same
         * bucket to the same node,
         * and the RPC target itself handles sequencing of these messages.
         * NB !! It is vital that this number is kept in sync with
         * stor-filestor:num_network_threads.
         * Only skilled vespa core developers should touch this.
         */
        int32_t numTargetsPerNode; // Default: 2
        Compress compress;
    };
    typedef std::vector<Rpc> RpcVector;

    typedef std::map<vespalib::string, Rpc> RpcMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalStorCommunicationmanagerType(const ::config::ConfigValue & __value);
    InternalStorCommunicationmanagerType(const ::config::ConfigDataBuffer & __value);
    InternalStorCommunicationmanagerType(const ::config::ConfigPayload & __payload);
    InternalStorCommunicationmanagerType()  noexcept;
    InternalStorCommunicationmanagerType(const InternalStorCommunicationmanagerType & __rhs) noexcept;
    InternalStorCommunicationmanagerType & operator = (const InternalStorCommunicationmanagerType & __rhs) noexcept;
    InternalStorCommunicationmanagerType(InternalStorCommunicationmanagerType && __rhs) noexcept;
    InternalStorCommunicationmanagerType & operator = (InternalStorCommunicationmanagerType && __rhs) noexcept;
    ~InternalStorCommunicationmanagerType();

    bool operator==(const InternalStorCommunicationmanagerType& __rhs) const noexcept;
    bool operator!=(const InternalStorCommunicationmanagerType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;

    int32_t mbusport; // Default: -1
    int32_t rpcport; // Default: 6000

    /**
     * Limits for max pending concurrent number of operations towards a node
     * before
     * MessageBus starts busy-bouncing messages. Distributor and content nodes
     * are
     * treated separately. 0 means no limit.
     */
    int32_t mbusDistributorNodeMaxPendingCount; // Default: 5000
    int32_t mbusContentNodeMaxPendingCount; // Default: 0

    /**
     * Limits for max total amount of memory (in bytes) used by operations
     * towards
     * a node before MessageBus starts busy-bouncing messages. Distributor and
     * content nodes are treated separately. 0 means no limit.
     */
    int32_t mbusDistributorNodeMaxPendingSize; // Default: 0
    int32_t mbusContentNodeMaxPendingSize; // Default: 0
    Mbus mbus;
    Rpc rpc;
};

} // namespace internal

typedef internal::InternalStorCommunicationmanagerType StorCommunicationmanagerConfigBuilder;
typedef const internal::InternalStorCommunicationmanagerType StorCommunicationmanagerConfig;

} // namespace vespa::config::content::core
#endif // VESPA_config_VESPA_CONFIG_CONTENT_CORE_STORCOMMUNICATIONMANAGER_H
