#include "config-stor-distributormanager.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalStorDistributormanagerType::CONFIG_DEF_MD5("98fa0e3c0f00d5b35842b1c6f611f485");
const vespalib::string InternalStorDistributormanagerType::CONFIG_DEF_NAME("stor-distributormanager");
const vespalib::string InternalStorDistributormanagerType::CONFIG_DEF_NAMESPACE("vespa.config.content.core");
const int64_t InternalStorDistributormanagerType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content.core",
"splitsize int default=16772216",
"splitcount int default=1024",
"joinsize int default=16000000",
"joincount int default=512",
"minsplitcount int default=16",
"garbagecollection.selectiontoremove string default=\"\"",
"garbagecollection.interval int default=0",
"inlinebucketsplitting bool default=true",
"maximum_nodes_per_merge int default=16",
"start_distributor_thread bool default=true restart",
"enable_join_for_sibling_less_buckets bool default=false",
"enable_inconsistent_join bool default=false",
"minimum_replica_counting_mode enum { TRUSTED, ANY } default=TRUSTED",
"disable_bucket_activation bool default=false",
"max_cluster_clock_skew_sec int default=1",
"inhibit_merge_sending_on_busy_node_duration_sec int default=1",
"allow_stale_reads_during_cluster_state_transitions bool default=false",
"simulated_db_pruning_latency_msec int default=0",
"simulated_db_merging_latency_msec int default=0",
"merge_operations_disabled bool default=false",
"use_weak_internal_read_consistency_for_client_gets bool default=false",
"max_consecutively_inhibited_maintenance_ticks int default=20",
"max_activation_inhibited_out_of_sync_groups int default=0",
"num_distributor_stripes int default=0 restart",
"enable_operation_cancellation bool default=false",
"symmetric_put_and_activate_replica_selection bool default=false",
"priority_merge_out_of_sync_copies int default=120",
"use_btree_database bool default=true",
};
const ::config::StringVector InternalStorDistributormanagerType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalStorDistributormanagerType::Garbagecollection::Garbagecollection() noexcept
    : selectiontoremove(""),
      interval(0)
{
}

InternalStorDistributormanagerType::Garbagecollection::Garbagecollection(const Garbagecollection & __rhs) noexcept = default;
InternalStorDistributormanagerType::Garbagecollection & InternalStorDistributormanagerType::Garbagecollection::operator = (const Garbagecollection & __rhs) noexcept = default;
InternalStorDistributormanagerType::Garbagecollection::Garbagecollection(Garbagecollection && __rhs) noexcept = default;
InternalStorDistributormanagerType::Garbagecollection & InternalStorDistributormanagerType::Garbagecollection::operator = (Garbagecollection && __rhs) noexcept = default;
InternalStorDistributormanagerType::Garbagecollection::~Garbagecollection() = default; 
InternalStorDistributormanagerType::Garbagecollection::Garbagecollection(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    selectiontoremove = ConfigParser::parse<vespalib::string>("selectiontoremove", __lines, "");
    ConfigParser::stripLinesForKey("selectiontoremove", __remainingValuesToParse);
    interval = ConfigParser::parse<int32_t>("interval", __lines, 0);
    ConfigParser::stripLinesForKey("interval", __remainingValuesToParse);
}

bool
InternalStorDistributormanagerType::Garbagecollection::
operator==(const Garbagecollection& __rhs) const noexcept
{
    return (selectiontoremove == __rhs.selectiontoremove &&
            interval == __rhs.interval);
}

bool
InternalStorDistributormanagerType::Garbagecollection::
operator!=(const Garbagecollection& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorDistributormanagerType::Garbagecollection::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("selectiontoremove");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(selectiontoremove));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("interval");
        __c.setString("type", "int");
        __c.setLong("value", interval);
    }
}

InternalStorDistributormanagerType::Garbagecollection::Garbagecollection(const vespalib::slime::Inspector & __inspector)
{
    selectiontoremove = __inspector["selectiontoremove"]["value"].asString().make_string();
    interval = __inspector["interval"]["value"].asLong();
}

InternalStorDistributormanagerType::Garbagecollection::Garbagecollection(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    selectiontoremove = ::config::internal::ValueConverter<vespalib::string>()(__inspector["selectiontoremove"], "");
    interval = ::config::internal::ValueConverter<int32_t>()(__inspector["interval"], 0);
}

InternalStorDistributormanagerType::MinimumReplicaCountingMode
InternalStorDistributormanagerType::getMinimumReplicaCountingMode(const vespalib::string& name)
{
    if (name == "TRUSTED") {
        return MinimumReplicaCountingMode::TRUSTED;
    } else if (name == "ANY") {
        return MinimumReplicaCountingMode::ANY;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalStorDistributormanagerType::getMinimumReplicaCountingModeName(MinimumReplicaCountingMode t)
{
    switch (t) {
        case MinimumReplicaCountingMode::TRUSTED: return "TRUSTED";
        case MinimumReplicaCountingMode::ANY: return "ANY";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalStorDistributormanagerType::MinimumReplicaCountingMode InternalStorDistributormanagerType::InternalMinimumReplicaCountingModeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalStorDistributormanagerType::getMinimumReplicaCountingMode(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalStorDistributormanagerType::MinimumReplicaCountingMode InternalStorDistributormanagerType::InternalMinimumReplicaCountingModeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalStorDistributormanagerType::getMinimumReplicaCountingMode(__inspector.asString().make_string());
}
InternalStorDistributormanagerType::MinimumReplicaCountingMode InternalStorDistributormanagerType::InternalMinimumReplicaCountingModeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, MinimumReplicaCountingMode __eDefault) {
    if (__inspector.valid()) {
        return InternalStorDistributormanagerType::getMinimumReplicaCountingMode(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalStorDistributormanagerType::InternalStorDistributormanagerType() noexcept
    : splitsize(16772216),
      splitcount(1024),
      joinsize(16000000),
      joincount(512),
      minsplitcount(16),
      garbagecollection(),
      inlinebucketsplitting(true),
      maximumNodesPerMerge(16),
      startDistributorThread(true),
      enableJoinForSiblingLessBuckets(false),
      enableInconsistentJoin(false),
      minimumReplicaCountingMode(MinimumReplicaCountingMode::TRUSTED),
      disableBucketActivation(false),
      maxClusterClockSkewSec(1),
      inhibitMergeSendingOnBusyNodeDurationSec(1),
      allowStaleReadsDuringClusterStateTransitions(false),
      simulatedDbPruningLatencyMsec(0),
      simulatedDbMergingLatencyMsec(0),
      mergeOperationsDisabled(false),
      useWeakInternalReadConsistencyForClientGets(false),
      maxConsecutivelyInhibitedMaintenanceTicks(20),
      maxActivationInhibitedOutOfSyncGroups(0),
      numDistributorStripes(0),
      enableOperationCancellation(false),
      symmetricPutAndActivateReplicaSelection(false),
      priorityMergeOutOfSyncCopies(120),
      useBtreeDatabase(true)
{
}

InternalStorDistributormanagerType::InternalStorDistributormanagerType(const InternalStorDistributormanagerType & __rhs) noexcept = default;
InternalStorDistributormanagerType & InternalStorDistributormanagerType::operator =(const InternalStorDistributormanagerType & __rhs) noexcept = default;
InternalStorDistributormanagerType::InternalStorDistributormanagerType(InternalStorDistributormanagerType && __rhs) noexcept = default;
InternalStorDistributormanagerType & InternalStorDistributormanagerType::operator =(InternalStorDistributormanagerType && __rhs) noexcept = default;
InternalStorDistributormanagerType::~InternalStorDistributormanagerType() = default; 
InternalStorDistributormanagerType::InternalStorDistributormanagerType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        splitsize = ConfigParser::parse<int32_t>("splitsize", __lines, 16772216);
        ConfigParser::stripLinesForKey("splitsize", __remainingValuesToParse);
        splitcount = ConfigParser::parse<int32_t>("splitcount", __lines, 1024);
        ConfigParser::stripLinesForKey("splitcount", __remainingValuesToParse);
        joinsize = ConfigParser::parse<int32_t>("joinsize", __lines, 16000000);
        ConfigParser::stripLinesForKey("joinsize", __remainingValuesToParse);
        joincount = ConfigParser::parse<int32_t>("joincount", __lines, 512);
        ConfigParser::stripLinesForKey("joincount", __remainingValuesToParse);
        minsplitcount = ConfigParser::parse<int32_t>("minsplitcount", __lines, 16);
        ConfigParser::stripLinesForKey("minsplitcount", __remainingValuesToParse);
        garbagecollection = ConfigParser::parseStruct<Garbagecollection>("garbagecollection", __lines);
        ConfigParser::stripLinesForKey("garbagecollection", __remainingValuesToParse);
        inlinebucketsplitting = ConfigParser::parse<bool>("inlinebucketsplitting", __lines, true);
        ConfigParser::stripLinesForKey("inlinebucketsplitting", __remainingValuesToParse);
        maximumNodesPerMerge = ConfigParser::parse<int32_t>("maximum_nodes_per_merge", __lines, 16);
        ConfigParser::stripLinesForKey("maximum_nodes_per_merge", __remainingValuesToParse);
        startDistributorThread = ConfigParser::parse<bool>("start_distributor_thread", __lines, true);
        ConfigParser::stripLinesForKey("start_distributor_thread", __remainingValuesToParse);
        enableJoinForSiblingLessBuckets = ConfigParser::parse<bool>("enable_join_for_sibling_less_buckets", __lines, false);
        ConfigParser::stripLinesForKey("enable_join_for_sibling_less_buckets", __remainingValuesToParse);
        enableInconsistentJoin = ConfigParser::parse<bool>("enable_inconsistent_join", __lines, false);
        ConfigParser::stripLinesForKey("enable_inconsistent_join", __remainingValuesToParse);
        minimumReplicaCountingMode = getMinimumReplicaCountingMode(ConfigParser::parse<vespalib::string>("minimum_replica_counting_mode", __lines, "TRUSTED"));
        ConfigParser::stripLinesForKey("minimum_replica_counting_mode", __remainingValuesToParse);
        disableBucketActivation = ConfigParser::parse<bool>("disable_bucket_activation", __lines, false);
        ConfigParser::stripLinesForKey("disable_bucket_activation", __remainingValuesToParse);
        maxClusterClockSkewSec = ConfigParser::parse<int32_t>("max_cluster_clock_skew_sec", __lines, 1);
        ConfigParser::stripLinesForKey("max_cluster_clock_skew_sec", __remainingValuesToParse);
        inhibitMergeSendingOnBusyNodeDurationSec = ConfigParser::parse<int32_t>("inhibit_merge_sending_on_busy_node_duration_sec", __lines, 1);
        ConfigParser::stripLinesForKey("inhibit_merge_sending_on_busy_node_duration_sec", __remainingValuesToParse);
        allowStaleReadsDuringClusterStateTransitions = ConfigParser::parse<bool>("allow_stale_reads_during_cluster_state_transitions", __lines, false);
        ConfigParser::stripLinesForKey("allow_stale_reads_during_cluster_state_transitions", __remainingValuesToParse);
        simulatedDbPruningLatencyMsec = ConfigParser::parse<int32_t>("simulated_db_pruning_latency_msec", __lines, 0);
        ConfigParser::stripLinesForKey("simulated_db_pruning_latency_msec", __remainingValuesToParse);
        simulatedDbMergingLatencyMsec = ConfigParser::parse<int32_t>("simulated_db_merging_latency_msec", __lines, 0);
        ConfigParser::stripLinesForKey("simulated_db_merging_latency_msec", __remainingValuesToParse);
        mergeOperationsDisabled = ConfigParser::parse<bool>("merge_operations_disabled", __lines, false);
        ConfigParser::stripLinesForKey("merge_operations_disabled", __remainingValuesToParse);
        useWeakInternalReadConsistencyForClientGets = ConfigParser::parse<bool>("use_weak_internal_read_consistency_for_client_gets", __lines, false);
        ConfigParser::stripLinesForKey("use_weak_internal_read_consistency_for_client_gets", __remainingValuesToParse);
        maxConsecutivelyInhibitedMaintenanceTicks = ConfigParser::parse<int32_t>("max_consecutively_inhibited_maintenance_ticks", __lines, 20);
        ConfigParser::stripLinesForKey("max_consecutively_inhibited_maintenance_ticks", __remainingValuesToParse);
        maxActivationInhibitedOutOfSyncGroups = ConfigParser::parse<int32_t>("max_activation_inhibited_out_of_sync_groups", __lines, 0);
        ConfigParser::stripLinesForKey("max_activation_inhibited_out_of_sync_groups", __remainingValuesToParse);
        numDistributorStripes = ConfigParser::parse<int32_t>("num_distributor_stripes", __lines, 0);
        ConfigParser::stripLinesForKey("num_distributor_stripes", __remainingValuesToParse);
        enableOperationCancellation = ConfigParser::parse<bool>("enable_operation_cancellation", __lines, false);
        ConfigParser::stripLinesForKey("enable_operation_cancellation", __remainingValuesToParse);
        symmetricPutAndActivateReplicaSelection = ConfigParser::parse<bool>("symmetric_put_and_activate_replica_selection", __lines, false);
        ConfigParser::stripLinesForKey("symmetric_put_and_activate_replica_selection", __remainingValuesToParse);
        priorityMergeOutOfSyncCopies = ConfigParser::parse<int32_t>("priority_merge_out_of_sync_copies", __lines, 120);
        ConfigParser::stripLinesForKey("priority_merge_out_of_sync_copies", __remainingValuesToParse);
        useBtreeDatabase = ConfigParser::parse<bool>("use_btree_database", __lines, true);
        ConfigParser::stripLinesForKey("use_btree_database", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalStorDistributormanagerType::
operator==(const InternalStorDistributormanagerType& __rhs) const noexcept
{
    return (splitsize == __rhs.splitsize &&
            splitcount == __rhs.splitcount &&
            joinsize == __rhs.joinsize &&
            joincount == __rhs.joincount &&
            minsplitcount == __rhs.minsplitcount &&
            garbagecollection == __rhs.garbagecollection &&
            inlinebucketsplitting == __rhs.inlinebucketsplitting &&
            maximumNodesPerMerge == __rhs.maximumNodesPerMerge &&
            startDistributorThread == __rhs.startDistributorThread &&
            enableJoinForSiblingLessBuckets == __rhs.enableJoinForSiblingLessBuckets &&
            enableInconsistentJoin == __rhs.enableInconsistentJoin &&
            minimumReplicaCountingMode == __rhs.minimumReplicaCountingMode &&
            disableBucketActivation == __rhs.disableBucketActivation &&
            maxClusterClockSkewSec == __rhs.maxClusterClockSkewSec &&
            inhibitMergeSendingOnBusyNodeDurationSec == __rhs.inhibitMergeSendingOnBusyNodeDurationSec &&
            allowStaleReadsDuringClusterStateTransitions == __rhs.allowStaleReadsDuringClusterStateTransitions &&
            simulatedDbPruningLatencyMsec == __rhs.simulatedDbPruningLatencyMsec &&
            simulatedDbMergingLatencyMsec == __rhs.simulatedDbMergingLatencyMsec &&
            mergeOperationsDisabled == __rhs.mergeOperationsDisabled &&
            useWeakInternalReadConsistencyForClientGets == __rhs.useWeakInternalReadConsistencyForClientGets &&
            maxConsecutivelyInhibitedMaintenanceTicks == __rhs.maxConsecutivelyInhibitedMaintenanceTicks &&
            maxActivationInhibitedOutOfSyncGroups == __rhs.maxActivationInhibitedOutOfSyncGroups &&
            numDistributorStripes == __rhs.numDistributorStripes &&
            enableOperationCancellation == __rhs.enableOperationCancellation &&
            symmetricPutAndActivateReplicaSelection == __rhs.symmetricPutAndActivateReplicaSelection &&
            priorityMergeOutOfSyncCopies == __rhs.priorityMergeOutOfSyncCopies &&
            useBtreeDatabase == __rhs.useBtreeDatabase);
}

bool
InternalStorDistributormanagerType::
operator!=(const InternalStorDistributormanagerType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorDistributormanagerType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("splitsize");
        __c.setString("type", "int");
        __c.setLong("value", splitsize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("splitcount");
        __c.setString("type", "int");
        __c.setLong("value", splitcount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("joinsize");
        __c.setString("type", "int");
        __c.setLong("value", joinsize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("joincount");
        __c.setString("type", "int");
        __c.setLong("value", joincount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("minsplitcount");
        __c.setString("type", "int");
        __c.setLong("value", minsplitcount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("garbagecollection");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        garbagecollection.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("inlinebucketsplitting");
        __c.setString("type", "bool");
        __c.setBool("value", inlinebucketsplitting);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maximum_nodes_per_merge");
        __c.setString("type", "int");
        __c.setLong("value", maximumNodesPerMerge);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("start_distributor_thread");
        __c.setString("type", "bool");
        __c.setBool("value", startDistributorThread);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_join_for_sibling_less_buckets");
        __c.setString("type", "bool");
        __c.setBool("value", enableJoinForSiblingLessBuckets);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_inconsistent_join");
        __c.setString("type", "bool");
        __c.setBool("value", enableInconsistentJoin);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("minimum_replica_counting_mode");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getMinimumReplicaCountingModeName(minimumReplicaCountingMode)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("disable_bucket_activation");
        __c.setString("type", "bool");
        __c.setBool("value", disableBucketActivation);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_cluster_clock_skew_sec");
        __c.setString("type", "int");
        __c.setLong("value", maxClusterClockSkewSec);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("inhibit_merge_sending_on_busy_node_duration_sec");
        __c.setString("type", "int");
        __c.setLong("value", inhibitMergeSendingOnBusyNodeDurationSec);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("allow_stale_reads_during_cluster_state_transitions");
        __c.setString("type", "bool");
        __c.setBool("value", allowStaleReadsDuringClusterStateTransitions);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("simulated_db_pruning_latency_msec");
        __c.setString("type", "int");
        __c.setLong("value", simulatedDbPruningLatencyMsec);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("simulated_db_merging_latency_msec");
        __c.setString("type", "int");
        __c.setLong("value", simulatedDbMergingLatencyMsec);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("merge_operations_disabled");
        __c.setString("type", "bool");
        __c.setBool("value", mergeOperationsDisabled);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("use_weak_internal_read_consistency_for_client_gets");
        __c.setString("type", "bool");
        __c.setBool("value", useWeakInternalReadConsistencyForClientGets);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_consecutively_inhibited_maintenance_ticks");
        __c.setString("type", "int");
        __c.setLong("value", maxConsecutivelyInhibitedMaintenanceTicks);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_activation_inhibited_out_of_sync_groups");
        __c.setString("type", "int");
        __c.setLong("value", maxActivationInhibitedOutOfSyncGroups);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("num_distributor_stripes");
        __c.setString("type", "int");
        __c.setLong("value", numDistributorStripes);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_operation_cancellation");
        __c.setString("type", "bool");
        __c.setBool("value", enableOperationCancellation);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("symmetric_put_and_activate_replica_selection");
        __c.setString("type", "bool");
        __c.setBool("value", symmetricPutAndActivateReplicaSelection);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("priority_merge_out_of_sync_copies");
        __c.setString("type", "int");
        __c.setLong("value", priorityMergeOutOfSyncCopies);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("use_btree_database");
        __c.setString("type", "bool");
        __c.setBool("value", useBtreeDatabase);
    }
}

InternalStorDistributormanagerType::InternalStorDistributormanagerType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    splitsize = __inspector["splitsize"]["value"].asLong();
    splitcount = __inspector["splitcount"]["value"].asLong();
    joinsize = __inspector["joinsize"]["value"].asLong();
    joincount = __inspector["joincount"]["value"].asLong();
    minsplitcount = __inspector["minsplitcount"]["value"].asLong();
    garbagecollection = Garbagecollection(__inspector["garbagecollection"]["value"]);
    inlinebucketsplitting = __inspector["inlinebucketsplitting"]["value"].asBool();
    maximumNodesPerMerge = __inspector["maximum_nodes_per_merge"]["value"].asLong();
    startDistributorThread = __inspector["start_distributor_thread"]["value"].asBool();
    enableJoinForSiblingLessBuckets = __inspector["enable_join_for_sibling_less_buckets"]["value"].asBool();
    enableInconsistentJoin = __inspector["enable_inconsistent_join"]["value"].asBool();
    minimumReplicaCountingMode = getMinimumReplicaCountingMode(__inspector["minimum_replica_counting_mode"]["value"].asString().make_string());
    disableBucketActivation = __inspector["disable_bucket_activation"]["value"].asBool();
    maxClusterClockSkewSec = __inspector["max_cluster_clock_skew_sec"]["value"].asLong();
    inhibitMergeSendingOnBusyNodeDurationSec = __inspector["inhibit_merge_sending_on_busy_node_duration_sec"]["value"].asLong();
    allowStaleReadsDuringClusterStateTransitions = __inspector["allow_stale_reads_during_cluster_state_transitions"]["value"].asBool();
    simulatedDbPruningLatencyMsec = __inspector["simulated_db_pruning_latency_msec"]["value"].asLong();
    simulatedDbMergingLatencyMsec = __inspector["simulated_db_merging_latency_msec"]["value"].asLong();
    mergeOperationsDisabled = __inspector["merge_operations_disabled"]["value"].asBool();
    useWeakInternalReadConsistencyForClientGets = __inspector["use_weak_internal_read_consistency_for_client_gets"]["value"].asBool();
    maxConsecutivelyInhibitedMaintenanceTicks = __inspector["max_consecutively_inhibited_maintenance_ticks"]["value"].asLong();
    maxActivationInhibitedOutOfSyncGroups = __inspector["max_activation_inhibited_out_of_sync_groups"]["value"].asLong();
    numDistributorStripes = __inspector["num_distributor_stripes"]["value"].asLong();
    enableOperationCancellation = __inspector["enable_operation_cancellation"]["value"].asBool();
    symmetricPutAndActivateReplicaSelection = __inspector["symmetric_put_and_activate_replica_selection"]["value"].asBool();
    priorityMergeOutOfSyncCopies = __inspector["priority_merge_out_of_sync_copies"]["value"].asLong();
    useBtreeDatabase = __inspector["use_btree_database"]["value"].asBool();
}

InternalStorDistributormanagerType::InternalStorDistributormanagerType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    splitsize = ::config::internal::ValueConverter<int32_t>()(__inspector["splitsize"], 16772216);
    splitcount = ::config::internal::ValueConverter<int32_t>()(__inspector["splitcount"], 1024);
    joinsize = ::config::internal::ValueConverter<int32_t>()(__inspector["joinsize"], 16000000);
    joincount = ::config::internal::ValueConverter<int32_t>()(__inspector["joincount"], 512);
    minsplitcount = ::config::internal::ValueConverter<int32_t>()(__inspector["minsplitcount"], 16);
    garbagecollection = ::config::internal::ValueConverter<Garbagecollection>()(__inspector["garbagecollection"]);
    inlinebucketsplitting = ::config::internal::ValueConverter<bool>()(__inspector["inlinebucketsplitting"], true);
    maximumNodesPerMerge = ::config::internal::ValueConverter<int32_t>()(__inspector["maximum_nodes_per_merge"], 16);
    startDistributorThread = ::config::internal::ValueConverter<bool>()(__inspector["start_distributor_thread"], true);
    enableJoinForSiblingLessBuckets = ::config::internal::ValueConverter<bool>()(__inspector["enable_join_for_sibling_less_buckets"], false);
    enableInconsistentJoin = ::config::internal::ValueConverter<bool>()(__inspector["enable_inconsistent_join"], false);
    minimumReplicaCountingMode = InternalMinimumReplicaCountingModeConverter()(__inspector["minimum_replica_counting_mode"], MinimumReplicaCountingMode::TRUSTED);
    disableBucketActivation = ::config::internal::ValueConverter<bool>()(__inspector["disable_bucket_activation"], false);
    maxClusterClockSkewSec = ::config::internal::ValueConverter<int32_t>()(__inspector["max_cluster_clock_skew_sec"], 1);
    inhibitMergeSendingOnBusyNodeDurationSec = ::config::internal::ValueConverter<int32_t>()(__inspector["inhibit_merge_sending_on_busy_node_duration_sec"], 1);
    allowStaleReadsDuringClusterStateTransitions = ::config::internal::ValueConverter<bool>()(__inspector["allow_stale_reads_during_cluster_state_transitions"], false);
    simulatedDbPruningLatencyMsec = ::config::internal::ValueConverter<int32_t>()(__inspector["simulated_db_pruning_latency_msec"], 0);
    simulatedDbMergingLatencyMsec = ::config::internal::ValueConverter<int32_t>()(__inspector["simulated_db_merging_latency_msec"], 0);
    mergeOperationsDisabled = ::config::internal::ValueConverter<bool>()(__inspector["merge_operations_disabled"], false);
    useWeakInternalReadConsistencyForClientGets = ::config::internal::ValueConverter<bool>()(__inspector["use_weak_internal_read_consistency_for_client_gets"], false);
    maxConsecutivelyInhibitedMaintenanceTicks = ::config::internal::ValueConverter<int32_t>()(__inspector["max_consecutively_inhibited_maintenance_ticks"], 20);
    maxActivationInhibitedOutOfSyncGroups = ::config::internal::ValueConverter<int32_t>()(__inspector["max_activation_inhibited_out_of_sync_groups"], 0);
    numDistributorStripes = ::config::internal::ValueConverter<int32_t>()(__inspector["num_distributor_stripes"], 0);
    enableOperationCancellation = ::config::internal::ValueConverter<bool>()(__inspector["enable_operation_cancellation"], false);
    symmetricPutAndActivateReplicaSelection = ::config::internal::ValueConverter<bool>()(__inspector["symmetric_put_and_activate_replica_selection"], false);
    priorityMergeOutOfSyncCopies = ::config::internal::ValueConverter<int32_t>()(__inspector["priority_merge_out_of_sync_copies"], 120);
    useBtreeDatabase = ::config::internal::ValueConverter<bool>()(__inspector["use_btree_database"], true);
}

} // namespace internal

} // namespace vespa::config::content::core
