#include "config-stor-prioritymapping.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalStorPrioritymappingType::CONFIG_DEF_MD5("ee21822864e96112f919db03f79499e2");
const vespalib::string InternalStorPrioritymappingType::CONFIG_DEF_NAME("stor-prioritymapping");
const vespalib::string InternalStorPrioritymappingType::CONFIG_DEF_NAMESPACE("vespa.config.content.core");
const int64_t InternalStorPrioritymappingType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content.core",
"highest int default=50",
"very_high int default=60",
"high_1 int default=70",
"high_2 int default=80",
"high_3 int default=90",
"normal_1 int default=100",
"normal_2 int default=110",
"normal_3 int default=120",
"normal_4 int default=130",
"normal_5 int default=140",
"normal_6 int default=150",
"low_1 int default=160",
"low_2 int default=170",
"low_3 int default=180",
"very_low int default=190",
"lowest int default=200",
};
const ::config::StringVector InternalStorPrioritymappingType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalStorPrioritymappingType::InternalStorPrioritymappingType() noexcept
    : highest(50),
      veryHigh(60),
      high1(70),
      high2(80),
      high3(90),
      normal1(100),
      normal2(110),
      normal3(120),
      normal4(130),
      normal5(140),
      normal6(150),
      low1(160),
      low2(170),
      low3(180),
      veryLow(190),
      lowest(200)
{
}

InternalStorPrioritymappingType::InternalStorPrioritymappingType(const InternalStorPrioritymappingType & __rhs) noexcept = default;
InternalStorPrioritymappingType & InternalStorPrioritymappingType::operator =(const InternalStorPrioritymappingType & __rhs) noexcept = default;
InternalStorPrioritymappingType::InternalStorPrioritymappingType(InternalStorPrioritymappingType && __rhs) noexcept = default;
InternalStorPrioritymappingType & InternalStorPrioritymappingType::operator =(InternalStorPrioritymappingType && __rhs) noexcept = default;
InternalStorPrioritymappingType::~InternalStorPrioritymappingType() = default; 
InternalStorPrioritymappingType::InternalStorPrioritymappingType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        highest = ConfigParser::parse<int32_t>("highest", __lines, 50);
        ConfigParser::stripLinesForKey("highest", __remainingValuesToParse);
        veryHigh = ConfigParser::parse<int32_t>("very_high", __lines, 60);
        ConfigParser::stripLinesForKey("very_high", __remainingValuesToParse);
        high1 = ConfigParser::parse<int32_t>("high_1", __lines, 70);
        ConfigParser::stripLinesForKey("high_1", __remainingValuesToParse);
        high2 = ConfigParser::parse<int32_t>("high_2", __lines, 80);
        ConfigParser::stripLinesForKey("high_2", __remainingValuesToParse);
        high3 = ConfigParser::parse<int32_t>("high_3", __lines, 90);
        ConfigParser::stripLinesForKey("high_3", __remainingValuesToParse);
        normal1 = ConfigParser::parse<int32_t>("normal_1", __lines, 100);
        ConfigParser::stripLinesForKey("normal_1", __remainingValuesToParse);
        normal2 = ConfigParser::parse<int32_t>("normal_2", __lines, 110);
        ConfigParser::stripLinesForKey("normal_2", __remainingValuesToParse);
        normal3 = ConfigParser::parse<int32_t>("normal_3", __lines, 120);
        ConfigParser::stripLinesForKey("normal_3", __remainingValuesToParse);
        normal4 = ConfigParser::parse<int32_t>("normal_4", __lines, 130);
        ConfigParser::stripLinesForKey("normal_4", __remainingValuesToParse);
        normal5 = ConfigParser::parse<int32_t>("normal_5", __lines, 140);
        ConfigParser::stripLinesForKey("normal_5", __remainingValuesToParse);
        normal6 = ConfigParser::parse<int32_t>("normal_6", __lines, 150);
        ConfigParser::stripLinesForKey("normal_6", __remainingValuesToParse);
        low1 = ConfigParser::parse<int32_t>("low_1", __lines, 160);
        ConfigParser::stripLinesForKey("low_1", __remainingValuesToParse);
        low2 = ConfigParser::parse<int32_t>("low_2", __lines, 170);
        ConfigParser::stripLinesForKey("low_2", __remainingValuesToParse);
        low3 = ConfigParser::parse<int32_t>("low_3", __lines, 180);
        ConfigParser::stripLinesForKey("low_3", __remainingValuesToParse);
        veryLow = ConfigParser::parse<int32_t>("very_low", __lines, 190);
        ConfigParser::stripLinesForKey("very_low", __remainingValuesToParse);
        lowest = ConfigParser::parse<int32_t>("lowest", __lines, 200);
        ConfigParser::stripLinesForKey("lowest", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalStorPrioritymappingType::
operator==(const InternalStorPrioritymappingType& __rhs) const noexcept
{
    return (highest == __rhs.highest &&
            veryHigh == __rhs.veryHigh &&
            high1 == __rhs.high1 &&
            high2 == __rhs.high2 &&
            high3 == __rhs.high3 &&
            normal1 == __rhs.normal1 &&
            normal2 == __rhs.normal2 &&
            normal3 == __rhs.normal3 &&
            normal4 == __rhs.normal4 &&
            normal5 == __rhs.normal5 &&
            normal6 == __rhs.normal6 &&
            low1 == __rhs.low1 &&
            low2 == __rhs.low2 &&
            low3 == __rhs.low3 &&
            veryLow == __rhs.veryLow &&
            lowest == __rhs.lowest);
}

bool
InternalStorPrioritymappingType::
operator!=(const InternalStorPrioritymappingType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorPrioritymappingType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("highest");
        __c.setString("type", "int");
        __c.setLong("value", highest);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("very_high");
        __c.setString("type", "int");
        __c.setLong("value", veryHigh);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("high_1");
        __c.setString("type", "int");
        __c.setLong("value", high1);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("high_2");
        __c.setString("type", "int");
        __c.setLong("value", high2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("high_3");
        __c.setString("type", "int");
        __c.setLong("value", high3);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("normal_1");
        __c.setString("type", "int");
        __c.setLong("value", normal1);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("normal_2");
        __c.setString("type", "int");
        __c.setLong("value", normal2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("normal_3");
        __c.setString("type", "int");
        __c.setLong("value", normal3);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("normal_4");
        __c.setString("type", "int");
        __c.setLong("value", normal4);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("normal_5");
        __c.setString("type", "int");
        __c.setLong("value", normal5);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("normal_6");
        __c.setString("type", "int");
        __c.setLong("value", normal6);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("low_1");
        __c.setString("type", "int");
        __c.setLong("value", low1);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("low_2");
        __c.setString("type", "int");
        __c.setLong("value", low2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("low_3");
        __c.setString("type", "int");
        __c.setLong("value", low3);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("very_low");
        __c.setString("type", "int");
        __c.setLong("value", veryLow);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("lowest");
        __c.setString("type", "int");
        __c.setLong("value", lowest);
    }
}

InternalStorPrioritymappingType::InternalStorPrioritymappingType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    highest = __inspector["highest"]["value"].asLong();
    veryHigh = __inspector["very_high"]["value"].asLong();
    high1 = __inspector["high_1"]["value"].asLong();
    high2 = __inspector["high_2"]["value"].asLong();
    high3 = __inspector["high_3"]["value"].asLong();
    normal1 = __inspector["normal_1"]["value"].asLong();
    normal2 = __inspector["normal_2"]["value"].asLong();
    normal3 = __inspector["normal_3"]["value"].asLong();
    normal4 = __inspector["normal_4"]["value"].asLong();
    normal5 = __inspector["normal_5"]["value"].asLong();
    normal6 = __inspector["normal_6"]["value"].asLong();
    low1 = __inspector["low_1"]["value"].asLong();
    low2 = __inspector["low_2"]["value"].asLong();
    low3 = __inspector["low_3"]["value"].asLong();
    veryLow = __inspector["very_low"]["value"].asLong();
    lowest = __inspector["lowest"]["value"].asLong();
}

InternalStorPrioritymappingType::InternalStorPrioritymappingType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    highest = ::config::internal::ValueConverter<int32_t>()(__inspector["highest"], 50);
    veryHigh = ::config::internal::ValueConverter<int32_t>()(__inspector["very_high"], 60);
    high1 = ::config::internal::ValueConverter<int32_t>()(__inspector["high_1"], 70);
    high2 = ::config::internal::ValueConverter<int32_t>()(__inspector["high_2"], 80);
    high3 = ::config::internal::ValueConverter<int32_t>()(__inspector["high_3"], 90);
    normal1 = ::config::internal::ValueConverter<int32_t>()(__inspector["normal_1"], 100);
    normal2 = ::config::internal::ValueConverter<int32_t>()(__inspector["normal_2"], 110);
    normal3 = ::config::internal::ValueConverter<int32_t>()(__inspector["normal_3"], 120);
    normal4 = ::config::internal::ValueConverter<int32_t>()(__inspector["normal_4"], 130);
    normal5 = ::config::internal::ValueConverter<int32_t>()(__inspector["normal_5"], 140);
    normal6 = ::config::internal::ValueConverter<int32_t>()(__inspector["normal_6"], 150);
    low1 = ::config::internal::ValueConverter<int32_t>()(__inspector["low_1"], 160);
    low2 = ::config::internal::ValueConverter<int32_t>()(__inspector["low_2"], 170);
    low3 = ::config::internal::ValueConverter<int32_t>()(__inspector["low_3"], 180);
    veryLow = ::config::internal::ValueConverter<int32_t>()(__inspector["very_low"], 190);
    lowest = ::config::internal::ValueConverter<int32_t>()(__inspector["lowest"], 200);
}

} // namespace internal

} // namespace vespa::config::content::core
