#include "config-stor-server.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalStorServerType::CONFIG_DEF_MD5("4174e559fc4cec3779451701590a7369");
const vespalib::string InternalStorServerType::CONFIG_DEF_NAME("stor-server");
const vespalib::string InternalStorServerType::CONFIG_DEF_NAMESPACE("vespa.config.content.core");
const int64_t InternalStorServerType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content.core",
"root_folder string restart",
"cluster_name string default=\"storage\" restart",
"node_index int default=0 restart",
"is_distributor bool restart",
"node_capacity double default=1.0 restart",
"max_merges_per_node int default=16",
"max_merge_queue_size int default=100",
"merge_throttling_policy.type enum { STATIC, DYNAMIC } default=STATIC",
"merge_throttling_policy.min_window_size int default=16",
"merge_throttling_policy.max_window_size int default=128",
"merge_throttling_policy.window_size_increment double default=2.0",
"merge_throttling_memory_limit.max_usage_bytes long default=0",
"merge_throttling_memory_limit.auto_phys_mem_scale_factor double default=0.03",
"merge_throttling_memory_limit.auto_lower_bound_bytes long default=134217728",
"merge_throttling_memory_limit.auto_upper_bound_bytes long default=2147483648",
"resource_exhaustion_merge_back_pressure_duration_secs double default=30.0",
"enable_dead_lock_detector bool default=false",
"enable_dead_lock_detector_warnings bool default=true",
"dead_lock_detector_timeout_slack double default=240",
"persistence_provider.type enum {STORAGE, DUMMY, RPC } default=STORAGE restart",
"persistence_provider.rpc.connectspec string default=\"tcp/localhost:27777\" restart",
"bucket_rechecking_chunk_size int default=100",
"simulated_bucket_request_latency_msec int default=0",
"content_node_bucket_db_stripe_bits int default=4 restart",
"write_pid_file_on_startup bool default=true",
"require_strictly_increasing_cluster_state_versions bool default=false",
};
const ::config::StringVector InternalStorServerType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalStorServerType::MergeThrottlingPolicy::Type
InternalStorServerType::MergeThrottlingPolicy::getType(const vespalib::string& name)
{
    if (name == "STATIC") {
        return Type::STATIC;
    } else if (name == "DYNAMIC") {
        return Type::DYNAMIC;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalStorServerType::MergeThrottlingPolicy::getTypeName(Type t)
{
    switch (t) {
        case Type::STATIC: return "STATIC";
        case Type::DYNAMIC: return "DYNAMIC";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalStorServerType::MergeThrottlingPolicy::Type InternalStorServerType::MergeThrottlingPolicy::InternalTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalStorServerType::MergeThrottlingPolicy::getType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalStorServerType::MergeThrottlingPolicy::Type InternalStorServerType::MergeThrottlingPolicy::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalStorServerType::MergeThrottlingPolicy::getType(__inspector.asString().make_string());
}
InternalStorServerType::MergeThrottlingPolicy::Type InternalStorServerType::MergeThrottlingPolicy::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault) {
    if (__inspector.valid()) {
        return InternalStorServerType::MergeThrottlingPolicy::getType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalStorServerType::MergeThrottlingPolicy::MergeThrottlingPolicy() noexcept
    : type(Type::STATIC),
      minWindowSize(16),
      maxWindowSize(128),
      windowSizeIncrement(2.0)
{
}

InternalStorServerType::MergeThrottlingPolicy::MergeThrottlingPolicy(const MergeThrottlingPolicy & __rhs) noexcept = default;
InternalStorServerType::MergeThrottlingPolicy & InternalStorServerType::MergeThrottlingPolicy::operator = (const MergeThrottlingPolicy & __rhs) noexcept = default;
InternalStorServerType::MergeThrottlingPolicy::MergeThrottlingPolicy(MergeThrottlingPolicy && __rhs) noexcept = default;
InternalStorServerType::MergeThrottlingPolicy & InternalStorServerType::MergeThrottlingPolicy::operator = (MergeThrottlingPolicy && __rhs) noexcept = default;
InternalStorServerType::MergeThrottlingPolicy::~MergeThrottlingPolicy() = default; 
InternalStorServerType::MergeThrottlingPolicy::MergeThrottlingPolicy(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    type = getType(ConfigParser::parse<vespalib::string>("type", __lines, "STATIC"));
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
    minWindowSize = ConfigParser::parse<int32_t>("min_window_size", __lines, 16);
    ConfigParser::stripLinesForKey("min_window_size", __remainingValuesToParse);
    maxWindowSize = ConfigParser::parse<int32_t>("max_window_size", __lines, 128);
    ConfigParser::stripLinesForKey("max_window_size", __remainingValuesToParse);
    windowSizeIncrement = ConfigParser::parse<double>("window_size_increment", __lines, 2.0);
    ConfigParser::stripLinesForKey("window_size_increment", __remainingValuesToParse);
}

bool
InternalStorServerType::MergeThrottlingPolicy::
operator==(const MergeThrottlingPolicy& __rhs) const noexcept
{
    return (type == __rhs.type &&
            minWindowSize == __rhs.minWindowSize &&
            maxWindowSize == __rhs.maxWindowSize &&
            windowSizeIncrement == __rhs.windowSizeIncrement);
}

bool
InternalStorServerType::MergeThrottlingPolicy::
operator!=(const MergeThrottlingPolicy& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorServerType::MergeThrottlingPolicy::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getTypeName(type)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("min_window_size");
        __c.setString("type", "int");
        __c.setLong("value", minWindowSize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_window_size");
        __c.setString("type", "int");
        __c.setLong("value", maxWindowSize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("window_size_increment");
        __c.setString("type", "double");
        __c.setDouble("value", windowSizeIncrement);
    }
}

InternalStorServerType::MergeThrottlingPolicy::MergeThrottlingPolicy(const vespalib::slime::Inspector & __inspector)
{
    type = getType(__inspector["type"]["value"].asString().make_string());
    minWindowSize = __inspector["min_window_size"]["value"].asLong();
    maxWindowSize = __inspector["max_window_size"]["value"].asLong();
    windowSizeIncrement = __inspector["window_size_increment"]["value"].asDouble();
}

InternalStorServerType::MergeThrottlingPolicy::MergeThrottlingPolicy(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    type = InternalTypeConverter()(__inspector["type"], Type::STATIC);
    minWindowSize = ::config::internal::ValueConverter<int32_t>()(__inspector["min_window_size"], 16);
    maxWindowSize = ::config::internal::ValueConverter<int32_t>()(__inspector["max_window_size"], 128);
    windowSizeIncrement = ::config::internal::ValueConverter<double>()(__inspector["window_size_increment"], 2.0);
}

InternalStorServerType::MergeThrottlingMemoryLimit::MergeThrottlingMemoryLimit() noexcept
    : maxUsageBytes(0),
      autoPhysMemScaleFactor(0.03),
      autoLowerBoundBytes(134217728),
      autoUpperBoundBytes(2147483648)
{
}

InternalStorServerType::MergeThrottlingMemoryLimit::MergeThrottlingMemoryLimit(const MergeThrottlingMemoryLimit & __rhs) noexcept = default;
InternalStorServerType::MergeThrottlingMemoryLimit & InternalStorServerType::MergeThrottlingMemoryLimit::operator = (const MergeThrottlingMemoryLimit & __rhs) noexcept = default;
InternalStorServerType::MergeThrottlingMemoryLimit::MergeThrottlingMemoryLimit(MergeThrottlingMemoryLimit && __rhs) noexcept = default;
InternalStorServerType::MergeThrottlingMemoryLimit & InternalStorServerType::MergeThrottlingMemoryLimit::operator = (MergeThrottlingMemoryLimit && __rhs) noexcept = default;
InternalStorServerType::MergeThrottlingMemoryLimit::~MergeThrottlingMemoryLimit() = default; 
InternalStorServerType::MergeThrottlingMemoryLimit::MergeThrottlingMemoryLimit(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    maxUsageBytes = ConfigParser::parse<int64_t>("max_usage_bytes", __lines, 0);
    ConfigParser::stripLinesForKey("max_usage_bytes", __remainingValuesToParse);
    autoPhysMemScaleFactor = ConfigParser::parse<double>("auto_phys_mem_scale_factor", __lines, 0.03);
    ConfigParser::stripLinesForKey("auto_phys_mem_scale_factor", __remainingValuesToParse);
    autoLowerBoundBytes = ConfigParser::parse<int64_t>("auto_lower_bound_bytes", __lines, 134217728);
    ConfigParser::stripLinesForKey("auto_lower_bound_bytes", __remainingValuesToParse);
    autoUpperBoundBytes = ConfigParser::parse<int64_t>("auto_upper_bound_bytes", __lines, 2147483648);
    ConfigParser::stripLinesForKey("auto_upper_bound_bytes", __remainingValuesToParse);
}

bool
InternalStorServerType::MergeThrottlingMemoryLimit::
operator==(const MergeThrottlingMemoryLimit& __rhs) const noexcept
{
    return (maxUsageBytes == __rhs.maxUsageBytes &&
            autoPhysMemScaleFactor == __rhs.autoPhysMemScaleFactor &&
            autoLowerBoundBytes == __rhs.autoLowerBoundBytes &&
            autoUpperBoundBytes == __rhs.autoUpperBoundBytes);
}

bool
InternalStorServerType::MergeThrottlingMemoryLimit::
operator!=(const MergeThrottlingMemoryLimit& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorServerType::MergeThrottlingMemoryLimit::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_usage_bytes");
        __c.setString("type", "long");
        __c.setLong("value", maxUsageBytes);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("auto_phys_mem_scale_factor");
        __c.setString("type", "double");
        __c.setDouble("value", autoPhysMemScaleFactor);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("auto_lower_bound_bytes");
        __c.setString("type", "long");
        __c.setLong("value", autoLowerBoundBytes);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("auto_upper_bound_bytes");
        __c.setString("type", "long");
        __c.setLong("value", autoUpperBoundBytes);
    }
}

InternalStorServerType::MergeThrottlingMemoryLimit::MergeThrottlingMemoryLimit(const vespalib::slime::Inspector & __inspector)
{
    maxUsageBytes = __inspector["max_usage_bytes"]["value"].asLong();
    autoPhysMemScaleFactor = __inspector["auto_phys_mem_scale_factor"]["value"].asDouble();
    autoLowerBoundBytes = __inspector["auto_lower_bound_bytes"]["value"].asLong();
    autoUpperBoundBytes = __inspector["auto_upper_bound_bytes"]["value"].asLong();
}

InternalStorServerType::MergeThrottlingMemoryLimit::MergeThrottlingMemoryLimit(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    maxUsageBytes = ::config::internal::ValueConverter<int64_t>()(__inspector["max_usage_bytes"], 0);
    autoPhysMemScaleFactor = ::config::internal::ValueConverter<double>()(__inspector["auto_phys_mem_scale_factor"], 0.03);
    autoLowerBoundBytes = ::config::internal::ValueConverter<int64_t>()(__inspector["auto_lower_bound_bytes"], 134217728);
    autoUpperBoundBytes = ::config::internal::ValueConverter<int64_t>()(__inspector["auto_upper_bound_bytes"], 2147483648);
}

InternalStorServerType::PersistenceProvider::Type
InternalStorServerType::PersistenceProvider::getType(const vespalib::string& name)
{
    if (name == "STORAGE") {
        return Type::STORAGE;
    } else if (name == "DUMMY") {
        return Type::DUMMY;
    } else if (name == "RPC") {
        return Type::RPC;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalStorServerType::PersistenceProvider::getTypeName(Type t)
{
    switch (t) {
        case Type::STORAGE: return "STORAGE";
        case Type::DUMMY: return "DUMMY";
        case Type::RPC: return "RPC";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalStorServerType::PersistenceProvider::Type InternalStorServerType::PersistenceProvider::InternalTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalStorServerType::PersistenceProvider::getType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalStorServerType::PersistenceProvider::Type InternalStorServerType::PersistenceProvider::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalStorServerType::PersistenceProvider::getType(__inspector.asString().make_string());
}
InternalStorServerType::PersistenceProvider::Type InternalStorServerType::PersistenceProvider::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault) {
    if (__inspector.valid()) {
        return InternalStorServerType::PersistenceProvider::getType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalStorServerType::PersistenceProvider::Rpc::Rpc() noexcept
    : connectspec("tcp/localhost:27777")
{
}

InternalStorServerType::PersistenceProvider::Rpc::Rpc(const Rpc & __rhs) noexcept = default;
InternalStorServerType::PersistenceProvider::Rpc & InternalStorServerType::PersistenceProvider::Rpc::operator = (const Rpc & __rhs) noexcept = default;
InternalStorServerType::PersistenceProvider::Rpc::Rpc(Rpc && __rhs) noexcept = default;
InternalStorServerType::PersistenceProvider::Rpc & InternalStorServerType::PersistenceProvider::Rpc::operator = (Rpc && __rhs) noexcept = default;
InternalStorServerType::PersistenceProvider::Rpc::~Rpc() = default; 
InternalStorServerType::PersistenceProvider::Rpc::Rpc(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    connectspec = ConfigParser::parse<vespalib::string>("connectspec", __lines, "tcp/localhost:27777");
    ConfigParser::stripLinesForKey("connectspec", __remainingValuesToParse);
}

bool
InternalStorServerType::PersistenceProvider::Rpc::
operator==(const Rpc& __rhs) const noexcept
{
    return (connectspec == __rhs.connectspec);
}

bool
InternalStorServerType::PersistenceProvider::Rpc::
operator!=(const Rpc& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorServerType::PersistenceProvider::Rpc::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("connectspec");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(connectspec));
    }
}

InternalStorServerType::PersistenceProvider::Rpc::Rpc(const vespalib::slime::Inspector & __inspector)
{
    connectspec = __inspector["connectspec"]["value"].asString().make_string();
}

InternalStorServerType::PersistenceProvider::Rpc::Rpc(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    connectspec = ::config::internal::ValueConverter<vespalib::string>()(__inspector["connectspec"], "tcp/localhost:27777");
}

InternalStorServerType::PersistenceProvider::PersistenceProvider() noexcept
    : type(Type::STORAGE),
      rpc()
{
}

InternalStorServerType::PersistenceProvider::PersistenceProvider(const PersistenceProvider & __rhs) noexcept = default;
InternalStorServerType::PersistenceProvider & InternalStorServerType::PersistenceProvider::operator = (const PersistenceProvider & __rhs) noexcept = default;
InternalStorServerType::PersistenceProvider::PersistenceProvider(PersistenceProvider && __rhs) noexcept = default;
InternalStorServerType::PersistenceProvider & InternalStorServerType::PersistenceProvider::operator = (PersistenceProvider && __rhs) noexcept = default;
InternalStorServerType::PersistenceProvider::~PersistenceProvider() = default; 
InternalStorServerType::PersistenceProvider::PersistenceProvider(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    type = getType(ConfigParser::parse<vespalib::string>("type", __lines, "STORAGE"));
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
    rpc = ConfigParser::parseStruct<Rpc>("rpc", __lines);
    ConfigParser::stripLinesForKey("rpc", __remainingValuesToParse);
}

bool
InternalStorServerType::PersistenceProvider::
operator==(const PersistenceProvider& __rhs) const noexcept
{
    return (type == __rhs.type &&
            rpc == __rhs.rpc);
}

bool
InternalStorServerType::PersistenceProvider::
operator!=(const PersistenceProvider& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorServerType::PersistenceProvider::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getTypeName(type)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("rpc");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        rpc.serialize(__c2);
    }
}

InternalStorServerType::PersistenceProvider::PersistenceProvider(const vespalib::slime::Inspector & __inspector)
{
    type = getType(__inspector["type"]["value"].asString().make_string());
    rpc = Rpc(__inspector["rpc"]["value"]);
}

InternalStorServerType::PersistenceProvider::PersistenceProvider(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    type = InternalTypeConverter()(__inspector["type"], Type::STORAGE);
    rpc = ::config::internal::ValueConverter<Rpc>()(__inspector["rpc"]);
}

InternalStorServerType::InternalStorServerType() noexcept
    : rootFolder(),
      clusterName("storage"),
      nodeIndex(0),
      isDistributor(false),
      nodeCapacity(1.0),
      maxMergesPerNode(16),
      maxMergeQueueSize(100),
      mergeThrottlingPolicy(),
      mergeThrottlingMemoryLimit(),
      resourceExhaustionMergeBackPressureDurationSecs(30.0),
      enableDeadLockDetector(false),
      enableDeadLockDetectorWarnings(true),
      deadLockDetectorTimeoutSlack(240),
      persistenceProvider(),
      bucketRecheckingChunkSize(100),
      simulatedBucketRequestLatencyMsec(0),
      contentNodeBucketDbStripeBits(4),
      writePidFileOnStartup(true),
      requireStrictlyIncreasingClusterStateVersions(false)
{
}

InternalStorServerType::InternalStorServerType(const InternalStorServerType & __rhs) noexcept = default;
InternalStorServerType & InternalStorServerType::operator =(const InternalStorServerType & __rhs) noexcept = default;
InternalStorServerType::InternalStorServerType(InternalStorServerType && __rhs) noexcept = default;
InternalStorServerType & InternalStorServerType::operator =(InternalStorServerType && __rhs) noexcept = default;
InternalStorServerType::~InternalStorServerType() = default; 
InternalStorServerType::InternalStorServerType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        rootFolder = ConfigParser::parse<vespalib::string>("root_folder", __lines);
        ConfigParser::stripLinesForKey("root_folder", __remainingValuesToParse);
        clusterName = ConfigParser::parse<vespalib::string>("cluster_name", __lines, "storage");
        ConfigParser::stripLinesForKey("cluster_name", __remainingValuesToParse);
        nodeIndex = ConfigParser::parse<int32_t>("node_index", __lines, 0);
        ConfigParser::stripLinesForKey("node_index", __remainingValuesToParse);
        isDistributor = ConfigParser::parse<bool>("is_distributor", __lines);
        ConfigParser::stripLinesForKey("is_distributor", __remainingValuesToParse);
        nodeCapacity = ConfigParser::parse<double>("node_capacity", __lines, 1.0);
        ConfigParser::stripLinesForKey("node_capacity", __remainingValuesToParse);
        maxMergesPerNode = ConfigParser::parse<int32_t>("max_merges_per_node", __lines, 16);
        ConfigParser::stripLinesForKey("max_merges_per_node", __remainingValuesToParse);
        maxMergeQueueSize = ConfigParser::parse<int32_t>("max_merge_queue_size", __lines, 100);
        ConfigParser::stripLinesForKey("max_merge_queue_size", __remainingValuesToParse);
        mergeThrottlingPolicy = ConfigParser::parseStruct<MergeThrottlingPolicy>("merge_throttling_policy", __lines);
        ConfigParser::stripLinesForKey("merge_throttling_policy", __remainingValuesToParse);
        mergeThrottlingMemoryLimit = ConfigParser::parseStruct<MergeThrottlingMemoryLimit>("merge_throttling_memory_limit", __lines);
        ConfigParser::stripLinesForKey("merge_throttling_memory_limit", __remainingValuesToParse);
        resourceExhaustionMergeBackPressureDurationSecs = ConfigParser::parse<double>("resource_exhaustion_merge_back_pressure_duration_secs", __lines, 30.0);
        ConfigParser::stripLinesForKey("resource_exhaustion_merge_back_pressure_duration_secs", __remainingValuesToParse);
        enableDeadLockDetector = ConfigParser::parse<bool>("enable_dead_lock_detector", __lines, false);
        ConfigParser::stripLinesForKey("enable_dead_lock_detector", __remainingValuesToParse);
        enableDeadLockDetectorWarnings = ConfigParser::parse<bool>("enable_dead_lock_detector_warnings", __lines, true);
        ConfigParser::stripLinesForKey("enable_dead_lock_detector_warnings", __remainingValuesToParse);
        deadLockDetectorTimeoutSlack = ConfigParser::parse<double>("dead_lock_detector_timeout_slack", __lines, 240);
        ConfigParser::stripLinesForKey("dead_lock_detector_timeout_slack", __remainingValuesToParse);
        persistenceProvider = ConfigParser::parseStruct<PersistenceProvider>("persistence_provider", __lines);
        ConfigParser::stripLinesForKey("persistence_provider", __remainingValuesToParse);
        bucketRecheckingChunkSize = ConfigParser::parse<int32_t>("bucket_rechecking_chunk_size", __lines, 100);
        ConfigParser::stripLinesForKey("bucket_rechecking_chunk_size", __remainingValuesToParse);
        simulatedBucketRequestLatencyMsec = ConfigParser::parse<int32_t>("simulated_bucket_request_latency_msec", __lines, 0);
        ConfigParser::stripLinesForKey("simulated_bucket_request_latency_msec", __remainingValuesToParse);
        contentNodeBucketDbStripeBits = ConfigParser::parse<int32_t>("content_node_bucket_db_stripe_bits", __lines, 4);
        ConfigParser::stripLinesForKey("content_node_bucket_db_stripe_bits", __remainingValuesToParse);
        writePidFileOnStartup = ConfigParser::parse<bool>("write_pid_file_on_startup", __lines, true);
        ConfigParser::stripLinesForKey("write_pid_file_on_startup", __remainingValuesToParse);
        requireStrictlyIncreasingClusterStateVersions = ConfigParser::parse<bool>("require_strictly_increasing_cluster_state_versions", __lines, false);
        ConfigParser::stripLinesForKey("require_strictly_increasing_cluster_state_versions", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalStorServerType::operator==(const InternalStorServerType& __rhs) const noexcept
{
    return (rootFolder == __rhs.rootFolder &&
            clusterName == __rhs.clusterName &&
            nodeIndex == __rhs.nodeIndex &&
            isDistributor == __rhs.isDistributor &&
            nodeCapacity == __rhs.nodeCapacity &&
            maxMergesPerNode == __rhs.maxMergesPerNode &&
            maxMergeQueueSize == __rhs.maxMergeQueueSize &&
            mergeThrottlingPolicy == __rhs.mergeThrottlingPolicy &&
            mergeThrottlingMemoryLimit == __rhs.mergeThrottlingMemoryLimit &&
            resourceExhaustionMergeBackPressureDurationSecs == __rhs.resourceExhaustionMergeBackPressureDurationSecs &&
            enableDeadLockDetector == __rhs.enableDeadLockDetector &&
            enableDeadLockDetectorWarnings == __rhs.enableDeadLockDetectorWarnings &&
            deadLockDetectorTimeoutSlack == __rhs.deadLockDetectorTimeoutSlack &&
            persistenceProvider == __rhs.persistenceProvider &&
            bucketRecheckingChunkSize == __rhs.bucketRecheckingChunkSize &&
            simulatedBucketRequestLatencyMsec == __rhs.simulatedBucketRequestLatencyMsec &&
            contentNodeBucketDbStripeBits == __rhs.contentNodeBucketDbStripeBits &&
            writePidFileOnStartup == __rhs.writePidFileOnStartup &&
            requireStrictlyIncreasingClusterStateVersions == __rhs.requireStrictlyIncreasingClusterStateVersions);
}

bool
InternalStorServerType::operator!=(const InternalStorServerType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorServerType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("root_folder");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(rootFolder));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("cluster_name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(clusterName));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("node_index");
        __c.setString("type", "int");
        __c.setLong("value", nodeIndex);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("is_distributor");
        __c.setString("type", "bool");
        __c.setBool("value", isDistributor);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("node_capacity");
        __c.setString("type", "double");
        __c.setDouble("value", nodeCapacity);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_merges_per_node");
        __c.setString("type", "int");
        __c.setLong("value", maxMergesPerNode);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_merge_queue_size");
        __c.setString("type", "int");
        __c.setLong("value", maxMergeQueueSize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("merge_throttling_policy");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        mergeThrottlingPolicy.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("merge_throttling_memory_limit");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        mergeThrottlingMemoryLimit.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("resource_exhaustion_merge_back_pressure_duration_secs");
        __c.setString("type", "double");
        __c.setDouble("value", resourceExhaustionMergeBackPressureDurationSecs);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_dead_lock_detector");
        __c.setString("type", "bool");
        __c.setBool("value", enableDeadLockDetector);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("enable_dead_lock_detector_warnings");
        __c.setString("type", "bool");
        __c.setBool("value", enableDeadLockDetectorWarnings);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("dead_lock_detector_timeout_slack");
        __c.setString("type", "double");
        __c.setDouble("value", deadLockDetectorTimeoutSlack);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("persistence_provider");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        persistenceProvider.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("bucket_rechecking_chunk_size");
        __c.setString("type", "int");
        __c.setLong("value", bucketRecheckingChunkSize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("simulated_bucket_request_latency_msec");
        __c.setString("type", "int");
        __c.setLong("value", simulatedBucketRequestLatencyMsec);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("content_node_bucket_db_stripe_bits");
        __c.setString("type", "int");
        __c.setLong("value", contentNodeBucketDbStripeBits);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("write_pid_file_on_startup");
        __c.setString("type", "bool");
        __c.setBool("value", writePidFileOnStartup);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("require_strictly_increasing_cluster_state_versions");
        __c.setString("type", "bool");
        __c.setBool("value", requireStrictlyIncreasingClusterStateVersions);
    }
}

InternalStorServerType::InternalStorServerType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    rootFolder = __inspector["root_folder"]["value"].asString().make_string();
    clusterName = __inspector["cluster_name"]["value"].asString().make_string();
    nodeIndex = __inspector["node_index"]["value"].asLong();
    isDistributor = __inspector["is_distributor"]["value"].asBool();
    nodeCapacity = __inspector["node_capacity"]["value"].asDouble();
    maxMergesPerNode = __inspector["max_merges_per_node"]["value"].asLong();
    maxMergeQueueSize = __inspector["max_merge_queue_size"]["value"].asLong();
    mergeThrottlingPolicy = MergeThrottlingPolicy(__inspector["merge_throttling_policy"]["value"]);
    mergeThrottlingMemoryLimit = MergeThrottlingMemoryLimit(__inspector["merge_throttling_memory_limit"]["value"]);
    resourceExhaustionMergeBackPressureDurationSecs = __inspector["resource_exhaustion_merge_back_pressure_duration_secs"]["value"].asDouble();
    enableDeadLockDetector = __inspector["enable_dead_lock_detector"]["value"].asBool();
    enableDeadLockDetectorWarnings = __inspector["enable_dead_lock_detector_warnings"]["value"].asBool();
    deadLockDetectorTimeoutSlack = __inspector["dead_lock_detector_timeout_slack"]["value"].asDouble();
    persistenceProvider = PersistenceProvider(__inspector["persistence_provider"]["value"]);
    bucketRecheckingChunkSize = __inspector["bucket_rechecking_chunk_size"]["value"].asLong();
    simulatedBucketRequestLatencyMsec = __inspector["simulated_bucket_request_latency_msec"]["value"].asLong();
    contentNodeBucketDbStripeBits = __inspector["content_node_bucket_db_stripe_bits"]["value"].asLong();
    writePidFileOnStartup = __inspector["write_pid_file_on_startup"]["value"].asBool();
    requireStrictlyIncreasingClusterStateVersions = __inspector["require_strictly_increasing_cluster_state_versions"]["value"].asBool();
}

InternalStorServerType::InternalStorServerType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    rootFolder = ::config::internal::ValueConverter<vespalib::string>()("root_folder", __inspector["root_folder"]);
    clusterName = ::config::internal::ValueConverter<vespalib::string>()(__inspector["cluster_name"], "storage");
    nodeIndex = ::config::internal::ValueConverter<int32_t>()(__inspector["node_index"], 0);
    isDistributor = ::config::internal::ValueConverter<bool>()("is_distributor", __inspector["is_distributor"]);
    nodeCapacity = ::config::internal::ValueConverter<double>()(__inspector["node_capacity"], 1.0);
    maxMergesPerNode = ::config::internal::ValueConverter<int32_t>()(__inspector["max_merges_per_node"], 16);
    maxMergeQueueSize = ::config::internal::ValueConverter<int32_t>()(__inspector["max_merge_queue_size"], 100);
    mergeThrottlingPolicy = ::config::internal::ValueConverter<MergeThrottlingPolicy>()(__inspector["merge_throttling_policy"]);
    mergeThrottlingMemoryLimit = ::config::internal::ValueConverter<MergeThrottlingMemoryLimit>()(__inspector["merge_throttling_memory_limit"]);
    resourceExhaustionMergeBackPressureDurationSecs = ::config::internal::ValueConverter<double>()(__inspector["resource_exhaustion_merge_back_pressure_duration_secs"], 30.0);
    enableDeadLockDetector = ::config::internal::ValueConverter<bool>()(__inspector["enable_dead_lock_detector"], false);
    enableDeadLockDetectorWarnings = ::config::internal::ValueConverter<bool>()(__inspector["enable_dead_lock_detector_warnings"], true);
    deadLockDetectorTimeoutSlack = ::config::internal::ValueConverter<double>()(__inspector["dead_lock_detector_timeout_slack"], 240);
    persistenceProvider = ::config::internal::ValueConverter<PersistenceProvider>()(__inspector["persistence_provider"]);
    bucketRecheckingChunkSize = ::config::internal::ValueConverter<int32_t>()(__inspector["bucket_rechecking_chunk_size"], 100);
    simulatedBucketRequestLatencyMsec = ::config::internal::ValueConverter<int32_t>()(__inspector["simulated_bucket_request_latency_msec"], 0);
    contentNodeBucketDbStripeBits = ::config::internal::ValueConverter<int32_t>()(__inspector["content_node_bucket_db_stripe_bits"], 4);
    writePidFileOnStartup = ::config::internal::ValueConverter<bool>()(__inspector["write_pid_file_on_startup"], true);
    requireStrictlyIncreasingClusterStateVersions = ::config::internal::ValueConverter<bool>()(__inspector["require_strictly_increasing_cluster_state_versions"], false);
}

} // namespace internal

} // namespace vespa::config::content::core
