/**
 * @class vespa::config::content::core::StorServer
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_CONTENT_CORE_STORSERVER_H
#define CLOUD_CONFIG_VESPA_CONFIG_CONTENT_CORE_STORSERVER_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::content::core {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalStorServerType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalStorServerType> UP;
    class MergeThrottlingPolicy {
    public:
        enum class Type { STATIC, DYNAMIC };
        typedef std::vector<Type> TypeVector;
        typedef std::map<vespalib::string, Type> TypeMap;
        static Type getType(const vespalib::string&);
        static vespalib::string getTypeName(Type e);

        struct InternalTypeConverter {
            Type operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Type operator()(const ::vespalib::slime::Inspector & __inspector);
            Type operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault);
        };
        MergeThrottlingPolicy(const ::config::StringVector & __lines);
        MergeThrottlingPolicy(const vespalib::slime::Inspector & __inspector);
        MergeThrottlingPolicy(const ::config::ConfigPayload & __payload);
        MergeThrottlingPolicy()  noexcept;
        MergeThrottlingPolicy(const MergeThrottlingPolicy & __rhs) noexcept;
        MergeThrottlingPolicy & operator = (const MergeThrottlingPolicy & __rhs) noexcept;
        MergeThrottlingPolicy(MergeThrottlingPolicy && __rhs) noexcept;
        MergeThrottlingPolicy & operator = (MergeThrottlingPolicy && __rhs) noexcept;
        ~MergeThrottlingPolicy();

        bool operator==(const MergeThrottlingPolicy& __rhs) const noexcept;
        bool operator!=(const MergeThrottlingPolicy& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * Chooses the throttling policy used to control the active merge window
         * size
         * of the MergeThrottler component.
         */
        Type type; // Default: STATIC

        /** Only used if merge_throttling_policy.type == DYNAMIC: */
        int32_t minWindowSize; // Default: 16
        int32_t maxWindowSize; // Default: 128
        double windowSizeIncrement; // Default: 2.0
    };
    typedef std::vector<MergeThrottlingPolicy> MergeThrottlingPolicyVector;

    typedef std::map<vespalib::string, MergeThrottlingPolicy> MergeThrottlingPolicyMap;

    class MergeThrottlingMemoryLimit {
    public:
        MergeThrottlingMemoryLimit(const ::config::StringVector & __lines);
        MergeThrottlingMemoryLimit(const vespalib::slime::Inspector & __inspector);
        MergeThrottlingMemoryLimit(const ::config::ConfigPayload & __payload);
        MergeThrottlingMemoryLimit()  noexcept;
        MergeThrottlingMemoryLimit(const MergeThrottlingMemoryLimit & __rhs) noexcept;
        MergeThrottlingMemoryLimit & operator = (const MergeThrottlingMemoryLimit & __rhs) noexcept;
        MergeThrottlingMemoryLimit(MergeThrottlingMemoryLimit && __rhs) noexcept;
        MergeThrottlingMemoryLimit & operator = (MergeThrottlingMemoryLimit && __rhs) noexcept;
        ~MergeThrottlingMemoryLimit();

        bool operator==(const MergeThrottlingMemoryLimit& __rhs) const noexcept;
        bool operator!=(const MergeThrottlingMemoryLimit& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * If positive, nodes enforce a soft limit on the estimated amount of
         * memory that
         * can be used by merges touching a particular content node. If a merge
         * arrives
         * to the node that would violate the soft limit, it will be bounced
         * with BUSY.
         * Note that this also counts merges where the node is part of the
         * source-only set,
         * since these use memory when/if data is read from the local node.
         *  
         * Semantics:
         * > 0 explicit limit in bytes
         * == 0 limit automatically deduced by content node
         * < 0 unlimited (legacy behavior)
         */
        int64_t maxUsageBytes; // Default: 0

        /**
         * If merge_throttling_memory_limit.max_usage_bytes == 0, this factor is
         * used
         * as a multiplier to automatically deduce a memory limit for merges on
         * the
         * content node. Note that the result of this multiplication is capped
         * at both
         * ends by the auto_(lower|upper)_bound_bytes config values.
         *  
         * Default: 3% of physical memory
         */
        double autoPhysMemScaleFactor; // Default: 0.03

        /**
         * The absolute minimum memory limit that can be set when automatically
         * deducing the limit from physical memory on the node.
         *  
         * Default: 128MiB
         */
        int64_t autoLowerBoundBytes; // Default: 134217728

        /**
         * The absolute maximum memory limit that can be set when automatically
         * deducing the limit from physical memory on the node.
         *  
         * Default: 2GiB
         */
        int64_t autoUpperBoundBytes; // Default: 2147483648
    };
    typedef std::vector<MergeThrottlingMemoryLimit> MergeThrottlingMemoryLimitVector;

    typedef std::map<vespalib::string, MergeThrottlingMemoryLimit> MergeThrottlingMemoryLimitMap;

    class PersistenceProvider {
    public:
        enum class Type { STORAGE, DUMMY, RPC };
        typedef std::vector<Type> TypeVector;
        typedef std::map<vespalib::string, Type> TypeMap;
        static Type getType(const vespalib::string&);
        static vespalib::string getTypeName(Type e);

        struct InternalTypeConverter {
            Type operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Type operator()(const ::vespalib::slime::Inspector & __inspector);
            Type operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault);
        };
        class Rpc {
        public:
            Rpc(const ::config::StringVector & __lines);
            Rpc(const vespalib::slime::Inspector & __inspector);
            Rpc(const ::config::ConfigPayload & __payload);
            Rpc()  noexcept;
            Rpc(const Rpc & __rhs) noexcept;
            Rpc & operator = (const Rpc & __rhs) noexcept;
            Rpc(Rpc && __rhs) noexcept;
            Rpc & operator = (Rpc && __rhs) noexcept;
            ~Rpc();

            bool operator==(const Rpc& __rhs) const noexcept;
            bool operator!=(const Rpc& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;
            vespalib::string connectspec; // Default: "tcp/localhost:27777"
        };
        typedef std::vector<Rpc> RpcVector;

        typedef std::map<vespalib::string, Rpc> RpcMap;

        PersistenceProvider(const ::config::StringVector & __lines);
        PersistenceProvider(const vespalib::slime::Inspector & __inspector);
        PersistenceProvider(const ::config::ConfigPayload & __payload);
        PersistenceProvider()  noexcept;
        PersistenceProvider(const PersistenceProvider & __rhs) noexcept;
        PersistenceProvider & operator = (const PersistenceProvider & __rhs) noexcept;
        PersistenceProvider(PersistenceProvider && __rhs) noexcept;
        PersistenceProvider & operator = (PersistenceProvider && __rhs) noexcept;
        ~PersistenceProvider();

        bool operator==(const PersistenceProvider& __rhs) const noexcept;
        bool operator!=(const PersistenceProvider& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /** Configure persistence provider. Temporary here to test. */
        Type type; // Default: STORAGE
        Rpc rpc;
    };
    typedef std::vector<PersistenceProvider> PersistenceProviderVector;

    typedef std::map<vespalib::string, PersistenceProvider> PersistenceProviderMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalStorServerType(const ::config::ConfigValue & __value);
    InternalStorServerType(const ::config::ConfigDataBuffer & __value);
    InternalStorServerType(const ::config::ConfigPayload & __payload);
    InternalStorServerType()  noexcept;
    InternalStorServerType(const InternalStorServerType & __rhs) noexcept;
    InternalStorServerType & operator = (const InternalStorServerType & __rhs) noexcept;
    InternalStorServerType(InternalStorServerType && __rhs) noexcept;
    InternalStorServerType & operator = (InternalStorServerType && __rhs) noexcept;
    ~InternalStorServerType();

    bool operator==(const InternalStorServerType& __rhs) const noexcept;
    bool operator!=(const InternalStorServerType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /**
     * Root directory for all files related to this storage node.
     * Will typically be "$VESPA_HOME/var/db/vespa/vds/<cluster>/<nodetype>/<ind-
     * ex>
     */
    vespalib::string rootFolder;

    /** VDS cluster */
    vespalib::string clusterName; // Default: "storage"

    /**
     * The index of this node. Each node of the same type in the same cluster
     * need
     * to have unique indexes. This should not be changed, as this is what we
     * use
     * to identify the node, and to decide what data should be on it.
     */
    int32_t nodeIndex; // Default: 0

    /**
     * Set whether this is a distributor or a storage node. This will decide
     * what
     * storage links are set up.
     */
    bool isDistributor;

    /**
     * Capacity of the node. How much data and load this node will get relative
     * to
     * other nodes.
     */
    double nodeCapacity; // Default: 1.0

    /**
     * The upper bound of merges that any storage node can have active.
     * A merge operation will be chained through all nodes involved in the
     * merge, only actually starting the operation when every node has
     * allowed it to pass through.
     * NOTE: these config values are _not_ used if merge_throttling_policy.type
     * is configured to DYNAMIC (see below).
     */
    int32_t maxMergesPerNode; // Default: 16
    int32_t maxMergeQueueSize; // Default: 100
    MergeThrottlingPolicy mergeThrottlingPolicy;
    MergeThrottlingMemoryLimit mergeThrottlingMemoryLimit;

    /**
     * If the persistence provider indicates that it has exhausted one or more
     * of its internal resources during a mutating operation, new merges will
     * be bounced for this duration. Not allowing further merges helps take
     * load off the node while it e.g. compacts its data stores or memory in
     * the background.
     * Note: this does not affect merges where the current node is marked as
     * "source only", as merges do not cause mutations on such nodes.
     */
    double resourceExhaustionMergeBackPressureDurationSecs; // Default: 30.0

    /**
     * Whether the deadlock detector should be enabled or not. If disabled, it
     * will
     * still run, but it will never actually abort the process it is running in.
     */
    bool enableDeadLockDetector; // Default: false

    /**
     * Whether to enable deadlock detector warnings in log or not. If enabled,
     * warnings will be written even if dead lock detecting is not enabled.
     */
    bool enableDeadLockDetectorWarnings; // Default: true

    /**
     * Each thread registers how often it will at minimum register ticks (given
     * that
     * the system is not overloaded. If you are running Vespa on overloaded
     * nodes,
     * you can use this slack timeout to add to the thread timeouts in order to
     * allow for more slack before dead lock detector kicks in. The value is in
     * seconds.
     */
    double deadLockDetectorTimeoutSlack; // Default: 240
    PersistenceProvider persistenceProvider;

    /**
     * When the content layer receives a set of changed buckets from the
     * persistence
     * layer, it must recheck all of these. Each such recheck results in an
     * operation scheduled against the persistence queust and since the total
     * number of buckets to recheck may reach hundreds of thousands in a large
     * system, we send these in chunks to avoid saturating the queues with
     * operations.
     */
    int32_t bucketRecheckingChunkSize; // Default: 100

    /**
     * If greater than zero, simulates added latency caused by CPU processing
     * during
     * full bucket info requests. The latency is added per batch of operations
     * processed.
     * Only useful for testing!
     */
    int32_t simulatedBucketRequestLatencyMsec; // Default: 0

    /**
     * If non-zero, the bucket DB will be striped into 2^bits sub-databases,
     * each handling
     * a disjoint subset of the node's buckets, in order to reduce locking
     * contention.
     * Max value is unspecified, but will be clamped internally.
     */
    int32_t contentNodeBucketDbStripeBits; // Default: 4

    /**
     * Iff set, a special `pidfile` file is written under the node's root
     * directory upon
     * startup containing the PID of the running process.
     */
    bool writePidFileOnStartup; // Default: true

    /**
     * Iff true, received cluster state versions that are lower than the current
     * active
     * (or pending to be active) version on the node will be explicitly
     * rejected. This
     * prevents race conditions caused by multiple cluster controllers believing
     * they
     * are the leader during overlapping time intervals, as only the most recent
     * leader
     * is able to increment the current state version in ZooKeeper, but the old
     * controller
     * may still attempt to publish its old state.
     */
    bool requireStrictlyIncreasingClusterStateVersions; // Default: false
};

} // namespace internal

typedef internal::InternalStorServerType StorServerConfigBuilder;
typedef const internal::InternalStorServerType StorServerConfig;

} // namespace vespa::config::content::core
#endif // VESPA_config_VESPA_CONFIG_CONTENT_CORE_STORSERVER_H
