#include "config-stor-visitordispatcher.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalStorVisitordispatcherType::CONFIG_DEF_MD5("92d10d053137ebd51c88d1f1db9edf10");
const vespalib::string InternalStorVisitordispatcherType::CONFIG_DEF_NAME("stor-visitordispatcher");
const vespalib::string InternalStorVisitordispatcherType::CONFIG_DEF_NAMESPACE("vespa.config.content.core");
const int64_t InternalStorVisitordispatcherType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content.core",
"maxvisitorspernodeperclientvisitor int default=16",
"minbucketspervisitor int default=1",
};
const ::config::StringVector InternalStorVisitordispatcherType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalStorVisitordispatcherType::InternalStorVisitordispatcherType() noexcept
    : maxvisitorspernodeperclientvisitor(16),
      minbucketspervisitor(1)
{
}

InternalStorVisitordispatcherType::InternalStorVisitordispatcherType(const InternalStorVisitordispatcherType & __rhs) noexcept = default;
InternalStorVisitordispatcherType & InternalStorVisitordispatcherType::operator =(const InternalStorVisitordispatcherType & __rhs) noexcept = default;
InternalStorVisitordispatcherType::InternalStorVisitordispatcherType(InternalStorVisitordispatcherType && __rhs) noexcept = default;
InternalStorVisitordispatcherType & InternalStorVisitordispatcherType::operator =(InternalStorVisitordispatcherType && __rhs) noexcept = default;
InternalStorVisitordispatcherType::~InternalStorVisitordispatcherType() = default; 
InternalStorVisitordispatcherType::InternalStorVisitordispatcherType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        maxvisitorspernodeperclientvisitor = ConfigParser::parse<int32_t>("maxvisitorspernodeperclientvisitor", __lines, 16);
        ConfigParser::stripLinesForKey("maxvisitorspernodeperclientvisitor", __remainingValuesToParse);
        minbucketspervisitor = ConfigParser::parse<int32_t>("minbucketspervisitor", __lines, 1);
        ConfigParser::stripLinesForKey("minbucketspervisitor", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalStorVisitordispatcherType::
operator==(const InternalStorVisitordispatcherType& __rhs) const noexcept
{
    return (maxvisitorspernodeperclientvisitor == __rhs.maxvisitorspernodeperclientvisitor &&
            minbucketspervisitor == __rhs.minbucketspervisitor);
}

bool
InternalStorVisitordispatcherType::
operator!=(const InternalStorVisitordispatcherType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorVisitordispatcherType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxvisitorspernodeperclientvisitor");
        __c.setString("type", "int");
        __c.setLong("value", maxvisitorspernodeperclientvisitor);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("minbucketspervisitor");
        __c.setString("type", "int");
        __c.setLong("value", minbucketspervisitor);
    }
}

InternalStorVisitordispatcherType::InternalStorVisitordispatcherType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    maxvisitorspernodeperclientvisitor = __inspector["maxvisitorspernodeperclientvisitor"]["value"].asLong();
    minbucketspervisitor = __inspector["minbucketspervisitor"]["value"].asLong();
}

InternalStorVisitordispatcherType::InternalStorVisitordispatcherType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    maxvisitorspernodeperclientvisitor = ::config::internal::ValueConverter<int32_t>()(__inspector["maxvisitorspernodeperclientvisitor"], 16);
    minbucketspervisitor = ::config::internal::ValueConverter<int32_t>()(__inspector["minbucketspervisitor"], 1);
}

} // namespace internal

} // namespace vespa::config::content::core
