#include "config-stor-visitor.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::content::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalStorVisitorType::CONFIG_DEF_MD5("dc821286779da6a40e6ddcc671bd2b02");
const vespalib::string InternalStorVisitorType::CONFIG_DEF_NAME("stor-visitor");
const vespalib::string InternalStorVisitorType::CONFIG_DEF_NAMESPACE("vespa.config.content.core");
const int64_t InternalStorVisitorType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.content.core",
"visitorthreads int default=16 restart",
"defaultparalleliterators int default=8",
"maxconcurrentvisitors int default=64",
"maxconcurrentvisitors_fixed int default=16",
"maxconcurrentvisitors_variable int default=64",
"maxvisitorqueuesize int default=1024",
"visitor_memory_usage_limit int default=25165824",
};
const ::config::StringVector InternalStorVisitorType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalStorVisitorType::InternalStorVisitorType() noexcept
    : visitorthreads(16),
      defaultparalleliterators(8),
      maxconcurrentvisitors(64),
      maxconcurrentvisitorsFixed(16),
      maxconcurrentvisitorsVariable(64),
      maxvisitorqueuesize(1024),
      visitorMemoryUsageLimit(25165824)
{
}

InternalStorVisitorType::InternalStorVisitorType(const InternalStorVisitorType & __rhs) noexcept = default;
InternalStorVisitorType & InternalStorVisitorType::operator =(const InternalStorVisitorType & __rhs) noexcept = default;
InternalStorVisitorType::InternalStorVisitorType(InternalStorVisitorType && __rhs) noexcept = default;
InternalStorVisitorType & InternalStorVisitorType::operator =(InternalStorVisitorType && __rhs) noexcept = default;
InternalStorVisitorType::~InternalStorVisitorType() = default; 
InternalStorVisitorType::InternalStorVisitorType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        visitorthreads = ConfigParser::parse<int32_t>("visitorthreads", __lines, 16);
        ConfigParser::stripLinesForKey("visitorthreads", __remainingValuesToParse);
        defaultparalleliterators = ConfigParser::parse<int32_t>("defaultparalleliterators", __lines, 8);
        ConfigParser::stripLinesForKey("defaultparalleliterators", __remainingValuesToParse);
        maxconcurrentvisitors = ConfigParser::parse<int32_t>("maxconcurrentvisitors", __lines, 64);
        ConfigParser::stripLinesForKey("maxconcurrentvisitors", __remainingValuesToParse);
        maxconcurrentvisitorsFixed = ConfigParser::parse<int32_t>("maxconcurrentvisitors_fixed", __lines, 16);
        ConfigParser::stripLinesForKey("maxconcurrentvisitors_fixed", __remainingValuesToParse);
        maxconcurrentvisitorsVariable = ConfigParser::parse<int32_t>("maxconcurrentvisitors_variable", __lines, 64);
        ConfigParser::stripLinesForKey("maxconcurrentvisitors_variable", __remainingValuesToParse);
        maxvisitorqueuesize = ConfigParser::parse<int32_t>("maxvisitorqueuesize", __lines, 1024);
        ConfigParser::stripLinesForKey("maxvisitorqueuesize", __remainingValuesToParse);
        visitorMemoryUsageLimit = ConfigParser::parse<int32_t>("visitor_memory_usage_limit", __lines, 25165824);
        ConfigParser::stripLinesForKey("visitor_memory_usage_limit", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalStorVisitorType::operator==(const InternalStorVisitorType& __rhs) const noexcept
{
    return (visitorthreads == __rhs.visitorthreads &&
            defaultparalleliterators == __rhs.defaultparalleliterators &&
            maxconcurrentvisitors == __rhs.maxconcurrentvisitors &&
            maxconcurrentvisitorsFixed == __rhs.maxconcurrentvisitorsFixed &&
            maxconcurrentvisitorsVariable == __rhs.maxconcurrentvisitorsVariable &&
            maxvisitorqueuesize == __rhs.maxvisitorqueuesize &&
            visitorMemoryUsageLimit == __rhs.visitorMemoryUsageLimit);
}

bool
InternalStorVisitorType::operator!=(const InternalStorVisitorType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalStorVisitorType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("visitorthreads");
        __c.setString("type", "int");
        __c.setLong("value", visitorthreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("defaultparalleliterators");
        __c.setString("type", "int");
        __c.setLong("value", defaultparalleliterators);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxconcurrentvisitors");
        __c.setString("type", "int");
        __c.setLong("value", maxconcurrentvisitors);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxconcurrentvisitors_fixed");
        __c.setString("type", "int");
        __c.setLong("value", maxconcurrentvisitorsFixed);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxconcurrentvisitors_variable");
        __c.setString("type", "int");
        __c.setLong("value", maxconcurrentvisitorsVariable);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxvisitorqueuesize");
        __c.setString("type", "int");
        __c.setLong("value", maxvisitorqueuesize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("visitor_memory_usage_limit");
        __c.setString("type", "int");
        __c.setLong("value", visitorMemoryUsageLimit);
    }
}

InternalStorVisitorType::InternalStorVisitorType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    visitorthreads = __inspector["visitorthreads"]["value"].asLong();
    defaultparalleliterators = __inspector["defaultparalleliterators"]["value"].asLong();
    maxconcurrentvisitors = __inspector["maxconcurrentvisitors"]["value"].asLong();
    maxconcurrentvisitorsFixed = __inspector["maxconcurrentvisitors_fixed"]["value"].asLong();
    maxconcurrentvisitorsVariable = __inspector["maxconcurrentvisitors_variable"]["value"].asLong();
    maxvisitorqueuesize = __inspector["maxvisitorqueuesize"]["value"].asLong();
    visitorMemoryUsageLimit = __inspector["visitor_memory_usage_limit"]["value"].asLong();
}

InternalStorVisitorType::InternalStorVisitorType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    visitorthreads = ::config::internal::ValueConverter<int32_t>()(__inspector["visitorthreads"], 16);
    defaultparalleliterators = ::config::internal::ValueConverter<int32_t>()(__inspector["defaultparalleliterators"], 8);
    maxconcurrentvisitors = ::config::internal::ValueConverter<int32_t>()(__inspector["maxconcurrentvisitors"], 64);
    maxconcurrentvisitorsFixed = ::config::internal::ValueConverter<int32_t>()(__inspector["maxconcurrentvisitors_fixed"], 16);
    maxconcurrentvisitorsVariable = ::config::internal::ValueConverter<int32_t>()(__inspector["maxconcurrentvisitors_variable"], 64);
    maxvisitorqueuesize = ::config::internal::ValueConverter<int32_t>()(__inspector["maxvisitorqueuesize"], 1024);
    visitorMemoryUsageLimit = ::config::internal::ValueConverter<int32_t>()(__inspector["visitor_memory_usage_limit"], 25165824);
}

} // namespace internal

} // namespace vespa::config::content::core
