/**
 * @class vespa::config::content::core::StorVisitor
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_CONTENT_CORE_STORVISITOR_H
#define CLOUD_CONFIG_VESPA_CONFIG_CONTENT_CORE_STORVISITOR_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::content::core {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalStorVisitorType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalStorVisitorType> UP;
    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalStorVisitorType(const ::config::ConfigValue & __value);
    InternalStorVisitorType(const ::config::ConfigDataBuffer & __value);
    InternalStorVisitorType(const ::config::ConfigPayload & __payload);
    InternalStorVisitorType()  noexcept;
    InternalStorVisitorType(const InternalStorVisitorType & __rhs) noexcept;
    InternalStorVisitorType & operator = (const InternalStorVisitorType & __rhs) noexcept;
    InternalStorVisitorType(InternalStorVisitorType && __rhs) noexcept;
    InternalStorVisitorType & operator = (InternalStorVisitorType && __rhs) noexcept;
    ~InternalStorVisitorType();

    bool operator==(const InternalStorVisitorType& __rhs) const noexcept;
    bool operator!=(const InternalStorVisitorType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /**
     * Number of separate threads that runs visitors.
     * Keep in sync with #stor-filestor:num_visitor_threads
     */
    int32_t visitorthreads; // Default: 16

    /**
     * The number of buckets that are visited in parallel in a visitor visiting
     * multiple buckets. Default is 8, meaning if you send a create visitor to
     * visit
     * 100 buckets, 8 of them will be visited in parallel.
     */
    int32_t defaultparalleliterators; // Default: 8

    /** Max concurrent visitors (legacy) */
    int32_t maxconcurrentvisitors; // Default: 64

    /**
     * Priority-based max concurrent visitors. Fixed is the total number of
     * concurrent visitors that can run for any priorities. Variable
     * increases the concurrency limit for higher priorities, the limit
     * being linear with a messages priority. Example: if Fixed is 16
     * and Variable is 64, maxconcurrent for a pri 255 message is 16 and
     * maxconcurrent for a pri 0 message is 16+64=80.
     * If fixed is left as 0, it will take the value of maxconcurrentvisitors,
     * allowing backwards compatability
     */
    int32_t maxconcurrentvisitorsFixed; // Default: 16
    int32_t maxconcurrentvisitorsVariable; // Default: 64

    /** Max size of visitor priority queue */
    int32_t maxvisitorqueuesize; // Default: 1024

    /**
     * Limit of memory used _per visitor_ in bytes.
     * Due to optimistic parallelization, it is possible for this limit to be
     * initially violated when the visitor is first started. This can happen
     * since
     * the visitor does not know the size of the bucket contents before fetching
     * any data from it and it will do so based on parallelization factors
     * specified
     * in the CreateVisitor command. If 3 buckets are initially visited in
     * parallel
     * and these both contain a single 100 MiB document, the memory usage of the
     * visitor will peak at 300 MiB even if the configured limit is e.g. 20 MiB.
     * Default value is set to 20 MiB, which attempts to keep a reasonably safe
     * level in the face of a default number of max concurrent visitors (64).
     */
    int32_t visitorMemoryUsageLimit; // Default: 25165824
};

} // namespace internal

typedef internal::InternalStorVisitorType StorVisitorConfigBuilder;
typedef const internal::InternalStorVisitorType StorVisitorConfig;

} // namespace vespa::config::content::core
#endif // VESPA_config_VESPA_CONFIG_CONTENT_CORE_STORVISITOR_H
