#include "config-vsm.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search::vsm {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalVsmType::CONFIG_DEF_MD5("418301f17a08ba858242b3094b00b8dc");
const vespalib::string InternalVsmType::CONFIG_DEF_NAME("vsm");
const vespalib::string InternalVsmType::CONFIG_DEF_NAMESPACE("vespa.config.search.vsm");
const int64_t InternalVsmType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search.vsm",
"doctype reference",
"storagecfg reference",
"vsmfields reference",
};
const ::config::StringVector InternalVsmType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalVsmType::InternalVsmType() noexcept
    : doctype(),
      storagecfg(),
      vsmfields()
{
}

InternalVsmType::InternalVsmType(const InternalVsmType & __rhs) noexcept = default;
InternalVsmType & InternalVsmType::operator =(const InternalVsmType & __rhs) noexcept = default;
InternalVsmType::InternalVsmType(InternalVsmType && __rhs) noexcept = default;
InternalVsmType & InternalVsmType::operator =(InternalVsmType && __rhs) noexcept = default;
InternalVsmType::~InternalVsmType() = default; 
InternalVsmType::InternalVsmType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        doctype = ConfigParser::parse<vespalib::string>("doctype", __lines);
        ConfigParser::stripLinesForKey("doctype", __remainingValuesToParse);
        storagecfg = ConfigParser::parse<vespalib::string>("storagecfg", __lines);
        ConfigParser::stripLinesForKey("storagecfg", __remainingValuesToParse);
        vsmfields = ConfigParser::parse<vespalib::string>("vsmfields", __lines);
        ConfigParser::stripLinesForKey("vsmfields", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalVsmType::operator==(const InternalVsmType& __rhs) const noexcept
{
    return (doctype == __rhs.doctype &&
            storagecfg == __rhs.storagecfg &&
            vsmfields == __rhs.vsmfields);
}

bool
InternalVsmType::operator!=(const InternalVsmType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalVsmType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("doctype");
        __c.setString("type", "reference");
        __c.setString("value", vespalib::Memory(doctype));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("storagecfg");
        __c.setString("type", "reference");
        __c.setString("value", vespalib::Memory(storagecfg));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("vsmfields");
        __c.setString("type", "reference");
        __c.setString("value", vespalib::Memory(vsmfields));
    }
}

InternalVsmType::InternalVsmType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    doctype = __inspector["doctype"]["value"].asString().make_string();
    storagecfg = __inspector["storagecfg"]["value"].asString().make_string();
    vsmfields = __inspector["vsmfields"]["value"].asString().make_string();
}

InternalVsmType::InternalVsmType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    doctype = ::config::internal::ValueConverter<vespalib::string>()("doctype", __inspector["doctype"]);
    storagecfg = ::config::internal::ValueConverter<vespalib::string>()("storagecfg", __inspector["storagecfg"]);
    vsmfields = ::config::internal::ValueConverter<vespalib::string>()("vsmfields", __inspector["vsmfields"]);
}

} // namespace internal

} // namespace vespa::config::search::vsm
