#include "config-vsmfields.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search::vsm {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalVsmfieldsType::CONFIG_DEF_MD5("410f27ed212871762b984e362e694c94");
const vespalib::string InternalVsmfieldsType::CONFIG_DEF_NAME("vsmfields");
const vespalib::string InternalVsmfieldsType::CONFIG_DEF_NAMESPACE("vespa.config.search.vsm");
const int64_t InternalVsmfieldsType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search.vsm",
"documentverificationlevel int default=0",
"searchall int default=1",
"fieldspec[].name string",
"fieldspec[].searchmethod enum { NONE, BOOL, AUTOUTF8, UTF8, SSE2UTF8, INT8, INT16, INT32, INT64, FLOAT16, FLOAT, DOUBLE, GEOPOS, NEAREST_NEIGHBOR } default=AUTOUTF8",
"fieldspec[].arg1 string default=\"\"",
"fieldspec[].normalize enum { NONE, LOWERCASE, LOWERCASE_AND_FOLD } default=LOWERCASE_AND_FOLD",
"fieldspec[].maxlength int default=1048576",
"fieldspec[].fieldtype enum {ATTRIBUTE, INDEX} default=INDEX",
"documenttype[].name string",
"documenttype[].index[].name string",
"documenttype[].index[].field[].name string",
};
const ::config::StringVector InternalVsmfieldsType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalVsmfieldsType::Fieldspec::Searchmethod
InternalVsmfieldsType::Fieldspec::getSearchmethod(const vespalib::string& name)
{
    if (name == "NONE") {
        return Searchmethod::NONE;
    } else if (name == "BOOL") {
        return Searchmethod::BOOL;
    } else if (name == "AUTOUTF8") {
        return Searchmethod::AUTOUTF8;
    } else if (name == "UTF8") {
        return Searchmethod::UTF8;
    } else if (name == "SSE2UTF8") {
        return Searchmethod::SSE2UTF8;
    } else if (name == "INT8") {
        return Searchmethod::INT8;
    } else if (name == "INT16") {
        return Searchmethod::INT16;
    } else if (name == "INT32") {
        return Searchmethod::INT32;
    } else if (name == "INT64") {
        return Searchmethod::INT64;
    } else if (name == "FLOAT16") {
        return Searchmethod::FLOAT16;
    } else if (name == "FLOAT") {
        return Searchmethod::FLOAT;
    } else if (name == "DOUBLE") {
        return Searchmethod::DOUBLE;
    } else if (name == "GEOPOS") {
        return Searchmethod::GEOPOS;
    } else if (name == "NEAREST_NEIGHBOR") {
        return Searchmethod::NEAREST_NEIGHBOR;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalVsmfieldsType::Fieldspec::getSearchmethodName(Searchmethod t)
{
    switch (t) {
        case Searchmethod::NONE: return "NONE";
        case Searchmethod::BOOL: return "BOOL";
        case Searchmethod::AUTOUTF8: return "AUTOUTF8";
        case Searchmethod::UTF8: return "UTF8";
        case Searchmethod::SSE2UTF8: return "SSE2UTF8";
        case Searchmethod::INT8: return "INT8";
        case Searchmethod::INT16: return "INT16";
        case Searchmethod::INT32: return "INT32";
        case Searchmethod::INT64: return "INT64";
        case Searchmethod::FLOAT16: return "FLOAT16";
        case Searchmethod::FLOAT: return "FLOAT";
        case Searchmethod::DOUBLE: return "DOUBLE";
        case Searchmethod::GEOPOS: return "GEOPOS";
        case Searchmethod::NEAREST_NEIGHBOR: return "NEAREST_NEIGHBOR";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalVsmfieldsType::Fieldspec::Searchmethod InternalVsmfieldsType::Fieldspec::InternalSearchmethodConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalVsmfieldsType::Fieldspec::getSearchmethod(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalVsmfieldsType::Fieldspec::Searchmethod InternalVsmfieldsType::Fieldspec::InternalSearchmethodConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalVsmfieldsType::Fieldspec::getSearchmethod(__inspector.asString().make_string());
}
InternalVsmfieldsType::Fieldspec::Searchmethod InternalVsmfieldsType::Fieldspec::InternalSearchmethodConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Searchmethod __eDefault) {
    if (__inspector.valid()) {
        return InternalVsmfieldsType::Fieldspec::getSearchmethod(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalVsmfieldsType::Fieldspec::Normalize
InternalVsmfieldsType::Fieldspec::getNormalize(const vespalib::string& name)
{
    if (name == "NONE") {
        return Normalize::NONE;
    } else if (name == "LOWERCASE") {
        return Normalize::LOWERCASE;
    } else if (name == "LOWERCASE_AND_FOLD") {
        return Normalize::LOWERCASE_AND_FOLD;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalVsmfieldsType::Fieldspec::getNormalizeName(Normalize t)
{
    switch (t) {
        case Normalize::NONE: return "NONE";
        case Normalize::LOWERCASE: return "LOWERCASE";
        case Normalize::LOWERCASE_AND_FOLD: return "LOWERCASE_AND_FOLD";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalVsmfieldsType::Fieldspec::Normalize InternalVsmfieldsType::Fieldspec::InternalNormalizeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalVsmfieldsType::Fieldspec::getNormalize(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalVsmfieldsType::Fieldspec::Normalize InternalVsmfieldsType::Fieldspec::InternalNormalizeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalVsmfieldsType::Fieldspec::getNormalize(__inspector.asString().make_string());
}
InternalVsmfieldsType::Fieldspec::Normalize InternalVsmfieldsType::Fieldspec::InternalNormalizeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Normalize __eDefault) {
    if (__inspector.valid()) {
        return InternalVsmfieldsType::Fieldspec::getNormalize(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalVsmfieldsType::Fieldspec::Fieldtype
InternalVsmfieldsType::Fieldspec::getFieldtype(const vespalib::string& name)
{
    if (name == "ATTRIBUTE") {
        return Fieldtype::ATTRIBUTE;
    } else if (name == "INDEX") {
        return Fieldtype::INDEX;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalVsmfieldsType::Fieldspec::getFieldtypeName(Fieldtype t)
{
    switch (t) {
        case Fieldtype::ATTRIBUTE: return "ATTRIBUTE";
        case Fieldtype::INDEX: return "INDEX";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalVsmfieldsType::Fieldspec::Fieldtype InternalVsmfieldsType::Fieldspec::InternalFieldtypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalVsmfieldsType::Fieldspec::getFieldtype(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalVsmfieldsType::Fieldspec::Fieldtype InternalVsmfieldsType::Fieldspec::InternalFieldtypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalVsmfieldsType::Fieldspec::getFieldtype(__inspector.asString().make_string());
}
InternalVsmfieldsType::Fieldspec::Fieldtype InternalVsmfieldsType::Fieldspec::InternalFieldtypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Fieldtype __eDefault) {
    if (__inspector.valid()) {
        return InternalVsmfieldsType::Fieldspec::getFieldtype(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalVsmfieldsType::Fieldspec::Fieldspec() noexcept
    : name(),
      searchmethod(Searchmethod::AUTOUTF8),
      arg1(""),
      normalize(Normalize::LOWERCASE_AND_FOLD),
      maxlength(1048576),
      fieldtype(Fieldtype::INDEX)
{
}

InternalVsmfieldsType::Fieldspec::Fieldspec(const Fieldspec & __rhs) noexcept = default;
InternalVsmfieldsType::Fieldspec & InternalVsmfieldsType::Fieldspec::operator = (const Fieldspec & __rhs) noexcept = default;
InternalVsmfieldsType::Fieldspec::Fieldspec(Fieldspec && __rhs) noexcept = default;
InternalVsmfieldsType::Fieldspec & InternalVsmfieldsType::Fieldspec::operator = (Fieldspec && __rhs) noexcept = default;
InternalVsmfieldsType::Fieldspec::~Fieldspec() = default; 
InternalVsmfieldsType::Fieldspec::Fieldspec(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    searchmethod = getSearchmethod(ConfigParser::parse<vespalib::string>("searchmethod", __lines, "AUTOUTF8"));
    ConfigParser::stripLinesForKey("searchmethod", __remainingValuesToParse);
    arg1 = ConfigParser::parse<vespalib::string>("arg1", __lines, "");
    ConfigParser::stripLinesForKey("arg1", __remainingValuesToParse);
    normalize = getNormalize(ConfigParser::parse<vespalib::string>("normalize", __lines, "LOWERCASE_AND_FOLD"));
    ConfigParser::stripLinesForKey("normalize", __remainingValuesToParse);
    maxlength = ConfigParser::parse<int32_t>("maxlength", __lines, 1048576);
    ConfigParser::stripLinesForKey("maxlength", __remainingValuesToParse);
    fieldtype = getFieldtype(ConfigParser::parse<vespalib::string>("fieldtype", __lines, "INDEX"));
    ConfigParser::stripLinesForKey("fieldtype", __remainingValuesToParse);
}

bool
InternalVsmfieldsType::Fieldspec::operator==(const Fieldspec& __rhs) const noexcept
{
    return (name == __rhs.name &&
            searchmethod == __rhs.searchmethod &&
            arg1 == __rhs.arg1 &&
            normalize == __rhs.normalize &&
            maxlength == __rhs.maxlength &&
            fieldtype == __rhs.fieldtype);
}

bool
InternalVsmfieldsType::Fieldspec::operator!=(const Fieldspec& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalVsmfieldsType::Fieldspec::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("searchmethod");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getSearchmethodName(searchmethod)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("arg1");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(arg1));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("normalize");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getNormalizeName(normalize)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxlength");
        __c.setString("type", "int");
        __c.setLong("value", maxlength);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fieldtype");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getFieldtypeName(fieldtype)));
    }
}

InternalVsmfieldsType::Fieldspec::Fieldspec(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    searchmethod = getSearchmethod(__inspector["searchmethod"]["value"].asString().make_string());
    arg1 = __inspector["arg1"]["value"].asString().make_string();
    normalize = getNormalize(__inspector["normalize"]["value"].asString().make_string());
    maxlength = __inspector["maxlength"]["value"].asLong();
    fieldtype = getFieldtype(__inspector["fieldtype"]["value"].asString().make_string());
}

InternalVsmfieldsType::Fieldspec::Fieldspec(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    searchmethod = InternalSearchmethodConverter()(__inspector["searchmethod"], Searchmethod::AUTOUTF8);
    arg1 = ::config::internal::ValueConverter<vespalib::string>()(__inspector["arg1"], "");
    normalize = InternalNormalizeConverter()(__inspector["normalize"], Normalize::LOWERCASE_AND_FOLD);
    maxlength = ::config::internal::ValueConverter<int32_t>()(__inspector["maxlength"], 1048576);
    fieldtype = InternalFieldtypeConverter()(__inspector["fieldtype"], Fieldtype::INDEX);
}

InternalVsmfieldsType::Documenttype::Index::Field::Field() noexcept
    : name()
{
}

InternalVsmfieldsType::Documenttype::Index::Field::Field(const Field & __rhs) noexcept = default;
InternalVsmfieldsType::Documenttype::Index::Field & InternalVsmfieldsType::Documenttype::Index::Field::operator = (const Field & __rhs) noexcept = default;
InternalVsmfieldsType::Documenttype::Index::Field::Field(Field && __rhs) noexcept = default;
InternalVsmfieldsType::Documenttype::Index::Field & InternalVsmfieldsType::Documenttype::Index::Field::operator = (Field && __rhs) noexcept = default;
InternalVsmfieldsType::Documenttype::Index::Field::~Field() = default; 
InternalVsmfieldsType::Documenttype::Index::Field::Field(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
}

bool
InternalVsmfieldsType::Documenttype::Index::Field::
operator==(const Field& __rhs) const noexcept
{
    return (name == __rhs.name);
}

bool
InternalVsmfieldsType::Documenttype::Index::Field::
operator!=(const Field& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalVsmfieldsType::Documenttype::Index::Field::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
}

InternalVsmfieldsType::Documenttype::Index::Field::Field(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
}

InternalVsmfieldsType::Documenttype::Index::Field::Field(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
}

InternalVsmfieldsType::Documenttype::Index::Index()
    : name(),
      field()
{
}

InternalVsmfieldsType::Documenttype::Index::Index(const Index & __rhs) = default;
InternalVsmfieldsType::Documenttype::Index & InternalVsmfieldsType::Documenttype::Index::operator = (const Index & __rhs) = default;
InternalVsmfieldsType::Documenttype::Index::Index(Index && __rhs) noexcept = default;
InternalVsmfieldsType::Documenttype::Index & InternalVsmfieldsType::Documenttype::Index::operator = (Index && __rhs) noexcept = default;
InternalVsmfieldsType::Documenttype::Index::~Index() = default; 
InternalVsmfieldsType::Documenttype::Index::Index(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    field = ConfigParser::parseArray<FieldVector>("field", __lines);
    ConfigParser::stripLinesForKey("field", __remainingValuesToParse);
}

bool
InternalVsmfieldsType::Documenttype::Index::operator==(const Index& __rhs) const noexcept
{
    return (name == __rhs.name &&
            field == __rhs.field);
}

bool
InternalVsmfieldsType::Documenttype::Index::operator!=(const Index& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalVsmfieldsType::Documenttype::Index::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("field");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : field) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalVsmfieldsType::Documenttype::Index::Index(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["field"]["value"].children(); __i++) {
        field.push_back(Field(__inspector["field"]["value"][__i]["value"]));
    }
}

InternalVsmfieldsType::Documenttype::Index::Index(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    ::config::internal::VectorInserter<FieldVector> __fieldInserter(field);
    __inspector["field"].traverse(__fieldInserter);
}

InternalVsmfieldsType::Documenttype::Documenttype()
    : name(),
      index()
{
}

InternalVsmfieldsType::Documenttype::Documenttype(const Documenttype & __rhs) = default;
InternalVsmfieldsType::Documenttype & InternalVsmfieldsType::Documenttype::operator = (const Documenttype & __rhs) = default;
InternalVsmfieldsType::Documenttype::Documenttype(Documenttype && __rhs) noexcept = default;
InternalVsmfieldsType::Documenttype & InternalVsmfieldsType::Documenttype::operator = (Documenttype && __rhs) noexcept = default;
InternalVsmfieldsType::Documenttype::~Documenttype() = default; 
InternalVsmfieldsType::Documenttype::Documenttype(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    name = ConfigParser::parse<vespalib::string>("name", __lines);
    ConfigParser::stripLinesForKey("name", __remainingValuesToParse);
    index = ConfigParser::parseArray<IndexVector>("index", __lines);
    ConfigParser::stripLinesForKey("index", __remainingValuesToParse);
}

bool
InternalVsmfieldsType::Documenttype::operator==(const Documenttype& __rhs) const noexcept
{
    return (name == __rhs.name &&
            index == __rhs.index);
}

bool
InternalVsmfieldsType::Documenttype::operator!=(const Documenttype& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalVsmfieldsType::Documenttype::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("name");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(name));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("index");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : index) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalVsmfieldsType::Documenttype::Documenttype(const vespalib::slime::Inspector & __inspector)
{
    name = __inspector["name"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["index"]["value"].children(); __i++) {
        index.push_back(Index(__inspector["index"]["value"][__i]["value"]));
    }
}

InternalVsmfieldsType::Documenttype::Documenttype(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    name = ::config::internal::ValueConverter<vespalib::string>()("name", __inspector["name"]);
    ::config::internal::VectorInserter<IndexVector> __indexInserter(index);
    __inspector["index"].traverse(__indexInserter);
}

InternalVsmfieldsType::InternalVsmfieldsType()
    : documentverificationlevel(0),
      searchall(1),
      fieldspec(),
      documenttype()
{
}

InternalVsmfieldsType::InternalVsmfieldsType(const InternalVsmfieldsType & __rhs) = default;
InternalVsmfieldsType & InternalVsmfieldsType::operator =(const InternalVsmfieldsType & __rhs) = default;
InternalVsmfieldsType::InternalVsmfieldsType(InternalVsmfieldsType && __rhs) noexcept = default;
InternalVsmfieldsType & InternalVsmfieldsType::operator =(InternalVsmfieldsType && __rhs) noexcept = default;
InternalVsmfieldsType::~InternalVsmfieldsType() = default; 
InternalVsmfieldsType::InternalVsmfieldsType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        documentverificationlevel = ConfigParser::parse<int32_t>("documentverificationlevel", __lines, 0);
        ConfigParser::stripLinesForKey("documentverificationlevel", __remainingValuesToParse);
        searchall = ConfigParser::parse<int32_t>("searchall", __lines, 1);
        ConfigParser::stripLinesForKey("searchall", __remainingValuesToParse);
        fieldspec = ConfigParser::parseArray<FieldspecVector>("fieldspec", __lines);
        ConfigParser::stripLinesForKey("fieldspec", __remainingValuesToParse);
        documenttype = ConfigParser::parseArray<DocumenttypeVector>("documenttype", __lines);
        ConfigParser::stripLinesForKey("documenttype", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalVsmfieldsType::operator==(const InternalVsmfieldsType& __rhs) const noexcept
{
    return (documentverificationlevel == __rhs.documentverificationlevel &&
            searchall == __rhs.searchall &&
            fieldspec == __rhs.fieldspec &&
            documenttype == __rhs.documenttype);
}

bool
InternalVsmfieldsType::operator!=(const InternalVsmfieldsType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalVsmfieldsType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("documentverificationlevel");
        __c.setString("type", "int");
        __c.setLong("value", documentverificationlevel);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("searchall");
        __c.setString("type", "int");
        __c.setLong("value", searchall);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fieldspec");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : fieldspec) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("documenttype");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : documenttype) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalVsmfieldsType::InternalVsmfieldsType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    documentverificationlevel = __inspector["documentverificationlevel"]["value"].asLong();
    searchall = __inspector["searchall"]["value"].asLong();
    for (size_t __i = 0; __i < __inspector["fieldspec"]["value"].children(); __i++) {
        fieldspec.push_back(Fieldspec(__inspector["fieldspec"]["value"][__i]["value"]));
    }
    for (size_t __i = 0; __i < __inspector["documenttype"]["value"].children(); __i++) {
        documenttype.push_back(Documenttype(__inspector["documenttype"]["value"][__i]["value"]));
    }
}

InternalVsmfieldsType::InternalVsmfieldsType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    documentverificationlevel = ::config::internal::ValueConverter<int32_t>()(__inspector["documentverificationlevel"], 0);
    searchall = ::config::internal::ValueConverter<int32_t>()(__inspector["searchall"], 1);
    ::config::internal::VectorInserter<FieldspecVector> __fieldspecInserter(fieldspec);
    __inspector["fieldspec"].traverse(__fieldspecInserter);
    ::config::internal::VectorInserter<DocumenttypeVector> __documenttypeInserter(documenttype);
    __inspector["documenttype"].traverse(__documenttypeInserter);
}

} // namespace internal

} // namespace vespa::config::search::vsm
