/**
 * @class vespa::config::search::vsm::Vsmfields
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_SEARCH_VSM_VSMFIELDS_H
#define CLOUD_CONFIG_VESPA_CONFIG_SEARCH_VSM_VSMFIELDS_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::search::vsm {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalVsmfieldsType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalVsmfieldsType> UP;
    class Fieldspec {
    public:
        enum class Searchmethod { NONE, BOOL, AUTOUTF8, UTF8, SSE2UTF8, INT8, INT16, INT32, INT64, FLOAT16, FLOAT, DOUBLE, GEOPOS, NEAREST_NEIGHBOR };
        typedef std::vector<Searchmethod> SearchmethodVector;
        typedef std::map<vespalib::string, Searchmethod> SearchmethodMap;
        static Searchmethod getSearchmethod(const vespalib::string&);
        static vespalib::string getSearchmethodName(Searchmethod e);

        struct InternalSearchmethodConverter {
            Searchmethod operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Searchmethod operator()(const ::vespalib::slime::Inspector & __inspector);
            Searchmethod operator()(const ::vespalib::slime::Inspector & __inspector, Searchmethod __eDefault);
        };
        enum class Normalize { NONE, LOWERCASE, LOWERCASE_AND_FOLD };
        typedef std::vector<Normalize> NormalizeVector;
        typedef std::map<vespalib::string, Normalize> NormalizeMap;
        static Normalize getNormalize(const vespalib::string&);
        static vespalib::string getNormalizeName(Normalize e);

        struct InternalNormalizeConverter {
            Normalize operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Normalize operator()(const ::vespalib::slime::Inspector & __inspector);
            Normalize operator()(const ::vespalib::slime::Inspector & __inspector, Normalize __eDefault);
        };
        enum class Fieldtype { ATTRIBUTE, INDEX };
        typedef std::vector<Fieldtype> FieldtypeVector;
        typedef std::map<vespalib::string, Fieldtype> FieldtypeMap;
        static Fieldtype getFieldtype(const vespalib::string&);
        static vespalib::string getFieldtypeName(Fieldtype e);

        struct InternalFieldtypeConverter {
            Fieldtype operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Fieldtype operator()(const ::vespalib::slime::Inspector & __inspector);
            Fieldtype operator()(const ::vespalib::slime::Inspector & __inspector, Fieldtype __eDefault);
        };
        Fieldspec(const ::config::StringVector & __lines);
        Fieldspec(const vespalib::slime::Inspector & __inspector);
        Fieldspec(const ::config::ConfigPayload & __payload);
        Fieldspec()  noexcept;
        Fieldspec(const Fieldspec & __rhs) noexcept;
        Fieldspec & operator = (const Fieldspec & __rhs) noexcept;
        Fieldspec(Fieldspec && __rhs) noexcept;
        Fieldspec & operator = (Fieldspec && __rhs) noexcept;
        ~Fieldspec();

        bool operator==(const Fieldspec& __rhs) const noexcept;
        bool operator!=(const Fieldspec& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * The name of a field for which we are assigning a search method.
         * The field name refers directly to a field in the document model.
         */
        vespalib::string name;

        /**
         * The search method for a given field. Note: same field in 2 different
         * document types must match on type if not a random result might be
         * expected.
         */
        Searchmethod searchmethod; // Default: AUTOUTF8
        vespalib::string arg1; // Default: ""
        Normalize normalize; // Default: LOWERCASE_AND_FOLD

        /** Maximum number of chars to search per field. */
        int32_t maxlength; // Default: 1048576

        /** Type of the field */
        Fieldtype fieldtype; // Default: INDEX
    };
    typedef std::vector<Fieldspec> FieldspecVector;

    typedef std::map<vespalib::string, Fieldspec> FieldspecMap;

    class Documenttype {
    public:
        class Index {
        public:
            class Field {
            public:
                Field(const ::config::StringVector & __lines);
                Field(const vespalib::slime::Inspector & __inspector);
                Field(const ::config::ConfigPayload & __payload);
                Field()  noexcept;
                Field(const Field & __rhs) noexcept;
                Field & operator = (const Field & __rhs) noexcept;
                Field(Field && __rhs) noexcept;
                Field & operator = (Field && __rhs) noexcept;
                ~Field();

                bool operator==(const Field& __rhs) const noexcept;
                bool operator!=(const Field& __rhs) const noexcept;

                void serialize(vespalib::slime::Cursor & __cursor) const;

                /**
                 * The name of a field part of an index.
                 * The field name refers directly to a field in the document
                 * model.
                 */
                vespalib::string name;
            };
            typedef std::vector<Field> FieldVector;

            typedef std::map<vespalib::string, Field> FieldMap;

            Index(const ::config::StringVector & __lines);
            Index(const vespalib::slime::Inspector & __inspector);
            Index(const ::config::ConfigPayload & __payload);
            Index() ;
            Index(const Index & __rhs);
            Index & operator = (const Index & __rhs);
            Index(Index && __rhs) noexcept;
            Index & operator = (Index && __rhs) noexcept;
            ~Index();

            bool operator==(const Index& __rhs) const noexcept;
            bool operator!=(const Index& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * The name of an index of a documenttype for which we are assigning
             * a set of fields.
             */
            vespalib::string name;
            FieldVector field;
        };
        typedef std::vector<Index> IndexVector;

        typedef std::map<vespalib::string, Index> IndexMap;

        Documenttype(const ::config::StringVector & __lines);
        Documenttype(const vespalib::slime::Inspector & __inspector);
        Documenttype(const ::config::ConfigPayload & __payload);
        Documenttype() ;
        Documenttype(const Documenttype & __rhs);
        Documenttype & operator = (const Documenttype & __rhs);
        Documenttype(Documenttype && __rhs) noexcept;
        Documenttype & operator = (Documenttype && __rhs) noexcept;
        ~Documenttype();

        bool operator==(const Documenttype& __rhs) const noexcept;
        bool operator!=(const Documenttype& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * The name of a documenttype for which we are assigning a set of
         * indexes.
         */
        vespalib::string name;
        IndexVector index;
    };
    typedef std::vector<Documenttype> DocumenttypeVector;

    typedef std::map<vespalib::string, Documenttype> DocumenttypeMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalVsmfieldsType(const ::config::ConfigValue & __value);
    InternalVsmfieldsType(const ::config::ConfigDataBuffer & __value);
    InternalVsmfieldsType(const ::config::ConfigPayload & __payload);
    InternalVsmfieldsType() ;
    InternalVsmfieldsType(const InternalVsmfieldsType & __rhs);
    InternalVsmfieldsType & operator = (const InternalVsmfieldsType & __rhs);
    InternalVsmfieldsType(InternalVsmfieldsType && __rhs) noexcept;
    InternalVsmfieldsType & operator = (InternalVsmfieldsType && __rhs) noexcept;
    ~InternalVsmfieldsType();

    bool operator==(const InternalVsmfieldsType& __rhs) const noexcept;
    bool operator!=(const InternalVsmfieldsType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /** Level of verification applied to the documents received. */
    int32_t documentverificationlevel; // Default: 0

    /** Set if one should ignore limit hits. */
    int32_t searchall; // Default: 1
    FieldspecVector fieldspec;
    DocumenttypeVector documenttype;
};

} // namespace internal

typedef internal::InternalVsmfieldsType VsmfieldsConfigBuilder;
typedef const internal::InternalVsmfieldsType VsmfieldsConfig;

} // namespace vespa::config::search::vsm
#endif // VESPA_config_VESPA_CONFIG_SEARCH_VSM_VSMFIELDS_H
