#include "config-vsmsummary.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search::vsm {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalVsmsummaryType::CONFIG_DEF_MD5("07145fe84cbe44e8e0d54c27f55971b4");
const vespalib::string InternalVsmsummaryType::CONFIG_DEF_NAME("vsmsummary");
const vespalib::string InternalVsmsummaryType::CONFIG_DEF_NAMESPACE("vespa.config.search.vsm");
const int64_t InternalVsmsummaryType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search.vsm",
"outputclass string default=\"\"",
"fieldmap[].summary string",
"fieldmap[].document[].field string",
"fieldmap[].command enum { NONE, FLATTENJUNIPER, FLATTENSPACE } default=NONE",
};
const ::config::StringVector InternalVsmsummaryType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalVsmsummaryType::Fieldmap::Document::Document() noexcept
    : field()
{
}

InternalVsmsummaryType::Fieldmap::Document::Document(const Document & __rhs) noexcept = default;
InternalVsmsummaryType::Fieldmap::Document & InternalVsmsummaryType::Fieldmap::Document::operator = (const Document & __rhs) noexcept = default;
InternalVsmsummaryType::Fieldmap::Document::Document(Document && __rhs) noexcept = default;
InternalVsmsummaryType::Fieldmap::Document & InternalVsmsummaryType::Fieldmap::Document::operator = (Document && __rhs) noexcept = default;
InternalVsmsummaryType::Fieldmap::Document::~Document() = default; 
InternalVsmsummaryType::Fieldmap::Document::Document(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    field = ConfigParser::parse<vespalib::string>("field", __lines);
    ConfigParser::stripLinesForKey("field", __remainingValuesToParse);
}

bool
InternalVsmsummaryType::Fieldmap::Document::
operator==(const Document& __rhs) const noexcept
{
    return (field == __rhs.field);
}

bool
InternalVsmsummaryType::Fieldmap::Document::
operator!=(const Document& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalVsmsummaryType::Fieldmap::Document::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("field");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(field));
    }
}

InternalVsmsummaryType::Fieldmap::Document::Document(const vespalib::slime::Inspector & __inspector)
{
    field = __inspector["field"]["value"].asString().make_string();
}

InternalVsmsummaryType::Fieldmap::Document::Document(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    field = ::config::internal::ValueConverter<vespalib::string>()("field", __inspector["field"]);
}

InternalVsmsummaryType::Fieldmap::Command
InternalVsmsummaryType::Fieldmap::getCommand(const vespalib::string& name)
{
    if (name == "NONE") {
        return Command::NONE;
    } else if (name == "FLATTENJUNIPER") {
        return Command::FLATTENJUNIPER;
    } else if (name == "FLATTENSPACE") {
        return Command::FLATTENSPACE;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalVsmsummaryType::Fieldmap::getCommandName(Command t)
{
    switch (t) {
        case Command::NONE: return "NONE";
        case Command::FLATTENJUNIPER: return "FLATTENJUNIPER";
        case Command::FLATTENSPACE: return "FLATTENSPACE";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalVsmsummaryType::Fieldmap::Command InternalVsmsummaryType::Fieldmap::InternalCommandConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalVsmsummaryType::Fieldmap::getCommand(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalVsmsummaryType::Fieldmap::Command InternalVsmsummaryType::Fieldmap::InternalCommandConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalVsmsummaryType::Fieldmap::getCommand(__inspector.asString().make_string());
}
InternalVsmsummaryType::Fieldmap::Command InternalVsmsummaryType::Fieldmap::InternalCommandConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Command __eDefault) {
    if (__inspector.valid()) {
        return InternalVsmsummaryType::Fieldmap::getCommand(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalVsmsummaryType::Fieldmap::Fieldmap()
    : summary(),
      document(),
      command(Command::NONE)
{
}

InternalVsmsummaryType::Fieldmap::Fieldmap(const Fieldmap & __rhs) = default;
InternalVsmsummaryType::Fieldmap & InternalVsmsummaryType::Fieldmap::operator = (const Fieldmap & __rhs) = default;
InternalVsmsummaryType::Fieldmap::Fieldmap(Fieldmap && __rhs) noexcept = default;
InternalVsmsummaryType::Fieldmap & InternalVsmsummaryType::Fieldmap::operator = (Fieldmap && __rhs) noexcept = default;
InternalVsmsummaryType::Fieldmap::~Fieldmap() = default; 
InternalVsmsummaryType::Fieldmap::Fieldmap(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    summary = ConfigParser::parse<vespalib::string>("summary", __lines);
    ConfigParser::stripLinesForKey("summary", __remainingValuesToParse);
    document = ConfigParser::parseArray<DocumentVector>("document", __lines);
    ConfigParser::stripLinesForKey("document", __remainingValuesToParse);
    command = getCommand(ConfigParser::parse<vespalib::string>("command", __lines, "NONE"));
    ConfigParser::stripLinesForKey("command", __remainingValuesToParse);
}

bool
InternalVsmsummaryType::Fieldmap::operator==(const Fieldmap& __rhs) const noexcept
{
    return (summary == __rhs.summary &&
            document == __rhs.document &&
            command == __rhs.command);
}

bool
InternalVsmsummaryType::Fieldmap::operator!=(const Fieldmap& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalVsmsummaryType::Fieldmap::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("summary");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(summary));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("document");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : document) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("command");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getCommandName(command)));
    }
}

InternalVsmsummaryType::Fieldmap::Fieldmap(const vespalib::slime::Inspector & __inspector)
{
    summary = __inspector["summary"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["document"]["value"].children(); __i++) {
        document.push_back(Document(__inspector["document"]["value"][__i]["value"]));
    }
    command = getCommand(__inspector["command"]["value"].asString().make_string());
}

InternalVsmsummaryType::Fieldmap::Fieldmap(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    summary = ::config::internal::ValueConverter<vespalib::string>()("summary", __inspector["summary"]);
    ::config::internal::VectorInserter<DocumentVector> __documentInserter(document);
    __inspector["document"].traverse(__documentInserter);
    command = InternalCommandConverter()(__inspector["command"], Command::NONE);
}

InternalVsmsummaryType::InternalVsmsummaryType()
    : outputclass(""),
      fieldmap()
{
}

InternalVsmsummaryType::InternalVsmsummaryType(const InternalVsmsummaryType & __rhs) = default;
InternalVsmsummaryType & InternalVsmsummaryType::operator =(const InternalVsmsummaryType & __rhs) = default;
InternalVsmsummaryType::InternalVsmsummaryType(InternalVsmsummaryType && __rhs) noexcept = default;
InternalVsmsummaryType & InternalVsmsummaryType::operator =(InternalVsmsummaryType && __rhs) noexcept = default;
InternalVsmsummaryType::~InternalVsmsummaryType() = default; 
InternalVsmsummaryType::InternalVsmsummaryType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        outputclass = ConfigParser::parse<vespalib::string>("outputclass", __lines, "");
        ConfigParser::stripLinesForKey("outputclass", __remainingValuesToParse);
        fieldmap = ConfigParser::parseArray<FieldmapVector>("fieldmap", __lines);
        ConfigParser::stripLinesForKey("fieldmap", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalVsmsummaryType::operator==(const InternalVsmsummaryType& __rhs) const noexcept
{
    return (outputclass == __rhs.outputclass &&
            fieldmap == __rhs.fieldmap);
}

bool
InternalVsmsummaryType::operator!=(const InternalVsmsummaryType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalVsmsummaryType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("outputclass");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(outputclass));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("fieldmap");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : fieldmap) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
}

InternalVsmsummaryType::InternalVsmsummaryType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    outputclass = __inspector["outputclass"]["value"].asString().make_string();
    for (size_t __i = 0; __i < __inspector["fieldmap"]["value"].children(); __i++) {
        fieldmap.push_back(Fieldmap(__inspector["fieldmap"]["value"][__i]["value"]));
    }
}

InternalVsmsummaryType::InternalVsmsummaryType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    outputclass = ::config::internal::ValueConverter<vespalib::string>()(__inspector["outputclass"], "");
    ::config::internal::VectorInserter<FieldmapVector> __fieldmapInserter(fieldmap);
    __inspector["fieldmap"].traverse(__fieldmapInserter);
}

} // namespace internal

} // namespace vespa::config::search::vsm
