/**
 * @class vespa::config::search::vsm::Vsmsummary
 * @ingroup config
 *
 * @brief This is an autogenerated class for handling VESPA config.
 *
 * This class is autogenerated by vespa from a config definition file.
 * To subscribe to config, you need to include the config/config.h header, 
 * and create a ConfigSubscriber in order to subscribe for config.
 *
 * Copyright Vespa.ai. Licensed under the terms of the Apache 2.0 license. See LICENSE in the project root.
 */
#ifndef CLOUD_CONFIG_VESPA_CONFIG_SEARCH_VSM_VSMSUMMARY_H
#define CLOUD_CONFIG_VESPA_CONFIG_SEARCH_VSM_VSMSUMMARY_H

#include <vespa/config/configgen/configinstance.h>
#include <vespa/config/common/types.h>

namespace config {
    class ConfigValue;
    class ConfigPayload;
}

namespace vespalib::slime {
    struct Inspector;
    struct Cursor;
}

namespace vespa::config::search::vsm {

namespace internal {

/**
 * This class contains the config. DO NOT USE THIS CLASS DIRECTLY. Use the typedeffed
 * versions after this class declaration.
 */
class InternalVsmsummaryType : public ::config::ConfigInstance
{
public:
    typedef std::unique_ptr<const InternalVsmsummaryType> UP;
    class Fieldmap {
    public:
        class Document {
        public:
            Document(const ::config::StringVector & __lines);
            Document(const vespalib::slime::Inspector & __inspector);
            Document(const ::config::ConfigPayload & __payload);
            Document()  noexcept;
            Document(const Document & __rhs) noexcept;
            Document & operator = (const Document & __rhs) noexcept;
            Document(Document && __rhs) noexcept;
            Document & operator = (Document && __rhs) noexcept;
            ~Document();

            bool operator==(const Document& __rhs) const noexcept;
            bool operator!=(const Document& __rhs) const noexcept;

            void serialize(vespalib::slime::Cursor & __cursor) const;

            /**
             * Mapping of field names between the result class and the document
             * model. This field vector represents the names in the document
             * model
             * that should be used as input when generating the summary field.
             */
            vespalib::string field;
        };
        typedef std::vector<Document> DocumentVector;

        typedef std::map<vespalib::string, Document> DocumentMap;

        enum class Command { NONE, FLATTENJUNIPER, FLATTENSPACE };
        typedef std::vector<Command> CommandVector;
        typedef std::map<vespalib::string, Command> CommandMap;
        static Command getCommand(const vespalib::string&);
        static vespalib::string getCommandName(Command e);

        struct InternalCommandConverter {
            Command operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector);
            Command operator()(const ::vespalib::slime::Inspector & __inspector);
            Command operator()(const ::vespalib::slime::Inspector & __inspector, Command __eDefault);
        };
        Fieldmap(const ::config::StringVector & __lines);
        Fieldmap(const vespalib::slime::Inspector & __inspector);
        Fieldmap(const ::config::ConfigPayload & __payload);
        Fieldmap() ;
        Fieldmap(const Fieldmap & __rhs);
        Fieldmap & operator = (const Fieldmap & __rhs);
        Fieldmap(Fieldmap && __rhs) noexcept;
        Fieldmap & operator = (Fieldmap && __rhs) noexcept;
        ~Fieldmap();

        bool operator==(const Fieldmap& __rhs) const noexcept;
        bool operator!=(const Fieldmap& __rhs) const noexcept;

        void serialize(vespalib::slime::Cursor & __cursor) const;

        /**
         * Mapping of field names between the result class and the document
         * model. This value represents the name in the result class. Fields
         * not mentioned here will get the identity mapping.
         */
        vespalib::string summary;
        DocumentVector document;

        /**
         * This command specifies how the document fields should be combined
         * when generating the summary field.
         */
        Command command; // Default: NONE
    };
    typedef std::vector<Fieldmap> FieldmapVector;

    typedef std::map<vespalib::string, Fieldmap> FieldmapMap;

    const vespalib::string & defName() const override { return CONFIG_DEF_NAME; }
    const vespalib::string & defMd5() const override { return CONFIG_DEF_MD5; }
    const vespalib::string & defNamespace() const override { return CONFIG_DEF_NAMESPACE; }
    void serialize(::config::ConfigDataBuffer & __buffer) const override;
    InternalVsmsummaryType(const ::config::ConfigValue & __value);
    InternalVsmsummaryType(const ::config::ConfigDataBuffer & __value);
    InternalVsmsummaryType(const ::config::ConfigPayload & __payload);
    InternalVsmsummaryType() ;
    InternalVsmsummaryType(const InternalVsmsummaryType & __rhs);
    InternalVsmsummaryType & operator = (const InternalVsmsummaryType & __rhs);
    InternalVsmsummaryType(InternalVsmsummaryType && __rhs) noexcept;
    InternalVsmsummaryType & operator = (InternalVsmsummaryType && __rhs) noexcept;
    ~InternalVsmsummaryType();

    bool operator==(const InternalVsmsummaryType& __rhs) const noexcept;
    bool operator!=(const InternalVsmsummaryType& __rhs) const noexcept;

    static const vespalib::string CONFIG_DEF_MD5;
    static const vespalib::string CONFIG_DEF_NAME;
    static const vespalib::string CONFIG_DEF_NAMESPACE;
    static const ::config::StringVector CONFIG_DEF_SCHEMA;
    static const int64_t CONFIG_DEF_SERIALIZE_VERSION;


    /**
     * The name of the result class that should be generated for documents
     * returned from the VSM. If this value is empty, the first found
     * result class will be used.
     */
    vespalib::string outputclass; // Default: ""
    FieldmapVector fieldmap;
};

} // namespace internal

typedef internal::InternalVsmsummaryType VsmsummaryConfigBuilder;
typedef const internal::InternalVsmsummaryType VsmsummaryConfig;

} // namespace vespa::config::search::vsm
#endif // VESPA_config_VESPA_CONFIG_SEARCH_VSM_VSMSUMMARY_H
