#include "config-proton.h"
#include <vespa/config/common/configvalue.h>
#include <vespa/config/common/exceptions.h>
#include <vespa/config/configgen/configpayload.h>
#include <vespa/config/print/configdatabuffer.h>
#include <vespa/config/common/configparser.h>
#include <vespa/vespalib/data/slime/convenience.h>
#include <vespa/vespalib/data/slime/slime.h>
#include <vespa/vespalib/stllike/asciistream.h>
#include <vespa/config/configgen/vector_inserter.hpp>
#include <vespa/config/configgen/map_inserter.hpp>

namespace vespa::config::search::core {

namespace internal {

using ::config::ConfigParser;
using ::config::InvalidConfigException;
using ::config::ConfigInstance;
using ::config::ConfigValue;
using namespace vespalib::slime::convenience;

const vespalib::string InternalProtonType::CONFIG_DEF_MD5("2bb0106afa5d093788fc76e006eb275c");
const vespalib::string InternalProtonType::CONFIG_DEF_NAME("proton");
const vespalib::string InternalProtonType::CONFIG_DEF_NAMESPACE("vespa.config.search.core");
const int64_t InternalProtonType::CONFIG_DEF_SERIALIZE_VERSION(1);
const static vespalib::string __internalDefSchema[] = {
"namespace=vespa.config.search.core",
"basedir string default=\".\" restart",
"rpcport int default=8004 restart",
"httpport int default=0 restart",
"clustername string default=\"\" restart",
"distributionkey int default=-1",
"rpc.transportthreads int default=0 restart",
"search.async bool default=true",
"docsum.async bool default=true",
"numsearcherthreads int default=64 restart",
"numthreadspersearch int default=1 restart",
"numsummarythreads int default=16 restart",
"validate_and_sanitize_docstore enum {NO, YES} default = NO",
"flush.maxconcurrent int default=2 restart",
"flush.idleinterval double default=10.0 restart",
"flush.strategy enum {SIMPLE, MEMORY} default=MEMORY restart",
"flush.memory.maxmemory long default=4294967296",
"flush.memory.diskbloatfactor double default=0.25",
"flush.memory.maxtlssize long default=21474836480",
"flush.memory.each.maxmemory long default=1073741824",
"flush.memory.each.diskbloatfactor double default=0.25",
"flush.memory.maxage.time double default=111600.0",
"flush.memory.conservative.memorylimitfactor double default=0.5",
"flush.memory.conservative.disklimitfactor double default=0.5",
"flush.memory.conservative.highwatermarkfactor double default=0.95",
"flush.memory.conservative.lowwatermarkfactor double default=0.9",
"flush.preparerestart.replaycost double default=8.0",
"flush.preparerestart.replayoperationcost double default=3000.0",
"flush.preparerestart.writecost double default=1.0",
"indexing.write.io enum {NORMAL, OSYNC, DIRECTIO} default=DIRECTIO restart",
"indexing.read.io enum {NORMAL, DIRECTIO} default=DIRECTIO restart",
"indexing.optimize enum {LATENCY, THROUGHPUT, ADAPTIVE} default=THROUGHPUT restart",
"indexing.tasklimit int default=-1000",
"indexing.kind_of_watermark int default = 0 restart",
"indexing.reactiontime double default = 0.001 restart",
"index.warmup.time double default=0.0 restart",
"index.warmup.unpack bool default=false restart",
"index.maxflushed int default=2",
"index.maxflushedretired int default=20",
"attribute.write.io enum {NORMAL, OSYNC, DIRECTIO} default=DIRECTIO restart",
"search.mmap.options[] enum {POPULATE, HUGETLB} restart",
"search.mmap.advise enum {NORMAL, RANDOM, SEQUENTIAL} default=NORMAL restart",
"search.memory.limiter.maxthreads int default=0",
"search.memory.limiter.mincoverage double default=1.0",
"search.memory.limiter.minhits int default=1000000",
"grouping.sessionmanager.maxentries int default=500 restart",
"grouping.sessionmanager.pruning.interval double default=1.0",
"distribution.redundancy long default=1",
"distribution.searchablecopies long default=1",
"summary.cache.maxbytes long default=-4",
"summary.cache.compression.type enum {NONE, LZ4, ZSTD} default=LZ4",
"summary.cache.compression.level int default=6",
"summary.cache.update_strategy enum {INVALIDATE, UPDATE} default=INVALIDATE",
"summary.log.compact.compression.type enum {NONE, LZ4, ZSTD} default=ZSTD",
"summary.log.compact.compression.level int default=9",
"summary.log.chunk.compression.type enum {NONE, LZ4, ZSTD} default=ZSTD",
"summary.log.chunk.compression.level int default=9",
"summary.log.chunk.maxbytes int default=65536",
"summary.log.maxfilesize long default=1000000000",
"summary.log.maxnumlids int default=8388608",
"summary.log.maxdiskbloatfactor double default=0.1",
"summary.log.maxbucketspread double default=2.5",
"summary.log.minfilesizefactor double default=0.2",
"summary.write.io enum {NORMAL, OSYNC, DIRECTIO} default=DIRECTIO",
"summary.read.io enum {NORMAL, DIRECTIO, MMAP } default=MMAP restart",
"summary.read.mmap.options[] enum {POPULATE, HUGETLB} restart",
"summary.read.mmap.advise enum {NORMAL, RANDOM, SEQUENTIAL} default=NORMAL restart",
"documentdb[].inputdoctypename string",
"documentdb[].mode enum {INDEX, STREAMING, STORE_ONLY} default=INDEX",
"documentdb[].configid string",
"documentdb[].visibilitydelay double default=0.0",
"documentdb[].global bool default=false",
"documentdb[].allocation.initialnumdocs long default=1024",
"documentdb[].allocation.growfactor double default=0.2",
"documentdb[].allocation.growbias int default=1",
"documentdb[].allocation.amortizecount int default=10000",
"documentdb[].allocation.multivaluegrowfactor double default=0.2",
"documentdb[].allocation.max_dead_bytes_ratio double default=0.05",
"documentdb[].allocation.max_dead_address_space_ratio double default=0.2",
"documentdb[].allocation.max_compact_buffers int default=1",
"documentdb[].allocation.active_buffers_ratio double default=0.1",
"periodic.interval double default=3600.0",
"tlsspec string default=\"tcp/localhost:13700\" restart",
"tlsconfigid string default=\"\" restart",
"slobrokconfigid string default=\"\" restart",
"routingconfigid string default=\"\" restart",
"filedistributor.configid reference default=\"\" restart",
"pruneremoveddocumentsinterval double default=0.0",
"pruneremoveddocumentsage double default=1209600.0",
"packetcompresslimit int default=1024",
"packetcompresslevel int default=3",
"packetcompresstype enum {NONE, LZ4} default=LZ4",
"lidspacecompaction.interval double default=10.0",
"lidspacecompaction.allowedlidbloat int default=1",
"lidspacecompaction.allowedlidbloatfactor double default=0.01",
"lidspacecompaction.removebatchblockrate double default=0.5",
"lidspacecompaction.removeblockrate double default=100.0",
"bucketmove.maxdocstomoveperbucket int default=1",
"maxvisibilitydelay double default=1.0",
"visit.defaultserializedsize long default=1",
"visit.ignoremaxbytes bool default=true",
"initialize.threads int default = 0",
"writefilter.attribute.address_space_limit double default = 0.92",
"writefilter.memorylimit double default = 0.8",
"writefilter.disklimit double default = 0.8",
"writefilter.sampleinterval double default = 10.0",
"hwinfo.disk.size long default = 0 restart",
"hwinfo.disk.shared bool default = false restart",
"hwinfo.disk.writespeed double default = 200.0 restart",
"hwinfo.disk.samplewritesize long default = 1073741824 restart",
"hwinfo.disk.slowwritespeedlimit double default = 100.0 restart",
"hwinfo.memory.size long default = 0 restart",
"hwinfo.cpu.cores int default = 0 restart",
"feeding.concurrency double default = 0.2 restart",
"feeding.niceness double default = 0.0 restart",
"feeding.master_task_limit int default = 0",
"maintenancejobs.resourcelimitfactor double default = 1.05",
"maintenancejobs.maxoutstandingmoveops int default=100",
"bucketdb.checksumtype enum {LEGACY, XXHASH64} default = LEGACY restart",
"replay_throttling_policy.type enum { UNLIMITED, DYNAMIC } default=DYNAMIC",
"replay_throttling_policy.min_window_size int default=100",
"replay_throttling_policy.max_window_size int default=10000",
"replay_throttling_policy.window_size_increment int default=20",
"indexing.semiunboundtasklimit int default = 1000",
"summary.cache.allowvisitcaching bool default=true",
"summary.cache.initialentries long default=0 restart",
"summary.log.chunk.skipcrconread bool default=false",
"indexing.threads int default=1 restart",
"index.cache.size long default=0 restart",
"tensor_implementation enum {TENSOR_ENGINE, FAST_VALUE} default = FAST_VALUE",
"forward_issues bool default = true",
};
const ::config::StringVector InternalProtonType::CONFIG_DEF_SCHEMA(__internalDefSchema,
    __internalDefSchema + (sizeof(__internalDefSchema) / 
                           sizeof(__internalDefSchema[0])));

InternalProtonType::Rpc::Rpc() noexcept
    : transportthreads(0)
{
}

InternalProtonType::Rpc::Rpc(const Rpc & __rhs) noexcept = default;
InternalProtonType::Rpc & InternalProtonType::Rpc::operator = (const Rpc & __rhs) noexcept = default;
InternalProtonType::Rpc::Rpc(Rpc && __rhs) noexcept = default;
InternalProtonType::Rpc & InternalProtonType::Rpc::operator = (Rpc && __rhs) noexcept = default;
InternalProtonType::Rpc::~Rpc() = default; 
InternalProtonType::Rpc::Rpc(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    transportthreads = ConfigParser::parse<int32_t>("transportthreads", __lines, 0);
    ConfigParser::stripLinesForKey("transportthreads", __remainingValuesToParse);
}

bool
InternalProtonType::Rpc::operator==(const Rpc& __rhs) const noexcept
{
    return (transportthreads == __rhs.transportthreads);
}

bool
InternalProtonType::Rpc::operator!=(const Rpc& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Rpc::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("transportthreads");
        __c.setString("type", "int");
        __c.setLong("value", transportthreads);
    }
}

InternalProtonType::Rpc::Rpc(const vespalib::slime::Inspector & __inspector)
{
    transportthreads = __inspector["transportthreads"]["value"].asLong();
}

InternalProtonType::Rpc::Rpc(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    transportthreads = ::config::internal::ValueConverter<int32_t>()(__inspector["transportthreads"], 0);
}

InternalProtonType::Search::Mmap::Options
InternalProtonType::Search::Mmap::getOptions(const vespalib::string& name)
{
    if (name == "POPULATE") {
        return Options::POPULATE;
    } else if (name == "HUGETLB") {
        return Options::HUGETLB;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Search::Mmap::getOptionsName(Options t)
{
    switch (t) {
        case Options::POPULATE: return "POPULATE";
        case Options::HUGETLB: return "HUGETLB";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Search::Mmap::Options InternalProtonType::Search::Mmap::InternalOptionsConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Search::Mmap::getOptions(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Search::Mmap::Options InternalProtonType::Search::Mmap::InternalOptionsConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Search::Mmap::getOptions(__inspector.asString().make_string());
}
InternalProtonType::Search::Mmap::Options InternalProtonType::Search::Mmap::InternalOptionsConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Options __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Search::Mmap::getOptions(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Search::Mmap::Advise
InternalProtonType::Search::Mmap::getAdvise(const vespalib::string& name)
{
    if (name == "NORMAL") {
        return Advise::NORMAL;
    } else if (name == "RANDOM") {
        return Advise::RANDOM;
    } else if (name == "SEQUENTIAL") {
        return Advise::SEQUENTIAL;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Search::Mmap::getAdviseName(Advise t)
{
    switch (t) {
        case Advise::NORMAL: return "NORMAL";
        case Advise::RANDOM: return "RANDOM";
        case Advise::SEQUENTIAL: return "SEQUENTIAL";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Search::Mmap::Advise InternalProtonType::Search::Mmap::InternalAdviseConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Search::Mmap::getAdvise(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Search::Mmap::Advise InternalProtonType::Search::Mmap::InternalAdviseConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Search::Mmap::getAdvise(__inspector.asString().make_string());
}
InternalProtonType::Search::Mmap::Advise InternalProtonType::Search::Mmap::InternalAdviseConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Advise __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Search::Mmap::getAdvise(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Search::Mmap::Mmap()
    : options(),
      advise(Advise::NORMAL)
{
}

InternalProtonType::Search::Mmap::Mmap(const Mmap & __rhs) = default;
InternalProtonType::Search::Mmap & InternalProtonType::Search::Mmap::operator = (const Mmap & __rhs) = default;
InternalProtonType::Search::Mmap::Mmap(Mmap && __rhs) noexcept = default;
InternalProtonType::Search::Mmap & InternalProtonType::Search::Mmap::operator = (Mmap && __rhs) noexcept = default;
InternalProtonType::Search::Mmap::~Mmap() = default; 
InternalProtonType::Search::Mmap::Mmap(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    ::config::StringVector options__ValueList(
            ConfigParser::parseArray<::config::StringVector>("options", __lines));
    options.reserve(options__ValueList.size());
    for (const auto & item : options__ValueList) {
        options.push_back(getOptions(item));
    }
    ConfigParser::stripLinesForKey("options", __remainingValuesToParse);
    advise = getAdvise(ConfigParser::parse<vespalib::string>("advise", __lines, "NORMAL"));
    ConfigParser::stripLinesForKey("advise", __remainingValuesToParse);
}

bool
InternalProtonType::Search::Mmap::operator==(const Mmap& __rhs) const noexcept
{
    return (options == __rhs.options &&
            advise == __rhs.advise);
}

bool
InternalProtonType::Search::Mmap::operator!=(const Mmap& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Search::Mmap::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("options");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : options) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "enum");
            __c3.setString("value", vespalib::Memory(getOptionsName(child)));
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("advise");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getAdviseName(advise)));
    }
}

InternalProtonType::Search::Mmap::Mmap(const vespalib::slime::Inspector & __inspector)
{
    for (size_t __i = 0; __i < __inspector["options"]["value"].children(); __i++) {
        options.push_back(getOptions(__inspector["options"]["value"][__i]["value"].asString().make_string()));
    }
    advise = getAdvise(__inspector["advise"]["value"].asString().make_string());
}

InternalProtonType::Search::Mmap::Mmap(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<OptionsVector, InternalOptionsConverter> __optionsInserter(options);
    __inspector["options"].traverse(__optionsInserter);
    advise = InternalAdviseConverter()(__inspector["advise"], Advise::NORMAL);
}

InternalProtonType::Search::Memory::Limiter::Limiter() noexcept
    : maxthreads(0),
      mincoverage(1.0),
      minhits(1000000)
{
}

InternalProtonType::Search::Memory::Limiter::Limiter(const Limiter & __rhs) noexcept = default;
InternalProtonType::Search::Memory::Limiter & InternalProtonType::Search::Memory::Limiter::operator = (const Limiter & __rhs) noexcept = default;
InternalProtonType::Search::Memory::Limiter::Limiter(Limiter && __rhs) noexcept = default;
InternalProtonType::Search::Memory::Limiter & InternalProtonType::Search::Memory::Limiter::operator = (Limiter && __rhs) noexcept = default;
InternalProtonType::Search::Memory::Limiter::~Limiter() = default; 
InternalProtonType::Search::Memory::Limiter::Limiter(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    maxthreads = ConfigParser::parse<int32_t>("maxthreads", __lines, 0);
    ConfigParser::stripLinesForKey("maxthreads", __remainingValuesToParse);
    mincoverage = ConfigParser::parse<double>("mincoverage", __lines, 1.0);
    ConfigParser::stripLinesForKey("mincoverage", __remainingValuesToParse);
    minhits = ConfigParser::parse<int32_t>("minhits", __lines, 1000000);
    ConfigParser::stripLinesForKey("minhits", __remainingValuesToParse);
}

bool
InternalProtonType::Search::Memory::Limiter::
operator==(const Limiter& __rhs) const noexcept
{
    return (maxthreads == __rhs.maxthreads &&
            mincoverage == __rhs.mincoverage &&
            minhits == __rhs.minhits);
}

bool
InternalProtonType::Search::Memory::Limiter::
operator!=(const Limiter& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Search::Memory::Limiter::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxthreads");
        __c.setString("type", "int");
        __c.setLong("value", maxthreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("mincoverage");
        __c.setString("type", "double");
        __c.setDouble("value", mincoverage);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("minhits");
        __c.setString("type", "int");
        __c.setLong("value", minhits);
    }
}

InternalProtonType::Search::Memory::Limiter::Limiter(const vespalib::slime::Inspector & __inspector)
{
    maxthreads = __inspector["maxthreads"]["value"].asLong();
    mincoverage = __inspector["mincoverage"]["value"].asDouble();
    minhits = __inspector["minhits"]["value"].asLong();
}

InternalProtonType::Search::Memory::Limiter::Limiter(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    maxthreads = ::config::internal::ValueConverter<int32_t>()(__inspector["maxthreads"], 0);
    mincoverage = ::config::internal::ValueConverter<double>()(__inspector["mincoverage"], 1.0);
    minhits = ::config::internal::ValueConverter<int32_t>()(__inspector["minhits"], 1000000);
}

InternalProtonType::Search::Memory::Memory() noexcept
    : limiter()
{
}

InternalProtonType::Search::Memory::Memory(const Memory & __rhs) noexcept = default;
InternalProtonType::Search::Memory & InternalProtonType::Search::Memory::operator = (const Memory & __rhs) noexcept = default;
InternalProtonType::Search::Memory::Memory(Memory && __rhs) noexcept = default;
InternalProtonType::Search::Memory & InternalProtonType::Search::Memory::operator = (Memory && __rhs) noexcept = default;
InternalProtonType::Search::Memory::~Memory() = default; 
InternalProtonType::Search::Memory::Memory(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    limiter = ConfigParser::parseStruct<Limiter>("limiter", __lines);
    ConfigParser::stripLinesForKey("limiter", __remainingValuesToParse);
}

bool
InternalProtonType::Search::Memory::operator==(const Memory& __rhs) const noexcept
{
    return (limiter == __rhs.limiter);
}

bool
InternalProtonType::Search::Memory::operator!=(const Memory& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Search::Memory::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("limiter");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        limiter.serialize(__c2);
    }
}

InternalProtonType::Search::Memory::Memory(const vespalib::slime::Inspector & __inspector)
{
    limiter = Limiter(__inspector["limiter"]["value"]);
}

InternalProtonType::Search::Memory::Memory(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    limiter = ::config::internal::ValueConverter<Limiter>()(__inspector["limiter"]);
}

InternalProtonType::Search::Search()
    : async(true),
      mmap(),
      memory()
{
}

InternalProtonType::Search::Search(const Search & __rhs) = default;
InternalProtonType::Search & InternalProtonType::Search::operator = (const Search & __rhs) = default;
InternalProtonType::Search::Search(Search && __rhs) noexcept = default;
InternalProtonType::Search & InternalProtonType::Search::operator = (Search && __rhs) noexcept = default;
InternalProtonType::Search::~Search() = default; 
InternalProtonType::Search::Search(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    async = ConfigParser::parse<bool>("async", __lines, true);
    ConfigParser::stripLinesForKey("async", __remainingValuesToParse);
    mmap = ConfigParser::parseStruct<Mmap>("mmap", __lines);
    ConfigParser::stripLinesForKey("mmap", __remainingValuesToParse);
    memory = ConfigParser::parseStruct<Memory>("memory", __lines);
    ConfigParser::stripLinesForKey("memory", __remainingValuesToParse);
}

bool
InternalProtonType::Search::operator==(const Search& __rhs) const noexcept
{
    return (async == __rhs.async &&
            mmap == __rhs.mmap &&
            memory == __rhs.memory);
}

bool
InternalProtonType::Search::operator!=(const Search& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Search::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("async");
        __c.setString("type", "bool");
        __c.setBool("value", async);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("mmap");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        mmap.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("memory");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        memory.serialize(__c2);
    }
}

InternalProtonType::Search::Search(const vespalib::slime::Inspector & __inspector)
{
    async = __inspector["async"]["value"].asBool();
    mmap = Mmap(__inspector["mmap"]["value"]);
    memory = Memory(__inspector["memory"]["value"]);
}

InternalProtonType::Search::Search(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    async = ::config::internal::ValueConverter<bool>()(__inspector["async"], true);
    mmap = ::config::internal::ValueConverter<Mmap>()(__inspector["mmap"]);
    memory = ::config::internal::ValueConverter<Memory>()(__inspector["memory"]);
}

InternalProtonType::Docsum::Docsum() noexcept
    : async(true)
{
}

InternalProtonType::Docsum::Docsum(const Docsum & __rhs) noexcept = default;
InternalProtonType::Docsum & InternalProtonType::Docsum::operator = (const Docsum & __rhs) noexcept = default;
InternalProtonType::Docsum::Docsum(Docsum && __rhs) noexcept = default;
InternalProtonType::Docsum & InternalProtonType::Docsum::operator = (Docsum && __rhs) noexcept = default;
InternalProtonType::Docsum::~Docsum() = default; 
InternalProtonType::Docsum::Docsum(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    async = ConfigParser::parse<bool>("async", __lines, true);
    ConfigParser::stripLinesForKey("async", __remainingValuesToParse);
}

bool
InternalProtonType::Docsum::operator==(const Docsum& __rhs) const noexcept
{
    return (async == __rhs.async);
}

bool
InternalProtonType::Docsum::operator!=(const Docsum& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Docsum::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("async");
        __c.setString("type", "bool");
        __c.setBool("value", async);
    }
}

InternalProtonType::Docsum::Docsum(const vespalib::slime::Inspector & __inspector)
{
    async = __inspector["async"]["value"].asBool();
}

InternalProtonType::Docsum::Docsum(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    async = ::config::internal::ValueConverter<bool>()(__inspector["async"], true);
}

InternalProtonType::ValidateAndSanitizeDocstore
InternalProtonType::getValidateAndSanitizeDocstore(const vespalib::string& name)
{
    if (name == "NO") {
        return ValidateAndSanitizeDocstore::NO;
    } else if (name == "YES") {
        return ValidateAndSanitizeDocstore::YES;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::getValidateAndSanitizeDocstoreName(ValidateAndSanitizeDocstore t)
{
    switch (t) {
        case ValidateAndSanitizeDocstore::NO: return "NO";
        case ValidateAndSanitizeDocstore::YES: return "YES";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::ValidateAndSanitizeDocstore InternalProtonType::InternalValidateAndSanitizeDocstoreConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::getValidateAndSanitizeDocstore(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::ValidateAndSanitizeDocstore InternalProtonType::InternalValidateAndSanitizeDocstoreConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::getValidateAndSanitizeDocstore(__inspector.asString().make_string());
}
InternalProtonType::ValidateAndSanitizeDocstore InternalProtonType::InternalValidateAndSanitizeDocstoreConverter::operator()(const ::vespalib::slime::Inspector & __inspector, ValidateAndSanitizeDocstore __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::getValidateAndSanitizeDocstore(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Flush::Strategy
InternalProtonType::Flush::getStrategy(const vespalib::string& name)
{
    if (name == "SIMPLE") {
        return Strategy::SIMPLE;
    } else if (name == "MEMORY") {
        return Strategy::MEMORY;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Flush::getStrategyName(Strategy t)
{
    switch (t) {
        case Strategy::SIMPLE: return "SIMPLE";
        case Strategy::MEMORY: return "MEMORY";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Flush::Strategy InternalProtonType::Flush::InternalStrategyConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Flush::getStrategy(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Flush::Strategy InternalProtonType::Flush::InternalStrategyConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Flush::getStrategy(__inspector.asString().make_string());
}
InternalProtonType::Flush::Strategy InternalProtonType::Flush::InternalStrategyConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Strategy __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Flush::getStrategy(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Flush::Memory::Each::Each() noexcept
    : maxmemory(1073741824),
      diskbloatfactor(0.25)
{
}

InternalProtonType::Flush::Memory::Each::Each(const Each & __rhs) noexcept = default;
InternalProtonType::Flush::Memory::Each & InternalProtonType::Flush::Memory::Each::operator = (const Each & __rhs) noexcept = default;
InternalProtonType::Flush::Memory::Each::Each(Each && __rhs) noexcept = default;
InternalProtonType::Flush::Memory::Each & InternalProtonType::Flush::Memory::Each::operator = (Each && __rhs) noexcept = default;
InternalProtonType::Flush::Memory::Each::~Each() = default; 
InternalProtonType::Flush::Memory::Each::Each(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    maxmemory = ConfigParser::parse<int64_t>("maxmemory", __lines, 1073741824);
    ConfigParser::stripLinesForKey("maxmemory", __remainingValuesToParse);
    diskbloatfactor = ConfigParser::parse<double>("diskbloatfactor", __lines, 0.25);
    ConfigParser::stripLinesForKey("diskbloatfactor", __remainingValuesToParse);
}

bool
InternalProtonType::Flush::Memory::Each::operator==(const Each& __rhs) const noexcept
{
    return (maxmemory == __rhs.maxmemory &&
            diskbloatfactor == __rhs.diskbloatfactor);
}

bool
InternalProtonType::Flush::Memory::Each::operator!=(const Each& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Flush::Memory::Each::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxmemory");
        __c.setString("type", "long");
        __c.setLong("value", maxmemory);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("diskbloatfactor");
        __c.setString("type", "double");
        __c.setDouble("value", diskbloatfactor);
    }
}

InternalProtonType::Flush::Memory::Each::Each(const vespalib::slime::Inspector & __inspector)
{
    maxmemory = __inspector["maxmemory"]["value"].asLong();
    diskbloatfactor = __inspector["diskbloatfactor"]["value"].asDouble();
}

InternalProtonType::Flush::Memory::Each::Each(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    maxmemory = ::config::internal::ValueConverter<int64_t>()(__inspector["maxmemory"], 1073741824);
    diskbloatfactor = ::config::internal::ValueConverter<double>()(__inspector["diskbloatfactor"], 0.25);
}

InternalProtonType::Flush::Memory::Maxage::Maxage() noexcept
    : time(111600.0)
{
}

InternalProtonType::Flush::Memory::Maxage::Maxage(const Maxage & __rhs) noexcept = default;
InternalProtonType::Flush::Memory::Maxage & InternalProtonType::Flush::Memory::Maxage::operator = (const Maxage & __rhs) noexcept = default;
InternalProtonType::Flush::Memory::Maxage::Maxage(Maxage && __rhs) noexcept = default;
InternalProtonType::Flush::Memory::Maxage & InternalProtonType::Flush::Memory::Maxage::operator = (Maxage && __rhs) noexcept = default;
InternalProtonType::Flush::Memory::Maxage::~Maxage() = default; 
InternalProtonType::Flush::Memory::Maxage::Maxage(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    time = ConfigParser::parse<double>("time", __lines, 111600.0);
    ConfigParser::stripLinesForKey("time", __remainingValuesToParse);
}

bool
InternalProtonType::Flush::Memory::Maxage::operator==(const Maxage& __rhs) const noexcept
{
    return (time == __rhs.time);
}

bool
InternalProtonType::Flush::Memory::Maxage::operator!=(const Maxage& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Flush::Memory::Maxage::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("time");
        __c.setString("type", "double");
        __c.setDouble("value", time);
    }
}

InternalProtonType::Flush::Memory::Maxage::Maxage(const vespalib::slime::Inspector & __inspector)
{
    time = __inspector["time"]["value"].asDouble();
}

InternalProtonType::Flush::Memory::Maxage::Maxage(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    time = ::config::internal::ValueConverter<double>()(__inspector["time"], 111600.0);
}

InternalProtonType::Flush::Memory::Conservative::Conservative() noexcept
    : memorylimitfactor(0.5),
      disklimitfactor(0.5),
      highwatermarkfactor(0.95),
      lowwatermarkfactor(0.9)
{
}

InternalProtonType::Flush::Memory::Conservative::Conservative(const Conservative & __rhs) noexcept = default;
InternalProtonType::Flush::Memory::Conservative & InternalProtonType::Flush::Memory::Conservative::operator = (const Conservative & __rhs) noexcept = default;
InternalProtonType::Flush::Memory::Conservative::Conservative(Conservative && __rhs) noexcept = default;
InternalProtonType::Flush::Memory::Conservative & InternalProtonType::Flush::Memory::Conservative::operator = (Conservative && __rhs) noexcept = default;
InternalProtonType::Flush::Memory::Conservative::~Conservative() = default; 
InternalProtonType::Flush::Memory::Conservative::Conservative(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    memorylimitfactor = ConfigParser::parse<double>("memorylimitfactor", __lines, 0.5);
    ConfigParser::stripLinesForKey("memorylimitfactor", __remainingValuesToParse);
    disklimitfactor = ConfigParser::parse<double>("disklimitfactor", __lines, 0.5);
    ConfigParser::stripLinesForKey("disklimitfactor", __remainingValuesToParse);
    highwatermarkfactor = ConfigParser::parse<double>("highwatermarkfactor", __lines, 0.95);
    ConfigParser::stripLinesForKey("highwatermarkfactor", __remainingValuesToParse);
    lowwatermarkfactor = ConfigParser::parse<double>("lowwatermarkfactor", __lines, 0.9);
    ConfigParser::stripLinesForKey("lowwatermarkfactor", __remainingValuesToParse);
}

bool
InternalProtonType::Flush::Memory::Conservative::
operator==(const Conservative& __rhs) const noexcept
{
    return (memorylimitfactor == __rhs.memorylimitfactor &&
            disklimitfactor == __rhs.disklimitfactor &&
            highwatermarkfactor == __rhs.highwatermarkfactor &&
            lowwatermarkfactor == __rhs.lowwatermarkfactor);
}

bool
InternalProtonType::Flush::Memory::Conservative::
operator!=(const Conservative& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Flush::Memory::Conservative::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("memorylimitfactor");
        __c.setString("type", "double");
        __c.setDouble("value", memorylimitfactor);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("disklimitfactor");
        __c.setString("type", "double");
        __c.setDouble("value", disklimitfactor);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("highwatermarkfactor");
        __c.setString("type", "double");
        __c.setDouble("value", highwatermarkfactor);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("lowwatermarkfactor");
        __c.setString("type", "double");
        __c.setDouble("value", lowwatermarkfactor);
    }
}

InternalProtonType::Flush::Memory::Conservative::Conservative(const vespalib::slime::Inspector & __inspector)
{
    memorylimitfactor = __inspector["memorylimitfactor"]["value"].asDouble();
    disklimitfactor = __inspector["disklimitfactor"]["value"].asDouble();
    highwatermarkfactor = __inspector["highwatermarkfactor"]["value"].asDouble();
    lowwatermarkfactor = __inspector["lowwatermarkfactor"]["value"].asDouble();
}

InternalProtonType::Flush::Memory::Conservative::Conservative(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    memorylimitfactor = ::config::internal::ValueConverter<double>()(__inspector["memorylimitfactor"], 0.5);
    disklimitfactor = ::config::internal::ValueConverter<double>()(__inspector["disklimitfactor"], 0.5);
    highwatermarkfactor = ::config::internal::ValueConverter<double>()(__inspector["highwatermarkfactor"], 0.95);
    lowwatermarkfactor = ::config::internal::ValueConverter<double>()(__inspector["lowwatermarkfactor"], 0.9);
}

InternalProtonType::Flush::Memory::Memory() noexcept
    : maxmemory(4294967296),
      diskbloatfactor(0.25),
      maxtlssize(21474836480),
      each(),
      maxage(),
      conservative()
{
}

InternalProtonType::Flush::Memory::Memory(const Memory & __rhs) noexcept = default;
InternalProtonType::Flush::Memory & InternalProtonType::Flush::Memory::operator = (const Memory & __rhs) noexcept = default;
InternalProtonType::Flush::Memory::Memory(Memory && __rhs) noexcept = default;
InternalProtonType::Flush::Memory & InternalProtonType::Flush::Memory::operator = (Memory && __rhs) noexcept = default;
InternalProtonType::Flush::Memory::~Memory() = default; 
InternalProtonType::Flush::Memory::Memory(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    maxmemory = ConfigParser::parse<int64_t>("maxmemory", __lines, 4294967296);
    ConfigParser::stripLinesForKey("maxmemory", __remainingValuesToParse);
    diskbloatfactor = ConfigParser::parse<double>("diskbloatfactor", __lines, 0.25);
    ConfigParser::stripLinesForKey("diskbloatfactor", __remainingValuesToParse);
    maxtlssize = ConfigParser::parse<int64_t>("maxtlssize", __lines, 21474836480);
    ConfigParser::stripLinesForKey("maxtlssize", __remainingValuesToParse);
    each = ConfigParser::parseStruct<Each>("each", __lines);
    ConfigParser::stripLinesForKey("each", __remainingValuesToParse);
    maxage = ConfigParser::parseStruct<Maxage>("maxage", __lines);
    ConfigParser::stripLinesForKey("maxage", __remainingValuesToParse);
    conservative = ConfigParser::parseStruct<Conservative>("conservative", __lines);
    ConfigParser::stripLinesForKey("conservative", __remainingValuesToParse);
}

bool
InternalProtonType::Flush::Memory::operator==(const Memory& __rhs) const noexcept
{
    return (maxmemory == __rhs.maxmemory &&
            diskbloatfactor == __rhs.diskbloatfactor &&
            maxtlssize == __rhs.maxtlssize &&
            each == __rhs.each &&
            maxage == __rhs.maxage &&
            conservative == __rhs.conservative);
}

bool
InternalProtonType::Flush::Memory::operator!=(const Memory& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Flush::Memory::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxmemory");
        __c.setString("type", "long");
        __c.setLong("value", maxmemory);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("diskbloatfactor");
        __c.setString("type", "double");
        __c.setDouble("value", diskbloatfactor);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxtlssize");
        __c.setString("type", "long");
        __c.setLong("value", maxtlssize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("each");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        each.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxage");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        maxage.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("conservative");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        conservative.serialize(__c2);
    }
}

InternalProtonType::Flush::Memory::Memory(const vespalib::slime::Inspector & __inspector)
{
    maxmemory = __inspector["maxmemory"]["value"].asLong();
    diskbloatfactor = __inspector["diskbloatfactor"]["value"].asDouble();
    maxtlssize = __inspector["maxtlssize"]["value"].asLong();
    each = Each(__inspector["each"]["value"]);
    maxage = Maxage(__inspector["maxage"]["value"]);
    conservative = Conservative(__inspector["conservative"]["value"]);
}

InternalProtonType::Flush::Memory::Memory(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    maxmemory = ::config::internal::ValueConverter<int64_t>()(__inspector["maxmemory"], 4294967296);
    diskbloatfactor = ::config::internal::ValueConverter<double>()(__inspector["diskbloatfactor"], 0.25);
    maxtlssize = ::config::internal::ValueConverter<int64_t>()(__inspector["maxtlssize"], 21474836480);
    each = ::config::internal::ValueConverter<Each>()(__inspector["each"]);
    maxage = ::config::internal::ValueConverter<Maxage>()(__inspector["maxage"]);
    conservative = ::config::internal::ValueConverter<Conservative>()(__inspector["conservative"]);
}

InternalProtonType::Flush::Preparerestart::Preparerestart() noexcept
    : replaycost(8.0),
      replayoperationcost(3000.0),
      writecost(1.0)
{
}

InternalProtonType::Flush::Preparerestart::Preparerestart(const Preparerestart & __rhs) noexcept = default;
InternalProtonType::Flush::Preparerestart & InternalProtonType::Flush::Preparerestart::operator = (const Preparerestart & __rhs) noexcept = default;
InternalProtonType::Flush::Preparerestart::Preparerestart(Preparerestart && __rhs) noexcept = default;
InternalProtonType::Flush::Preparerestart & InternalProtonType::Flush::Preparerestart::operator = (Preparerestart && __rhs) noexcept = default;
InternalProtonType::Flush::Preparerestart::~Preparerestart() = default; 
InternalProtonType::Flush::Preparerestart::Preparerestart(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    replaycost = ConfigParser::parse<double>("replaycost", __lines, 8.0);
    ConfigParser::stripLinesForKey("replaycost", __remainingValuesToParse);
    replayoperationcost = ConfigParser::parse<double>("replayoperationcost", __lines, 3000.0);
    ConfigParser::stripLinesForKey("replayoperationcost", __remainingValuesToParse);
    writecost = ConfigParser::parse<double>("writecost", __lines, 1.0);
    ConfigParser::stripLinesForKey("writecost", __remainingValuesToParse);
}

bool
InternalProtonType::Flush::Preparerestart::
operator==(const Preparerestart& __rhs) const noexcept
{
    return (replaycost == __rhs.replaycost &&
            replayoperationcost == __rhs.replayoperationcost &&
            writecost == __rhs.writecost);
}

bool
InternalProtonType::Flush::Preparerestart::
operator!=(const Preparerestart& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Flush::Preparerestart::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("replaycost");
        __c.setString("type", "double");
        __c.setDouble("value", replaycost);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("replayoperationcost");
        __c.setString("type", "double");
        __c.setDouble("value", replayoperationcost);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("writecost");
        __c.setString("type", "double");
        __c.setDouble("value", writecost);
    }
}

InternalProtonType::Flush::Preparerestart::Preparerestart(const vespalib::slime::Inspector & __inspector)
{
    replaycost = __inspector["replaycost"]["value"].asDouble();
    replayoperationcost = __inspector["replayoperationcost"]["value"].asDouble();
    writecost = __inspector["writecost"]["value"].asDouble();
}

InternalProtonType::Flush::Preparerestart::Preparerestart(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    replaycost = ::config::internal::ValueConverter<double>()(__inspector["replaycost"], 8.0);
    replayoperationcost = ::config::internal::ValueConverter<double>()(__inspector["replayoperationcost"], 3000.0);
    writecost = ::config::internal::ValueConverter<double>()(__inspector["writecost"], 1.0);
}

InternalProtonType::Flush::Flush() noexcept
    : maxconcurrent(2),
      idleinterval(10.0),
      strategy(Strategy::MEMORY),
      memory(),
      preparerestart()
{
}

InternalProtonType::Flush::Flush(const Flush & __rhs) noexcept = default;
InternalProtonType::Flush & InternalProtonType::Flush::operator = (const Flush & __rhs) noexcept = default;
InternalProtonType::Flush::Flush(Flush && __rhs) noexcept = default;
InternalProtonType::Flush & InternalProtonType::Flush::operator = (Flush && __rhs) noexcept = default;
InternalProtonType::Flush::~Flush() = default; 
InternalProtonType::Flush::Flush(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    maxconcurrent = ConfigParser::parse<int32_t>("maxconcurrent", __lines, 2);
    ConfigParser::stripLinesForKey("maxconcurrent", __remainingValuesToParse);
    idleinterval = ConfigParser::parse<double>("idleinterval", __lines, 10.0);
    ConfigParser::stripLinesForKey("idleinterval", __remainingValuesToParse);
    strategy = getStrategy(ConfigParser::parse<vespalib::string>("strategy", __lines, "MEMORY"));
    ConfigParser::stripLinesForKey("strategy", __remainingValuesToParse);
    memory = ConfigParser::parseStruct<Memory>("memory", __lines);
    ConfigParser::stripLinesForKey("memory", __remainingValuesToParse);
    preparerestart = ConfigParser::parseStruct<Preparerestart>("preparerestart", __lines);
    ConfigParser::stripLinesForKey("preparerestart", __remainingValuesToParse);
}

bool
InternalProtonType::Flush::operator==(const Flush& __rhs) const noexcept
{
    return (maxconcurrent == __rhs.maxconcurrent &&
            idleinterval == __rhs.idleinterval &&
            strategy == __rhs.strategy &&
            memory == __rhs.memory &&
            preparerestart == __rhs.preparerestart);
}

bool
InternalProtonType::Flush::operator!=(const Flush& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Flush::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxconcurrent");
        __c.setString("type", "int");
        __c.setLong("value", maxconcurrent);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("idleinterval");
        __c.setString("type", "double");
        __c.setDouble("value", idleinterval);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("strategy");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getStrategyName(strategy)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("memory");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        memory.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("preparerestart");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        preparerestart.serialize(__c2);
    }
}

InternalProtonType::Flush::Flush(const vespalib::slime::Inspector & __inspector)
{
    maxconcurrent = __inspector["maxconcurrent"]["value"].asLong();
    idleinterval = __inspector["idleinterval"]["value"].asDouble();
    strategy = getStrategy(__inspector["strategy"]["value"].asString().make_string());
    memory = Memory(__inspector["memory"]["value"]);
    preparerestart = Preparerestart(__inspector["preparerestart"]["value"]);
}

InternalProtonType::Flush::Flush(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    maxconcurrent = ::config::internal::ValueConverter<int32_t>()(__inspector["maxconcurrent"], 2);
    idleinterval = ::config::internal::ValueConverter<double>()(__inspector["idleinterval"], 10.0);
    strategy = InternalStrategyConverter()(__inspector["strategy"], Strategy::MEMORY);
    memory = ::config::internal::ValueConverter<Memory>()(__inspector["memory"]);
    preparerestart = ::config::internal::ValueConverter<Preparerestart>()(__inspector["preparerestart"]);
}

InternalProtonType::Indexing::Write::Io
InternalProtonType::Indexing::Write::getIo(const vespalib::string& name)
{
    if (name == "NORMAL") {
        return Io::NORMAL;
    } else if (name == "OSYNC") {
        return Io::OSYNC;
    } else if (name == "DIRECTIO") {
        return Io::DIRECTIO;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Indexing::Write::getIoName(Io t)
{
    switch (t) {
        case Io::NORMAL: return "NORMAL";
        case Io::OSYNC: return "OSYNC";
        case Io::DIRECTIO: return "DIRECTIO";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Indexing::Write::Io InternalProtonType::Indexing::Write::InternalIoConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Indexing::Write::getIo(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Indexing::Write::Io InternalProtonType::Indexing::Write::InternalIoConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Indexing::Write::getIo(__inspector.asString().make_string());
}
InternalProtonType::Indexing::Write::Io InternalProtonType::Indexing::Write::InternalIoConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Io __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Indexing::Write::getIo(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Indexing::Write::Write() noexcept
    : io(Io::DIRECTIO)
{
}

InternalProtonType::Indexing::Write::Write(const Write & __rhs) noexcept = default;
InternalProtonType::Indexing::Write & InternalProtonType::Indexing::Write::operator = (const Write & __rhs) noexcept = default;
InternalProtonType::Indexing::Write::Write(Write && __rhs) noexcept = default;
InternalProtonType::Indexing::Write & InternalProtonType::Indexing::Write::operator = (Write && __rhs) noexcept = default;
InternalProtonType::Indexing::Write::~Write() = default; 
InternalProtonType::Indexing::Write::Write(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    io = getIo(ConfigParser::parse<vespalib::string>("io", __lines, "DIRECTIO"));
    ConfigParser::stripLinesForKey("io", __remainingValuesToParse);
}

bool
InternalProtonType::Indexing::Write::operator==(const Write& __rhs) const noexcept
{
    return (io == __rhs.io);
}

bool
InternalProtonType::Indexing::Write::operator!=(const Write& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Indexing::Write::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("io");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getIoName(io)));
    }
}

InternalProtonType::Indexing::Write::Write(const vespalib::slime::Inspector & __inspector)
{
    io = getIo(__inspector["io"]["value"].asString().make_string());
}

InternalProtonType::Indexing::Write::Write(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    io = InternalIoConverter()(__inspector["io"], Io::DIRECTIO);
}

InternalProtonType::Indexing::Read::Io
InternalProtonType::Indexing::Read::getIo(const vespalib::string& name)
{
    if (name == "NORMAL") {
        return Io::NORMAL;
    } else if (name == "DIRECTIO") {
        return Io::DIRECTIO;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Indexing::Read::getIoName(Io t)
{
    switch (t) {
        case Io::NORMAL: return "NORMAL";
        case Io::DIRECTIO: return "DIRECTIO";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Indexing::Read::Io InternalProtonType::Indexing::Read::InternalIoConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Indexing::Read::getIo(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Indexing::Read::Io InternalProtonType::Indexing::Read::InternalIoConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Indexing::Read::getIo(__inspector.asString().make_string());
}
InternalProtonType::Indexing::Read::Io InternalProtonType::Indexing::Read::InternalIoConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Io __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Indexing::Read::getIo(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Indexing::Read::Read() noexcept
    : io(Io::DIRECTIO)
{
}

InternalProtonType::Indexing::Read::Read(const Read & __rhs) noexcept = default;
InternalProtonType::Indexing::Read & InternalProtonType::Indexing::Read::operator = (const Read & __rhs) noexcept = default;
InternalProtonType::Indexing::Read::Read(Read && __rhs) noexcept = default;
InternalProtonType::Indexing::Read & InternalProtonType::Indexing::Read::operator = (Read && __rhs) noexcept = default;
InternalProtonType::Indexing::Read::~Read() = default; 
InternalProtonType::Indexing::Read::Read(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    io = getIo(ConfigParser::parse<vespalib::string>("io", __lines, "DIRECTIO"));
    ConfigParser::stripLinesForKey("io", __remainingValuesToParse);
}

bool
InternalProtonType::Indexing::Read::operator==(const Read& __rhs) const noexcept
{
    return (io == __rhs.io);
}

bool
InternalProtonType::Indexing::Read::operator!=(const Read& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Indexing::Read::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("io");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getIoName(io)));
    }
}

InternalProtonType::Indexing::Read::Read(const vespalib::slime::Inspector & __inspector)
{
    io = getIo(__inspector["io"]["value"].asString().make_string());
}

InternalProtonType::Indexing::Read::Read(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    io = InternalIoConverter()(__inspector["io"], Io::DIRECTIO);
}

InternalProtonType::Indexing::Optimize
InternalProtonType::Indexing::getOptimize(const vespalib::string& name)
{
    if (name == "LATENCY") {
        return Optimize::LATENCY;
    } else if (name == "THROUGHPUT") {
        return Optimize::THROUGHPUT;
    } else if (name == "ADAPTIVE") {
        return Optimize::ADAPTIVE;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Indexing::getOptimizeName(Optimize t)
{
    switch (t) {
        case Optimize::LATENCY: return "LATENCY";
        case Optimize::THROUGHPUT: return "THROUGHPUT";
        case Optimize::ADAPTIVE: return "ADAPTIVE";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Indexing::Optimize InternalProtonType::Indexing::InternalOptimizeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Indexing::getOptimize(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Indexing::Optimize InternalProtonType::Indexing::InternalOptimizeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Indexing::getOptimize(__inspector.asString().make_string());
}
InternalProtonType::Indexing::Optimize InternalProtonType::Indexing::InternalOptimizeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Optimize __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Indexing::getOptimize(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Indexing::Indexing() noexcept
    : write(),
      read(),
      optimize(Optimize::THROUGHPUT),
      tasklimit(-1000),
      kindOfWatermark(0),
      reactiontime(0.001),
      semiunboundtasklimit(1000),
      threads(1)
{
}

InternalProtonType::Indexing::Indexing(const Indexing & __rhs) noexcept = default;
InternalProtonType::Indexing & InternalProtonType::Indexing::operator = (const Indexing & __rhs) noexcept = default;
InternalProtonType::Indexing::Indexing(Indexing && __rhs) noexcept = default;
InternalProtonType::Indexing & InternalProtonType::Indexing::operator = (Indexing && __rhs) noexcept = default;
InternalProtonType::Indexing::~Indexing() = default; 
InternalProtonType::Indexing::Indexing(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    write = ConfigParser::parseStruct<Write>("write", __lines);
    ConfigParser::stripLinesForKey("write", __remainingValuesToParse);
    read = ConfigParser::parseStruct<Read>("read", __lines);
    ConfigParser::stripLinesForKey("read", __remainingValuesToParse);
    optimize = getOptimize(ConfigParser::parse<vespalib::string>("optimize", __lines, "THROUGHPUT"));
    ConfigParser::stripLinesForKey("optimize", __remainingValuesToParse);
    tasklimit = ConfigParser::parse<int32_t>("tasklimit", __lines, -1000);
    ConfigParser::stripLinesForKey("tasklimit", __remainingValuesToParse);
    kindOfWatermark = ConfigParser::parse<int32_t>("kind_of_watermark", __lines, 0);
    ConfigParser::stripLinesForKey("kind_of_watermark", __remainingValuesToParse);
    reactiontime = ConfigParser::parse<double>("reactiontime", __lines, 0.001);
    ConfigParser::stripLinesForKey("reactiontime", __remainingValuesToParse);
    semiunboundtasklimit = ConfigParser::parse<int32_t>("semiunboundtasklimit", __lines, 1000);
    ConfigParser::stripLinesForKey("semiunboundtasklimit", __remainingValuesToParse);
    threads = ConfigParser::parse<int32_t>("threads", __lines, 1);
    ConfigParser::stripLinesForKey("threads", __remainingValuesToParse);
}

bool
InternalProtonType::Indexing::operator==(const Indexing& __rhs) const noexcept
{
    return (write == __rhs.write &&
            read == __rhs.read &&
            optimize == __rhs.optimize &&
            tasklimit == __rhs.tasklimit &&
            kindOfWatermark == __rhs.kindOfWatermark &&
            reactiontime == __rhs.reactiontime &&
            semiunboundtasklimit == __rhs.semiunboundtasklimit &&
            threads == __rhs.threads);
}

bool
InternalProtonType::Indexing::operator!=(const Indexing& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Indexing::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("write");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        write.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("read");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        read.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("optimize");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getOptimizeName(optimize)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tasklimit");
        __c.setString("type", "int");
        __c.setLong("value", tasklimit);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("kind_of_watermark");
        __c.setString("type", "int");
        __c.setLong("value", kindOfWatermark);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("reactiontime");
        __c.setString("type", "double");
        __c.setDouble("value", reactiontime);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("semiunboundtasklimit");
        __c.setString("type", "int");
        __c.setLong("value", semiunboundtasklimit);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("threads");
        __c.setString("type", "int");
        __c.setLong("value", threads);
    }
}

InternalProtonType::Indexing::Indexing(const vespalib::slime::Inspector & __inspector)
{
    write = Write(__inspector["write"]["value"]);
    read = Read(__inspector["read"]["value"]);
    optimize = getOptimize(__inspector["optimize"]["value"].asString().make_string());
    tasklimit = __inspector["tasklimit"]["value"].asLong();
    kindOfWatermark = __inspector["kind_of_watermark"]["value"].asLong();
    reactiontime = __inspector["reactiontime"]["value"].asDouble();
    semiunboundtasklimit = __inspector["semiunboundtasklimit"]["value"].asLong();
    threads = __inspector["threads"]["value"].asLong();
}

InternalProtonType::Indexing::Indexing(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    write = ::config::internal::ValueConverter<Write>()(__inspector["write"]);
    read = ::config::internal::ValueConverter<Read>()(__inspector["read"]);
    optimize = InternalOptimizeConverter()(__inspector["optimize"], Optimize::THROUGHPUT);
    tasklimit = ::config::internal::ValueConverter<int32_t>()(__inspector["tasklimit"], -1000);
    kindOfWatermark = ::config::internal::ValueConverter<int32_t>()(__inspector["kind_of_watermark"], 0);
    reactiontime = ::config::internal::ValueConverter<double>()(__inspector["reactiontime"], 0.001);
    semiunboundtasklimit = ::config::internal::ValueConverter<int32_t>()(__inspector["semiunboundtasklimit"], 1000);
    threads = ::config::internal::ValueConverter<int32_t>()(__inspector["threads"], 1);
}

InternalProtonType::Index::Warmup::Warmup() noexcept
    : time(0.0),
      unpack(false)
{
}

InternalProtonType::Index::Warmup::Warmup(const Warmup & __rhs) noexcept = default;
InternalProtonType::Index::Warmup & InternalProtonType::Index::Warmup::operator = (const Warmup & __rhs) noexcept = default;
InternalProtonType::Index::Warmup::Warmup(Warmup && __rhs) noexcept = default;
InternalProtonType::Index::Warmup & InternalProtonType::Index::Warmup::operator = (Warmup && __rhs) noexcept = default;
InternalProtonType::Index::Warmup::~Warmup() = default; 
InternalProtonType::Index::Warmup::Warmup(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    time = ConfigParser::parse<double>("time", __lines, 0.0);
    ConfigParser::stripLinesForKey("time", __remainingValuesToParse);
    unpack = ConfigParser::parse<bool>("unpack", __lines, false);
    ConfigParser::stripLinesForKey("unpack", __remainingValuesToParse);
}

bool
InternalProtonType::Index::Warmup::operator==(const Warmup& __rhs) const noexcept
{
    return (time == __rhs.time &&
            unpack == __rhs.unpack);
}

bool
InternalProtonType::Index::Warmup::operator!=(const Warmup& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Index::Warmup::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("time");
        __c.setString("type", "double");
        __c.setDouble("value", time);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("unpack");
        __c.setString("type", "bool");
        __c.setBool("value", unpack);
    }
}

InternalProtonType::Index::Warmup::Warmup(const vespalib::slime::Inspector & __inspector)
{
    time = __inspector["time"]["value"].asDouble();
    unpack = __inspector["unpack"]["value"].asBool();
}

InternalProtonType::Index::Warmup::Warmup(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    time = ::config::internal::ValueConverter<double>()(__inspector["time"], 0.0);
    unpack = ::config::internal::ValueConverter<bool>()(__inspector["unpack"], false);
}

InternalProtonType::Index::Cache::Cache() noexcept
    : size(0)
{
}

InternalProtonType::Index::Cache::Cache(const Cache & __rhs) noexcept = default;
InternalProtonType::Index::Cache & InternalProtonType::Index::Cache::operator = (const Cache & __rhs) noexcept = default;
InternalProtonType::Index::Cache::Cache(Cache && __rhs) noexcept = default;
InternalProtonType::Index::Cache & InternalProtonType::Index::Cache::operator = (Cache && __rhs) noexcept = default;
InternalProtonType::Index::Cache::~Cache() = default; 
InternalProtonType::Index::Cache::Cache(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    size = ConfigParser::parse<int64_t>("size", __lines, 0);
    ConfigParser::stripLinesForKey("size", __remainingValuesToParse);
}

bool
InternalProtonType::Index::Cache::operator==(const Cache& __rhs) const noexcept
{
    return (size == __rhs.size);
}

bool
InternalProtonType::Index::Cache::operator!=(const Cache& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Index::Cache::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("size");
        __c.setString("type", "long");
        __c.setLong("value", size);
    }
}

InternalProtonType::Index::Cache::Cache(const vespalib::slime::Inspector & __inspector)
{
    size = __inspector["size"]["value"].asLong();
}

InternalProtonType::Index::Cache::Cache(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    size = ::config::internal::ValueConverter<int64_t>()(__inspector["size"], 0);
}

InternalProtonType::Index::Index() noexcept
    : warmup(),
      maxflushed(2),
      maxflushedretired(20),
      cache()
{
}

InternalProtonType::Index::Index(const Index & __rhs) noexcept = default;
InternalProtonType::Index & InternalProtonType::Index::operator = (const Index & __rhs) noexcept = default;
InternalProtonType::Index::Index(Index && __rhs) noexcept = default;
InternalProtonType::Index & InternalProtonType::Index::operator = (Index && __rhs) noexcept = default;
InternalProtonType::Index::~Index() = default; 
InternalProtonType::Index::Index(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    warmup = ConfigParser::parseStruct<Warmup>("warmup", __lines);
    ConfigParser::stripLinesForKey("warmup", __remainingValuesToParse);
    maxflushed = ConfigParser::parse<int32_t>("maxflushed", __lines, 2);
    ConfigParser::stripLinesForKey("maxflushed", __remainingValuesToParse);
    maxflushedretired = ConfigParser::parse<int32_t>("maxflushedretired", __lines, 20);
    ConfigParser::stripLinesForKey("maxflushedretired", __remainingValuesToParse);
    cache = ConfigParser::parseStruct<Cache>("cache", __lines);
    ConfigParser::stripLinesForKey("cache", __remainingValuesToParse);
}

bool
InternalProtonType::Index::operator==(const Index& __rhs) const noexcept
{
    return (warmup == __rhs.warmup &&
            maxflushed == __rhs.maxflushed &&
            maxflushedretired == __rhs.maxflushedretired &&
            cache == __rhs.cache);
}

bool
InternalProtonType::Index::operator!=(const Index& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Index::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("warmup");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        warmup.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxflushed");
        __c.setString("type", "int");
        __c.setLong("value", maxflushed);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxflushedretired");
        __c.setString("type", "int");
        __c.setLong("value", maxflushedretired);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("cache");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        cache.serialize(__c2);
    }
}

InternalProtonType::Index::Index(const vespalib::slime::Inspector & __inspector)
{
    warmup = Warmup(__inspector["warmup"]["value"]);
    maxflushed = __inspector["maxflushed"]["value"].asLong();
    maxflushedretired = __inspector["maxflushedretired"]["value"].asLong();
    cache = Cache(__inspector["cache"]["value"]);
}

InternalProtonType::Index::Index(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    warmup = ::config::internal::ValueConverter<Warmup>()(__inspector["warmup"]);
    maxflushed = ::config::internal::ValueConverter<int32_t>()(__inspector["maxflushed"], 2);
    maxflushedretired = ::config::internal::ValueConverter<int32_t>()(__inspector["maxflushedretired"], 20);
    cache = ::config::internal::ValueConverter<Cache>()(__inspector["cache"]);
}

InternalProtonType::Attribute::Write::Io
InternalProtonType::Attribute::Write::getIo(const vespalib::string& name)
{
    if (name == "NORMAL") {
        return Io::NORMAL;
    } else if (name == "OSYNC") {
        return Io::OSYNC;
    } else if (name == "DIRECTIO") {
        return Io::DIRECTIO;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Attribute::Write::getIoName(Io t)
{
    switch (t) {
        case Io::NORMAL: return "NORMAL";
        case Io::OSYNC: return "OSYNC";
        case Io::DIRECTIO: return "DIRECTIO";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Attribute::Write::Io InternalProtonType::Attribute::Write::InternalIoConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Attribute::Write::getIo(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Attribute::Write::Io InternalProtonType::Attribute::Write::InternalIoConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Attribute::Write::getIo(__inspector.asString().make_string());
}
InternalProtonType::Attribute::Write::Io InternalProtonType::Attribute::Write::InternalIoConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Io __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Attribute::Write::getIo(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Attribute::Write::Write() noexcept
    : io(Io::DIRECTIO)
{
}

InternalProtonType::Attribute::Write::Write(const Write & __rhs) noexcept = default;
InternalProtonType::Attribute::Write & InternalProtonType::Attribute::Write::operator = (const Write & __rhs) noexcept = default;
InternalProtonType::Attribute::Write::Write(Write && __rhs) noexcept = default;
InternalProtonType::Attribute::Write & InternalProtonType::Attribute::Write::operator = (Write && __rhs) noexcept = default;
InternalProtonType::Attribute::Write::~Write() = default; 
InternalProtonType::Attribute::Write::Write(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    io = getIo(ConfigParser::parse<vespalib::string>("io", __lines, "DIRECTIO"));
    ConfigParser::stripLinesForKey("io", __remainingValuesToParse);
}

bool
InternalProtonType::Attribute::Write::operator==(const Write& __rhs) const noexcept
{
    return (io == __rhs.io);
}

bool
InternalProtonType::Attribute::Write::operator!=(const Write& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Attribute::Write::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("io");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getIoName(io)));
    }
}

InternalProtonType::Attribute::Write::Write(const vespalib::slime::Inspector & __inspector)
{
    io = getIo(__inspector["io"]["value"].asString().make_string());
}

InternalProtonType::Attribute::Write::Write(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    io = InternalIoConverter()(__inspector["io"], Io::DIRECTIO);
}

InternalProtonType::Attribute::Attribute() noexcept
    : write()
{
}

InternalProtonType::Attribute::Attribute(const Attribute & __rhs) noexcept = default;
InternalProtonType::Attribute & InternalProtonType::Attribute::operator = (const Attribute & __rhs) noexcept = default;
InternalProtonType::Attribute::Attribute(Attribute && __rhs) noexcept = default;
InternalProtonType::Attribute & InternalProtonType::Attribute::operator = (Attribute && __rhs) noexcept = default;
InternalProtonType::Attribute::~Attribute() = default; 
InternalProtonType::Attribute::Attribute(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    write = ConfigParser::parseStruct<Write>("write", __lines);
    ConfigParser::stripLinesForKey("write", __remainingValuesToParse);
}

bool
InternalProtonType::Attribute::operator==(const Attribute& __rhs) const noexcept
{
    return (write == __rhs.write);
}

bool
InternalProtonType::Attribute::operator!=(const Attribute& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Attribute::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("write");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        write.serialize(__c2);
    }
}

InternalProtonType::Attribute::Attribute(const vespalib::slime::Inspector & __inspector)
{
    write = Write(__inspector["write"]["value"]);
}

InternalProtonType::Attribute::Attribute(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    write = ::config::internal::ValueConverter<Write>()(__inspector["write"]);
}

InternalProtonType::Grouping::Sessionmanager::Pruning::Pruning() noexcept
    : interval(1.0)
{
}

InternalProtonType::Grouping::Sessionmanager::Pruning::Pruning(const Pruning & __rhs) noexcept = default;
InternalProtonType::Grouping::Sessionmanager::Pruning & InternalProtonType::Grouping::Sessionmanager::Pruning::operator = (const Pruning & __rhs) noexcept = default;
InternalProtonType::Grouping::Sessionmanager::Pruning::Pruning(Pruning && __rhs) noexcept = default;
InternalProtonType::Grouping::Sessionmanager::Pruning & InternalProtonType::Grouping::Sessionmanager::Pruning::operator = (Pruning && __rhs) noexcept = default;
InternalProtonType::Grouping::Sessionmanager::Pruning::~Pruning() = default; 
InternalProtonType::Grouping::Sessionmanager::Pruning::Pruning(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    interval = ConfigParser::parse<double>("interval", __lines, 1.0);
    ConfigParser::stripLinesForKey("interval", __remainingValuesToParse);
}

bool
InternalProtonType::Grouping::Sessionmanager::Pruning::
operator==(const Pruning& __rhs) const noexcept
{
    return (interval == __rhs.interval);
}

bool
InternalProtonType::Grouping::Sessionmanager::Pruning::
operator!=(const Pruning& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Grouping::Sessionmanager::Pruning::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("interval");
        __c.setString("type", "double");
        __c.setDouble("value", interval);
    }
}

InternalProtonType::Grouping::Sessionmanager::Pruning::Pruning(const vespalib::slime::Inspector & __inspector)
{
    interval = __inspector["interval"]["value"].asDouble();
}

InternalProtonType::Grouping::Sessionmanager::Pruning::Pruning(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    interval = ::config::internal::ValueConverter<double>()(__inspector["interval"], 1.0);
}

InternalProtonType::Grouping::Sessionmanager::Sessionmanager() noexcept
    : maxentries(500),
      pruning()
{
}

InternalProtonType::Grouping::Sessionmanager::Sessionmanager(const Sessionmanager & __rhs) noexcept = default;
InternalProtonType::Grouping::Sessionmanager & InternalProtonType::Grouping::Sessionmanager::operator = (const Sessionmanager & __rhs) noexcept = default;
InternalProtonType::Grouping::Sessionmanager::Sessionmanager(Sessionmanager && __rhs) noexcept = default;
InternalProtonType::Grouping::Sessionmanager & InternalProtonType::Grouping::Sessionmanager::operator = (Sessionmanager && __rhs) noexcept = default;
InternalProtonType::Grouping::Sessionmanager::~Sessionmanager() = default; 
InternalProtonType::Grouping::Sessionmanager::Sessionmanager(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    maxentries = ConfigParser::parse<int32_t>("maxentries", __lines, 500);
    ConfigParser::stripLinesForKey("maxentries", __remainingValuesToParse);
    pruning = ConfigParser::parseStruct<Pruning>("pruning", __lines);
    ConfigParser::stripLinesForKey("pruning", __remainingValuesToParse);
}

bool
InternalProtonType::Grouping::Sessionmanager::
operator==(const Sessionmanager& __rhs) const noexcept
{
    return (maxentries == __rhs.maxentries &&
            pruning == __rhs.pruning);
}

bool
InternalProtonType::Grouping::Sessionmanager::
operator!=(const Sessionmanager& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Grouping::Sessionmanager::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxentries");
        __c.setString("type", "int");
        __c.setLong("value", maxentries);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("pruning");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        pruning.serialize(__c2);
    }
}

InternalProtonType::Grouping::Sessionmanager::Sessionmanager(const vespalib::slime::Inspector & __inspector)
{
    maxentries = __inspector["maxentries"]["value"].asLong();
    pruning = Pruning(__inspector["pruning"]["value"]);
}

InternalProtonType::Grouping::Sessionmanager::Sessionmanager(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    maxentries = ::config::internal::ValueConverter<int32_t>()(__inspector["maxentries"], 500);
    pruning = ::config::internal::ValueConverter<Pruning>()(__inspector["pruning"]);
}

InternalProtonType::Grouping::Grouping() noexcept
    : sessionmanager()
{
}

InternalProtonType::Grouping::Grouping(const Grouping & __rhs) noexcept = default;
InternalProtonType::Grouping & InternalProtonType::Grouping::operator = (const Grouping & __rhs) noexcept = default;
InternalProtonType::Grouping::Grouping(Grouping && __rhs) noexcept = default;
InternalProtonType::Grouping & InternalProtonType::Grouping::operator = (Grouping && __rhs) noexcept = default;
InternalProtonType::Grouping::~Grouping() = default; 
InternalProtonType::Grouping::Grouping(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    sessionmanager = ConfigParser::parseStruct<Sessionmanager>("sessionmanager", __lines);
    ConfigParser::stripLinesForKey("sessionmanager", __remainingValuesToParse);
}

bool
InternalProtonType::Grouping::operator==(const Grouping& __rhs) const noexcept
{
    return (sessionmanager == __rhs.sessionmanager);
}

bool
InternalProtonType::Grouping::operator!=(const Grouping& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Grouping::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("sessionmanager");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        sessionmanager.serialize(__c2);
    }
}

InternalProtonType::Grouping::Grouping(const vespalib::slime::Inspector & __inspector)
{
    sessionmanager = Sessionmanager(__inspector["sessionmanager"]["value"]);
}

InternalProtonType::Grouping::Grouping(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    sessionmanager = ::config::internal::ValueConverter<Sessionmanager>()(__inspector["sessionmanager"]);
}

InternalProtonType::Distribution::Distribution() noexcept
    : redundancy(1),
      searchablecopies(1)
{
}

InternalProtonType::Distribution::Distribution(const Distribution & __rhs) noexcept = default;
InternalProtonType::Distribution & InternalProtonType::Distribution::operator = (const Distribution & __rhs) noexcept = default;
InternalProtonType::Distribution::Distribution(Distribution && __rhs) noexcept = default;
InternalProtonType::Distribution & InternalProtonType::Distribution::operator = (Distribution && __rhs) noexcept = default;
InternalProtonType::Distribution::~Distribution() = default; 
InternalProtonType::Distribution::Distribution(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    redundancy = ConfigParser::parse<int64_t>("redundancy", __lines, 1);
    ConfigParser::stripLinesForKey("redundancy", __remainingValuesToParse);
    searchablecopies = ConfigParser::parse<int64_t>("searchablecopies", __lines, 1);
    ConfigParser::stripLinesForKey("searchablecopies", __remainingValuesToParse);
}

bool
InternalProtonType::Distribution::operator==(const Distribution& __rhs) const noexcept
{
    return (redundancy == __rhs.redundancy &&
            searchablecopies == __rhs.searchablecopies);
}

bool
InternalProtonType::Distribution::operator!=(const Distribution& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Distribution::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("redundancy");
        __c.setString("type", "long");
        __c.setLong("value", redundancy);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("searchablecopies");
        __c.setString("type", "long");
        __c.setLong("value", searchablecopies);
    }
}

InternalProtonType::Distribution::Distribution(const vespalib::slime::Inspector & __inspector)
{
    redundancy = __inspector["redundancy"]["value"].asLong();
    searchablecopies = __inspector["searchablecopies"]["value"].asLong();
}

InternalProtonType::Distribution::Distribution(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    redundancy = ::config::internal::ValueConverter<int64_t>()(__inspector["redundancy"], 1);
    searchablecopies = ::config::internal::ValueConverter<int64_t>()(__inspector["searchablecopies"], 1);
}

InternalProtonType::Summary::Cache::Compression::Type
InternalProtonType::Summary::Cache::Compression::getType(const vespalib::string& name)
{
    if (name == "NONE") {
        return Type::NONE;
    } else if (name == "LZ4") {
        return Type::LZ4;
    } else if (name == "ZSTD") {
        return Type::ZSTD;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Summary::Cache::Compression::getTypeName(Type t)
{
    switch (t) {
        case Type::NONE: return "NONE";
        case Type::LZ4: return "LZ4";
        case Type::ZSTD: return "ZSTD";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Summary::Cache::Compression::Type InternalProtonType::Summary::Cache::Compression::InternalTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Cache::Compression::getType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Summary::Cache::Compression::Type InternalProtonType::Summary::Cache::Compression::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Summary::Cache::Compression::getType(__inspector.asString().make_string());
}
InternalProtonType::Summary::Cache::Compression::Type InternalProtonType::Summary::Cache::Compression::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Cache::Compression::getType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Summary::Cache::Compression::Compression() noexcept
    : type(Type::LZ4),
      level(6)
{
}

InternalProtonType::Summary::Cache::Compression::Compression(const Compression & __rhs) noexcept = default;
InternalProtonType::Summary::Cache::Compression & InternalProtonType::Summary::Cache::Compression::operator = (const Compression & __rhs) noexcept = default;
InternalProtonType::Summary::Cache::Compression::Compression(Compression && __rhs) noexcept = default;
InternalProtonType::Summary::Cache::Compression & InternalProtonType::Summary::Cache::Compression::operator = (Compression && __rhs) noexcept = default;
InternalProtonType::Summary::Cache::Compression::~Compression() = default; 
InternalProtonType::Summary::Cache::Compression::Compression(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    type = getType(ConfigParser::parse<vespalib::string>("type", __lines, "LZ4"));
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
    level = ConfigParser::parse<int32_t>("level", __lines, 6);
    ConfigParser::stripLinesForKey("level", __remainingValuesToParse);
}

bool
InternalProtonType::Summary::Cache::Compression::
operator==(const Compression& __rhs) const noexcept
{
    return (type == __rhs.type &&
            level == __rhs.level);
}

bool
InternalProtonType::Summary::Cache::Compression::
operator!=(const Compression& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Summary::Cache::Compression::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getTypeName(type)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("level");
        __c.setString("type", "int");
        __c.setLong("value", level);
    }
}

InternalProtonType::Summary::Cache::Compression::Compression(const vespalib::slime::Inspector & __inspector)
{
    type = getType(__inspector["type"]["value"].asString().make_string());
    level = __inspector["level"]["value"].asLong();
}

InternalProtonType::Summary::Cache::Compression::Compression(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    type = InternalTypeConverter()(__inspector["type"], Type::LZ4);
    level = ::config::internal::ValueConverter<int32_t>()(__inspector["level"], 6);
}

InternalProtonType::Summary::Cache::UpdateStrategy
InternalProtonType::Summary::Cache::getUpdateStrategy(const vespalib::string& name)
{
    if (name == "INVALIDATE") {
        return UpdateStrategy::INVALIDATE;
    } else if (name == "UPDATE") {
        return UpdateStrategy::UPDATE;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Summary::Cache::getUpdateStrategyName(UpdateStrategy t)
{
    switch (t) {
        case UpdateStrategy::INVALIDATE: return "INVALIDATE";
        case UpdateStrategy::UPDATE: return "UPDATE";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Summary::Cache::UpdateStrategy InternalProtonType::Summary::Cache::InternalUpdateStrategyConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Cache::getUpdateStrategy(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Summary::Cache::UpdateStrategy InternalProtonType::Summary::Cache::InternalUpdateStrategyConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Summary::Cache::getUpdateStrategy(__inspector.asString().make_string());
}
InternalProtonType::Summary::Cache::UpdateStrategy InternalProtonType::Summary::Cache::InternalUpdateStrategyConverter::operator()(const ::vespalib::slime::Inspector & __inspector, UpdateStrategy __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Cache::getUpdateStrategy(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Summary::Cache::Cache() noexcept
    : maxbytes(-4),
      compression(),
      updateStrategy(UpdateStrategy::INVALIDATE),
      allowvisitcaching(true),
      initialentries(0)
{
}

InternalProtonType::Summary::Cache::Cache(const Cache & __rhs) noexcept = default;
InternalProtonType::Summary::Cache & InternalProtonType::Summary::Cache::operator = (const Cache & __rhs) noexcept = default;
InternalProtonType::Summary::Cache::Cache(Cache && __rhs) noexcept = default;
InternalProtonType::Summary::Cache & InternalProtonType::Summary::Cache::operator = (Cache && __rhs) noexcept = default;
InternalProtonType::Summary::Cache::~Cache() = default; 
InternalProtonType::Summary::Cache::Cache(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    maxbytes = ConfigParser::parse<int64_t>("maxbytes", __lines, -4);
    ConfigParser::stripLinesForKey("maxbytes", __remainingValuesToParse);
    compression = ConfigParser::parseStruct<Compression>("compression", __lines);
    ConfigParser::stripLinesForKey("compression", __remainingValuesToParse);
    updateStrategy = getUpdateStrategy(ConfigParser::parse<vespalib::string>("update_strategy", __lines, "INVALIDATE"));
    ConfigParser::stripLinesForKey("update_strategy", __remainingValuesToParse);
    allowvisitcaching = ConfigParser::parse<bool>("allowvisitcaching", __lines, true);
    ConfigParser::stripLinesForKey("allowvisitcaching", __remainingValuesToParse);
    initialentries = ConfigParser::parse<int64_t>("initialentries", __lines, 0);
    ConfigParser::stripLinesForKey("initialentries", __remainingValuesToParse);
}

bool
InternalProtonType::Summary::Cache::operator==(const Cache& __rhs) const noexcept
{
    return (maxbytes == __rhs.maxbytes &&
            compression == __rhs.compression &&
            updateStrategy == __rhs.updateStrategy &&
            allowvisitcaching == __rhs.allowvisitcaching &&
            initialentries == __rhs.initialentries);
}

bool
InternalProtonType::Summary::Cache::operator!=(const Cache& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Summary::Cache::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxbytes");
        __c.setString("type", "long");
        __c.setLong("value", maxbytes);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("compression");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        compression.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("update_strategy");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getUpdateStrategyName(updateStrategy)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("allowvisitcaching");
        __c.setString("type", "bool");
        __c.setBool("value", allowvisitcaching);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("initialentries");
        __c.setString("type", "long");
        __c.setLong("value", initialentries);
    }
}

InternalProtonType::Summary::Cache::Cache(const vespalib::slime::Inspector & __inspector)
{
    maxbytes = __inspector["maxbytes"]["value"].asLong();
    compression = Compression(__inspector["compression"]["value"]);
    updateStrategy = getUpdateStrategy(__inspector["update_strategy"]["value"].asString().make_string());
    allowvisitcaching = __inspector["allowvisitcaching"]["value"].asBool();
    initialentries = __inspector["initialentries"]["value"].asLong();
}

InternalProtonType::Summary::Cache::Cache(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    maxbytes = ::config::internal::ValueConverter<int64_t>()(__inspector["maxbytes"], -4);
    compression = ::config::internal::ValueConverter<Compression>()(__inspector["compression"]);
    updateStrategy = InternalUpdateStrategyConverter()(__inspector["update_strategy"], UpdateStrategy::INVALIDATE);
    allowvisitcaching = ::config::internal::ValueConverter<bool>()(__inspector["allowvisitcaching"], true);
    initialentries = ::config::internal::ValueConverter<int64_t>()(__inspector["initialentries"], 0);
}

InternalProtonType::Summary::Log::Compact::Compression::Type
InternalProtonType::Summary::Log::Compact::Compression::getType(const vespalib::string& name)
{
    if (name == "NONE") {
        return Type::NONE;
    } else if (name == "LZ4") {
        return Type::LZ4;
    } else if (name == "ZSTD") {
        return Type::ZSTD;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Summary::Log::Compact::Compression::getTypeName(Type t)
{
    switch (t) {
        case Type::NONE: return "NONE";
        case Type::LZ4: return "LZ4";
        case Type::ZSTD: return "ZSTD";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Summary::Log::Compact::Compression::Type InternalProtonType::Summary::Log::Compact::Compression::InternalTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Log::Compact::Compression::getType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Summary::Log::Compact::Compression::Type InternalProtonType::Summary::Log::Compact::Compression::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Summary::Log::Compact::Compression::getType(__inspector.asString().make_string());
}
InternalProtonType::Summary::Log::Compact::Compression::Type InternalProtonType::Summary::Log::Compact::Compression::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Log::Compact::Compression::getType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Summary::Log::Compact::Compression::Compression() noexcept
    : type(Type::ZSTD),
      level(9)
{
}

InternalProtonType::Summary::Log::Compact::Compression::Compression(const Compression & __rhs) noexcept = default;
InternalProtonType::Summary::Log::Compact::Compression & InternalProtonType::Summary::Log::Compact::Compression::operator = (const Compression & __rhs) noexcept = default;
InternalProtonType::Summary::Log::Compact::Compression::Compression(Compression && __rhs) noexcept = default;
InternalProtonType::Summary::Log::Compact::Compression & InternalProtonType::Summary::Log::Compact::Compression::operator = (Compression && __rhs) noexcept = default;
InternalProtonType::Summary::Log::Compact::Compression::~Compression() = default; 
InternalProtonType::Summary::Log::Compact::Compression::Compression(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    type = getType(ConfigParser::parse<vespalib::string>("type", __lines, "ZSTD"));
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
    level = ConfigParser::parse<int32_t>("level", __lines, 9);
    ConfigParser::stripLinesForKey("level", __remainingValuesToParse);
}

bool
InternalProtonType::Summary::Log::Compact::Compression::
operator==(const Compression& __rhs) const noexcept
{
    return (type == __rhs.type &&
            level == __rhs.level);
}

bool
InternalProtonType::Summary::Log::Compact::Compression::
operator!=(const Compression& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Summary::Log::Compact::Compression::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getTypeName(type)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("level");
        __c.setString("type", "int");
        __c.setLong("value", level);
    }
}

InternalProtonType::Summary::Log::Compact::Compression::Compression(const vespalib::slime::Inspector & __inspector)
{
    type = getType(__inspector["type"]["value"].asString().make_string());
    level = __inspector["level"]["value"].asLong();
}

InternalProtonType::Summary::Log::Compact::Compression::Compression(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    type = InternalTypeConverter()(__inspector["type"], Type::ZSTD);
    level = ::config::internal::ValueConverter<int32_t>()(__inspector["level"], 9);
}

InternalProtonType::Summary::Log::Compact::Compact() noexcept
    : compression()
{
}

InternalProtonType::Summary::Log::Compact::Compact(const Compact & __rhs) noexcept = default;
InternalProtonType::Summary::Log::Compact & InternalProtonType::Summary::Log::Compact::operator = (const Compact & __rhs) noexcept = default;
InternalProtonType::Summary::Log::Compact::Compact(Compact && __rhs) noexcept = default;
InternalProtonType::Summary::Log::Compact & InternalProtonType::Summary::Log::Compact::operator = (Compact && __rhs) noexcept = default;
InternalProtonType::Summary::Log::Compact::~Compact() = default; 
InternalProtonType::Summary::Log::Compact::Compact(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    compression = ConfigParser::parseStruct<Compression>("compression", __lines);
    ConfigParser::stripLinesForKey("compression", __remainingValuesToParse);
}

bool
InternalProtonType::Summary::Log::Compact::
operator==(const Compact& __rhs) const noexcept
{
    return (compression == __rhs.compression);
}

bool
InternalProtonType::Summary::Log::Compact::
operator!=(const Compact& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Summary::Log::Compact::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("compression");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        compression.serialize(__c2);
    }
}

InternalProtonType::Summary::Log::Compact::Compact(const vespalib::slime::Inspector & __inspector)
{
    compression = Compression(__inspector["compression"]["value"]);
}

InternalProtonType::Summary::Log::Compact::Compact(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    compression = ::config::internal::ValueConverter<Compression>()(__inspector["compression"]);
}

InternalProtonType::Summary::Log::Chunk::Compression::Type
InternalProtonType::Summary::Log::Chunk::Compression::getType(const vespalib::string& name)
{
    if (name == "NONE") {
        return Type::NONE;
    } else if (name == "LZ4") {
        return Type::LZ4;
    } else if (name == "ZSTD") {
        return Type::ZSTD;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Summary::Log::Chunk::Compression::getTypeName(Type t)
{
    switch (t) {
        case Type::NONE: return "NONE";
        case Type::LZ4: return "LZ4";
        case Type::ZSTD: return "ZSTD";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Summary::Log::Chunk::Compression::Type InternalProtonType::Summary::Log::Chunk::Compression::InternalTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Log::Chunk::Compression::getType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Summary::Log::Chunk::Compression::Type InternalProtonType::Summary::Log::Chunk::Compression::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Summary::Log::Chunk::Compression::getType(__inspector.asString().make_string());
}
InternalProtonType::Summary::Log::Chunk::Compression::Type InternalProtonType::Summary::Log::Chunk::Compression::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Log::Chunk::Compression::getType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Summary::Log::Chunk::Compression::Compression() noexcept
    : type(Type::ZSTD),
      level(9)
{
}

InternalProtonType::Summary::Log::Chunk::Compression::Compression(const Compression & __rhs) noexcept = default;
InternalProtonType::Summary::Log::Chunk::Compression & InternalProtonType::Summary::Log::Chunk::Compression::operator = (const Compression & __rhs) noexcept = default;
InternalProtonType::Summary::Log::Chunk::Compression::Compression(Compression && __rhs) noexcept = default;
InternalProtonType::Summary::Log::Chunk::Compression & InternalProtonType::Summary::Log::Chunk::Compression::operator = (Compression && __rhs) noexcept = default;
InternalProtonType::Summary::Log::Chunk::Compression::~Compression() = default; 
InternalProtonType::Summary::Log::Chunk::Compression::Compression(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    type = getType(ConfigParser::parse<vespalib::string>("type", __lines, "ZSTD"));
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
    level = ConfigParser::parse<int32_t>("level", __lines, 9);
    ConfigParser::stripLinesForKey("level", __remainingValuesToParse);
}

bool
InternalProtonType::Summary::Log::Chunk::Compression::
operator==(const Compression& __rhs) const noexcept
{
    return (type == __rhs.type &&
            level == __rhs.level);
}

bool
InternalProtonType::Summary::Log::Chunk::Compression::
operator!=(const Compression& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Summary::Log::Chunk::Compression::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getTypeName(type)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("level");
        __c.setString("type", "int");
        __c.setLong("value", level);
    }
}

InternalProtonType::Summary::Log::Chunk::Compression::Compression(const vespalib::slime::Inspector & __inspector)
{
    type = getType(__inspector["type"]["value"].asString().make_string());
    level = __inspector["level"]["value"].asLong();
}

InternalProtonType::Summary::Log::Chunk::Compression::Compression(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    type = InternalTypeConverter()(__inspector["type"], Type::ZSTD);
    level = ::config::internal::ValueConverter<int32_t>()(__inspector["level"], 9);
}

InternalProtonType::Summary::Log::Chunk::Chunk() noexcept
    : compression(),
      maxbytes(65536),
      skipcrconread(false)
{
}

InternalProtonType::Summary::Log::Chunk::Chunk(const Chunk & __rhs) noexcept = default;
InternalProtonType::Summary::Log::Chunk & InternalProtonType::Summary::Log::Chunk::operator = (const Chunk & __rhs) noexcept = default;
InternalProtonType::Summary::Log::Chunk::Chunk(Chunk && __rhs) noexcept = default;
InternalProtonType::Summary::Log::Chunk & InternalProtonType::Summary::Log::Chunk::operator = (Chunk && __rhs) noexcept = default;
InternalProtonType::Summary::Log::Chunk::~Chunk() = default; 
InternalProtonType::Summary::Log::Chunk::Chunk(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    compression = ConfigParser::parseStruct<Compression>("compression", __lines);
    ConfigParser::stripLinesForKey("compression", __remainingValuesToParse);
    maxbytes = ConfigParser::parse<int32_t>("maxbytes", __lines, 65536);
    ConfigParser::stripLinesForKey("maxbytes", __remainingValuesToParse);
    skipcrconread = ConfigParser::parse<bool>("skipcrconread", __lines, false);
    ConfigParser::stripLinesForKey("skipcrconread", __remainingValuesToParse);
}

bool
InternalProtonType::Summary::Log::Chunk::operator==(const Chunk& __rhs) const noexcept
{
    return (compression == __rhs.compression &&
            maxbytes == __rhs.maxbytes &&
            skipcrconread == __rhs.skipcrconread);
}

bool
InternalProtonType::Summary::Log::Chunk::operator!=(const Chunk& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Summary::Log::Chunk::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("compression");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        compression.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxbytes");
        __c.setString("type", "int");
        __c.setLong("value", maxbytes);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("skipcrconread");
        __c.setString("type", "bool");
        __c.setBool("value", skipcrconread);
    }
}

InternalProtonType::Summary::Log::Chunk::Chunk(const vespalib::slime::Inspector & __inspector)
{
    compression = Compression(__inspector["compression"]["value"]);
    maxbytes = __inspector["maxbytes"]["value"].asLong();
    skipcrconread = __inspector["skipcrconread"]["value"].asBool();
}

InternalProtonType::Summary::Log::Chunk::Chunk(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    compression = ::config::internal::ValueConverter<Compression>()(__inspector["compression"]);
    maxbytes = ::config::internal::ValueConverter<int32_t>()(__inspector["maxbytes"], 65536);
    skipcrconread = ::config::internal::ValueConverter<bool>()(__inspector["skipcrconread"], false);
}

InternalProtonType::Summary::Log::Log() noexcept
    : compact(),
      chunk(),
      maxfilesize(1000000000),
      maxnumlids(8388608),
      maxdiskbloatfactor(0.1),
      maxbucketspread(2.5),
      minfilesizefactor(0.2)
{
}

InternalProtonType::Summary::Log::Log(const Log & __rhs) noexcept = default;
InternalProtonType::Summary::Log & InternalProtonType::Summary::Log::operator = (const Log & __rhs) noexcept = default;
InternalProtonType::Summary::Log::Log(Log && __rhs) noexcept = default;
InternalProtonType::Summary::Log & InternalProtonType::Summary::Log::operator = (Log && __rhs) noexcept = default;
InternalProtonType::Summary::Log::~Log() = default; 
InternalProtonType::Summary::Log::Log(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    compact = ConfigParser::parseStruct<Compact>("compact", __lines);
    ConfigParser::stripLinesForKey("compact", __remainingValuesToParse);
    chunk = ConfigParser::parseStruct<Chunk>("chunk", __lines);
    ConfigParser::stripLinesForKey("chunk", __remainingValuesToParse);
    maxfilesize = ConfigParser::parse<int64_t>("maxfilesize", __lines, 1000000000);
    ConfigParser::stripLinesForKey("maxfilesize", __remainingValuesToParse);
    maxnumlids = ConfigParser::parse<int32_t>("maxnumlids", __lines, 8388608);
    ConfigParser::stripLinesForKey("maxnumlids", __remainingValuesToParse);
    maxdiskbloatfactor = ConfigParser::parse<double>("maxdiskbloatfactor", __lines, 0.1);
    ConfigParser::stripLinesForKey("maxdiskbloatfactor", __remainingValuesToParse);
    maxbucketspread = ConfigParser::parse<double>("maxbucketspread", __lines, 2.5);
    ConfigParser::stripLinesForKey("maxbucketspread", __remainingValuesToParse);
    minfilesizefactor = ConfigParser::parse<double>("minfilesizefactor", __lines, 0.2);
    ConfigParser::stripLinesForKey("minfilesizefactor", __remainingValuesToParse);
}

bool
InternalProtonType::Summary::Log::operator==(const Log& __rhs) const noexcept
{
    return (compact == __rhs.compact &&
            chunk == __rhs.chunk &&
            maxfilesize == __rhs.maxfilesize &&
            maxnumlids == __rhs.maxnumlids &&
            maxdiskbloatfactor == __rhs.maxdiskbloatfactor &&
            maxbucketspread == __rhs.maxbucketspread &&
            minfilesizefactor == __rhs.minfilesizefactor);
}

bool
InternalProtonType::Summary::Log::operator!=(const Log& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Summary::Log::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("compact");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        compact.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("chunk");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        chunk.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxfilesize");
        __c.setString("type", "long");
        __c.setLong("value", maxfilesize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxnumlids");
        __c.setString("type", "int");
        __c.setLong("value", maxnumlids);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxdiskbloatfactor");
        __c.setString("type", "double");
        __c.setDouble("value", maxdiskbloatfactor);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxbucketspread");
        __c.setString("type", "double");
        __c.setDouble("value", maxbucketspread);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("minfilesizefactor");
        __c.setString("type", "double");
        __c.setDouble("value", minfilesizefactor);
    }
}

InternalProtonType::Summary::Log::Log(const vespalib::slime::Inspector & __inspector)
{
    compact = Compact(__inspector["compact"]["value"]);
    chunk = Chunk(__inspector["chunk"]["value"]);
    maxfilesize = __inspector["maxfilesize"]["value"].asLong();
    maxnumlids = __inspector["maxnumlids"]["value"].asLong();
    maxdiskbloatfactor = __inspector["maxdiskbloatfactor"]["value"].asDouble();
    maxbucketspread = __inspector["maxbucketspread"]["value"].asDouble();
    minfilesizefactor = __inspector["minfilesizefactor"]["value"].asDouble();
}

InternalProtonType::Summary::Log::Log(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    compact = ::config::internal::ValueConverter<Compact>()(__inspector["compact"]);
    chunk = ::config::internal::ValueConverter<Chunk>()(__inspector["chunk"]);
    maxfilesize = ::config::internal::ValueConverter<int64_t>()(__inspector["maxfilesize"], 1000000000);
    maxnumlids = ::config::internal::ValueConverter<int32_t>()(__inspector["maxnumlids"], 8388608);
    maxdiskbloatfactor = ::config::internal::ValueConverter<double>()(__inspector["maxdiskbloatfactor"], 0.1);
    maxbucketspread = ::config::internal::ValueConverter<double>()(__inspector["maxbucketspread"], 2.5);
    minfilesizefactor = ::config::internal::ValueConverter<double>()(__inspector["minfilesizefactor"], 0.2);
}

InternalProtonType::Summary::Write::Io
InternalProtonType::Summary::Write::getIo(const vespalib::string& name)
{
    if (name == "NORMAL") {
        return Io::NORMAL;
    } else if (name == "OSYNC") {
        return Io::OSYNC;
    } else if (name == "DIRECTIO") {
        return Io::DIRECTIO;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Summary::Write::getIoName(Io t)
{
    switch (t) {
        case Io::NORMAL: return "NORMAL";
        case Io::OSYNC: return "OSYNC";
        case Io::DIRECTIO: return "DIRECTIO";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Summary::Write::Io InternalProtonType::Summary::Write::InternalIoConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Write::getIo(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Summary::Write::Io InternalProtonType::Summary::Write::InternalIoConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Summary::Write::getIo(__inspector.asString().make_string());
}
InternalProtonType::Summary::Write::Io InternalProtonType::Summary::Write::InternalIoConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Io __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Write::getIo(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Summary::Write::Write() noexcept
    : io(Io::DIRECTIO)
{
}

InternalProtonType::Summary::Write::Write(const Write & __rhs) noexcept = default;
InternalProtonType::Summary::Write & InternalProtonType::Summary::Write::operator = (const Write & __rhs) noexcept = default;
InternalProtonType::Summary::Write::Write(Write && __rhs) noexcept = default;
InternalProtonType::Summary::Write & InternalProtonType::Summary::Write::operator = (Write && __rhs) noexcept = default;
InternalProtonType::Summary::Write::~Write() = default; 
InternalProtonType::Summary::Write::Write(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    io = getIo(ConfigParser::parse<vespalib::string>("io", __lines, "DIRECTIO"));
    ConfigParser::stripLinesForKey("io", __remainingValuesToParse);
}

bool
InternalProtonType::Summary::Write::operator==(const Write& __rhs) const noexcept
{
    return (io == __rhs.io);
}

bool
InternalProtonType::Summary::Write::operator!=(const Write& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Summary::Write::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("io");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getIoName(io)));
    }
}

InternalProtonType::Summary::Write::Write(const vespalib::slime::Inspector & __inspector)
{
    io = getIo(__inspector["io"]["value"].asString().make_string());
}

InternalProtonType::Summary::Write::Write(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    io = InternalIoConverter()(__inspector["io"], Io::DIRECTIO);
}

InternalProtonType::Summary::Read::Io
InternalProtonType::Summary::Read::getIo(const vespalib::string& name)
{
    if (name == "NORMAL") {
        return Io::NORMAL;
    } else if (name == "DIRECTIO") {
        return Io::DIRECTIO;
    } else if (name == "MMAP") {
        return Io::MMAP;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Summary::Read::getIoName(Io t)
{
    switch (t) {
        case Io::NORMAL: return "NORMAL";
        case Io::DIRECTIO: return "DIRECTIO";
        case Io::MMAP: return "MMAP";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Summary::Read::Io InternalProtonType::Summary::Read::InternalIoConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Read::getIo(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Summary::Read::Io InternalProtonType::Summary::Read::InternalIoConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Summary::Read::getIo(__inspector.asString().make_string());
}
InternalProtonType::Summary::Read::Io InternalProtonType::Summary::Read::InternalIoConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Io __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Read::getIo(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Summary::Read::Mmap::Options
InternalProtonType::Summary::Read::Mmap::getOptions(const vespalib::string& name)
{
    if (name == "POPULATE") {
        return Options::POPULATE;
    } else if (name == "HUGETLB") {
        return Options::HUGETLB;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Summary::Read::Mmap::getOptionsName(Options t)
{
    switch (t) {
        case Options::POPULATE: return "POPULATE";
        case Options::HUGETLB: return "HUGETLB";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Summary::Read::Mmap::Options InternalProtonType::Summary::Read::Mmap::InternalOptionsConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Read::Mmap::getOptions(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Summary::Read::Mmap::Options InternalProtonType::Summary::Read::Mmap::InternalOptionsConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Summary::Read::Mmap::getOptions(__inspector.asString().make_string());
}
InternalProtonType::Summary::Read::Mmap::Options InternalProtonType::Summary::Read::Mmap::InternalOptionsConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Options __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Read::Mmap::getOptions(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Summary::Read::Mmap::Advise
InternalProtonType::Summary::Read::Mmap::getAdvise(const vespalib::string& name)
{
    if (name == "NORMAL") {
        return Advise::NORMAL;
    } else if (name == "RANDOM") {
        return Advise::RANDOM;
    } else if (name == "SEQUENTIAL") {
        return Advise::SEQUENTIAL;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Summary::Read::Mmap::getAdviseName(Advise t)
{
    switch (t) {
        case Advise::NORMAL: return "NORMAL";
        case Advise::RANDOM: return "RANDOM";
        case Advise::SEQUENTIAL: return "SEQUENTIAL";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Summary::Read::Mmap::Advise InternalProtonType::Summary::Read::Mmap::InternalAdviseConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Read::Mmap::getAdvise(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Summary::Read::Mmap::Advise InternalProtonType::Summary::Read::Mmap::InternalAdviseConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Summary::Read::Mmap::getAdvise(__inspector.asString().make_string());
}
InternalProtonType::Summary::Read::Mmap::Advise InternalProtonType::Summary::Read::Mmap::InternalAdviseConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Advise __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Summary::Read::Mmap::getAdvise(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Summary::Read::Mmap::Mmap()
    : options(),
      advise(Advise::NORMAL)
{
}

InternalProtonType::Summary::Read::Mmap::Mmap(const Mmap & __rhs) = default;
InternalProtonType::Summary::Read::Mmap & InternalProtonType::Summary::Read::Mmap::operator = (const Mmap & __rhs) = default;
InternalProtonType::Summary::Read::Mmap::Mmap(Mmap && __rhs) noexcept = default;
InternalProtonType::Summary::Read::Mmap & InternalProtonType::Summary::Read::Mmap::operator = (Mmap && __rhs) noexcept = default;
InternalProtonType::Summary::Read::Mmap::~Mmap() = default; 
InternalProtonType::Summary::Read::Mmap::Mmap(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    ::config::StringVector options__ValueList(
            ConfigParser::parseArray<::config::StringVector>("options", __lines));
    options.reserve(options__ValueList.size());
    for (const auto & item : options__ValueList) {
        options.push_back(getOptions(item));
    }
    ConfigParser::stripLinesForKey("options", __remainingValuesToParse);
    advise = getAdvise(ConfigParser::parse<vespalib::string>("advise", __lines, "NORMAL"));
    ConfigParser::stripLinesForKey("advise", __remainingValuesToParse);
}

bool
InternalProtonType::Summary::Read::Mmap::operator==(const Mmap& __rhs) const noexcept
{
    return (options == __rhs.options &&
            advise == __rhs.advise);
}

bool
InternalProtonType::Summary::Read::Mmap::operator!=(const Mmap& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Summary::Read::Mmap::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("options");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : options) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "enum");
            __c3.setString("value", vespalib::Memory(getOptionsName(child)));
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("advise");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getAdviseName(advise)));
    }
}

InternalProtonType::Summary::Read::Mmap::Mmap(const vespalib::slime::Inspector & __inspector)
{
    for (size_t __i = 0; __i < __inspector["options"]["value"].children(); __i++) {
        options.push_back(getOptions(__inspector["options"]["value"][__i]["value"].asString().make_string()));
    }
    advise = getAdvise(__inspector["advise"]["value"].asString().make_string());
}

InternalProtonType::Summary::Read::Mmap::Mmap(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    ::config::internal::VectorInserter<OptionsVector, InternalOptionsConverter> __optionsInserter(options);
    __inspector["options"].traverse(__optionsInserter);
    advise = InternalAdviseConverter()(__inspector["advise"], Advise::NORMAL);
}

InternalProtonType::Summary::Read::Read()
    : io(Io::MMAP),
      mmap()
{
}

InternalProtonType::Summary::Read::Read(const Read & __rhs) = default;
InternalProtonType::Summary::Read & InternalProtonType::Summary::Read::operator = (const Read & __rhs) = default;
InternalProtonType::Summary::Read::Read(Read && __rhs) noexcept = default;
InternalProtonType::Summary::Read & InternalProtonType::Summary::Read::operator = (Read && __rhs) noexcept = default;
InternalProtonType::Summary::Read::~Read() = default; 
InternalProtonType::Summary::Read::Read(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    io = getIo(ConfigParser::parse<vespalib::string>("io", __lines, "MMAP"));
    ConfigParser::stripLinesForKey("io", __remainingValuesToParse);
    mmap = ConfigParser::parseStruct<Mmap>("mmap", __lines);
    ConfigParser::stripLinesForKey("mmap", __remainingValuesToParse);
}

bool
InternalProtonType::Summary::Read::operator==(const Read& __rhs) const noexcept
{
    return (io == __rhs.io &&
            mmap == __rhs.mmap);
}

bool
InternalProtonType::Summary::Read::operator!=(const Read& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Summary::Read::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("io");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getIoName(io)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("mmap");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        mmap.serialize(__c2);
    }
}

InternalProtonType::Summary::Read::Read(const vespalib::slime::Inspector & __inspector)
{
    io = getIo(__inspector["io"]["value"].asString().make_string());
    mmap = Mmap(__inspector["mmap"]["value"]);
}

InternalProtonType::Summary::Read::Read(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    io = InternalIoConverter()(__inspector["io"], Io::MMAP);
    mmap = ::config::internal::ValueConverter<Mmap>()(__inspector["mmap"]);
}

InternalProtonType::Summary::Summary()
    : cache(),
      log(),
      write(),
      read()
{
}

InternalProtonType::Summary::Summary(const Summary & __rhs) = default;
InternalProtonType::Summary & InternalProtonType::Summary::operator = (const Summary & __rhs) = default;
InternalProtonType::Summary::Summary(Summary && __rhs) noexcept = default;
InternalProtonType::Summary & InternalProtonType::Summary::operator = (Summary && __rhs) noexcept = default;
InternalProtonType::Summary::~Summary() = default; 
InternalProtonType::Summary::Summary(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    cache = ConfigParser::parseStruct<Cache>("cache", __lines);
    ConfigParser::stripLinesForKey("cache", __remainingValuesToParse);
    log = ConfigParser::parseStruct<Log>("log", __lines);
    ConfigParser::stripLinesForKey("log", __remainingValuesToParse);
    write = ConfigParser::parseStruct<Write>("write", __lines);
    ConfigParser::stripLinesForKey("write", __remainingValuesToParse);
    read = ConfigParser::parseStruct<Read>("read", __lines);
    ConfigParser::stripLinesForKey("read", __remainingValuesToParse);
}

bool
InternalProtonType::Summary::operator==(const Summary& __rhs) const noexcept
{
    return (cache == __rhs.cache &&
            log == __rhs.log &&
            write == __rhs.write &&
            read == __rhs.read);
}

bool
InternalProtonType::Summary::operator!=(const Summary& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Summary::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("cache");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        cache.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("log");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        log.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("write");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        write.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("read");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        read.serialize(__c2);
    }
}

InternalProtonType::Summary::Summary(const vespalib::slime::Inspector & __inspector)
{
    cache = Cache(__inspector["cache"]["value"]);
    log = Log(__inspector["log"]["value"]);
    write = Write(__inspector["write"]["value"]);
    read = Read(__inspector["read"]["value"]);
}

InternalProtonType::Summary::Summary(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    cache = ::config::internal::ValueConverter<Cache>()(__inspector["cache"]);
    log = ::config::internal::ValueConverter<Log>()(__inspector["log"]);
    write = ::config::internal::ValueConverter<Write>()(__inspector["write"]);
    read = ::config::internal::ValueConverter<Read>()(__inspector["read"]);
}

InternalProtonType::Documentdb::Mode
InternalProtonType::Documentdb::getMode(const vespalib::string& name)
{
    if (name == "INDEX") {
        return Mode::INDEX;
    } else if (name == "STREAMING") {
        return Mode::STREAMING;
    } else if (name == "STORE_ONLY") {
        return Mode::STORE_ONLY;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Documentdb::getModeName(Mode t)
{
    switch (t) {
        case Mode::INDEX: return "INDEX";
        case Mode::STREAMING: return "STREAMING";
        case Mode::STORE_ONLY: return "STORE_ONLY";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Documentdb::Mode InternalProtonType::Documentdb::InternalModeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Documentdb::getMode(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Documentdb::Mode InternalProtonType::Documentdb::InternalModeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Documentdb::getMode(__inspector.asString().make_string());
}
InternalProtonType::Documentdb::Mode InternalProtonType::Documentdb::InternalModeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Mode __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Documentdb::getMode(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Documentdb::Allocation::Allocation() noexcept
    : initialnumdocs(1024),
      growfactor(0.2),
      growbias(1),
      amortizecount(10000),
      multivaluegrowfactor(0.2),
      maxDeadBytesRatio(0.05),
      maxDeadAddressSpaceRatio(0.2),
      maxCompactBuffers(1),
      activeBuffersRatio(0.1)
{
}

InternalProtonType::Documentdb::Allocation::Allocation(const Allocation & __rhs) noexcept = default;
InternalProtonType::Documentdb::Allocation & InternalProtonType::Documentdb::Allocation::operator = (const Allocation & __rhs) noexcept = default;
InternalProtonType::Documentdb::Allocation::Allocation(Allocation && __rhs) noexcept = default;
InternalProtonType::Documentdb::Allocation & InternalProtonType::Documentdb::Allocation::operator = (Allocation && __rhs) noexcept = default;
InternalProtonType::Documentdb::Allocation::~Allocation() = default; 
InternalProtonType::Documentdb::Allocation::Allocation(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    initialnumdocs = ConfigParser::parse<int64_t>("initialnumdocs", __lines, 1024);
    ConfigParser::stripLinesForKey("initialnumdocs", __remainingValuesToParse);
    growfactor = ConfigParser::parse<double>("growfactor", __lines, 0.2);
    ConfigParser::stripLinesForKey("growfactor", __remainingValuesToParse);
    growbias = ConfigParser::parse<int32_t>("growbias", __lines, 1);
    ConfigParser::stripLinesForKey("growbias", __remainingValuesToParse);
    amortizecount = ConfigParser::parse<int32_t>("amortizecount", __lines, 10000);
    ConfigParser::stripLinesForKey("amortizecount", __remainingValuesToParse);
    multivaluegrowfactor = ConfigParser::parse<double>("multivaluegrowfactor", __lines, 0.2);
    ConfigParser::stripLinesForKey("multivaluegrowfactor", __remainingValuesToParse);
    maxDeadBytesRatio = ConfigParser::parse<double>("max_dead_bytes_ratio", __lines, 0.05);
    ConfigParser::stripLinesForKey("max_dead_bytes_ratio", __remainingValuesToParse);
    maxDeadAddressSpaceRatio = ConfigParser::parse<double>("max_dead_address_space_ratio", __lines, 0.2);
    ConfigParser::stripLinesForKey("max_dead_address_space_ratio", __remainingValuesToParse);
    maxCompactBuffers = ConfigParser::parse<int32_t>("max_compact_buffers", __lines, 1);
    ConfigParser::stripLinesForKey("max_compact_buffers", __remainingValuesToParse);
    activeBuffersRatio = ConfigParser::parse<double>("active_buffers_ratio", __lines, 0.1);
    ConfigParser::stripLinesForKey("active_buffers_ratio", __remainingValuesToParse);
}

bool
InternalProtonType::Documentdb::Allocation::
operator==(const Allocation& __rhs) const noexcept
{
    return (initialnumdocs == __rhs.initialnumdocs &&
            growfactor == __rhs.growfactor &&
            growbias == __rhs.growbias &&
            amortizecount == __rhs.amortizecount &&
            multivaluegrowfactor == __rhs.multivaluegrowfactor &&
            maxDeadBytesRatio == __rhs.maxDeadBytesRatio &&
            maxDeadAddressSpaceRatio == __rhs.maxDeadAddressSpaceRatio &&
            maxCompactBuffers == __rhs.maxCompactBuffers &&
            activeBuffersRatio == __rhs.activeBuffersRatio);
}

bool
InternalProtonType::Documentdb::Allocation::
operator!=(const Allocation& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Documentdb::Allocation::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("initialnumdocs");
        __c.setString("type", "long");
        __c.setLong("value", initialnumdocs);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("growfactor");
        __c.setString("type", "double");
        __c.setDouble("value", growfactor);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("growbias");
        __c.setString("type", "int");
        __c.setLong("value", growbias);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("amortizecount");
        __c.setString("type", "int");
        __c.setLong("value", amortizecount);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("multivaluegrowfactor");
        __c.setString("type", "double");
        __c.setDouble("value", multivaluegrowfactor);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_dead_bytes_ratio");
        __c.setString("type", "double");
        __c.setDouble("value", maxDeadBytesRatio);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_dead_address_space_ratio");
        __c.setString("type", "double");
        __c.setDouble("value", maxDeadAddressSpaceRatio);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_compact_buffers");
        __c.setString("type", "int");
        __c.setLong("value", maxCompactBuffers);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("active_buffers_ratio");
        __c.setString("type", "double");
        __c.setDouble("value", activeBuffersRatio);
    }
}

InternalProtonType::Documentdb::Allocation::Allocation(const vespalib::slime::Inspector & __inspector)
{
    initialnumdocs = __inspector["initialnumdocs"]["value"].asLong();
    growfactor = __inspector["growfactor"]["value"].asDouble();
    growbias = __inspector["growbias"]["value"].asLong();
    amortizecount = __inspector["amortizecount"]["value"].asLong();
    multivaluegrowfactor = __inspector["multivaluegrowfactor"]["value"].asDouble();
    maxDeadBytesRatio = __inspector["max_dead_bytes_ratio"]["value"].asDouble();
    maxDeadAddressSpaceRatio = __inspector["max_dead_address_space_ratio"]["value"].asDouble();
    maxCompactBuffers = __inspector["max_compact_buffers"]["value"].asLong();
    activeBuffersRatio = __inspector["active_buffers_ratio"]["value"].asDouble();
}

InternalProtonType::Documentdb::Allocation::Allocation(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    initialnumdocs = ::config::internal::ValueConverter<int64_t>()(__inspector["initialnumdocs"], 1024);
    growfactor = ::config::internal::ValueConverter<double>()(__inspector["growfactor"], 0.2);
    growbias = ::config::internal::ValueConverter<int32_t>()(__inspector["growbias"], 1);
    amortizecount = ::config::internal::ValueConverter<int32_t>()(__inspector["amortizecount"], 10000);
    multivaluegrowfactor = ::config::internal::ValueConverter<double>()(__inspector["multivaluegrowfactor"], 0.2);
    maxDeadBytesRatio = ::config::internal::ValueConverter<double>()(__inspector["max_dead_bytes_ratio"], 0.05);
    maxDeadAddressSpaceRatio = ::config::internal::ValueConverter<double>()(__inspector["max_dead_address_space_ratio"], 0.2);
    maxCompactBuffers = ::config::internal::ValueConverter<int32_t>()(__inspector["max_compact_buffers"], 1);
    activeBuffersRatio = ::config::internal::ValueConverter<double>()(__inspector["active_buffers_ratio"], 0.1);
}

InternalProtonType::Documentdb::Documentdb() noexcept
    : inputdoctypename(),
      mode(Mode::INDEX),
      configid(),
      visibilitydelay(0.0),
      global(false),
      allocation()
{
}

InternalProtonType::Documentdb::Documentdb(const Documentdb & __rhs) noexcept = default;
InternalProtonType::Documentdb & InternalProtonType::Documentdb::operator = (const Documentdb & __rhs) noexcept = default;
InternalProtonType::Documentdb::Documentdb(Documentdb && __rhs) noexcept = default;
InternalProtonType::Documentdb & InternalProtonType::Documentdb::operator = (Documentdb && __rhs) noexcept = default;
InternalProtonType::Documentdb::~Documentdb() = default; 
InternalProtonType::Documentdb::Documentdb(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    inputdoctypename = ConfigParser::parse<vespalib::string>("inputdoctypename", __lines);
    ConfigParser::stripLinesForKey("inputdoctypename", __remainingValuesToParse);
    mode = getMode(ConfigParser::parse<vespalib::string>("mode", __lines, "INDEX"));
    ConfigParser::stripLinesForKey("mode", __remainingValuesToParse);
    configid = ConfigParser::parse<vespalib::string>("configid", __lines);
    ConfigParser::stripLinesForKey("configid", __remainingValuesToParse);
    visibilitydelay = ConfigParser::parse<double>("visibilitydelay", __lines, 0.0);
    ConfigParser::stripLinesForKey("visibilitydelay", __remainingValuesToParse);
    global = ConfigParser::parse<bool>("global", __lines, false);
    ConfigParser::stripLinesForKey("global", __remainingValuesToParse);
    allocation = ConfigParser::parseStruct<Allocation>("allocation", __lines);
    ConfigParser::stripLinesForKey("allocation", __remainingValuesToParse);
}

bool
InternalProtonType::Documentdb::operator==(const Documentdb& __rhs) const noexcept
{
    return (inputdoctypename == __rhs.inputdoctypename &&
            mode == __rhs.mode &&
            configid == __rhs.configid &&
            visibilitydelay == __rhs.visibilitydelay &&
            global == __rhs.global &&
            allocation == __rhs.allocation);
}

bool
InternalProtonType::Documentdb::operator!=(const Documentdb& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Documentdb::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("inputdoctypename");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(inputdoctypename));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("mode");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getModeName(mode)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("configid");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(configid));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("visibilitydelay");
        __c.setString("type", "double");
        __c.setDouble("value", visibilitydelay);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("global");
        __c.setString("type", "bool");
        __c.setBool("value", global);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("allocation");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        allocation.serialize(__c2);
    }
}

InternalProtonType::Documentdb::Documentdb(const vespalib::slime::Inspector & __inspector)
{
    inputdoctypename = __inspector["inputdoctypename"]["value"].asString().make_string();
    mode = getMode(__inspector["mode"]["value"].asString().make_string());
    configid = __inspector["configid"]["value"].asString().make_string();
    visibilitydelay = __inspector["visibilitydelay"]["value"].asDouble();
    global = __inspector["global"]["value"].asBool();
    allocation = Allocation(__inspector["allocation"]["value"]);
}

InternalProtonType::Documentdb::Documentdb(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    inputdoctypename = ::config::internal::ValueConverter<vespalib::string>()("inputdoctypename", __inspector["inputdoctypename"]);
    mode = InternalModeConverter()(__inspector["mode"], Mode::INDEX);
    configid = ::config::internal::ValueConverter<vespalib::string>()("configid", __inspector["configid"]);
    visibilitydelay = ::config::internal::ValueConverter<double>()(__inspector["visibilitydelay"], 0.0);
    global = ::config::internal::ValueConverter<bool>()(__inspector["global"], false);
    allocation = ::config::internal::ValueConverter<Allocation>()(__inspector["allocation"]);
}

InternalProtonType::Periodic::Periodic() noexcept
    : interval(3600.0)
{
}

InternalProtonType::Periodic::Periodic(const Periodic & __rhs) noexcept = default;
InternalProtonType::Periodic & InternalProtonType::Periodic::operator = (const Periodic & __rhs) noexcept = default;
InternalProtonType::Periodic::Periodic(Periodic && __rhs) noexcept = default;
InternalProtonType::Periodic & InternalProtonType::Periodic::operator = (Periodic && __rhs) noexcept = default;
InternalProtonType::Periodic::~Periodic() = default; 
InternalProtonType::Periodic::Periodic(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    interval = ConfigParser::parse<double>("interval", __lines, 3600.0);
    ConfigParser::stripLinesForKey("interval", __remainingValuesToParse);
}

bool
InternalProtonType::Periodic::operator==(const Periodic& __rhs) const noexcept
{
    return (interval == __rhs.interval);
}

bool
InternalProtonType::Periodic::operator!=(const Periodic& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Periodic::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("interval");
        __c.setString("type", "double");
        __c.setDouble("value", interval);
    }
}

InternalProtonType::Periodic::Periodic(const vespalib::slime::Inspector & __inspector)
{
    interval = __inspector["interval"]["value"].asDouble();
}

InternalProtonType::Periodic::Periodic(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    interval = ::config::internal::ValueConverter<double>()(__inspector["interval"], 3600.0);
}

InternalProtonType::Filedistributor::Filedistributor() noexcept
    : configid("")
{
}

InternalProtonType::Filedistributor::Filedistributor(const Filedistributor & __rhs) noexcept = default;
InternalProtonType::Filedistributor & InternalProtonType::Filedistributor::operator = (const Filedistributor & __rhs) noexcept = default;
InternalProtonType::Filedistributor::Filedistributor(Filedistributor && __rhs) noexcept = default;
InternalProtonType::Filedistributor & InternalProtonType::Filedistributor::operator = (Filedistributor && __rhs) noexcept = default;
InternalProtonType::Filedistributor::~Filedistributor() = default; 
InternalProtonType::Filedistributor::Filedistributor(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    configid = ConfigParser::parse<vespalib::string>("configid", __lines, "");
    ConfigParser::stripLinesForKey("configid", __remainingValuesToParse);
}

bool
InternalProtonType::Filedistributor::
operator==(const Filedistributor& __rhs) const noexcept
{
    return (configid == __rhs.configid);
}

bool
InternalProtonType::Filedistributor::
operator!=(const Filedistributor& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Filedistributor::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("configid");
        __c.setString("type", "reference");
        __c.setString("value", vespalib::Memory(configid));
    }
}

InternalProtonType::Filedistributor::Filedistributor(const vespalib::slime::Inspector & __inspector)
{
    configid = __inspector["configid"]["value"].asString().make_string();
}

InternalProtonType::Filedistributor::Filedistributor(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    configid = ::config::internal::ValueConverter<vespalib::string>()(__inspector["configid"], "");
}

InternalProtonType::Packetcompresstype
InternalProtonType::getPacketcompresstype(const vespalib::string& name)
{
    if (name == "NONE") {
        return Packetcompresstype::NONE;
    } else if (name == "LZ4") {
        return Packetcompresstype::LZ4;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::getPacketcompresstypeName(Packetcompresstype t)
{
    switch (t) {
        case Packetcompresstype::NONE: return "NONE";
        case Packetcompresstype::LZ4: return "LZ4";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Packetcompresstype InternalProtonType::InternalPacketcompresstypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::getPacketcompresstype(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Packetcompresstype InternalProtonType::InternalPacketcompresstypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::getPacketcompresstype(__inspector.asString().make_string());
}
InternalProtonType::Packetcompresstype InternalProtonType::InternalPacketcompresstypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Packetcompresstype __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::getPacketcompresstype(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Lidspacecompaction::Lidspacecompaction() noexcept
    : interval(10.0),
      allowedlidbloat(1),
      allowedlidbloatfactor(0.01),
      removebatchblockrate(0.5),
      removeblockrate(100.0)
{
}

InternalProtonType::Lidspacecompaction::Lidspacecompaction(const Lidspacecompaction & __rhs) noexcept = default;
InternalProtonType::Lidspacecompaction & InternalProtonType::Lidspacecompaction::operator = (const Lidspacecompaction & __rhs) noexcept = default;
InternalProtonType::Lidspacecompaction::Lidspacecompaction(Lidspacecompaction && __rhs) noexcept = default;
InternalProtonType::Lidspacecompaction & InternalProtonType::Lidspacecompaction::operator = (Lidspacecompaction && __rhs) noexcept = default;
InternalProtonType::Lidspacecompaction::~Lidspacecompaction() = default; 
InternalProtonType::Lidspacecompaction::Lidspacecompaction(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    interval = ConfigParser::parse<double>("interval", __lines, 10.0);
    ConfigParser::stripLinesForKey("interval", __remainingValuesToParse);
    allowedlidbloat = ConfigParser::parse<int32_t>("allowedlidbloat", __lines, 1);
    ConfigParser::stripLinesForKey("allowedlidbloat", __remainingValuesToParse);
    allowedlidbloatfactor = ConfigParser::parse<double>("allowedlidbloatfactor", __lines, 0.01);
    ConfigParser::stripLinesForKey("allowedlidbloatfactor", __remainingValuesToParse);
    removebatchblockrate = ConfigParser::parse<double>("removebatchblockrate", __lines, 0.5);
    ConfigParser::stripLinesForKey("removebatchblockrate", __remainingValuesToParse);
    removeblockrate = ConfigParser::parse<double>("removeblockrate", __lines, 100.0);
    ConfigParser::stripLinesForKey("removeblockrate", __remainingValuesToParse);
}

bool
InternalProtonType::Lidspacecompaction::
operator==(const Lidspacecompaction& __rhs) const noexcept
{
    return (interval == __rhs.interval &&
            allowedlidbloat == __rhs.allowedlidbloat &&
            allowedlidbloatfactor == __rhs.allowedlidbloatfactor &&
            removebatchblockrate == __rhs.removebatchblockrate &&
            removeblockrate == __rhs.removeblockrate);
}

bool
InternalProtonType::Lidspacecompaction::
operator!=(const Lidspacecompaction& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Lidspacecompaction::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("interval");
        __c.setString("type", "double");
        __c.setDouble("value", interval);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("allowedlidbloat");
        __c.setString("type", "int");
        __c.setLong("value", allowedlidbloat);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("allowedlidbloatfactor");
        __c.setString("type", "double");
        __c.setDouble("value", allowedlidbloatfactor);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("removebatchblockrate");
        __c.setString("type", "double");
        __c.setDouble("value", removebatchblockrate);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("removeblockrate");
        __c.setString("type", "double");
        __c.setDouble("value", removeblockrate);
    }
}

InternalProtonType::Lidspacecompaction::Lidspacecompaction(const vespalib::slime::Inspector & __inspector)
{
    interval = __inspector["interval"]["value"].asDouble();
    allowedlidbloat = __inspector["allowedlidbloat"]["value"].asLong();
    allowedlidbloatfactor = __inspector["allowedlidbloatfactor"]["value"].asDouble();
    removebatchblockrate = __inspector["removebatchblockrate"]["value"].asDouble();
    removeblockrate = __inspector["removeblockrate"]["value"].asDouble();
}

InternalProtonType::Lidspacecompaction::Lidspacecompaction(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    interval = ::config::internal::ValueConverter<double>()(__inspector["interval"], 10.0);
    allowedlidbloat = ::config::internal::ValueConverter<int32_t>()(__inspector["allowedlidbloat"], 1);
    allowedlidbloatfactor = ::config::internal::ValueConverter<double>()(__inspector["allowedlidbloatfactor"], 0.01);
    removebatchblockrate = ::config::internal::ValueConverter<double>()(__inspector["removebatchblockrate"], 0.5);
    removeblockrate = ::config::internal::ValueConverter<double>()(__inspector["removeblockrate"], 100.0);
}

InternalProtonType::Bucketmove::Bucketmove() noexcept
    : maxdocstomoveperbucket(1)
{
}

InternalProtonType::Bucketmove::Bucketmove(const Bucketmove & __rhs) noexcept = default;
InternalProtonType::Bucketmove & InternalProtonType::Bucketmove::operator = (const Bucketmove & __rhs) noexcept = default;
InternalProtonType::Bucketmove::Bucketmove(Bucketmove && __rhs) noexcept = default;
InternalProtonType::Bucketmove & InternalProtonType::Bucketmove::operator = (Bucketmove && __rhs) noexcept = default;
InternalProtonType::Bucketmove::~Bucketmove() = default; 
InternalProtonType::Bucketmove::Bucketmove(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    maxdocstomoveperbucket = ConfigParser::parse<int32_t>("maxdocstomoveperbucket", __lines, 1);
    ConfigParser::stripLinesForKey("maxdocstomoveperbucket", __remainingValuesToParse);
}

bool
InternalProtonType::Bucketmove::operator==(const Bucketmove& __rhs) const noexcept
{
    return (maxdocstomoveperbucket == __rhs.maxdocstomoveperbucket);
}

bool
InternalProtonType::Bucketmove::operator!=(const Bucketmove& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Bucketmove::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxdocstomoveperbucket");
        __c.setString("type", "int");
        __c.setLong("value", maxdocstomoveperbucket);
    }
}

InternalProtonType::Bucketmove::Bucketmove(const vespalib::slime::Inspector & __inspector)
{
    maxdocstomoveperbucket = __inspector["maxdocstomoveperbucket"]["value"].asLong();
}

InternalProtonType::Bucketmove::Bucketmove(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    maxdocstomoveperbucket = ::config::internal::ValueConverter<int32_t>()(__inspector["maxdocstomoveperbucket"], 1);
}

InternalProtonType::Visit::Visit() noexcept
    : defaultserializedsize(1),
      ignoremaxbytes(true)
{
}

InternalProtonType::Visit::Visit(const Visit & __rhs) noexcept = default;
InternalProtonType::Visit & InternalProtonType::Visit::operator = (const Visit & __rhs) noexcept = default;
InternalProtonType::Visit::Visit(Visit && __rhs) noexcept = default;
InternalProtonType::Visit & InternalProtonType::Visit::operator = (Visit && __rhs) noexcept = default;
InternalProtonType::Visit::~Visit() = default; 
InternalProtonType::Visit::Visit(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    defaultserializedsize = ConfigParser::parse<int64_t>("defaultserializedsize", __lines, 1);
    ConfigParser::stripLinesForKey("defaultserializedsize", __remainingValuesToParse);
    ignoremaxbytes = ConfigParser::parse<bool>("ignoremaxbytes", __lines, true);
    ConfigParser::stripLinesForKey("ignoremaxbytes", __remainingValuesToParse);
}

bool
InternalProtonType::Visit::operator==(const Visit& __rhs) const noexcept
{
    return (defaultserializedsize == __rhs.defaultserializedsize &&
            ignoremaxbytes == __rhs.ignoremaxbytes);
}

bool
InternalProtonType::Visit::operator!=(const Visit& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Visit::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("defaultserializedsize");
        __c.setString("type", "long");
        __c.setLong("value", defaultserializedsize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("ignoremaxbytes");
        __c.setString("type", "bool");
        __c.setBool("value", ignoremaxbytes);
    }
}

InternalProtonType::Visit::Visit(const vespalib::slime::Inspector & __inspector)
{
    defaultserializedsize = __inspector["defaultserializedsize"]["value"].asLong();
    ignoremaxbytes = __inspector["ignoremaxbytes"]["value"].asBool();
}

InternalProtonType::Visit::Visit(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    defaultserializedsize = ::config::internal::ValueConverter<int64_t>()(__inspector["defaultserializedsize"], 1);
    ignoremaxbytes = ::config::internal::ValueConverter<bool>()(__inspector["ignoremaxbytes"], true);
}

InternalProtonType::Initialize::Initialize() noexcept
    : threads(0)
{
}

InternalProtonType::Initialize::Initialize(const Initialize & __rhs) noexcept = default;
InternalProtonType::Initialize & InternalProtonType::Initialize::operator = (const Initialize & __rhs) noexcept = default;
InternalProtonType::Initialize::Initialize(Initialize && __rhs) noexcept = default;
InternalProtonType::Initialize & InternalProtonType::Initialize::operator = (Initialize && __rhs) noexcept = default;
InternalProtonType::Initialize::~Initialize() = default; 
InternalProtonType::Initialize::Initialize(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    threads = ConfigParser::parse<int32_t>("threads", __lines, 0);
    ConfigParser::stripLinesForKey("threads", __remainingValuesToParse);
}

bool
InternalProtonType::Initialize::operator==(const Initialize& __rhs) const noexcept
{
    return (threads == __rhs.threads);
}

bool
InternalProtonType::Initialize::operator!=(const Initialize& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Initialize::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("threads");
        __c.setString("type", "int");
        __c.setLong("value", threads);
    }
}

InternalProtonType::Initialize::Initialize(const vespalib::slime::Inspector & __inspector)
{
    threads = __inspector["threads"]["value"].asLong();
}

InternalProtonType::Initialize::Initialize(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    threads = ::config::internal::ValueConverter<int32_t>()(__inspector["threads"], 0);
}

InternalProtonType::Writefilter::Attribute::Attribute() noexcept
    : addressSpaceLimit(0.92)
{
}

InternalProtonType::Writefilter::Attribute::Attribute(const Attribute & __rhs) noexcept = default;
InternalProtonType::Writefilter::Attribute & InternalProtonType::Writefilter::Attribute::operator = (const Attribute & __rhs) noexcept = default;
InternalProtonType::Writefilter::Attribute::Attribute(Attribute && __rhs) noexcept = default;
InternalProtonType::Writefilter::Attribute & InternalProtonType::Writefilter::Attribute::operator = (Attribute && __rhs) noexcept = default;
InternalProtonType::Writefilter::Attribute::~Attribute() = default; 
InternalProtonType::Writefilter::Attribute::Attribute(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    addressSpaceLimit = ConfigParser::parse<double>("address_space_limit", __lines, 0.92);
    ConfigParser::stripLinesForKey("address_space_limit", __remainingValuesToParse);
}

bool
InternalProtonType::Writefilter::Attribute::
operator==(const Attribute& __rhs) const noexcept
{
    return (addressSpaceLimit == __rhs.addressSpaceLimit);
}

bool
InternalProtonType::Writefilter::Attribute::
operator!=(const Attribute& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Writefilter::Attribute::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("address_space_limit");
        __c.setString("type", "double");
        __c.setDouble("value", addressSpaceLimit);
    }
}

InternalProtonType::Writefilter::Attribute::Attribute(const vespalib::slime::Inspector & __inspector)
{
    addressSpaceLimit = __inspector["address_space_limit"]["value"].asDouble();
}

InternalProtonType::Writefilter::Attribute::Attribute(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    addressSpaceLimit = ::config::internal::ValueConverter<double>()(__inspector["address_space_limit"], 0.92);
}

InternalProtonType::Writefilter::Writefilter() noexcept
    : attribute(),
      memorylimit(0.8),
      disklimit(0.8),
      sampleinterval(10.0)
{
}

InternalProtonType::Writefilter::Writefilter(const Writefilter & __rhs) noexcept = default;
InternalProtonType::Writefilter & InternalProtonType::Writefilter::operator = (const Writefilter & __rhs) noexcept = default;
InternalProtonType::Writefilter::Writefilter(Writefilter && __rhs) noexcept = default;
InternalProtonType::Writefilter & InternalProtonType::Writefilter::operator = (Writefilter && __rhs) noexcept = default;
InternalProtonType::Writefilter::~Writefilter() = default; 
InternalProtonType::Writefilter::Writefilter(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    attribute = ConfigParser::parseStruct<Attribute>("attribute", __lines);
    ConfigParser::stripLinesForKey("attribute", __remainingValuesToParse);
    memorylimit = ConfigParser::parse<double>("memorylimit", __lines, 0.8);
    ConfigParser::stripLinesForKey("memorylimit", __remainingValuesToParse);
    disklimit = ConfigParser::parse<double>("disklimit", __lines, 0.8);
    ConfigParser::stripLinesForKey("disklimit", __remainingValuesToParse);
    sampleinterval = ConfigParser::parse<double>("sampleinterval", __lines, 10.0);
    ConfigParser::stripLinesForKey("sampleinterval", __remainingValuesToParse);
}

bool
InternalProtonType::Writefilter::operator==(const Writefilter& __rhs) const noexcept
{
    return (attribute == __rhs.attribute &&
            memorylimit == __rhs.memorylimit &&
            disklimit == __rhs.disklimit &&
            sampleinterval == __rhs.sampleinterval);
}

bool
InternalProtonType::Writefilter::operator!=(const Writefilter& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Writefilter::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("attribute");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        attribute.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("memorylimit");
        __c.setString("type", "double");
        __c.setDouble("value", memorylimit);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("disklimit");
        __c.setString("type", "double");
        __c.setDouble("value", disklimit);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("sampleinterval");
        __c.setString("type", "double");
        __c.setDouble("value", sampleinterval);
    }
}

InternalProtonType::Writefilter::Writefilter(const vespalib::slime::Inspector & __inspector)
{
    attribute = Attribute(__inspector["attribute"]["value"]);
    memorylimit = __inspector["memorylimit"]["value"].asDouble();
    disklimit = __inspector["disklimit"]["value"].asDouble();
    sampleinterval = __inspector["sampleinterval"]["value"].asDouble();
}

InternalProtonType::Writefilter::Writefilter(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    attribute = ::config::internal::ValueConverter<Attribute>()(__inspector["attribute"]);
    memorylimit = ::config::internal::ValueConverter<double>()(__inspector["memorylimit"], 0.8);
    disklimit = ::config::internal::ValueConverter<double>()(__inspector["disklimit"], 0.8);
    sampleinterval = ::config::internal::ValueConverter<double>()(__inspector["sampleinterval"], 10.0);
}

InternalProtonType::Hwinfo::Disk::Disk() noexcept
    : size(0),
      shared(false),
      writespeed(200.0),
      samplewritesize(1073741824),
      slowwritespeedlimit(100.0)
{
}

InternalProtonType::Hwinfo::Disk::Disk(const Disk & __rhs) noexcept = default;
InternalProtonType::Hwinfo::Disk & InternalProtonType::Hwinfo::Disk::operator = (const Disk & __rhs) noexcept = default;
InternalProtonType::Hwinfo::Disk::Disk(Disk && __rhs) noexcept = default;
InternalProtonType::Hwinfo::Disk & InternalProtonType::Hwinfo::Disk::operator = (Disk && __rhs) noexcept = default;
InternalProtonType::Hwinfo::Disk::~Disk() = default; 
InternalProtonType::Hwinfo::Disk::Disk(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    size = ConfigParser::parse<int64_t>("size", __lines, 0);
    ConfigParser::stripLinesForKey("size", __remainingValuesToParse);
    shared = ConfigParser::parse<bool>("shared", __lines, false);
    ConfigParser::stripLinesForKey("shared", __remainingValuesToParse);
    writespeed = ConfigParser::parse<double>("writespeed", __lines, 200.0);
    ConfigParser::stripLinesForKey("writespeed", __remainingValuesToParse);
    samplewritesize = ConfigParser::parse<int64_t>("samplewritesize", __lines, 1073741824);
    ConfigParser::stripLinesForKey("samplewritesize", __remainingValuesToParse);
    slowwritespeedlimit = ConfigParser::parse<double>("slowwritespeedlimit", __lines, 100.0);
    ConfigParser::stripLinesForKey("slowwritespeedlimit", __remainingValuesToParse);
}

bool
InternalProtonType::Hwinfo::Disk::operator==(const Disk& __rhs) const noexcept
{
    return (size == __rhs.size &&
            shared == __rhs.shared &&
            writespeed == __rhs.writespeed &&
            samplewritesize == __rhs.samplewritesize &&
            slowwritespeedlimit == __rhs.slowwritespeedlimit);
}

bool
InternalProtonType::Hwinfo::Disk::operator!=(const Disk& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Hwinfo::Disk::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("size");
        __c.setString("type", "long");
        __c.setLong("value", size);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("shared");
        __c.setString("type", "bool");
        __c.setBool("value", shared);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("writespeed");
        __c.setString("type", "double");
        __c.setDouble("value", writespeed);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("samplewritesize");
        __c.setString("type", "long");
        __c.setLong("value", samplewritesize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("slowwritespeedlimit");
        __c.setString("type", "double");
        __c.setDouble("value", slowwritespeedlimit);
    }
}

InternalProtonType::Hwinfo::Disk::Disk(const vespalib::slime::Inspector & __inspector)
{
    size = __inspector["size"]["value"].asLong();
    shared = __inspector["shared"]["value"].asBool();
    writespeed = __inspector["writespeed"]["value"].asDouble();
    samplewritesize = __inspector["samplewritesize"]["value"].asLong();
    slowwritespeedlimit = __inspector["slowwritespeedlimit"]["value"].asDouble();
}

InternalProtonType::Hwinfo::Disk::Disk(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    size = ::config::internal::ValueConverter<int64_t>()(__inspector["size"], 0);
    shared = ::config::internal::ValueConverter<bool>()(__inspector["shared"], false);
    writespeed = ::config::internal::ValueConverter<double>()(__inspector["writespeed"], 200.0);
    samplewritesize = ::config::internal::ValueConverter<int64_t>()(__inspector["samplewritesize"], 1073741824);
    slowwritespeedlimit = ::config::internal::ValueConverter<double>()(__inspector["slowwritespeedlimit"], 100.0);
}

InternalProtonType::Hwinfo::Memory::Memory() noexcept
    : size(0)
{
}

InternalProtonType::Hwinfo::Memory::Memory(const Memory & __rhs) noexcept = default;
InternalProtonType::Hwinfo::Memory & InternalProtonType::Hwinfo::Memory::operator = (const Memory & __rhs) noexcept = default;
InternalProtonType::Hwinfo::Memory::Memory(Memory && __rhs) noexcept = default;
InternalProtonType::Hwinfo::Memory & InternalProtonType::Hwinfo::Memory::operator = (Memory && __rhs) noexcept = default;
InternalProtonType::Hwinfo::Memory::~Memory() = default; 
InternalProtonType::Hwinfo::Memory::Memory(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    size = ConfigParser::parse<int64_t>("size", __lines, 0);
    ConfigParser::stripLinesForKey("size", __remainingValuesToParse);
}

bool
InternalProtonType::Hwinfo::Memory::operator==(const Memory& __rhs) const noexcept
{
    return (size == __rhs.size);
}

bool
InternalProtonType::Hwinfo::Memory::operator!=(const Memory& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Hwinfo::Memory::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("size");
        __c.setString("type", "long");
        __c.setLong("value", size);
    }
}

InternalProtonType::Hwinfo::Memory::Memory(const vespalib::slime::Inspector & __inspector)
{
    size = __inspector["size"]["value"].asLong();
}

InternalProtonType::Hwinfo::Memory::Memory(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    size = ::config::internal::ValueConverter<int64_t>()(__inspector["size"], 0);
}

InternalProtonType::Hwinfo::Cpu::Cpu() noexcept
    : cores(0)
{
}

InternalProtonType::Hwinfo::Cpu::Cpu(const Cpu & __rhs) noexcept = default;
InternalProtonType::Hwinfo::Cpu & InternalProtonType::Hwinfo::Cpu::operator = (const Cpu & __rhs) noexcept = default;
InternalProtonType::Hwinfo::Cpu::Cpu(Cpu && __rhs) noexcept = default;
InternalProtonType::Hwinfo::Cpu & InternalProtonType::Hwinfo::Cpu::operator = (Cpu && __rhs) noexcept = default;
InternalProtonType::Hwinfo::Cpu::~Cpu() = default; 
InternalProtonType::Hwinfo::Cpu::Cpu(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    cores = ConfigParser::parse<int32_t>("cores", __lines, 0);
    ConfigParser::stripLinesForKey("cores", __remainingValuesToParse);
}

bool
InternalProtonType::Hwinfo::Cpu::operator==(const Cpu& __rhs) const noexcept
{
    return (cores == __rhs.cores);
}

bool
InternalProtonType::Hwinfo::Cpu::operator!=(const Cpu& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Hwinfo::Cpu::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("cores");
        __c.setString("type", "int");
        __c.setLong("value", cores);
    }
}

InternalProtonType::Hwinfo::Cpu::Cpu(const vespalib::slime::Inspector & __inspector)
{
    cores = __inspector["cores"]["value"].asLong();
}

InternalProtonType::Hwinfo::Cpu::Cpu(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    cores = ::config::internal::ValueConverter<int32_t>()(__inspector["cores"], 0);
}

InternalProtonType::Hwinfo::Hwinfo() noexcept
    : disk(),
      memory(),
      cpu()
{
}

InternalProtonType::Hwinfo::Hwinfo(const Hwinfo & __rhs) noexcept = default;
InternalProtonType::Hwinfo & InternalProtonType::Hwinfo::operator = (const Hwinfo & __rhs) noexcept = default;
InternalProtonType::Hwinfo::Hwinfo(Hwinfo && __rhs) noexcept = default;
InternalProtonType::Hwinfo & InternalProtonType::Hwinfo::operator = (Hwinfo && __rhs) noexcept = default;
InternalProtonType::Hwinfo::~Hwinfo() = default; 
InternalProtonType::Hwinfo::Hwinfo(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    disk = ConfigParser::parseStruct<Disk>("disk", __lines);
    ConfigParser::stripLinesForKey("disk", __remainingValuesToParse);
    memory = ConfigParser::parseStruct<Memory>("memory", __lines);
    ConfigParser::stripLinesForKey("memory", __remainingValuesToParse);
    cpu = ConfigParser::parseStruct<Cpu>("cpu", __lines);
    ConfigParser::stripLinesForKey("cpu", __remainingValuesToParse);
}

bool
InternalProtonType::Hwinfo::operator==(const Hwinfo& __rhs) const noexcept
{
    return (disk == __rhs.disk &&
            memory == __rhs.memory &&
            cpu == __rhs.cpu);
}

bool
InternalProtonType::Hwinfo::operator!=(const Hwinfo& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Hwinfo::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("disk");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        disk.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("memory");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        memory.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("cpu");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        cpu.serialize(__c2);
    }
}

InternalProtonType::Hwinfo::Hwinfo(const vespalib::slime::Inspector & __inspector)
{
    disk = Disk(__inspector["disk"]["value"]);
    memory = Memory(__inspector["memory"]["value"]);
    cpu = Cpu(__inspector["cpu"]["value"]);
}

InternalProtonType::Hwinfo::Hwinfo(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    disk = ::config::internal::ValueConverter<Disk>()(__inspector["disk"]);
    memory = ::config::internal::ValueConverter<Memory>()(__inspector["memory"]);
    cpu = ::config::internal::ValueConverter<Cpu>()(__inspector["cpu"]);
}

InternalProtonType::Feeding::Feeding() noexcept
    : concurrency(0.2),
      niceness(0.0),
      masterTaskLimit(0)
{
}

InternalProtonType::Feeding::Feeding(const Feeding & __rhs) noexcept = default;
InternalProtonType::Feeding & InternalProtonType::Feeding::operator = (const Feeding & __rhs) noexcept = default;
InternalProtonType::Feeding::Feeding(Feeding && __rhs) noexcept = default;
InternalProtonType::Feeding & InternalProtonType::Feeding::operator = (Feeding && __rhs) noexcept = default;
InternalProtonType::Feeding::~Feeding() = default; 
InternalProtonType::Feeding::Feeding(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    concurrency = ConfigParser::parse<double>("concurrency", __lines, 0.2);
    ConfigParser::stripLinesForKey("concurrency", __remainingValuesToParse);
    niceness = ConfigParser::parse<double>("niceness", __lines, 0.0);
    ConfigParser::stripLinesForKey("niceness", __remainingValuesToParse);
    masterTaskLimit = ConfigParser::parse<int32_t>("master_task_limit", __lines, 0);
    ConfigParser::stripLinesForKey("master_task_limit", __remainingValuesToParse);
}

bool
InternalProtonType::Feeding::operator==(const Feeding& __rhs) const noexcept
{
    return (concurrency == __rhs.concurrency &&
            niceness == __rhs.niceness &&
            masterTaskLimit == __rhs.masterTaskLimit);
}

bool
InternalProtonType::Feeding::operator!=(const Feeding& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Feeding::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("concurrency");
        __c.setString("type", "double");
        __c.setDouble("value", concurrency);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("niceness");
        __c.setString("type", "double");
        __c.setDouble("value", niceness);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("master_task_limit");
        __c.setString("type", "int");
        __c.setLong("value", masterTaskLimit);
    }
}

InternalProtonType::Feeding::Feeding(const vespalib::slime::Inspector & __inspector)
{
    concurrency = __inspector["concurrency"]["value"].asDouble();
    niceness = __inspector["niceness"]["value"].asDouble();
    masterTaskLimit = __inspector["master_task_limit"]["value"].asLong();
}

InternalProtonType::Feeding::Feeding(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    concurrency = ::config::internal::ValueConverter<double>()(__inspector["concurrency"], 0.2);
    niceness = ::config::internal::ValueConverter<double>()(__inspector["niceness"], 0.0);
    masterTaskLimit = ::config::internal::ValueConverter<int32_t>()(__inspector["master_task_limit"], 0);
}

InternalProtonType::Maintenancejobs::Maintenancejobs() noexcept
    : resourcelimitfactor(1.05),
      maxoutstandingmoveops(100)
{
}

InternalProtonType::Maintenancejobs::Maintenancejobs(const Maintenancejobs & __rhs) noexcept = default;
InternalProtonType::Maintenancejobs & InternalProtonType::Maintenancejobs::operator = (const Maintenancejobs & __rhs) noexcept = default;
InternalProtonType::Maintenancejobs::Maintenancejobs(Maintenancejobs && __rhs) noexcept = default;
InternalProtonType::Maintenancejobs & InternalProtonType::Maintenancejobs::operator = (Maintenancejobs && __rhs) noexcept = default;
InternalProtonType::Maintenancejobs::~Maintenancejobs() = default; 
InternalProtonType::Maintenancejobs::Maintenancejobs(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    resourcelimitfactor = ConfigParser::parse<double>("resourcelimitfactor", __lines, 1.05);
    ConfigParser::stripLinesForKey("resourcelimitfactor", __remainingValuesToParse);
    maxoutstandingmoveops = ConfigParser::parse<int32_t>("maxoutstandingmoveops", __lines, 100);
    ConfigParser::stripLinesForKey("maxoutstandingmoveops", __remainingValuesToParse);
}

bool
InternalProtonType::Maintenancejobs::
operator==(const Maintenancejobs& __rhs) const noexcept
{
    return (resourcelimitfactor == __rhs.resourcelimitfactor &&
            maxoutstandingmoveops == __rhs.maxoutstandingmoveops);
}

bool
InternalProtonType::Maintenancejobs::
operator!=(const Maintenancejobs& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Maintenancejobs::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("resourcelimitfactor");
        __c.setString("type", "double");
        __c.setDouble("value", resourcelimitfactor);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxoutstandingmoveops");
        __c.setString("type", "int");
        __c.setLong("value", maxoutstandingmoveops);
    }
}

InternalProtonType::Maintenancejobs::Maintenancejobs(const vespalib::slime::Inspector & __inspector)
{
    resourcelimitfactor = __inspector["resourcelimitfactor"]["value"].asDouble();
    maxoutstandingmoveops = __inspector["maxoutstandingmoveops"]["value"].asLong();
}

InternalProtonType::Maintenancejobs::Maintenancejobs(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    resourcelimitfactor = ::config::internal::ValueConverter<double>()(__inspector["resourcelimitfactor"], 1.05);
    maxoutstandingmoveops = ::config::internal::ValueConverter<int32_t>()(__inspector["maxoutstandingmoveops"], 100);
}

InternalProtonType::Bucketdb::Checksumtype
InternalProtonType::Bucketdb::getChecksumtype(const vespalib::string& name)
{
    if (name == "LEGACY") {
        return Checksumtype::LEGACY;
    } else if (name == "XXHASH64") {
        return Checksumtype::XXHASH64;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::Bucketdb::getChecksumtypeName(Checksumtype t)
{
    switch (t) {
        case Checksumtype::LEGACY: return "LEGACY";
        case Checksumtype::XXHASH64: return "XXHASH64";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::Bucketdb::Checksumtype InternalProtonType::Bucketdb::InternalChecksumtypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::Bucketdb::getChecksumtype(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::Bucketdb::Checksumtype InternalProtonType::Bucketdb::InternalChecksumtypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::Bucketdb::getChecksumtype(__inspector.asString().make_string());
}
InternalProtonType::Bucketdb::Checksumtype InternalProtonType::Bucketdb::InternalChecksumtypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Checksumtype __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::Bucketdb::getChecksumtype(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::Bucketdb::Bucketdb() noexcept
    : checksumtype(Checksumtype::LEGACY)
{
}

InternalProtonType::Bucketdb::Bucketdb(const Bucketdb & __rhs) noexcept = default;
InternalProtonType::Bucketdb & InternalProtonType::Bucketdb::operator = (const Bucketdb & __rhs) noexcept = default;
InternalProtonType::Bucketdb::Bucketdb(Bucketdb && __rhs) noexcept = default;
InternalProtonType::Bucketdb & InternalProtonType::Bucketdb::operator = (Bucketdb && __rhs) noexcept = default;
InternalProtonType::Bucketdb::~Bucketdb() = default; 
InternalProtonType::Bucketdb::Bucketdb(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    checksumtype = getChecksumtype(ConfigParser::parse<vespalib::string>("checksumtype", __lines, "LEGACY"));
    ConfigParser::stripLinesForKey("checksumtype", __remainingValuesToParse);
}

bool
InternalProtonType::Bucketdb::operator==(const Bucketdb& __rhs) const noexcept
{
    return (checksumtype == __rhs.checksumtype);
}

bool
InternalProtonType::Bucketdb::operator!=(const Bucketdb& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::Bucketdb::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("checksumtype");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getChecksumtypeName(checksumtype)));
    }
}

InternalProtonType::Bucketdb::Bucketdb(const vespalib::slime::Inspector & __inspector)
{
    checksumtype = getChecksumtype(__inspector["checksumtype"]["value"].asString().make_string());
}

InternalProtonType::Bucketdb::Bucketdb(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    checksumtype = InternalChecksumtypeConverter()(__inspector["checksumtype"], Checksumtype::LEGACY);
}

InternalProtonType::ReplayThrottlingPolicy::Type
InternalProtonType::ReplayThrottlingPolicy::getType(const vespalib::string& name)
{
    if (name == "UNLIMITED") {
        return Type::UNLIMITED;
    } else if (name == "DYNAMIC") {
        return Type::DYNAMIC;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::ReplayThrottlingPolicy::getTypeName(Type t)
{
    switch (t) {
        case Type::UNLIMITED: return "UNLIMITED";
        case Type::DYNAMIC: return "DYNAMIC";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::ReplayThrottlingPolicy::Type InternalProtonType::ReplayThrottlingPolicy::InternalTypeConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::ReplayThrottlingPolicy::getType(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::ReplayThrottlingPolicy::Type InternalProtonType::ReplayThrottlingPolicy::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::ReplayThrottlingPolicy::getType(__inspector.asString().make_string());
}
InternalProtonType::ReplayThrottlingPolicy::Type InternalProtonType::ReplayThrottlingPolicy::InternalTypeConverter::operator()(const ::vespalib::slime::Inspector & __inspector, Type __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::ReplayThrottlingPolicy::getType(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::ReplayThrottlingPolicy::ReplayThrottlingPolicy() noexcept
    : type(Type::DYNAMIC),
      minWindowSize(100),
      maxWindowSize(10000),
      windowSizeIncrement(20)
{
}

InternalProtonType::ReplayThrottlingPolicy::ReplayThrottlingPolicy(const ReplayThrottlingPolicy & __rhs) noexcept = default;
InternalProtonType::ReplayThrottlingPolicy & InternalProtonType::ReplayThrottlingPolicy::operator = (const ReplayThrottlingPolicy & __rhs) noexcept = default;
InternalProtonType::ReplayThrottlingPolicy::ReplayThrottlingPolicy(ReplayThrottlingPolicy && __rhs) noexcept = default;
InternalProtonType::ReplayThrottlingPolicy & InternalProtonType::ReplayThrottlingPolicy::operator = (ReplayThrottlingPolicy && __rhs) noexcept = default;
InternalProtonType::ReplayThrottlingPolicy::~ReplayThrottlingPolicy() = default; 
InternalProtonType::ReplayThrottlingPolicy::ReplayThrottlingPolicy(const ::config::StringVector & __lines)
{
    std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
    type = getType(ConfigParser::parse<vespalib::string>("type", __lines, "DYNAMIC"));
    ConfigParser::stripLinesForKey("type", __remainingValuesToParse);
    minWindowSize = ConfigParser::parse<int32_t>("min_window_size", __lines, 100);
    ConfigParser::stripLinesForKey("min_window_size", __remainingValuesToParse);
    maxWindowSize = ConfigParser::parse<int32_t>("max_window_size", __lines, 10000);
    ConfigParser::stripLinesForKey("max_window_size", __remainingValuesToParse);
    windowSizeIncrement = ConfigParser::parse<int32_t>("window_size_increment", __lines, 20);
    ConfigParser::stripLinesForKey("window_size_increment", __remainingValuesToParse);
}

bool
InternalProtonType::ReplayThrottlingPolicy::
operator==(const ReplayThrottlingPolicy& __rhs) const noexcept
{
    return (type == __rhs.type &&
            minWindowSize == __rhs.minWindowSize &&
            maxWindowSize == __rhs.maxWindowSize &&
            windowSizeIncrement == __rhs.windowSizeIncrement);
}

bool
InternalProtonType::ReplayThrottlingPolicy::
operator!=(const ReplayThrottlingPolicy& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::ReplayThrottlingPolicy::serialize(vespalib::slime::Cursor & __cursor) const
{
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("type");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getTypeName(type)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("min_window_size");
        __c.setString("type", "int");
        __c.setLong("value", minWindowSize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("max_window_size");
        __c.setString("type", "int");
        __c.setLong("value", maxWindowSize);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("window_size_increment");
        __c.setString("type", "int");
        __c.setLong("value", windowSizeIncrement);
    }
}

InternalProtonType::ReplayThrottlingPolicy::ReplayThrottlingPolicy(const vespalib::slime::Inspector & __inspector)
{
    type = getType(__inspector["type"]["value"].asString().make_string());
    minWindowSize = __inspector["min_window_size"]["value"].asLong();
    maxWindowSize = __inspector["max_window_size"]["value"].asLong();
    windowSizeIncrement = __inspector["window_size_increment"]["value"].asLong();
}

InternalProtonType::ReplayThrottlingPolicy::ReplayThrottlingPolicy(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    type = InternalTypeConverter()(__inspector["type"], Type::DYNAMIC);
    minWindowSize = ::config::internal::ValueConverter<int32_t>()(__inspector["min_window_size"], 100);
    maxWindowSize = ::config::internal::ValueConverter<int32_t>()(__inspector["max_window_size"], 10000);
    windowSizeIncrement = ::config::internal::ValueConverter<int32_t>()(__inspector["window_size_increment"], 20);
}

InternalProtonType::TensorImplementation
InternalProtonType::getTensorImplementation(const vespalib::string& name)
{
    if (name == "TENSOR_ENGINE") {
        return TensorImplementation::TENSOR_ENGINE;
    } else if (name == "FAST_VALUE") {
        return TensorImplementation::FAST_VALUE;
    } else {
        throw InvalidConfigException("Illegal enum value '" + name + "'");
    }
}

vespalib::string
InternalProtonType::getTensorImplementationName(TensorImplementation t)
{
    switch (t) {
        case TensorImplementation::TENSOR_ENGINE: return "TENSOR_ENGINE";
        case TensorImplementation::FAST_VALUE: return "FAST_VALUE";
        default:
        {
            vespalib::asciistream ost;
            ost << "UNKNOWN(" << static_cast<int>(t) << ")";
            return ost.str();
        }
    }
}

InternalProtonType::TensorImplementation InternalProtonType::InternalTensorImplementationConverter::operator()(const ::vespalib::string & __fieldName, const ::vespalib::slime::Inspector & __inspector) {
    if (__inspector.valid()) {
        return InternalProtonType::getTensorImplementation(__inspector.asString().make_string());
    }
    throw InvalidConfigException("Value for '" + __fieldName + "' required but not found");
}
InternalProtonType::TensorImplementation InternalProtonType::InternalTensorImplementationConverter::operator()(const ::vespalib::slime::Inspector & __inspector) {
    return InternalProtonType::getTensorImplementation(__inspector.asString().make_string());
}
InternalProtonType::TensorImplementation InternalProtonType::InternalTensorImplementationConverter::operator()(const ::vespalib::slime::Inspector & __inspector, TensorImplementation __eDefault) {
    if (__inspector.valid()) {
        return InternalProtonType::getTensorImplementation(__inspector.asString().make_string());
    }
    return __eDefault;
}

InternalProtonType::InternalProtonType()
    : basedir("."),
      rpcport(8004),
      httpport(0),
      clustername(""),
      distributionkey(-1),
      rpc(),
      search(),
      docsum(),
      numsearcherthreads(64),
      numthreadspersearch(1),
      numsummarythreads(16),
      validateAndSanitizeDocstore(ValidateAndSanitizeDocstore::NO),
      flush(),
      indexing(),
      index(),
      attribute(),
      grouping(),
      distribution(),
      summary(),
      documentdb(),
      periodic(),
      tlsspec("tcp/localhost:13700"),
      tlsconfigid(""),
      slobrokconfigid(""),
      routingconfigid(""),
      filedistributor(),
      pruneremoveddocumentsinterval(0.0),
      pruneremoveddocumentsage(1209600.0),
      packetcompresslimit(1024),
      packetcompresslevel(3),
      packetcompresstype(Packetcompresstype::LZ4),
      lidspacecompaction(),
      bucketmove(),
      maxvisibilitydelay(1.0),
      visit(),
      initialize(),
      writefilter(),
      hwinfo(),
      feeding(),
      maintenancejobs(),
      bucketdb(),
      replayThrottlingPolicy(),
      tensorImplementation(TensorImplementation::FAST_VALUE),
      forwardIssues(true)
{
}

InternalProtonType::InternalProtonType(const InternalProtonType & __rhs) = default;
InternalProtonType & InternalProtonType::operator =(const InternalProtonType & __rhs) = default;
InternalProtonType::InternalProtonType(InternalProtonType && __rhs) noexcept = default;
InternalProtonType & InternalProtonType::operator =(InternalProtonType && __rhs) noexcept = default;
InternalProtonType::~InternalProtonType() = default; 
InternalProtonType::InternalProtonType(const ConfigValue & __value)
{
    try {
        const ::config::StringVector & __lines(__value.getLines());
        std::set<vespalib::string> __remainingValuesToParse = ConfigParser::getUniqueNonWhiteSpaceLines(__lines);
        basedir = ConfigParser::parse<vespalib::string>("basedir", __lines, ".");
        ConfigParser::stripLinesForKey("basedir", __remainingValuesToParse);
        rpcport = ConfigParser::parse<int32_t>("rpcport", __lines, 8004);
        ConfigParser::stripLinesForKey("rpcport", __remainingValuesToParse);
        httpport = ConfigParser::parse<int32_t>("httpport", __lines, 0);
        ConfigParser::stripLinesForKey("httpport", __remainingValuesToParse);
        clustername = ConfigParser::parse<vespalib::string>("clustername", __lines, "");
        ConfigParser::stripLinesForKey("clustername", __remainingValuesToParse);
        distributionkey = ConfigParser::parse<int32_t>("distributionkey", __lines, -1);
        ConfigParser::stripLinesForKey("distributionkey", __remainingValuesToParse);
        rpc = ConfigParser::parseStruct<Rpc>("rpc", __lines);
        ConfigParser::stripLinesForKey("rpc", __remainingValuesToParse);
        search = ConfigParser::parseStruct<Search>("search", __lines);
        ConfigParser::stripLinesForKey("search", __remainingValuesToParse);
        docsum = ConfigParser::parseStruct<Docsum>("docsum", __lines);
        ConfigParser::stripLinesForKey("docsum", __remainingValuesToParse);
        numsearcherthreads = ConfigParser::parse<int32_t>("numsearcherthreads", __lines, 64);
        ConfigParser::stripLinesForKey("numsearcherthreads", __remainingValuesToParse);
        numthreadspersearch = ConfigParser::parse<int32_t>("numthreadspersearch", __lines, 1);
        ConfigParser::stripLinesForKey("numthreadspersearch", __remainingValuesToParse);
        numsummarythreads = ConfigParser::parse<int32_t>("numsummarythreads", __lines, 16);
        ConfigParser::stripLinesForKey("numsummarythreads", __remainingValuesToParse);
        validateAndSanitizeDocstore = getValidateAndSanitizeDocstore(ConfigParser::parse<vespalib::string>("validate_and_sanitize_docstore", __lines, "NO"));
        ConfigParser::stripLinesForKey("validate_and_sanitize_docstore", __remainingValuesToParse);
        flush = ConfigParser::parseStruct<Flush>("flush", __lines);
        ConfigParser::stripLinesForKey("flush", __remainingValuesToParse);
        indexing = ConfigParser::parseStruct<Indexing>("indexing", __lines);
        ConfigParser::stripLinesForKey("indexing", __remainingValuesToParse);
        index = ConfigParser::parseStruct<Index>("index", __lines);
        ConfigParser::stripLinesForKey("index", __remainingValuesToParse);
        attribute = ConfigParser::parseStruct<Attribute>("attribute", __lines);
        ConfigParser::stripLinesForKey("attribute", __remainingValuesToParse);
        grouping = ConfigParser::parseStruct<Grouping>("grouping", __lines);
        ConfigParser::stripLinesForKey("grouping", __remainingValuesToParse);
        distribution = ConfigParser::parseStruct<Distribution>("distribution", __lines);
        ConfigParser::stripLinesForKey("distribution", __remainingValuesToParse);
        summary = ConfigParser::parseStruct<Summary>("summary", __lines);
        ConfigParser::stripLinesForKey("summary", __remainingValuesToParse);
        documentdb = ConfigParser::parseArray<DocumentdbVector>("documentdb", __lines);
        ConfigParser::stripLinesForKey("documentdb", __remainingValuesToParse);
        periodic = ConfigParser::parseStruct<Periodic>("periodic", __lines);
        ConfigParser::stripLinesForKey("periodic", __remainingValuesToParse);
        tlsspec = ConfigParser::parse<vespalib::string>("tlsspec", __lines, "tcp/localhost:13700");
        ConfigParser::stripLinesForKey("tlsspec", __remainingValuesToParse);
        tlsconfigid = ConfigParser::parse<vespalib::string>("tlsconfigid", __lines, "");
        ConfigParser::stripLinesForKey("tlsconfigid", __remainingValuesToParse);
        slobrokconfigid = ConfigParser::parse<vespalib::string>("slobrokconfigid", __lines, "");
        ConfigParser::stripLinesForKey("slobrokconfigid", __remainingValuesToParse);
        routingconfigid = ConfigParser::parse<vespalib::string>("routingconfigid", __lines, "");
        ConfigParser::stripLinesForKey("routingconfigid", __remainingValuesToParse);
        filedistributor = ConfigParser::parseStruct<Filedistributor>("filedistributor", __lines);
        ConfigParser::stripLinesForKey("filedistributor", __remainingValuesToParse);
        pruneremoveddocumentsinterval = ConfigParser::parse<double>("pruneremoveddocumentsinterval", __lines, 0.0);
        ConfigParser::stripLinesForKey("pruneremoveddocumentsinterval", __remainingValuesToParse);
        pruneremoveddocumentsage = ConfigParser::parse<double>("pruneremoveddocumentsage", __lines, 1209600.0);
        ConfigParser::stripLinesForKey("pruneremoveddocumentsage", __remainingValuesToParse);
        packetcompresslimit = ConfigParser::parse<int32_t>("packetcompresslimit", __lines, 1024);
        ConfigParser::stripLinesForKey("packetcompresslimit", __remainingValuesToParse);
        packetcompresslevel = ConfigParser::parse<int32_t>("packetcompresslevel", __lines, 3);
        ConfigParser::stripLinesForKey("packetcompresslevel", __remainingValuesToParse);
        packetcompresstype = getPacketcompresstype(ConfigParser::parse<vespalib::string>("packetcompresstype", __lines, "LZ4"));
        ConfigParser::stripLinesForKey("packetcompresstype", __remainingValuesToParse);
        lidspacecompaction = ConfigParser::parseStruct<Lidspacecompaction>("lidspacecompaction", __lines);
        ConfigParser::stripLinesForKey("lidspacecompaction", __remainingValuesToParse);
        bucketmove = ConfigParser::parseStruct<Bucketmove>("bucketmove", __lines);
        ConfigParser::stripLinesForKey("bucketmove", __remainingValuesToParse);
        maxvisibilitydelay = ConfigParser::parse<double>("maxvisibilitydelay", __lines, 1.0);
        ConfigParser::stripLinesForKey("maxvisibilitydelay", __remainingValuesToParse);
        visit = ConfigParser::parseStruct<Visit>("visit", __lines);
        ConfigParser::stripLinesForKey("visit", __remainingValuesToParse);
        initialize = ConfigParser::parseStruct<Initialize>("initialize", __lines);
        ConfigParser::stripLinesForKey("initialize", __remainingValuesToParse);
        writefilter = ConfigParser::parseStruct<Writefilter>("writefilter", __lines);
        ConfigParser::stripLinesForKey("writefilter", __remainingValuesToParse);
        hwinfo = ConfigParser::parseStruct<Hwinfo>("hwinfo", __lines);
        ConfigParser::stripLinesForKey("hwinfo", __remainingValuesToParse);
        feeding = ConfigParser::parseStruct<Feeding>("feeding", __lines);
        ConfigParser::stripLinesForKey("feeding", __remainingValuesToParse);
        maintenancejobs = ConfigParser::parseStruct<Maintenancejobs>("maintenancejobs", __lines);
        ConfigParser::stripLinesForKey("maintenancejobs", __remainingValuesToParse);
        bucketdb = ConfigParser::parseStruct<Bucketdb>("bucketdb", __lines);
        ConfigParser::stripLinesForKey("bucketdb", __remainingValuesToParse);
        replayThrottlingPolicy = ConfigParser::parseStruct<ReplayThrottlingPolicy>("replay_throttling_policy", __lines);
        ConfigParser::stripLinesForKey("replay_throttling_policy", __remainingValuesToParse);
        tensorImplementation = getTensorImplementation(ConfigParser::parse<vespalib::string>("tensor_implementation", __lines, "FAST_VALUE"));
        ConfigParser::stripLinesForKey("tensor_implementation", __remainingValuesToParse);
        forwardIssues = ConfigParser::parse<bool>("forward_issues", __lines, true);
        ConfigParser::stripLinesForKey("forward_issues", __remainingValuesToParse);
    } catch (InvalidConfigException & __ice) {
        throw InvalidConfigException("Error parsing config '" + CONFIG_DEF_NAME + "' in namespace '" + CONFIG_DEF_NAMESPACE + "': " + __ice.getMessage());
    }
}

bool
InternalProtonType::operator==(const InternalProtonType& __rhs) const noexcept
{
    return (basedir == __rhs.basedir &&
            rpcport == __rhs.rpcport &&
            httpport == __rhs.httpport &&
            clustername == __rhs.clustername &&
            distributionkey == __rhs.distributionkey &&
            rpc == __rhs.rpc &&
            search == __rhs.search &&
            docsum == __rhs.docsum &&
            numsearcherthreads == __rhs.numsearcherthreads &&
            numthreadspersearch == __rhs.numthreadspersearch &&
            numsummarythreads == __rhs.numsummarythreads &&
            validateAndSanitizeDocstore == __rhs.validateAndSanitizeDocstore &&
            flush == __rhs.flush &&
            indexing == __rhs.indexing &&
            index == __rhs.index &&
            attribute == __rhs.attribute &&
            grouping == __rhs.grouping &&
            distribution == __rhs.distribution &&
            summary == __rhs.summary &&
            documentdb == __rhs.documentdb &&
            periodic == __rhs.periodic &&
            tlsspec == __rhs.tlsspec &&
            tlsconfigid == __rhs.tlsconfigid &&
            slobrokconfigid == __rhs.slobrokconfigid &&
            routingconfigid == __rhs.routingconfigid &&
            filedistributor == __rhs.filedistributor &&
            pruneremoveddocumentsinterval == __rhs.pruneremoveddocumentsinterval &&
            pruneremoveddocumentsage == __rhs.pruneremoveddocumentsage &&
            packetcompresslimit == __rhs.packetcompresslimit &&
            packetcompresslevel == __rhs.packetcompresslevel &&
            packetcompresstype == __rhs.packetcompresstype &&
            lidspacecompaction == __rhs.lidspacecompaction &&
            bucketmove == __rhs.bucketmove &&
            maxvisibilitydelay == __rhs.maxvisibilitydelay &&
            visit == __rhs.visit &&
            initialize == __rhs.initialize &&
            writefilter == __rhs.writefilter &&
            hwinfo == __rhs.hwinfo &&
            feeding == __rhs.feeding &&
            maintenancejobs == __rhs.maintenancejobs &&
            bucketdb == __rhs.bucketdb &&
            replayThrottlingPolicy == __rhs.replayThrottlingPolicy &&
            tensorImplementation == __rhs.tensorImplementation &&
            forwardIssues == __rhs.forwardIssues);
}

bool
InternalProtonType::operator!=(const InternalProtonType& __rhs) const noexcept
{
    return !(operator==(__rhs));
}

void
InternalProtonType::serialize(::config::ConfigDataBuffer & __buffer) const
{
    vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Cursor & __croot = __slime.setObject();
    __croot.setDouble("version", CONFIG_DEF_SERIALIZE_VERSION);
    vespalib::slime::Cursor & __key = __croot.setObject("configKey");
    __key.setString("defName", vespalib::Memory(CONFIG_DEF_NAME));
    __key.setString("defNamespace", vespalib::Memory(CONFIG_DEF_NAMESPACE));
    __key.setString("defMd5", vespalib::Memory(CONFIG_DEF_MD5));
    vespalib::slime::Cursor & __keySchema =__key.setArray("defSchema");
    for (size_t i = 0; i < CONFIG_DEF_SCHEMA.size(); i++) {
        __keySchema.addString(vespalib::Memory(CONFIG_DEF_SCHEMA[i]));
    }
    vespalib::slime::Cursor & __cursor = __croot.setObject("configPayload");
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("basedir");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(basedir));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("rpcport");
        __c.setString("type", "int");
        __c.setLong("value", rpcport);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("httpport");
        __c.setString("type", "int");
        __c.setLong("value", httpport);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("clustername");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(clustername));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("distributionkey");
        __c.setString("type", "int");
        __c.setLong("value", distributionkey);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("rpc");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        rpc.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("search");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        search.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("docsum");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        docsum.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("numsearcherthreads");
        __c.setString("type", "int");
        __c.setLong("value", numsearcherthreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("numthreadspersearch");
        __c.setString("type", "int");
        __c.setLong("value", numthreadspersearch);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("numsummarythreads");
        __c.setString("type", "int");
        __c.setLong("value", numsummarythreads);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("validate_and_sanitize_docstore");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getValidateAndSanitizeDocstoreName(validateAndSanitizeDocstore)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("flush");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        flush.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("indexing");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        indexing.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("index");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        index.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("attribute");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        attribute.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("grouping");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        grouping.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("distribution");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        distribution.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("summary");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        summary.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("documentdb");
        __c.setString("type", "array");
        vespalib::slime::Cursor & __c2 = __c.setArray("value");
        for (const auto & child : documentdb) {
            vespalib::slime::Cursor & __c3 = __c2.addObject();
            __c3.setString("type", "struct");
            Cursor & __c4 = __c3.setObject("value");
            child.serialize(__c4);
        }
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("periodic");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        periodic.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tlsspec");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(tlsspec));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tlsconfigid");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(tlsconfigid));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("slobrokconfigid");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(slobrokconfigid));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("routingconfigid");
        __c.setString("type", "string");
        __c.setString("value", vespalib::Memory(routingconfigid));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("filedistributor");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        filedistributor.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("pruneremoveddocumentsinterval");
        __c.setString("type", "double");
        __c.setDouble("value", pruneremoveddocumentsinterval);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("pruneremoveddocumentsage");
        __c.setString("type", "double");
        __c.setDouble("value", pruneremoveddocumentsage);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("packetcompresslimit");
        __c.setString("type", "int");
        __c.setLong("value", packetcompresslimit);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("packetcompresslevel");
        __c.setString("type", "int");
        __c.setLong("value", packetcompresslevel);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("packetcompresstype");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getPacketcompresstypeName(packetcompresstype)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("lidspacecompaction");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        lidspacecompaction.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("bucketmove");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        bucketmove.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maxvisibilitydelay");
        __c.setString("type", "double");
        __c.setDouble("value", maxvisibilitydelay);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("visit");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        visit.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("initialize");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        initialize.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("writefilter");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        writefilter.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("hwinfo");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        hwinfo.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("feeding");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        feeding.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("maintenancejobs");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        maintenancejobs.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("bucketdb");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        bucketdb.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("replay_throttling_policy");
        __c.setString("type", "struct");
        Cursor & __c2 = __c.setObject("value");
        replayThrottlingPolicy.serialize(__c2);
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("tensor_implementation");
        __c.setString("type", "enum");
        __c.setString("value", vespalib::Memory(getTensorImplementationName(tensorImplementation)));
    }
    {
        vespalib::slime::Cursor & __c = __cursor.setObject("forward_issues");
        __c.setString("type", "bool");
        __c.setBool("value", forwardIssues);
    }
}

InternalProtonType::InternalProtonType(const ::config::ConfigDataBuffer & __buffer)
{
    const vespalib::Slime & __slime(__buffer.slimeObject());
    vespalib::slime::Inspector & __croot = __slime.get();
    vespalib::slime::Inspector & __inspector = __croot["configPayload"];
    basedir = __inspector["basedir"]["value"].asString().make_string();
    rpcport = __inspector["rpcport"]["value"].asLong();
    httpport = __inspector["httpport"]["value"].asLong();
    clustername = __inspector["clustername"]["value"].asString().make_string();
    distributionkey = __inspector["distributionkey"]["value"].asLong();
    rpc = Rpc(__inspector["rpc"]["value"]);
    search = Search(__inspector["search"]["value"]);
    docsum = Docsum(__inspector["docsum"]["value"]);
    numsearcherthreads = __inspector["numsearcherthreads"]["value"].asLong();
    numthreadspersearch = __inspector["numthreadspersearch"]["value"].asLong();
    numsummarythreads = __inspector["numsummarythreads"]["value"].asLong();
    validateAndSanitizeDocstore = getValidateAndSanitizeDocstore(__inspector["validate_and_sanitize_docstore"]["value"].asString().make_string());
    flush = Flush(__inspector["flush"]["value"]);
    indexing = Indexing(__inspector["indexing"]["value"]);
    index = Index(__inspector["index"]["value"]);
    attribute = Attribute(__inspector["attribute"]["value"]);
    grouping = Grouping(__inspector["grouping"]["value"]);
    distribution = Distribution(__inspector["distribution"]["value"]);
    summary = Summary(__inspector["summary"]["value"]);
    for (size_t __i = 0; __i < __inspector["documentdb"]["value"].children(); __i++) {
        documentdb.push_back(Documentdb(__inspector["documentdb"]["value"][__i]["value"]));
    }
    periodic = Periodic(__inspector["periodic"]["value"]);
    tlsspec = __inspector["tlsspec"]["value"].asString().make_string();
    tlsconfigid = __inspector["tlsconfigid"]["value"].asString().make_string();
    slobrokconfigid = __inspector["slobrokconfigid"]["value"].asString().make_string();
    routingconfigid = __inspector["routingconfigid"]["value"].asString().make_string();
    filedistributor = Filedistributor(__inspector["filedistributor"]["value"]);
    pruneremoveddocumentsinterval = __inspector["pruneremoveddocumentsinterval"]["value"].asDouble();
    pruneremoveddocumentsage = __inspector["pruneremoveddocumentsage"]["value"].asDouble();
    packetcompresslimit = __inspector["packetcompresslimit"]["value"].asLong();
    packetcompresslevel = __inspector["packetcompresslevel"]["value"].asLong();
    packetcompresstype = getPacketcompresstype(__inspector["packetcompresstype"]["value"].asString().make_string());
    lidspacecompaction = Lidspacecompaction(__inspector["lidspacecompaction"]["value"]);
    bucketmove = Bucketmove(__inspector["bucketmove"]["value"]);
    maxvisibilitydelay = __inspector["maxvisibilitydelay"]["value"].asDouble();
    visit = Visit(__inspector["visit"]["value"]);
    initialize = Initialize(__inspector["initialize"]["value"]);
    writefilter = Writefilter(__inspector["writefilter"]["value"]);
    hwinfo = Hwinfo(__inspector["hwinfo"]["value"]);
    feeding = Feeding(__inspector["feeding"]["value"]);
    maintenancejobs = Maintenancejobs(__inspector["maintenancejobs"]["value"]);
    bucketdb = Bucketdb(__inspector["bucketdb"]["value"]);
    replayThrottlingPolicy = ReplayThrottlingPolicy(__inspector["replay_throttling_policy"]["value"]);
    tensorImplementation = getTensorImplementation(__inspector["tensor_implementation"]["value"].asString().make_string());
    forwardIssues = __inspector["forward_issues"]["value"].asBool();
}

InternalProtonType::InternalProtonType(const ::config::ConfigPayload & __payload)
{
    const vespalib::slime::Inspector & __inspector(__payload.get());
    basedir = ::config::internal::ValueConverter<vespalib::string>()(__inspector["basedir"], ".");
    rpcport = ::config::internal::ValueConverter<int32_t>()(__inspector["rpcport"], 8004);
    httpport = ::config::internal::ValueConverter<int32_t>()(__inspector["httpport"], 0);
    clustername = ::config::internal::ValueConverter<vespalib::string>()(__inspector["clustername"], "");
    distributionkey = ::config::internal::ValueConverter<int32_t>()(__inspector["distributionkey"], -1);
    rpc = ::config::internal::ValueConverter<Rpc>()(__inspector["rpc"]);
    search = ::config::internal::ValueConverter<Search>()(__inspector["search"]);
    docsum = ::config::internal::ValueConverter<Docsum>()(__inspector["docsum"]);
    numsearcherthreads = ::config::internal::ValueConverter<int32_t>()(__inspector["numsearcherthreads"], 64);
    numthreadspersearch = ::config::internal::ValueConverter<int32_t>()(__inspector["numthreadspersearch"], 1);
    numsummarythreads = ::config::internal::ValueConverter<int32_t>()(__inspector["numsummarythreads"], 16);
    validateAndSanitizeDocstore = InternalValidateAndSanitizeDocstoreConverter()(__inspector["validate_and_sanitize_docstore"], ValidateAndSanitizeDocstore::NO);
    flush = ::config::internal::ValueConverter<Flush>()(__inspector["flush"]);
    indexing = ::config::internal::ValueConverter<Indexing>()(__inspector["indexing"]);
    index = ::config::internal::ValueConverter<Index>()(__inspector["index"]);
    attribute = ::config::internal::ValueConverter<Attribute>()(__inspector["attribute"]);
    grouping = ::config::internal::ValueConverter<Grouping>()(__inspector["grouping"]);
    distribution = ::config::internal::ValueConverter<Distribution>()(__inspector["distribution"]);
    summary = ::config::internal::ValueConverter<Summary>()(__inspector["summary"]);
    ::config::internal::VectorInserter<DocumentdbVector> __documentdbInserter(documentdb);
    __inspector["documentdb"].traverse(__documentdbInserter);
    periodic = ::config::internal::ValueConverter<Periodic>()(__inspector["periodic"]);
    tlsspec = ::config::internal::ValueConverter<vespalib::string>()(__inspector["tlsspec"], "tcp/localhost:13700");
    tlsconfigid = ::config::internal::ValueConverter<vespalib::string>()(__inspector["tlsconfigid"], "");
    slobrokconfigid = ::config::internal::ValueConverter<vespalib::string>()(__inspector["slobrokconfigid"], "");
    routingconfigid = ::config::internal::ValueConverter<vespalib::string>()(__inspector["routingconfigid"], "");
    filedistributor = ::config::internal::ValueConverter<Filedistributor>()(__inspector["filedistributor"]);
    pruneremoveddocumentsinterval = ::config::internal::ValueConverter<double>()(__inspector["pruneremoveddocumentsinterval"], 0.0);
    pruneremoveddocumentsage = ::config::internal::ValueConverter<double>()(__inspector["pruneremoveddocumentsage"], 1209600.0);
    packetcompresslimit = ::config::internal::ValueConverter<int32_t>()(__inspector["packetcompresslimit"], 1024);
    packetcompresslevel = ::config::internal::ValueConverter<int32_t>()(__inspector["packetcompresslevel"], 3);
    packetcompresstype = InternalPacketcompresstypeConverter()(__inspector["packetcompresstype"], Packetcompresstype::LZ4);
    lidspacecompaction = ::config::internal::ValueConverter<Lidspacecompaction>()(__inspector["lidspacecompaction"]);
    bucketmove = ::config::internal::ValueConverter<Bucketmove>()(__inspector["bucketmove"]);
    maxvisibilitydelay = ::config::internal::ValueConverter<double>()(__inspector["maxvisibilitydelay"], 1.0);
    visit = ::config::internal::ValueConverter<Visit>()(__inspector["visit"]);
    initialize = ::config::internal::ValueConverter<Initialize>()(__inspector["initialize"]);
    writefilter = ::config::internal::ValueConverter<Writefilter>()(__inspector["writefilter"]);
    hwinfo = ::config::internal::ValueConverter<Hwinfo>()(__inspector["hwinfo"]);
    feeding = ::config::internal::ValueConverter<Feeding>()(__inspector["feeding"]);
    maintenancejobs = ::config::internal::ValueConverter<Maintenancejobs>()(__inspector["maintenancejobs"]);
    bucketdb = ::config::internal::ValueConverter<Bucketdb>()(__inspector["bucketdb"]);
    replayThrottlingPolicy = ::config::internal::ValueConverter<ReplayThrottlingPolicy>()(__inspector["replay_throttling_policy"]);
    tensorImplementation = InternalTensorImplementationConverter()(__inspector["tensor_implementation"], TensorImplementation::FAST_VALUE);
    forwardIssues = ::config::internal::ValueConverter<bool>()(__inspector["forward_issues"], true);
}

} // namespace internal

} // namespace vespa::config::search::core
